C  ---------------------------------------------------------------------
      SUBROUTINE ABORT ( MESS, SUB )
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE WILL BE CALLED TO ISSUE INFORMATIVE MESSAGES AND STOP
C  THE PROGRAM WHEN A SITUATION THAT NORMALLY SHOULD NEVER HAPPEN,
C  IS ENCOUNTERED.
C  POSSIBLY THERE IS SOME PROBLEM WITH THE MCLCOM SOURCE.
C
C  INPUT ARGUMENTS:
C
C    MESS       CHARACTER VARIABLE. THIS IS AN INFORMATIVE MESSAGE TO
C               BE DISPLAYED.
C    SUB        CHARACTER VARIABLE. THIS IS THE ROUTINE NAME IN WHICH
C               THE ABNORMAL SITUATION HAPPENED.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      LOGICAL EOPEN
      COMMON /ISOPEN/ EOPEN
C
      CHARACTER*(*) MESS,SUB
C
 100  FORMAT ('  ROUTINE ',A,' ABORTED.')
 200  FORMAT (2X,A)
C
      WRITE(*,100) SUB
      WRITE(*,200) MESS
C
C  WRITE THE SAME MESSAGE ON THE ERROR LIST FILE.
C
      IF (EOPEN) THEN
         WRITE (ERL,100) SUB
         WRITE (ERL,200) MESS
      END IF
C
      STOP ' -- MCLCOM EXECUTION ABORT --'
      END
C  ---------------------------------------------------------------------
 
      SUBROUTINE CHARR ( TOKEN, NT, NDIM, ERR, ARR, INFO, NUM, NN )
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE WILL CHECK IF AN ARRAY IS DEFINED IN A "VAR" STATEMENT.
C  IN THIS CASE, IT WILL PARSE THE DEFINITION AND UPDATE THE INFORMATION
C  TABLE.
C  THE PART OF THE INPUT OF THE INPUT LINE THAT IS ACTUALLY PARSED IS:
C
C  [ L1:U1, L2:U2, ... LN:UN ]
C
C  WITH L1,L2,...LN BEING THE LOW DIMENSION BOUNDS, AND
C  U1,U2,...UN BEING THE UPPER DIMENSION BOUNDS RESPECTIVELY.
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    NUM        REAL ARRAY. THE NUMBERS ENCOUNTERED IN THE INPUT LINE.
C
C  OUTPUT ARGUMENTS:
C
C    NDIM       INTEGER VARIABLE. THIS IS THE NUMBER OF DIMENSIONS FOR
C               THE DECLARED ARRAY.
C    ERR        LOGICAL VARIABLE. INDICATES WHETHER AN ERROR OCCURRED
C               (ERR=.TRUE.) IN THE DECLARATION, OR NOT. (ERR=.FALSE.)
C    ARR        LOGICAL VARIABLE. INDICATES WHETHER THE ITEM IN THE
C               "VAR" DECLARATION IS AN ARRAY ( ARR=.TRUE. ) ,
C               OR NOT. ( ARR=.FALSE. )
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    NT         INTEGER VARIABLE. THIS IS A POINTER TO THE TOKEN ARRAY
C               AND IS THE NUMBER OF TOKENS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    NN         INTEGER VARIABLE. THIS IS A POINTER TO THE NUM ARRAY
C               AND IS THE NUMBER OF NUMBERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINE ERROR.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      INTEGER TOKEN(*),INFO(*)
      REAL NUM(*)
      LOGICAL ERR,ARR
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
C  INITIALIZE THE OUTPUT ARGUMENTS.
C
      NDIM=0
      ERR=.FALSE.
      ARR=.TRUE.
C  IF A LEFT BRACKET FOLLOWS, THIS IS AN ARRAY DEFINITION,
C  A SIMPLE VARIABLE OTHERWISE.
      IF (TOKEN(NT+1).NE.LBRA) THEN
         ARR=.FALSE.
         RETURN
      END IF
C
      NT=NT+1
C
C  START A LOOP OVER ALL DIMENSIONS IN THE DECLARATION.
C
 10   NDIM=NDIM+1
      NT=NT+1
C  CHECK LOW DIMENSION BOUND.
      ISI=1
      CUR=TOKEN(NT)
      IF (CUR.EQ.PLUS) THEN
C  A PLUS SIGN IS DEFINED.
         NT=NT+1
         CUR=TOKEN(NT)
      ELSE IF (CUR.EQ.MINUS) THEN
C  A MINUS SIGN IS DEFINED.
         ISI=-1
         NT=NT+1
         CUR=TOKEN(NT)
      END IF
C
      IF (CUR.NE.UNSI) THEN
C  PLUS, MINUS SIGNS NOT DEFINED.
C  THIS ISN'T EITHER AN UNSIGNED NUMBER.
         ERR=.TRUE.
         CALL ERROR(55,' ')
         RETURN
      END IF
C  DETERMINE AND PUSH LOWER DIMENSION BOUND IN THE INFORMATION TABLE.
      NN=NN+1
      NINF=NINF+1
      LOW=NINT(ISI*NUM(NN))
      INFO(NINF)=LOW
C
C  A COLON SHOULD SEPARATE LOWER,UPPER DIMENSION BOUNDS.
C
      NT=NT+1
      IF (TOKEN(NT).NE.COLON) THEN
         ERR=.TRUE.
         CALL ERROR(56,' ')
         RETURN
      END IF
C  CHECK UPPER DIMENSION BOUND.
      NT=NT+1
      CUR=TOKEN(NT)
      ISI=1
      IF (CUR.EQ.PLUS) THEN
C  A PLUS SIGN IS DEFINED.
         NT=NT+1
         CUR=TOKEN(NT)
      ELSE IF (CUR.EQ.MINUS) THEN
C  A MINUS SIGN IS DEFINED.
         ISI=-1
         NT=NT+1
         CUR=TOKEN(NT)
      END IF
C  PLUS, MINUS SIGNS NOT DEFINED.
C  THIS ISN'T EITHER AN UNSIGNED NUMBER.
      IF (CUR.NE.UNSI) THEN
         ERR=.TRUE.
         CALL ERROR(57,' ')
         RETURN
      END IF
C  DETERMINE AND PUSH UPPER DIMENSION BOUND IN THE INFORMATION TABLE.
      NN=NN+1
      NINF=NINF+1
      UPPER=NINT(ISI*NUM(NN))
      INFO(NINF)=UPPER
C  LOWER DIMENSION BOUND SHOULD BE LESS OR EQUAL TO THE
C  UPPER DIMENSION BOUND.
      IF (UPPER.LT.LOW) CALL ERROR(59,' ')
C
      NT=NT+1
      CUR=TOKEN(NT)
C  A COMMA DENOTES ANOTHER DIMENSION IS DECLARED.
      IF (CUR.EQ.COMMA) GOTO 10
C  A RIGHT BRACKET ENDS THE ARRAY DECLARATION.
      IF (CUR.EQ.RBRA) RETURN
C  NEITHER COMMA, NOR RIGHT BRACKET FOLLOWS.
      CALL ERROR(58,' ')
      ERR=.TRUE.
      RETURN
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE CMSORT ( COMMAN, NAME, IND, NCOM )
C
      CHARACTER*(*) COMMAN(NCOM)
      INTEGER NAME(NCOM), IND(NCOM)
C
      LOGICAL SORTED
      CHARACTER*15 CTEMP
C
      DO 10,K=1,NCOM-1
        SORTED = .TRUE.
        DO 20,I=NCOM,K+1,-1
          IF (COMMAN(I).LT.COMMAN(I-1)) THEN
             SORTED = .FALSE.
          IM1 = I-1
             CTEMP = COMMAN(I)
             COMMAN(I) = COMMAN(IM1)
             COMMAN(IM1) = CTEMP
             ITEMP = IND(I)
             IND(I) = IND(IM1)
             IND(IM1) = ITEMP
             ITEMP = NAME(I)
             NAME(I) = NAME(IM1)
             NAME(IM1) = ITEMP
          END IF
 20     CONTINUE
        IF (SORTED) RETURN
 10   CONTINUE
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE COCAN ( NAME, DLINE, IS, IE, NIT, ICAN )
      IMPLICIT INTEGER (A-Z)
      CHARACTER*(*) NAME, DLINE
      INTEGER IS(*), IE(*)
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
C  LOCATE COMMAND IN THE DESCRIPTION FILE.
      OPEN(DESCRI,FILE='DESC',ERR=222)
      REWIND DESCRI
120   CONTINUE
      READ (DESCRI,'(A)',END=130) DLINE
      CALL LCR(DLINE,IS,IE,NIT)
C  EMPTY LINE ?
      IF (NIT.EQ.0) GOTO 120
C  COMMENT LINE ?
      IF (DLINE(IS(1):IS(1)).EQ.'#') GOTO 120
      IF (DLINE(IS(1):IE(1)).EQ.NAME) GOTO 140
      GOTO 120
C
130   CONTINUE
      CLOSE(DESCRI,ERR=223)
C  THIS IS NOT AN MCL STATEMENT.
      CALL ERROR(13,NAME)
      RETURN
C
140   CONTINUE
      CLOSE(DESCRI,ERR=223)
C  COMMAND FOUND IN THE DESCRIPTION FILE.
C  NOW DETERMINE THE INDEX OF THE CANCEL BUTTON.
      DO 160,I=2,NIT
         IF (DLINE(IS(I):IE(I)).EQ.'CANCEL') THEN
         ICAN = I-1
         RETURN
         END IF
160   CONTINUE
      CALL ERROR(168,NAME)
      RETURN
C
C  AN ERROR OCCURRED WHILE OPENING THE DESCRIPTION FILE.
C
 222  CALL ABORT('COULD NOT OPEN THE DESCRIPTION FILE.','COCAN')
C
C  AN ERROR OCCURRED WHILE CLOSING THE DESCRIPTION FILE.
C
 223  CALL ABORT('COULD NOT CLOSE THE DESCRIPTION FILE.','COCAN')
      END
C  ---------------------------------------------------------------------
      SUBROUTINE COPY ( INIT, FINAL )
C  ---------------------------------------------------------------------
C
C  COPIES A PART OF THE SCRATCH FILE (UNIT SCR) GENERATED BY THE
C  COMPILER TO THE FINAL OBJECT FILE. THE PART TO BE COPIED IS SPECIFIED
C  BY THE INITIAL AND FINAL LINE COUNTERS. THESE COUNTERS HAVE NOTHING
C  TO DO WITH THE ACTUAL LOCATION OF THE LINES IN THE FILE. LINES ARE
C  COPIED UNTIL THE (INITIAL COUNTER)+1 REACHES THE (FINAL COUNTER)-1.
C
C  INPUT ARGUMENTS:
C
C    INIT     INITIAL LINE COUNTER. COPYING STARTS AT INIT+1.
C    FINAL    FINAL LINE COUNTER. COPYING STOPS AT FINAL-1.
C
C  NOTES:
C    NOTE THAT ACTUALLY  (FINAL-1)-(INIT+1)+1 = FINAL-INIT-1 LINES
C    ARE COPIED STARTING AT THE CURRENT POSITION OF THE SCR FILE.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      CHARACTER*80 LINE
C
      DO 10,I=INIT+1,FINAL-1
        READ (SCR,'(A)') LINE
        LE = LENGTH(LINE)
        IF (LE.GT.0) THEN
           WRITE(BIN,'(A)') LINE(1:LE)
        ELSE
           WRITE(BIN,'('' '')')
        END IF
 10   CONTINUE
C
      END
C  ---------------------------------------------------------------------
      LOGICAL FUNCTION DEF ( V,VALUE,FILE,NF )
C  ---------------------------------------------------------------------
C
C  THIS FUNCTION WILL TEST IF A GIVEN ARGUMENT (V) IS USED AS A FILE
C  NAME IN A PREVIOUSLY ENCOUNTERED KEYWORD. IN SUCH A CASE, A VALUE
C  OF .TRUE. WILL BE RETURNED; .FALSE. OTHERWISE.
C
C  INPUT ARGUMENTS:
C
C    V          CHARACTER VARIABLE. THE FUNCTION WILL CHECK IF THE
C               THIS FILE NAME HAS BEEN ENTERED AS A VALUE TO A
C               PREVIOUSLY ENCOUNTERED KEYWORD.
C    VALUE      CHARACTER ARRAY. CONTAINS THE VALUES OF THE KEYWORDS
C               ENTERED IN THE INPUT LINE.
C    FILE       INTEGER ARRAY. EACH ENTRY OF THIS ARRAY IS A POINTER
C               TO THE VALUE ARRAY. IT POINTS TO THE POSITIONS OF
C               THE VALUE ARRAY, THAT HAVE BEEN USED AS FILE NAMES
C               IN THE I,E OR B PARAMETERS.
C    NF         INTEGER VARIABLE. NUMBER OF ENTRIES IN THE FILE ARRAY.
C               THIS IS ACTUALLY THE NUMBER OF KEYWORDS THAT USED A
C               FILE NAME AS A VALUE, SO FAR.
C
C  ---------------------------------------------------------------------
C
      CHARACTER*(*) V,VALUE(*)
      INTEGER FILE(*)
C
      DEF=.TRUE.
      DO 10,I=1,NF
        IF (V.EQ.VALUE(FILE(I))) RETURN
 10   CONTINUE
C
      DEF=.FALSE.
      RETURN
      END
CDFL--------------------------------------------------------------------
      SUBROUTINE DFL(ID,TOKENS,TOKN,TUSED,ALPHAC,NA,NUMBC,NN,VAR,
     >                ADDR, DIMARG, INFO, INFPOS, VRKIND, STVAR)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: IT PARSES DEMARGIN FIX AND LOOSE STATEMENTS.
C
C   %..   ARGUMENT INFORMATION:
C
C      -ID    : PROCESSING OPTION, SEE NOTES.
C      -TOKENS: IS THE LINE TO BE PROCESSED.
C      -TOKN  : IS THE NUMBER OF TOKENS "TOKENS" IS COMPOSED OF
C      -TUSED : POINTS TO THE LAST USED ELEMENT OF "TOKENS".
C      -ALPHAC: ARE THE IDENTIFIERS OF "TOKENS".
C      -NUMBC : ARE THE NUMERIC CONSTANTS OF "TOKENS".
C      -NA    : POINTS TO THE LAST USED ELEMENT OF ALPHAC.
C      -NN    : POINTS TO THE LAST USED ELEMENT OF NUMBC.
C      -VAR   : THE VAR TABLE.
C      -ADDR  : ARRAY KEEPING LOCATIONS. SEE XGEN FER DETAILS.
C      -DIMARG: ARRAY KEEPING DIMENSIONALITIES. SEE XGEN.
C      -INFO  : ARRAY WITH MCL-ITEMS INFORMATION. SEE XGEN.
C      -INFPOS: ARRAY RELATED TO INFO. SEE XGEN.
C      -VRKIND: PARALLEL ARRAY TO VAR. KEEPS "KIND"S. SEE XGEN.
C      -STVAR : KEEPS DUMMY NAMES FOR FUNCTIONS.
C
C   %..   SUBPROGRAMS CALLED:
C
C              SUBROUTINES: ERROR, EXTRA, XPR, ABORT,
C              FUNCTIONS  : NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C              MCLUNT (A) , MCLDTA (M) , MCLSCD (A)
C
C   %..   POSSIBLE ABORTS:
C
C              ABORT MAY OCCUR IF THE SPARE FILE COULD NOT BE OPENED.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS: NONE.
C
C   %..   NOTES:
C
C            ID IS 1 FOR DEMARGIN, 2 FOR FIX, 3 FOR LOOSE.
C
C    SYNTAX:
C
C    ('DEMARGIN'\'FIX'\'LOOSE') '(' <KEY> '.' <EXPRESSION>
C                                [ ';' <KEY> '.' <EXPRESSION> ] ')'
C    WITH <KEY> : : = 'X' \ 'R' \ 'L'
C
C    CODE WRITTEN ( IF SYNTAX OK ) :
C
C       $WFIXLOO                                 $WDEMARG
C       NUMBER-OF-INDEXES                        NU-LFT, NU-RT
C       COMMAND                       OR         $SWAP
C       $SWAP
C
C    AND "NUMBER-OF-INDEXES" TOP POSI-       AND FIRST "NU-LFT"
C    TIONS OF STACK,HOLD THOSE INDICES.      TOP POSITIONS OF STACK HOLD
C                                            LEFT INDICES,WHILE THE
C                                            FOLLOWING "NU-RT" POSI-
C                                            TIONS OF
C                                            STACK HOLD RIGHT INDICES.
C
C-----------------------------------------------------------------------
C
      IMPLICIT INTEGER(A-Z)
      DIMENSION TOKENS(TOKN),EXPR(76),ADDR(*),DIMARG(*),INFO(*),
     >          INFPOS(*),VRKIND(*)
      PARAMETER (LINLEN=80, CHOICE=2)
      LOGICAL CORR,GENE,WFX,L,R,TDK,TSM,TKRT,FDECLR,BOUNDS,DEBUG
      CHARACTER*(*) ALPHAC(*), VAR(*), C*30, OLDLIN*(LINLEN),
     >              STVAR(*)
      REAL NUMBC(*)
      COMMON / MCLUNT / INP, ERL, BIN, SCR, ISTF, SPARE, DUMMYF,DESCRI
     >       / MCLDTA / NLL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
     >       / MCLSCD / LEFTP, RIGHTP, DOT, COMMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUMBER,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
C
      WFX()=(ID.EQ.2).OR.(ID.EQ.3)
      R()=(C.EQ.'R')
      L()=(C.EQ.'L')
      TDK()=(TOKENS(TP).EQ.IDENT)
      TKRT()=(TOKENS(TP).EQ.RIGHTP)
      TSM()=(TOKENS(TP).EQ.SEMI)
C
C    START
C
      NEW=SPARE
      OPEN(UNIT=NEW,STATUS='SCRATCH',ERR=1111)
      FDECLR=.FALSE.
      NX=0
      NR=0
      NL=0
      OLD=SCR
      TP=TUSED+1
      IF(TOKENS(TP).EQ.LEFTP.AND.ID.EQ.1)THEN
         REWIND NEW
         NLBLOC=0
 120     CONTINUE
         TP=TP+1
         IF ( TP .EQ. TOKN ) GO TO 100
         NA=NA+1
         C=ALPHAC(NA)
         IF(R().AND.TDK())THEN
            SCR=OLD
            TP=TP+1
            IF ( TP .EQ. TOKN ) GO TO 100
            IF(TOKENS(TP).EQ.DOT)THEN
               CALL EXTRA(TOKENS,TOKN,TP,EXPR,NEX,CHOICE,ALPHAC,NA)
               TP=TP+1
               IF ( TP .EQ. TOKN ) GO TO 100
               IF((TKRT().AND.TP+1.EQ.TOKN).OR.TSM())THEN
                  CALL XPR ( EXPR,NEX,ALPHAC,NUMBC,NA,NN,CORR,VAR,
     >            FDECLR,BLOC,ADDR,DIMARG,INFO,INFPOS,VRKIND,SCR,STVAR)
                  IF(.NOT.CORR)GO TO 1717
                  NR=NR+1
                  IF(TSM())GOTO 120
                  IF(TKRT().AND.TP+1.EQ.TOKN)THEN
                     GOTO 250
                  ELSE
                     CALL ERROR (154,' ')
                  ENDIF
               ELSE
                  CALL ERROR (156,' ')
               ENDIF
            ELSE
               CALL ERROR (155,' ')
            ENDIF
         ELSEIF(L().AND.TDK())THEN
            SCR=NEW
            TP=TP+1
            IF ( TP .EQ. TOKN ) GO TO 100
            IF(TOKENS(TP).EQ.DOT)THEN
               CALL EXTRA(TOKENS,TOKN,TP,EXPR,NEX,CHOICE,ALPHAC,NA)
               TP=TP+1
               IF ( TP .EQ. TOKN ) GO TO 100
               IF((TKRT().AND.TP+1.EQ.TOKN).OR.TSM())THEN
                  SAVBL=BLOC
                  CALL XPR ( EXPR,NEX,ALPHAC,NUMBC,NA,NN,CORR,VAR,
     >            FDECLR,BLOC,ADDR,DIMARG,INFO,INFPOS,VRKIND,SCR,STVAR)
                  IF(.NOT.CORR)GO TO 1717
                  NL=NL+1
                  NLBLOC=NLBLOC+BLOC-SAVBL
                  IF(TSM())GOTO 120
                  IF(TKRT().AND.TP+1.EQ.TOKN)THEN
                     GOTO 250
                  ELSE
                     CALL ERROR (154,' ')
                  ENDIF
               ELSE
                  CALL ERROR (156,' ')
               ENDIF
            ELSE
               CALL ERROR (155,' ')
            ENDIF
         ELSE
            CALL ERROR (156,' ')
         ENDIF
 1717    CONTINUE
         SCR=OLD
C
         RETURN
C
 250     CONTINUE
         IF (GENE) WRITE(SCR,'(''$PUSHC''/I2)') NR
         BLOC = BLOC+2
         SCR=OLD
         REWIND NEW
         IF(NL.NE.0)THEN
            DO 918,I=1,NLBLOC
               READ(NEW,'(A)')OLDLIN
               WRITE(SCR,'(A)')OLDLIN
 918        CONTINUE
         ENDIF
         IF(GENE)THEN
            WRITE(SCR,'(''$PUSHC''/I2)') NL
            WRITE(SCR,'(''DEMARGIN'')')
         ENDIF
         BLOC=BLOC+3
      ELSEIF(TOKENS(TP).EQ.LEFTP.AND.WFX())THEN
 180     CONTINUE
         TP=TP+1
         IF ( TP .EQ. TOKN ) GO TO 100
         NA=NA+1
         C=ALPHAC(NA)
         IF(C.EQ.'X'.AND.TDK())THEN
            TP=TP+1
            IF ( TP .EQ. TOKN ) GO TO 100
            IF(TOKENS(TP).EQ.DOT)THEN
               CALL EXTRA (TOKENS,TOKN,TP,EXPR,NEX,CHOICE,ALPHAC,NA)
               TP = TP+1
               IF ( TP .EQ. TOKN ) GO TO 100
               IF((TKRT().AND.TP+1.EQ.TOKN).OR.TSM())THEN
                  CALL XPR ( EXPR,NEX,ALPHAC,NUMBC,NA,NN,CORR,VAR,
     >            FDECLR,BLOC,ADDR,DIMARG,INFO,INFPOS,VRKIND,SCR,STVAR)
                  IF(.NOT.CORR)RETURN
                  NX=NX+1
                  IF(TSM())GOTO 180
                  IF(.NOT.(TKRT().AND.TP+1.EQ.TOKN))THEN
                     CALL ERROR (154,' ')
                  ENDIF
                  BLOC=BLOC+3
                  IF(GENE)THEN
               WRITE(SCR,'(''$PUSHC'')')
               WRITE(SCR,*)NX
                  IF (ID.EQ.2) THEN
                        WRITE(SCR,'(''FIX'')')
                  ELSE
                     WRITE(SCR,'(''LOOSE'')')
                  END IF
                  ENDIF
               ELSE
                  CALL ERROR (154,' ')
               ENDIF
            ELSE
               CALL ERROR (155,' ')
            ENDIF
         ELSE
            CALL ERROR (156,' ')
         ENDIF
      ELSE
         CALL ERROR (157,' ')
      ENDIF
      CLOSE(SPARE,STATUS='DELETE')
      RETURN
 100  CONTINUE
      CALL ERROR (158,' ')
      CLOSE(SPARE,STATUS='DELETE')
      RETURN
C
C ABORT SITUATION
C
 1111 CALL ABORT ('UNABLE TO OPEN SPARE FILE','DFL')
      END
CDISCA------------------------------------------------------------------
      SUBROUTINE DISCA (TOKENS,TOKN,TUSED,ALPHAC,NA)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: PARSES DISCARD STATEMENT.
C
C   %..   ARGUMENT INFORMATION:
C
C      -TOKENS: IS THE LINE TO BE PROCESSED.
C      -TOKN  : IS THE NUMBER OF TOKENS "TOKENS" IS COMPOSED OF
C      -TUSED : POINTS TO THE LAST USED ELEMENT OF "TOKENS".
C      -ALPHAC: ARE THE IDENTIFIERS OF "TOKENS".
C      -NA    : POINTS TO THE LAST USED ELEMENT OF ALPHAC.
C
C   %..   SUBPROGRAMS CALLED:
C
C     SUBROUTINES: ERROR,
C     FUNCTIONS  : NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C       MCLDTA (M) , MCLUNT (A) , MCLSCD (A)
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS: NONE.
C
C   %..   NOTES:
C
C      SYNTAX :
C      <DISCARD> : : =  'DISCARD' '('['FILE' '=']<IDENT> ')'
C      WITH <IDENT> : : = 'DISPO' \ 'STORE' \ 'BACKUP'
C
C-----------------------------------------------------------------------
C
      IMPLICIT INTEGER ( A - Z)
      DIMENSION TOKENS(TOKN)
      LOGICAL GENE,DEBUG,BOUNDS
      CHARACTER*(*) ALPHAC(*),FILENM*30
      COMMON / MCLUNT / INP, ERL, BIN, SCR, ISTF, SPARZZ, DUMMYF,DESCRI
     >       / MCLDTA / NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
     >       / MCLSCD / LEFTP, RIGHTP, DOT, COMMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUMBER,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
C
C   START
C
      TP=TUSED+1
      IF(TP.EQ.TOKN)GOTO 100
         IF(TOKENS(TP).EQ.LEFTP)THEN
            TP=TP+1
            NA=NA+1
            IF(TP.EQ.TOKN)GOTO 100
            IF(ALPHAC(NA).EQ.'FILE')THEN
               TP=TP+1
               IF(TP.GE.TOKN)GOTO 100
               IF(TOKENS(TP).EQ.EQ)THEN
                  TP = TP+1
                  IF(TP.EQ.TOKN)GOTO 100
                  IF(TOKENS(TP).EQ.IDENT)THEN
                     NA=NA+1
                     FILENM=ALPHAC(NA)
                     IF(FILENM.EQ.'DISPO'.OR.FILENM.EQ.'STORE'.OR.
     >                  FILENM.EQ.'BACKUP')THEN
                        TP=TP+1
                        IF(TP.EQ.TOKN)GOTO 100
                        IF(TOKENS(TP).EQ.RIGHTP.AND.TP+1.EQ.TOKN)THEN
                           IF(GENE)WRITE(SCR,'(''DISCARD'',/,A)')FILENM
                           BLOC=BLOC+2
                        ELSE
                           CALL ERROR (108,' ')
                        ENDIF
                     ELSE
                        CALL ERROR (109,' ')
                     ENDIF
                  ELSE
                     CALL ERROR (110,' ')
                  ENDIF
               ELSE
                  CALL ERROR (111,' ')
               ENDIF
            ELSE
               IF(TOKENS(TP).EQ.IDENT)THEN
                  FILENM=ALPHAC(NA)
                  IF(FILENM.EQ.'DISPO'.OR.FILENM.EQ.'STORE'.OR.
     >               FILENM.EQ.'BACKUP')THEN
                     TP=TP+1
                     IF(TP.EQ.TOKN)GOTO 100
                     IF(TOKENS(TP).EQ.RIGHTP.AND.TP+1.EQ.TOKN)THEN
                        IF(GENE)WRITE(SCR,'(''DISCARD'',/,A)')FILENM
                        BLOC=BLOC+2
                     ELSE
                        CALL ERROR (108,' ')
                     ENDIF
                  ELSE
                     CALL ERROR (109,' ')
                  ENDIF
               ELSE
                  CALL ERROR (110,' ')
               ENDIF
            ENDIF
         ELSE
            CALL ERROR (112,' ')
         ENDIF
      RETURN
 100  CALL ERROR (113,' ')
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE DISPLA ( TOKEN,NTOK,NT,ALPHA,NA,VAR,NUM,NN,LOC,DIM,
     >                    INFO,IPOS,KIND )
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE HANDLES THE "DISPLAY" STATEMENT.
C  THE SYNTAX OF THIS STATEMENT IS:
C
C  "DISPLAY" ITEM1;ITEM2;...ITEMN       OR
C  "DISPLAY" ITEM1;ITEM2;...ITEMN "TO" <FILENAME>
C
C  WHERE <FILENAME> IS THE FILE TO RECEIVE THE OUTPUT,
C  AND ITEM1;ITEM2;...ITEMN IS A LIST OF ITEMS.
C  EACH ITEM MAY BE:
C  1) A VALID MCL EXPRESSION, OR
C  2) A SEQUENCE OF CHARACTERS, ( A STRING ) DELIMITED BY SINGLE QUOTES.
C
C  THE ROUTINE WILL FIRST DETERMINE THE INPUT <FILENAME>. IF ITS
C  OMITTED, THE DEFAULT MERLIN INPUT FILE WILL BE USED.
C  <FILENAME> IS NOT ALLOWED TO BE ANY OF THE MERLIN FILES:
C  HELP, DATA, STORE, INIPO, DISPO, BACKUP, MACROF, DEFINED IN THE
C  BLOCK DATA MCLDEF.
C  THEN IT WILL DETERMINE THE NUMBER OF VARIABLES TO GET BY COUNTING
C  THE SEMICOLONS IN THE INPUT LINE.
C  A LOOP OVER ALL ITEMS IN THE ITEM LIST WILL START. FOR AN EXPRESSION
C  THE CODE GENERATED IS:
C
C  <CODE TO EVALUATE THE EXPRESSION>
C  $OUT
C  <FILENAME>
C
C  WHERE <CODE TO EVALUATE THE EXPRESSION> IS THE CODE NECESSARY TO
C  EVALUATE THE EXPRESSION. ( OBVIOUSLY MORE THAN ONE LINE )
C  FOR A STRING THE CODE GENERATED IS:
C
C  $NOTE
C  <LENGTH>
C  <STRING>
C  <FILENAME>
C
C  WHERE <LENGTH> IS THE LENGTH OF THE STRING, IN CHARACTERS,
C  <STRING> IS THE STRING OF CHARACTERS TO BE DISPLAYED, AND
C  <FILENAME> IS THE FILE TO RECEIVE THE OUTPUT. ( ".STANDARD" WILL
C  BE USED IF THE <FILENAME> IS OMITTED FROM THE "DISPLAY" STATEMENT )
C  WHEN ALL THE ITEMS IN THE LIST HAVE BEEN PROCESSED, THE FOLLOWING
C  TWO MORE LINES OF OBJECT CODE ARE WRITTEN:
C
C  $FLUSH
C  <FILENAME>
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C               THE INPUT LINE.
C    NUM        REAL ARRAY. THE NUMBERS ENCOUNTERED IN THE INPUT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    NT         INTEGER VARIABLE. THIS IS A POINTER TO THE TOKEN ARRAY
C               AND IS THE NUMBER OF TOKENS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    NA         INTEGER VARIABLE. THIS IS A POINTER TO THE ALPHA ARRAY
C               AND IS THE NUMBER OF IDENTIFIERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    VAR        CHARACTER ARRAY. THIS IS THE VAR TABLE.
C    NN         INTEGER VARIABLE. THIS IS A POINTER TO THE NUM ARRAY
C               AND IS THE NUMBER OF NUMBERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    LOC        INTEGER ARRAY. THIS IS THE MEMORY LOCATION TABLE.
C    DIM        INTEGER ARRAY. THIS IS THE DIMENSIONALITY TABLE.
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    IPOS       INTEGER ARRAY. THIS IS THE POSITION TABLE.
C    KIND       INTEGER ARRAY. THIS IS THE KIND TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES EXTRA, XPR, ERROR.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER (  IDLEN = 30   )
C
C  THE LENGTH OF A LINE. (CHARS)
      PARAMETER (  LINLEN = 80  )
C
      INTEGER TOKEN(*),LOC(*),DIM(*),INFO(*),IPOS(*),KIND(*)
      CHARACTER*(*) ALPHA(*),VAR(*)
      REAL NUM(*)
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      CHARACTER*10 MFILE(7)
      COMMON /MCLMFL/ MFILE
C
      INTEGER EX(LINLEN)
      LOGICAL CORRE
      CHARACTER LINE*80
      CHARACTER*(IDLEN) FILE,STARG(1)
C
C  DETERMINE <FILENAME>.
C  FIRST COUNT THE IDENTIFIERS IN THE INPUT LINE.
C
      NID=NA
      DO 99,I=NT+1,NTOK
        IF (TOKEN(I).EQ.IDENT) NID=NID+1
 99   CONTINUE
C  CHECK TO SEE IF <FILENAME> IS DEFINED. USE THE DEFAULT OTHERWISE.
      FILE='.STANDARD'
      IF (NID.NE.NA) THEN
         IF (ALPHA(NID-1).EQ.'FROM') FILE=ALPHA(NID)
      END IF
C
C  CHECK IF THIS IS A MERLIN FILE.
C
      DO 62,I=1,7
        IF (FILE.EQ.MFILE(I)) THEN
           CALL ERROR(77,FILE)
           GOTO 30
        END IF
 62   CONTINUE
C
C  START A LOOP OVER ALL ITEMS IN THE LIST.
C
 30   NT=NT+1
      IF (TOKEN(NT).EQ.APOST) THEN
         NT=NT+1
         IF (TOKEN(NT).NE.STRING) THEN
             CALL ERROR(30,' ')
             RETURN
         END IF
C  THIS ITEM IS A STRING.
         LINE=ALPHA(NA+1)//ALPHA(NA+2)
         NA=NA+2
         NN=NN+1
C  LL IS THE STRING LENGTH IN CHARACTERS.
         LL=NINT(NUM(NN))
         NT=NT+1
         IF (TOKEN(NT).EQ.APOST) THEN
C  THE STRING TERMINATES NORMALLY.
C  NOW WRITE THE NECESSARY OBJECT CODE.
            BLOC=BLOC+4
            IF (GENE) WRITE(SCR,'(A/I3/A/A)') '$NOTE',LL,LINE,FILE
         ELSE
            CALL ERROR(28,' ')
            RETURN
         END IF
      ELSE
C  THIS ITEM IN THE LIST IS EXPECTED TO BE AN EXPRESSION.
         NT=NT-1
C  EXTRACT THE EXPRESSION.
         CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,1,ALPHA,NA)
C  EVALUATE THE EXPRESSION.
         CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,BLOC,LOC,
     >            DIM,INFO,IPOS,KIND,SCR,STARG)
         IF (.NOT.CORRE) RETURN
C  IF THE EXPRESSION IS CORRECT, GENERATE CODE.
         BLOC=BLOC+2
         IF (GENE) WRITE(SCR,'(A/A)') '$OUT',FILE
      END IF
C
 20   NT=NT+1
      IF (NT.EQ.NTOK) THEN
C  ALL TOKENS IN THE INPUT LINE EXHAUSTED.
C  WRITE THE FINAL INSTRUCTIONS AND RETURN.
         BLOC=BLOC+2
         IF (GENE) WRITE(SCR,'(A/A)') '$FLUSH','.STANDARD'
         RETURN
      END IF
C
      CUR=TOKEN(NT)
C  A SEMICOLON DENOTES ANOTHER ITEM IN THE LIST.
      IF (CUR.EQ.SEMI) GOTO 30
      IF (CUR.EQ.IDENT) THEN
C  IF AN IDENTIFIER FOLLOWS, IT SHOULD BE "TO".
         NA=NA+1
         IF (ALPHA(NA).NE.'TO') THEN
            CALL ERROR(67,ALPHA(NA))
            RETURN
         END IF
         NT=NT+1
C  AN IDENTIFIER ( THE <FILENAME> ) SHOULD FOLLOW.
         IF (TOKEN(NT).NE.IDENT) THEN
            CALL ERROR(68,' ')
            RETURN
         END IF
         NA=NA+1
C  WRITE THE FINAL INSTRUCTIONS.
         BLOC=BLOC+2
         IF (GENE) WRITE(SCR,'(A/A)') '$FLUSH',ALPHA(NA)
         NT=NT+1
C  IF ANYTHING ELSE, OTHER THAN AN END OF LINE FOLLOWS.
         IF (NT.NE.NTOK) CALL ERROR(69,' ')
         RETURN
      END IF
C  NEITHER SEMICOLON, NOR AN IDENTIFIER FOLLOWS.
      CALL ERROR(29,' ')
      RETURN
C
      END
C  ---------------------------------------------------------------------
      SUBROUTINE ELSE ( NTOK,TYPE,TDEF,TREL )
C  ---------------------------------------------------------------------
C
C  PROCESSES THE "ELSE" STATEMENT. THE SYNTAX OF THIS STATEMENT IS AS:
C  "ELSE"
C  THE CODE GENERATED IS:
C
C  $MOVE
C  <BLANK LINE>
C
C  THE TYPE TABLE WILL BE SEARCHED BACKWARDS UNTIL AN ENTRY OF 1
C  ( INDICATING THE PRESENCE OF THE "IF" STATEMENT, CORRESPONDING
C  TO THIS "ELSE" ) IS REACHED.
C  THE NUMBER OF LINES TO JUMP SO AS TO REACH THIS "ELSE" STATEMENT
C  ( IN CASE THE EXPRESSION INVOLVED IN THE "IF" STATEMENT IS NOT
C  SATISFIED ) WILL BE CALCULATED AND PUSHED IN THE APPROPRIATE
C  POSITION, IN THE TYPE-LINK TABLE. THE ENTRY IN THE TYPE TABLE
C  FOR THIS "IF" WILL BE SET TO -1, INDICATING THAT IT HAS BEEN
C  PROCESSED. A VALUE OF 2 IS PUSHED IN THE TYPE TABLE, ( INDICATING
C  THE PRESENCE OF THE "ELSE" STATEMENT ) WHILE THE LOCATION OF
C  THE ABOVE <BLANK LINE> IS PUSHED IN THE TYPE-DEFINITION TABLE, IN
C  ORDER TO BE USED LATER ON.
C
C  INPUT ARGUMENTS:
C
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C    TREL       INTEGER ARRAY. THIS IS THE TYPE-LINK TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINE ERROR, OVER.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      INTEGER TYPE(*),TDEF(*),TREL(*)
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
C  IF MORE THAN 2 TOKENS ( "ELSE" EOL ) ARE IN THE SAME LINE,
C  THIS IS AN ERROR.
C
      IF (NTOK.NE.2) THEN
         CALL ERROR(9,' ')
         RETURN
      END IF
C
C  GENERATE CODE, SO THAT WHEN EXECUTION OF THE "IF...THEN" BLOCK
C  COMPLETES, THE "ELSE...ENDIF" BLOCK WILL BE SKIPPED, AND THE "ENDIF"
C  STATEMENT WILL BE REACHED.
C  A BLANK LINE WILL BE LEFT AFTER THE "$MOVE" INSTRUCTION. THE CORRECT
C  NUMBER OF LINES TO JUMP WILL BE WRITTEN IN PASS 2.
C
      BLOC=BLOC+2
      IF (GENE) WRITE(SCR,'(A/)') '$MOVE'
C
C  SEARCH THE TYPE TABLE BACKWARDS UNTIL AN ENTRY OF 1 IS LOCATED.
C
      DO 10,I=NTYPE,1,-1
        TP=TYPE(I)
        IF (TP.GT.0) THEN
           IF (TP.EQ.1) THEN
C  AN ENTRY OF 1 IS FOUND IN THE TYPE TABLE. CALCULATE THE TYPE-LINK
C  ENTRY FOR THIS "IF".
              TREL(I)=BLOC-TDEF(I)+1
              TYPE(I)=-TP
C  PUT THIS "ELSE" IN THE TYPE TABLE.
              NTYPE=NTYPE+1
              IF (NTYPE.GT.MAXTYP)
     >              CALL OVER('TYPE','TYPE','ELSE',MAXTYP,'MXTYP')
              TYPE(NTYPE)=2
              TDEF(NTYPE)=BLOC
           ELSE
C  THIS IS AN "ELSE" WITH NO CORRESPONDING "IF".
              CALL ERROR(144,' ')
           END IF
           RETURN
        END IF
C  THIS IS AN "ELSE" WITH NO CORRESPONDING "IF".
 10   CONTINUE
      CALL ERROR(144,' ')
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE ENDIF ( NTOK,NT,TYPE,TDEF,TREL )
C  ---------------------------------------------------------------------
C
C  PROCESSES THE "ENDIF" STATEMENT. THE SYNTAX OF THIS STATEMENT IS AS:
C  "ENDIF", OR "END" "IF"
C  ONLY ONE LINE OF OBJECT CODE IS GENERATED:
C  $CONTINUE
C
C  THE TYPE TABLE WILL BE SEARCHED BACKWARDS UNTIL AN ENTRY OF 1
C  OR 2 ( INDICATING THE PRESENCE OF AN "IF" OR "ELSE" STATEMENT )
C  IS REACHED.
C  THE NUMBER OF LINES TO JUMP SO AS TO REACH THIS "ENDIF" STATEMENT
C  WILL BE CALCULATED AND PUSHED IN THE APPROPRIATE POSITION,IN THE
C  TYPE-LINK TABLE. THE ENTRY IN THE TYPE TABLE FOR THE "IF" OR "ELSE"
C  LOCATED WILL BE SET TO -1 OR -2 RESPECTIVELY, INDICATING THAT THEY
C  HAVE BEEN PROCESSED. A VALUE OF -3 IS PUSHED IN THE TYPE TABLE,
C  ( INDICATING THE PRESENCE OF THE "ENDIF" STATEMENT ) WHILE THE
C  LOCATION OF THE ABOVE $CONTINUE INSTRUCTION IS PUSHED IN THE
C  TYPE-DEFINITION TABLE, IN ORDER TO BE USED LATER ON.
C
C  INPUT ARGUMENTS:
C
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    NT         NUMBER OF TOKENS USED BY PREVIOUSLY CALLED PARSING
C               ROUTINES.
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C    TREL       INTEGER ARRAY. THIS IS THE TYPE-LINK TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINE ERROR, OVER.
C
C  NOTES:
C    NOTE THAT ONLY TWO VALUES ARE POSSIBLE FOR NT UPON ENTRY: 1 OR 2.
C    IF THE STATEMENT IS ENTERED AS "ENDIF" ( ONE WORD ), THIS WILL
C    CAUSE NT TO TAKE ON THE VALUE 1, SINCE ONLY ONE TOKEN
C    ( AN IDENTIFIER ) IS FOUND.
C    IF IF THE STATEMENT IS ENTERED AS "END" "IF" (TWO DISTINCT WORDS),
C    THIS WILL CAUSE NT TO TAKE ON THE VALUE 2, SINCE TWO TOKENS
C    ( TWO IDENTIFIERS ) ARE FOUND. BOTH OF THEM ARE USED BY THE
C    PCG SUBROUTINE IN ORDER TO RECOGNIZE THE STATEMENT.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      INTEGER TYPE(*),TDEF(*),TREL(*)
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
C  IF ANYTHING ELSE FOLLOWS THE "ENDIF" STATEMENT, THIS IS AN ERROR.
C
      NT=NT+1
      IF (NT.NE.NTOK) THEN
         CALL ERROR(10,' ')
         RETURN
      END IF
C
C  GENERATE CODE.
C
      BLOC=BLOC+1
      IF (GENE) WRITE(SCR,'(A)') '$CONTINUE'
C
C  SEARCH THE TYPE TABLE BACKWARDS UNTIL AN ENTRY OF 1 OR 2 IS REACHED.
C
      DO 10,I=NTYPE,1,-1
        TP=TYPE(I)
        IF (TP.GT.0) THEN
           IF (TP.EQ.1 .OR. TP.EQ.2) THEN
C  A VALUE OF 1 OR 2 HAS BEEN REACHED. CALCULATE THE APPROPRIATE
C  ENTRY IN THE TYPE-LINK TABLE.
              TREL(I)=BLOC-TDEF(I)
              TYPE(I)=-TP
C  ENTER THIS "ENDIF" IN THE TYPE TABLE.
              NTYPE=NTYPE+1
              IF (NTYPE.GT.MAXTYP)
     >              CALL OVER('TYPE','TYPE','ENDIF',MAXTYP,'MXTYP')
              TYPE(NTYPE)=-3
              TDEF(NTYPE)=BLOC
           ELSE
C  THIS IS AN "ENDIF" WITHOUT A CORRESPONDING "IF".
              CALL ERROR(145,' ')
           END IF
           RETURN
        END IF
C
 10   CONTINUE
C  THIS IS AN "ENDIF" WITHOUT A CORRESPONDING "IF".
      CALL ERROR(145,' ')
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE ENDLOO ( NTOK, NT, TYPE, TDEF, TREL, LLOC )
C  ---------------------------------------------------------------------
C
C  PARSES THE ENDLOOP STATEMENT AND GENERATES APPROPRIATE OBJECT CODE.
C  THE TYPE TABLE IS SEARCHED BACKWARDS UNTIL AN ENTRY OF 4 ( INDICATING
C  THE CORRESPONDING LOOP ) IS REACHED. AT THIS POINT, THE
C  TYPE-LINK TABLE ENTRY FOR THE LOOP IS CALCULATED. MEANWHILE FOR EVERY
C  ENTRY OF 8 IN THE TYPE TABLE ( INDICATING AN EXIT STATEMENT ) THE
C  CORRESPONDING TYPE-LINK TABLE ENTRY IS CALCULATED.
C  17 LINES OF OBJECT CODE ARE GENERATED.
C  INITIALLY THE LOOP CONTROL INDEX IS INCREMENTED BY THE STEP VALUE:
C
C  $PUSH
C  VAR_MLOC
C  $PUSH
C  STEP_MLOC
C  $+
C  $POP
C  VAR_MLOC
C
C  WHERE VAR_MLOC IS THE MEMORY LOCATION ASSIGNED TO THE LOOP CONTROL
C  INDEX AND
C  STEP_MLOC IS THE MEMORY LOCATION USED TO STORE THE STEP FOR
C  THIS LOOP.
C
C  THE TRIP COUNTER IS THEN INCREMENTED BY 1:
C
C  $PUSHC
C  1
C  $PUSH
C  CTC_MLOC
C  $+
C  $POP
C  CTC_MLOC
C
C  WHERE CTC_MLOC IS THE MEMORY LOCATION USED TO STORE THE TRIP COUNTER.
C
C  THEN THE LOOP IS REPEATED ONCE MORE:
C
C  $MOVE
C  <BLANK LINE>
C  $CONTINUE
C
C  A VALUE OF -5 IS PUSHED IN THE TYPE TABLE, INDICATING THE ENDLOOP,
C  WHILE THE LOCATION OF THE ABOVE <BLANK LINE> IN THE SCRATCH FILE,
C  IS PUSHED IN THE TYPE-DEFINITION TABLE. THE ENTRY IN THE TYPE-LINK
C  TABLE FOR THIS ENDLOOP IS CALCULATED.
C
C  INPUT ARGUMENTS:
C
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C    LLOC       INTEGER ARRAY. THIS IS THE LOOP TABLE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    NT         INTEGER VARIABLE. THIS IS THE NUMBER OF TOKENS USED
C               SO FAR BY THE PREVIOUSLY CALLED PARSING ROUTINES.
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C    TREL       INTEGER ARRAY. THIS IS THE TYPE-LINK TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINE ERROR.
C
C  ABORT CONDITIONS THAT MAY OCCUR:
C    IF THE TYPE TABLE OVERFLOWS, THE PROGRAM WILL STOP AND
C    ISSUE AN INFORMATIVE MESSAGE.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      INTEGER TYPE(*),TDEF(*),TREL(*),LLOC(*)
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
C  IF ANYTHING ELSE THAN AN EOL FOLLOWS THE STATEMENT, THIS IS AN ERROR.
C
      NT=NT+1
      IF (NT.NE.NTOK) THEN
         CALL ERROR(139,' ')
         RETURN
      END IF
C
C  SEARCH THE TYPE ARRAY BACKWARDS.
C
      DO 10,I=NTYPE,1,-1
        TP=TYPE(I)
        IF (TP.GT.0) THEN
           IF (TP.EQ.4) THEN
C  THE CORRESPONDING "LOOP" HAS BEEN FOUND.
              LP=BLOC-TDEF(I)
              TREL(I)=LP+17
              TYPE(I)=-4
              GOTO 90
           ELSE IF (TP.EQ.8) THEN
C  AN "EXIT" STATEMENT FOR THIS LOOP HAS BEEN FOUND.
              TREL(I)=BLOC-TDEF(I)+17
              TYPE(I)=-8
           ELSE
C  THIS IS AN "ENDLOOP" WITHOUT A CORRESPONDING "LOOP" STATEMENT.
              CALL ERROR(140,' ')
              RETURN
           END IF
        END IF
 10   CONTINUE
C  THIS IS AN "ENDLOOP" WITHOUT A CORRESPONDING "LOOP" STATEMENT.
      CALL ERROR(140,' ')
      RETURN
C
 90   CONTINUE
C
C  GET NECESSARY INFORMATION FOR THE STORAGE USED BY THIS LOOP.
C
      L     = LLOC(NLOO)
      ISTEP = LMEM+2
      ITC   = LMEM+3
      NLOO  = NLOO-1
      LMEM  = LMEM-5
C
C  WRITE OBJECT CODE.
C
      BLOC=BLOC+17
      IF (GENE) THEN
C  INCREMENT THE LOOP CONTROL INDEX.
         WRITE(SCR,'(A)') '$PUSH'
         WRITE(SCR,*) L
         WRITE(SCR,'(A)') '$PUSH'
         WRITE(SCR,*) ISTEP
         WRITE(SCR,'(A/A)') '$+','$POP'
         WRITE(SCR,*) L
C  INCREMENT THE TRIP COUNTER.
         WRITE(SCR,'(A/I2/A)') '$PUSHC',1,'$PUSH'
         WRITE(SCR,*) ITC
         WRITE(SCR,'(A/A)') '$+','$POP'
         WRITE(SCR,*) ITC
C  REPEAT THE LOOP ONCE MORE.
         WRITE(SCR,'(A//A)') '$MOVE','$CONTINUE'
      END IF
C
C  UPDATE THE TYPE, TYPE-DEFINITION AND TYPE-LINK TABLES.
C
      NTYPE=NTYPE+1
      IF (NTYPE.GT.MAXTYP)
     >           CALL OVER('TYPE','TYPE','ENDLOO',MAXTYP,'MXTYP')
      TYPE(NTYPE)=-5
      TDEF(NTYPE)=BLOC-1
      TREL(NTYPE)=-(LP+21)
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE ERROR ( CODE, MESS )
C  ---------------------------------------------------------------------
C
C  HANDLES ERROR PROCESSING. WHEN A SYNTACTIC ERROR IS DETECTED DURING
C  THE PARSING OF AN MCL SOURCE LINE, THIS ROUTINE IS CALLED TO ISSUE
C  THE APPROPRIATE MESSAGE.
C  ACCORDING TO THE INPUT CODE, A MESSAGE WILL BE PLACED IN THE ERROR
C  BUFFER.
C  ANOTHER MESSAGE ( A SHORT ONE, UP TO 10 CHARACTERS  ), WILL ALSO BE
C  PLACED IN THE ERROR BUFFER. IF THE ACTUAL ARGUMENT IN THE ROUTINE
C  CALL IS LONGER THAN 10 CHARACTERS, IT WILL BE TRUNCATED, AND 3 DOTS
C  WILL BE APPENDED TO IT.
C  AN OPTION IS AVAILABLE THAT CONTROLS DISPLAY OF THE ERROR CODE TO
C  THE OUTPUT FILE. THIS MAY BE USEFUL FOR DEBUGGING PURPOSES, OR
C  FOR FURTHER DEVELOPMENT.
C  WHEN THIS ROUTINE IS CALLED, FURTHER CODE GENERATION
C  WILL BE DISABLED.
C
C  INPUT ARGUMENTS:
C
C    CODE       INTEGER VARIABLE. THIS IS THE ERROR CODE NUMBER TO BE
C               ISSUED.
C    MESS       CHARACTER VARIABLE. THIS A SHORT (UP TO 10 CHARACTERS)
C               DESCRIPTION OF THE PROBLEM, TO BE DISPLAYED ALONG WITH
C               THE FULL ERROR MESSAGE DESCRIPTION.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINE ABORT.
C
C  ABORT CONDITIONS THAT MAY OCCUR:
C    IF THE ERROR BUFFER IS FULL, THE PROGRAM WILL STOP AND ISSUE AN
C    INFORMATIVE MESSAGE.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  TOTAL NUMBER OF ERROR MESSAGES ISSUED BY THE COMPILER.
      PARAMETER ( MAXMES = 168 )
C
C  MAXIMUM NUMBER OF ERROR MESSAGES IN ONE LINE OF THE SOURCE PROGRAM.
      PARAMETER ( MAXERR = 20 )
C
      CHARACTER*60 MESAGE(MAXMES)
      CHARACTER*(*) MESS
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      CHARACTER*90 EBUFF(MAXERR)
      COMMON /MCLERB/ EBUFF
      SAVE /MCLERB/
C
      CHARACTER ELINE*90,WMESS*13
C
C  PARAMETER INDICATING, WHETHER THE ERROR CODE WILL BE DISPLAYED
C  ON THE ERROR LIST, OR NOT.
      LOGICAL CDIS
      DATA CDIS / .FALSE. /
C
C  THE COMPILER'S ERROR MESSAGES.
C
      DATA (MESAGE(I),I=  1, 15)/
     > 'LABEL ALREADY DEFINED.                                      ',
     > 'LABEL DEFINITION NOT FOLLOWED BY "EOL".                     ',
     > 'IDENTIFIER EXPECTED IN THE LIST OF VARIABLES.               ',
     > 'VARIABLE ALREADY DEFINED.                                   ',
     > 'SEMICOLON EXPECTED IN "VAR" DECLARATION.                    ',
     > '"JUST" EXPECTED.                                            ',
     > 'STATEMENT NOT RECOGNIZED.                                   ',
     > '"THEN" EXPECTED.                                            ',
     > '"EOL" EXPECTED AFTER "ELSE".                                ',
     > '"EOL" EXPECTED AFTER "ENDIF".                               ',
     > 'LABEL NOT DEFINED.                                          ',
     > 'RESERVED NAME USED IN DECLARATION STATEMENT.                ',
     > 'THIS IS NOT AN MCL STATEMENT.                               ',
     > 'STATEMENT NOT FOLLOWED BY "EOL".                            ',
     > '"MOVE TO" STATEMENT NOT FOLLOWED BY "EOL".                  '/
      DATA (MESAGE(I),I= 16, 30)/
     > 'LABEL NAME EXPECTED IN "MOVE TO" STATEMENT.                 ',
     > '"TO" EXPECTED AFTER "MOVE".                                 ',
     > 'RESERVED VARIABLES MAY NOT BE ASSIGNED A VALUE.             ',
     > 'VARIABLE NOT DECLARED.                                      ',
     > '"EOL" EXPECTED AFTER THE ASSIGNMENT.                        ',
     > 'INITIAL DOT NOT FOUND.                                      ',
     > 'IDENTIFIER EXPECTED AS MACRO NAME.                          ',
     > 'STATEMENT NOT FOLLOWED BY "EOL".                            ',
     > 'MACRO FILE COULD NOT BE OPENED.                             ',
     > 'INITIAL MACRO "*" NOT FOUND IN THE MACRO FILE.              ',
     > 'MACRO NOT FOUND IN THE MACRO FILE.                          ',
     > 'THE MACRO SPECIFIED IS EMPTY.                               ',
     > 'CLOSING "''" NOT FOUND.                                     ',
     > 'SEMICOLON EXPECTED.                                         ',
     > 'STRING EXPECTED AFTER "''".                                 '/
      DATA (MESAGE(I),I= 31, 45)/
     > 'LEFT PARENTHESIS SHOULD FOLLOW THE STATEMENT.               ',
     > 'KEYWORD EXPECTED.                                           ',
     > 'KEYWORD NOT VALID FOR THIS COMMAND.                         ',
     > '"." EXPECTED AFTER KEYWORD.                                 ',
     > '"=" NOT FOUND.                                              ',
     > 'RIGHT PARENTHESIS EXPECTED.                                 ',
     > 'LEFT PARENTHESIS EXPECTED.                                  ',
     > '<KEYWORD> "="   EXPECTED.                                   ',
     > 'KEYWORD ILLEGAL IN THE ACCUM STATEMENT.                     ',
     > 'RIGHT PARENTHESIS EXPECTED.                                 ',
     > 'KEYWORD MISSING.                                            ',
     > '"VAR" / "FUNCTION" STATEMENT MISPLACED.                     ',
     > 'OPERAND MISSING, OR OPERATOR TROUBLES IN EXPRESSION.        ',
     > 'MISSING OPERAND, OR MISSING OPERATOR IN EXPRESSION.         ',
     > 'EXPRESSION CONTAINS ILLEGAL SEQUENCE.                       '/
      DATA (MESAGE(I),I= 46, 60)/
     > 'EMPTY EXPRESSION INTO PARENTHESES.                          ',
     > 'VARIABLE NOT DECLARED.                                      ',
     > 'UNMATCHED ")" OR "]" OR COMMA IN WRONG PLACE.               ',
     > 'UNMATCHED LEFT PARENTHESIS OR LEFT BRACKET.                 ',
     > 'IDENTIFIER NOT FOUND BEFORE "[".                            ',
     > 'LEFT PARENTHESIS NOT FOUND AFTER PARAMETRIC STATEMENT NAME. ',
     > 'KEYWORD EXPECTED AFTER LEFT PARENTHESIS."EOL" FOUND INSTEAD.',
     > 'KEYWORD ILLEGAL FOR THIS STATEMENT.                         ',
     > 'RIGHT PARENTHESIS EXPECTED.                                 ',
     > 'LOW ARRAY BOUND NOT NUMERICAL.                              ',
     > 'COLON EXPECTED AFTER LOW ARRAY BOUND.                       ',
     > 'ARRAY, HIGH BOUND NOT NUMERICAL.                            ',
     > 'RIGHT BRACKET NOT FOUND IN ARRAY DECLARATION.               ',
     > 'LOW BOUND GREATER THAN HIGH BOUND.                          ',
     > 'IDENTIFIER EXPECTED AS FUNCTION NAME.                       '/
      DATA (MESAGE(I),I= 61, 75)/
     > 'LEFT BRACKET MISSING.                                       ',
     > 'ARGUMENT IN ERROR.                                          ',
     > 'RIGHT BRACKET MISSING IN ARRAY DECLARATION.                 ',
     > '"=" EXPECTED.                                               ',
     > 'FUNCTION NAME ALREADY DECLARED.                             ',
     > 'FUNCTION ARGUMENT DUPLICATED.                               ',
     > '"TO" NOT FOUND.                                             ',
     > 'FILE NAME EXPECTED AFTER "TO".                              ',
     > '"EOL" NOT FOUND AFTER FILE NAME.                            ',
     > 'LEFT PARENTHESIS NOT FOUND.                                 ',
     > '"=" EXPECTED AFTER KEYWORD.                                 ',
     > 'LEFT PARENTHESIS EXPECTED.                                  ',
     > '"FLAG" MISSING.                                             ',
     > 'KEYWORD INTO ILLEGAL COMBINATION.                           ',
     > 'STATEMENT SYNTACTICALLY INCOMPLETE.                         '/
      DATA (MESAGE(I),I= 76, 90)/
     > 'STATEMENT MUST END WITH THE SEQUENCE: <ARITHMEXPR>, ")".    ',
     > 'I/O OPERATIONS SHOULD NOT BE PERFORMED ON MERLIN FILES.     ',
     > 'NUMBER OF SUBSCRIPTS OR ARGUMENTS, DIFFERENT THAN DECLARED. ',
     > 'FILE NAME WAS EXPECTED AFTER "=".                           ',
     > 'PICK STATEMENT MUST END WITH RIGHT PARENTHESIS.             ',
     > '"=" EXPECTED AFTER THE "FILE" KEYWORD.                      ',
     > '"FILE" EXPECTED AFTER ";".                                  ',
     > 'SEMICOLON EXPECTED AFTER EXPRESSION.                        ',
     > '"=" EXPECTED AFTER THE "REC" KEYWORD.                       ',
     > 'SEMICOLON WAS EXPECTED AFTER FILE NAME.                     ',
     > '"REC" OR "FILE" EXPECTED AFTER "(", OR "=" NOT DETECTED.    ',
     > 'LEFT PARENTHESIS EXPECTED AFTER "PICK".                     ',
     > 'PICK STATEMENT SYNTACTICALLY INCOMPLETE.                    ',
     > '"=" EXPECTED AFTER "REC".                                   ',
     > '"REC" EXPECTED.                                             '/
      DATA (MESAGE(I),I= 91,105)/
     > '"=" EXPECTED.                                               ',
     > 'THIS STATEMENT MUST END WITH A RIGHT PARENTHESIS.           ',
     > 'IDENTIFIER EXPECTED.                                        ',
     > '"." EXPECTED.                                               ',
     > '"X" EXPECTED.                                               ',
     > 'LEFT PARENTHESIS EXPECTED.                                  ',
     > 'GODFATHER STATEMENT SYNTACTICALLY INCOMPLETE.               ',
     > 'THIS STATEMENT MUST END WITH A RIGHT PARENTHESIS.           ',
     > 'IDENTIFIER EXPECTED AFTER "=".                              ',
     > 'REWIND STATEMENT SYNTACTICALLY INCOMPLETE.                  ',
     > '"=" EXPECTED AFTER FILE INDICATOR.                          ',
     > '"REWIND" MUST BE FOLLOWED BY LEFT PARENTHESIS.              ',
     > 'THIS STATEMENT MUST END MITH A RIGHT PARENTHESIS.           ',
     > 'IDENTIFIER EXPECTED AFTER "=".                              ',
     > 'HIDEOUT STATEMENT SYNTACTICALLY INCOMPLETE.                 '/
      DATA (MESAGE(I),I=106,120)/
     > '"=" EXPECTED AFTER FILE INDICATOR.                          ',
     > '"HIDEOUT" MUST BE FOLLOWED BY LEFT PARENTHESIS.             ',
     > '"DISCARD" MUST END WITH A RIGHT PARENTHESIS.                ',
     > 'INVALID FILE NAME TO DISCARD.                               ',
     > 'FILE NAME EXPECTED.                                         ',
     > '"=" EXPECTED.                                               ',
     > '"DISCARD" MUST BE FOLLOWED BY LEFT PARENTHESIS.             ',
     > 'DISCARD STATEMENT SYNTACTICALLY INCOMPLETE.                 ',
     > '"=" EXPECTED AFTER ARRAY ELEMENT.                           ',
     > '"FROM" SHOULD FOLLOW MACRO NAME.                            ',
     > 'FILE NAME EXPECTED AFTER "FROM".                            ',
     > '"FROM" OR "EOL" EXPECTED AFTER MACRO NAME.                  ',
     > 'LEFT PARENTHESIS EXPECTED.                                  ',
     > 'IDENTIFIER EXPECTED.                                        ',
     > 'BE CAREFULL, YOU RE-INITIALIZE ARRAY X.                     '/
      DATA (MESAGE(I),I=121,135)/
     > '"." EXPECTED AFTER "X".                                     ',
     > 'IDENTIFIER EXPECTED AS FILE NAME, AFTER "X.".               ',
     > 'BE CAREFULL, YOU RE-INITIALIZE ARRAY L.                     ',
     > '"." EXPECTED AFTER "L".                                     ',
     > 'IDENTIFIER EXPECTED AS FILE NAME AFTER "L.".                ',
     > 'BE CAREFULL, YOU RE-INITIALIZE ARRAY R.                     ',
     > '"." EXPECTED AFTER "R".                                     ',
     > 'IDENTIFIER EXPECTED AS FILE NAME AFTER "R.".                ',
     > '"X", "L" OR "R" KEYWORD EXPECTED.                           ',
     > 'ILLEGAL CHARACTER SEQUENCE DETECTED.                        ',
     > 'INIT STATEMENT SYNTACTICALLY INCOMPLETE.                    ',
     > 'ARITHMETIC EXPRESSION EXPECTED INTO STATEMENT.              ',
     > 'IDENTIFIER EXPECTED AS LOOP CONTROL INDEX.                  ',
     > 'LOOP CONTROL INDEX NOT DECLARED.                            ',
     > 'NESTED LOOPS, HAVE THE SAME CONTROL INDEX.                  '/
      DATA (MESAGE(I),I=136,150)/
     > '"FROM" EXPECTED.                                            ',
     > '"TO" EXPECTED.                                              ',
     > '"BY" EXPECTED.                                              ',
     > '"ENDLOOP" NOT FOLLOWED BY EOL.                              ',
     > '"ENDLOOP" FOUND, WITH NO CORRESPONDING "LOOP" STATEMENT.    ',
     > 'A LOOP CONTROL VARIABLE SHOULD NOT BE ASSIGNED A VALUE.     ',
     > '"EXIT" NOT FOLLOWED BY EOL.                                 ',
     > '"EXIT" ILLEGAL OUTSIDE A LOOP STRUCTURE.                    ',
     > '"ELSE" FOUND WITHOUT A CORRESPONDING "IF".                  ',
     > '"ENDIF" FOUND WITHOUT A CORRESPONDING "IF".                 ',
     > 'ONE OR MORE LOOPS DO NOT TERMINATE. ("ENDLOOP" MISSING)     ',
     > 'ONE OR MORE BLOCK IFS DO NOT TERMINATE. ("ENDIF" MISSING)   ',
     > 'ILLEGAL JUMP INTO A BLOCK IF OR LOOP.                       ',
     > '"FROM" EXPECTED AFTER VARIABLE LIST.                        ',
     > 'IDENTIFIER EXPECTED AS FILE NAME AFTER "FROM".              '/
      DATA (MESAGE(I),I=151,165)/
     > '"FROM" EXPECTED AFTER VARIABLE LIST.                        ',
     > '"GET" STATEMENT NOT FOLLOWED BY "EOL".                      ',
     > 'IDENTIFIER IN ASSIGNMENT, IS NOT A SIMPLE VARIABLE.         ',
     > 'STATEMENT MUST END WITH RIGHT PARENTHESIS.                  ',
     > '"." EXPECTED AFTER KEYWORD.                                 ',
     > 'INCOMPATIBLE INDEXED PARAMETER WITH STATEMENT.              ',
     > 'LEFT PARENTHESIS EXPECTED.                                  ',
     > 'STATEMENT SYNTACTICALLY INCOMPLETE.                         ',
     > 'SIMPLE VARIABLE FOLLOWED BY "(", OR "[", INTO AN EXPRESSION.',
     > 'INTRINSIC VARIABLE FOLLOWED BY "(",OR "[".                  ',
     > 'REFERENCE TO A FUNCTION WITH NO "[".                        ',
     > 'SIMPLE ARRAY CALLED WITH NO "[" AFTER ITS NAME.             ',
     > 'INTRINSIC ARRAY CALLED WITH NO "[" AFTER ITS NAME.          ',
     > 'MORE THAN ONE SUBSCRIPT IN INTRINSIC ARRAY.                 ',
     > 'THE WHOLE MACRO FILE READ. TERMINATING "CLEAR" NOT FOUND.   '/
      DATA (MESAGE(I),I=166,MAXMES)/
     > 'MULTIPLE ENTRY OF A KEYWORD.                                ',
     > 'A LOOP CONTROL VARIABLE SHOULD NOT BE INPUT A VALUE.        ',
     > 'THIS COMMAND HAS NO CANCEL KEYWORD IN THE DESCRIPTION FILE. '/
C
C  CHECK IF THE MESSAGE, INPUT TO THE ROUTINE IS LONGER THAN 10
C  CHARACTERS. IF SO, TRUNCATE IT AND APPEND TO IT 3 DOTS.
C
      L=LENGTH(MESS)
      IF (L.LE.10) THEN
         WMESS=MESS
      ELSE
         WMESS=MESS(1:10)//'...'
      END IF
C
C  DISABLE CODE GENERATION.
C
      GENE=.FALSE.
C
C  CHECK IF THE ERROR CODE SHOULD BE DISPLAYED.
C
      IF (CDIS) THEN
         WRITE(ELINE,'(A,I4,4X,A)') 'ERROR* ',CODE,WMESS
      ELSE
         WRITE(ELINE,'(A,4X,4X,A)') 'ERROR* ',WMESS
      END IF
      ELINE(30:)=MESAGE(CODE)
C  PUT THE ERROR LINE, JUST CONSTRUCTED IN THE ERROR BUFFER.
      NERR=NERR+1
      IF (NERR.GT.MAXERR) CALL ABORT('ERROR BUFFER .FLOW.','ERROR')
      EBUFF(NERR)=ELINE
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE EXE ( TOKEN, NTOK, ALPHA, NUM, NT, NA, NN, TYPE, TDEF,
     >                 VAR, MVELAB, LOC, DIM, INFO, IPOS, KIND, LLOC )
C  ---------------------------------------------------------------------
C
C  THIS IS THE MAIN ROUTINE THAT DRIVES OTHER PROGRAM UNITS ACCORDING
C  TO THE STATEMENT TO BE PROCESSED. IT HANDLES ANY STATEMENT THAT MAY
C  APPEAR NEXT TO A "WHEN" STATEMENT. UPON ENTRY IT ASSUMES THAT THE
C  FIRST TOKEN ENCOUNTERED IS AN IDENTIFIER. ( A COMMAND OR THE NAME OF
C  A VARIABLE, IN CASE OF AN ASSIGNMENT)
C  TWO ARRAYS ARE DEFINED IN THIS ROUTINE:
C  1) A CHARACTER ARRAY ( NAMED STAT ) THAT KEEPS THE STATEMENTS THIS
C     ROUTINE HANDLES, AND
C  2) AN INTEGER ARRAY ( NAMED NAME ) THAT KEEPS AN IDENTIFICATION CODE
C     FOR EACH STATEMENT, IN CASE A ROUTINE TAKES CARE OF MORE THAN
C     ONE STATEMENT.
C
C  EXE WILL FIRST CHECK IF THE NEXT TOKEN IN THE SEQUENCE IS AN =
C  OR A [. IN SUCH A CASE IT WILL ASSUME THAT AN ASSIGNMENT TAKES PLACE.
C  OTHERWISE IT WILL CHECK WHETHER THE FIRST IDENTIFIER UPON ENTRY IS
C  ONE OF THE VALID COMMANDS ( DEFINED IN THE STAT ARRAY ), AND IF SO
C  IT WILL PERFORM A COMPUTED GOTO AND CALL THE APPROPRIATE ROUTINE TO
C  PROCESS THE STATEMENT.
C  OBJECT CODE IS GENERATED BY THIS ROUTINE ONLY IN CASE OF AN
C  ASSIGNMENT AS:
C
C  <CODE TO CALCULATE MEMORY ADDRESS OF THE ELEMENT TO GET THE VALUE>
C  <CODE TO CALCULATE THE EXPRESSION ON THE RIGHT SIDE OF THE = SIGN>
C  $POPCONT.
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C               THE INPUT LINE.
C    NUM        REAL ARRAY. THE NUMBERS ENCOUNTERED IN THE INPUT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    NT         INTEGER VARIABLE. THIS IS A POINTER TO THE TOKEN ARRAY
C               AND IS THE NUMBER OF TOKENS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    NA         INTEGER VARIABLE. THIS IS A POINTER TO THE ALPHA ARRAY
C               AND IS THE NUMBER OF IDENTIFIERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    NN         INTEGER VARIABLE. THIS IS A POINTER TO THE NUM ARRAY
C               AND IS THE NUMBER OF NUMBERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    VAR        CHARACTER ARRAY. THIS IS THE VAR TABLE.
C    LOC        INTEGER ARRAY. THIS IS THE MEMORY LOCATION TABLE.
C    DIM        INTEGER ARRAY. THIS IS THE DIMENSIONALITY TABLE.
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    IPOS       INTEGER ARRAY. THIS IS THE POSITION TABLE.
C    KIND       INTEGER ARRAY. THIS IS THE KIND TABLE.
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C    LLOC       INTEGER ARRAY. THIS IS THE LOOP TABLE.
C    MVELAB     CHARACTER ARRAY. THIS IS THE MOVE TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES LOCAN, ERROR, INSERT, EXTRA, XPR, ABORT, NOPAR,
C                GRADQ, REWI, DISCA, DFL, PICK, STEPO, GODFA, MULTI,
C                EXECU, MOVE, DISPLA, GET, ACCUM, HIDE, INI, MARGI,
C                EXITL.
C
C  NOTES:
C    IF AN ARRAY IS BEING ASSIGNED A VALUE, THE WHOLE LEFT SIDE OF THE
C    = SIGN IS CONSIDERED AS AN EXPRESSION AND SUBROUTINE XPR IS CALLED
C    TO PROCESS IT. XPR WILL GENERATE CODE TO CALCULATE THE ARRAY
C    ELEMENT ADDRESS AND PUSH IT ON THE TOP OF THE STACK. THEN A
C    $GETCONT INSTRUCTION WILL BE GENERATED BY THE SAME ROUTINE, SO AS
C    WHEN EXECUTED, TO REPLACE THE TOP OF THE STACK WITH THE ACTUAL
C    VALUE OF THE ARRAY ELEMENT.
C    THE OBJECT CODE JUST DESCRIBED WILL BE WRITTEN IN THE SPARE FILE.
C    AS SOON AS XPR RETURNS, ALL OF THE SPARE FILE WILL BE COPIED TO
C    THE SCRATCH FILE, EXCLUDING THE FINAL $GETCONT INSTRUCTION.
C
C  ---------------------------------------------------------------------
C
C  THE NUMBER OF STATEMENTS THAT MAY FOLLOW A "WHEN" STATEMENT".
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF A LINE. (CHARS)
      PARAMETER (  LINLEN = 80  )
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER ( IDLEN = 30 )
C
C  NUMBER OF BUILD-IN MCL STATEMENTS, THIS ROUTINE HANDLES.
C  COMMANDS READ IN FROM THE DESCRIPTION FILE ARE EXCLUDED FROM THIS COUNT.
      PARAMETER ( NES = 67 )
C
      INTEGER TOKEN(*),TYPE(*),TDEF(*),LOC(*),DIM(*),INFO(*),IPOS(*),
     >        KIND(*),LLOC(*)
      CHARACTER*(*) ALPHA(*),MVELAB(*),VAR(*)
      REAL NUM(*)
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      CHARACTER*(IDLEN) VARID,STARG(1)
      CHARACTER*80 LINE
      INTEGER NAME(NES), EX(LINLEN)
      INTEGER INDE(NES)
      LOGICAL FOUND, CORRE, FIRST
      CHARACTER*10 STAT(NES)
C
      SAVE FIRST, STAT, NAME, INDE
      DATA FIRST / .TRUE. /
C
C  STAT HOLDS THE EXECUTABLE COMMANDS.
C  NAME IS AN IDENTIFICATION CODE FOR EACH ONE OF THE STAT ARRAY COMMANDS.
C  IT IS USED WHEN A SINGLE SUBROUTINE PROCESSES MORE THAN ONE COMMAND.
C
      DATA (STAT(I), NAME(I), I=1,6) / 
     >           'ADJUST    ', 0,
     >           'ANAL      ', 0,
     >           'CATALOG   ', 0,
     >           'DATASKIP  ', 0,
     >           'DATAWRITE ', 0,
     >           'DEMARGIN  ', 1 /
      DATA (STAT(I), NAME(I), I=7,15) / 
     >           'DISCARD   ', 0,
     >           'DISKOFF   ', 0,
     >           'DISKON    ', 0,
     >           'DISPLAY   ', 0,
     >           'DISPOSKIP ', 0,
     >           'DISPOWRITE', 0,
     >           'EVALUATE  ', 0,
     >           'EXECUTE   ', 0,
     >           'EXIT      ', 0 /
      DATA (STAT(I), NAME(I), I=16,25) / 
     >           'FAST      ', 0,
     >           'FINISH    ', 0,
     >           'FIX       ', 2,
     >           'FIXALL    ', 0,
     >           'FLAG      ', 3,
     >           'FLAGDIS   ', 0,
     >           'FULLBACK  ', 0,
     >           'FULLPRINT ', 0,
     >           'GET       ', 0,
     >           'GODFATHER ', 0 /
      DATA (STAT(I), NAME(I), I=26,34) / 
     >           'GRADCHECK ', 1,
     >           'GRADDIS   ', 2,
     >           'CFLAGDIS  ', 0,
     >           'HALFPRINT ', 0,
     >           'HIDEOUT   ', 0,
     >           'INDEX     ', 0,
     >           'INIT      ', 0,
     >           'LASTBACK  ', 0,
     >           'LIMITS    ', 0 /
      DATA (STAT(I), NAME(I), I=35,44) / 
     >           'LOOSALL   ', 0,
     >           'LOOSE     ', 3,
     >           'MARGIN    ', 0,
     >           'MEMO      ', 0,
     >           'MODEDIS   ', 0,
     >           'MOVE      ', 0,
     >           'MOVETO    ', 0,
     >           'NAME      ', 0,
     >           'NOBACK    ', 0,
     >           'NOEVAL    ', 0 /
      DATA (STAT(I), NAME(I), I=45,54) / 
     >           'NOPRINT   ', 0,
     >           'NOTARGET  ', 0,
     >           'NUMER     ', 0,
     >           'PANELOFF  ', 0,
     >           'PANELON   ', 0,
     >           'PAUSE     ', 0,
     >           'PICK      ', 0,
     >           'POINT     ', 1,
     >           'QUAD      ', 0,
     >           'QUIT      ', 3 /
      DATA (STAT(I), NAME(I), I=55,61) / 
     >           'RESET     ', 0,
     >           'RETURN    ', 0,
     >           'REVEAL    ', 0,
     >           'REWIND    ', 0,
     >           'SHORTDIS  ', 0,
     >           'STEP      ', 2,
     >           'STEPALL   ', 0 /
      DATA (STAT(I), NAME(I), I=62,NES) / 
     >           'STEPDIS   ', 0,
     >           'STOP      ', 0,
     >           'VALDIS    ', 0,
     >           'GENERAL   ', 0,
     >           'SOS       ', 0,
     >           'ALIASDIS  ', 0 /
C
C  SORT THE COMMAND ARRAY IF THIS IS THE FIRST ENTRY OF THE ROUTINE.
C  NECESSARY WHEN EXPANDING THE MCL COMMAND SET.
C
      IF (FIRST) THEN
         FIRST = .FALSE.
         DO 69,I=1,NES
69       INDE(I) = I
         CALL CMSORT(STAT,NAME,INDE,NES)
      END IF
C
      NT=NT+1
      NA=NA+1
C
C  GET THE STATEMENT, AND CHECK IF ITS AN ASSIGNMENT.
C
      VARID=ALPHA(NA)
      CUR=TOKEN(NT+1)
      IF (CUR.EQ.EQ .OR. CUR.EQ.LBRA) THEN
C  THIS IS AN ASSIGNMENT.
         NT=NT+1
C  CHECK IF VARIABLE PRESENT IN THE VAR TABLE.
         CALL LOCAN(VAR,NVAR,VARID,FOUND,IND)
         IF (.NOT.FOUND) THEN
C  VARIABLE NOT FOUND IN THE VAR TABLE.
            CALL ERROR(19,VARID)
C  PUT VARIABLE IN THE VAR TABLE. UPDATE RELATED TABLES ALSO.
            CALL INSERT(VAR,DIM,LOC,IPOS,KIND,VARID,0,0,0,-1,IND)
         END IF
C
C  CALCULATE ADDRESS OF THE ELEMENT THAT ACCEPTS THE VALUE.
C
         IF (CUR.EQ.EQ) THEN
            K=KIND(IND)
C  IF AN = SIGN WAS FOUND, ONLY SIMPLE VARIABLES OR VARIABLES THAT
C  HAVE NOT BEEN DECLARED ARE ALLOWED.
            IF (K.NE.1 .AND. K.NE.-1) CALL ERROR(153,VARID)
C  CHECK WHETHER THE VARIABLE IS USED AS A LOOP CONTROL INDEX.
            L=LOC(IND)
            DO 62,I=1,NLOO
              IF (LLOC(I).EQ.L) THEN
                 CALL ERROR(141,VARID)
                 GOTO 63
              END IF
 62         CONTINUE
C  GENERATE NECESSARY CODE TO PUSH THE ADDRESS OF THE VARIABLE ON
C  THE TOP OF THE STACK.
 63         BLOC=BLOC+2
            IF (GENE) THEN
               WRITE(SCR,'(A)') '$PUSHC'
               WRITE(SCR,*) LOC(IND)
            END IF
C  CORRE INDICATES WHETHER CALCULATION OF THE ADDRESS OF THE ELEMENT
C  ON THE LEFT SIDE OF THE = SIGN WAS SUCCESSFUL.
            CORRE=.TRUE.
         ELSE
C  THIS IS THE CASE OF AN ARRAY ELEMENT BEING ASSIGNED A VALUE.
            NT=NT-2
            NA=NA-1
C  EXTRACT THE ARRAY ELEMENT FROM THE SEQUENCE OF TOKENS.
            CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,3,ALPHA,NA)
C  NLS COUNTS THE OBJECT CODE LINES NECESSARY FOR THE EVALUATION OF
C  THE MEMORY ADDRESS OF THE ARRAY ELEMENT.
            NLS=0
            OPEN(SPARE,STATUS='SCRATCH',ERR=222)
            REWIND SPARE
C  THE ARRAY ELEMENT IS PROCESSED AS AN EXPRESSION. OBJECT CODE
C  WILL BE WRITTEN ON THE SPARE FILE, WHILE THE NUMBER OF LINES
C  WRITTEN ON THIS FILE IS COUNTED BY NLS.
            CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,
     >               NLS,LOC,DIM,INFO,IPOS,KIND,SPARE,STARG)
            IF (GENE) THEN
C  COPY ALL OF THE SPARE FILE TO THE SCRATCH FILE EXCLUDING THE
C  LAST LINE. ( A $GETCONT INSTRUCTION )
               REWIND SPARE
               DO 88,I=1,NLS-1
                 READ(SPARE,'(A)') LINE
                 WRITE(SCR,'(A)') LINE
  88           CONTINUE
C  UPDATE THE SCRATCH FILE COUNTER.
               BLOC=BLOC+NLS-1
            END IF
            CLOSE(SPARE,ERR=223)
C  ARRAY ELEMENT LOCATION IS NOW ON THE TOP OF THE STACK.
            IF (CORRE) THEN
C  CONFIRM THAT AN = SIGN FOLLOWS.
               NT=NT+1
               IF (TOKEN(NT).NE.EQ) THEN
                  CALL ERROR(114,' ')
                  RETURN
               END IF
            END IF
         END IF
C
C  PROCESS EXPRESSION ON THE LEFT SIDE OF THE "=" SIGN.
C
         IF (CORRE) THEN
C  EXTRACT THE EXPRESSION FROM THE SEQUENCE OF TOKENS.
            CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,0,ALPHA,NA)
C  EVALUATE THE EXPRESSION.
            CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,BLOC,
     >               LOC,DIM,INFO,IPOS,KIND,SCR,STARG)
C  IF THE EXPRESSION IS INCORRECT.
            IF (.NOT.CORRE) RETURN
C  WRITE THE FINAL $POPCONT INSTRUCTION.
            BLOC=BLOC+1
            IF (GENE) WRITE(SCR,'(A)') '$POPCONT'
C  CHECK IF ANYTHING ELSE BUT AN EOL FOLLOWS THE EXPRESSION.
            NT=NT+1
            IF (NT.NE.NTOK) CALL ERROR(20,' ')
         END IF
C
C  CHECK IF THE VARIABLE THAT ACCEPTS THE EXPRESSION'S VALUE IS A
C  MERLIN VARIABLE, MERLIN ARRAY OR INTRINSIC FUNCTION.
C
         K=KIND(IND)
         IF (K.EQ.4 .OR. K.EQ.5 .OR. K.EQ.6) THEN
            CALL ERROR(18,' ')
            RETURN
         END IF
      ELSE
C  NEITHER = NOR [ FOLLOWS THE FIRST TOKEN.
C  THIS IS NOT THE CASE OF AN ASSIGNMENT.
C  CHECK IF COMMAND PRESENT IN THE STAT ARRAY.
         CALL LOCAN(STAT,NES,VARID,FOUND,IND)
         IF (FOUND) THEN
C
C  JUMP TO THE APPROPRIATE LABEL, AND PROCESS THE STATEMENT.
C
            GOTO ( 1,  1,
     >              1,  1, 1, 5,
     >              4, 1, 1, 14, 1, 1, 1, 11,
     >             20, 1, 10,  5,  1, 7, 1, 1,
     >              1, 15,  8,  2,
     >              2,  1,  1,
     >             17,  1, 18,  1,
     >              1, 1,  5, 19,  1,
     >              1, 13, 12,  1,
     >              1,  1, 1,  1, 1,  1,
     >              1, 10,  6,  7,
     >              1, 2,   1,  1,
     >              1,  3,  1,
     >              7,  1,  1, 1,
     >              1, 1, 1, 1, 1 ) , INDE(IND)
C
            CALL ABORT('PROGRAMMING ERROR. IND INVALID.','EXE')
         ELSE
C  THE STATEMENT WAS NOT FOUND IN THE STAT ARRAY.
C  MAYBE IN THE DESCRIPTION FILE.
         CALL MULTI(TOKEN,NT,ALPHA,NA,NUM,NN,NTOK,VAR,ID,VARID,
     >                  LOC,DIM,INFO,IPOS,KIND)
         ENDIF
      END IF
      RETURN
C
C  NO PARAMETER COMMANDS ALREADY IN THE MERLIN OPERATING SYSTEM.
C
 1    CALL NOPAR(NTOK,NT,VARID,0)
      RETURN
C
C  GRADCHECK - GRADDIS - QUIT.
C
 2    ID=NAME(IND)
      CALL GRADQ(ID,TOKEN,NTOK,NT,ALPHA,NA,NUM,NN,VAR,LOC,DIM,
     >          INFO,IPOS,KIND,STARG)
      RETURN
C
C  REWIND.
C
 3    CALL REWI(TOKEN,NTOK,NT,ALPHA,NA)
      RETURN
C
C  DISCARD.
C
 4    CALL DISCA(TOKEN,NTOK,NT,ALPHA,NA)
      RETURN
C
C  DEMARGIN - FIX - LOOSE.
C
 5    ID=NAME(IND)
      CALL DFL(ID,TOKEN,NTOK,NT,ALPHA,NA,NUM,NN,VAR,LOC,DIM,
     >         INFO,IPOS,KIND,STARG)
      RETURN
C
C  PICK.
C
 6    CALL PICK(TOKEN,NTOK,NT,ALPHA,NA,NUM,NN,VAR,LOC,DIM,
     >          INFO,IPOS,KIND,STARG)
      RETURN
C
C  POINT - STEP.
C
 7    ID=NAME(IND)
      CALL STEPOF(TOKEN,NTOK,NT,ALPHA,NA,NUM,NN,VAR,ID,LOC,DIM,
     >           INFO,IPOS,KIND)
      RETURN
C
C  GODFATHER.
C
 8    CALL GODFA(TOKEN,NTOK,NT,ALPHA,NA,NUM,NN,VAR,LOC,DIM,
     >           INFO,IPOS,KIND,STARG)
      RETURN
C
C   NO PARAMETER COMMANDS ADDED TO MERLIN FOR MCL USE ONLY.
C
 10   CALL NOPAR(NTOK,NT,VARID,1)
      RETURN
C
C  EXECUTE.
C
 11   CALL EXECU(TOKEN,NTOK,NT,ALPHA,NA)
      RETURN
C
C  MOVE TO. (ENTERED AS "MOVETO" )
C
 12   CALL MOVE(TOKEN,NTOK,ALPHA,NT,NA,MVELAB,TYPE,TDEF)
      RETURN
C
C  MOVE TO. (ENTERED AS "MOVE TO")
C
 13   CONTINUE
C  AFTER "MOVE" AN IDENTIFIER IS EXPECTED. ( "TO" )
      NT=NT+1
      IF (TOKEN(NT).EQ.IDENT) THEN
C  CHECK IF THE IDENTIFIER IS "TO".
         NA=NA+1
         IF (ALPHA(NA).EQ.'TO') THEN
            CALL MOVE(TOKEN,NTOK,ALPHA,NT,NA,MVELAB,TYPE,TDEF)
         ELSE
C  "TO" DOES NOT FOLLOW "MOVE".
            CALL ERROR(17,ALPHA(NA))
         END IF
      ELSE
C  AN IDENTIFIER DOES NOT FOLLOW "MOVE".
         CALL ERROR(17,' ')
      END IF
      RETURN
C
C  DISPLAY.
C
 14   CALL DISPLA(TOKEN,NTOK,NT,ALPHA,NA,VAR,NUM,NN,LOC,DIM,INFO,
     >            IPOS,KIND)
      RETURN
C
C  GET.
C
 15   CALL GET(TOKEN,NTOK,ALPHA,VAR,NT,NA,NUM,NN,LOC,DIM,INFO,
     >         IPOS,KIND,LLOC)
      RETURN
C
C  HIDEOUT.
C
 17   CALL HIDE(TOKEN,NTOK,NT,ALPHA,NA)
      RETURN
C
C  INIT.
C
 18   CALL INI(TOKEN,NTOK,NT,ALPHA,NA)
      RETURN
C
C  MARGIN.
C
 19   CALL MARGI(TOKEN,NTOK,NT,ALPHA,NA,NUM,NN,VAR,LOC,DIM,
     >           INFO,IPOS,KIND)
      RETURN
C
C  EXIT.
C
 20   CALL EXITL(NT,NTOK,TYPE,TDEF)
      RETURN
C
C  AN ERROR OCCURRED WHILE OPENING THE SPARE FILE.
C
 222  CALL ABORT('COULD NOT OPEN THE SPARE FILE.','EXE')
C
C  AN ERROR OCCURRED WHILE CLOSING THE SPARE FILE.
C
 223  CALL ABORT('COULD NOT CLOSE THE SPARE FILE.','EXE')
C
C  AN ERROR OCCURRED WHILE OPENING THE DESCRIPTION FILE.
C
 224  CALL ABORT('COULD NOT OPEN THE DESCRIPTION FILE.','COCAN')
C
C  AN ERROR OCCURRED WHILE CLOSING THE DESCRIPTION FILE.
C
 225  CALL ABORT('COULD NOT CLOSE THE DESCRIPTION FILE.','COCAN')
      END
C  ---------------------------------------------------------------------
      SUBROUTINE EXECU ( TOKEN, NTOK, NT, ALPHA, NA )
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE PARSES THE "EXECUTE" STATEMENT AND GENERATES
C  APPROPRIATE OBJECT CODE.
C  THE SYNTAX OF THIS STATEMENT IS AS:
C
C  "EXECUTE" "." <MACRONAME>    OR
C  "EXECUTE" "." <MACRONAME> "FROM" <FILENAME>
C
C  WHERE <MACRONAME> IS THE NAME OF THE MACRO THAT IS TO BE INCLUDED
C  IN THE OBJECT CODE, AND <FILENAME> IS THE FILE IN WHICH THE
C  SPECIFIED MACRO RESIDES.
C  IF <FILENAME> IS NOT PRESENT THE DEFAULT MERLIN FILE: MACROF WILL
C  BE USED. IF THE SYNTAX OF THE STATEMENT IS CORRECT, THE COMPILER
C  WILL ATTEMPT TO OPEN THE SPECIFIED <FILENAME> AND FIND IN IT THE
C  GIVEN <MACRONAME>. A VALID MERLIN MACRO IS AS FOLLOWS:
C
C  *
C  .<MACRONAME>
C  <CODE FOR THE MACRO>
C  CLEAR
C
C  WHERE <CODE FOR THE MACRO> IS THE SEQUENCE OF MERLIN COMMANDS THAT
C  CONSTITUTE THE MACRO.
C  ONLY THE <CODE FOR THE MACRO> IS INCLUDED IN THE COMPILERS
C  OBJECT CODE.
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C               THE INPUT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    NT         INTEGER VARIABLE. THIS IS A POINTER TO THE TOKEN ARRAY
C               AND IS THE NUMBER OF TOKENS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    NA         INTEGER VARIABLE. THIS IS A POINTER TO THE ALPHA ARRAY
C               AND IS THE NUMBER OF IDENTIFIERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES ERROR, ABORT.
C
C  ABORT CONDITIONS THAT MAY OCCUR:
C    THIS ROUTINE USES THE SPARE FILE, SO WHEN ITS CLOSING FAILS,
C    THE PROGRAM WILL STOP AND ISSUE AN INFORMATIVE MESSAGE.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER (  IDLEN = 30   )
C
      CHARACTER*(*) ALPHA(*)
      INTEGER TOKEN(*)
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      CHARACTER*(IDLEN) MACID,FILEID,LINE*80
C
C  THE PERIOD IS REQUESTED BEFORE THE MACRO NAME.
C
      NT=NT+1
      IF (TOKEN(NT).NE.PERIOD) THEN
         CALL ERROR(21,' ')
         RETURN
      END IF
C
C  AN IDENTIFIER IS EXPECTED AS A MACRO NAME.
C
      NT=NT+1
      IF (TOKEN(NT).NE.IDENT) THEN
         CALL ERROR(22,' ')
         RETURN
      END IF
C
C  GET THE MACRO NAME.
C
      NA=NA+1
      MACID='.'//ALPHA(NA)
C
C  CHECK IF FILE SPECIFICATION IS PRESENT.
C
      NT=NT+1
      CUR=TOKEN(NT)
      IF (CUR.EQ.IDENT) THEN
C  AN IDENTIFIER FOLLOWS THE <MACRONAME>.
         NA=NA+1
         IF (ALPHA(NA).NE.'FROM') THEN
C  "FROM" DOES NOT FOLLOW THE <MACRONAME>.
            CALL ERROR(115,ALPHA(NA))
            RETURN
         END IF
         NT=NT+1
         IF (TOKEN(NT).NE.IDENT) THEN
C  AN IDENTIFIER ( STANDING FOR <FILENAME> ) SHOULD FOLLOW
C  AFTER "FROM".
            CALL ERROR(116,' ')
            RETURN
         END IF
         NA=NA+1
C  GET THE <FILENAME>.
         FILEID=ALPHA(NA)
C  END OF THE INPUT LINE MUST FOLLOW THE <FILENAME>.
         NT=NT+1
         IF (NT.NE.NTOK) CALL ERROR(23,' ')
      ELSE IF (NT.EQ.NTOK) THEN
C  FILE NOT SPECIFIED. USE THE DEFAULT MERLIN FILE: "MACROF".
         FILEID='MACROF'
      ELSE
C  NEITHER AN IDENTIFIER NOR AN END OF LINE FOLLOWS THE <MACRONAME>.
         CALL ERROR(117,' ')
         RETURN
      END IF
C
C  OPEN THE MACRO FILE.
C
      OPEN(SPARE,FILE=FILEID,STATUS='OLD',IOSTAT=IERR)
      IF (IERR.NE.0) THEN
C  AN ERROR OCCURRED DURING THE MACRO FILE OPENING.
         CALL ERROR(24,' ')
         RETURN
      END IF
C
C  TRY TO FIND THE MACRO.
C  A LEGAL MACRO IS AS:       *
C                             .<MACRONAME>
C                             <CODE FOR THE MACRO>
C                             CLEAR
C
C  THIS SEQUENCE OF LINES IS IN QUEST.
C
      REWIND SPARE
C  READ LINES FROM THE MACRO FILE UNTIL THE INITIAL * IS REACHED.
 10   READ(SPARE,'(A)',END=100) LINE
      IF (LINE.NE.'*') GOTO 10
C  THE INITIAL * HAS BEEN FOUND. THE MACRO NAME PREFIXED BY A
C  PERIOD, SHOULD FOLLOW.
      READ(SPARE,'(A)',END=200) LINE
      IF (LINE.NE.MACID) GOTO 10
C  COPY THE LINES OF THE MACRO TO THE OBJECT CODE FILE,
C  UNTIL THE TERMINATING CLEAR IS FOUND.
      INILOC=BLOC
 20   READ(SPARE,'(A)',END=400) LINE
      IF (LINE.EQ.'CLEAR') GOTO 300
      BLOC=BLOC+1
      WRITE(SCR,'(A)') LINE
      GOTO 20
C
C  END OF THE MACRO FILE HAS BEEN REACHED, BUT THE REQUESTED MACRO,
C  WAS NOT FOUND.
C
 100  CALL ERROR(25,' ')
      CLOSE(SPARE,ERR=1010)
      RETURN
C
C  END OF THE MACRO FILE HAS BEEN REACHED. ONLY THE * HAS BEEN FOUND.
C
 200  CALL ERROR(26,' ')
      CLOSE(SPARE,ERR=1010)
      RETURN
C
C  THE REQUESTED MACRO HAS BEEN FOUND AND PROCESSED.
C  IF NO CODE WAS GENERATED DUE TO THE MACRO, THIS IS AN ERROR.
C
 300  IF (BLOC-INILOC.EQ.0) CALL ERROR(27,' ')
      CLOSE(SPARE,ERR=1010)
      RETURN
C
C  END OF THE MACRO FILE HAS BEEN REACHED, BUT THE TERMINATING
C  CLEAR WAS NOT FOUND.
C
 400  CALL ERROR(165,' ')
      CLOSE(SPARE,ERR=1010)
      RETURN
C
C  PROBLEMS ENCOUNTERED WHILE CLOSING THE SPARE FILE.
C
 1010 CALL ABORT('COULD NOT CLOSE THE SPARE FILE.','EXECU')
C
      END
C  ---------------------------------------------------------------------
      SUBROUTINE EXITL ( NT, NTOK, TYPE, TDEF )
C  ---------------------------------------------------------------------
C
C  HANDLES THE EXIT STATEMENT. AN MCL ERROR WILL BE ISSUED IF THE
C  STATEMENT IS NOT USED INSIDE A LOOP STRUCTURE.
C  THE OBJECT CODE GENERATED IS:
C
C  $MOVE
C  <BLANK LINE>
C
C  THE TARGET OF THE $MOVE INSTRUCTION, IS THE $CONTINUE INSTRUCTION
C  GENERATED BY THE "ENDLOOP" STATEMENT OF THE LOOP.
C  A VALUE OF 8 IS PUSHED IN THE TYPE TABLE ( TO INDICATE THE EXIT
C  STATEMENT ), WHILE THE LOCATION IN THE SCRATCH FILE OF THE ABOVE
C  <BLANK LINE> IS PUSHED IN THE TYPE-DEFINITION TABLE.
C
C  INPUT ARGUMENTS:
C
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    NT         INTEGER VARIABLE. THIS IS THE NUMBER OF TOKENS USED
C               SO FAR BY THE PREVIOUSLY CALLED PARSING ROUTINES.
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINE ERROR.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      INTEGER TYPE(*),TDEF(*)
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      LOGICAL ERR
C  ERR WILL BE SET TO .TRUE. IF "EXIT" IS FOLLOWED BY ANYTHING ELSE
C  BUT AN EOL, OR IF THE STATEMENT IS USED OUTSIDE A LOOP STRUCTURE.
      ERR=.FALSE.
C
C  IF ANYTHING ELSE THAN AN EOL FOLLOWS THE "EXIT" STATEMENT,
C  THIS IS AN ERROR.
C
      NT=NT+1
      IF (NT.NE.NTOK) THEN
         CALL ERROR(142,' ')
         ERR=.TRUE.
      END IF
C
C  "EXIT" IS VALID ONLY INSIDE A LOOP.
C
      IF (NLOO.EQ.0) THEN
         CALL ERROR(143,' ')
         ERR=.TRUE.
      END IF
C
      IF (ERR) RETURN
C  GENERATE CODE.
      BLOC=BLOC+2
      IF (GENE) WRITE(SCR,'(A/)') '$MOVE'
C  UPDATE THE TYPE AND TYPE-DEFINITION TABLES.
      NTYPE=NTYPE+1
      IF (NTYPE.GT.MAXTYP)
     >           CALL OVER('TYPE','TYPE','EXITL',MAXTYP,'MXTYP')
      TYPE(NTYPE)=8
      TDEF(NTYPE)=BLOC
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE EXTRA ( TOKEN,NTOK,NT,EX,NEX,IOPT,ALPHA,NA )
C  ---------------------------------------------------------------------
C
C  EXTRACTS FROM A LINE, ALL TOKENS, VALID IN AN MCL EXPRESSION AND
C  STORES THEM IN AN ARRAY THAT IS RETURNED AS OUTPUT.
C  THE LINE IS GIVEN IN THE FORM OF TOKENS. EXPRESSION SCANNING ALWAYS
C  STOPS WHEN AN INVALID TOKEN IS REACHED. ADDITIONALLY ACCORDING TO AN
C  INPUT OPTION, EXPRESSION SCANNING STOPS WHEN DIFFERENT SITUATIONS
C  ARE MET.
C
C  A TOKEN IS ACCEPTED IN AN EXPRESSION SEQUENCE, IF IT'S ONE
C  OF THE FOLLOWING:
C
C  RIGHT PARENTHESIS       )           LEFT PARENTHESIS         (
C  MULTIPLICATION          *           DIVISION                 /
C  ADDITION                +           SUBTRACTION              -
C  IDENTIFIER                          UNSIGNED NUMBER
C  RAISE TO A POWER       **           COMMA                    ,
C  RIGHT BRACKET           ]           LEFT BRACKET             [
C  GREATER THAN            >           LESS THAN SIGN           <
C  GREATER OR EQUAL THAN  >=           LESS OR EQUAL THAN      <=
C  EQUALITY OPERATOR     :=:           NOT EQUAL                #
C  EXCLUSIVE OR OPERATOR               NOT OPERATOR
C  AND OPERATOR                        OR OPERATOR
C
C  INPUT ARGUMENTS:
C
C    TOKEN   INTEGER ARRAY. THE TOKENS IN THE CURRENT LINE.
C    NTOK    INTEGER VARIABLE. NUMBER OF TOKENS IN THE CURRENT LINE.
C    ALPHA   CHARACTER*(*) ARRAY. THE IDENTIFIERS IN THE CURRENT LINE.
C    NA      INTEGER VARIABLE. NUMBER OF IDENTIFIERS IN THE CURRENT
C            LINE, THAT HAVE BEEN USED BY PREVIOUSLY CALLED
C            PARSING ROUTINES.
C    IOPT    INTEGER VARIABLE. THIS IS AN OPTION.
C            IT MAY TAKE ON THE VALUES:
C               0  EXPRESSION SCANNING STOPS AT THE FIRST INVALID
C                  TOKEN REACHED.
C               1  EXPRESSION SCANNING STOPS ALSO WHEN ONE OF THE
C                  IDENTIFIERS: THEN, JUST, FROM, BY, TO IS ENCOUNTERED.
C               2  EXPRESSION SCANNING STOPS ALSO WHEN A RIGHT
C                  PARENTHESIS IS REACHED BEFORE THE END OF THE LINE.
C               3  EXPRESSION SCANNING STOPS ALSO AT THE EQUALITY
C                  SIGN ( = ) APPEARING AFTER A RIGHT BRACKET.
C
C  OUTPUT ARGUMENTS:
C
C    EX      INTEGER ARRAY. UPON RETURN, CONTAINS THE VALID TOKENS
C            THAT HAVE BEEN FOUND IN THE TOKEN ARRAY.
C    NEX     INTEGER VARIABLE. NUMBER OF ENTRIES IN THE EX ARRAY.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    NT      WHEN THE ROUTINE IS ENTERED, THIS IS THE NUMBER OF TOKENS
C            USED BY THE PREVIOUSLY CALLED PARSING MODULES. WHEN THIS
C            ROUTINE RETURNS, THIS IS THE TOTAL NUMBER OF TOKENS
C            ( INCLUDING THE ONES EXTRACTED AS AN EXPRESSION ) USED
C            FROM THE CURRENT SOURCE LINE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINE ABORT.
C    FUNCTION   RESE.
C
C  ABORT CONDITIONS THAT MAY OCCUR:
C    IF AN END OF LINE CODE IS NOT PRESENT IN THE TOKEN ARRAY
C    AND ALL PREVIOUS CODES ARE ACCEPTED IN AN EXPRESSION, THE
C    PROGRAM WILL STOP AND ISSUE AN INFORMATIVE MESSAGE.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      INTEGER TOKEN(*),EX(*)
      CHARACTER*(*) ALPHA(*)
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      LOGICAL RESE,OK
C
C  THE FOLLOWING STATEMENT FUNCTION WILL TEST IF A TOKEN IS ACCEPTED
C  IN AN MCL EXPRESSION.
C
      OK(M)=M.EQ.RIGHTP .OR. M.EQ.LEFTP .OR.
     >      M.EQ.STAR   .OR. M.EQ.SLASH .OR.
     >      M.EQ.PLUS   .OR. M.EQ.MINUS .OR.
     >      M.EQ.IDENT  .OR. M.EQ.UNSI  .OR.
     >      M.EQ.BISTAR .OR. M.EQ.COMMA .OR.
     >      M.EQ.RBRA   .OR. M.EQ.LBRA  .OR.
     >      M.EQ.GT     .OR. M.EQ.LT    .OR.
     >      M.EQ.GE     .OR. M.EQ.LE    .OR.
     >      M.EQ.EQUAL  .OR. M.EQ.NE    .OR.
     >      M.EQ.XOR    .OR. M.EQ.NOT   .OR.
     >      M.EQ.AND    .OR. M.EQ.OR
C
C  INIT IS THE NEXT AVAILABLE TOKEN TO BE SEARCHED.
      INIT=NT+1
      NNA=NA
C  NEX IS THE NUMBER OF TOKENS IN THE EXPRESSION STREAM.
      NEX=0
C
C  THE FOLLOWING DO-LOOP WILL CHECK ALL TOKENS IN THE TOKEN ARRAY.
C
      DO 10,I=INIT,NTOK
        CUR=TOKEN(I)
        IF (OK(CUR)) THEN
C  THE TOKEN IS ACCEPTED IN THE EXPRESSION.
           IF (IOPT.EQ.1 .AND. CUR.EQ.IDENT) THEN
C  CHECK IF ONE OF THE RESERVED IDENTIFIERS.
              NNA=NNA+1
              IF (RESE(ALPHA(NNA))) THEN
                 NT=I-1
                 RETURN
              END IF
           ELSE IF (IOPT.EQ.3 .AND. I.NE.INIT) THEN
              IF (CUR.EQ.EQ .AND. TOKEN(I-1).EQ.RBRA) THEN
                 NT=I-1
                 RETURN
              END IF
           END IF
           NEX=NEX+1
           EX(NEX)=CUR
        ELSE
C  AN INVALID TOKEN HAS BEEN FOUND.
           IF (IOPT.EQ.2  .AND.
     >         CUR.EQ.EOL .AND. TOKEN(I-1).EQ.RIGHTP) THEN
              NT=I-2
              NEX=NEX-1
           ELSE
              NT=I-1
           END IF
           RETURN
        END IF
 10   CONTINUE
C
C  TOKENS IN THE CURRENT LINE HAVE BEEN EXHAUSTED, YET NO
C  INVALID TOKEN HAS BEEN FOUND. ( AT LEAST ONE INVALID TOKEN
C  SHOULD BE PRESENT: THE END OF LINE )
C
      CALL ABORT('PROGRAMING ERROR. EOL NOT FOUND.','EXTRA')
      END
C  ---------------------------------------------------------------------
      SUBROUTINE FEBUFF ( LINE, TOTAL )
C  ---------------------------------------------------------------------
C
C  FLUSHES THE ERROR BUFFER ON THE ERROR LIST FILE.
C  ASSUMES THAT UPON ENTRY THE BUFFER CONTAINS AT LEAST ONE ERROR.
C
C  INPUT ARGUMENTS:
C
C    LINE       CHARACTER VARIABLE. THIS IS THE MCL SOURCE LINE THAT IS
C               CURRENTLY UNDER PROCESSING.
C    TOTAL      INTEGER VARIABLE. THIS IS THE TOTAL NUMBER OF ERRORS
C               ENCOUNTERED IN THE PROGRAM, SO FAR.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      SAVE FIRST
C
C  MAXIMUM NUMBER OF ERROR MESSAGES IN ONE LINE.
      PARAMETER ( MAXERR = 20 )
C
      CHARACTER*(*) LINE
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      CHARACTER*90 EBUFF(MAXERR)
      COMMON /MCLERB/ EBUFF
      SAVE /MCLERB/
C
      CHARACTER*20 IFILE,EFILE,BFILE
      COMMON /FILES/ IFILE,EFILE,BFILE
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      LOGICAL EOPEN
      COMMON /ISOPEN/ EOPEN
C
C  FIRST INDICATES WHETHER THIS IS THE FIRST TIME THIS ROUTINE
C  IS CALLED.
      LOGICAL FIRST
      DATA FIRST / .TRUE. /
C
C  IF ITS THE FIRST TIME, THIS ROUTINE IS CALLED, ISSUE THE HEADING
C  MESSAGE IN THE ERROR LIST FILE.
C
      IF (FIRST) THEN
         IF (EOPEN) THEN
            WRITE (ERL,20)
     >            IFILE(1:LENGTH(IFILE)),EFILE(1:LENGTH(EFILE)),
     >            BFILE(1:LENGTH(BFILE)),DEBUG,BOUNDS
C  SUBSEQUENT CALLS TO THIS ROUTINE SHOULDN'T ISSUE THE ABOVE MESSAGE.
         ELSE
            WRITE (*,20)
     >            IFILE(1:LENGTH(IFILE)),'*',
     >            BFILE(1:LENGTH(BFILE)),DEBUG,BOUNDS
         END IF
         FIRST=.FALSE.
      END IF
C  WRITE THE MCL SOURCE LINE AND ITS LINE NUMBER AS WELL.
      IF (EOPEN) THEN
         WRITE (ERL,30) NL,LINE
      ELSE
         WRITE (*,30) NL,LINE
      END IF
C  FLUSH THE ERROR BUFFER.
      DO 10,I=1,NERR
        IF (EOPEN) THEN
           WRITE (ERL,40) EBUFF(I)
        ELSE
           WRITE (*,40) EBUFF(I)
        END IF
 10   CONTINUE
C  UPDATE THE TOTAL ERROR COUNTER.
      TOTAL=TOTAL+NERR
C  NO MORE ERRORS PRESENT IN THE ERROR BUFFER.
      NERR=0
20    FORMAT (/T2,'M C L - 1.0  PLUS'/
     >         T2,'COMPILATION PARAMETERS SUPPLIED :'/
     >         T2,'- I=',A,', E=',A,', B=',A,
     >         ', DEBUG=',L1,', BOUNDS=',L1,'.'//
     >         T2,'- ERROR REPORT :'/
     >         T2,'  ----- ------')
30    FORMAT (/T2,'LINE ',I4,6X,A)
40    FORMAT (T2,A)
      END
C  ---------------------------------------------------------------------
      SUBROUTINE GET ( TOKEN,NTOK,ALPHA,VAR,NT,NA,NUM,NN,LOC,DIM,
     >                 INFO,IPOS,KIND,LLOC )
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE PARSES THE "GET" STATEMENT AND GENERATES APPROPRIATE
C  OBJECT CODE.
C  THE SYNTAX OF THIS STATEMENT IS:
C
C  "GET" VAR1;VAR2;...VARN    OR
C  "GET" VAR1;VAR2;...VARN "FROM" <FILENAME>
C
C  WHERE VAR1,VAR2,...VARN IS A LIST OF VARIABLES.
C  <FILENAME> IS A FILENAME ( AN IDENTIFIER )
C
C  THE ROUTINE WILL FIRST DETERMINE THE NUMBER OF VARIABLES TO GET BY
C  COUNTING THE SEMICOLONS IN THE INPUT LINE. THEN IT WILL DETERMINE
C  THE INPUT <FILENAME>. IF ITS OMITTED THE DEFAULT MERLIN INPUT FILE
C  WILL BE USED. <FILENAME> IS NOT ALLOWED TO BE ANY OF THE MERLIN
C  FILES: HELP, DATA, STORE, INIPO, DISPO, BACKUP, MACROF, DEFINED
C  IN THE BLOCK DATA MCLDEF.
C  ONLY SIMPLE VARIABLES ( NOT USED AS LOOP CONTROL VARIABLES ) OR
C  ARRAY ELEMENTS ( DECLARED BY THE USER ) MAY APPEAR IN THE LIST OF
C  VARIABLES. IF ANY OF THE VARIABLES IS NOT DECLARED, OR IS NOT OF
C  THE TYPE MENTIONED ABOVE, AN MCL ERROR WILL BE ISSUED.
C  THE OBJECT CODE GENERATED IS AS:
C
C  $IN
C  <NUMBER OF VARIABLES>
C  <FILENAME>
C  <CODE TO CALCULATE MEMORY ADDRESS FOR VAR1>
C  $POPLV
C  <CODE TO CALCULATE MEMORY ADDRESS FOR VAR2>
C  $POPLV
C  ...
C  <CODE TO CALCULATE MEMORY ADDRESS FOR VARN>
C  $POPLV
C
C  WHERE <NUMBER OF VARIABLES> IS THE NUMBER OF VARIABLES IN THE
C  VARIABLE LIST AND <FILENAME> IS THE FILE NAME FROM WHICH THE VALUES
C  WILL BE INPUT ( ".STANDARD" WILL BE USED IF <FILENAME> IS OMITTED
C  FROM THE "GET" STATEMENT )
C  <CODE TO CALCULATE MEMORY ADDRESS FOR VARN> IS EITHER JUST AN INTEGER
C  NUMBER PUSHED ON THE STACK ( BY A $PUSHC INSTRUCTION ), IN CASE OF A
C  SIMPLE VARIABLE OR A SEQUENCE OF CODE THAT WILL CALCULATE THE
C  REQUESTED ADDRESS, IN CASE OF AN ARRAY ELEMENT.
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C               THE INPUT LINE.
C    NUM        REAL ARRAY. THE NUMBERS ENCOUNTERED IN THE INPUT LINE.
C    LLOC       INTEGER ARRAY. THIS IS THE LOOP TABLE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    VAR        CHARACTER ARRAY. THIS IS THE VAR TABLE.
C    NT         INTEGER VARIABLE. THIS IS A POINTER TO THE TOKEN ARRAY
C               AND IS THE NUMBER OF TOKENS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    NA         INTEGER VARIABLE. THIS IS A POINTER TO THE ALPHA ARRAY
C               AND IS THE NUMBER OF IDENTIFIERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    NN         INTEGER VARIABLE. THIS IS A POINTER TO THE NUM ARRAY
C               AND IS THE NUMBER OF NUMBERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    LOC        INTEGER ARRAY. THIS IS THE MEMORY LOCATION TABLE.
C    DIM        INTEGER ARRAY. THIS IS THE DIMENSIONALITY TABLE.
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    IPOS       INTEGER ARRAY. THIS IS THE POSITION TABLE.
C    KIND       INTEGER ARRAY. THIS IS THE KIND TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES ERROR, LOCAN, INSERT, EXTRA, XPR, ABORT.
C
C  ABORT CONDITIONS THAT MAY OCCUR:
C    THIS ROUTINE USES THE SPARE FILE, SO WHEN ITS OPENINIG OR CLOSING
C    FAILS, THE PROGRAM WILL STOP AND ISSUE AN INFORMATIVE MESSAGE.
C
C  NOTES:
C    IF AN ARRAY ELEMENT APPEARS IN THE LIST OF VARIABLES, IT IS
C    CONSIDERED AS AN EXPRESSION AND SUBROUTINE XPR IS CALLED
C    TO PROCESS IT. XPR WILL GENERATE CODE TO CALCULATE THE ARRAY
C    ELEMENT ADDRESS AND PUSH IT ON THE TOP OF THE STACK. THEN A
C    $GETCONT INSTRUCTION WILL BE GENERATED BY THE SAME ROUTINE, SO AS
C    WHEN EXECUTED, TO REPLACE THE TOP OF THE STACK WITH THE ACTUAL
C    VALUE OF THE ARRAY ELEMENT.
C    THE OBJECT CODE JUST DESCRIBED WILL BE WRITTEN IN THE SPARE FILE.
C    AS SOON AS XPR RETURNS, ALL OF THE SPARE FILE WILL BE COPIED TO
C    THE SCRATCH FILE, EXCLUDING THE FINAL $GETCONT INSTRUCTION.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF A LINE. (CHARS)
      PARAMETER (  LINLEN = 80  )
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER (  IDLEN = 30   )
C
      INTEGER TOKEN(*),LOC(*),DIM(*),INFO(*),IPOS(*),KIND(*),LLOC(*)
      CHARACTER*(*) ALPHA(*),VAR(*)
      REAL NUM(*)
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      CHARACTER*10 MFILE(7)
      COMMON /MCLMFL/ MFILE
C
      CHARACTER LINE*80
      CHARACTER*(IDLEN) NAME,STARG(1)
      LOGICAL FOUND,CORRE
      INTEGER EX(LINLEN)
C
C  CALCULATE NG, ( THAT IS THE NUMBER OF VARIABLES TO GET ) BY
C  COUNTING THE NUMBER OF SEMICOLONS IN THE INPUT LINE.
C
      NG=1
      DO 10,I=NT+1,NTOK
        IF (TOKEN(I).EQ.SEMI) NG=NG+1
 10   CONTINUE
C
C  DETERMINE THE INPUT FILE NAME.
C  FIRST COUNT THE NUMBER OF IDENTIFIERS IN THE INPUT LINE.
C
      NID=NA
      DO 99,I=NT+1,NTOK
        IF (TOKEN(I).EQ.IDENT) NID=NID+1
 99   CONTINUE
C  NOW CHECK TO SEE IF AN INPUT FILENAME IS DEFINE.
C  USE THE DEFAULT OTHERWISE.
      NAME='.STANDARD'
      IF (NID.NE.NA .AND. ALPHA(NID-1).EQ.'FROM') NAME=ALPHA(NID)
C
C  CHECK IF THE INPUT FILE IS ONE OF THE MERLIN FILES.
C
      DO 62,I=1,7
        IF (NAME.EQ.MFILE(I)) THEN
           CALL ERROR(77,NAME)
           GOTO 40
        END IF
 62   CONTINUE
C
C  WRITE INITIAL LINES OF OBJECT CODE.
C
      BLOC=BLOC+3
      IF (GENE) WRITE(SCR,'(A/I4/A)') '$IN',NG,NAME
C
C  START A LOOP FOR ALL VARIABLES IN THE VARIABLE LIST.
C
C  CHECK TO SEE IF THE NEXT TOKEN IS AN IDENTIFIER.
C
  40  NT=NT+1
      IF (TOKEN(NT).NE.IDENT) THEN
         CALL ERROR(3,' ')
         RETURN
      END IF
C
C  GET VARIABLE NAME.
C
      NA=NA+1
      NAME=ALPHA(NA)
C
C  CHECK IF PRESENT IN THE VAR TABLE.
C
      CALL LOCAN(VAR,NVAR,NAME,FOUND,IND)
      IF (.NOT.FOUND) THEN
C  VARIABLE NOT PRESENT IN THE VAR TABLE.
         CALL ERROR(19,NAME)
C  PUT VARIABLE IN THE VAR TABLE. UPDATE RELATED TABLES ALSO.
         CALL INSERT(VAR,DIM,LOC,IPOS,KIND,NAME,0,0,0,-1,IND)
      END IF
C
C  CALCULATE MEMORY ADDRESS FOR THIS VARIABLE.
C
      IF (TOKEN(NT+1).NE.LBRA) THEN
C  THE NEXT TOKEN IS NOT A LEFT BRACKET. THUS THIS IS A SIMPLE
C  VARIABLE TO GET.
         K=KIND(IND)
         IF (K.NE.1 .AND. K.NE.-1) CALL ERROR(153,' ')
C  CHECK WHETHER VARIABLE IS USED AS A LOOP CONTROL INDEX.
         L=LOC(IND)
         DO 67,I=1,NLOO
           IF (LLOC(I).EQ.L) THEN
              CALL ERROR(167,NAME)
              GOTO 63
           END IF
 67      CONTINUE
C  GENERATE CODE NECESSARY FOR ITS ADDRESS EVALUATION.
 63      BLOC=BLOC+2
         IF (GENE) THEN
            WRITE(SCR,'(A)') '$PUSHC'
            WRITE(SCR,*) L
         END IF
      ELSE
C  THIS IS AN ARRAY ELEMENT.
         NT=NT-1
         NA=NA-1
C  EXTRACT THE ARRAY ELEMENT AS AN EXPRESSION.
         CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,1,ALPHA,NA)
C  EVALUATE THIS EXPRESSION. THE OBJECT CODE WILL BE WRITTEN
C  IN THE SPARE FILE. THE NUMBER OF OBJECT CODE LINES WRITTEN
C  THERE IS COUNTED BY NLS.
         NLS=0
         OPEN(SPARE,STATUS='SCRATCH',ERR=1818)
         REWIND SPARE
         CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,
     >            NLS,LOC,DIM,INFO,IPOS,KIND,SPARE,STARG)
         IF (GENE) THEN
C  COPY THE SPARE FILE TO THE SCRATCH FILE. THE FINAL $GETCONT
C  INSTRUCTION WILL NOT BE WRITTEN IN THE SCRATCH FILE.
            REWIND SPARE
            DO 88,I=1,NLS-1
              READ(SPARE,'(A)') LINE
              WRITE(SCR,'(A)') LINE
  88        CONTINUE
            BLOC=BLOC+NLS-1
         END IF
         CLOSE(SPARE,STATUS='DELETE',ERR=1919)
C  ARRAY ELEMENT LOCATION IS NOW ON THE TOP OF THE STACK.
      END IF
C  WRITE THE $POPLV INSTRUCTION FOR THIS VARIABLE.
      BLOC=BLOC+1
      IF (GENE) WRITE(SCR,'(A)') '$POPLV'
C
C  CHECK IF THE NAME IS ONE OF THE MERLIN VARIABLES,
C  MERLIN ARRAYS OR INTRINSIC FUNCTIONS.
C
      K=KIND(IND)
      IF (K.EQ.4 .OR. K.EQ.5 .OR. K.EQ.6) CALL ERROR(18,NAME)
C
      NT=NT+1
C  IF ALL TOKENS EXHAUSTED.
      IF (NT.EQ.NTOK) RETURN
      CUR=TOKEN(NT)
C  IF THERE'S A SEMICOLON, ANOTHER VARIABLE SHOULD FOLLOW IN THE LIST.
      IF (CUR.EQ.SEMI) GOTO 40
      IF (CUR.EQ.IDENT) THEN
C  IF AN IDENTIFIER FOLLOWS, THIS SHOULD BE "FROM".
         NA=NA+1
         IF (ALPHA(NA).EQ.'FROM') THEN
            NT=NT+1
            IF (TOKEN(NT).EQ.IDENT) THEN
C  AN IDENTIFIER ( THE <FILENAME> ) SHOULD FOLLOW AFTER "FROM".
               NT=NT+1
               IF (NT.NE.NTOK) CALL ERROR(152,' ')
            ELSE
               CALL ERROR(150,' ')
            END IF
         ELSE
C  "FROM" DOES NOT FOLLOW THE VARIABLE LIST.
            CALL ERROR(151,' ')
         END IF
         RETURN
      END IF
C  NEITHER SEMICOLON, NOR AN IDENTIFIER WAS FOUND AFTER
C  THE VARIABLE LIST.
      CALL ERROR(149,' ')
      RETURN
C
C  IN CASE THE SPARE FILE OPENING FAILS.
C
 1818 CALL ABORT('COULD NOT OPEN THE SPARE FILE.','GET')
C
C  IN CASE THE SPARE FILE CLOSING FAILS.
C
 1919 CALL ABORT('COULD NOT CLOSE THE SPARE FILE.','GET')
C
      END
C  ---------------------------------------------------------------------
      SUBROUTINE GETPAR
C  ---------------------------------------------------------------------
C
C  DISPLAYS THE COMPILERS INITIAL MESSAGE AND PROMPTS THE USER
C  TO ENTER THE COMPILATION PARAMETERS.
C  THESE ARE ENTERED IN ONE LINE, UP TO 80 CHARACTERS LONG.
C  THE INPUT LINE MUST HAVE THE FORM:
C  K1=V1,K2=V2,...,KN=VN
C  WHERE K1,K2,...,KN ARE KEYWORDS AND V1,V2,...,VN IS THE
C  VALUE ASSIGNED TO THEM.
C  THE KEYWORDS, THEIR SIGNIFICANCE AND THE VALUES THAT SHOULD
C  BE ASSIGNED TO THEM ARE AS:
C
C  KEYWORD     SIGNIFICANCE             ALLOWABLE VALUE
C  -------     ------------             ---------------
C  I           THE MCL SOURCE           ANY FILE NAME
C  E           THE ERROR LIST FILE      ANY FILE NAME
C  B           THE OBJECT CODE FILE     ANY FILE NAME
C  DEBUG       THE DEBUG PARAMETER      TRUE,FALSE,T,F
C  BOUNDS      THE BOUNDS PARAMETER     TRUE,FALSE,T,F
C
C  THE FILE NAMES SPECIFIED AS INPUT TO THE COMPILER MAY
C  CONTAIN SPECIAL CHARACTERS (SUCH AS . / + - * ETC) AND MAY
C  BE UP TO 20 CHARACTERS AT MOST. THIS IS PARTICULARY USEFUL,
C  SINCE MOST OPERATING SYSTEMS ALLOW SPECIAL CHARACTERS IN A
C  FILE NAME. NOTE THAT THE COMMA MAY NOT BE USED IN A FILE NAME
C  SINCE IT SERVES SEPARATION PURPOSES IN THE INPUT LINE.
C  NOTE ALSO THAT MCL ITSELF DOES NOT SUPPORT SPECIAL CHARACTERS
C  IN FILE NAMES.
C
C  THE INPUT FILE (THE I PARAMETER) MUST BE ALWAYS SPECIFIED.
C  THE OTHER PARAMETERS WHEN NOT EXPLICITLY DECLARED
C  ASSUME DEFAULT VALUES AS:
C
C  PARAMETER          DEFAULT VALUE
C  ---------          -------------
C  E                  ERRORS
C  B                  MOC
C  DEBUG              FALSE
C  BOUNDS             FALSE
C
C  THE ABOVE PARAMETERS OBTAIN THEIR DEFAULT VALUES IN THE
C  BLOCK DATA MCLDEF.
C  BLANKS ARE ALLOWED IN THE INPUT LINE AT ANY POINT.
C  DUPLICATE ENTRIES OF KEYWORDS ARE NOT ALLOWED. THUS THE
C  FOLLOWING LINE IS INCORRECT:
C  I=INP, B=BIN, E=ERR, B=BIN
C  IF A FILE NAME IS SPECIFIED AS A VALUE TO A KEYWORD, IT MAY
C  NOT BE SPECIFIED AS A VALUE TO ANOTHER KEYWORD.
C  FOR THIS REASON THE FOLLOWING LINE IS INCORRECT.
C  I=FILE1, E=XFILE, BOUNDS=T, B=XFILE
C  IF ANY OF THE ABOVE RULES IS VIOLATED, OR THE INPUT LINE IS
C  INCORRECTLY FORMATTED, THE PROGRAM WILL STOP AND ISSUE
C  AN INFORMATIVE MESSAGE.
C
C  UPON RETURN OF THIS ROUTINE, THE VALUES OF IFILE, EFILE AND BFILE
C  IN THE /FILES/ COMMON BLOCK WILL BE UPDATED. THE VALUES OF
C  BOUNDS AND DEBUG IN THE /MCLDTA/ COMMON BLOCK WILL ALSO BE UPDATED.
C
C  OTHER PROGRAM UNITS CALLED:
C    FUNCTIONS LENGTH, DEF.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      CHARACTER*20 IFILE,EFILE,BFILE
      COMMON /FILES/ IFILE,EFILE,BFILE
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      CHARACTER*20 VALUE(20),V
      CHARACTER*10 KEY(20),K
      CHARACTER*82 LINE
      INTEGER FILE(3)
      LOGICAL DEF,DEFINP,TRUE,FALSE
C
C  THE FOLLOWING STATEMENT FUNCTION WILL TEST IF A VALUE
C  OF TRUE/FALSE IS SUPPLIED AS THEIR ARGUMENT.
C
      TRUE(V) = V.EQ.'T' .OR. V.EQ.'TRUE'
      FALSE(V)= V.EQ.'F' .OR. V.EQ.'FALSE'
C
C  FORMATS FOR THE INITIAL PRINTOUT.
C
 500  FORMAT ( 79('-') )
 600  FORMAT ( T30,'M C L - 1.0  PLUS')
 900  FORMAT ( '  ENTER COMPILATION PARAMETERS   - I, B, E, DEBUG, ',
     >         'BOUNDS -' )
C
C  OUTPUT INITIAL MESSAGE.
C
      WRITE(*,500)
      WRITE(*,600)
      WRITE(*,500)
      WRITE(*,900)
C
C  THIS VARIABLE INDICATES WHETHER THE INPUT FILE (THE I PARAMETER)
C  HAS BEEN DEFINED.
      DEFINP=.FALSE.
C
C  GET LINE CONTAINING COMPILATION PARAMETERS.
      READ(*,'(A)',END=400) LINE
C  PACK THE LINE JUST READ, SO AS TO ELIMINATE ANY EMBEDDED BLANKS.
      LONG=LENGTH(LINE)
 15   IL=INDEX(LINE,' ')
      IF (IL.LT.LONG .AND. IL.NE.0) THEN
         LINE(IL:)=LINE(IL+1:)
         LONG=LONG-1
         GOTO 15
      END IF
C  ADD A COMMA AND A BLANK TO THE END OF THE PACKED LINE.
C  KEYWORD SEARCHING WILL STOP AS SOON AS THESE TWO CHARACTERS
C  ARE REACHED.
      LINE(81:82)=', '
C  NK COUNTS THE KEYWORDS THAT HAVE BEEN READ.
C  L IS THE CHARACTER POSITION IN LINE, WHERE THE NEXT KEYWORD
C  IS EXPECTED TO BE FOUND.
      NK=0
      L=1
C
C  IEQ IS THE CHARACTER POSITION IN LINE, WHERE THE "=" FOR
C  THE NEXT KEYWORD APPEARS.
 10   IEQ=INDEX(LINE(L:),'=')+L-1
C  IF AN "=" WAS NOT FOUND, OR "=" WAS FOUND JUST AFTER THE COMMA.
      IF (IEQ.EQ.L-1 .OR. IEQ.EQ.L) GOTO 100
C  GET THE KEYWORD.
      NK=NK+1
      KEY(NK)=LINE(L:IEQ-1)
C  IC IS THE INDEX OF THE COMMA, FOLLOWING THE CURRENT KEYWORD.
      IC=INDEX(LINE(IEQ+1:),',')
      IC=IC+IEQ
C  IF THE COMMA FOLLOWS THE "=".
      IF (IEQ+1.EQ.IC) GOTO 100
C  GET THE VALUE ASSIGNED TO THIS KEYWORD.
      VALUE(NK)=LINE(IEQ+1:IC-1)
C  THE CHARACTER POSITION, WHERE THE NEXT KEYWORD SHOULD
C  START, IS THE ONE, JUST AFTER THE COMMA.
      L=IC+1
C  IF ANYTHING BUT BLANKS FOLLOW, TRY TO GET ANOTHER KEYWORD.
      IF (LINE(L:).NE.' ') GOTO 10
C
C  NF COUNTS HOW MANY PARAMETERS THAT ACCEPT A FILE NAME AS
C  A VALUE, HAVE BEEN PROCESSED.
      NF=0
C
C  THE FOLLOWING LOOP WILL PROCESS ALL KEYWORDS AND THEIR VALUES,
C  CHECKING THE VALIDITY OF EACH ENTRY.
C
      DO 20,M=1,NK
        CALL UPCASE(KEY(M))
        K=KEY(M)
        V=VALUE(M)
C  CHECK IF THE KEYWORD HAS ALREADY BEEN ENTERED.
        DO 30,J=1,M-1
          IF (K.EQ.KEY(J)) THEN
             WRITE(*,*) '-- KEYWORD ALREADY ENTERED :  ',K
             STOP 'MCLCOM - PARAMETER ERRORS.'
          END IF
 30     CONTINUE
C
C  CHECK IF KEYWORD BELONGS TO THE VALID ONES, AND IF SO,
C  TAKE APPROPRIATE ACTION.
        IF (K.EQ.'I') THEN
C  CHECK WHETHER THE VALUE OF THIS KEYWORD HAS BEEN ENTERED
C  AS A FILE NAME IN ANOTHER KEYWORD.
           IF (DEF(V,VALUE,FILE,NF)) GOTO 300
C  INDICATE THAT THIS KEYWORD, ACCEPTS A FILE NAME AS A VALUE.
           NF=NF+1
           FILE(NF)=M
C  INDICATE THAT THE INPUT FILE ( THE I PARAMETER ) HAS BEEN ENTERED.
           DEFINP=.TRUE.
C  ASSIGN THE VALUE TO BE RETURNED BY THE ROUTINE.
           IFILE=V
        ELSE IF (K.EQ.'B') THEN
C  CHECK WHETHER THE VALUE OF THIS KEYWORD HAS BEEN ENTERED
C  AS A FILE NAME IN ANOTHER KEYWORD.
           IF (DEF(V,VALUE,FILE,NF)) GOTO 300
C  INDICATE THAT THIS KEYWORD ACCEPTS A FILE NAME AS A VALUE.
           NF=NF+1
           FILE(NF)=M
C  ASSIGN THE VALUE TO BE RETURNED BY THE ROUTINE.
           BFILE=V
        ELSE IF (K.EQ.'E') THEN
C  CHECK WHETHER THE VALUE OF THIS KEYWORD HAS BEEN ENTERED
C  AS A FILE NAME IN ANOTHER KEYWORD.
           IF (DEF(V,VALUE,FILE,NF)) GOTO 300
C  INDICATE THAT THIS KEYWORD ACCEPTS A FILE NAME AS A VALUE.
           NF=NF+1
           FILE(NF)=M
C  ASSIGN THE VALUE TO BE RETURNED BY THE ROUTINE.
           EFILE=V
        ELSE IF (K.EQ.'DEBUG') THEN
C  CHECK IF A VALUE THAT INDICATES DEBUG SELECTION HAS BEEN ENTERED.
           CALL UPCASE(V)
           IF ( TRUE(V) ) THEN
              DEBUG=.TRUE.
           ELSE IF ( FALSE(V) ) THEN
              DEBUG=.FALSE.
           ELSE
              WRITE(*,*) '-- INCORRECT VALUE FOR THE -DEBUG- PARAMETER.'
           STOP 'MCLCOM - PARAMETER ERRORS.'
           END IF
        ELSE IF (K.EQ.'BOUNDS') THEN
C  CHECK IF A VALUE THAT INDICATES BOUNDS SELECTION HAS BEEN ENTERED.
           CALL UPCASE(V)
           IF ( TRUE(V) ) THEN
              BOUNDS=.TRUE.
           ELSE IF ( FALSE(V) ) THEN
              BOUNDS=.FALSE.
           ELSE
              WRITE(*,*)'-- INCORRECT VALUE FOR THE -BOUNDS- PARAMETER.'
              STOP 'MCLCOM - PARAMETER ERRORS.'
           END IF
        ELSE
C  THIS IS NOT A CORRECT KEYWORD.
           WRITE(*,*) '-- INCORRECT KEYWORD :  ',K
           STOP 'MCLCOM - PARAMETER ERRORS.'
        END IF
 20   CONTINUE
C
C  IF AN INPUT FILE HAS BEEN DEFINED, RETURN NORMALLY.
 400  IF ( DEFINP ) RETURN
      WRITE(*,*) '-- INPUT FILE NOT SPECIFIED.'
      STOP 'MCLCOM - PARAMETER ERRORS.'
C
C  IF ANY ERROR OCCURRED IN KEYWORD/VALUE SEPARATION.
 100  WRITE(*,*) '-- INCORRECT ENTRY.'
      STOP 'MCLCOM - PARAMETER ERRORS.'
C
C  IF THE SAME FILE IS SPECIFIED FOR DIFFERENT KEYWORDS.
 300  WRITE(*,*) '-- FILE ALREADY SPECIFIED :  ',V
      STOP 'MCLCOM - PARAMETER ERRORS.'
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE UPCASE ( CHA )
      CHARACTER*(*) CHA
C
      CHARACTER C
      LOGICAL ISCAP
C
      DO 10,I=1,LEN(CHA)
         C = CHA(I:I)
         IF (.NOT.ISCAP(C)) THEN
            IC = ICHAR(C)
            IF (IC.GE.97 .AND. IC.LE.122) CHA(I:I) = CHAR(IC-32)
         END IF
10    CONTINUE
C
      END
C
C  ---------------------------------------------------------------------
C
      LOGICAL FUNCTION ISCAP ( C )
      CHARACTER C
C
      CHARACTER*26 LETT
      DATA LETT / 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' /
C
      ISCAP = INDEX(LETT,C).NE.0
      END
CGODFA------------------------------------------------------------------
      SUBROUTINE GODFA(TOKENS, TOKN, TUSED, ALPHAC, NA, NUMBC, NN, VAR,
     >                 ADDR, DIMARG, INFO, INFPOS, VRKIND, STVAR)
C-----------------------------------------------------------------------
C
C
C   %..   PURPOSE: PARSES GODFATHER STATEMENT.
C
C   %..   ARGUMENT INFORMATION:
C
C      -TOKENS: IS THE LINE TO BE PROCESSED.
C      -TOKN  : IS THE NUMBER OF TOKENS "TOKENS" IS COMPOSED OF
C      -TUSED : POINTS TO THE LAST USED ELEMENT OF "TOKENS".
C      -ALPHAC: ARE THE IDENTIFIERS OF "TOKENS".
C      -NUMBC : ARE THE NUMERIC CONSTANTS OF "TOKENS".
C      -NA    : POINTS TO THE LAST USED ELEMENT OF ALPHAC.
C      -NN    : POINTS TO THE LAST USED ELEMENT OF NUMBC.
C      -VAR   : THE VAR TABLE.
C      -ADDR  : ARRAY KEEPING LOCATIONS. SEE XGEN FER DETAILS.
C      -DIMARG: ARRAY KEEPING DIMENSIONALITIES. SEE XGEN.
C      -INFO  : ARRAY WITH MCL-ITEMS INFORMATION. SEE XGEN.
C      -INFPOS: ARRAY RELATED TO INFO. SEE XGEN.
C      -VRKIND: PARALLEL ARRAY TO VAR. KEEPS "KIND"S. SEE XGEN.
C      -STVAR : KEEPS DUMMY NAMES FOR FUNCTIONS.
C
C   %..   SUBPROGRAMS CALLED:
C
C             SUBROUTINES: ERROR, EXTRA, XPR,
C             FUNCTIONS  : NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C             MCLUNT (A) , MCLDTA (M) , MCLSCD(A)
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS: NONE.
C
C   %..   NOTES:
C
C    SYNTAX :
C
C    <GODFATHER> : : = 'GODFATHER' '(' 'X' '.' <EXPRESSION> '='
C             <IDENT> [ ';' 'X' '.' <EXPRESSION> '=' <IDENT> ] ')'
C
C    CODE WRITTEN (IF SYNTAX CORRECT) :
C
C    $WGODF
C    NONAM
C    NAME
C    NAME
C    ...
C    NAME
C    $SWAP
C
C    AND  THE (NONAM) TOP POSITIONS OF STACK HOLD THE
C         CORRESPONDING INDICES.
C
C-----------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
      DIMENSION TOKENS(TOKN),EXPR(76),ADDR(*),DIMARG(*),INFO(*),
     >          INFPOS(*),VRKIND(*)
      PARAMETER (CHOICE=2)
      LOGICAL CORR,GENE,TDK,TSM,FDECLR,DEBUG,BOUNDS
      CHARACTER*(*) ALPHAC(*), VAR(*), C*30, NAMES(1:40)*30, STVAR(*)
      REAL NUMBC(*)
      COMMON / MCLUNT / INP, ERL, BIN, SCR, ISTF, SPARZZ, DUMMYF,DESCRI
     >       / MCLDTA / NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
     >       / MCLSCD / LEFTP, RIGHTP, DOT, COMMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUMBER,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
      TDK()=(TOKENS(TP).EQ.IDENT)
      TSM()=(TOKENS(TP).EQ.SEMI)
C
C   START
C
      TP=TUSED+1
      NONAM=0
      FDECLR= .FALSE.
      IF(TOKENS(TP).EQ.LEFTP)THEN
 111     CONTINUE
         TP=TP+1
         NA=NA+1
         IF(TP.EQ.TOKN)GOTO 100
         C=ALPHAC(NA)
         IF(TDK().AND.C(1:1).EQ.'X')THEN
            TP=TP+1
            IF(TP.EQ.TOKN)GOTO 100
            IF(TOKENS(TP).EQ.DOT)THEN
               CALL EXTRA (TOKENS,TOKN,TP,EXPR,NEX,CHOICE,ALPHAC,NA)
               TP = TP+1
               IF(TP.GE.TOKN)GOTO 100
               IF(TOKENS(TP).NE.EQ)THEN
                 CALL ERROR (91,' ')
               ELSE
                 TP=TP+1
                 IF(TDK())THEN
C                   WRITE CODE:
                    CALL XPR ( EXPR,NEX,ALPHAC,NUMBC,NA,NN,CORR,VAR,
     >              FDECLR,BLOC,ADDR,DIMARG,INFO,INFPOS,VRKIND,SCR,
     >              STVAR)
                    IF(.NOT.CORR)RETURN
                    NA=NA+1
                    NONAM=NONAM+1
                    NAMES(NONAM)=ALPHAC(NA)
                    TP=TP+1
                    IF(TSM())GOTO 111
                    IF(.NOT.(TOKENS(TP).EQ.RIGHTP.AND.TP+1.EQ.TOKN))
     >                CALL ERROR (92,' ')
                    BLOC=BLOC+3+NONAM
                    IF(GENE)THEN
                       WRITE(SCR,'(''$WGODF'')')
                       WRITE(SCR,*)NONAM
                       DO 987,IN=NONAM,1,-1
                         WRITE(SCR,'(A)')NAMES(IN)
 987                   CONTINUE
                       WRITE(SCR,'(''$SWAP'')')
                    ENDIF
                 ELSE
                    CALL ERROR (93,' ')
                 ENDIF
               ENDIF
            ELSE
               CALL ERROR (94,' ')
            ENDIF
         ELSE
            CALL ERROR (95,' ')
         ENDIF
      ELSE
         CALL ERROR (96,' ')
      ENDIF
      RETURN
 100  CALL ERROR (97,' ')
      RETURN
      END
CGRADQ------------------------------------------------------------------
      SUBROUTINE GRADQ(ID,TOKENS,TOKN,TUSED,ALPHAC,NA,NUMBC,NN,VAR,
     >                ADDR, DIMARG, INFO, INFPOS, VRKIND, STVAR)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: IT PARSES GRADCHECK, GRADDIS AND QUIT
C                STATEMENTS.
C
C   %..   ARGUMENT INFORMATION:
C
C      -ID    : PROCESSING OPTION, SEE NOTES.
C      -TOKENS: IS THE LINE TO BE PROCESSED.
C      -TOKN  : IS THE NUMBER OF TOKENS "TOKENS" IS COMPOSED OF
C      -TUSED : POINTS TO THE LAST USED ELEMENT OF "TOKENS".
C      -ALPHAC: ARE THE IDENTIFIERS OF "TOKENS".
C      -NUMBC : ARE THE NUMERIC CONSTANTS OF "TOKENS".
C      -NA    : POINTS TO THE LAST USED ELEMENT OF ALPHAC.
C      -NN    : POINTS TO THE LAST USED ELEMENT OF NUMBC.
C      -VAR   : THE VAR TABLE.
C      -ADDR  : ARRAY KEEPING LOCATIONS. SEE XGEN FER DETAILS.
C      -DIMARG: ARRAY KEEPING DIMENSIONALITIES. SEE XGEN.
C      -INFO  : ARRAY WITH MCL-ITEMS INFORMATION. SEE XGEN.
C      -INFPOS: ARRAY RELATED TO INFO. SEE XGEN.
C      -VRKIND: PARALLEL ARRAY TO VAR. KEEPS "KIND"S. SEE XGEN.
C      -STVAR : KEEPS DUMMY NAMES FOR FUNCTIONS.
C
C   %..   SUBPROGRAMS CALLED:
C
C     SUBROUTINES: ERROR, EXTRA, XPR, ABORT,
C     FUNCTIONS  : NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C       MCLSCD (A), MCLUNT(A), MCLDTA(M)
C
C   %..   POSSIBLE ABORTS:
C
C           ABORT MAY OCCUR IF ID IS GIVEN ERRONEOUS.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS: NONE.
C
C   %..   NOTES:
C
C     CONCERNING ID, WE HAVE:
C
C         1 = GRADCHECK
C         2 = GRADDIS
C         3 = QUIT
C
C     SYNTAX :
C
C   ('GRADDIS' \ 'GRADCHECK' ) '(' [ 'ERR' '=' ] [ <EXPRESSION> ] ')'
C               OR
C           'QUIT' '(' [ 'FLAG' '=' ] <EXPRESSION> ')'
C
C
C
C   CODE WRITTEN (IF SYNTAX CORRECT) :
C
C   $WGRAD
C   $SWAP
C            OR
C   GRADCHECK
C   0.0
C
C   $WDIS
C   $SWAP
C            OR
C   GRADDIS
C   0.0
C
C   $WQUIT
C   $SWAP
C
C-----------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
      DIMENSION TOKENS(TOKN),EXPR(76),ADDR(*),DIMARG(*),INFO(*),
     >          INFPOS(*),VRKIND(*)
      PARAMETER (CHOICE=2)
      LOGICAL CORR,GENE,FDECLR,DEBUG,BOUNDS
      CHARACTER*(*) ALPHAC(*),VAR(*),NAME(3)*9,WNAME(3)*6, STVAR(*)
      REAL NUMBC(*)
      COMMON / MCLUNT / INP, ERL, BIN, SCR, ISTF, SPARZZ, DUMMYF,DESCRI
     >       / MCLDTA / NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
     >       / MCLSCD / LEFTP, RIGHTP, DOT, COMMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUMBER,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
C
      DATA NAME /'GRADCHECK','GRADDIS  ','QUIT     '/
      DATA WNAME /'$WGRAD','$WDIS ','$WQUIT'/
C
C  START
C
      IF(ID.GT.3.OR.ID.LT.1) CALL ABORT ('ID WRONG','GRAD')
      FDECLR=.FALSE.
      TP=TUSED+1
      IF(TP.EQ.TOKN.AND.ALPHAC(NA).NE.'QUIT')THEN
C
C     THE DEFAULT VALUE FOR ERR IS ZERO.
C
         IF(GENE)WRITE(SCR,'(A9,/,'' 0.000'')')NAME(ID)
         BLOC = BLOC + 2
      ELSE IF(TP.EQ.TOKN.AND.ALPHAC(NA).EQ.'QUIT')THEN
         CALL ERROR (73,' ')
      ELSE
         IF(TOKENS(TP).EQ.LEFTP)THEN
            TP=TP+1
            NA=NA+1
            IF(TP.EQ.TOKN)GOTO 100
            IF((ALPHAC(NA).EQ.'ERR'.OR.ALPHAC(NA).EQ.'FLAG')
     >         .AND.TOKENS(TP).EQ.IDENT)THEN
               IF((ID.NE.3.AND.ALPHAC(NA).EQ.'FLAG').OR.
     >            (ID.EQ.3.AND.ALPHAC(NA).NE.'FLAG'))THEN
                  CALL ERROR (74,ALPHAC(NA))
                  GOTO 50
               ENDIF
               TP=TP+1
               IF(TP.GE.TOKN)GOTO 100
               IF(TOKENS(TP).EQ.EQ)THEN
                  CALL EXTRA (TOKENS,TOKN,TP,EXPR,NEX,CHOICE,ALPHAC,NA)
                  TP = TP+1
                  IF(TP.EQ.TOKN)GOTO 100
                  IF(TOKENS(TP).EQ.RIGHTP.AND.TP+1.EQ.TOKN)THEN
                     CALL XPR ( EXPR,NEX,ALPHAC,NUMBC,NA,NN,CORR,VAR,
     >               FDECLR,BLOC,ADDR,DIMARG,INFO,INFPOS,VRKIND,SCR,
     >               STVAR)
                     IF(.NOT.CORR)RETURN
                     IF(GENE)THEN
                       WRITE(SCR,'(A)')WNAME(ID)
                       WRITE(SCR,'(''$SWAP'')')
                     ENDIF
                     BLOC=BLOC+2
                  ELSE
                     CALL ERROR (76,' ')
                  ENDIF
               ELSE
                  CALL ERROR (71,' ')
               ENDIF
            ELSE
               TP=TP-1
               NA=NA-1
               CALL EXTRA(TOKENS,TOKN,TP,EXPR,NEX,CHOICE,ALPHAC,NA)
               TP = TP+1
               IF(TP.EQ.TOKN)GOTO 100
               IF(TOKENS(TP).EQ.RIGHTP.AND.TP+1.EQ.TOKN)THEN
                  CALL XPR ( EXPR,NEX,ALPHAC,NUMBC,NA,NN,CORR,VAR,
     >            FDECLR,BLOC,ADDR,DIMARG,INFO,INFPOS,VRKIND,SCR,STVAR)
                  IF(.NOT.CORR)RETURN
                  IF(GENE)THEN
                    WRITE(SCR,'(A)')WNAME(ID)
                    WRITE(SCR,'(''$SWAP'')')
                  ENDIF
                  BLOC=BLOC+2
               ELSE
                  CALL ERROR (76,' ')
               ENDIF
            ENDIF
         ELSE
            CALL ERROR (72,' ')
         ENDIF
      ENDIF
 50   RETURN
 100  CALL ERROR (75,' ')
      RETURN
      END
CHIDE-------------------------------------------------------------------
      SUBROUTINE HIDE (TOKENS,TOKN,TUSED,ALPHAC,NA)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: PARSES HIDEOUT STATEMENT.
C
C   %..   ARGUMENT INFORMATION:
C
C             - TOKENS : THE LINE TO BE PARSED.
C             - TOKN   : THE NUMBER OF TOKENS INTO LINE.
C             - TUSED  : LAST USED ELEMENT OF "TOKENS".
C             - ALPHAC : THE ALPHABETIC CONTENTS OF LINE.
C             - NA     : POINTS TO THE LAST USED ELEMENT OF ALPHAC.
C
C   %..   SUBPROGRAMS CALLED:
C
C            SUBROUTINES : ERROR,
C            FUNCTIONS   : NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C              MCLSCD (A), MCLUNT(A), MCLDTA(M)
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS: NONE.
C
C   %..   NOTES:
C
C      SYNTAX :
C      <HIDEOUT> : : = 'HIDEOUT' '(' ['FILE' '=' ] <IDENT> ')'
C
C-----------------------------------------------------------------------
C
      IMPLICIT INTEGER ( A - Z)
      DIMENSION TOKENS(TOKN)
      LOGICAL GENE,BOUNDS,DEBUG
      CHARACTER*(*) ALPHAC(*),FILENM*30
      COMMON / MCLUNT / INP, ERL, BIN, SCR, ISTF, SPARZZ, DUMMYF,DESCRI
     >       / MCLDTA / NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
     >       / MCLSCD / LEFTP, RIGHTP, DOT, COMMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUMBER,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
C
C   START
C
      TP=TUSED+1
      IF(TP.EQ.TOKN)GOTO 100
      IF(TOKENS(TP).EQ.LEFTP)THEN
         TP=TP+1
         NA=NA+1
         IF(TP.EQ.TOKN)GOTO 100
         IF(ALPHAC(NA).EQ.'FILE'.AND.TOKENS(TP).EQ.IDENT)THEN
            TP=TP+1
            IF(TP.GE.TOKN)GOTO 100
            IF(TOKENS(TP).EQ.EQ)THEN
               TP = TP+1
               IF(TP.EQ.TOKN)GOTO 100
               IF(TOKENS(TP).EQ.IDENT)THEN
                  TP=TP+1
                  IF(TP.EQ.TOKN)GOTO 100
                  NA=NA+1
                  FILENM=ALPHAC(NA)
                  IF(TOKENS(TP).EQ.RIGHTP.AND.TP+1.EQ.TOKN)THEN
                     BLOC=BLOC+2
                     IF(GENE)WRITE(SCR,'(''HIDEOUT'',/,A)')FILENM
                  ELSE
                     CALL ERROR (103,' ')
                  ENDIF
               ELSE
                  CALL ERROR (104,' ')
               ENDIF
            ELSE
               CALL ERROR (106,' ')
            ENDIF
         ELSE
            IF(TOKENS(TP).EQ.IDENT)THEN
               TP=TP+1
               IF(TP.EQ.TOKN)GOTO 100
               FILENM=ALPHAC(NA)
               IF(TOKENS(TP).EQ.RIGHTP.AND.TP+1.EQ.TOKN)THEN
                  BLOC=BLOC+2
                  IF(GENE)WRITE(SCR,'(''HIDEOUT'',/,A)')FILENM
               ELSE
                  CALL ERROR (103,' ')
               ENDIF
            ELSE
               CALL ERROR (104,' ')
            ENDIF
         ENDIF
      ELSE
         CALL ERROR (107,' ')
      ENDIF
      RETURN
 100  CALL ERROR (105,' ')
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE IF ( TOKEN, NTOK, ALPHA, NUM, TYPE ,TDEF, VAR, LOC,
     >                DIM, INFO, IPOS, KIND)
C  ---------------------------------------------------------------------
C
C  PROCESSES THE "IF" STATEMENT. THE SYNTAX OF THIS STATEMENT IS:
C
C  "IF" <MCL EXPRESSION> "THEN"
C
C  THE OBJECT CODE GENERATED IS AS:
C
C  ( CODE TO EVALUATE THE <MCL EXPRESSION> )
C  $NOT
C  $TESTMOVE
C  <BLANK LINE>
C
C  IF THE STATEMENT IS CORRECT, A VALUE OF 1 IS PUSHED IN THE TYPE
C  TABLE, WHILE THE LOCATION OF THE ABOVE <BLANK LINE> IS PUSHED IN THE
C  TYPE-DEFINITION TABLE.
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C               THE INPUT LINE.
C    NUM        REAL ARRAY. THE NUMBERS ENCOUNTERED IN THE INPUT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C    VAR        CHARACTER ARRAY. THIS IS THE VAR TABLE.
C    LOC        INTEGER ARRAY. THIS IS THE MEMORY LOCATION TABLE.
C    DIM        INTEGER ARRAY. THIS IS THE DIMENSIONALITY TABLE.
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    IPOS       INTEGER ARRAY. THIS IS THE POSITION TABLE.
C    KIND       INTEGER ARRAY. THIS IS THE KIND TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES EXTRA, XPR, ERROR, OVER.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF A LINE. (CHARS)
      PARAMETER (  LINLEN = 80  )
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER ( IDLEN = 30 )
C
      INTEGER TOKEN(*),TYPE(*),TDEF(*),LOC(*),DIM(*),INFO(*),IPOS(*),
     >        KIND(*)
      CHARACTER*(*) ALPHA(*),VAR(*)
      REAL NUM(*)
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      LOGICAL CORRE
      INTEGER EX(LINLEN)
      CHARACTER*(IDLEN) STARG(1)
C
C  INITIALIZE POINTERS TO THE TOKEN, ALPHA AND NUM ARRAYS RESPECTIVELY.
C
      NT=1
      NA=1
      NN=0
C
C  EXTRACT AND EVALUATE THE <MCL EXPRESSION>.
C
      CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,1,ALPHA,NA)
      CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,BLOC,LOC,
     >         DIM,INFO,IPOS,KIND,SCR,STARG)
C  IF EXPRESSION INCORRECT.
      IF (.NOT.CORRE) RETURN
C  WRITE CODE TO SKIP THE IF-PART OF THE BLOCK IF, IN CASE THE
C  EXPRESSION YIELDS A ZERO VALUE. (WHICH MEANS THAT THE GIVEN
C  CONDITION IS NOT SATISFIED)
      BLOC=BLOC+3
      IF (GENE)  WRITE(SCR,'(A/A/)') '$NOT','$TESTMOVE'
C
C  "THEN" IS EXPECTED AFTER THE <MCL EXPRESSION> IN THE "IF" STATEMENT.
C
      NT=NT+1
      IF (TOKEN(NT).NE.IDENT) THEN
         CALL ERROR(8,' ')
         RETURN
      ELSE
         NA=NA+1
         IF (ALPHA(NA).NE.'THEN') THEN
            CALL ERROR(8,ALPHA(NA))
            RETURN
         END IF
      END IF
 
C
C  THE "IF" STATEMENT, IS FOUND TO BE CORRECT. UPDATE THE TYPE
C  AND TYPE-DEFINITION TABLES.
C
      NTYPE=NTYPE+1
      IF (NTYPE.GT.MAXTYP) CALL OVER('TYPE','TYPE','IF',MAXTYP,'MXTYP')
      TYPE(NTYPE)=1
      TDEF(NTYPE)=BLOC
C
      RETURN
      END
CINI--------------------------------------------------------------------
      SUBROUTINE INI (TOKENS,TOKN,TUSED,ALPHAC,NA)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: PARSES INIT STATEMENT.
C
C   %..   ARGUMENT INFORMATION:
C
C    -TOKENS : THE INPUT LINE TO BE PARSED,
C    -TOKN   : NUMBER OF TOKENS,
C    -TUSED  : POINTS TO THE LAST USED ELEMENT OF TOKES ARRAY,
C    -ALPHAC : ALPHABETIC CONTENTS OF THE LINE BEING SCANNED,
C    -NA     : LAST USED ELEMENT OF ALPHAC ARRAY.
C
C   %..   SUBPROGRAMS CALLED:
C
C     SUBROUTINES: ERROR,
C     FUNCTIONS  : NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C      MCLUNT (A), MCLDTA (M) , MCLSCD (A)
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS: NONE.
C
C   %..   NOTES:
C
C      SYNTAX: 'INIT' '('  'X'\'L'\'R'  '.'  <FILENAME> [';'
C                'X'\'L'\'R' '.' <FILENAME> ] ')'
C
C-----------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
      DIMENSION TOKENS(TOKN)
      LOGICAL GENE,BOUNDS,DEBUG,XFLAG,LFLAG,RFLAG,EOL
      CHARACTER*(*) ALPHAC(*),FILENM*30,ID*30
C
      PARAMETER (LOPTIO=-1,XOPTIO=0,ROPTIO=1)
C
      COMMON / MCLUNT / INP, ERL, BIN, SCR, ISTF, SPARZZ, DUMMYF,DESCRI
     >       / MCLDTA / NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
     >       / MCLSCD / LEFTP, RIGHTP, DOT, COMMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUMBER,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
C
      EOL()=(TP.EQ.TOKN)
C
C  START:
C
      TP=TUSED+1
      IF (EOL()) GOTO 200
      XFLAG=.FALSE.
      RFLAG=.FALSE.
      LFLAG=.FALSE.
      IF (TOKENS(TP).NE.LEFTP) THEN
         CALL ERROR ( 118 ,' ' )
         RETURN
      END IF
 100  CONTINUE
      TP=TP+1
      IF (EOL()) GOTO 200
      NA=NA+1
      ID=ALPHAC(NA)
      IF (TOKENS(TP).NE.IDENT) THEN
         CALL ERROR ( 119 , ' ' )
         RETURN
      END IF
      IF (ID.EQ.'X') THEN
          IF (XFLAG) THEN
            CALL ERROR ( 120 , ' ' )
            RETURN
          END IF
          TP=TP+1
          IF (EOL()) GOTO 200
          IF (TOKENS(TP).NE.DOT) THEN
            CALL ERROR ( 121 , ' ' )
            RETURN
          END IF
          TP=TP+1
          IF (EOL()) GOTO 200
          IF (TOKENS(TP).NE.IDENT) THEN
            CALL ERROR ( 122 , ' ' )
            RETURN
          END IF
          NA=NA+1
          FILENM=ALPHAC(NA)
          XFLAG=.TRUE.
          BLOC = BLOC + 3
          IF (GENE) WRITE(SCR,'(''INIT'',/,I1,/,A)')XOPTIO,FILENM
      ELSE IF (ID.EQ.'L') THEN
          IF (LFLAG) THEN
            CALL ERROR ( 123 , ' ' )
            RETURN
          END IF
          TP=TP+1
          IF (EOL()) GOTO 200
          IF (TOKENS(TP).NE.DOT) THEN
            CALL ERROR ( 124 , ' ' )
            RETURN
          END IF
          TP=TP+1
          IF (EOL()) GOTO 200
          IF (TOKENS(TP).NE.IDENT) THEN
            CALL ERROR ( 125 , ' ' )
            RETURN
          END IF
          NA=NA+1
          FILENM=ALPHAC(NA)
          LFLAG=.TRUE.
          BLOC = BLOC + 3
          IF (GENE) WRITE(SCR,'(''INIT'',/,I2,/,A)')LOPTIO,FILENM
      ELSE IF (ID.EQ.'R') THEN
          IF (RFLAG) THEN
            CALL ERROR ( 126 , ' ' )
            RETURN
          END IF
          TP=TP+1
          IF (EOL()) GOTO 200
          IF (TOKENS(TP).NE.DOT) THEN
            CALL ERROR ( 127 , ' ' )
            RETURN
          END IF
          TP=TP+1
          IF (EOL()) GOTO 200
          IF (TOKENS(TP).NE.IDENT) THEN
            CALL ERROR ( 128 , ' ' )
            RETURN
          END IF
          NA=NA+1
          FILENM=ALPHAC(NA)
          RFLAG=.TRUE.
          BLOC = BLOC + 3
          IF (GENE) WRITE(SCR,'(''INIT'',/,I1,/,A)')ROPTIO,FILENM
      ELSE
          CALL ERROR ( 129 , ' ' )
          RETURN
      END IF
      TP=TP+1
      IF (EOL()) GOTO 200
      IF (TOKENS(TP).EQ.SEMI) THEN
        GOTO 100
      ELSE IF (TOKENS(TP).EQ.RIGHTP.AND.TP+1.EQ.TOKN) THEN
        RETURN
      ELSE
        CALL ERROR ( 130 , ' ' )
        RETURN
      END IF
 200  CALL ERROR ( 131 , ' ' )
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE INSERT ( VAR,DIM,LOC,IPOS,KIND,ID,ND,ML,IP,KI,IND )
C  ---------------------------------------------------------------------
C
C  INSERTS INFORMATION IN THE TABLES RELATED TO VARIABLES, ARRAYS AND
C  STATEMENT FUNCTIONS. THE TABLES AFFECTED ARE:
C
C  1) THE VAR TABLE
C  2) THE DIMENSIONALITY TABLE
C  3) THE MEMORY LOCATION TABLE
C  4) THE POSITION TABLE AND
C  5) THE KIND TABLE.
C
C  THE POSITION WHERE INFORMATION SHOULD BE INSERTED IS
C  INPUT TO THE ROUTINE.
C
C  INPUT ARGUMENTS:
C
C    ID         CHARACTER VARIABLE. THIS VALUE WILL BE INSERTED IN
C               THE VAR TABLE.
C    ND         INTEGER VARIABLE. THIS VALUE WILL BE INSERTED IN THE
C               DIMENSIONALITY TABLE.
C    ML         INTEGER VARIABLE. THIS VALUE WILL BE INSERTED IN THE
C               MEMORY LOCATION TABLE.
C    IP         INTEGER VARIABLE. THIS VALUE WILL BE INSERTED IN THE
C               POSITION TABLE.
C    KI         INTEGER VARIABLE. THIS VALUE WILL BE INSERTED IN THE
C               KIND TABLE.
C    IND        INTEGER VARIABLE. THIS IS THE POSITION WHERE THE INPUT
C               VALUES SHOULD BE INSERTED IN THE TABLES STATED ABOVE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    VAR        CHARACTER ARRAY. THIS IS THE VAR TABLE.
C    DIM        INTEGER ARRAY. THIS IS THE DIMENSIONALITY TABLE.
C    LOC        INTEGER ARRAY. THIS IS THE MEMORY LOCATION TABLE.
C    IPOS       INTEGER ARRAY. THIS IS THE POSITION TABLE.
C    KIND       INTEGER ARRAY. THIS IS THE KIND TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINE ABORT.
C
C  ABORT CONDITIONS THAT MAY OCCUR:
C    IN CASE THE TABLES STATED, OVERFLOW, THE PROGRAM WILL STOP AND
C    ISSUE AN INFORMATIVE MESSAGE.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      CHARACTER*(*) VAR(*),ID
      INTEGER DIM(*),LOC(*),IPOS(*),KIND(*)
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      NVAR=NVAR+1
      IF (NVAR.GT.MAXVAR) CALL OVER('VAR','VAR','INSERT',MAXVAR,'MXVAR')
C
C  MOVE UP ALL TABLE ENTRIES BY ONE POSITION
C
      DO 80,K=NVAR,IND+1,-1
        KM1=K-1
        VAR(K)  = VAR(KM1)
        DIM(K)  = DIM(KM1)
        LOC(K)  = LOC(KM1)
        IPOS(K) = IPOS(KM1)
        KIND(K) = KIND(KM1)
 80   CONTINUE
C
C  INSERT THE INPUT VALUES AT THE SPECIFIED POSITION.
C
      VAR(IND) = ID
      DIM(IND) = ND
      LOC(IND) = ML
      IPOS(IND)= IP
      KIND(IND)= KI
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE JUMPIN ( F1,F2,T1,T2,TYPE,TDEF,TREL )
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE CHECKS WHETHER ANY JUMPS ARE PERFORMED FROM A BLOCK
C  OF STATEMENTS ( SPECIFIED BY TWO POINTERS IN THE TYPE TABLE ) TO
C  ANOTHER BLOCK OF STATEMENTS. ( SPECIFIED ALSO BY TWO POINTERS IN
C  THE TYPE TABLE ) UPON EXIT, ALL "MOVE TO" STATEMENTS THAT HAVE BEEN
C  PROCESSED WILL ASSUME A VALUE OF 7.
C
C  INPUT ARGUMENTS:
C
C    F1         INTEGER VARIABLE. THIS IS A POINTER TO THE TYPE TABLE,
C               AND IS THE LOWER LIMIT FROM WHERE THE SEARCH FOR
C               ANY "MOVE TO" STATEMENTS WILL START.
C    F2         INTEGER VARIABLE. THIS IS A POINTER TO THE TYPE TABLE,
C               AND IS THE UPPER LIMIT TO WHERE THE SEARCH FOR
C               ANY "MOVE TO" STATEMENTS WILL TERMINATE.
C    T1         INTEGER VARIABLE. THIS IS A POINTER TO THE TYPE TABLE,
C               AND IS THE LOWER LIMIT OF THE BLOCK THAT SHOULDN'T
C               BE THE TARGET OF A "MOVE TO" STATEMENT RESIDING
C               IN THE FIRST BLOCK.
C    T2         INTEGER VARIABLE. THIS IS A POINTER TO THE TYPE TABLE,
C               AND IS THE UPPER LIMIT OF THE BLOCK THAT SHOULDN'T
C               BE THE TARGET OF A "MOVE TO" STATEMENT RESIDING
C               IN THE FIRST BLOCK.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C    TREL       INTEGER ARRAY. THIS IS THE TYPE-LINK TABLE.
C
C  ---------------------------------------------------------------------
C
      INTEGER F1,F2,T1,T2,TYPE(*),TDEF(*),TREL(*)
C
C  CALCULATE DEFINITION LOCATIONS FOR THE SECOND BLOCK.
C
      LOC1 = TDEF(T1)
      LOC2 = TDEF(T2)
C
C  START SEARCHING THE TYPE TABLE.
C
      DO 10,I=F1,F2
        IF (TYPE(I).EQ.-7) THEN
C  A "MOVE TO" STATEMENT HAS BEEN FOUND.
C  SINCE THE TYPE-LINK TABLE HOLDS ONLY RELATIVE NUMBERS, CALCULATE
C  THE ABSOLUTE LOCATION OF THE TARGET LABEL.
           IR=TREL(I)
           IF (IR.LT.0) IR=IR-1
           JT=TDEF(I)+IR
C  IF THE TARGET LOCATION LIES INSIDE THE SECOND BLOCK,
C  THIS IS AN ERROR.
           IF (JT.GE.LOC1 .AND. JT.LE.LOC2) THEN
              CALL ERROR(148,' ')
              TYPE(I) = 7
              TREL(I) = -TDEF(I)+1
           END IF
        END IF
 10   CONTINUE
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE LBDEF ( NTOK, LABEL, LABLOC, NAME )
C  ---------------------------------------------------------------------
C
C  PROCESSES LABEL DEFINITION. IF THE LABEL HAS ALREADY BEEN DEFINED
C  AN ERROR OCCURS.
C  ONLY ONE LINE OF OBJECT CODE IS GENERATED:
C
C  $CONTINUE
C
C  THE LABEL IS ASSIGNED THE LOCATION OF THE ABOVE LINE IN THE SCRATCH
C  FILE ( UNIT SCR ).
C
C  INPUT ARGUMENTS:
C
C    NTOK   NUMBER OF TOKENS IN THE CURRENT LINE.
C    NAME   THE LABEL SYMBOLIC NAME TO BE PROCESSED.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    LABEL   THE LABEL TABLE.
C    LABLOC  THE LABEL LOCATION TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES LOCAN, ERROR, OVER.
C
C  ABORT CONDITIONS THAT MAY OCCUR:
C    IF THE LABEL TABLE OVERFLOWS, THE PROGRAM WILL STOP AND ISSUE AN
C    INFORMATIVE MESSAGE.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      INTEGER LABLOC(*)
      CHARACTER*(*) LABEL(*),NAME
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      LOGICAL FOUND
C
C  LOCATE LABEL IN THE ASSOCIATED TABLE.
C
      CALL LOCAN(LABEL,NLAB,NAME,FOUND,IND)
      IF (FOUND) THEN
C  LABEL ALREADY IN TABLE. ISSUE ERROR MESSAGE.
         CALL ERROR(1,NAME)
      ELSE
C  LABEL NOT IN TABLE, SO INSERT IT AND ASSIGN TO IT
C  THE APPROPRIATE LOCATION.
         NLAB=NLAB+1
         IF (NLAB.GT.MAXLAB)
     >              CALL OVER('LABEL','LABEL','LBDEF',MAXLAB,'MXLAB')
         DO 20,I=NLAB,IND+1,-1
           LABEL(I)=LABEL(I-1)
           LABLOC(I)=LABLOC(I-1)
 20      CONTINUE
         BLOC=BLOC+1
C  PUT THE LABEL IN THE LABEL TABLE.
         LABEL(IND)=NAME
C  ASSIGN THE SCRATCH FILE LOCATION TO THE LABEL.
         LABLOC(IND)=BLOC
         IF (GENE) WRITE(SCR,'(A)') '$CONTINUE'
      END IF
C
C  IF MORE THAN 3 TOKENS ( LABEL COLON EOL ) ARE IN THE SAME LINE,
C  THIS IS AN ERROR.
C
      IF (NTOK.NE.3) CALL ERROR(2,' ')
      RETURN
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE LCR(LINE,IS,IE,NIT)
      CHARACTER*(*) LINE
      INTEGER IS(*), IE(*)
      LOGICAL INWORD
      CHARACTER C
C
      L = LEN(LINE)
      NIT = 0
      INWORD = LINE(1:1).NE.' '
      IF (INWORD) THEN
         NIT = 1
         IS(1) = 1
      END IF
C
      DO 10,I=2,L
        C = LINE(I:I)
        IF (C.EQ.' ') THEN
           IF (INWORD) THEN
            INWORD = .FALSE.
              IE(NIT) = I-1
           END IF
        ELSE
           IF (.NOT.INWORD) THEN
              INWORD = .TRUE.
              NIT = NIT+1
              IS(NIT) = I
           END IF
        END IF
 10   CONTINUE
C
      IF (INWORD) IE(NIT) = L
      RETURN
      END
CLENGTH-----------------------------------------------------------------
      INTEGER FUNCTION LENGTH(LINE)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: THIS FUNCTION FINDS THE EFFECTIVE
C                  LENGTH OF A STRING OF CHARACTERS.
C
C   %..   ARGUMENT INFORMATION:
C
C             LINE = THE INPUT STRING OF CHARACTERS TO BE
C                    MEASURED FOR LENGTH. INITIAL SPACES
C                    DO COUNT. TAIL SPACES ARE IGNORED.
C
C   %..   SUBPROGRAMS CALLED: NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M): NONE.
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   NOTES: IT IS A GENERAL PURPOSE ROUTINE.
C                 IT ELIMINATES THE BLANKS IN THE TAIL
C                 OF A GIVEN STRING OF CHARACTERS.
C                 IN MCLCOM IS USED BY ROUTINE LEX.
C
C-----------------------------------------------------------------------
C
      CHARACTER LINE*(*)
      INTEGER LE
C
      LE=LEN(LINE)
 100  CONTINUE
         IF ( LE .GT. 0 ) THEN
            IF ( LINE(LE:LE).EQ.' ')THEN
               LE=LE-1
            ELSE
               LENGTH=LE
               RETURN
            ENDIF
         ELSE
            LENGTH=0
            RETURN
         ENDIF
      GOTO 100
      END
CLEX--------------------------------------------------------------------
      SUBROUTINE LEX  (LINE,TOKENS,ALLTOK,NUMBC,ALPHAC)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: LEXICAL ANALYSER OF MCLCOM.
C
C     THIS ROUTINE, SCANS A GIVEN SEQUENCE OF CHARACTERS ("LINE"),
C     DETERMINES ITS STRUCTURE (LEXICAL ANALYSIS), AND KEEPS A CODE FOR
C     EVERY TOKEN FOUND, TOGETHER WITH ITS VALUE (I.E. CHARACTER NAMES
C     FOR IDENTIFIERS (NOTE THAT ALPHABETICAL MCL OPERATORS ARE
C     NOT CONSIDERED IDENTIFIERS), REAL NUMBERS FOR NUMERIC
C     CONSTANTS, AND STRING-CONTENTS PLUS THEIR LENGTH FOR STRINGS).
C     BLANKS AND UNDERSCORES ARE IGNORED WHEN DETECTED (PACKING).
C
C   %..   ARGUMENT INFORMATION:
C
C    -LINE    THE INPUT LINE TO BE LEXICALLY ANALYSED.
C             IT IS CHARACTER*(*) VARIABLE.
C    -TOKENS  INTEGER (*) ARRAY WHICH UPON RETURN, HOLDS THE STRUCTURE
C             OF THE "LINE" IN FORM OF INTEGER CODES.
C    -ALLTOK  INTEGER HOLDING UPON EXIT THE TOTAL NUMBER OF ALL
C             DIFFERENT TOKENS DETECTED INTO "LINE". IT IS THE EFFECTIVE
C             LENGTH OF ARRAY TOKENS.
C    -NUMBC   REAL ARRAY (*) HOLDING THE NUMERIC CONTENTS OF LINE IN
C             FORM OF REAL NUMBERS.
C    -ALPHAC  CHARACTER*(IDLEN) ARRAY (*) KEEPING THE ALPHABETIC
C             CONTENTS OF LINE, I.E. SYMBOLIC NAMES, AND
C             STRING-DOUBLETS.(NOTE:IDLEN=30).
C
C   %..   SUBPROGRAMS CALLED:
C
C              SUBROUTINES: PSHSTR, PSHNUM, PSHALP,
C              FUNCTIONS  : LENGTH, LXTRAN.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C     TTSTR(M), TTALP(M), TTNUM(M), ALPH(M), STRUC(M),
C     NUMB(M), MCLSCD(A).
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS:
C
C    -LINE    CHARACTER*(*) VARIABLE HOLDING THE LINE BEING SCANNED.
C    -FINGER  INTEGER POINTER TO THE CHARACTER OF LINE WHERE LEX IS ON.
C    -IDBUF   CHARACTER*(IDLEN) VARIABLE HOLDING SYMBOLIC NAME OF AN
C             IDENTIFIER EACH TIME ONE OCCURS INTO THE LINE.
C    -ALPCON  USED TO LOAD ALPHAC. CHARACTER*(IDLEN) ARRAY (1:LINLEN).
C    -NUMCON  USED TO LOAD NUMBC. REAL ARRAY (1:LINLEN).
C    -CSTRUC  INTEGER ARRAY (1:LINLEN+1) USED TO LOAD TOKENS.
C             THE ABOVE THREE ARRAYS HOLD THE DATA FOR EACH LINE.
C    -STBUF   CHARACTER*(2*IDLEN) VARIABLE HOLDING STRING IF ANY OCCURS
C             EACH TIME IT OCCURS.
C
C   %..   NOTES:
C
C     CSTRUC, NUMCON AND ALPCON ARRAYS ( AND CORRESPONDINGLY TOKENS,
C     NUMBC AND ALPHAC ARRAYS) ARE LOADED VIA A STACK MECHANISM,
C     SO AS UPON RETURN, THEIR CONTENTS ARE AS FOLLOWS:
C     FIRST TOKEN DETECTED IS FIRST ELEMENT OF ARRAY TOKENS,
C     SECOND TOKEN IS SECOND ELEMENT OF TOKENS, ETC.
C     FINAL ELEMENT (I.E ALLTOK ELEMENT) IS THE EOL SIGN.
C     THE SAME PHILOSOPHY HOLDS FOR THE
C     CONTENTS OF THE OTHER ARRAYS (NUMBC AND ALPHAC).
C     A STRING IS ANYTHING BETWEEN SINGLE QUOTES ('). IT IS KEPT AS A
C     DOUBLET INTO ALPHAC INTO TWO ADJACENT PLACES. ITS LENGTH IS
C     KEPT INTO NUMBC. (A MINI-PARSING IS PERFORMED
C     HERE AS PART OF THE STRING PROCESSING. THIS IS DONE SO
C     THAT TO ALLOW ANYTHING EXCEPT SINGLE QUOTES TO APPEAR
C     INTO A STRING (SEE MCL DEFINITION)).
C
C     THERE IS NO NEED TO GIVE OUT THE LENGTHS OF NUMBC AND ALPHAC
C     BECAUSE THE OTHER PARTS OF MCLCOM USE POINTERS TO THOSE ARRAYS.
C     WHENEVER AN ELEMENT FROM THOSE IS NEEDED, THESE POINTERS ARE
C     CONTROLLED BY THE CONTENTS OF TOKENS.
C
C-----------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      PARAMETER ( LINLEN = 80 , IDLEN = 30 )
C
C   LINLEN STANDS FOR MCL-LINE-LENGTH.
C   IDLEN REPRESENTS MCL-IDENTIFIER-LENGTH.
C
      DIMENSION CSTRUC(LINLEN+1), TOKENS(*)
C
      CHARACTER LINE*(*),IDBUF*(IDLEN),ALPCON(LINLEN)*(IDLEN),
     >          ALPHAC(*)*(*),STBUF*(2*IDLEN),CHA*1
      CHARACTER TAB, UCHA, UPPER
C
      LOGICAL DOTFND,EOL,ALPHA,NMERIC,SPCIAL,UNDER
C
      REAL NUMBER, NUMCON(LINLEN), NUMBC(*), RE
C
      COMMON / TTSTR / TOPSTR
     >       / TTNUM / TOPNUM
     >       / TTALP / TOPALP
     >       / ALPH  / ALPCON
     >       / STRUC / CSTRUC
     >       / NUMB  / NUMCON
     >       / MCLSCD / LEFTP, RIGHTP, DOT, COMMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUM,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
C
      EOL()=(FINGER.GE.LENG)
      ALPHA(CHA)=('A'.LE.CHA).AND.(CHA.LE.'Z')
      NMERIC()=('0'.LE.CHA).AND.(CHA.LE.'9')
      SPCIAL()=(CHA.EQ.'=').OR.(CHA.EQ.'(').OR.(CHA.EQ.')').OR.
     >         (CHA.EQ.'.').OR.(CHA.EQ.',').OR.(CHA.EQ.';').OR.
     >         (CHA.EQ.'+').OR.(CHA.EQ.'-').OR.(CHA.EQ.'/').OR.
     >         (CHA.EQ.'[').OR.(CHA.EQ.']').OR.(CHA.EQ.':').OR.
     >         (CHA.EQ.'#')
      UNDER()=(CHA.EQ.'_')
C
C  START
C
C  SET TAB = ' ' OR TAB = CHAR(WHATEVER) IF USING 
C  A NON-ASCII CHARACTER SET.
C
      TAB = CHAR(9)
      FINGER=0
C
C     GET THE EFFECTIVE LENGTH OF LINE BY ELIMINATING BLANKS OF TAIL.
C
      LENG=LENGTH(LINE)
C
C     TOPSTR, TOPNUM, TOPALP ARE THE TOPS OF THE CORRESPONDING STACKS
C     AND THEIR PROCESSING IS PERFORMED VIA PSHSTR, PSHNUM, PSHALP.
C
      TOPSTR=0
      TOPNUM=0
      TOPALP=0
C
 100  CONTINUE
C
      IF ( EOL() ) THEN
C
C         BLANK LINE GIVEN AS INPUT.
C
          GO TO 9000
      ELSE
          FINGER=FINGER+1
      END IF
C
C     CHA ALLWAYS KEEPS CURRENT CHARACTER ON WHICH LEX IS WORKING ON.
C
      CHA = LINE ( FINGER:FINGER )
      UCHA = UPPER(CHA)
C
C     WE JUMP TO 200, WHEN THERE IS FURTHER PROCESSING TO
C     BE DONE FROM OTHER PARTS OF LEX DURING SCANNING.
C
 200  CONTINUE
C
      IF ( ALPHA(UCHA) ) THEN
C
C          ALPHABETIC CHARACTER LOCATED.
C          HERE, LOGICAL OPERATORS (AND,OR,XOR,NOT) ARE
C          BEING IDENTIFIED TOO.
C
           IDBUF=UCHA
           IDPOIN=1
 300       IF ( .NOT.EOL() ) THEN
               FINGER=FINGER+1
               CHA=LINE ( FINGER:FINGER )
               UCHA = UPPER(CHA)
               IF ( (ALPHA(UCHA).OR.NMERIC().OR.UNDER()).AND.
     >             (IDPOIN.LT.IDLEN) ) THEN
C
C                  IGNORE UNDERSCORES WHEN APPEAR INTO IDENTIFIERS.
C
                   IF(UNDER()) GO TO 300
                   IDPOIN=IDPOIN+1
                   IDBUF(IDPOIN:IDPOIN)=UCHA
                   GOTO 300
               ELSE
C
C                  OBTAIN OPERATOR CODE (IF OPERATOR).
C
                   IF ( IDBUF.EQ.'AND' ) THEN
                      CALL PSHSTR(AND)
                   ELSE IF ( IDBUF.EQ.'OR' ) THEN
                      CALL PSHSTR(OR)
                   ELSE IF ( IDBUF.EQ.'XOR' ) THEN
                      CALL PSHSTR(XOR)
                   ELSE IF ( IDBUF.EQ.'NOT' ) THEN
                      CALL PSHSTR(NOT)
                   ELSE
                      CALL PSHSTR(IDENT)
                      CALL PSHALP(IDBUF)
                   END IF
                   GOTO 200
               END IF
           ELSE
               IF ( IDBUF.EQ.'AND' ) THEN
                  CALL PSHSTR(AND)
               ELSE IF ( IDBUF.EQ.'OR' ) THEN
                  CALL PSHSTR(OR)
               ELSE IF ( IDBUF.EQ.'XOR' ) THEN
                  CALL PSHSTR(XOR)
               ELSE IF ( IDBUF.EQ.'NOT' ) THEN
                  CALL PSHSTR(NOT)
               ELSE
                  CALL PSHSTR(IDENT)
                  CALL PSHALP(IDBUF)
               END IF
           END IF
C
      ELSE IF ( NMERIC() ) THEN
C
C         NUMERIC CHARACTER LOCATED. PROCEED TO TRANSFORM
C         THE SEQUENCE OF CHARACTERS TO REAL NUMBER.
C
          DOTFND=.FALSE.
          NUMBER=REAL(ICHAR(CHA)-ICHAR('0'))
          EXPO=0
          POWER=-1
 500      CONTINUE
          IF ( .NOT.EOL() )THEN
             FINGER=FINGER+1
             CHA=LINE(FINGER:FINGER)
             UCHA = UPPER(CHA)
             IF ( NMERIC() )THEN
                IF( POWER.EQ.-1 ) THEN
                   NUMBER=NUMBER*10.+REAL(ICHAR(CHA)-ICHAR('0'))
                   GOTO 500
                ELSE
                   POWER=POWER+1
                   NUMBER=NUMBER+REAL(ICHAR(CHA)-ICHAR('0'))/(10.**
     >             POWER)
                   GOTO 500
                END IF
             ELSE IF ( CHA.EQ.'.' ) THEN
                POWER=0
                IF ( DOTFND ) THEN
                   CALL PSHSTR(NUM)
                   CALL PSHNUM(NUMBER)
                   CALL PSHSTR(DOT)
                   GOTO 100
                ELSE
                   DOTFND=.TRUE.
                   GOTO 500
                END IF
             ELSE IF ( UCHA.EQ.'E' ) THEN
                IF ( .NOT.EOL() ) THEN
                   FINGER=FINGER+1
                   CHA=LINE(FINGER:FINGER)
                   UCHA = UPPER(CHA)
                   IF ( NMERIC() ) THEN
                      EXPO=EXPO*10+(ICHAR(CHA)-ICHAR('0'))
 600                  CONTINUE
                      IF ( .NOT.EOL() ) THEN
                         FINGER=FINGER+1
                         CHA=LINE(FINGER:FINGER)
                         UCHA = UPPER(CHA)
                         IF ( NMERIC() ) THEN
                            EXPO=EXPO*10+(ICHAR(CHA)-ICHAR('0'))
                            GOTO 600
                         ELSE
                            NUMBER=NUMBER*(10.**EXPO)
                            CALL PSHSTR(NUM)
                            CALL PSHNUM(NUMBER)
                            GOTO 200
                         END IF
                      ELSE
                         NUMBER=NUMBER*(10.**EXPO)
                         CALL PSHSTR(NUM)
                         CALL PSHNUM(NUMBER)
                      END IF
                   ELSE IF ( CHA.EQ.'+' ) THEN
                      IF ( .NOT.EOL() ) THEN
                         FINGER=FINGER+1
                         CHA=LINE(FINGER:FINGER)
                         UCHA = UPPER(CHA)
                         IF ( NMERIC() ) THEN
                            EXPO=EXPO*10+(ICHAR(CHA)-ICHAR('0'))
 800                        CONTINUE
                            IF ( .NOT.EOL() ) THEN
                               FINGER=FINGER+1
                               CHA=LINE(FINGER:FINGER)
                               UCHA = UPPER(CHA)
                               IF ( NMERIC() ) THEN
                                  EXPO=EXPO*10+(ICHAR(CHA)-ICHAR(
     >                            '0'))
                                  GOTO 800
                               ELSE
                                  NUMBER=NUMBER*(10.**EXPO)
                                  CALL PSHSTR(NUM)
                                  CALL PSHNUM(NUMBER)
                                  GOTO 200
                               END IF
                            ELSE
                               NUMBER=NUMBER*(10.**EXPO)
                               CALL PSHSTR(NUM)
                               CALL PSHNUM(NUMBER)
                            ENDIF
                         ELSE
                            CALL PSHSTR(NUM)
                            CALL PSHNUM(NUMBER)
                            CALL PSHSTR(IDENT)
                            CALL PSHALP('E')
                            CALL PSHSTR(PLUS)
                            GOTO 200
                         END IF
                      ELSE
                         CALL PSHSTR(NUM)
                         CALL PSHNUM(NUMBER)
                         CALL PSHSTR(IDENT)
                         CALL PSHALP('E')
                         CALL PSHSTR(PLUS)
                      END IF
                   ELSE IF ( CHA.EQ.'-' ) THEN
                      IF ( .NOT.EOL() ) THEN
                         FINGER=FINGER+1
                         CHA=LINE(FINGER:FINGER)
                         UCHA = UPPER(CHA)
                         IF ( NMERIC() ) THEN
                            EXPO=EXPO*10+(ICHAR(CHA)-ICHAR('0'))
 820                        CONTINUE
                            IF ( .NOT.EOL() ) THEN
                               FINGER=FINGER+1
                               CHA=LINE(FINGER:FINGER)
                               UCHA = UPPER(CHA)
                               IF ( NMERIC() ) THEN
                                  EXPO=EXPO*10+(ICHAR(CHA)-ICHAR(
     >                            '0'))
                                  GOTO 820
                               ELSE
                                  NUMBER=NUMBER*(10.**(-EXPO))
                                  CALL PSHSTR(NUM)
                                  CALL PSHNUM(NUMBER)
                                  GOTO 200
                               END IF
                            ELSE
                               NUMBER=NUMBER*(10.**(-EXPO))
                               CALL PSHSTR(NUM)
                               CALL PSHNUM(NUMBER)
                            ENDIF
                         ELSE
                            CALL PSHSTR(NUM)
                            CALL PSHNUM(NUMBER)
                            CALL PSHSTR(IDENT)
                            CALL PSHALP('E')
                            CALL PSHSTR(MINUS)
                            GOTO 200
                         END IF
                      ELSE
                         CALL PSHSTR(NUM)
                         CALL PSHNUM(NUMBER)
                         CALL PSHSTR(IDENT)
                         CALL PSHALP('E')
                         CALL PSHSTR(MINUS)
                      END IF
                   ELSE
                      FINGER=FINGER-1
C
C                     MINUS BECAUSE WE UNGET CHA
C
                      CALL PSHSTR(NUM)
                      CALL PSHNUM(NUMBER)
                      GOTO 200
                   END IF
               ELSE
                   CALL PSHSTR(NUM)
                   CALL PSHNUM(NUMBER)
                   CALL PSHSTR(IDENT)
                   CALL PSHALP(CHA)
                END IF
             ELSE
                CALL PSHSTR(NUM)
                CALL PSHNUM(NUMBER)
                GOTO 200
             END IF
          ELSE
             CALL PSHSTR(NUM)
             CALL PSHNUM(NUMBER)
          END IF
C
      ELSE IF ( SPCIAL() ) THEN
C
C         SPECIAL SITUATION.
C
          IF ( CHA.EQ.':' ) THEN
            IF ( .NOT. EOL() ) THEN
               FINGER = FINGER + 1
               CHA = LINE ( FINGER : FINGER )
               UCHA = UPPER(CHA)
               IF ( UCHA.EQ.'=' ) THEN
                 IF ( .NOT.EOL() ) THEN
                   FINGER = FINGER + 1
                   CHA = LINE ( FINGER : FINGER )
                   UCHA = UPPER(CHA)
                   IF ( UCHA.EQ.':' ) THEN
C
C                    EQUALITY OPERATOR DETECTED.
C
                     CALL PSHSTR(EQOPER)
                     GOTO 100
                   ELSE
                     GOTO 200
                   END IF
                 ELSE
                   CALL PSHSTR(COLON)
                   CALL PSHSTR(EQ)
                 END IF
               ELSE
                 CALL PSHSTR(COLON)
                 GOTO 200
               END IF
             ELSE
               CALL PSHSTR(COLON)
             END IF
           ELSE
C
C            SPECIAL CHARACTER DETECTED.
C
             SPCODE=LXTRAN(CHA)
             CALL PSHSTR(SPCODE)
             GOTO 100
           END IF
C
      ELSE IF ( CHA.EQ.' ' .OR. CHA.EQ.TAB ) THEN
C
C         PACK SPACES AND TABS SINCE THEY ARE NOT USED LATER.
C
          GOTO 100
      ELSE IF ( CHA.EQ.'*' ) THEN
C
C         STAR LOCATED.
C
          IF ( .NOT.EOL() ) THEN
              FINGER=FINGER+1
              CHA=LINE ( FINGER:FINGER )
              UCHA = UPPER(CHA)
              IF ( UCHA.EQ.'*') THEN
                  CALL PSHSTR(BISTAR)
                  GOTO 100
              ELSE
                  CALL PSHSTR(STAR)
                  GOTO 200
              END IF
          ELSE
              CALL PSHSTR(STAR)
          END IF
C
      ELSE IF ( CHA.EQ.'>' ) THEN
C
C         GT LOCATED.
C
          IF ( .NOT.EOL() ) THEN
              FINGER=FINGER+1
              CHA=LINE ( FINGER:FINGER )
              UCHA = UPPER(CHA)
              IF ( UCHA.EQ.'=') THEN
                  CALL PSHSTR(GE)
                  GOTO 100
              ELSE
                  CALL PSHSTR(GT)
                  GOTO 200
              END IF
          ELSE
              CALL PSHSTR(GT)
          END IF
C
      ELSE IF ( CHA.EQ.'<' ) THEN
C
C         LT LOCATED.
C
          IF ( .NOT.EOL() ) THEN
              FINGER=FINGER+1
              CHA=LINE ( FINGER:FINGER )
              UCHA = UPPER(CHA)
              IF ( UCHA.EQ.'=') THEN
                  CALL PSHSTR(LE)
                  GOTO 100
              ELSE
                  CALL PSHSTR(LT)
                  GOTO 200
              END IF
          ELSE
              CALL PSHSTR(LT)
          END IF
C
      ELSE IF ( CHA .EQ. '''' ) THEN
C
C       QUOTE LOCATED. PROCESS STRING:
C
C       IT IS ASSUMED THAT STRINGS TOTAL LENGTH INTO QUOTES
C       (BESIDES BEING GREATER THAN ZERO), IS AT
C       MOST 2*IDLEN CHARACTERS (ANY CHARACTERS EXCEPT '''') LONG.
C       THE PART WHICH EXCEEDS 2*IDLEN CH. WILL BE TRUNCATED,AND NO
C       CLOSE QUOTE WILL BE SIGNED.
C       STRING"S LENGTH IS PUSHED INTO ARRAY NUMC.
C
              STFING=0
              STBUF=' '
              CALL PSHSTR(QUOTE)
 987          IF(.NOT.EOL())THEN
                 FINGER=FINGER+1
                 CHA=LINE(FINGER:FINGER)
                 IF(.NOT.(CHA.EQ.''''))THEN
                   STFING=STFING+1
                   IF(STFING.GT.2*IDLEN)GOTO 489
                   STBUF(STFING:STFING)=CHA
                   GOTO 987
 489               CONTINUE
                   RE = STFING
                   CALL PSHNUM(RE)
                   CALL PSHSTR(STRING)
                   CALL PSHALP(STBUF(1:IDLEN))
                   CALL PSHALP(STBUF(IDLEN+1:2*IDLEN))
                 ELSE
                   IF(STFING.GT.0)THEN
                     RE = STFING
                     CALL PSHNUM(RE)
                     CALL PSHSTR(STRING)
                     CALL PSHALP(STBUF(1:IDLEN))
                     CALL PSHALP(STBUF(IDLEN+1:2*IDLEN))
                   ENDIF
                   CALL PSHSTR(QUOTE)
C
C                  IN CASE OF NULL STRING WE SENT JUST TWO QUOTE
C                  CODES.
C
                   GO TO 100
                 ENDIF
              ELSE
C
C                EOL REACHED, SO...
C
                 IF(STFING.GT.0)THEN
                   RE = STFING
                   CALL PSHNUM(RE)
                   CALL PSHSTR(STRING)
                   CALL PSHALP(STBUF(1:IDLEN))
                   CALL PSHALP(STBUF(IDLEN+1:2*IDLEN))
                 ENDIF
              ENDIF
C
      ELSE IF ( CHA .EQ. '%' ) THEN
C
C         ARTIFICIAL EOL FACILITY.
C         THIS CODE PORTION IS DUMMY, AS DUMMY IS ALSO THE
C         SPECIAL CODE 26, RESERVED BY MCL-SCANNER (LEX)
C         FOR FUTURE USE.
C
      ELSE
C
C         UNRECOGNIZABLE CHARACTER. SEND MESSAGE-CODE.
C
          CALL PSHSTR(UNRCGN)
C         CONTINUE SCANNING.
          GOTO 100
C
      END IF
C
C        END OF SCANNING.
C
 9000 CONTINUE
C
      CALL PSHSTR(EOLCOD)
C
      ALLTOK=TOPSTR
      DO 124,I=1,ALLTOK
 124     TOKENS(I)=CSTRUC(I)
      DO 125,K=1,TOPALP
 125     ALPHAC(K)=ALPCON(K)
      DO 126,L=1,TOPNUM
 126     NUMBC(L)=NUMCON(L)
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE LOCAN ( TABLE, ELEM, NAME, FOUND, POSIT )
C  ---------------------------------------------------------------------
C
C  LOCATES A NAME IN A CHARACTER TYPE ARRAY, SORTED IN ASCENDING ORDER.
C  THE METHOD USED, IS A BINARY SEARCH.
C
C  INPUT ARGUMENTS:
C
C    TABLE   THE CHARACTER ARRAY THAT WILL BE SEARCHED. IT SHOULD
C            BE SORTED IN ASCENDING ORDER.IT IS ASSUMED TO BE
C            DIMENSIONED AS: TABLE(1:ELEM)
C    ELEM    NUMBER OF ELEMENTS IN THE ABOVE ARRAY.
C    NAME    THE NAME WHOS POSITION IN THE ARRAY WE ARE LOOKING FOR.
C
C  OUTPUT ARGUMENTS:
C
C    FOUND   LOGICAL VARIABLE INDICATING, WHETHER THE REQUESTED NAME
C            WAS (FOUND=.TRUE.), OR WAS NOT FOUND (FOUND=.FALSE.) IN
C            THE GIVEN ARRAY.
C    POSIT   IF THE NAME WAS FOUND IN THE ARRAY, THIS IS ITS POSITION;
C            OTHERWISE, THIS IS THE POSITION THAT THE GIVEN NAME
C            SHOULD OCCUPY.
C
C  ---------------------------------------------------------------------
C
      CHARACTER*(*) TABLE(*),NAME
      LOGICAL FOUND
      INTEGER ELEM,POSIT,FIRST
C
      FIRST=1
      LAST=ELEM
C
 10   CONTINUE
        IF (FIRST.GT.LAST) GOTO 20
        MIDDLE=(FIRST+LAST)/2
        IF (NAME.GT.TABLE(MIDDLE)) THEN
           FIRST=MIDDLE+1
        ELSE
           LAST=MIDDLE-1
        END IF
      GOTO 10
C
 20   POSIT=FIRST
      IF (POSIT.GT.ELEM) THEN
         FOUND=.FALSE.
      ELSE
         FOUND=NAME.EQ.TABLE(POSIT)
      END IF
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE LOCIF (IF,ELSE,END,TYPE)
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE WILL LOCATE AN "IF" AND AN "ELSE" IN THE TYPE TABLE,
C  GIVEN THE POSITION OF THE "ENDIF".
C  IT IS USED WHILE CHECKING THE VALIDITY OF THE "MOVE TO" STATEMENTS
C  AND ASSUMES THAT ENTRIES IN THE TYPE TABLE THAT HAVE NOT BEEN
C  PROCESSED ARE NEGATIVE; POSITIVE OTHERWISE.
C
C  INPUT ARGUMENTS:
C
C    END        INTEGER VARIABLE. THIS IS THE POSITION OF THE "ENDIF"
C               IN THE TYPE TABLE.
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C
C  OUTPUT ARGUMENTS:
C
C    IF         INTEGER VARIABLE. THIS IS THE POSITION OF THE "IF" IN
C               THE TYPE TABLE, CORRESPONDING TO THE GIVEN "ENDIF".
C    ELSE       INTEGER VARIABLE. THIS IS THE POSITION OF THE "ELSE" IN
C               THE TYPE TABLE, CORRESPONDING TO THE GIVEN "ENDIF".
C               A VALUE OF 0 IS RETURNED IF THIS IS AN "IF"
C               WITHOUT "ELSE".
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINE ABORT.
C
C  ABORT CONDITIONS THAT MAY OCCUR:
C    IF AN "IF" CAN'T BE LOCATED IN THE TYPE TABLE, THE PROGRAM WILL
C    STOP AND ISSUE AN INFORMATIVE MESSAGE. NORMALLY THIS SHOULD NEVER
C    HAPPEN. IF IT DOES, CHECK IF MCLCOM SOURCE HAS BEEN ALTERED IN
C    ANY WAY.
C
C  ---------------------------------------------------------------------
C
      INTEGER ELSE,END,TYPE(*)
C
      ELSE=0
C
C  SEARCH THE TYPE ARRAY BACKWARDS, STARTING AT THE "ENDIF" POSITION.
C
      DO 10,I=END,1,-1
        IF (TYPE(I).EQ.-1) THEN
C  AN "IF" HAS BEEN FOUND.
           IF=I
           RETURN
        ELSE IF (TYPE(I).EQ.-2) THEN
C  AN "ELSE" HAS BEEN FOUND. DON'T RETURN, BUT LOCATE THE
C  CORRESPONDING "IF" AS WELL.
           ELSE=I
        END IF
 10   CONTINUE
C
C  AN "IF" COULD NOT BE LOCATED IN THE TYPE TABLE.
C  THIS PROGRAM IS MALFUNCTIONING.
C
      CALL ABORT('COULD NOT LOCATE "IF" IN THE TYPE TABLE.','LOCIF')
      END
C  ---------------------------------------------------------------------
      SUBROUTINE LOCLOO ( LOO, ENDLOO, TYPE )
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE WILL LOCATE A "LOOP" IN THE TYPE TABLE, GIVEN THE
C  POSITION OF THE "ENDLOOP".
C  IT IS USED WHILE CHECKING THE VALIDITY OF THE "MOVE TO" STATEMENTS
C  AND ASSUMES THAT ENTRIES IN THE TYPE TABLE THAT HAVE NOT BEEN
C  PROCESSED ARE NEGATIVE; POSITIVE OTHERWISE.
C
C  INPUT ARGUMENTS:
C
C    ENDLOO     INTEGER VARIABLE. THIS IS THE POSITION OF THE "ENDLOOP"
C               IN THE TYPE TABLE.
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C
C  OUTPUT ARGUMENTS:
C
C    LOO        INTEGER VARIABLE. THIS IS THE POSITION OF THE "LOOP"
C               IN THE TYPE TABLE, CORRESPONDING TO THE GIVEN "ENDLOOP".
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINE ABORT.
C
C  ABORT CONDITIONS THAT MAY OCCUR:
C    IF A "LOOP" CAN'T BE LOCATED IN THE TYPE TABLE, THE PROGRAM WILL
C    STOP AND ISSUE AN INFORMATIVE MESSAGE. NORMALLY THIS SHOULD NEVER
C    HAPPEN. IF IT DOES, CHECK IF MCLCOM SOURCE HAS BEEN ALTERED IN
C    ANY WAY.
C
C  ---------------------------------------------------------------------
C
      INTEGER ENDLOO,TYPE(*)
C
C  START SEARCHING THE TYPE TABLE.
C
      DO 10,LOO=ENDLOO,1,-1
        IF (TYPE(LOO).EQ.-4) RETURN
 10   CONTINUE
C
C  A "LOOP" COULD NOT BE LOCATED IN THE TYPE TABLE.
C  THIS PROGRAM IS MALFUNCTIONING.
C
      CALL ABORT('COULD NOT LOCATE "LOOP" IN THE TYPE TABLE.','LOCLOO')
      END
C  ---------------------------------------------------------------------
      SUBROUTINE LOOP ( TOKEN,ALPHA,NUM,NTOK,VAR,LOC,DIM,INFO,
     >                  IPOS,KIND,LLOC,TYPE,TDEF )
C  ---------------------------------------------------------------------
C
C  PARSES THE LOOP STATEMENT AN GENERATES APPROPRIATE OBJECT CODE.
C  THE STATEMENT SYNTAX IS:
C
C  LOOP <VAR> FROM INIT TO FINAL BY STEP         OR
C  LOOP <VAR> FROM INIT TO FINAL
C
C  WHERE <VAR> IS A SIMPLE VARIABLE ( REFERRED TO AS THE LOOP CONTROL
C  INDEX ) AND
C  INIT,FINAL,STEP ARE EXPRESSIONS SPECIFYING THE INITIAL, FINAL AND
C  STEP VALUES FOR THE LOOP.
C  FOR EACH LOOP 5 CONSECUTIVE STORAGE LOCATIONS ARE NEEDED. THEIR
C  CONTENTS ARE LISTED ALONG WITH THE SYMBOLIC NAME, THEY WILL BE
C  BE REFERRED LATER IN THIS DESCRIPTION:
C
C  1) STORAGE FOR THE VALUE OF INIT. ( INIT_MLOC )
C  2) STORAGE FOR THE VALUE OF FINAL. ( FINAL_MLOC)
C  3) STORAGE FOR THE VALUE OF STEP. ( STEP_MLOC)
C  4) STORAGE FOR THE CURRENT TRIP COUNT OF THE LOOP. ( CTC_MLOC )
C  5) STORAGE FOR THE FINAL TRIP COUNT OF THE LOOP. (FTC_MLOC )
C
C  SINCE THESE MEMORY LOCATIONS ARE USED ONLY WHILE THE LOOP IS
C  EXECUTING, THE STORAGE REQUIRED FOR ALL LOOPS IN THE PROGRAM IS:
C  5*MAXL ,  WHERE MAXL IS THE MAXIMUM NUMBER OF NESTED LOOPS
C  ENCOUNTERED IN THE SOURCE.
C
C  INITIALLY THE INIT,FINAL AND STEP EXPRESSIONS ARE CALCULATED:
C
C  <CODE TO EVALUATE INIT>
C  $POP
C  INIT_MLOC
C  <CODE TO EVALUATE FINAL>
C  $POP
C  FINAL_MLOC
C  <CODE TO EVALUATE STEP>
C  $POP
C  STEP_MLOC
C
C  THEN THE FINAL TRIP COUNT IS CALCULATED BY THE FORMULA:
C  FTC = MAX( INT ( (FINAL-INIT+STEP)/STEP ) , 0 )
C
C  $PUSH
C  FINAL_MLOC
C  $PUSH
C  INIT_MLOC
C  $-
C  $PUSH
C  STEP_MLOC
C  $+
C  $PUSH
C  STEP_MLOC
C  $/
C  $TRUNC
C  $PUSHC
C  0
C  $MAX
C  2
C  $POP
C  FTC_MLOC
C
C  <VAR> IS THEN INITIALIZED TO THE VALUE OF INIT.
C
C  $PUSH
C  INIT_MLOC
C  $POP
C  VAR_MLOC
C
C  WHERE VAR_MLOC IS THE MEMORY LOCATION ASSIGNED TO <VAR>.
C  THE TRIP COUNTER IS THEN INITIALIZED TO 1.
C
C  $PUSHC
C  1
C  $POP
C  CTC_MLOC
C
C  A CHECK WILL BE MADE IF THE TRIP COUNTER HAS REACHED THE FINAL
C  TRIP COUNT.
C
C  $PUSH
C  CTC_MLOC
C  $PUSH
C  FTC_MLOC
C  $>
C  $TESTMOVE
C  <BLANK LINE>
C
C  THE TARGET OF THE $TESTMOVE INSTRUCTION IS THE ENDLOOP INSTRUCTION,
C  TO BE FOUND LATER ON.
C  A VALUE OF 4 IS PUSHED  IN THE TYPE TABLE, INDICATING THAT A LOOP
C  STATEMENT HAS BEEN REACHED. THE LOCATION OF THE ABOVE <BLANK LINE>
C  IS ALSO PUSHED IN THE TYPE-DEFINITION TABLE.
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C    NUM        REAL ARRAY. THE NUMBERS ENCOUNTERED IN THE INPUT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C               THE INPUT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C    LLOC       INTEGER ARRAY. THIS IS THE LOOP TABLE.
C    VAR        CHARACTER ARRAY. THIS IS THE VAR TABLE.
C    LOC        INTEGER ARRAY. THIS IS THE MEMORY LOCATION TABLE.
C    DIM        INTEGER ARRAY. THIS IS THE DIMENSIONALITY TABLE.
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    IPOS       INTEGER ARRAY. THIS IS THE POSITION TABLE.
C    KIND       INTEGER ARRAY. THIS IS THE KIND TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES ERROR, OVER, LOCAN, INSERT, EXTRA, XPR.
C
C  ABORT CONDITIONS THAT MAY OCCUR:
C    IF EITHER THE TYPE, OR THE LOOP TABLE OVERFLOWS, THE PROGRAM
C    WILL STOP AND ISSUE AN INFORMATIVE MESSAGE.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER (  IDLEN = 30   )
C
C  THE LENGTH OF A LINE. (CHARS)
      PARAMETER (  LINLEN = 80  )
C
      INTEGER TOKEN(*),LOC(*),DIM(*),INFO(*),IPOS(*),KIND(*),TDEF(*),
     >        LLOC(*),TYPE(*)
      CHARACTER*(*) ALPHA(*),VAR(*)
      REAL NUM(*)
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      INTEGER EX(LINLEN)
      LOGICAL FOUND,CORRE
      CHARACTER*(IDLEN) NAME,STARG(1)
C
C  INITIALIZE POINTERS TO THE TOKEN, ALPHA AND NUM ARRAYS RESPECTIVELY.
C
      NT=1
      NA=1
      NN=0
C
C  AN IDENTIFIER IS EXPECTED AS THE LOOP CONTROL INDEX.
C
      NT=NT+1
      IF (TOKEN(NT).NE.IDENT) THEN
          CALL ERROR(133,' ')
          RETURN
      END IF
C  NLT IS THE NUMBER OF NESTED LOOPS CURRENTLY IN EFFECT.
      NLT=NLOO+1
C  IF THE LOOP TABLE OVERFLOWS.
      IF (NLT.GT.MAXLOO)
     >           CALL OVER('LOOP','LLOC','LOOP',MAXLOO,'MXLOO')
C  GET THE LOOP CONTROL INDEX NAME.
      NA=NA+1
      NAME=ALPHA(NA)
C  CHECK IF IN THE VAR TABLE.
      CALL LOCAN(VAR,NVAR,NAME,FOUND,IND)
C
      IF (FOUND) THEN
C  ITS KIND SHOULD BE 1 (SIMPLE VARIABLE) OR -1 (UNDECLARED VARIABLE)
         K=KIND(IND)
         IF (K.NE.1 .AND. K.NE.-1) THEN
            CALL ERROR(134,' ')
C  L IS THE MEMORY LOCATION OF THE LOOP CONTROL INDEX. SINCE THE
C  KIND IS INCORRECT, ASSIGN A FALSE LOCATION TO IT, DIFFERENT
C  HOWEVER FROM ALL LOOPS NESTED OUTSIDE OR POSSIBLY INSIDE THIS ONE.
            L=MEMLOC+NLT
         ELSE
C  NOW L IS THE CORRECT MEMORY LOCATION OF THE LOOP CONTROL INDEX.
            L=LOC(IND)
         END IF
      ELSE
C  THE LOOP CONTROL INDEX WAS NOT FOUND IN THE VAR TABLE.
         CALL ERROR(134,' ')
         L=MEMLOC+NLT
C  INSERT IT IN THE TABLE AS A SIMPLE VARIABLE. (KIND=1, NOT -1)
         CALL INSERT(VAR,DIM,LOC,IPOS,KIND,NAME,0,L,0,1,IND)
      END IF
C  PUT THE LOCATION OF THE LOOP CONTROL INDEX IN THE LOOP TABLE.
      LLOC(NLT)=L
C
C  FIND OUT WHETHER THIS LOOP CONTROL INDEX HAS BEEN USED AS CONTROL
C  INDEX IN A LOOP NESTED OUTSIDE THIS ONE.
C
      DO 99,I=1,NLT-1
        IF (LLOC(I).EQ.L) THEN
C  THIS LOOP CONTROL INDEX HAS BEEN USED AS CONTROL INDEX IN
C  A LOOP NESTED OUTSIDE THIS ONE.
           CALL ERROR(135,' ')
           GOTO 40
        END IF
 99   CONTINUE
C
 40   CONTINUE
C
C  LMEM IN THE MEMORY LOCATION WHERE THE 5 STORAGE POSITIONS FOR
C  THIS LOOP START.
C  ASSIGN THE APPROPRIATE LOCATION TO LMEM. ALSO DETERMINE LOCATIONS
C  FOR THE NECESSARY LOOP INFORMATION.
C
      LMEM  = MEMLOC+FMEM+1+5*NLOO
      INIT  = LMEM
      IFIN  = LMEM+1
      ISTEP = LMEM+2
      ITC   = LMEM+3
      ITCF  = LMEM+4
C
C  AN IDENTIFIER ( "FROM" ) IS EXPECTED AFTER THE LOOP CONTROL INDEX.
C
      NT=NT+1
      IF (TOKEN(NT).NE.IDENT) THEN
         CALL ERROR(136,' ')
         RETURN
      END IF
C  "FROM" IS EXPECTED AFTER THE LOOP CONTROL INDEX.
      NA=NA+1
      IF (ALPHA(NA).NE.'FROM') THEN
          CALL ERROR(136,ALPHA(NA))
          RETURN
      END IF
C
C  CALCULATE INITIAL LOOP VALUE.
C
      CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,1,ALPHA,NA)
      CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,BLOC,
     >         LOC,DIM,INFO,IPOS,KIND,SCR,STARG)
      IF (.NOT.CORRE) RETURN
C  POP THE VALUE FROM THE STACK IN THE APPROPRIATE MEMORY LOCATION.
      BLOC=BLOC+2
      IF (GENE) THEN
         WRITE(SCR,'(A)') '$POP'
         WRITE(SCR,*) INIT
      END IF
C
C  AN IDENTIFIER ( "TO" ) IS EXPECTED.
C
      NT=NT+1
      IF (TOKEN(NT).NE.IDENT) THEN
         CALL ERROR(137,' ')
         RETURN
      END IF
C  TO IS EXPECTED AFTER THE EXPRESSION FOR THE INITIAL LOOP VALUE.
      NA=NA+1
      IF (ALPHA(NA).NE.'TO') THEN
         CALL ERROR(137,ALPHA(NA))
         RETURN
      END IF
C
C  CALCULATE THE FINAL LOOP VALUE.
C
      CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,1,ALPHA,NA)
      CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,BLOC,
     >         LOC,DIM,INFO,IPOS,KIND,SCR,STARG)
      IF (.NOT.CORRE) RETURN
C  POP THE FINAL LOOP VALUE TO THE APPROPRIATE MEMORY LOCATION.
      BLOC=BLOC+2
      IF (GENE) THEN
         WRITE(SCR,'(A)') '$POP'
         WRITE(SCR,*) IFIN
      END IF
C
C  CHECK IF A STEP IS DEFINED.
C
      NT=NT+1
      IF (NT.NE.NTOK) THEN
C  THE STATEMENT HAS NOT BEEN COMPLETED YET.
C  SO A STEP VALUE MAY BE DEFINED.
         IF (TOKEN(NT).NE.IDENT) THEN
C  AN IDENTIFIER ( "BY" ) DOES NOT FOLLOW.
            CALL ERROR(138,' ')
            RETURN
         END IF
         NA=NA+1
         IF (ALPHA(NA).NE.'BY') THEN
C  AN IDENTIFIER FOLLOWS BUT IS NOT "BY".
            CALL ERROR(138,ALPHA(NA))
            RETURN
         END IF
C
C  EVERYTHING IN ORDER. NOW CALCULATE STEP VALUE.
C
         CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,1,ALPHA,NA)
         CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,BLOC,
     >            LOC,DIM,INFO,IPOS,KIND,SCR,STARG)
         IF (.NOT.CORRE) RETURN
C  POP THE STEP VALUE TO THE APPROPRIATE MEMORY LOCATION.
         BLOC=BLOC+2
         IF (GENE) THEN
            WRITE(SCR,'(A)') '$POP'
            WRITE(SCR,*) ISTEP
         END IF
      ELSE
C
C  A STEP IS NOT DEFINED. USE A DEFAULT OF 1.
C
         BLOC=BLOC+4
         IF (GENE) THEN
            WRITE(SCR,'(A/I2/A)') '$PUSHC',1,'$POP'
            WRITE(SCR,*) ISTEP
         END IF
      END IF
C
C  LOOP STATEMENT CORRECT.
C
C     PROCEEDE WITH CODE GENERATION.
C
      BLOC=BLOC+33
      IF (GENE) THEN
C  CALCULATE FINAL TRIP COUNT.
         WRITE(SCR,'(A)') '$PUSH'
         WRITE(SCR,*) IFIN
         WRITE(SCR,'(A)') '$PUSH'
         WRITE(SCR,*) INIT
         WRITE(SCR,'(A/A)') '$-','$PUSH'
         WRITE(SCR,*) ISTEP
         WRITE(SCR,'(A/A)') '$+','$PUSH'
         WRITE(SCR,*) ISTEP
         WRITE(SCR,'(A/A/A/I2)')'$/','$TRUNC','$PUSHC',0
         WRITE(SCR,'(A/I2/A)') '$MAX',2,'$POP'
         WRITE(SCR,*) ITCF
C  INITIALIZE THE LOOP CONTROL INDEX.
         WRITE(SCR,'(A)') '$PUSH'
         WRITE(SCR,*) INIT
         WRITE(SCR,'(A)') '$POP'
         WRITE(SCR,*) L
C  INITIALIZE THE TRIP COUNTER.
         WRITE(SCR,'(A/I2/A)') '$PUSHC',1,'$POP'
         WRITE(SCR,*) ITC
C  CHECK IF THE TRIP COUNTER HAS REACHED ITS TARGET VALUE.
         WRITE(SCR,'(A)') '$PUSH'
         WRITE(SCR,*) ITC
         WRITE(SCR,'(A)') '$PUSH'
         WRITE(SCR,*) ITCF
         WRITE(SCR,'(A/A/)') '$>','$TESTMOVE'
      END IF
C
      NLOO=NLT
C  MAXL IS THE MAXIMUM NUMBER OF NESTED LOOPS FOUND SO FAR.
      MAXL=MAX(NLOO,MAXL)
C  PUSH A VALUE OF 4 IN THE TYPE TABLE TO INDICATE THE LOOP STATEMENT.
C  ALSO PUSH THE LOCATION OF THE BLANK LINE JUST WRITTEN TO THE SCRATCH
C  FILE ( FOLLOWING THE $TESTMOVE INSTRUCTION ) IN THE
C  TYPE-DEFINITION TABLE.
      NTYPE=NTYPE+1
      IF (NTYPE.GT.MAXTYP)
     >           CALL OVER('TYPE','TYPE','LOOP',MAXTYP,'MXTYP')
      TYPE(NTYPE) = 4
      TDEF(NTYPE) = BLOC
C
      RETURN
      END
CLXTRAN-----------------------------------------------------------------
      INTEGER FUNCTION LXTRAN ( CHAR )
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: TRANSFORMS CHARACTERS TO SPECIFIC
C                  CODES TO BE USED IN ROUTINE LEX.
C
C   %..   ARGUMENT INFORMATION:
C
C      -LXTRAN = THE VALUE RETURNED
C      -CHAR  =  THE CHARACTER FOR WHICH CODE IS ASKED.
C
C   %..   SUBPROGRAMS CALLED: ABORT ROUTINE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M): NONE.
C
C   %..   POSSIBLE ABORTS:
C
C         THE CHARACTER CHAR INSERTED MAY BE WRONG IN THE
C         THE GENERAL CASE. NORMALLY THIS IS IMPOSSIBLE.
C
C   %..   NOTES:
C
C        THIS FUNCTION IS USED IN SPECIAL CHARACTER PROCESSING
C        INTO ROUTINE LEX.
C
C-----------------------------------------------------------------------
C
      CHARACTER*(*) CHAR
C
      IF(CHAR.EQ.'(')THEN
         LXTRAN=1
      ELSE IF(CHAR.EQ.')')THEN
         LXTRAN=2
      ELSE IF(CHAR.EQ.'.')THEN
         LXTRAN=3
      ELSE IF(CHAR.EQ.',')THEN
         LXTRAN=4
      ELSE IF(CHAR.EQ.'#')THEN
         LXTRAN=7
      ELSE IF(CHAR.EQ.'=')THEN
         LXTRAN=8
      ELSE IF(CHAR.EQ.'+')THEN
         LXTRAN=11
      ELSE IF(CHAR.EQ.'-')THEN
         LXTRAN=12
      ELSE IF(CHAR.EQ.'/')THEN
         LXTRAN=14
      ELSE IF(CHAR.EQ.'[')THEN
         LXTRAN=23
      ELSE IF(CHAR.EQ.']')THEN
         LXTRAN=24
      ELSE IF(CHAR.EQ.';')THEN
         LXTRAN=25
      ELSE
         CALL ABORT ('ILLEGAL CHARACTER GIVEN','LXTRAN')
      END IF
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE MARGI ( TOKEN,NTOK,NT,ALPHA,NA,NUM,NN,VAR,LOC,DIM,
     >                   INFO,IPOS,KIND )
C  ---------------------------------------------------------------------
C
C  PARSES THE MARGIN STATEMENT AND GENERATES APPROPRIATE OBJECT CODE.
C  THE STATEMENT HAS THE FORM:
C
C  MARGIN ( <K1>.EXPR1=VAL1; <K2>.EXPR2=VAL2;...<KN>=EXPRN=VALN )
C
C  WHERE <KN> IS A KEYWORD. ITS VALUE IS EITHER L ( FOR A LEFT MARGIN )
C  OR R. ( FOR A RIGHT MARGIN )
C  EXPR1,EXPR2,...EXPRN ARE EXPRESSIONS.
C  VAL1,VAL2,...VALN ARE ALSO EXPRESSIONS. THEIR VALUES WILL BE ASSIGNED
C  TO THE MARGIN VARIABLES INDEXED BY EXPR1,EXPR2,...EXPRN
C  CORRESPONDINGLY. THE OBJECT CODE GENERATED IS AS:
C
C  <CODE TO EVALUATE EXPR FOR THE 1-ST R KEYWORD>
C  <CODE TO EVALUATE VAL FOR THE 1-ST R KEYWORD>
C  <CODE TO EVALUATE EXPR FOR THE 2-ND R KEYWORD>
C  <CODE TO EVALUATE VAL FOR THE 2-ND R KEYWORD>
C  ...
C  <CODE TO EVALUATE EXPR FOR THE N-TH R KEYWORD>
C  <CODE TO EVALUATE VAL FOR THE N-TH R KEYWORD>
C  <CODE TO EVALUATE EXPR FOR THE 1-ST L KEYWORD>
C  <CODE TO EVALUATE VAL FOR THE 1-ST L KEYWORD>
C  <CODE TO EVALUATE EXPR FOR THE 2-ND L KEYWORD>
C  <CODE TO EVALUATE VAL FOR THE 2-ND L KEYWORD>
C  ...
C  <CODE TO EVALUATE EXPR FOR THE N-TH L KEYWORD>
C  <CODE TO EVALUATE VAL FOR THE N-TH L KEYWORD>
C  $WMARG
C  <NL> <NR>
C  $SWAP
C
C  WHERE <CODE TO EVALUATE EXPR FOR THE N-THE R KEYWORD> IS THE OBJECT
C  CODE NECESSARY TO EVALUATE EXPR FOR THE N-TH OCCURRENCE OF AN R
C  KEYWORD IN THE STATEMENT.
C  <NL> IS THE TOTAL NUMBER OF "<K>.EXPR=VAL" ENTRIES IN THE STATEMENT
C  USING THE L KEYWORD AND <NR> IS THE CORRESPONDING NUMBER FOR THE
C  R KEYWORD.
C  SINCE THE L AND R KEYWORDS MAY BE APPEAR IN ANY ORDER IN THE
C  STATEMENT, BUT THE CODE FOR EACH ONE MUST BE SEPARATE, AS SHOWN
C  ABOVE, THE FOLLOWING METHOD IS USED:
C  WHEN AN R KEYWORD IS ENCOUNTERED IN THE STATEMENT, THE CODE THAT
C  EVALUATES EXPR AND VAL FOR THIS KEYWORD IS WRITTEN NORMALLY IN THE
C  SCRATCH FILE. HOWEVER WHEN AN L KEYWORD IS REACHED, THE OBJECT CODE
C  THAT EVALUATES EXPR AND VAL IS WRITTEN TO THE SPARE FILE. WHEN ALL
C  KEYWORDS HAVE BEEN PROCESSED AND NO ERROR HAS BEEN ENCOUNTERED,
C  THE SPARE FILE IS COPIED ON THE SCRATCH FILE.
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C               THE INPUT LINE.
C    NUM        REAL ARRAY. THE NUMBERS ENCOUNTERED IN THE INPUT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C               THE INPUT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    NT         INTEGER VARIABLE. THIS IS A POINTER TO THE TOKEN ARRAY
C               AND IS THE NUMBER OF TOKENS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    NA         INTEGER VARIABLE. THIS IS A POINTER TO THE ALPHA ARRAY
C               AND IS THE NUMBER OF IDENTIFIERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    NN         INTEGER VARIABLE. THIS IS A POINTER TO THE NUM ARRAY
C               AND IS THE NUMBER OF NUMBERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    VAR        CHARACTER ARRAY. THIS IS THE VAR TABLE.
C    LOC        INTEGER ARRAY. THIS IS THE MEMORY LOCATION TABLE.
C    DIM        INTEGER ARRAY. THIS IS THE DIMENSIONALITY TABLE.
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    IPOS       INTEGER ARRAY. THIS IS THE POSITION TABLE.
C    KIND       INTEGER ARRAY. THIS IS THE KIND TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES ERROR, EXTRA, XPR, ABORT.
C
C  ABORT CONDITIONS THAT MAY OCCUR:
C    IN CASE AN ERROR OCCURS WHILE OPENING OR CLOSING THE SPARE FILE,
C    THE PROGRAM WILL STOP AND ISSUE AN INFORMATIVE MESSAGE.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER (  IDLEN = 30   )
C
C  THE LENGTH OF A LINE. (CHARS)
      PARAMETER (  LINLEN = 80  )
C
      INTEGER TOKEN(*),LOC(*),DIM(*),INFO(*),IPOS(*),KIND(*)
      CHARACTER*(*) ALPHA(*),VAR(*)
      REAL NUM(*)
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      CHARACTER*(IDLEN) KEY,LINE,STARG(1)
      LOGICAL CORRE
      INTEGER EX(LINLEN)
C
C  A LEFT PARENTHESIS SHOULD FOLLOW "MARGIN".
C
      NT=NT+1
      IF (TOKEN(NT).NE.LEFTP) THEN
         CALL ERROR(70,' ')
         RETURN
      END IF
C  OPEN THE SPARE FILE.
      OPEN(SPARE,STATUS='SCRATCH',ERR=1751)
      REWIND SPARE
C  NLE IS THE NUMBER OF L KEYWORDS IN THE STATEMENT.
      NLE=0
C  NRI IS THE NUMBER OF R KEYWORDS IN THE STATEMENT.
      NRI=0
C  LI IS THE NUMBER OF OBJECT CODE LINES WRITTEN TO THE SPARE FILE.
      LI=0
C
C  START A LOOP OVER ALL KEYWORDS ENTERED IN THE STATEMENT.
C
 1000 CONTINUE
C  AN IDENTIFIER IS EXPECTED AS A KEYWORD.
      NT=NT+1
      IF (TOKEN(NT).NE.IDENT) THEN
         CALL ERROR(32,' ')
         CLOSE(SPARE,ERR=1752)
         RETURN
      END IF
C  GET THE KEYWORD.
      NA=NA+1
      KEY=ALPHA(NA)
C  THE KEYWORD SHOULD BE EITHER R OR L.
      IF (KEY.NE.'L' .AND. KEY.NE.'R') THEN
         CALL ERROR(33,' ')
         CLOSE(SPARE,ERR=1752)
         RETURN
      END IF
C
      IF (KEY.EQ.'L') THEN
C  THIS AN ENTRY FOR A LEFT MARGIN.
         NLE=NLE+1
         IUN=SPARE
         ICT=LI
      ELSE
C  THIS IS AN ENTRY FOR A RIGHT MARGIN.
         NRI=NRI+1
         IUN=SCR
         ICT=BLOC
      END IF
C  A PERIOD SHOULD FOLLOW THE KEYWORD.
      NT=NT+1
      IF (TOKEN(NT).NE.PERIOD) THEN
         CALL ERROR(34,' ')
         CLOSE(SPARE,ERR=1752)
         RETURN
      END IF
C  EXTRACT THE EXPRESSION FOLLOWING THE KEYWORD.
      CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,2,ALPHA,NA)
C  EVALUATE THE EXPRESSION. IUN IS THE UNIT IN WHICH THE OBJECT CODE
C  FOR THE EXPRESSION WILL BE WRITTEN, AND ICT IS ITS LINE COUNTER,
C  BOTH PREVIOUSLY SET.
      CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,ICT,LOC,
     >         DIM,INFO,IPOS,KIND,IUN,STARG)
C  IF THE EXPRESSION IS INCORRECT.
      IF (.NOT.CORRE) THEN
         CLOSE(SPARE,ERR=1752)
         RETURN
      END IF
C  AN = SIGN IS EXPECTED AFTER THE EXPRESSION.
      NT=NT+1
      IF (TOKEN(NT).NE.EQ) THEN
         CALL ERROR(35,' ')
         CLOSE(SPARE,ERR=1752)
         RETURN
      END IF
C  EXTRACT THE EXPRESSION FOLLOWING THE = SIGN.
      CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,2,ALPHA,NA)
C  EVALUATE THE EXPRESSION. OBJECT CODE WILL BE WRITTEN IN UNIT IUN,
C  WHILE ICT IS THE LINE COUNTER OF THIS UNIT.
      CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,ICT,LOC,
     >         DIM,INFO,IPOS,KIND,IUN,STARG)
C  IF THE EXPRESSION IS INCORRECT.
      IF (.NOT.CORRE) THEN
         CLOSE(SPARE,ERR=1752)
         RETURN
      END IF
C  RESTORE TO THE NORMAL VARIABLES THE ICT LINE COUNTER.
      IF (KEY.EQ.'L') THEN
         LI=ICT
      ELSE
         BLOC=ICT
      END IF
C  GET THE NEXT TOKEN.
      NT=NT+1
      CUR=TOKEN(NT)
C  A SEMICOLON DENOTES ANOTHER KEYWORD.
      IF (CUR.EQ.SEMI) GOTO 1000
      IF (CUR.EQ.RIGHTP) THEN
C  IF A RIGHT PARENTHESIS FOLLOWS, THE STATEMENT HAS BEEN COMPLETED.
C  NOW COPY THE SPARE FILE TO THE SCRATCH FILE.
         BLOC=BLOC+5+LI
         IF (GENE) THEN
            WRITE(SCR,'(''$PUSHC''/I2)') NRI
            REWIND SPARE
            DO 10,I=1,LI
              READ(SPARE,'(A)') LINE
              WRITE(SCR,'(A)') LINE
 10         CONTINUE
C  WRITE THE FINAL OBJECT CODE INSTRUCTIONS.
            WRITE(SCR,'(''$PUSHC''/I2/''MARGIN'')') NLE
         END IF
         CLOSE(SPARE,ERR=1752)
         RETURN
      ELSE
         CALL ERROR(36,' ')
      END IF
C
      CLOSE(SPARE,ERR=1752)
      RETURN
C
C  IN CASE THE SPARE FILE OPENING FAILS.
C
 1751 CALL ABORT('SPARE FILE OPENING FAILED.','MARGI')
C
C  IN CASE THE SPARE FILE CLOSING FAILS.
C
 1752 CALL ABORT('SPARE FILE CLOSING FAILED.','MARGI')
      END
C  ---------------------------------------------------------------------
      SUBROUTINE MCL
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF A LINE. (CHARS)
      PARAMETER (  LINLEN = 80  )
C
C  MAXIMUM NUMBER OF LABELS IN A PROGRAM.
      PARAMETER (  MXLAB = 100  )
C
C  MAXIMUM NUMBER OF SIMPLE VARIABLES, ARRAYS AND FUNCTIONS.
C  (ALL TOGETHER WITH MERLIN VARIABLES,
C   MERLIN ARRAYS AND INTRINSIC FUNCTIONS).
      PARAMETER (  MXVAR = 500  )
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER (  IDLEN = 30   )
C
C  MAXIMUM NUMBER OF "MOVE TO" STATEMENTS.
      PARAMETER (  MXMOV = 100  )
C
C  MAXIMUM NESTING LEVEL FOR THE LOOP STRUCTURE.
      PARAMETER (  MXLOO = 100  )
C
C  TOTAL NUMBER OF INTRINSIC FUNCTIONS AND MERLIN VARIABLES AND ARRAYS.
      PARAMETER (  NPRE = 45  )
C
C  LENGTH OF THE INFO ARRAY. (HOLDING INFO. ABOUT EACH VARIABLE,
C                             ARRAY OR FUNCTION)
      PARAMETER (  MXINFO = 200  )
C
C  LENGTH OF THE TYPE ARRAY. (INVOLVED WHEN USING THE: IF, ELSE, ENDIF,
C                             LOOP, ENDLOOP, WHEN, MOVE TO AND EXIT
C                             STATEMENTS)
      PARAMETER (  MXTYP = 500 )
C
C     THE FOLLOWING PARAMETERS ARE CALCULATED FROM THE ONES JUST STATED.
C
      PARAMETER (  MXTOK = LINLEN+1  )
C
C  MAXIMUM NUMBER OF IDENTIFIERS IN A LINE.
      PARAMETER (  MXID = (LINLEN+1)/2 )
C
C  MAXIMUM NUMBER OF NUMBERS IN A LINE.
      PARAMETER (  MXNUM = (LINLEN+1)/2 )
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      LOGICAL EOPEN
      COMMON /ISOPEN/ EOPEN
C
      INTEGER LABLOC(MXLAB),TYPE(MXTYP),TOKEN(MXTOK),INFO(MXINFO),
     >        TDEF(MXTYP),DIM(MXVAR),LOC(MXVAR),LLOC(MXLOO),
     >        IPOS(MXVAR),KIND(MXVAR),TREL(MXTYP)
      REAL NUM(MXNUM)
      CHARACTER*(IDLEN) LABEL(MXLAB),VAR(MXVAR),MVELAB(MXMOV),
     >                  ALPHA(MXID),LABID
C
      LOGICAL FOUND
      CHARACTER*(LINLEN) LINE
      CHARACTER S
C
      DATA (VAR(I),DIM(I),KIND(I),I=1,10) /
     >'ABS                           ', 1, 6,
     >'ACOS                          ', 1, 6,
     >'ACOSH                         ', 1, 6,
     >'ASIN                          ', 1, 6,
     >'ASINH                         ', 1, 6,
     >'ATAN                          ', 1, 6,
     >'ATANH                         ', 1, 6,
     >'BACKUP                        ', 0, 4,
     >'CALLBY                        ', 0, 4,
     >'COS                           ', 1, 6 /
      DATA (VAR(I),DIM(I),KIND(I),I=11,20) /
     >'COSH                          ', 1, 6,
     >'DERIVA                        ', 0, 4,
     >'DIM                           ', 0, 4,
     >'EXP                           ', 1, 6,
     >'FACT                          ', 1, 6,
     >'FIX                           ', 1, 5,
     >'GNORM                         ', 1, 6,
     >'GRAD                          ', 1, 5,
     >'L                             ', 1, 5,
     >'LOG                           ', 1, 6 /
      DATA (VAR(I),DIM(I),KIND(I),I=21,30) /
     >'LOG10                         ', 1, 6,
     >'MARG                          ', 1, 5,
     >'MAX                           ', 0, 6,
     >'MEAN                          ', 0, 6,
     >'MIN                           ', 0, 6,
     >'MOD                           ', 2, 6,
     >'PANEL                         ', 0, 4,
     >'PCOUNT                        ', 0, 4,
     >'PRECISION                     ', 0, 4,
     >'PRINTO                        ', 0, 4 /
      DATA (VAR(I),DIM(I),KIND(I),I=31,40) /
     >'PROCES                        ', 0, 4,
     >'R                             ', 1, 5,
     >'RAN                           ', 1, 6,
     >'ROUND                         ', 1, 6,
     >'SIN                           ', 1, 6,
     >'SINH                          ', 1, 6,
     >'SQRT                          ', 1, 6,
     >'STEP                          ', 1, 5,
     >'TAN                           ', 1, 6,
     >'TANH                          ', 1, 6 /
      DATA (VAR(I),DIM(I),KIND(I),I=41,NPRE) /
     >'TCOUNT                        ', 0, 4,
     >'TRUNC                         ', 1, 6,
     >'USERSO                        ', 0, 4,
     >'VALUE                         ', 0, 4,
     >'X                             ', 1, 5 /
      DATA (LOC(I),I=1,NPRE)  / NPRE*0 /
      DATA (IPOS(I),I=1,NPRE) / NPRE*0 /
      DATA (INFO(I),I=1,NPRE) / NPRE*0 /
C
C  THE TOTAL NUMBER OF ERRORS ENCOUNTERED IN THE MCL SOURCE PROGRAM.
      DATA TOTAL / 0 /
C
C  REWIND NECESSARY FILES.
C
      REWIND INP
      IF (EOPEN) REWIND ERL
      REWIND SCR
      REWIND STF
C
C  INITIALIZE VARIABLES IN THE MCLCNT COMMON BLOCK.
C
      NLAB=0
      MAXLAB=MXLAB
      NVAR=NPRE
      MAXVAR=MXVAR
      NTYPE=0
      MAXTYP=MXTYP
      NMOVE=0
      MAXMOV=MXMOV
      NINF=NPRE
      MAXIN=MXINFO
      NSTL=0
      FMEM=0
      MAXL=0
      NLOO=0
      MAXLOO=MXLOO
      NERR=0
C
C  START COMPILING THE MCL PROGRAM.
C
C  TRY TO GET LINES CONTAINING A "VAR" STATEMENT.
C
 10   READ(INP,'(A)',END=20) LINE
      NL=NL+1
C  ANALYSE THE INPUT LINE IN A SEQUENCE OF TOKENS.
      CALL LEX(LINE,TOKEN,NTOK,NUM,ALPHA)
C  CHECK IF THIS IS A COMMENT LINE, OR A BLANK LINE.
      INIT=TOKEN(1)
      IF (INIT.EQ.GT .OR. INIT.EQ.EOL) GOTO 10
C  IF THIS IS NOT A "VAR" DECLARATION.
      IF (ALPHA(1).NE.'VAR') GOTO 47
      CALL VARIA(TOKEN,NTOK,ALPHA,VAR,DIM,LOC,IPOS,INFO,KIND,NUM)
C  IF ANY ERROR OCCURRED IN THIS LINE, FLUSH THE ERROR BUFFER.
      IF (NERR.NE.0) CALL FEBUFF(LINE,TOTAL)
      GOTO 10
C
C  TRY TO GET LINES CONTAINING A "FUNCTION" DECLARATION.
C
 37   READ(INP,'(A)',END=20) LINE
      NL=NL+1
C  ANALYSE THE INPUT LINE IN A SEQUENCE OF TOKENS.
      CALL LEX(LINE,TOKEN,NTOK,NUM,ALPHA)
C  CHECK IF THIS IS A COMMENT LINE, OR A BLANK LINE.
      INIT=TOKEN(1)
      IF (INIT.EQ.GT .OR. INIT.EQ.EOL) GOTO 37
C  IF THIS IS NOT A "FUNCTION" DECLARATION.
 47   IF (ALPHA(1).NE.'FUNCTION') GOTO 30
      CALL STAFU(TOKEN,NTOK,ALPHA,VAR,DIM,LOC,IPOS,INFO,KIND,NUM)
C  IF ANY ERROR OCCURRED IN THIS LINE, FLUSH THE ERROR BUFFER.
      IF (NERR.NE.0) CALL FEBUFF(LINE,TOTAL)
      GOTO 37
C
C  "VAR" AND "FUNCTION" STATEMENTS PROCESSED. NOW COMPILE
C  THE REST OF THE PROGRAM.
C
 60   READ(INP,'(A)',END=70) LINE
      NL=NL+1
C  ANALYSE THE INPUT LINE IN A SEQUENCE OF TOKENS.
      CALL LEX(LINE,TOKEN,NTOK,NUM,ALPHA)
C  CHECK IF THIS IS A COMMENT LINE, OR A BLANK LINE.
      INIT=TOKEN(1)
      IF (INIT.EQ.GT .OR. INIT.EQ.EOL) GOTO 60
C
 30   CONTINUE
C  IF THE DEBUG OPTION HAS BEEN SELECTED, WRITE DEBUG INFORMATION.
      IF (DEBUG .AND. GENE) THEN
         BLOC=BLOC+3
         WRITE(SCR,'(A)') '$LINE'
         WRITE(SCR,*)NL
         WRITE(SCR,'(''$>>'',T11,A)') LINE
      END IF
C
C  PCG IS THE MAIN ROUTINE THAT WILL DETERMINE WHAT KIND OF STATEMENT
C  IS IT ABOUT, AND WILL CALL APPROPRIATE PROGRAM UNITS TO PROCESS IT.
C
      CALL PCG(TOKEN,NTOK,NUM,ALPHA,LABEL,LABLOC,TYPE,TDEF,LLOC,VAR,
     >         MVELAB,LOC,DIM,INFO,IPOS,KIND,TREL)
C  IF ANY ERROR OCCURRED IN THIS LINE, FLUSH THE ERROR BUFFER.
      IF (NERR.NE.0) CALL FEBUFF(LINE,TOTAL)
      GOTO 60
C
 70   CONTINUE
C  IF ANY ERROR, FLUSH THE ERROR BUFFER.
      IF (NERR.NE.0) CALL FEBUFF(LINE,TOTAL)
C
C  THE FIRST PASS OVER THE MCL SOURCE HAS BEEN COMPLETED.
C
C  LINK JUMPS DUE TO "MOVE TO" STATEMENTS.
C
      K=0
      DO 700,I=1,NMOVE
C  TRY TO LOCATE A TYPE OF -7 (A "MOVE TO") IN THE TYPE ARRAY.
 600    K=K+1
        IF (TYPE(K).NE.-7) GOTO 600
C  A TYPE OF -7 WAS FOUND IN POSITION K OF THE TYPE ARRAY.
C  GET THE DESTINATION LABEL OF THIS "MOVE TO" AND CHECK
C  WHETHER IT IS DEFINED.
        LABID=MVELAB(I)
        CALL LOCAN(LABEL,NLAB,LABID,FOUND,IND)
        IF (FOUND) THEN
C  CALCULATE THE TYPE-LINK TABLE ENTRY FOR THIS "MOVE TO".
           IR=LABLOC(IND)-TDEF(K)
           IF (IR.LT.0) IR=IR+1
           TREL(K)=IR
        ELSE
C  THE LABEL IS NOT DEFINED.
           CALL ERROR(11,LABID)
C  PUT THE LABEL IN THE LABEL TABLE TO AVOID FURTHER ERROR MESSAGES.
C  ... ONE IS ENOUGH!  ( THE LOCATION ASSIGNED TO THIS LABEL IS: 0 )
           NLAB=NLAB+1
           IF (NLAB.GT.MAXLAB)
     >              CALL OVER('LABEL','LABEL','MCL',MAXLAB,'MXLAB')
           DO 21,J=NLAB,IND+1,-1
             LABEL(J)=LABEL(J-1)
             LABLOC(J)=LABLOC(J-1)
 21        CONTINUE
           LABEL(IND)=LABID
           LABLOC(IND)=0
        END IF
 700  CONTINUE
C
C  CHECK FOR BLOCK IFS AND LOOPS THAT DO NOT TERMINATE.
C
      CALL TERMIN(TYPE,TDEF)
C
C  CHECK THE VALIDITY OF THE "MOVE TO" STATEMENTS IN RELATION WITH
C  BLOCK IFS AND LOOPS.
C
      CALL VALIJ(TYPE,TDEF,TREL)
C
C  IF ANY ERROR FLUSH THE ERROR BUFFER.
C
      IF (NERR.NE.0) CALL FEBUFF(' ',TOTAL)
C
C  IF EVERYTHING OK GENERATE THE FINAL OBJECT CODE.
C
      IF (GENE) THEN
         REWIND BIN
C  WRITE THE INITIAL $LENGTH INSTRUCTION. TOTAL MEMORY SPACE NEEDED
C  FOR THE RUN, INCLUDES STORAGE FOR FUNCTION ARGUMENTS, AND
C  LOOP TEMPORARY STORAGE.
         WRITE(BIN,'(A)') '$LENGTH'
         WRITE(BIN,*) MEMLOC+FMEM+5*MAXL
C
C  THE SECOND PASS IS PERFORMED NOW. THE SCRATCH FILE IS COPIED ON
C  THE FINAL OBJECT CODE FILE, WHILE APPROPRIATE NUMBERS ARE WRITTEN
C  WHERE NECESSARY.
C
         REWIND SCR
         IC=0
         DO 100,I=1,NTYPE
           IF (TYPE(I).EQ.3) GOTO 100
           M=TDEF(I)
           CALL COPY(IC,M)
           WRITE(BIN,*) TREL(I)
           IC=M
           READ(SCR,'(A)') S
 100     CONTINUE
         CALL COPY(IC,BLOC+1)
         WRITE(BIN,'(A)')'$FINISH'
C
      END IF
C
 20   IF (TOTAL.EQ.1) THEN
         IF (EOPEN) THEN
            WRITE (ERL,71) TOTAL
         ELSE
            WRITE (*,71) TOTAL
         END IF
      ELSE IF (TOTAL.GT.1) THEN
         IF (EOPEN) THEN
            WRITE (ERL,72) TOTAL
         ELSE
            WRITE (*,72) TOTAL
         END IF
      END IF
C
71    FORMAT (//T10,I6,' MCL ERROR.')
72    FORMAT (//T10,I6,' MCL ERRORS.')
      END
C  ---------------------------------------------------------------------
      PROGRAM MCLCOM
C  ---------------------------------------------------------------------
C
C  MCL COMPILER. VERSION 1.0 PLUS
C
      IMPLICIT INTEGER (A-Z)
C
      CHARACTER*20 IFILE,EFILE,BFILE
      COMMON /FILES/ IFILE,EFILE,BFILE
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      LOGICAL EOPEN
      COMMON /ISOPEN/ EOPEN
C
C  GET COMPILATION PARAMETERS.
      CALL GETPAR
C  OPEN NECESSARY FILES.
      OPEN(INP,FILE=IFILE,ERR=10)
      IF (EFILE.NE.' ') THEN
         EOPEN = .TRUE.
         OPEN(ERL,FILE=EFILE,ERR=20)
      END IF
      OPEN(BIN,FILE=BFILE,ERR=30)
      OPEN(SCR,STATUS='SCRATCH',ERR=40)
      OPEN(STF,STATUS='SCRATCH',ERR=50)
      OPEN(FHELP,STATUS='SCRATCH',ERR=60)
C  COMPILE THE PROGRAM.
      CALL MCL
C  CLOSE ANY FILES IF NECESSARY AND STOP.
      CLOSE(SCR,STATUS='DELETE',ERR=70)
      CLOSE(STF,STATUS='DELETE',ERR=80)
      CLOSE(FHELP,STATUS='DELETE',ERR=90)
      IF (GENE) THEN
         IF (EOPEN) CLOSE(ERL,STATUS='DELETE')
         STOP'MCLCOM COMPLETE.'
      ELSE
         CLOSE(BIN,STATUS='DELETE')
         IF (EOPEN) THEN
            STOP'MCLCOM COMPLETE - SEE ERROR FILE.'
         ELSE
            STOP'MCLCOM COMPLETE - ERRORS DETECTED.'
         END IF
      END IF
C  ERROR STOPS, WHILE OPENING THE NECESSARY FILES.
 10   STOP'INPUT FILE OPENING FAILED.'
 20   STOP'ERROR LIST FILE OPENING FAILED.'
 30   STOP'OBJECT CODE FILE OPENING FAILED.'
 40   STOP'SCRATCH FILE OPENING FAILED.'
 50   STOP'STATEMENT FUNCTION FILE OPENING FAILED.'
 60   STOP'STATEMENT FUNCTION HELP FILE OPENING FAILED.'
 70   STOP'SCRATCH FILE CLOSING FAILED.'
 80   STOP'STATEMENT FUNCTION FILE CLOSING FAILED.'
 90   STOP'STATEMENT FUNCTION HELP FILE CLOSING FAILED.'
C
      END
C  ---------------------------------------------------------------------
      BLOCK DATA MCLDEF
C  ---------------------------------------------------------------------
C
C  INITIALIZES SOME OF THE COMMON BLOCKS USED BY MCLCOM.
C  THE BLOCKS INITIALIZED ARE:
C
C  MCLUNT       THE I/O UNIT NUMBERS.
C  MCLSCD       THE LEXICAL ANALYSIS CODES.
C  MCLMFL       THE FILE NAMES USED BY MERLIN.
C  MCLDTA       VARIOUS COMPILATION PARAMETERS.
C  FILES        INPUT AND OUTPUT FILES OF THE COMPILER.
C
C  THE VALUES IN ALL OF THE ABOVE BLOCKS BUT MCLDTA
C  WILL NOT CHANGE DURING PROGRAM'S EXECUTION.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      CHARACTER*10 MFILE(7)
      COMMON /MCLMFL/ MFILE
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      CHARACTER*20 IFILE,EFILE,BFILE
      COMMON /FILES/ IFILE,EFILE,BFILE
C
      LOGICAL EOPEN
      COMMON /ISOPEN/ EOPEN
C
      DATA INP, ERL, BIN, SCR, STF, SPARE, FHELP, DESCRI /
     >      21,  22,  23,  24,  25,    26,    27,     28 /
C
      DATA LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,PLUS,MINUS,
     >     STAR,SLASH,BISTAR,COLON,EOL,IDENT,UNSI,NOTREC,STRING,APOST,
     >     LBRA,RBRA,SEMI,PERCE,AND,OR,XOR,NOT,EQUAL /
     >     1,    2,     3,     4,    5, 6, 7, 8, 9, 10,11,  12,
     >     13,  14,   15,    16,   17, 18,   19,   20,   21,    22,
     >     23,  24,  25,  26,   27, 28,29, 30, 31 /
C
      DATA MFILE / 'HELP      ','DATA      ','STORE     ','INIPO     ',
     >             'DISPO     ','BACKUP    ','MACROF    ' /
C
      DATA NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG /
     >      0,.TRUE.,0,0,.FALSE.,.FALSE. /
C
      DATA EFILE / '                    ' /
     >     BFILE / 'MOC                 ' /
C
      DATA EOPEN / .FALSE. /
C
      END
C  ---------------------------------------------------------------------
      FUNCTION MEMSPC ( INFO, IP, NDIM )
C  ---------------------------------------------------------------------
C
C  CALCULATES TOTAL MEMORY SPACE OCCUPIED BY AN MCL ARRAY.
C  THIS IS CALCULATED AS A PRODUCT OF THE SPACE REQUIRED FOR EACH ONE OF
C  THE ARRAY DIMENSIONS.
C
C  INPUT ARGUMENTS:
C
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    IP         INTEGER VARIABLE. INFORMATION FOR THE ARRAY STARTS
C               AT POSITION IP OF THE INFORMATION TABLE.
C    NDIM       DIMENSIONALITY OF THE ARRAY.
C
C  ---------------------------------------------------------------------
C
      INTEGER INFO(*)
      INTEGER HIGH
C
C  IND IS A POINTER TO THE INFORMATION TABLE.
C
      IND=IP-1
      MEMSPC=1
C
C  THIS LOOP WILL PROCESS ALL ARRAY DIMENSIONS.
C
      DO  10,I=1,NDIM
C  ADVANCE POINTER.
        IND=IND+1
C  GET LOWER ARRAY BOUND.
        LOW=INFO(IND)
C  ADVANCE POINTER.
        IND=IND+1
C  GET UPPER DIMENSION BOUND.
        HIGH=INFO(IND)
C  HIGH-LOW+1 IS THE SPACE OCCUPIED BY THIS DIMENSION OF THE ARRAY
        MEMSPC=MEMSPC*(HIGH-LOW+1)
 10   CONTINUE
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE MOVE ( TOKEN, NTOK, ALPHA, NT, NA, MVELAB, TYPE, TDEF )
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE PARSES THE "MOVE TO" STATEMENT AND GENERATES APPROPRIATE
C  OBJECT CODE. THE SYNTAX OF THIS STATEMENT IS:
C
C  "MOVE" "TO" <LABEL NAME>
C
C  WHERE <LABEL NAME> IS THE DESTINATION LABEL.
C  IF THE SYNTAX IS CORRECT, THE FOLLOWING TWO LINES OF OBJECT CODE
C  ARE GENERATED:
C
C  $MOVE
C  <BLANK LINE>
C
C  THE <LABEL NAME> IS PUSHED IN THE MOVE TABLE. A VALUE OF -7 IS PUSHED
C  IN THE TYPE TABLE, TO INDICATE THE "MOVE TO" STATEMENT, WHILE THE
C  LOCATION OF THE ABOVE <BLANK LINE> IS ALSO PUSHED IN THE
C  TYPE-DEFINITION TABLE. THE ACTUAL ARGUMENT FOR THE $MOVE INSTRUCTION
C  WILL BE CALCULATED IN PASS 2, AFTER ALL LABEL DEFINITIONS
C  HAVE BEEN READ.
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C               THE INPUT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    NT         INTEGER VARIABLE. THIS IS A POINTER TO THE TOKEN ARRAY
C               AND IS THE NUMBER OF TOKENS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    NA         INTEGER VARIABLE. THIS IS A POINTER TO THE ALPHA ARRAY
C               AND IS THE NUMBER OF IDENTIFIERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    MVELAB     CHARACTER ARRAY. THIS IS THE MOVE TABLE.
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINE ERROR, ABORT.
C
C  ABORT CONDITIONS THAT MAY OCCUR:
C    IF THE MOVE TABLE OVERFLOWS, THIS WILL CAUSE THE PROGRAM TO STOP
C    AND ISSUE AN INFORMATIVE MESSAGE.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      INTEGER TOKEN(*),TYPE(*),TDEF(*)
      CHARACTER*(*) ALPHA(*),MVELAB(*)
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
C  AN IDENTIFIER ( THE <LABEL NAME> ) IS EXPECTED AFTER THE
C  "MOVE TO" STATEMENT.
C
      NT=NT+1
      IF (TOKEN(NT).EQ.IDENT) THEN
         NA=NA+1
C  PUSH THE DESTINATION LABEL IN THE MOVE TABLE.
         NMOVE=NMOVE+1
         IF (NMOVE.GT.MAXMOV)
     >              CALL OVER('MOVE','MVELAB','MOVE',MAXMOV,'MXMOV')
         MVELAB(NMOVE)=ALPHA(NA)
C  GENERATE CODE.
         BLOC=BLOC+2
         IF (GENE) WRITE(SCR,'(A/)')'$MOVE'
C  UPDATE THE TYPE AND TYPE-DEFINITION TABLES.
         NTYPE=NTYPE+1
         IF (NTYPE.GT.MAXTYP)
     >              CALL OVER('TYPE','TYPE','MOVE',MAXTYP,'MXTYP')
         TYPE(NTYPE)=-7
         TDEF(NTYPE)=BLOC
         NT=NT+1
C  IF ANYTHING ELSE THAN AN END OF LINE FOLLOWS THE STATEMENT.
         IF (NT.NE.NTOK) CALL ERROR(15,' ')
      ELSE
C  AN IDENTIFIER DOES NOT FOLLOW THE "MOVE TO" STATEMENT.
         CALL ERROR(16,' ')
      END IF
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE MULTI ( TOKEN, NT, ALPHA ,NA, NUM, NN,NTOK,VAR,ID,NAME,
     >                   LOC,DIM,INFO,IPOS,KIND )
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE HANDLES STATEMENTS WITH MANY PARAMETERS, OF THE FORM:
C
C  <COMMAND> ( KEY1=EXPR1; KEY2=EXPR2; ... KEYN=EXPRN )  OR
C  <COMMAND>
C
C  WHERE <COMMAND> IS THE COMMAND NAME,
C  KEY1,KEY2,...KEYN ARE KEYWORDS CORRESPONDING TO MERLIN-PANEL
C  PARAMETERS TO BE CHANGED AND
C  EXPR1,EXPR2,...EXPRN ARE EXPRESSIONS. THE VALUES THEY YIELD, ARE
C  ASSIGNED TO THE CORRESPONDING MERLIN PARAMETERS.
C  <COMMAND> MAY BE ONE OF THE FOLLOWING:
C
C  1) RANDOM
C  2) CONGRA
C  3) DFP
C  4) BFGS
C  5) ROLL
C  6) GRAPH
C  7) SIMPLEX
C  8) AUTO
C
C  EACH ONE OF THE ABOVE COMMANDS HAS A UNIQUE SET OF ALLOWABLE
C  KEYWORDS AS:
C
C  RANDOM   CONGRA   DFP      BFGS     ROLL     GRAPH    SIMPLEX  AUTO
C  ------   ------   ---      ----     ----     -----    -------  ----
C  CANCEL   CANCEL   CANCEL   CANCEL   CANCEL   CANCEL   CANCEL   CANCEL
C  CSIZE    ERR      ERR      ERR      FAIL     COLS     INCALL   NOC
C  FAIL     NOC      NOC      NOC      NOC      FROM     INIT     TARGET
C  LINE     PRI      PRI      PRI      PRI      LINES    INITOL
C  NOC      TOL      TOL      TOL      STEP     NOP      NOC
C  PRI      USEG     USEG     USEG     TOL      TO       PRI
C  STEP     WALL     USEH     USEG     WALL     VAR      TOL
C  VEX               WALL     WALL
C
C  ANY OF THESE KEYWORDS MAY BE ENTERED ONLY ONCE IN THE STATEMENT.
C  THE OBJECT CODE GENERATED IS:
C
C  $PUSHC
C  IND1
C  <CODE TO EVALUATE EXPR1>
C  $PUSHC
C  IND2
C  <CODE TO EVALUATE EXPR2>
C  ...
C  $PUSHC
C  INDN
C  <CODE TO EVALUATE EXPRN>
C  $WMULTI
C  <CANCEL>
C  <COMMAND>
C  <CHANGES>
C  $SWAP
C
C  WHERE IND1,IND2,...INDN ARE THE NUMBERS IN THE MERLIN-PANELS OF THE
C  PARAMETERS TO BE CHANGED.
C  <CODE TO EVALUATE EXPRN> IS THE OBJECT CODE THAT EVALUATES EXPRN,
C  <CANCEL> IS THE CANCEL BUTTON NUMBER IN THE MERLIN-PANEL
C  CORRESPONDING TO THIS COMMAND, AND
C  <CHANGES> IS THE NUMBER OF CHANGES MADE TO THE PANEL PARAMETERS.
C  (ACTUALLY THE NUMBER OF "KEY=EXPR" PAIRS ENTERED IN THE STATEMENT)
C
C  IF JUST <COMMAND> IS ENTERED, THE OBJECT CODE GENERATED IS:
C  $WMULTI
C  <CANCEL>
C  <COMMAND>
C  0
C  $SWAP
C
C  FOR EACH COMMAND, THIS ROUTINE HANDLES, 3 ARRAYS ARE DEFINED
C  AS FOLLOWS:
C
C  1)  THE KEYWORD TABLE.
C      CHARACTER*8 ARRAY. IT IS SORTED IN ASCENDING ORDER. THESE
C      ARE THE KEYWORDS ALLOWED FOR THIS COMMAND.
C  2)  THE KEYWORD INDICATOR.
C      LOGICAL ARRAY. AN ENTRY IN THIS ARRAY, YIELDS A VALUE OF .TRUE.
C      IF THE CORRESPONDING KEYWORD ( IN THE KEYWORD TABLE ) HAS BEEN
C      ENTERED IN THE STATEMENT, .FALSE. OTHERWISE.
C  3)  THE KEYWORD INDEX TABLE.
C      INTEGER ARRAY. EACH ENTRY IN THIS ARRAY IS THE NUMBER IN THE
C      MERLIN-PANEL FOR THE CORRESPONDING KEYWORD IN THE KEYWORD TABLE.
C
C  8 PARAMETERS ARE DEFINED ( ONE FOR EACH COMMAND ), THAT INDICATE THE
C  NUMBER OF KEYWORDS ALLOWED FOR THE COMMAND. THE ABOVE ARRAYS ARE
C  DIMENSIONED BY THESE PARAMETERS.
C  AN ADDITIONAL 8-ELEMENT ARRAY ( NAMED ICAN ) IS DEFINED, TO STORE THE
C  CANCEL BUTTON NUMBERS. EACH ENTRY IN THIS ARRAY IS THE NUMBER OF THE
C  CANCEL BUTTON IN THE MERLIN-PANEL FOR THE CORRESPONDING COMMAND.
C
C  INITIALLY ALL KEYWORD INDICATOR ENTRIES WILL BE SET TO .FALSE.
C  WHEN A KEYWORD IS ENCOUNTERED, WHILE PARSING THE STATEMENT AND
C  THE CORRESPONDING ENTRY IN THE KEYWORD INDICATOR IS .TRUE., AN
C  MCL ERROR WILL BE ISSUED. ( PREVENTING MULTIPLE ENTRY OF THE SAME
C  KEYWORD )
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C    NUM        REAL ARRAY. THE NUMBERS ENCOUNTERED IN THE INPUT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C    ID         INTEGER VARIABLE. AN IDENTIFICATION CODE FOR EACH
C               ONE OF THE COMMANDS THIS ROUTINE CAN HANDLE.
C               ITS VALUE AND THE CORRESPONDING COMMANDS ARE AS:
C                 1  RANDOM
C                 2  CONGRA
C                 3  DFP
C                 4  BFGS
C                 5  ROLL
C                 6  GRAPH
C                 7  SIMPLEX
C                 8  AUTO
C    NAME       CHARACTER VARIABLE. THE COMMAND TO BE PROCESSED.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    NT         INTEGER VARIABLE. THIS IS A POINTER TO THE TOKEN ARRAY
C               AND IS THE NUMBER OF TOKENS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    NA         INTEGER VARIABLE. THIS IS A POINTER TO THE ALPHA ARRAY
C               AND IS THE NUMBER OF IDENTIFIERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    NN         INTEGER VARIABLE. THIS IS A POINTER TO THE NUM ARRAY
C               AND IS THE NUMBER OF NUMBERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    VAR        CHARACTER ARRAY. THIS IS THE VAR TABLE.
C    LOC        INTEGER ARRAY. THIS IS THE MEMORY LOCATION TABLE.
C    DIM        INTEGER ARRAY. THIS IS THE DIMENSIONALITY TABLE.
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    IPOS       INTEGER ARRAY. THIS IS THE POSITION TABLE.
C    KIND       INTEGER ARRAY. THIS IS THE KIND TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES LOCAN, ERROR, EXTRA, XPR, ABORT.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER (  IDLEN = 30   )
C
C  THE LENGTH OF A LINE. (CHARS)
      PARAMETER (  LINLEN = 80  )
C
C  MAXIMUM NUMBER OF KEYWORDS FOR ANY COMMAND.
      PARAMETER ( MAXKEY = LINLEN/2 )
C
      INTEGER TOKEN(*),LOC(*),DIM(*),INFO(*),IPOS(*),KIND(*)
      REAL NUM(*)
      CHARACTER*(*) ALPHA(*),VAR(*),NAME
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      LOGICAL CORRE
      LOGICAL DEFKEY(MAXKEY)
      INTEGER EX(LINLEN), IS(MAXKEY), IE(MAXKEY)
      CHARACTER*(IDLEN) KEY,STARG(1)
      CHARACTER*(LINLEN) DLINE
C
C  IF JUST THE <COMMAND> IS ENTERED WITHOUT ANY PARAMETERS.
C
      NT=NT+1
      IF (NT.EQ.NTOK) THEN
         CALL COCAN(NAME,DLINE,IS,IE,NIT,ICAN)
         BLOC=BLOC+5
         IF (GENE) WRITE(SCR,'(A/I3/A/I3/A)')
     >            '$WMULTI',ICAN,NAME,0,'$SWAP'
         RETURN
      END IF
C
C  A LEFT PARENTHESIS SHOULD FOLLOW THE <COMMAND>.
C
      IF (TOKEN(NT).NE.LEFTP) THEN
         CALL ERROR(51,' ')
         RETURN
      END IF
C
C  INITIALIZE KEYWORD INDICATORS.
C
      DO 101,I=1,MAXKEY
        DEFKEY(I) = .FALSE.
 101  CONTINUE
C
C  NC COUNTS THE NUMBER OF CHANGES TO THE PANEL PARAMETERS.
      NC=0
      CALL COCAN(NAME,DLINE,IS,IE,NIT,ICAN)
C  IS IT A COMMAND FROM THE DESCRIPTION FILE ?
      IF (.NOT.GENE) RETURN
C
C  LOOP OVER ALL KEYWORDS ENTERED.
C
 1000 CONTINUE
      NT=NT+1
      CUR=TOKEN(NT)
      NT=NT+1
      IF (NT.GT.NTOK) THEN
         CALL ERROR(52,' ')
         RETURN
      END IF
      NEXT=TOKEN(NT)
C  A SEQUENCE OF AN IDENTIFIER AND THE = SIGN IS EXPECTED.
      IF (CUR.NE.IDENT .OR. NEXT.NE.EQ) THEN
         CALL ERROR(52,' ')
         RETURN
      END IF
C  GET THE KEYWORD.
      NA=NA+1
      KEY=ALPHA(NA)
C  LOCATE THE KEYWORD IN THE DESCRIPTION LINE.
      DO 150,I=2,NIT
         IF (KEY.EQ.DLINE(IS(I):IE(I))) THEN
         IF (DEFKEY(I-1)) CALL ERROR(166,KEY)
         DEFKEY(I-1) = .TRUE.
         GOTO 20
         END IF
150   CONTINUE
C
C  KEYWORD NOT VALID FOR THIS COMMAND.
      CALL ERROR(53,KEY)
      RETURN
20    CONTINUE
C
C  WRITE CODE, TO PUSH THE NUMBER OF THE KEYWORD IN THE
C  MERLIN-PANEL, ON THE STACK.
C
      NC=NC+1
      BLOC=BLOC+2
      IF (GENE) WRITE(SCR,'(A/I2)') '$PUSHC',I-1
C  EXTRACT THE EXPRESSION FOLLOWING THE = SIGN.
C  STOP AT THE FIRST INVALID TOKEN, OR THE LAST RIGHT PARENTHESIS
C  BEFORE THE END OF LINE.
      CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,2,ALPHA,NA)
C  EVALUATE THE EXPRESSION.
      CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,BLOC,LOC,DIM,
     >         INFO,IPOS,KIND,SCR,STARG)
C  IF EXPRESSION INCORRECT.
      IF (.NOT.CORRE) RETURN
C
      NT=NT+1
      CUR=TOKEN(NT)
      IF (CUR.EQ.SEMI) THEN
C  A SEMICOLON DENOTES ANOTHER "KEY=VALUE" PAIR IS PRESENT.
         GOTO 1000
      ELSE
         IF (CUR.EQ.RIGHTP) THEN
C  A RIGHT PARENTHESIS TERMINATES THE STATEMENT, SO WRITE THE 
C  FINAL OBJECT CODE.
            BLOC=BLOC+5
            IF (GENE) WRITE(SCR,'(A/I3/A/I3/A)')
     >                '$WMULTI',ICAN,NAME,NC,'$SWAP'
         ELSE
C  NEITHER SEMICOLON NOR A RIGHT PARENTHESIS WAS FOUND.
            CALL ERROR(54,' ')
         END IF
      END IF
C
      END
C  ---------------------------------------------------------------------
      SUBROUTINE NOPAR ( NTOK, NT, NAME, OPTION )
C  ---------------------------------------------------------------------
C
C  PROCESSES COMMANDS WITH NO PARAMETERS. ONLY ONE LINE OF OBJECT CODE
C  IS GENERATED:
C
C  <NAME>
C
C  IF THE COMMAND RESIDES ALREADY IN THE MERLIN OPERATING SYSTEM, OR:
C
C  $<NAME>
C
C  IF THE COMMAND IS IMPLEMENTED FOR MCL USE ONLY.
C  <NAME> IS THE COMMAND TO BE PROCESSED AND IS INPUT TO THE ROUTINE.
C
C  INPUT ARGUMENTS:
C
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C    NAME       THE COMMAND TO BE PROCESSED.
C    OPTION     THIS IS AN OPTION. IF OPTION=0 THE COMMAND RESIDES IN
C               THE MERLIN OPERATING SYSTEM. IF OPTION=1 THE COMMAND IS
C               IMPLEMENTED FOR MCL USE ONLY AND WILL BE PREFIXED BY A
C               DOLLAR SIGN WHEN WRITTEN TO THE SCRATCH FILE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    NT         INTEGER VARIABLE. THIS IS THE NUMBER OF TOKENS IN THE
C               INPUT LINE USED SO FAR BY THE PREVIOUSLY CALLED
C               PARSING ROUTINES.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINE ERROR.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      CHARACTER*(*) NAME
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      NT=NT+1
C  IF NOTHING ELSE FOLLOWS THE COMMAND, GENERATE CODE.
      IF (NT.EQ.NTOK) THEN
          BLOC=BLOC+1
          IF (GENE) THEN
C  CODE IS GENERATED, ACCORDING TO THE INPUT OPTION.
             IF (OPTION.EQ.0) WRITE(SCR,'(A)') NAME
             IF (OPTION.EQ.1) WRITE(SCR,'(1H$,A)') NAME
          END IF
      ELSE
C  THE COMMAND IS NOT FOLLOWED BY AN END OF LINE.
         CALL ERROR(14,' ')
      END  IF
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE OVER ( TABLE, ARRAY, SUB, MAXIM, PARAM )
C  ---------------------------------------------------------------------
C
C  WHEN ANY OF THE MCLCOM TABLES OVERFLOWS, THIS ROUTINE WILL BE CALLED
C  TO ISSUE INFORMATIVE MESSAGES AND STOP EXECUTION OF THE PROGRAM.
C
C  INPUT ARGUMENTS:
C
C    TABLE      CHARACTER VARIABLE. THIS THE NAME OF THE TABLE THAT
C               OVERFLOWED.
C    ARRAY      CHARACTER VARIABLE. THIS IS THE NAME OF THE ARRAY
C               IMPLEMENTING THE TABLE.
C    SUB        CHARACTER VARIABLE. THIS IS THE NAME OF THE ROUTINE
C               IN WHICH THE OVERFLOW OCCURRED.
C    MAXIM      INTEGER VARIABLE. THIS IS THE MAXIMUM NUMBER OF ENTRIES
C               ALLOWED IN THE TABLE.
C    PARAM      CHARACTER VARIABLE. THIS IS THE NAME OF THE PARAMETER
C               THAT DIMENSIONS THE TABLE.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      LOGICAL EOPEN
      COMMON /ISOPEN/ EOPEN
C
      CHARACTER*(*) TABLE,ARRAY,SUB,PARAM
C
 100  FORMAT (2X,A,' TABLE OVERFLOW. OCCURRED IN ROUTINE ',A)
 200  FORMAT('  ARRAY ',A,' WAS ASSIGNED ONLY',I6,' STORAGE LOCATIONS.')
 300  FORMAT ('  THIS ARRAY IS DIMENSIONED BY THE ',A,' PARAMETER.')
C
      WRITE(*,100) TABLE,SUB
      WRITE(*,200) ARRAY,MAXIM
      WRITE(*,300) PARAM
C
C  WRITE THE SAME MESSAGE ON THE ERROR LIST FILE.
C
      IF (EOPEN) THEN
         WRITE (ERL,100) TABLE,SUB
         WRITE (ERL,200) ARRAY,MAXIM
         WRITE (ERL,300) PARAM
      END IF
C
      STOP ' -- MCLCOM EXECUTION ABORT --'
      END
C  ---------------------------------------------------------------------
      SUBROUTINE PCG ( TOKEN,NTOK,NUM,ALPHA,LABEL,LABLOC,TYPE,TDEF,LLOC,
     >                 VAR,MVELAB,LOC,DIM,INFO,IPOS,KIND,TREL )
C  ---------------------------------------------------------------------
C
C  THIS IS THE MAIN ROUTINE THAT DRIVES THE OTHERS ACCORDING TO THE
C  INPUT LINE. THIS ROUTINE IS CALLED FOR EXECUTABLE STATEMENTS. (I.E.
C  STATEMENTS THAT PRODUCE OBJECT CODE) AT THIS POINT ALL DECLARATION
C  STATEMENTS HAVE BEEN PROCESSED.
C  THE FIRST TOKEN ENCOUNTERED IN THE INPUT LINE, SHOULD BE AN
C  IDENTIFIER, OTHERWISE AN MCL-ERROR IS ISSUED.
C  IF A COLON FOLLOWS, THEN THE ROUTINE WILL ASSUME THAT THIS IS A
C  LABEL DEFINITION LINE.
C  IF AN EQUALITY SIGN, OR A LEFT BRACKET FOLLOWS, THE ROUTINE WILL
C  ASSUME, THAT THIS IS AN ASSIGNMENT.
C  IF ITS NEITHER A LABEL, NOR AN ASSIGNMENT, THE ROUTINE WILL TEST
C  FOR ONE OF THE FOLLOWING STATEMENTS:
C  WHEN, IF, ELSE, ENDIF, LOOP, ENDLOOP
C  IF THESE TESTS FAIL ALSO, THE EXE ROUTINE WILL BE CALLED TO TEST
C  FOR THE REST OF THE MCL STATEMENTS.
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C    NUM        REAL ARRAY. THE NUMBERS ENCOUNTERED IN THE INPUT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C               THE INPUT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    LABEL      CHARACTER ARRAY. THIS IS THE LABEL TABLE.
C    LABLOC     INTEGER ARRAY. THIS IS THE LABEL-LOCATION TABLE.
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C    LLOC       INTEGER ARRAY. THIS IS THE LOOP TABLE.
C    VAR        CHARACTER ARRAY. THIS IS THE VAR TABLE.
C    MVELAB     CHARACTER ARRAY. THIS IS THE MOVE TABLE.
C    LOC        INTEGER ARRAY. THIS IS THE MEMORY LOCATION TABLE.
C    DIM        INTEGER ARRAY. THIS IS THE DIMENSIONALITY TABLE.
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    IPOS       INTEGER ARRAY. THIS IS THE POSITION TABLE.
C    KIND       INTEGER ARRAY. THIS IS THE KIND TABLE.
C    TREL       INTEGER ARRAY. THIS IS THE TYPE-LINK TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES LBDEF, EXE, WHEN, IF, ELSE, ENDIF, LOOP, ENDLOO.
C
C  NOTES:
C    NOTE THAT THIS ROUTINE PROCESSES DIRECTLY ALL MCL-STATEMENTS THAT
C    MAY APPEAR NEXT TO A "WHEN" STATEMENT. SUBROUTINE EXE WILL TAKE
C    CARE OF THE REST OF THE STATEMENTS.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER (  IDLEN = 30   )
C
      INTEGER TOKEN(*),LABLOC(*),TYPE(*),TDEF(*),LOC(*),DIM(*),INFO(*),
     >        IPOS(*),KIND(*),LLOC(*),TREL(*)
      REAL NUM(*)
      CHARACTER*(*) ALPHA(*),LABEL(*),VAR(*),MVELAB(*)
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      CHARACTER*(IDLEN) NAME
C  GET THE FIRST TOKEN FROM THE INPUT LINE.
      INIT=TOKEN(1)
C  THE FIRST TOKEN SHOULD BE AN IDENTIFIER, SINCE ALL MCL-STATEMENTS
C  START THIS WAY.
      IF (INIT.EQ.IDENT) THEN
         NAME=ALPHA(1)
         NEXT=TOKEN(2)
         IF (NEXT.EQ.COLON) THEN
C  POSSIBLY A LABEL.
            CALL LBDEF(NTOK,LABEL,LABLOC,NAME)
         ELSE IF (NEXT.EQ.EQ .OR. NEXT.EQ.LBRA) THEN
C  POSSIBLY AN ASSIGNMENT.
            NT=0
            NA=0
            NN=0
            CALL EXE(TOKEN,NTOK,ALPHA,NUM,NT,NA,NN,TYPE,TDEF,VAR,
     >               MVELAB,LOC,DIM,INFO,IPOS,KIND,LLOC)
         ELSE IF (NAME.EQ.'WHEN') THEN
C  THIS IS A "WHEN" STATEMENT.
            CALL WHEN(TOKEN,NTOK,ALPHA,NUM,TYPE,TDEF,TREL,VAR,
     >                MVELAB,LOC,DIM,INFO,IPOS,KIND,LLOC)
         ELSE IF (NAME.EQ.'IF') THEN
C  THIS IS AN "IF" STATEMENT.
            CALL IF(TOKEN,NTOK,ALPHA,NUM,TYPE,TDEF,VAR,LOC,DIM,INFO,
     >              IPOS,KIND)
         ELSE IF (NAME.EQ.'ELSE') THEN
C  THIS IS AN "ELSE" STATEMENT.
            CALL ELSE(NTOK,TYPE,TDEF,TREL)
C  THIS IS AN "ENDIF" STATEMENT.
         ELSE IF (NAME.EQ.'ENDIF') THEN
            NT=1
            CALL ENDIF(NTOK,NT,TYPE,TDEF,TREL)
C  THIS IS A "LOOP" STATEMENT.
         ELSE IF (NAME.EQ.'LOOP') THEN
            CALL LOOP (TOKEN,ALPHA,NUM,NTOK,VAR,LOC,DIM,INFO,
     >                 IPOS,KIND,LLOC,TYPE,TDEF)
C  THIS IS AN "ENDLOOP" STATEMENT.
         ELSE IF (NAME.EQ.'ENDLOOP') THEN
            NT=1
            CALL ENDLOO (NTOK,NT,TYPE,TDEF,TREL,LLOC)
         ELSE IF (NAME.EQ.'END' .AND. NEXT.EQ.IDENT) THEN
C  THIS MAY BE THE CASE OF AN "END IF" OR AN "END LOOP".
            IF (ALPHA(2).EQ.'IF') THEN
               NT=2
               CALL ENDIF(NTOK,NT,TYPE,TDEF,TREL)
            ELSE IF (ALPHA(2).EQ.'LOOP') THEN
               NT=2
               CALL ENDLOO (NTOK,NT,TYPE,TDEF,TREL,LLOC)
            ELSE
               NT=0
               NA=0
               NN=0
C  NEITHER "END IF" NOR "END LOOP" WAS FOUND.
               CALL EXE(TOKEN,NTOK,ALPHA,NUM,NT,NA,NN,TYPE,TDEF,VAR,
     >              MVELAB,LOC,DIM,INFO,IPOS,KIND,LLOC)
            END IF
         ELSE IF (NAME.EQ.'VAR' .OR. NAME.EQ.'FUNCTION') THEN
C  WHEN THIS ROUTINE IS CALLED, ALL DECLARATION STATEMENTS HAVE BEEN
C  PROCESSED. A "VAR" OR "FUNCTION" STATEMENT IS MISPLACED HERE.
            CALL ERROR(42,NAME)
         ELSE
C  POSSIBLY ANY OF THE OTHER MCL STATEMENTS.
            NT=0
            NA=0
            NN=0
            CALL EXE(TOKEN,NTOK,ALPHA,NUM,NT,NA,NN,TYPE,TDEF,VAR,MVELAB,
     >               LOC,DIM,INFO,IPOS,KIND,LLOC)
         END IF
      ELSE
C  ALL MCL STATEMENTS BEGIN WITH AN IDENTIFIER.
         CALL ERROR(7,' ')
      END IF
C
      RETURN
      END
CPFIX-------------------------------------------------------------------
      SUBROUTINE PFIX ( EXPR, TOKNUM, NEWTOK )
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: TRANSFORMS EXPRESSIONS FROM INFIX
C                  FORM TO A HYBRID POSTFIX FORM.
C
C   %..   ARGUMENT INFORMATION:
C
C   -EXPR =   IS AN ARRAY HOLDING THE TOKENS OF THE EXPRESSION,
C   -TOKNUM = IS THE NUMBER OF TOKENS IN EXPR ARRAY,
C   -NEWTOK = IS THE NUMBER OF TOKENS LEFT IN EXPR UPON RETURN.
C
C   %..   SUBPROGRAMS CALLED:
C
C     SUBROUTINES: PUSH, POP, ABORT,
C     FUNCTIONS  : NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C         EXPRST (M) , MCLSCD (A)
C
C   %..   POSSIBLE ABORTS:
C
C     NORMALLY NOTHING ERRONEOUS MAY APPEAR. THE ONLY
C     SITUATION IS WHEN XCHECK ROUTINE HAS MISSED SOMETHING
C     WHICH WILL BLOCK PFIX AND DRIVE IT TO STOP.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS:
C
C    -STACK = THIS STACK HOLDS OPERATORS, PARENTHESES AND
C             BRAKETS FOUND, DURING SCANNING OF INFIX EXPR. IT IS
C             MANUPULATED BY PUSH AND POP ROUTINES.
C    -PFBUF = THIS IS AN ARRAY ON WHICH POSTFIX EXPRESSION
C             GRADUALLY IS BUILDED.
C    -PRIOR = IS AN ARRAY HOLDING THE PRIORITIES OF MCL OPERATORS.
C
C   %..   NOTES:
C
C    (I) THE HYBRID NOTATION MENTIONED EARLIER IS AS FOLLOWS:
C        IF THE EXPRESSION DOES NOT CONTAIN FUNCTION REFE-
C        RENCES OR ARRAY ELEMENTS, THEN IT COINCIDES WITH THE
C        STANDARD POSTFIX FORM. FOR EXAMPLE:
C        (A+B)-C*2 (INFIX) BECOMES AB+C2*- (POSTFIX)
C        IF EXPRESSION CONTAINS EITHER ARRAYS OR FUNCTION
C        REFERENCES, THEN THE FOLLOWING IS DONE:
C        A[1,2]*MAX[4,X] (INFIX) BECOMES A1,2]MAX4,X]* (POSTFIX)
C        THAT IS, IT IS POSTFIX BUT THE ARGUMENTS AND\OR SUB-
C        SCRIPTS REMAIN IN THEIR ORDER AS THE EXPRESSION IS
C        BEING TRANSFORMED, AND THE FINAL BRAKET ALSO REMAINS
C        TO TERMINATE THE SEQUENCE (COMMAS ALSO REMAIN). IF WE
C        FACE : A[X-2*B,1] (INFIX) WE WRITE AX2B*-,1] (POSTFIX)
C        THAT IS, THE ITEMS ENCLOSED IN BRAKETS, WHICH ARE
C        SIMPLE EXPRESSIONS ARE TRANSFORMED IN STANDARD POSTFIX
C        BUT APPEAR IN THEIR ORDER IN THE FINAL HYBRID FORM.
C
C        FROM THE ABOVE DESCRIPTION IT SHOULD BE CLEAR THAT
C        SPECIAL PROCESSING IS NEEDED (WHEN CODE IS TO BE GENERATED
C        FROM THE HYBRID POSTFIX FORM) SO THAT ITEMS ENCLOSED
C        IN BRAKETS, BE PROCESSED IN A WAY TO COMPENSATE FOR THE
C        ABSENCE OF FULL POSTFIX ARRANGEMENT.
C
C   (II) IT IS ASSUMED THAT EXPR HAS BEEN CHECKED FOR VALIDITY BY
C        XCHECK ROUTINE.
C
C  (III) THE PRIORITY OF OPERATORS IS :
C                                      NOT        9
C                                       **        8
C                                      *,/        7
C                                      +,-        6
C                             >,<,:=:,#,<=,>=     5
C                                      AND        4
C                                       OR        3
C                                      XOR        2
C                              PARENTHESES        1
C
C         OPERANDS (I.E. NUMERIC CONSTANTS AND IDENTIFIERS ) TYPICALLY
C         HAVE PRIORITY 10, SINCE THEY ARE APPENDED TO POSTFIX ONCE
C         THEY HAVE BEEN DETECTED. THE SAME IS TRUE FOR PARENTHESIZED
C         EXPRESSIONS, FOR FUNCTIONS CALLS OR ARRAY REFERENCES.
C         OFCOURSE THE VALIDITY OF THE LAST STATEMENT, IS DESCRI-
C         PTORY, SINCE "(" HAS PRIORITY 1. NOTE THOUGH, THAT THIS
C         IS EXACTLY THE END RESULT.
C
C   (IV)  THE ASSOCIATIVITY OF ALL THE ABOVE OPERATORS IS
C         FROM LEFT TO RIGHT.
C
C   (V)  THE ALGORITHM FOLLOWED, CAN BE SUMMARIZED AS FOLLOWS:
C
C        FOR EACH TOKEN INTO EXPRESSION DO:
C           CHECK IF TOKEN = OPERAND, THEN
C               APPEND TOKEN TO POSTFIX
C           ELSE IF TOKEN = LEFTPAREN OR TOKEN = LEFTBRA THEN
C               PUSH TOKEN TO STACK
C           ELSE IF TOKEN = RIGHTPAREN THEN
C               POP ITEM FROM STACK
C               WHILE ITEM NOT LEFTPAREN DO
C                    APPEND ITEM TO POSTFIX
C                    POP ITEM FROM STACK
C           ELSE IF TOKEN = RIGHTBRA THEN
C               POP ITEM FROM STACK
C               WHILE ITEM NOT LEFTBRA DO
C                    APPEND ITEM TO POSTFIX
C                    POP ITEM FROM STACK
C               IF ITEM = LEFTBRA THEN
C                    APPEND TOKEN TO POSTFIX
C           ELSE IF TOKEN = COMMA THEN
C               POP ITEM FROM STACK
C               WHILE ITEM NOT LEFTBRA AND ITEM NOT COMMA DO
C                    APPEND ITEM TO POSTFIX
C                    POP ITEM FROM STACK
C               APPEND TOKEN TO POSTFIX
C               IF ITEM = LEFTBRA THEN
C                    PUSH LEFTBRA TO STACK
C               ELSE IF TOKEN = OPERATOR THEN
C                   POP ITEM FROM STACK
C                   IF NO ITEM LEFT THEN
C                       PUSH TOKEN TO STACK
C                   ELSE IF TOKENPRIORITY > ITEMPRIORITY THEN
C                       PUSH ITEM TO STACK
C                       PUSH TOKEN TO STACK
C                   ELSE
C                       WHILE ITEM NOT LEFTPAREN DO
C                            APPEND TOKEN TO POSTFIX
C                            POP ITEM FROM STACK
C                            IF NO ITEM LEFT THEN
C                                PUSH TOKEN TO STACK
C                            ELSE IF TOKENPRIORITY > ITEMPRIORITY THEN
C                                PUSH ITEM TO STACK
C                                PUSH TOKEN TO STACK
C                        IF ITEM = LEFTPAREN THEN
C                            PUSH LEFTPAREN TO STACK
C                            PUSH TOKEN TO STACK
C             ELSE
C                 STOP
C        WHILE NOT EMPTY STACK DO
C           POP ITEM FROM STACK
C           APPEND ITEM TO POSTFIX
C        END
C
C
C-----------------------------------------------------------------------
C
      IMPLICIT INTEGER ( A - Z )
C
      PARAMETER ( LINLEN=80, IDLEN=30, ONE=1000, ZERO=-1000 )
      DIMENSION STACK(1:LINLEN), PFBUF(1:LINLEN), EXPR(TOKNUM),
     >          PRIOR(1:31)
C
C
      LOGICAL OPERAN, OPRTOR, LFTPAR, RTPAR, EMPTY, RTBRA, LFTBRA, COMMA
C
      COMMON / EXPRST / TOPBUF, STACK
     >       / MCLSCD / LEFTP, RIGHTP, DOT, COMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUMBER,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
C
      DATA (PRIOR(I),I=1,31) / 1,0,0,0,5,5,5,0,5,5,6,6,7,7,8,
     >                         0,0,0,0,0,0,0,0,1,0,0,4,3,2,9,5/
C
      OPERAN() = (TEMPO.EQ.IDENT).OR.(TEMPO.EQ.NUMBER).OR.
     >           (TEMPO.EQ.ZERO).OR.(TEMPO.EQ.ONE)
      OPRTOR() = (TEMPO.EQ.PLUS).OR.(TEMPO.EQ.MINUS).OR.
     >           (TEMPO.EQ.STAR).OR.(TEMPO.EQ.BISTAR).OR.
     >           (TEMPO.EQ.SLASH).OR.(TEMPO.EQ.AND).OR.
     >           (TEMPO.EQ.OR).OR.(TEMPO.EQ.LT).OR.
     >           (TEMPO.EQ.LE).OR.(TEMPO.EQ.GT).OR.
     >           (TEMPO.EQ.GE).OR.(TEMPO.EQ.EQOPER).OR.
     >           (TEMPO.EQ.NE).OR.(TEMPO.EQ.XOR).OR.
     >           (TEMPO.EQ.NOT)
      LFTPAR ()   = ( TEMPO.EQ.LEFTP )
      RTPAR  ()   = ( TEMPO.EQ.RIGHTP )
      RTBRA  ()   = ( TEMPO.EQ.RBRA )
      COMMA  ()   = ( TEMPO.EQ.COMA )
      LFTBRA ()   = ( TEMPO.EQ.LBRA )
C
      TOPBUF=0
      BUFIND=0
C
      DO 1000, K=1, TOKNUM
         TEMPO=EXPR ( K )
         IF  (  OPERAN () )  THEN
C
C           PLACE OPERAND AT THE RIGHTMOST
C           EMPTY POSITION OF POSTFIX EXPRESSION
C
            BUFIND=BUFIND+1
            PFBUF ( BUFIND ) =TEMPO
C
C           GO TO GET NEXT TOKEN.
C
         ELSE IF  (  LFTPAR ()   )  THEN
C
C           PUSH LEFT PARENTHESIS INTO STACK
C
            CALL PUSH ( LEFTP )
C
         ELSE IF  (  LFTBRA ()   )  THEN
C
C           PUSH LEFT BRAKET INTO STACK
C
            CALL PUSH ( LBRA )
C
         ELSE IF  (  RTPAR ()   )  THEN
C
 12         CONTINUE
C           POP OPERATOR
            CALL POP ( OPER, EMPTY )
            IF  ( OPER.EQ.LEFTP )  THEN
               GOTO 1000
            ELSE
               BUFIND=BUFIND+1
               PFBUF ( BUFIND ) =OPER
C
C              GO TO GET NEXT OPERATOR.
C
               GOTO 12
            END IF
C
         ELSE IF  (  RTBRA ()   )  THEN
C
C           BRAKET CASE. CONTINUE TO GET A SEQUENCE OF OPERATORS.
C
 122        CONTINUE
            CALL POP ( OPER, EMPTY )
            IF ( OPER.EQ.LBRA ) THEN
               BUFIND=BUFIND+1
               PFBUF ( BUFIND ) = RBRA
               GOTO 1000
            ELSE
               BUFIND=BUFIND+1
               PFBUF ( BUFIND ) =OPER
C
C              ANOTHER OPERATOR APPENDED TO POSTFIX. GO TO GET NEXT.
C
               GOTO 122
            END IF
C
        ELSE IF ( COMMA() ) THEN
C
C          FOR COMMAS THE PROCESSING IS SIMILAR TO RTBRA.
C
 152       CONTINUE
           CALL POP (OPER, EMPTY )
           IF ( OPER.EQ.LBRA .OR. OPER.EQ.COMA ) THEN
                BUFIND = BUFIND + 1
                PFBUF ( BUFIND ) = COMA
                IF ( OPER.EQ.LBRA ) THEN
                     CALL PUSH ( OPER )
                END IF
                GO TO 1000
           ELSE
                BUFIND = BUFIND + 1
                PFBUF ( BUFIND ) = OPER
C
C               GO TO GET NEXT OPER.
C
                GO TO 152
            END IF
C
         ELSE IF  (  OPRTOR ()   )  THEN
C
C           POP TOPSTACK-ELEMENT AND IF NOT NIL, CHECK PRIORITIES.
C
            CALL POP ( TOPSTK, EMPTY )
            IF ( EMPTY ) THEN
               CALL PUSH ( TEMPO )
               GOTO 1000
            ENDIF
            IF ( PRIOR ( TEMPO ) .GT. PRIOR ( TOPSTK ) )  THEN
               CALL PUSH  ( TOPSTK )
               CALL PUSH ( TEMPO )
               GOTO 1000
            ELSE
 14            CONTINUE
               IF ( TOPSTK.EQ.LEFTP ) THEN
                  CALL PUSH  ( TOPSTK )
                  CALL PUSH  ( TEMPO )
                  GOTO 1000
               ENDIF
               BUFIND=BUFIND+1
               PFBUF ( BUFIND ) =TOPSTK
               CALL POP ( TOPSTK, EMPTY )
               IF ( EMPTY ) THEN
                  CALL PUSH  (  TEMPO  )
                  GOTO 1000
               ENDIF
               IF  (  PRIOR ( TOPSTK ) .GE. PRIOR ( TEMPO ) )  THEN
                  GOTO 14
               ELSE
                  CALL PUSH  (  TOPSTK  )
                  CALL PUSH  (  TEMPO  )
                  GOTO 1000
               ENDIF
            ENDIF
C
         ELSE
            CALL ABORT ('ILLEGAL ITEM INTO EXPRESSION','PFIX')
         END IF
C
 1000 CONTINUE
C
C     PUSH  COMPONENTS LEFT AT STACK TO POSTFIX
C
 99   CONTINUE
C
      CALL POP (TOPSTK,EMPTY)
      IF ( .NOT.EMPTY ) THEN
         BUFIND=BUFIND+1
         PFBUF ( BUFIND ) =TOPSTK
C
C        ANOTHER ITEM APPENDED TO POSTFIX. GO TO GET NEXT.
C
         GOTO 99
      ENDIF
C
      DO 8, L=1, BUFIND
         EXPR ( L ) =PFBUF ( L )
 8    CONTINUE
C
      NEWTOK=BUFIND
      RETURN
      END
CPICK-------------------------------------------------------------------
      SUBROUTINE PICK (TOKENS,TOKN,TUSED,ALPHAC,NA,NUMBC,NN,VAR,
     >                 ADDR, DIMARG, INFO, INFPOS, VRKIND, STVAR)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: THIS ROUTINE PARSES PICK STATEMENT.
C
C   %..   ARGUMENT INFORMATION:
C
C      -TOKENS: IS THE LINE TO BE PROCESSED.
C      -TOKN  : IS THE NUMBER OF TOKENS "TOKENS" IS COMPOSED OF
C      -TUSED : POINTS TO THE LAST USED ELEMENT OF "TOKENS".
C      -ALPHAC: ARE THE IDENTIFIERS OF "TOKENS".
C      -NUMBC : ARE THE NUMERIC CONSTANTS OF "TOKENS".
C      -NA    : POINTS TO THE LAST USED ELEMENT OF ALPHAC.
C      -NN    : POINTS TO THE LAST USED ELEMENT OF NUMBC.
C      -VAR   : THE VAR TABLE.
C      -ADDR  : ARRAY KEEPING LOCATIONS. SEE XGEN FER DETAILS.
C      -DIMARG: ARRAY KEEPING DIMENSIONALITIES. SEE XGEN.
C      -INFO  : ARRAY WITH MCL-ITEMS INFORMATION. SEE XGEN.
C      -INFPOS: ARRAY RELATED TO INFO. SEE XGEN.
C      -VRKIND: PARALLEL ARRAY TO VAR. KEEPS "KIND"S. SEE XGEN.
C      -STVAR : KEEPS DUMMY NAMES FOR FUNCTIONS.
C
C   %..   SUBPROGRAMS CALLED:
C
C     SUBROUTINES: ERROR, EXTRA, XPR,
C     FUNCTIONS  : NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C           MCLDTA (M) , MCLUNT (A) , MCLSCD (A)
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS: NONE.
C
C   %..   NOTES:
C
C     THIS ROUTINE PERFORMS PARSING OF <PICKSTATEMENT>. ITS SYNTAX IS:
C     <PICKSTATEMENT> : : ='PICK'('REC' = <EXPRESSION> ;'FILE' =
C      <FILENAME> )
C              OR          'PICK'('FILE' = <FILENAME> ;'REC' =
C              <EXPRESSION> )
C
C     CODE GENERATED IS ASSOCIATED WITH A FILE WHERE $SWAP DIRECTS
C     READING OF MERLIN.
C
C   CODE WRITTEN IF SYNTAX CORRECT, IS :
C
C           $WPICK
C           FILENAME
C           $SWAP
C                         AND THE TOP OF THE STACK HOLDS
C                         THE NUMBER OF $"S TO BE WRITTEN
C                         ON SWAPPED (SCRATCH) FILE.
C
C-----------------------------------------------------------------------
C
      IMPLICIT INTEGER ( A - Z )
      DIMENSION TOKENS(TOKN),EXPR(76),ADDR(*),DIMARG(*),INFO(*),
     >          INFPOS(*),VRKIND(*)
      PARAMETER (CHOICE=2)
      LOGICAL CORR,GENE,FDECLR,BOUNDS,DEBUG
      CHARACTER*(*) ALPHAC(*),VAR(*),FN*30,STVAR(*)
      REAL NUMBC(*)
      COMMON / MCLUNT / INP, ERL, BIN, SCR, ISTF, SPARZZ, DUMMYF,DESCRI
     >       / MCLDTA / NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
     >       / MCLSCD / LEFTP, RIGHTP, DOT, COMMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUMBER,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
C
C     START
C
      FDECLR=.FALSE.
      TP=TUSED+1
      IF(TP.EQ.TOKN)GOTO 100
      IF(TOKENS(TP).EQ.LEFTP)THEN
        TP=TP+1
        IF(TP.EQ.TOKN)GOTO 100
        NA=NA+1
        IF(ALPHAC(NA).EQ.'REC'.AND.TOKENS(TP).EQ.IDENT)THEN
          TP=TP+1
          IF(TP.GE.TOKN)GOTO 100
          IF(TOKENS(TP).EQ.EQ)THEN
            CALL EXTRA (TOKENS,TOKN,TP,EXPR,NEX,CHOICE,ALPHAC,NA)
            CALL XPR ( EXPR,NEX,ALPHAC,NUMBC,NA,NN,CORR,VAR,
     >                 FDECLR,BLOC,ADDR,DIMARG,INFO,INFPOS,VRKIND,SCR,
     >                 STVAR )
            IF(.NOT.CORR)RETURN
            TP = TP+1
            IF(TP.EQ.TOKN)GOTO 100
            IF(TOKENS(TP).EQ.SEMI)THEN
              TP=TP+1
              IF(TP.EQ.TOKN)GOTO 100
              NA=NA+1
              IF(TOKENS(TP).EQ.IDENT.AND.ALPHAC(NA).EQ.'FILE')THEN
                TP=TP+1
                IF(TP.GE.TOKN)GOTO 100
                IF(TOKENS(TP).EQ.EQ)THEN
                  TP=TP+1
                  IF(TP.EQ.TOKN)GOTO 100
                  IF(TOKENS(TP).EQ.IDENT)THEN
                    NA=NA+1
                    FN=ALPHAC(NA)
                    TP=TP+1
                    IF(TP.EQ.TOKN)GOTO 100
                    IF(TOKENS(TP).EQ.RIGHTP.AND.TP+1.EQ.TOKN)THEN
                      GOTO 50
                    ELSE
                      CALL ERROR (80,' ')
                    ENDIF
                  ELSE
                    CALL ERROR (79,' ')
                  ENDIF
                ELSE
                  CALL ERROR (81,' ')
                ENDIF
              ELSE
                CALL ERROR (82,' ')
              ENDIF
            ELSE
              CALL ERROR (83,' ')
            ENDIF
          ELSE
            CALL ERROR (84,' ')
          ENDIF
        ELSE IF(ALPHAC(NA).EQ.'FILE'.AND.TOKENS(TP).EQ.IDENT)THEN
          TP=TP+1
          IF(TP.GE.TOKN)GOTO 100
          IF(TOKENS(TP).EQ.EQ)THEN
            TP=TP+1
            IF(TP.EQ.TOKN)GOTO 100
            IF(TOKENS(TP).EQ.IDENT)THEN
              NA=NA+1
              FN=ALPHAC(NA)
              TP=TP+1
              IF(TP.EQ.TOKN)GOTO 100
              IF(TOKENS(TP).EQ.SEMI)THEN
                TP=TP+1
                IF(TP.EQ.TOKN)GOTO 100
                NA=NA+1
                IF(ALPHAC(NA).EQ.'REC'.AND.TOKENS(TP).EQ.IDENT)THEN
                  TP=TP+1
                  IF(TP.GE.TOKN)GOTO 100
                  IF(TOKENS(TP).EQ.EQ)THEN
                    CALL EXTRA (TOKENS,TOKN,TP,EXPR,NEX,CHOICE,ALPHAC,
     >                          NA)
                    CALL XPR (EXPR,NEX,ALPHAC,NUMBC,NA,NN,CORR,VAR,
     >                       FDECLR,BLOC,ADDR,DIMARG,INFO,INFPOS,VRKIND,
     >                       SCR,STVAR)
                    IF(.NOT.CORR)RETURN
                    TP = TP+1
                    IF(TP.EQ.TOKN)GOTO 100
                    IF(TOKENS(TP).EQ.RIGHTP.AND.TP+1.EQ.TOKN)THEN
                      GOTO 50
                    ELSE
                      CALL ERROR (80,' ')
                    ENDIF
                  ELSE
                    CALL ERROR (89,' ')
                  ENDIF
                ELSE
                  CALL ERROR (90,' ')
                ENDIF
              ELSE
                CALL ERROR (85,' ')
              ENDIF
            ELSE
              CALL ERROR (85,' ')
            ENDIF
          ELSE
            CALL ERROR (81,' ')
          ENDIF
        ELSE
          CALL ERROR (86,' ')
        ENDIF
      ELSE
        CALL ERROR (87,' ')
      ENDIF
      RETURN
 50   CONTINUE
C
C     WRITE CODE:
C
      IF(GENE) THEN
        WRITE(SCR,'(''$WPICK'')')
        WRITE(SCR,'(A,/,''$SWAP'')')FN
      ENDIF
      BLOC=BLOC+3
      RETURN
 100  CALL ERROR (88,' ')
      RETURN
      END
CPOP--------------------------------------------------------------------
      SUBROUTINE POP ( ITEM , EMPTY )
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: POPS OUT ITEMS FROM A STACK.
C
C   %..   ARGUMENT INFORMATION:
C
C             ITEM = THE ELEMENT POPED OUT. INTEGER.
C             EMPTY = LOGICAL VARIABLE, BEING TRUE WHEN
C                     THE STACK IS EMPTY.
C
C   %..   SUBPROGRAMS CALLED: NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C             EXPRST (M)
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   NOTES:
C
C             THIS ROUTINE IS USED IN SUBROUTINE PFIX.
C
C-----------------------------------------------------------------------
C
      INTEGER LINLEN
      PARAMETER ( LINLEN = 80 )
      INTEGER ITEM,TOP,STACK(1:LINLEN),TOPBUF
      LOGICAL EMPTY
      COMMON / EXPRST / TOPBUF,STACK
C
      TOP=TOPBUF
      IF ( TOP.GE.1 ) THEN
         ITEM=STACK(TOP)
         TOP=TOP-1
         EMPTY=.FALSE.
      ELSE
         EMPTY=.TRUE.
      END IF
      TOPBUF=TOP
      RETURN
      END
CPOPBR------------------------------------------------------------------
      SUBROUTINE POPBR ( ITEM )
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: POPS INTEGER POSITIONS UP FROM
C                  A STACK USED BY XCHECK.
C
C   %..   ARGUMENT INFORMATION:
C
C             ITEM = AN INTEGER POSITION RETURNED TO XCHECK
C                    TO BE USED IN BRAKETS PROCESSING.
C
C   %..   SUBPROGRAMS CALLED: NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C              TSTBST (M)
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   NOTES:
C
C             THIS ROUTINE IS USED BY SUBROUTINE XCHECK.
C
C-----------------------------------------------------------------------
C
      INTEGER ITEM,TOP,TOPMAX,LINLEN
      PARAMETER (LINLEN=80, TOPMAX=LINLEN)
      INTEGER BSTACK(1:TOPMAX)
      LOGICAL OK
      COMMON / TSTBST / TOP,BSTACK,OK
C
      IF(TOP.GE.1)THEN
         ITEM=BSTACK(TOP)
         TOP=TOP-1
         OK=.TRUE.
         RETURN
      ELSE
         OK=.FALSE.
         RETURN
      ENDIF
      END
CPOPUP------------------------------------------------------------------
      SUBROUTINE POPUP ( ITEM )
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: POPS NUMBERS FROM A STACK.
C
C   %..   ARGUMENT INFORMATION:
C
C      ITEM = THE NUMBER RETURNED FROM THE STACK.
C
C   %..   SUBPROGRAMS CALLED: NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C      TSTXST(M)
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS:
C
C     -TOP = HOLDS THE TOP POSITION OF THE STACK,
C     -STACK = THE ARRAY HOLDING THE ELEMENTS OF STACK,
C     -SUCCES = TRUE IF THERE IS SOMETHING TO BE POPED.
C
C   %..   NOTES:
C
C   THIS ROUTINE IS CALLED FROM ROUTINE XCHECK.
C
C-----------------------------------------------------------------------
C
      INTEGER ITEM,TOP,TOPMAX,LINLEN
      PARAMETER (LINLEN=80, TOPMAX=LINLEN)
      INTEGER STACK(1:TOPMAX)
      LOGICAL SUCCES
      COMMON / TSTXST / TOP,STACK,SUCCES
C
      IF ( TOP.GE.1 ) THEN
         ITEM = STACK(TOP)
         TOP = TOP - 1
         SUCCES = .TRUE.
         RETURN
      ELSE
         SUCCES = .FALSE.
         RETURN
      END IF
      END
CPSHALP-----------------------------------------------------------------
      SUBROUTINE PSHALP (ITEM)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: PUSHES ITEMS INTO THE ALPHANUMERIC
C                  STACK BUILDED BY SUBROUTINE LEX.
C
C   %..   ARGUMENT INFORMATION:
C
C             ITEM = THE CHARACTER TYPE ITEM TO BE PUSHED.
C
C   %..   SUBPROGRAMS CALLED: ABORT ROUTINE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C             ALPH (M) , TTALP (M).
C
C   %..   POSSIBLE ABORTS:
C
C             THE STACK MAY BE OVERFLOWN IN WHICH CASE MCLCOM STOPS.
C
C   %..   NOTES:
C
C             THIS ROUTINE IS USED BY ROUTINE LEX TO BUILD ONE OF
C             ITS MAIN STACKS: THE ONE HOLDING IDENTIFIERS.
C
C-----------------------------------------------------------------------
C
      INTEGER LINLEN,IDLEN
      PARAMETER (LINLEN=80,IDLEN=30)
      CHARACTER ALPCON(1:LINLEN)*(IDLEN),ITEM*(*)
      COMMON /ALPH/ ALPCON
     >       /TTALP/TOPALP
      INTEGER TOP,TOPALP
C
      TOP=TOPALP
      IF ( TOP.LT.LINLEN ) THEN
         TOP=TOP+1
         ALPCON(TOP)=ITEM
      ELSE
         CALL ABORT ('STACK OVERFLOWN','PSHALP')
      ENDIF
      TOPALP=TOP
      RETURN
      END
CPSHNUM-----------------------------------------------------------------
      SUBROUTINE PSHNUM (ITEM)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: PUSHES NUMERIC ITEMS INTO THE
C                  NUMBERS-OF-LINE STACK BUILDED
C                  BY SUBROUTINE LEX.
C
C   %..   ARGUMENT INFORMATION:
C
C             ITEM =  A REAL NUMBER WHICH IS PUSHED INTO
C                     THE STACK WHICH HOLDS THE NUMERIC
C                     CONTENTS OF A LINE BEING LEXICALLY
C                     ANALYSED BY LEX.
C
C   %..   SUBPROGRAMS CALLED: ROUTINE ABORT.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C             NUMB (M), TTNUM (M).
C
C   %..   POSSIBLE ABORTS:
C
C             THE STACK MAY OVERFLOW. IN SUCH A CASE MCLCOM
C             STOPS. NORMALLY THIS SHOULD NEVER HAPPEN.
C
C   %..   NOTES:
C
C             THIS ROUTINE IS USED BY SUBROUTINE LEX.
C
C-----------------------------------------------------------------------
C
      INTEGER LINLEN
      PARAMETER (LINLEN=80)
      REAL NUMCON(1:LINLEN),ITEM
      COMMON /NUMB/ NUMCON
     >       /TTNUM/TOPNUM
      INTEGER TOP,TOPNUM
C
      TOP=TOPNUM
      IF ( TOP.LT.LINLEN ) THEN
         TOP=TOP+1
         NUMCON(TOP)=ITEM
      ELSE
         CALL ABORT ('STACK OVERFLOWN','PSHNUM')
      ENDIF
      TOPNUM=TOP
      RETURN
      END
CPSHSTR-----------------------------------------------------------------
      SUBROUTINE PSHSTR (ITEM)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: PUSHES ITEMS IN THE STRUCTURE-STACK
C                  BUILDED BY SUBROUTINE LEX.
C
C   %..   ARGUMENT INFORMATION:
C
C             ITEM = AN INTEGER CODE SPECIFYING THE TOKEN
C                    PUSHED IN THE STRUCTURE-OF-LINE-STACK.
C
C   %..   SUBPROGRAMS CALLED: ABORT ROUTINE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C             STRUC (M) ,  TTSTR (M) .
C
C   %..   POSSIBLE ABORTS:
C
C             THE STACK MAY OVERFLOW, IN WHICH CASE MCLCOM
C             STOPS.
C
C   %..   NOTES:
C
C             THIS ROUTINE IS USED BY SUBROUTINE LEX TO CONSTRUCT
C             THE STRUCTURE-OF-THE-LINE STACK OF A LINE.
C
C-----------------------------------------------------------------------
C
      INTEGER LINLEN
      PARAMETER (LINLEN=80)
      INTEGER CSTRUC(1:LINLEN+1),ITEM,TOP,TOPSTR
      COMMON /STRUC/ CSTRUC
     >       /TTSTR/TOPSTR
C
      TOP=TOPSTR
      IF(TOP.LT.LINLEN+1)THEN
         TOP=TOP+1
         CSTRUC(TOP)=ITEM
      ELSE
         CALL ABORT ('STACK OVERFLOWN','PSHSTR')
      ENDIF
      TOPSTR=TOP
      RETURN
      END
CPUSH-------------------------------------------------------------------
      SUBROUTINE PUSH ( ITEM )
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: PUSHES ITEMS INTO A STACK.
C
C   %..   ARGUMENT INFORMATION:
C
C             ITEM = THE ELEMENT PUSHED. INTEGER.
C
C   %..   SUBPROGRAMS CALLED: ABORT ROUTINE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C             EXPRST (M)
C
C   %..   POSSIBLE ABORTS:
C
C            THE STACK MAY OVERFLOW. HIGHLY IMPROPABLE.
C
C   %..   NOTES:
C
C             THIS ROUTINE IS USED IN SUBROUTINE PFIX.
C
C-----------------------------------------------------------------------
C
      INTEGER LINLEN
      PARAMETER ( LINLEN = 80 )
      INTEGER ITEM,TOP,STACK(1:LINLEN),TOPBUF
      COMMON / EXPRST / TOPBUF,STACK
C
      TOP=TOPBUF
      IF ( TOP.LT.LINLEN ) THEN
         TOP=TOP+1
         STACK(TOP)=ITEM
      ELSE
         CALL ABORT ('STACK OVERFLOWN','PUSH')
      END IF
      TOPBUF=TOP
      RETURN
      END
CPUSHBR-----------------------------------------------------------------
      SUBROUTINE PUSHBR ( ITEM )
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: PUSHES INTEGER POSITIONS INTO
C                  A STACK USED BY XCHECK ROUTINE.
C
C   %..   ARGUMENT INFORMATION:
C
C             ITEM = IS THE POSITION OF A BRAKET RETURNED
C                    TO CALLING ROUTINE XCHECK.
C
C
C   %..   SUBPROGRAMS CALLED: NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C             TSTBST (M).
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   NOTES:
C
C            THIS ROUTINE IS CALLED BY SUBROUTINE XCHECK.
C
C-----------------------------------------------------------------------
C
      INTEGER ITEM,TOPMAX,TOP,LINLEN
      PARAMETER (LINLEN=80, TOPMAX=LINLEN)
      INTEGER BSTACK(1:TOPMAX)
      LOGICAL OK
      COMMON / TSTBST / TOP,BSTACK,OK
C
      IF(TOP.LT.TOPMAX)THEN
         TOP=TOP+1
         BSTACK(TOP)=ITEM
         OK=.TRUE.
         RETURN
      ELSE
         OK=.FALSE.
         RETURN
      ENDIF
      END
CPUSHDN-----------------------------------------------------------------
      SUBROUTINE PUSHDN ( ITEM )
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: PUSHES ITEMS IN A STACK.
C
C   %..   ARGUMENT INFORMATION:
C
C             ITEM = THE PUSHED ELEMENT. INTEGER.
C
C   %..   SUBPROGRAMS CALLED: NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C             TSTXST (M)
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   NOTES:
C
C             THIS ROUTINE IS USED IN SUBROUTINE XCHECK.
C
C-----------------------------------------------------------------------
C
      INTEGER ITEM,TOPMAX,TOP,LINLEN
      PARAMETER (LINLEN=80, TOPMAX=LINLEN)
      INTEGER STACK(1:TOPMAX)
      LOGICAL SUCCES
      COMMON / TSTXST / TOP,STACK,SUCCES
C
      IF ( TOP.LT.TOPMAX ) THEN
         TOP = TOP + 1
         STACK(TOP) = ITEM
         SUCCES = .TRUE.
         RETURN
      ELSE
         SUCCES = .FALSE.
         RETURN
      END IF
      END
C  ---------------------------------------------------------------------
      LOGICAL FUNCTION RESE ( NAME )
C  ---------------------------------------------------------------------
C
C  CHECKS WHETHER AN IDENTIFIER CAN BE USED WITHIN A "VAR" OR "FUNCTION"
C  DECLARATION. THE VALUE RETURNED IS .TRUE. IF THE NAME BELONGS TO
C  THE RESERVED ONES; .FALSE. OTHERWISE.
C
C  INPUT ARGUMENTS:
C
C    NAME       CHARACTER VARIABLE. THIS IS THE NAME TO BE CHECKED.
C
C  ---------------------------------------------------------------------
C
C  NUMBER OF IDENTIFIERS THAT MAY NOT BE USED AS VARIABLES.
      PARAMETER ( NNUI = 5 )
C
      CHARACTER*(*) NAME
      CHARACTER*4  NUI(NNUI)
C
      DATA NUI / 'BY  ',  'JUST',  'THEN', 'TO  ','FROM' /
C
      RESE=.TRUE.
      DO 10,I=1,NNUI
        IF (NAME.EQ.NUI(I)) RETURN
 10   CONTINUE
C
      RESE=.FALSE.
      RETURN
      END
CREWI-------------------------------------------------------------------
      SUBROUTINE REWI (TOKENS,TOKN,TUSED,ALPHAC,NA)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: IT PARSES REWIND STATEMENT.
C
C   %..   ARGUMENT INFORMATION:
C
C      -TOKENS: IS THE LINE TO BE PROCESSED.
C      -TOKN  : IS THE NUMBER OF TOKENS "TOKENS" IS COMPOSED OF
C      -TUSED : POINTS TO THE LAST USED ELEMENT OF "TOKENS".
C      -ALPHAC: ARE THE IDENTIFIERS OF "TOKENS".
C      -NA    : POINTS TO THE LAST USED ELEMENT OF ALPHAC.
C
C   %..   SUBPROGRAMS CALLED:
C
C             SUBROUTINES: ERROR,
C             FUNCTIONS  : NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C             MCLUNT (A) , MCLDTA (M) , MCLSCD (A)
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS: NONE.
C
C   %..   NOTES:
C
C    SYNTAX :
C    <REWIND> : : =  'REWIND' '(' ['FILE' '='] <IDENT> ')'
C
C-----------------------------------------------------------------------
C
      IMPLICIT INTEGER ( A - Z)
      DIMENSION TOKENS(TOKN)
      LOGICAL GENE,BOUNDS,DEBUG
      CHARACTER*(*) ALPHAC(*),FILENM*30
      COMMON / MCLUNT / INP, ERL, BIN, SCR, ISTF, SPARZZ, DUMMYF,DESCRI
     >       / MCLDTA / NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
     >       / MCLSCD / LEFTP, RIGHTP, DOT, COMMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUMBER,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
C
C   START
C
      TP=TUSED+1
      IF(TP.EQ.TOKN)GOTO 100
      IF(TOKENS(TP).EQ.LEFTP)THEN
         TP=TP+1
         NA=NA+1
         IF(TP.EQ.TOKN)GOTO 100
         IF(ALPHAC(NA).EQ.'FILE'.AND.TOKENS(TP).EQ.IDENT)THEN
            TP=TP+1
            IF(TP.GE.TOKN)GOTO 100
            IF(TOKENS(TP).EQ.EQ)THEN
               TP = TP+1
               IF(TP.EQ.TOKN)GOTO 100
               IF(TOKENS(TP).EQ.IDENT)THEN
                  TP=TP+1
                  IF(TP.EQ.TOKN)GOTO 100
                  NA=NA+1
                  FILENM=ALPHAC(NA)
                  IF(TOKENS(TP).EQ.RIGHTP.AND.TP+1.EQ.TOKN)THEN
                     BLOC=BLOC+2
                     IF(GENE)WRITE(SCR,'(''REWIND'',/,A)')FILENM
                  ELSE
                     CALL ERROR (98,' ')
                  ENDIF
               ELSE
                  CALL ERROR (99,' ')
               ENDIF
            ELSE
               CALL ERROR (101,' ')
            ENDIF
         ELSE
            IF(TOKENS(TP).EQ.IDENT)THEN
               TP=TP+1
               IF(TP.EQ.TOKN)GOTO 100
               FILENM=ALPHAC(NA)
               IF(TOKENS(TP).EQ.RIGHTP.AND.TP+1.EQ.TOKN)THEN
                  BLOC=BLOC+2
                  IF(GENE)WRITE(SCR,'(''REWIND'',/,A)')FILENM
               ELSE
                  CALL ERROR (98,' ')
               ENDIF
            ELSE
               CALL ERROR (99,' ')
            ENDIF
         ENDIF
      ELSE
         CALL ERROR (102,' ')
      ENDIF
      RETURN
 100  CALL ERROR (100,' ')
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE SERLOC ( TABLE, ELEM, NAME, FOUND, POSIT )
C  ---------------------------------------------------------------------
C
C  LOCATES A NAME IN A CHARACTER TYPE ARRAY. A LINEAR SEARCH IS APPLIED
C  TO THE GIVEN ARRAY.
C
C  INPUT ARGUMENTS:
C
C    TABLE   THE CHARACTER ARRAY THAT WILL BE SEARCHED. IS ASSUMED
C            TO BE DIMENSIONED AS: TABLE(1:ELEM)
C    ELEM    NUMBER OF ELEMENTS IN THE ABOVE ARRAY.
C    NAME    THE NAME WHOS POSITION IN THE ARRAY WE ARE LOOKING FOR.
C
C  OUTPUT ARGUMENTS:
C
C    FOUND   LOGICAL VARIABLE INDICATING, WHETHER THE REQUESTED NAME
C            WAS (FOUND=.TRUE.), OR WAS NOT FOUND (FOUND=.FALSE.) IN
C            THE GIVEN ARRAY.
C    POSIT   IF THE NAME WAS FOUND IN THE ARRAY, THIS IS ITS POSITION;
C            OTHERWISE 0 IS RETURNED.
C
C  ---------------------------------------------------------------------
C
      CHARACTER*(*) TABLE(*),NAME
      LOGICAL FOUND
      INTEGER ELEM,POSIT
C
      FOUND=.TRUE.
      DO 10,POSIT=1,ELEM
        IF (TABLE(POSIT).EQ.NAME) RETURN
 10   CONTINUE
C
C  THE NAME WAS NOT FOUND IN THE GIVEN ARRAY.
C
      FOUND=.FALSE.
      POSIT=0
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE STAFU ( TOKEN, NTOK, ALPHA, VAR, DIM, LOC, IPOS, INFO,
     >                   KIND, NUM )
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE WILL HANDLE A STATEMENT FUNCTION DECLARATION.
C  THE DECLARATION HAS THE FORM:
C
C  "FUNCTION" <NAME> [ ARG1,ARG2,...ARGN ] = <EXPRESSION>
C
C  WHERE <NAME> IS THE SYMBOLIC NAME OF THE STATEMENT FUNCTION,
C  ARG1,ARG2,...ARGN IS A LIST OF DUMMY ARGUMENTS TO THE FUNCTION AND
C  <EXPRESSION> IS THE ACTUAL EXPRESSION THAT THIS FUNCTION WILL
C  CALCULATE. THE ARGUMENTS IN THE LIST ARE PUSHED IN THE STATEMENT
C  FUNCTION ARGUMENT TABLE, IN THEE ORDER THEY APPEAR IN THE LIST.
C  THE OBJECT CODE GENERATED BY THE <EXPRESSION> IS NOT ACTUALLY WRITTEN
C  IN THE SCRATCH FILE, BUT IN THE STATEMENT FUNCTION FILE ( UNIT STF )
C  INSTEAD.
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C               THE INPUT LINE.
C    NUM        REAL ARRAY. THE NUMBERS ENCOUNTERED IN THE INPUT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    VAR        CHARACTER ARRAY. THIS IS THE VAR TABLE.
C    DIM        INTEGER ARRAY. THIS IS THE DIMENSIONALITY TABLE.
C    LOC        INTEGER ARRAY. THIS IS THE MEMORY LOCATION TABLE.
C    IPOS       INTEGER ARRAY. THIS IS THE POSITION TABLE.
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    KIND       INTEGER ARRAY. THIS IS THE KIND TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES ERROR, LOCAN, SERLOC, EXTRA, XPR, INSERT.
C    FUNCTION    RESE.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER (  IDLEN = 30   )
C
C  THE LENGTH OF A LINE. (CHARS)
      PARAMETER (  LINLEN = 80  )
C
C  MAXIMUM NUMBER OF ARGUMENTS IN A STATEMENT FUNCTION.
      PARAMETER (  MXARG =(LINLEN-13)/2  )
C
      CHARACTER*(*) ALPHA(*),VAR(*)
      INTEGER DIM(*),LOC(*),IPOS(*),INFO(*),KIND(*),TOKEN(*)
      REAL NUM(*)
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      INTEGER EX(LINLEN)
      CHARACTER*(IDLEN) STAFID,VARID,STARG(MXARG)
      LOGICAL CORRE,FOUND,RESE
C
C  INITIALIZE POINTERS TO THE TOKEN, ALPHA AND NUM ARRAYS RESPECTIVELY.
C
      NT=1
      NA=1
      NN=0
C
C  AN IDENTIFIER IS EXPECTED AS THE FUNCTION NAME.
C
      NT=NT+1
      IF (TOKEN(NT).NE.IDENT) THEN
         CALL ERROR(60,' ')
         RETURN
      END IF
C
C  GET THE FUNCTION NAME.
C
      NA=NA+1
      STAFID=ALPHA(NA)
C
C  CHECK IF THE FUNCTION NAME SHOULD NOT BE USED.
C
      IF (RESE(STAFID)) CALL ERROR(12,STAFID)
C
C  CHECK IF THE FUNCTION NAME IS ALREADY PRESENT IN THE VAR TABLE.
C
      CALL LOCAN(VAR,NVAR,STAFID,FOUND,IND)
      IF (FOUND) THEN
         CALL ERROR(65,STAFID)
         RETURN
      END IF
C
C  A LEFT BRACKET SHOULD FOLLOW THE FUNCTION NAME.
C
      NT=NT+1
      IF (TOKEN(NT).NE.LBRA) THEN
         CALL ERROR(61,' ')
         RETURN
      END IF
C  NFARG COUNTS THE NUMBER OF ARGUMENTS IN THE LIST.
      NFARG=0
C
C  START A LOOP OVER ALL FUNCTION ARGUMENTS.
C
 100  NT=NT+1
      IF (TOKEN(NT).NE.IDENT) THEN
         CALL ERROR(62,' ')
         RETURN
      END IF
C  GET THE ARGUMENT NAME.
      NA=NA+1
      VARID=ALPHA(NA)
C  CHECK IF ALREADY PRESENT IN THE ARGUMENT LIST.
      CALL SERLOC(STARG,NFARG,VARID,FOUND,II)
      IF (FOUND) THEN
         CALL ERROR(66,VARID)
         RETURN
      END IF
C  PUSH THE ARGUMENT IN THE TABLE.
      NFARG=NFARG+1
      STARG(NFARG)=VARID
C
      NT=NT+1
      CUR=TOKEN(NT)
C  A COMMA DENOTES ANOTHER ARGUMENT IN THE LIST.
      IF (CUR.EQ.COMMA) GOTO 100
C  A RIGHT BRACKET DENOTES THE END OF THE ARGUMENT LIST.
      IF (CUR.EQ.RBRA) GOTO 200
C  NEITHER COMMA, NOR RIGHT BRACKET FOUND.
      CALL ERROR(63,' ')
      RETURN
C
C  AN = SIGN SHOULD FOLLOW THE RIGHT BRACKET.
C
 200  NT=NT+1
      IF (TOKEN(NT).NE.EQ) THEN
         CALL ERROR(64,' ')
         RETURN
      END IF
C
C  PUSH INITIAL LOCATION OF THE STATEMENT FUNCTION FILE IN THE
C  INFORMATION TABLE.
C
      IF (NINF+2.GT.MAXIN)
     >           CALL OVER('INFORMATION','INFO','STAFU',MAXIN,'MXINFO')
      NINF=NINF+1
      INFO(NINF)=NSTL+1
C  EXTRACT THE <EXPRESSION>.
      CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,0,ALPHA,NA)
      ISTF=STF
      INSTL=NSTL
C  CALCULATE <EXPRESSION>. THE OBJECT CODE WILL BE WRITTEN
C  IN THE STATEMENT FUNCTION FILE. ( UNIT STF )
      CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.TRUE.,INSTL,LOC,DIM,
     >         INFO,IPOS,KIND,ISTF,STARG)
C  COUNT NUMBER OF LINES IN THE STATEMENT FUNCTION FILE.
      NSTL=NSTL+INSTL
C  IF <EXPRESSION> INCORRECT.
      IF (.NOT.CORRE) RETURN
C
C  PUSH CURRENT ( AFTER CODE FOR THE <EXPRESSION> HAS BEEN WRITTEN )
C  LOCATION OF THE STATEMENT FUNCTION FILE IN THE INFORMATION TABLE.
C
      NINF=NINF+1
      INFO(NINF)=NSTL
C  INSERT INFORMATION ABOUT THIS STATEMENT FUNCTION IN THE
C  ASSOCIATED TABLES.
      CALL INSERT(VAR,DIM,LOC,IPOS,KIND,STAFID,NFARG,MEMLOC+FMEM,NINF-1,
     >            3,IND)
C  RESERVE STORAGE FOR THE ARGUMENTS OF THIS STATEMENT FUNCTION.
      FMEM=FMEM+NFARG
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE STEPOF ( TOKEN,NTOK,NT,ALPHA,NA,NUM,NN,VAR,ID,LOC,
     >                    DIM,INFO,IPOS,KIND )
C  ---------------------------------------------------------------------
C
C  HANDLES THE STEP AND POINT STATEMENTS. THEIR SYNTAX IS:
C
C  <COMMAND> ( <K>.EXPR1=VAL1; <K>.EXPR2=VAL2;... <K>.EXPRN=VALN )
C
C  WHERE <COMMAND> IS EITHER STEP OR POINT AND
C  EXPR1,EXPR2,...EXPRN ARE EXPRESSIONS.
C  VAL1,VAL2,...VALN ARE ALSO EXPRESSIONS. THEIR VALUES WILL BE ASSIGNED
C  TO THE POINT VARIABLES INDEXED BY EXPR1,EXPR2,...EXPRN
C  CORRESPONDINGLY.
C  <K> IS A KEYWORD. ITS VALUE IS EITHER X ( IN CASE OF THE POINT
C  STATEMENT ) OR S. ( IN CASE OF THE STEP STATEMENT )
C  THE OBJECT CODE GENERATED IS AS:
C
C  <CODE TO EVALUATE EXPR1>
C  <CODE TO EVALUATE VAL1>
C  <CODE TO EVALUATE EXPR2>
C  <CODE TO EVALUATE VAL2>
C  ...
C  <CODE TO EVALUATE EXPRN>
C  <CODE TO EVALUATE VALN>
C  <WCOM>
C  $<NC>
C  $SWAP
C
C  WHERE <CODE TO EVALUATE EXPRN> IS THE OBJECT CODE, NECESSARY FOR
C  THE EXPRN EVALUATION,
C  <CODE TO EVALUATE VALN> IS THE OBJECT CODE, NECESSARY FOR THE VALN
C  EVALUATION,
C  <WCOM> IS EITHER $WPOINT ( IN CASE OF THE POINT STATEMENT ) OR
C  $WSTEP ( IN CASE OF THE STEP STATEMENT ) AND
C  <NC> IS THE NUMBER OF "<K>.EXPR=VAL" ENTRIES IN THE STATEMENT.
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C    NUM        REAL ARRAY. THE NUMBERS ENCOUNTERED IN THE INPUT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C               THE INPUT LINE.
C    ID         INTEGER VARIABLE. AN IDENTIFICATION CODE FOR EACH
C               ONE OF THE COMMANDS THIS ROUTINE CAN HANDLE.
C               ITS VALUE AND THE CORRESPONDING COMMANDS ARE AS:
C                 1  STEP
C                 2  POINT
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    NT         INTEGER VARIABLE. THIS IS A POINTER TO THE TOKEN ARRAY
C               AND IS THE NUMBER OF TOKENS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    NA         INTEGER VARIABLE. THIS IS A POINTER TO THE ALPHA ARRAY
C               AND IS THE NUMBER OF IDENTIFIERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    NN         INTEGER VARIABLE. THIS IS A POINTER TO THE NUM ARRAY
C               AND IS THE NUMBER OF NUMBERS USED SO FAR BY THE
C               PREVIOUSLY CALLED PARSING ROUTINES.
C    VAR        CHARACTER ARRAY. THIS IS THE VAR TABLE.
C    LOC        INTEGER ARRAY. THIS IS THE MEMORY LOCATION TABLE.
C    DIM        INTEGER ARRAY. THIS IS THE DIMENSIONALITY TABLE.
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    IPOS       INTEGER ARRAY. THIS IS THE POSITION TABLE.
C    KIND       INTEGER ARRAY. THIS IS THE KIND TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES ERROR, EXTRA, XPR.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER (  IDLEN = 30   )
C
C  THE LENGTH OF A LINE. (CHARS)
      PARAMETER (  LINLEN = 80  )
C
      INTEGER TOKEN(*),LOC(*),DIM(*),INFO(*),IPOS(*),KIND(*)
      CHARACTER*(*) ALPHA(*),VAR(*)
      REAL NUM(*)
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      CHARACTER*(IDLEN) KEY(3),STARG(1),K
      CHARACTER*5 WCOM(3)
      LOGICAL CORRE
      INTEGER EX(LINLEN)
C
      DATA WCOM / 'POINT', 'STEP ', 'FLAG' /
      DATA KEY  / 'X', 'S', 'F' /
C
C  A LEFT PARENTHESIS IS EXPECTED AFTER THE COMMAND NAME.
C
      NT=NT+1
      IF (TOKEN(NT).NE.LEFTP) THEN
         CALL ERROR(31,' ')
         RETURN
      END IF
C  NC COUNTS THE NUMBER OF KEYWORDS ENTERED IN THE STATEMENT.
      NC=0
C
C  LOOP OVER ALL "<K>.EXPR=VAL" ENTRIES IN THE STATEMENT.
C
 1000 CONTINUE
C  AN IDENTIFIER IS EXPECTED AS A KEYWORD.
      NT=NT+1
      IF (TOKEN(NT).NE.IDENT) THEN
         CALL ERROR(32,' ')
         RETURN
      END IF
C  GET THE KEYWORD.
      NA=NA+1
      K=ALPHA(NA)
C  SEE IF THE KEYWORD IS VALID FOR THE COMMAND.
      IF (K.NE.KEY(ID)) THEN
         CALL ERROR(33,K)
         RETURN
      END IF
C  A PERIOD IS EXPECTED AFTER THE KEYWORD.
      NT=NT+1
      IF (TOKEN(NT).NE.PERIOD) THEN
         CALL ERROR(34,' ')
         RETURN
      END IF
C  EXTRACT THE EXPRESSION FOLLOWING THE PERIOD.
      CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,2,ALPHA,NA)
      CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,BLOC,LOC,DIM,
     >         INFO,IPOS,KIND,SCR,STARG)
C  IF EXPRESSION INCORRECT.
      IF (.NOT.CORRE) RETURN
C  AN = SIGN SHOULD FOLLOW THE EXPRESSION.
      NT=NT+1
      IF (TOKEN(NT).NE.EQ) THEN
         CALL ERROR(35,' ')
         RETURN
      END IF
C  EXTRACT THE EXPRESSION FOLLOWING THE = SIGN.
      CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,2,ALPHA,NA)
      CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,BLOC,LOC,DIM,
     >         INFO,IPOS,KIND,SCR,STARG)
C  IF EXPRESSION INCORRECT.
      IF (.NOT.CORRE) RETURN
C  GET THE NEXT TOKEN.
      NT=NT+1
      CUR=TOKEN(NT)
      NC=NC+1
C  IF A SEMICOLON, THEN ANOTHER KEYWORD FOLLOWS.
      IF (CUR.EQ.SEMI) GOTO 1000
      IF (CUR.EQ.RIGHTP) THEN
C  IF A RIGHT PARENHESIS, THEN THE STATEMENT HAS BEEN COMPLETED.
C  WRITE THE FINAL OBJECT CODE LINES.
         BLOC=BLOC+3
         IF (GENE) WRITE (SCR,'(''$PUSHC''/I3/A)') NC, WCOM(ID)
      ELSE
C  NEITHER SEMICOLON NOR A RIGHT PARENTHESIS FOLLOW.
         CALL ERROR(36,' ')
      END IF
C
      RETURN
      END
CSUBX-------------------------------------------------------------------
      SUBROUTINE SUBX (SIMPX , TOKN , CORR)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: CHECKS SUBEXPRESSIONS.
C
C   %..   ARGUMENT INFORMATION:
C
C    -SIMPX : THE INPUT SIMPLE EXPRESSION TO BE CHECKED.
C    -TOKN  : THE NUMBER OF TOKENS IN SIMPEX.
C    -CORR  : LOGICAL VARIABLE BEING TRUE IF SIMPEX CORRECT
C             UPON RETURN.
C
C   %..   SUBPROGRAMS CALLED:
C
C     SUBROUTINES: ERROR,
C     FUNCTIONS  : NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C          MCLSCD (A)
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS: NONE.
C
C   %..   NOTES:
C
C         THIS ROUTINE PERFORMS CHECKING OF SUBEXPRESSIONS (OR SIMPLE
C         EXPRESSIONS).A SIMPLE EXPRESSION IS DEFINED AS SHOWN :
C
C         <SIMPEX>  : : =  <OPERAND> [<OPERATOR><OPERAND>]
C
C         CLEARLY CORRESPONDS TO THE INNEST PART OF AN EXPRESSION INTO
C         PARENTHESES AND BRAKETS.
C
C-----------------------------------------------------------------------
C
      IMPLICIT INTEGER ( A - Z )
      LOGICAL CORR, OPERAN, OPRTOR
      DIMENSION SIMPX(TOKN)
      PARAMETER ( ONE=1000, ZERO=-1000 )
C
      COMMON / MCLSCD / LEFTP, RIGHTP, DOT, COMMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUMBER,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
C
      OPERAN(TEMPO) = (TEMPO.EQ.IDENT).OR.(TEMPO.EQ.NUMBER).OR.
     >                (TEMPO.EQ.ZERO).OR.(TEMPO.EQ.ONE)
      OPRTOR(TEMPO) =( TEMPO.EQ.PLUS).OR.(TEMPO.EQ.MINUS).OR.
     >               ( TEMPO.EQ.STAR).OR.(TEMPO.EQ.BISTAR).OR.
     >               ( TEMPO.EQ.SLASH).OR.(TEMPO.EQ.AND).OR.
     >               ( TEMPO.EQ.OR).OR.(TEMPO.EQ.LT).OR.
     >               ( TEMPO.EQ.LE).OR.(TEMPO.EQ.GT).OR.
     >               ( TEMPO.EQ.GE).OR.(TEMPO.EQ.EQOPER).OR.
     >               ( TEMPO.EQ.NE).OR.(TEMPO.EQ.XOR).OR.
     >               ( TEMPO.EQ.NOT)
C
C  START
C
      IF(.NOT.OPERAN(SIMPX(1))) THEN
         CORR=.FALSE.
         CALL ERROR (43,' ')
         RETURN
      ENDIF
C
      IF (MOD(TOKN,2).EQ.0) THEN
         CORR = .FALSE.
         CALL ERROR (44,' ')
         RETURN
      ENDIF
C
      DO 1000,I=2,TOKN,2
         IF (  .NOT.( OPRTOR(SIMPX(I))
     >         .AND.(SIMPX(I+1).EQ.IDENT.OR.SIMPX(I+1).EQ.NUMBER
     >               .OR.SIMPX(I+1).EQ.ONE.OR.SIMPX(I+1).EQ.ZERO))
     >      )  THEN
            CORR = .FALSE.
            CALL ERROR (45,' ')
            RETURN
         ENDIF
 1000 CONTINUE
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE TERMIN ( TYPE,TDEF )
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE WILL CHECK FOR ANY BLOCK IFS, OR LOOPS THAT DO NOT
C  TERMINATE. ( "ENDIF" OR "ENDLOOP" ABSENT )
C  AN "ENDIF" STATEMENT CAUSES THE ENTRIES FOR THE CORRESPONDING "IF"
C  AND "ELSE" STATEMENT IN THE TYPE TABLE, TO BECOME NEGATIVE.
C  ( -1 OR -2 ) AN "ENDLOOP" STATEMENT CAUSES THE ENTRY FOR THE
C  CORRESPONDING "LOOP" IN THE TYPE TABLE TO BECOME ALSO NEGATIVE. (-4)
C  THUS, WHEN THE FIRST PASS OVER THE MCL SOURCE HAS BEEN COMPLETED,
C  A VALUE OF 1 OR 2, INDICATES A BLOCK IF THAT DOES NOT TERMINATE,
C  WHILE A VALUE OF 4 INDICATES A LOOP THAT DOES NOT TERMINATE.
C  THE TYPE TABLE IS SEARCHED BACKWARDS AND IF A VALUE OF 1 OR 2 IS
C  FOUND A FALSE "ENDIF" IS PUSHED IN THE TYPE TABLE. THE ABOVE TWO
C  VALUES BECOME NEGATIVE. IF A 4 IS REACHED, A FALSE "ENDLOOP" IS
C  PUSHED IN THE TYPE TABLE, AND THE PREVIOUS VALUE BECOMES ALSO
C  NEGATIVE. IN BOTH CASES THE LOCATION PUSHED IN THE TYPE-DEFINITION
C  TABLE, IS THE CURRENT LOCATION COUNTER OF THE SCRATCH FILE. SINCE
C  THE FIRST PASS HAS BEEN COMPLETED THIS IS THE LAST LINE OF OBJECT
C  CODE WRITTEN. ( NOTE THAT NO MORE OBJECT CODE WILL BE WRITTEN IN THE
C  SCRATCH FILE ) FALSE "ENDIF" AND "ENDLOOP" STATEMENTS ARE EFFECTIVELY
C  "APPENDED" AT THE END OF THE MCL SOURCE.
C  IF ANY NON-TERMINATING BLOCK IFS OR LOOPS ARE FOUND, APPROPRIATE
C  ERROR MESSAGES ARE ISSUED.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES ERROR, OVER.
C
C  ABORT CONDITIONS THAT MAY OCCUR:
C    IF THE TYPE TABLE OVERFLOWS, THIS WILL CAUSE THE PROGRAM TO
C    STOP AND ISSUE AN INFORMATIVE MESSAGE.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      INTEGER TYPE(*),TDEF(*)
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      LOGICAL NOLOOP,NOEND
C  NOLOOP INDICATES WHETHER A NON-TERMINATING LOOP WAS FOUND.
C  NOEND  INDICATES WHETHER A NON-TERMINATING BLOCK IF WAS FOUND.
      NOLOOP = .FALSE.
      NOEND  = .FALSE.
      NTP    =  NTYPE
C
C  SEARCH THE TYPE TABLE BACKWARDS.
C
      DO 10,I=NTP,1,-1
        TP=TYPE(I)
        IF (TP.EQ.1 .OR. TP.EQ.2) THEN
C  A BLOCK IF THAT DOES NOT TERMINATE WAS JUST FOUND.
           NOEND = .TRUE.
           BLOC  = BLOC+1
C  PUSH A FALSE "ENDIF" IN THE TYPE TABLE.
           NTYPE = NTYPE+1
           IF (NTYPE.GT.MAXTYP)
     >              CALL OVER('TYPE','TYPE','TERMIN',MAXTYP,'MXTYP')
           TYPE(NTYPE) = -3
           TDEF(NTYPE) = BLOC
           TYPE(I)     = -TP
        ELSE IF (TP.EQ.4) THEN
C  A LOOP THAT DOES NOT TERMINATE HAS JUST BEEN FOUND.
           NOLOOP = .TRUE.
           BLOC   = BLOC+1
C  PUSH A FALSE "ENDLOOP" IN THE TYPE TABLE.
           NTYPE  = NTYPE+1
           IF (NTYPE.GT.MAXTYP)
     >              CALL OVER('TYPE','TYPE','TERMIN',MAXTYP,'MXTYP')
           TYPE(NTYPE) = -5
           TDEF(NTYPE) = BLOC
           TYPE(I)     = -TP
        END IF
 10   CONTINUE
C
C  IF ANYTHING WENT WRONG, ISSUE INFORMATIVE MESSAGES.
C
      IF ( NOLOOP) CALL ERROR(146,' ')
      IF ( NOEND ) CALL ERROR(147,' ')
C
      RETURN
      END
      CHARACTER FUNCTION UPPER ( CHA )
      CHARACTER CHA
C
      ICODE = ICHAR(CHA)
      IF (ICODE.GE.97 .AND. ICODE.LE.122) THEN
         UPPER = CHAR(ICODE-32)
      ELSE
         UPPER = CHA
      END IF
      END
C  ---------------------------------------------------------------------
      SUBROUTINE VALIJ ( TYPE,TDEF,TREL )
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE WILL CHECK THE VALIDITY OF THE "MOVE TO" STATEMENTS
C  IN THE PROGRAM, IN RELATION WITH THE BLOCK IFS AND LOOPS.
C  A BLOCK IS DEFINED TO BE A SEQUENCE OF STATEMENTS RESIDING BETWEEN:
C  IF ... ENDIF,
C  IF ... ELSE,
C  ELSE ... ENDIF AND
C  LOOP ... ENDLOOP.
C  A JUMP INSIDE A BLOCK IS NOT ALLOWED, UNLESS THE CORRESPONDING
C  "MOVE TO" STATEMENT, BELONGS TO THE BLOCK.
C  JUMPS OUTSIDE A BLOCK, ARE ALWAYS ALLOWED.
C  WHEN THIS ROUTINE IS ENTERED, IT IS ASSUMED THAT BLOCK IFS
C  AND LOOPS ARE PROPERLY NESTED AND PROPERLY TERMINATED.
C  IT IS ALSO ASSUMED THAT ALL "MOVE TO" STATEMENTS HAVE BEEN PROCESSED.
C  ( LINKED WITH THE CORRESPONDING LABELS ) THUS ONLY NEGATIVE ENTRIES
C  OCCUR IN THE TYPE TABLE.
C  THE ROUTINE WILL START SEARCHING THE TYPE TABLE. FOR EACH BLOCK
C  ENCOUNTERED IT WILL ASSIGN A VALUE OF 7 FOR ALL "MOVE TO" STATEMENTS
C  THAT BELONG TO THE BLOCK AS WELL AS THEIR TARGET LABEL.
C  THEN IT WILL CHECK THE VALIDITY OF THE REST OF THE "MOVE TO"
C  STATEMENTS, BELONGING TO THE BLOCK. UPON EXIT OF THE ROUTINE
C  ALL ENTRIES IN THE TYPE TABLE THAT INDICATE AN "IF", "ELSE",
C  "ENDIF", "LOOP", "ENDLOOP" OR "MOVE TO" STATEMENT WILL YIELD
C  POSITIVE VALUES: 1, 2, 3, 4, 5 AND 7 RESPECTIVELY.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C    TREL       INTEGER ARRAY. THIS IS THE TYPE-LINK TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES LOCIF, LOCLOO, ZEROM, JUMPIN.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
      INTEGER TYPE(*),TDEF(*),TREL(*)
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      INTEGER ELSE,END,ENDLOO
C
C  START SEARCHING THE TYPE TABLE.
C
      DO 900,I=1,NTYPE
        IF (TYPE(I).EQ.-3) THEN
C  AN "ENDIF" WAS FOUND.
           END=I
C  LOCATE THE CORRESPONDING "IF" AND "ELSE" IN THE TYPE TABLE.
C  0 WILL BE RETURNED IN ELSE IF NO "ELSE" IS DEFINED
C  FOR THIS BLOCK IF.
           CALL LOCIF(IF1,ELSE,END,TYPE)
C  INDICATE THAT THESE "IF" AND "ENDIF" STATEMENTS HAVE BEEN PROCESSED.
           TYPE(IF1) = 1
           TYPE(END) = 3
           IF (ELSE.EQ.0) THEN
C  THIS IS A BLOCK IF WITH NO "ELSE" PART.
C  JUMPS FROM THIS BLOCK, TO THIS BLOCK ARE ALLOWED.
              CALL ZEROM (IF1,END,TYPE,TDEF,TREL)
C  JUMPS INTO THE BLOCK CAUSED BY "MOVE TO" STATEMENTS BEFORE THE
C  BLOCK IF ARE NOT ALLOWED.
              CALL JUMPIN(1,IF1-1,IF1,END,TYPE,TDEF,TREL)
C  JUMPS INTO THE BLOCK CAUSED BY "MOVE TO" STATEMENTS AFTER THE
C  BLOCK IF ARE NOT ALLOWED.
              CALL JUMPIN(IF1+1,NTYPE,IF1,END,TYPE,TDEF,TREL)
           ELSE
C  THIS IS A BLOCK IF WITH AN "ELSE" PART.
C  INDICATE THAT THIS ELSE HAS BEEN PROCESSED.
              TYPE(ELSE)=2
C  JUMPS FROM THE IF...ELSE BLOCK INTO THE SAME BLOCK, ARE ALLOWED.
              CALL ZEROM (IF1,ELSE,TYPE,TDEF,TREL)
C  JUMPS FROM THE ELSE...ENDIF BLOCK INTO THE SAME BLOCK, ARE ALLOWED.
              CALL ZEROM (ELSE,END,TYPE,TDEF,TREL)
C  JUMPS INTO THE IF...ENDIF BLOCK CAUSED BY "MOVE TO" STATEMENTS
C  BEFORE THE BLOCK IF ARE NOT ALLOWED.
              CALL JUMPIN(1,IF1-1,IF1,END,TYPE,TDEF,TREL)
C  JUMPS INTO THE IF...ENDIF BLOCK CAUSED BY "MOVE TO" STATEMENTS
C  AFTER THE BLOCK IF ARE NOT ALLOWED.
              CALL JUMPIN(END+1,NTYPE,IF1,END,TYPE,TDEF,TREL)
C  JUMPS INTO THE ELSE...ENDIF BLOCK CAUSED BY "MOVE TO" STATEMENTS
C  RESIDING IN THE IF...ELSE BLOCK ARE NOT ALLOWED.
              CALL JUMPIN(IF1+1,ELSE-1,ELSE,END,TYPE,TDEF,TREL)
C  JUMPS INTO THE IF...ELSE BLOCK CAUSED BY "MOVE TO" STATEMENTS
C  RESIDING IN THE ELSE...ENDIF BLOCK ARE NOT ALLOWED.
              CALL JUMPIN(ELSE+1,END-1,IF1,ELSE,TYPE,TDEF,TREL)
           END IF
        ELSE IF (TYPE(I).EQ.-5) THEN
C  AN "ENDLOOP" WAS FOUND.
           ENDLOO=I
C  LOCATE THE CORRESPONDING "LOOP" IN THE TYPE TABLE.
           CALL LOCLOO(LOO,ENDLOO,TYPE)
C  JUMPS INTO THE LOOP BY "MOVE TO" STATEMENTS RESIDING IN THE LOOP
C  ARE ALLOWED.
           CALL ZEROM (LOO,ENDLOO,TYPE,TDEF,TREL)
C  JUMPS INTO THE LOOP...ENDLOOP BLOCK CAUSED BY "MOVE TO" STATEMENTS
C  RESIDING BEFORE THE BLOCK ARE NOT ALLOWED.
           CALL JUMPIN(1,LOO-1,LOO,ENDLOO,TYPE,TDEF,TREL)
C  JUMPS INTO THE LOOP...ENDLOOP BLOCK CAUSED BY "MOVE TO" STATEMENTS
C  RESIDING AFTER THE BLOCK ARE NOT ALLOWED.
           CALL JUMPIN(ENDLOO+1,NTYPE,LOO,ENDLOO,TYPE,TDEF,TREL)
C  INDICATE THAT THE "LOOP" AND "ENDLOOP" HAVE BEEN PROCESSED.
           TYPE(LOO)    = 4
           TYPE(ENDLOO) = 5
        END IF
 900  CONTINUE
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE VARIA (TOKEN,NTOK,ALPHA,VAR,DIM,LOC,IPOS,INFO,KIND,NUM)
C  ---------------------------------------------------------------------
C
C  PROCESSES THE "VAR" DECLARATION STATEMENT. RESERVED NAMES, AS WELL
C  AS NAMES ALREADY PRESENT IN THE VAR TABLE, SHOULD NOT BE DECLARED.
C  THE ROUTINE CHECKS FOR SYNTACTIC VALIDITY AND UPDATES THE RELATED
C  TABLES. THESE ARE:
C  THE VAR TABLE,
C  THE MEMORY LOCATION TABLE,
C  THE DIMENSIONALITY TABLE,
C  THE INFORMATION TABLE,
C  THE POSITION TABLE AND
C  THE KIND TABLE.
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C    NUM        REAL ARRAY. THE NUMBERS ENCOUNTERED IN THE INPUT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C               THE INPUT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    VAR        CHARACTER ARRAY. THIS IS THE VAR TABLE.
C    LOC        INTEGER ARRAY. THIS IS THE MEMORY LOCATION TABLE.
C    DIM        INTEGER ARRAY. THIS IS THE DIMENSIONALITY TABLE.
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    IPOS       INTEGER ARRAY. THIS IS THE POSITION TABLE.
C    KIND       INTEGER ARRAY. THIS IS THE KIND TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES LOCAN, CHARR, INSERT, ERROR.
C    FUNCTIONS   RESE, MEMSPC.
C
C  NOTES:
C    IF A SYNTACTIC ERROR IS ENCOUNTERED WHILE A SIMPLE VARIABLE IS
C    PROCESSED, THE ROUTINE WILL CONTINUE PARSING.
C    IF AN ERROR IS ENCOUNTERED WHILE AN ARRAY IS PROCESSED, THE
C    ROUTINE WILL RETURN, SINCE IT HAS NO MEANS TO DETERMINE IN WHICH
C    POINT OF THE INPUT LINE, THE NEXT VARIABLE DECLARATION STARTS.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER (  IDLEN = 30   )
C
      INTEGER TOKEN(*),DIM(*),LOC(*),IPOS(*),INFO(*),KIND(*)
      REAL NUM(*)
      CHARACTER*(*) ALPHA(*),VAR(*)
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      CHARACTER*(IDLEN) VARID
      LOGICAL FOUND,ERR,ARR,RESE
C
C  INITIALIZE POINTERS TO THE TOKEN, ALPHA AND NUM ARRAYS RESPECTIVELY.
C
      NT=1
      NA=1
      NN=0
C
C  START A LOOP FOR ALL VARIABLES DECLARED IN THIS LINE.
C  THE NEXT TOKEN ENCOUNTERED MUST BE AN IDENTIFIER.
C
 40   NT=NT+1
      IF (TOKEN(NT).NE.IDENT) THEN
         CALL ERROR(3,' ')
         RETURN
      END IF
C
C  GET THE NAME OF THE VARIABLE.
C
      NA=NA+1
      VARID=ALPHA(NA)
C
C  CHECK IF IT SHOULD NOT BE USED. ( IF IT'S RESERVED )
      IF (RESE(VARID)) CALL ERROR(12,VARID)
C  ININF IS THE NEXT AVAILABLE POSITION IN THE INFORMATION TABLE.
      ININF=NINF+1
C  LOCATE THE VARIABLE IN THE VAR TABLE.
      CALL LOCAN(VAR,NVAR,VARID,FOUND,IND)
C  CHECK IF THIS IS AN ARRAY DECLARATION. IF YES ITS DIMENSIONALITY
C  IS RETURNED.
      CALL CHARR(TOKEN,NT,NDIM,ERR,ARR,INFO,NUM,NN)
C
      IF (FOUND) THEN
C  IF THE NAME IS ALREADY IN THE TABLE, THIS IS AN ERROR.
         CALL ERROR(4,VARID)
C  HOWEVER, IF ITS A SIMPLE VARIABLE, DON'T RETURN, BUT CHECK THE
C  REST OF THE LINE AS WELL.
         IF (ARR .AND. ERR) RETURN
      ELSE
C
C  THE NAME WAS NOT FOUND IN THE TABLE.
         IF (ARR) THEN
C  IF ITS AN ARRAY, CALCULATE TOTAL MEMORY SPACE OCCUPIED.
            IF (ERR) RETURN
            SPACE=MEMSPC(INFO,ININF,NDIM)
            K=2
         ELSE
            NDIM=0
            SPACE=1
            ININF=0
            K=1
         END IF
C
C  INSERT INFORMATION IN THE RELATED TABLES.
C  FOR THIS VARIABLE ( OR ARRAY ) :
C  NDIM IS ITS DIMENSIONALITY,
C  MEM IS THE MEMORY LOCATION TO BE ASSIGNED TO IT,
C  ININF IS THE STARTING POSITION IN THE INFORMATION TABLE,
C  K IS ITS KIND AND
C  IND IS THE POSITION WHERE ALL THIS INFORMATION SHOULD BE INSERTED.
C
         MEM=MEMLOC+1
         CALL INSERT(VAR,DIM,LOC,IPOS,KIND,VARID,NDIM,MEM,ININF,K,IND)
C  UPDATE THE MEMORY COUNTER.
         MEMLOC=MEMLOC+SPACE
      END IF
C
C  CHECK WHETHER ANOTHER VARIABLE IS DECLARED. IF NOT, JUST RETURN.
      NT=NT+1
      IF (NT.EQ.NTOK) RETURN
      IF (TOKEN(NT).EQ.SEMI) GOTO 40
C  NEITHER SEMICOLON ( DENOTING ANOTHER VARIABLE ), NOR EOL WAS FOUND.
      CALL ERROR(5,' ')
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE WHEN ( TOKEN,NTOK,ALPHA,NUM,TYPE,TDEF,TREL,VAR,MVELAB,
     >                  LOC,DIM,INFO,IPOS,KIND,LLOC )
C  ---------------------------------------------------------------------
C
C  PROCESSES THE "WHEN" STATEMENT. ITS SYNTAX IS AS:
C  "WHEN" <MCL EXPRESSION> "JUST" <NCNLCS>
C  WHERE <MCL EXPRESSION> IS ANY VALID MCL EXPRESSION AND
C  <NCNLCS> IS ANY NON CONDITIONAL AND NON LOOP CONTROL STATEMENT.
C  THE OBJECT CODE GENERATED IS AS:
C
C  ( CODE TO EVALUATE THE <MCL EXPRESSION> )
C  $NOT
C  $TESTMOVE
C  <BLANK LINE>
C
C  IF THE SYNTAX UP TO THE "JUST" KEYWORD IS CORRECT, A VALUE OF
C  -6 IS PUSHED TO THE TYPE TABLE, TO INDICATE THE "WHEN" STATEMENT.
C  SUBROUTINE EXE IS CALLED THEN TO PROCESS THE <NCNLCS> STATEMENT.
C  UPON RETURN OF THE ABOVE ROUTINE, THE NUMBER OF OBJECT CODE LINES,
C  THAT IT HAS WRITTEN, IS PUSHED IN THE TYPE-LINK TABLE, IN THE
C  POSITION CORRESPONDING TO THIS "WHEN".
C
C  INPUT ARGUMENTS:
C
C    TOKEN      INTEGER ARRAY. THESE ARE THE TOKENS IN THE CURRENT LINE.
C    NTOK       NUMBER OF TOKENS IN THE CURRENT LINE.
C    ALPHA      CHARACTER ARRAY.  THE IDENTIFIERS ENCOUNTERED IN
C               THE INPUT LINE.
C    NUM        REAL ARRAY. THE NUMBERS ENCOUNTERED IN THE INPUT LINE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C    TREL       INTEGER ARRAY. THIS IS THE TYPE-LINK TABLE.
C    VAR        CHARACTER ARRAY. THIS IS THE VAR TABLE.
C    MVELAB     CHARACTER ARRAY. THIS IS THE MOVE TABLE.
C    LOC        INTEGER ARRAY. THIS IS THE MEMORY LOCATION TABLE.
C    DIM        INTEGER ARRAY. THIS IS THE DIMENSIONALITY TABLE.
C    INFO       INTEGER ARRAY. THIS IS THE INFORMATION TABLE.
C    IPOS       INTEGER ARRAY. THIS IS THE POSITION TABLE.
C    KIND       INTEGER ARRAY. THIS IS THE KIND TABLE.
C    LLOC       INTEGER ARRAY. THIS IS THE LOOP TABLE.
C
C  OTHER PROGRAM UNITS CALLED:
C    SUBROUTINES EXTRA, XPR, ERROR, EXE, OVER.
C
C  ---------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
C
C  THE LENGTH OF A LINE. (CHARS)
      PARAMETER (  LINLEN = 80  )
C
C  THE LENGTH OF AN IDENTIFIER.
      PARAMETER ( IDLEN = 30 )
C
      INTEGER TOKEN(*),TYPE(*),TDEF(*),TREL(*),LOC(*),DIM(*),INFO(*),
     >        KIND(*),IPOS(*),LLOC(*)
      CHARACTER*(*) ALPHA(*),VAR(*),MVELAB(*)
      REAL NUM(*)
C
      LOGICAL GENE,BOUNDS,DEBUG
      COMMON /MCLDTA/ NL,GENE,BLOC,MEMLOC,BOUNDS,DEBUG
C
      COMMON /MCLSCD/ LEFTP,RIGHTP,PERIOD,COMMA,GT,LT,NE,EQ,GE,LE,
     >                PLUS,MINUS,STAR,SLASH,BISTAR,COLON,EOL,IDENT,
     >                UNSI,NOTREC,STRING,APOST,LBRA,RBRA,SEMI,PERCE,
     >                AND,OR,XOR,NOT,EQUAL
C
      COMMON /MCLUNT/ INP,ERL,BIN,SCR,STF,SPARE,FHELP,DESCRI
C
      COMMON /MCLCNT/ NLAB,MAXLAB,NVAR,MAXVAR,NTYPE,MAXTYP,NMOVE,MAXMOV,
     >                NINF,MAXIN,NFARG,NSTL,LMEM,FMEM,MAXL,NLOO,
     >                MAXLOO,NERR
C
      LOGICAL CORRE
      INTEGER EX(LINLEN)
      CHARACTER*(IDLEN) STARG(1)
C
C  INITIALIZE POINTERS TO THE TOKEN, ALPHA AND NUM ARRAYS RESPECTIVELY.
C
      NT=1
      NA=1
      NN=0
C
C  EXTRACT AND EVALUATE THE <MCL EXPRESSION>.
C
      CALL EXTRA(TOKEN,NTOK,NT,EX,NEX,1,ALPHA,NA)
      CALL XPR(EX,NEX,ALPHA,NUM,NA,NN,CORRE,VAR,.FALSE.,BLOC,LOC,
     >         DIM,INFO,IPOS,KIND,SCR,STARG)
C  IF THE EXPRESSION IS INCORRECT.
      IF (.NOT.CORRE) RETURN
C  WRITE CODE TO SKIP THE <NCNLCS> STATEMENT, IN CASE THE
C  EXPRESSION YIELDS A ZERO VALUE. (WHICH MEANS THAT THE GIVEN
C  CONDITION IS NOT SATISFIED)
      BLOC=BLOC+3
      IF (GENE) WRITE(SCR,'(A/A/)') '$NOT','$TESTMOVE'
C
C  A "JUST" SHOULD FOLLOW THE <MCL EXPRESSION>.
C
      NT=NT+1
      IF (TOKEN(NT).NE.IDENT) THEN
         CALL ERROR(6,' ')
         RETURN
      ELSE
         NA=NA+1
         IF (ALPHA(NA).NE.'JUST') THEN
            CALL ERROR(6,ALPHA(NA))
            RETURN
         END IF
      END IF
C
C  UPDATE THE TYPE AND TYPE-DEFINITION TABLES.
C
      NTYPE=NTYPE+1
      IF (NTYPE.GT.MAXTYP)
     >              CALL OVER('TYPE','TYPE','WHEN',MAXTYP,'MXTYP')
      TYPE(NTYPE)=-6
      TDEF(NTYPE)=BLOC
C  SAVE VALUES, FOR LATER USE.
      INILOC=BLOC
      OLDNT=NTYPE
C
C  PROCESS THE <NCNLCS> STATEMENT.
C
      CALL EXE(TOKEN,NTOK,ALPHA,NUM,NT,NA,NN,TYPE,TDEF,VAR,MVELAB,
     >         LOC,DIM,INFO,IPOS,KIND,LLOC)
C
C  CALCULATE ENTRY IN THE TYPE-LINK TABLE, FOR THIS "WHEN".
C
      TREL(OLDNT)=BLOC-INILOC+1
C
      RETURN
      END
CXCHECK-----------------------------------------------------------------
      SUBROUTINE XCHECK ( EXPR , TOKNUM , CORR , ALPHAC , NA , FDECLR ,
     >                    STVAR , DIMARG , ADDR , INFPOS , VRKIND , VAR)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: CHECKS EXPRESSIONS FOR VALIDITY.
C
C   %..   ARGUMENT INFORMATION:
C
C    -EXPR   : THE EXPRESSION TO BE CHECKED.
C    -TOKNUM : THE NUMBER OF TOKENS IN EXPR.
C    -CORR   : TRUE AT EXIT IF EXPR CORRECT
C    -ALPHAC : ALPHABETIC CONTENTS OF EXPR. (SEE LEX).
C    -NA     : POINTER TO THE LAST USED ELEMENT OF ALPHAC.
C    -FDECLR : TRUE AT ENTRY IF XCHECK CALLED FOR
C              MCL-FUNCTION DECLARATION PROCESSING.
C    -STVAR  : ARRAY USED IF FDECLR TRUE. HOLDS THE VARIABLE
C              NAMES AT THE ARGUMENT LIST.
C    -DIMARG : HOLDS DIMENSIONALITY OF MCL ARRAYS, OR NUMBER OF
C              ARGUMENTS OF MCL FUNCTIONS.
C    -ADDR   : HOLDS FIRST WORD ADRESS (FWA) FOR EACH MCL ARRAY
C              AND LOCATION OF A SIMPLE MCL VARIABLE INTO MEMORY
C              ARRAY.
C    -INFPOS : ARRAY HOLDING UPPER AND LOWER BOUNDS FOR MCL ARRAYS
C              START END END LOCATION OF THE MCL-FUNCTION
C              MOC AT THE AUXILIARY FILE WITH THAT MOCS AND
C              AN IDENTITY NUMBER FOR INTRINSIC MCL ITEMS.
C    -VRKIND : HOLDS KIND FOR EACH ITEM IN VAR ARRAY.
C    -VAR    : HOLDS ALL IDENTIFIERS DECLARED FOR VARIABLES,
C              TOGETHER WITH ALL INTRINSIC ITEM NAMES.
C
C
C   %..   SUBPROGRAMS CALLED:
C
C     SUBROUTINES: ERROR,INSERT,LOCAN,POPBR,POPUP,PUSHBR,PUSHDN,
C                  SUBX,SERLOC,ABORT
C     FUNCTIONS  : NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C        TSTXST (M) , TSTBST (M) , MCLCNT (M) , MCLSCD (A)
C
C   %..   POSSIBLE ABORTS:
C
C             ILLEGAL EXPRESSION ELEMENT MAY BE GIVEN
C             DUE TO ERRONEOUS PREVIOUS PROCESSING.
C             NORMALLY THIS SHOULD NEVER HAPPEN.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS:
C
C   %..   NOTES:
C
C          TESTING ROUTINE FOR EXPRESSIONS. INPUT IS EXPR AND TOKNUM AND
C          OUTPUT IS CORR: A FLAG INDICATING CORRECTNESS (IF .TRUE.).
C
C          THIS ROUTINE CHECKS THE VALIDITY  OF ANY MCL EXPRESSION
C          USING THE FOLLOWING METHOD :
C          1. THE INPUT EXPRESSION RESIDING ON ARRAY EXPR IS CLOSED IN
C             PARENTHESES  (THIS IS DONE SO AS TO USE SUBX IN ANY WAY)
C          2. A SCANNING IS PERFORMED ON EXPR AND WHEN A LEFT
C             PARENTHESIS IS LOCATED,
C             ITS POSITION IS PUSHED INTO A STACK.
C             IF A LEFT BRAKET IS LOCATED ITS POSITION IS PUSHED IN A
C             SECOND STACK (REFERENCED HERE ON AS BSTACK).
C          3. WHEN A RIGHT PARENTHESIS IS MET THE POSITION OF THE
C             PARENTHESIS
C             MOST RECENTLY LOCATED IS POPED AND THIS POSITION IS
C             TRANSFORMED SO TO BECOME IDENT.
C             IF A RIGHT BRAKET IS MET THE POSITION OF THE MOST RECENTLY
C             LOCATED BRAKET IS POPED FROM THE BSTACK AND IGNORE
C             TRANSFORMATION IS PERFORMED.
C          4. ALL ITEMS FROM LEFT-PARENTHESIS'-POSITION + 1 UP TO AND
C             POSITION
C             OF RIGHT PAR.,ARE SET IGNORE AND SCANNING IS CONTINUED
C             FROM THE LAST POSITION
C             BEFORE "IGNORIZATION" WHERE SUBX WAS CALLED.
C             ALL ITEMS FROM LEFT-BRAKET-POSITION UP TO AND POSITION OF
C             RIGHT BRAKET ARE SET IGNORE.
C
C          IGNORE-TRANSFORMATION IS A TRICK WHICH IS MADE SO TO SKIP ALL
C          CORRECT CONSTRUCTS FROM INSIDE TO OUTSIDE AND DETERMINE THE
C          ILLEGAL SITUATIONS IF ANY.
C
C-----------------------------------------------------------------------
      IMPLICIT INTEGER ( A - Z )
C
      LOGICAL CORR,SUBCOR,SUCCES,OK,RTBRA,LFTBRA,COMMA,FOUND,
     >        OPERAN,OPRTOR,LFTPAR,RTPAR,PFLAG,BFLAG,INTARR,FDECLR
C
      PARAMETER (IGNORE=-283, ZERO=-1000, LINLEN=80,
     >           VARIA=1, ONE=1000, DNTCAR=-1, IDLEN=30,
     >           STFUN=3, INTRFN=6, INTRVR=4, ARR=2)
C
      DIMENSION EXPR(1:TOKNUM),WORKEX(LINLEN+2),SUBEX(1:LINLEN),
     >          STACK(1:LINLEN),BSTACK(1:LINLEN),ADDR(*),DIMARG(*),
     >          INFPOS(*),VRKIND(*)
C
      CHARACTER*(IDLEN)  ALPHAC(*)*(*), X, STRI,
     >                   STVAR(*)*(*), VAR(*)*(*)
C
      COMMON / TSTXST / TOPSTK,STACK,SUCCES
     >       / TSTBST / TOPBST,BSTACK,OK
     >       / MCLCNT / NLB , MXLB , NVAR , MXVR , NTYP , MXTYP , NMOV ,
     >                  MXMOV , NINF , MXINFO , NSTVAR,
     >                  X1,X21,X3,X4,X5,X6,NERR
     >       / MCLSCD / LEFTP, RIGHTP, DOT, COMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUMBER,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
C
      INTARR (X) = X.EQ.'R'    .OR. X.EQ.'L'    .OR. X.EQ.'FIX'  .OR.
     >             X.EQ.'GRAD' .OR. X.EQ.'STEP' .OR. X.EQ.'MARG' .OR.
     >             X.EQ.'X'
C
      OPERAN() = (TEMPO.EQ.IDENT).OR.(TEMPO.EQ.NUMBER).OR.
     >           (TEMPO.EQ.ZERO).OR.(TEMPO.EQ.ONE)
      OPRTOR() = (TEMPO.EQ.PLUS).OR.(TEMPO.EQ.MINUS).OR.
     >           (TEMPO.EQ.STAR).OR.(TEMPO.EQ.BISTAR).OR.
     >           (TEMPO.EQ.SLASH).OR.(TEMPO.EQ.AND).OR.
     >           (TEMPO.EQ.OR).OR.(TEMPO.EQ.LT).OR.
     >           (TEMPO.EQ.LE).OR.(TEMPO.EQ.GT).OR.
     >           (TEMPO.EQ.GE).OR.(TEMPO.EQ.EQOPER).OR.
     >           (TEMPO.EQ.NE).OR.(TEMPO.EQ.XOR).OR.
     >           (TEMPO.EQ.NOT)
      LFTPAR() = (TEMPO.EQ.LEFTP)
      RTPAR()  = (TEMPO.EQ.RIGHTP)
      LFTBRA() = (TEMPO.EQ.LBRA)
      RTBRA()  = (TEMPO.EQ.RBRA)
      COMMA()  = (TEMPO.EQ.COMA)
C
      WORKEX(1)=LEFTP
      WORKEX(TOKNUM+2)=RIGHTP
C
      DO 200,K=2,TOKNUM+1
         WORKEX(K) = EXPR(K-1)
 200  CONTINUE
C
      FINGER=0
      TOPSTK=0
      TOPBST=0
      CORR = .TRUE.
      NNA=NA
C
 1000 CONTINUE
C
      FINGER=FINGER+1
      PFLAG=.FALSE.
      BFLAG=.FALSE.
C
      IF(FINGER.GT.TOKNUM+2)THEN
         CALL POPUP(DUMMY)
         CALL POPBR(DUMMY)
         IF(SUCCES.OR.OK)THEN
            CORR=.FALSE.
            CALL ERROR (49,' ')
         END IF
         RETURN
      ENDIF
C
      TEMPO=WORKEX(FINGER)
C
      IF ( LFTPAR() ) THEN
         CALL PUSHDN (FINGER)
         GOTO 1000
C
      ELSE IF ( LFTBRA() ) THEN
C
C        CHECK THE CASE OF <OPRTOR> [<EXPR>] WHICH IS WRONG.
C
         IF (WORKEX(FINGER-1).NE.IDENT) THEN
           CORR = .FALSE.
           CALL ERROR (50,' ')
           RETURN
         ENDIF
         CALL PUSHBR(FINGER)
         GOTO 1000
C
      ELSE IF (OPRTOR())THEN
         GOTO 1000
C
      ELSE IF (OPERAN())THEN
        IF(TEMPO.EQ.IDENT) THEN
          NNA=NNA+1
          STRI=ALPHAC(NNA)
          FOUND=.FALSE.
          IF(FDECLR)CALL SERLOC (STVAR, NSTVAR, STRI, FOUND, POS)
          IF (.NOT.FOUND) THEN
            CALL LOCAN (VAR, NVAR, STRI, FOUND, POS)
            IF (FOUND) THEN
              KIND=VRKIND(POS)
            ELSE
              CALL INSERT
     >        (VAR,DIMARG,ADDR,INFPOS,VRKIND,STRI,0,0,0,-1,POS)
              CORR=.FALSE.
C
C             SIGN ERROR AND ANNOUNCE THE UNDECLARED VARIABLE.
C
              CALL ERROR (47,STRI)
              RETURN
            END IF
          ELSE
            KIND=VARIA
          END IF
C
C         PERFORM SOME SPECIAL CHECKS.
C
          IF (KIND.EQ.ARR.AND.WORKEX(FINGER+1).NE.LBRA) THEN
            CALL ERROR (162,' ')
            CORR = .FALSE.
            RETURN
          ELSE IF (KIND.EQ.STFUN.OR.KIND.EQ.INTRFN) THEN
            IF (WORKEX(FINGER+1).NE.LBRA) THEN
              CALL ERROR (161,' ')
              CORR = .FALSE.
              RETURN
            END IF
          ELSE IF (KIND.EQ.INTRVR.OR.KIND.EQ.VARIA) THEN
            W = WORKEX(FINGER+1)
            IF (W.EQ.LBRA.OR.W.EQ.LEFTP) THEN
              IF (KIND.EQ.VARIA) THEN
                CALL ERROR (159,' ')
              ELSE
                CALL ERROR (160,' ')
              END IF
              CORR = .FALSE.
              RETURN
            END IF
          ELSE IF (KIND.EQ.DNTCAR) THEN
            GOTO 1000
C         FINALLY, CHECK INTRINSIC ARRAYS WITH NO [  ].
          ELSE IF (INTARR(STRI).AND.WORKEX(FINGER+1).NE.LBRA)
     >      THEN
            CALL ERROR (163,' ')
            CORR = .FALSE.
            RETURN
          END IF
        END IF
        GOTO 1000
C
 
      ELSE IF (RTPAR()) THEN
         CALL POPUP ( BSTART )
         IF ( .NOT.SUCCES ) THEN
            CORR=.FALSE.
            CALL ERROR (48,' ')
            RETURN
         END IF
         PFLAG=.TRUE.
         GO TO 2000
C
      ELSE IF (RTBRA()) THEN
         CALL POPBR(BSTART)
         IF ( .NOT.OK ) THEN
            CORR=.FALSE.
            CALL ERROR (48,' ')
            RETURN
         END IF
         BFLAG=.TRUE.
         GOTO 2000
C
      ELSE IF (COMMA()) THEN
         CALL POPBR(BSTART)
         IF ( .NOT.OK ) THEN
            CORR=.FALSE.
            CALL ERROR (48,' ')
            RETURN
         END IF
         CALL PUSHBR(BSTART)
         GOTO 2000
C
      ELSE
         CALL ABORT ('ILLEGAL ITEM INTO EXPRESSION','XCHECK')
      END IF
C
 2000 CONTINUE
C
      IF ( FINGER-BSTART.LT.2 ) THEN
         CORR=.FALSE.
         CALL ERROR (46,' ')
         RETURN
      END IF
C
      SUBCOR=.TRUE.
      EFECT=0
      DO 555,J=BSTART+1,FINGER-1
         IF ( WORKEX(J).EQ.IGNORE ) GOTO 555
         EFECT=EFECT+1
         SUBEX(EFECT)=WORKEX(J)
 555  CONTINUE
C
      IF ( EFECT .NE. 0 ) THEN
         CALL SUBX ( SUBEX,EFECT,SUBCOR)
      END IF
C
      IF ( .NOT.SUBCOR ) THEN
         CORR=.FALSE.
         RETURN
      END IF
C
      IF ( BFLAG ) BSTART=BSTART-1
C
      DO 556,L=BSTART+1,FINGER
         WORKEX(L)=IGNORE
 556  CONTINUE
C
      IF ( PFLAG ) WORKEX(BSTART)=IDENT
C
      GOTO 1000
      END
CXGEN-------------------------------------------------------------------
      SUBROUTINE XGEN ( EXPR , NTOK , ALPHAC , NUMB, NA , NN , VAR ,
     >      CORREC , FDECLR , BLOC , LOC , DIMARG , INFO ,
     >      IPOS , VRKIND , WORKFL , STVAR )
C-----------------------------------------------------------------------
C
C   %..   PURPOSE: CODE GENERATOR FOR ARITHMETIC EXPRESSIONS.
C
C     THIS ROUTINE GENERATES THE CODE NEEDED FOR THE EVALUATION OF
C     EXPRESSIONS. IT ALSO CHECKS FOR THE PRESENCE OF ANY
C     CALLS FOR FUNCTIONS OR ARRAYS WITH INCORRECT
C     NUMBER OF ARGUMENTS. IT PERFORMS THE CALCULATIONS NEEDED TO
C     LOCATE POSITION OF MULTI-DIMENSIONAL ARRAYS INTO THE LINEAR
C     MEMORY ARRAY OF MERLIN 2.
C
C   %..   ARGUMENT INFORMATION:
C
C    -EXPR: THE EXPRESSION TO BE PROCESSED IN A HYBRID POSTFIX FORM.
C           SEE DOCUMENTATION OF PFIX FOR INFORMATION ON EXPR'S FORM.
C    -NTOK: IS THE NUMBER OF TOKENS RESIDING INTO EXPR.
C    -ALPHAC: ARE THE IDENTIFIERS OF EXPR.
C    -NUMB:   ARE THE NUMERIC CONSTANTS OF EXPR.
C    -NA: IS THE POINTER TO THE LAST USED ELEMENT OF ALPHAC.
C    -NN: IS THE POINTER TO THE LAST USED ELEMENT OF NUMB.
C    -VAR: ARRAY HOLDING ALL IDENTIFIERS PROPERLY DECLARED TOGETHER
C          WITH ALL INTRINSIC ITEMS.
C    -CORREC: AT ENTRY IS TRUE SIGNIFYING THAT UP TO NOW EXPR SEEMS
C             CORRECT. IF AN ERROR OCCURS IT BECOMES FALSE AND
C             IS RETURNED TO XPR ROUTINE.
C    -FDECLR: IS TRUE WHEN XGEN IS CALLED TO PROCESS A FUNCTION
C             DECLARATION.
C    -BLOC: HOLDS THE TOTAL NUMBER OF LINES OF MOC WRITTEN UP TO NOW TO
C           THE "BINARY" FILE.
C    -LOC: ARRAY HOLDING LOCATION OF EACH MCL-ARRAY OR
C          SIMPLE-MCL-VARIABLE INTO MEMORY ARRAY.
C          LOC IS ZERO FOR INTRINSIC ITEMS, AND EQUALS VARTOP PLUS
C          TOTAL NUMBER OF FUNCTION ARGUMENTS FOUND SINCE NOW,
C          FOR FUNCTION IDENTIFIERS.
C    -DIMARG: IS AN ARRAY WHICH HOLDS DIMENSION OF EACH DECLARED ARRAY,
C             AND NUMBER OF ARGUMENTS FOR FUNCTIONS.
C    -INFO: THIS IS A SIGNIFICANT ARRAY. IT HOLDS UPPER AN LOWER BOUNDS
C           FOR EVERY DIMENSION FOR ALL ARRAYS, STARTING AND
C           ENDING LOCATIONS OF CODE IN A SCRATCH (5TH ELEMENT
C           OF MCLUNT) FILE WHICH HOLDS MCL-FUNCTION CODE,
C           AND AN IDENTITY NUMBER FOR INTRINSIC ITEMS.
C    -IPOS: HOLDS POSITION OF INFO INTO INFO ARRAY.
C    -VRKIND: PARALLEL ARRAY TO VAR. KEEPS THE KIND OF EACH VAR ITEM.
C    -WORKFL: WORK FILE UNIT WHERE INTERMEDIATE CODE IS WRITTEN.
C    -STVAR: ARRAY HOLDING THE DUMMY SYMBOLIC NAMES USED IN
C            FUNCTION DECLARATION PROCESSING.
C
C   %..   SUBPROGRAMS CALLED:
C
C     SUBROUTINES: LOCAN, SERLOC, ERROR, ABORT,
C     FUNCTIONS  : NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C     MCLDTA(A), MCLUNT(A), MCLCNT(A), MCLSCD(A)
C
C   %..   POSSIBLE ABORTS:    ABORT MAY APPEAR, IN CASE XCHECK
C                             MISSED SOMETHING.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS:
C
C    -DIME:   ARRAY HOLDING DIMENSIONS UNDER PROCESSING.
C    -QUALIT: ARRAY HOLDING QUALITIES OF EACH ITEM UNDER PROCESSING.
C    -COMCNT: ARRAY HOLDING COMMAS FOUND SINCE NOW.
C    -OFFSET: ARRAY HOLDING MCL ARRAYS OFFSETS INTO MERLIN MEMORY.
C    -VOLUME: AUXILIARY ARRAY FOR MCL-ARRAYS CALCULATIONS.
C    -LOW:    ARRAY HOLDING LOWER BOUNDS.
C    -UP:     ARRAY HOLDING UPPER BOUNDS FOR MCL ARRAYS.
C    -POSIT:  ARRAY HOLDING POSITIONS OF ITEMS INTO VAR ARRAY.
C    -NDIM:   ALMOST ALL THE ABOVE ARRAYS ARE DIMENSIONED (1:NIM). NDIM
C     IS THE MAXIMUM NUMBER OF DIMENSIONS AN MCL ARRAY MAY HAVE. IT IS
C     ALSO THE MAXIMUM NUMBER OF ARGUMENTS AN MCL FUNCTION MAY HAVE.
C
C   %..   NOTES:
C
C  (I) FOR ARRAYS,THE PROCESSING IS BASED IN THE FOLLOWING FORMULAS:
C
C                    ----ARRAY DIMENSION   -+----+- (K-1)
C                   <                       !    !
C   (ARREL POS)  =   >   (SUBSCRIPT -LOWB )*!    ! (UPPERB -LOWB +1)
C                   <              K     K  !    !        M     M
C                    ----K=1                      M=1
C
C                    + OFFSET  = WSUM + LOWSUM + OFFSET ,
C
C                  ----ARRAY DIMENSION
C                 <
C    WHERE,WSUM =  >    VOLUME  * SUBSCRIPT  ,
C                 <           K            K
C                  ----K=1
C
C            ----ARRAY DIMENSION              -+---+- (K-1)
C           <                                  !   !
C  LOWSUM =  >   LOWB  -VOLUME   ,AND VOLUME = !   ! (UPPERB -LOWB +1)
C           <        K        K             K  !   !        M     M
C            ----K=1                                M=1
C
C     CONCERNING SEMANTICS: (ARREL-POS) MEANS ARRAY ELEMENT
C     POSITION , LOWB MEANS LOWER BOUND,
C     UPPERB MEANS UPPER BOUND, AND WSUM WEIGHTED SUM.
C
C (II) A CONVENTION THAT IS FOLLOWED THROUGH THE ENTIRE MCLCOM CODE
C       AND ASSOCIATED WITH MERLIN INTERFACE, IS THAT ALL
C      MOC WHICH NEEDS HANDLING FROM THE INTERFACE, IS PREFIXED
C      BY A DOLLAR SIGN. ( SEE ROUTINE USEMCL OF MERLIN).
C
C (III) ARRAYS WHICH HOLD KINDS ( VAR AND STVAR ),USE THE FOLLOWING
C       CODING:
C      -1  SIGNIFIES A DON'T-CARE SITUATION,
C       1     >>     A USER DEFINED VARIABLE,
C       2     >>     A USER DEFINED ARRAY,
C       3     >>     A USER DEFINED FUNCTION,
C       4     >>     AN INTRINSIC FUNCTION,
C       5     >>     AN      >>   VARIABLE,
C       6     >>     AN      >>   FUNCTION.
C
C (IV) IF GENE IS FALSE,CODE IS NOT GENERATED.
C
C (V)  THIS ROUTINE IS CALLED FROM XPR.
C
C-----------------------------------------------------------------------
C
      IMPLICIT INTEGER ( A - Z )
C
      LOGICAL GENE, CORREC, BOUNDS, FOUND, FDECLR, OPTEST, DEBUG
C
      REAL NUMB(*)
C
      PARAMETER ( IDLEN=30 , LINLEN=80 , NDIM=(LINLEN-13)/2 )
      DIMENSION DIME(NDIM) , QUALIT(NDIM) , COMCNT(NDIM-1) ,
     >          OFFSET(NDIM) , LOC(*) , DIMARG(*) , INFO(*) ,
     >          EXPR(NTOK) , VOLUME(NDIM) , LOW(NDIM) , UP(NDIM) ,
     >          IPOS(*) , VRKIND(*) , POSIT(NDIM)
C
      CHARACTER ALPHAC(*)*(*),VAR(*)*(*),STVAR(*)*(*),OPERS (5:31)*(4),
     >          CODLIN*(LINLEN), ST*(IDLEN), NMTEST*(10)
C
      PARAMETER ( VARIA=1 , ARR=2 , STFUN=3 , INTRVR=4 , INTARR=5 ,
     >            ZERO=-1000, INTRFN=6, DNTCAR=-1, ONE=1000 )
C
      COMMON / MCLDTA / NL, GENE, BLZZ, VARTOP, BOUNDS, DEBUG
     >       / MCLUNT / INP, ERRL, BIN, SXX, STFCOD, SZZ, DUMMYF, DESCRI
     >       / MCLCNT / NLB, MXLB, NVAR, MXVR, NTYP, MXTYP, NMOV,
     >                  MXMOV, NINF, MXINFO, NSTVAR, NLINES,
     >                  N1, FSPACE, N2, N3, N4, NERR
     >       / MCLSCD / LEFTP, RIGHTP, DOT, COMMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUMBER,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
C
C     ALL MCL-OPERATORS ARE HERE. THEIR NAMES ARE USED
C     WHEN XGEN NEEDS THEM FOR MOC GENERATION.
C
      DATA OPERS / '$>  ' , '$<  ' , '$#  ' , '    ' , '$>= ' , '$<= ' ,
     >             '$+  ' , '$-  ' , '$*  ' , '$/  ' , '$** ' ,
     >         11* '    ' , '$AND' , '$OR ' , '$XOR' , '$NOT' , '$=  ' /
C
      ITE1(T) = (T-PLUS)*(T-MINUS)*(T-STAR)*(T-SLASH)
      ITE2(T) = (T-BISTAR)*(T-LT)*(T-LE)*(T-GT)*(T-GE)
      ITE3(T) = (T-EQOPER)*(T-NE)*(T-AND)*(T-OR)*(T-XOR)*(T-NOT)
      OPTEST(T) = ITE1(T).EQ.0 .OR. ITE2(T).EQ.0 .OR. ITE3(T).EQ.0
C
C  START
C
      SCRUNT=WORKFL
      STACK=0
      ARRSTK =0
      SAVEB=BLOC
      IF (FDECLR) THEN
C
C       XGEN CALLED FOR FUNCTION DECLARATION PROCESSING, SO
C       ALL CODE SHOULD BE WRITTEN TO A COMPLETELY DUMMY FILE
C       AND THEN TRANSFORMED TO STFCOD. THIS IS DONE SO TO
C       ALLOW BACKWARD REFERENCE TO ALREADY DECLARED FUNCTIONS
C       FOR WHICH CODE IS ALREADY INTO STFCOD.
C
        BLOC=0
        SAVEB=0
        SCRUNT=DUMMYF
        REWIND SCRUNT
      END IF
C
C     LOOP OVER ALL TOKENS OF EXPR.
C
      DO 1000, I=1,NTOK
        TEMPO = EXPR(I)
C
        IF (TEMPO.EQ.IDENT) THEN
          NA=NA+1
          ST=ALPHAC(NA)
          IF (FDECLR) THEN
C
C           ENTER HERE ONLY IF CALLED FOR FUNCTION DECLARATION
C           PROCESSING.
C
C           CHECK DUMMY NAMES TABLE FIRST.
C
            CALL SERLOC (STVAR, NSTVAR, ST, FOUND, POS)
            IF (.NOT.FOUND) GOTO 110
            KIND=VARIA
            DIM=0
C
C           FWA STANDS FOR FIRST WORD ADDRESS, AND VARTOP IS POSITION OF
C           MERLIN MEMORY ARRAY WHERE ABOVE IT, MCL-FUNCTIONS ARE
C           LINKED (PUSHED AND POPED).
C             SINCE STVAR IS LOADED WITH THE ORDER OF THE DUMMY
C           ARGUMENTS PRESERVED, POS ALSO HOLDS ARGUMENT ORDER.
C
C           GET FWA VALUE FOR CURRENT DUMMY ARGUMENT:
C
            FWA = VARTOP + FSPACE + POS
C
            BNDFWA=0
            GO TO 111
          END IF
 110      CALL LOCAN (VAR, NVAR, ST, FOUND, POS)
C
C      IF XCHECK PERFORMED WRONG, XGEN ABORTS.
C
          IF (.NOT.FOUND) CALL ABORT ('NAME NOT FOUND','XGEN')
C
          KIND=VRKIND(POS)
          DIM=DIMARG(POS)
          FWA=LOC(POS)
          BNDFWA=IPOS(POS)
 111      CONTINUE
          IF (KIND.EQ.DNTCAR) THEN
C
C           SKIP PROCESSING. PREVIOUSLY UNDECLARED IDENTIFIER FOUND
C           AGAIN.
C
            RETURN
          ELSE IF (KIND.EQ.VARIA) THEN
            BLOC=BLOC+2
            IF(GENE)THEN
               WRITE(SCRUNT,'(''$PUSH'')')
               WRITE(SCRUNT,*)FWA
            END IF
          ELSE IF (KIND.EQ.ARR) THEN
            STACK=STACK+1
C
C           A STACK SCHEME IS USED HERE FOR PROCESSING DUE TO THE
C           SPECIAL STRUCTURE OF EXPR. ALL INFORMATION IS KEPT UNTIL A
C           KET IS LOCATED, AND THEREAFTER THEY ARE PROCESSED IN REVERSE
C           ORDER. ( "KET" SIGNIFIES A RIGHT BRAKET. REMEMBER DIRAC).
C
            COMCNT(STACK)=0
            DIME(STACK)=DIM
            QUALIT(STACK)=KIND
            OFFSET(STACK)=FWA
            POSIT(STACK) = POS
            ARRSTK = ARRSTK +1
            LOW( ARRSTK )=BNDFWA
            UP( ARRSTK )=BNDFWA+1
            IF (BOUNDS) THEN
              IF(GENE)THEN
                 WRITE(SCRUNT,'(''$PUSHC'')')
                 WRITE(SCRUNT,*)INFO(LOW( ARRSTK ))
              END IF
C
C             LOWER AND UPPER BOUNDS OF FIRST DIMENSION ARE PUSHED, SO
C             TO BE CHECKED VIA LOCCHECK FOR VIOLATION DURING RUN-TIME.
C
              IF(GENE)THEN
                 WRITE(SCRUNT,'(''$PUSHC'')')
                 WRITE(SCRUNT,*)INFO(UP( ARRSTK ))
              END IF
              BLOC=BLOC+4
            END IF
          ELSE IF (KIND.EQ.STFUN.OR.KIND.EQ.INTRFN) THEN
            STACK=STACK+1
            COMCNT(STACK)=0
            DIME(STACK)=DIM
            POSIT(STACK) = POS
            QUALIT(STACK)=KIND
            OFFSET(STACK)=FWA
          ELSE IF (KIND.EQ.INTRVR) THEN
            BLOC=BLOC+1
            IF(GENE)WRITE(SCRUNT,'(''$'',A)')VAR(POS)
          ELSE IF (KIND.EQ.INTARR) THEN
            STACK=STACK+1
            COMCNT(STACK)=0
            DIME(STACK)=DIM
            QUALIT(STACK)=KIND
            POSIT(STACK)=POS
            IF (BOUNDS) THEN
              IF(GENE)WRITE(SCRUNT,'(''$PUSHC''/'' 1.000''/''$DIM'')')
              BLOC=BLOC+3
            END IF
          END IF
C
        ELSE IF (TEMPO.EQ.ZERO) THEN
          BLOC=BLOC+2
          IF(GENE)WRITE(SCRUNT,'(''$PUSHC'',/,'' 0.000'')')
C
        ELSE IF (TEMPO.EQ.ONE) THEN
C
C         IGNORE EVERYTHING SINCE "ONE" IS USED JUST TO MAKE
C         "NOT" A BINARY OPERATOR (FOR CHECKS IN XCHECK).
C         PROCESSING PROCEEDS TO THE NEXT TOKEN.
C
        ELSE IF (TEMPO.EQ.NUMBER) THEN
          NN=NN+1
          BLOC=BLOC+2
          IF(GENE)THEN
             WRITE(SCRUNT,'(''$PUSHC'')')
             WRITE(SCRUNT,*)NUMB(NN)
          END IF
C
        ELSE IF (OPTEST(TEMPO)) THEN
          BLOC=BLOC+1
          IF(GENE)WRITE(SCRUNT,'(A)')OPERS(TEMPO)
C
        ELSE IF (TEMPO.EQ.COMMA) THEN
          COMCNT(STACK)=COMCNT(STACK)+1
          QTEST=QUALIT(STACK)
          IF (QTEST.EQ.ARR.AND.BOUNDS) THEN
            IF(GENE)THEN
               WRITE(SCRUNT,'(''$LOCCHECK''/''$PUSHC'')')
               WRITE(SCRUNT,*)
     >         INFO(LOW( ARRSTK )+COMCNT(STACK)*2)
               WRITE(SCRUNT,'(''$PUSHC'')')
               WRITE(SCRUNT,*)
     >         INFO(UP( ARRSTK )+COMCNT(STACK)*2)
            END IF
            BLOC=BLOC+5
          ELSE IF (QTEST.EQ.INTARR) THEN
            CORREC=.FALSE.
            CALL ERROR (164,' ')
            RETURN
          END IF
C
        ELSE IF (TEMPO.EQ.RBRA) THEN
C
C       MAIN REVERSE-ORDER PROCESSING ACTIVATED.
C
          IF (COMCNT(STACK)+1.EQ.DIME(STACK)) THEN
            QTEST=QUALIT(STACK)
            BNDFWA=IPOS(POSIT(STACK))
            IF (QTEST.EQ.STFUN) THEN
C
C             LINK ARGUMENT VALUES WITH THE POSITIONS WHERE THE
C             FOLLOWING MCL-FUNCTIONS CODE ADRESSES.
C
              DO 118, L=DIME(STACK) , 1 , -1
                BLOC=BLOC+2
                IF(GENE)THEN
                   WRITE(SCRUNT,'(''$POP'')')
                   WRITE(SCRUNT,*)OFFSET(STACK)+L
                END IF
 118          CONTINUE
C
C             READ MCL-FUNCTION CODE FROM SCRATCH AND WRITE IT TO
C             THE INTERMEDIATE MOC FILE.
C
              IF ( .NOT.GENE ) GOTO 123
              REWIND STFCOD
              DO 121, SKIP=1,INFO(BNDFWA)-1
 121            READ(STFCOD,'(A)')CODLIN
              DO 122, J=1,INFO(BNDFWA+1)-INFO(BNDFWA)+1
                READ(STFCOD,'(A)')CODLIN
                WRITE(SCRUNT,'(A)')CODLIN
                BLOC=BLOC+1
 122          CONTINUE
 123          CONTINUE
            ELSE IF (QTEST.EQ.INTRFN) THEN
              NMTEST=VAR(POSIT(STACK))
              BLOC=BLOC+1
              IF(GENE)WRITE(SCRUNT,'(A11)')'$'//NMTEST
            ELSE IF (QTEST.EQ.ARR) THEN
              IF (BOUNDS) THEN
                BLOC=BLOC+1
                IF(GENE)WRITE(SCRUNT,'(''$LOCCHECK'')')
              ENDIF
C
C             PERFORM ARRAY POSITION PROCESSING AS PRESENTED IN THE
C             INTRODUCTORY COMMENTS OF THIS ROUTINE.
C
              IF ( .NOT. GENE ) GO TO 818
C
              MM=LOW( ARRSTK )
              DMS=DIME(STACK)
              DO 101, VL=1,DMS
                VOLUME(VL)=1
C
C               NOTE THE (MM+1) BELOW. IT IS WRITTEN LIKE
C               THIS, SO TO BE CONSISTENT WITH THE FORMULAS
C               IN THE INTRODUCTORY COMMENTS.
C
                DO 101, L = MM+1 , MM+2*(VL-1) , 2
 101              VOLUME(VL)=VOLUME(VL)*(1+INFO(L)-INFO(L-1))
              LOWSUM=0
              MM=MM-2
              DO 102, M=1,DMS
                MM=MM+2
 102            LOWSUM=LOWSUM+INFO(MM)*VOLUME(M)
C
              DO 103, N= DMS , 1 , -1
                BLOC=BLOC+2
                WRITE(SCRUNT,'(''$PUSHC'')')
                WRITE(SCRUNT,*)VOLUME(N)
 103          CONTINUE
              BLOC=BLOC+6
              WRITE(SCRUNT,'(''$WSUM'')')
              WRITE(SCRUNT,*) DMS
              WRITE(SCRUNT,'(''$PUSHC'')')
              WRITE(SCRUNT,*) OFFSET(STACK)-LOWSUM
              WRITE(SCRUNT,'(''$+'',/,''$GETCONT'')')
 818          ARRSTK = ARRSTK -1
C
C             ONE ARRAY LOWER...
C
            ELSE IF (QTEST.EQ.INTARR) THEN
              BLOC=BLOC+1
              IF (BOUNDS) THEN
                BLOC=BLOC+1
                IF(GENE)WRITE(SCRUNT,'(''$LOCCHECK'')')
              ENDIF
              IF(GENE)WRITE(SCRUNT,'(''$'',A)')
     >        VAR(POSIT(STACK))
            END IF
C
            STACK=STACK-1
C
          ELSE
C
C           THE NUMBER OF COMMAS FOUND,
C           DOES NOT COINCIDE WITH THE EXPECTED
C           NUMBER IMPLIED FROM THE DECLARATION, SO...
C
            NMTEST=VAR(POSIT(STACK))
            IF (NMTEST.EQ.'MIN'.OR.NMTEST.EQ.'MAX'
     >         .OR.NMTEST.EQ.'MEAN') THEN
                BLOC=BLOC+2
                IF(GENE)THEN
                   WRITE(SCRUNT,'(A11)') '$'//NMTEST
                   WRITE(SCRUNT,*)COMCNT(STACK)+1
                END IF
C
                STACK=STACK-1
C
                GOTO 1000
            ENDIF
            CORREC=.FALSE.
            CALL ERROR (78,' ')
            RETURN
          END IF
C
        ELSE
          CALL ABORT ('ILLEGAL ITEM INTO EXPRESSION','XGEN')
        END IF
 1000 CONTINUE
C
      IF ( (.NOT.FDECLR) .OR. (.NOT.GENE) ) GO TO 9000
C
      REWIND STFCOD
      REWIND SCRUNT
      DO 1001,KL=1,NLINES
        READ(STFCOD,'(A)')CODLIN
 1001 CONTINUE
      DO 1002,ML=1,BLOC-SAVEB
        READ(SCRUNT,'(A)')CODLIN
        WRITE(STFCOD,'(A)')CODLIN
 1002 CONTINUE
C
 9000 RETURN
      END
CXPR--------------------------------------------------------------------
      SUBROUTINE XPR (EX, NTOKS, ALPHAC, NUMBC, NA, NN, CORR, VAR,
     >  FDECLR, BLOC, ADDR, DIMARG, INFO, INFPOS, VRKIND, SCRUNT, STVAR)
C-----------------------------------------------------------------------
C
C   %..   PURPOSE:
C
C     THIS ROUTINE, ORCHESTRATES EXPRESSION-PROCESSING.
C
C   %..   ARGUMENT INFORMATION:
C
C      -EX    : IS THE EXPRESSION TO BE PROCESSED
C      -NTOKS : IS THE NUMBER OF TOKENS EX IS COMPOSED OF
C      -ALPHAC: ARE THE IDENTIFIERS OF EX
C      -NUMBC : ARE THE NUMERIC CONSTANTS OF EX.
C      -NA    : POINTS TO THE LAST USED ELEMENT OF ALPHAC.
C      -NN    : POINTS TO THE LAST USED ELEMENT OF NUMBC.
C      -CORR  : IS TRUE IF EX CORRECT AT ENTRY OR EXIT.
C      -VAR   : THE VAR TABLE.
C      -FDECLR: TRUE IF XPR CALLED FOR FUNCTION DECLARATION.
C      -BLOC  : LOCATIONS (LINES) IN THE BINARY FILE WRITTEN.
C      -ADDR  : ARRAY KEEPING LOCATIONS. SEE XGEN FOR DETAILS.
C      -DIMARG: ARRAY KEEPING DIMENSIONALITIES. SEE XGEN.
C      -INFO  : ARRAY WITH MCL-ITEMS INFORMATION. SEE XGEN.
C      -INFPOS: ARRAY RELATED TO INFO. SEE XGEN.
C      -VRKIND: PARALLEL ARRAY TO VAR. KEEPS "KIND"S. SEE XGEN.
C      -SCRUNT: SCRATCH UNIT WHERE CODE IS TO BE WRITTEN.
C      -STVAR : IF FDECLR TRUE, IS USED. KEEPS DUMMY NAMES.
C
C   %..   SUBPROGRAMS CALLED:
C
C     SUBROUTINES: PFIX, XCHECK, XGEN, ERROR,
C     FUNCTIONS  : NONE.
C
C   %..   GLOBAL DATA ACCESSED(A) OR MODIFIED(M):
C
C            MCLSCD (A)
C
C   %..   POSSIBLE ABORTS: NONE.
C
C   %..   MAIN DATA STRUCTURES AND WORK AREAS: NONE.
C
C   %..   NOTES: NONE.
C
C-----------------------------------------------------------------------
C
      IMPLICIT INTEGER (A-Z)
      PARAMETER ( LINLEN = 80, ONE=1000, ZERO=-1000)
      DIMENSION  NEWEX ( 1:2*LINLEN ), EX ( NTOKS ),EXP2(3*LINLEN),
     >           ADDR(*), DIMARG(*), INFO(*), INFPOS(*), VRKIND(*)
      CHARACTER ALPHAC (*)*(*), STVAR (*)*(*), VAR (*)*(*)
      REAL NUMBC ( * )
      LOGICAL CORR, OPERAN, FDECLR
C
      COMMON / MCLSCD / LEFTP, RIGHTP, DOT, COMMA, GT, LT, NE,
     >                  EQ, GE, LE, PLUS, MINUS, STAR, SLASH,
     >                  BISTAR, COLON, EOLCOD, IDENT, NUMBER,
     >                  UNRCGN, STRING, QUOTE, LBRA, RBRA, SEMI,
     >                  PERCE, AND, OR, XOR, NOT, EQOPER
C
      OPERAN ( XPART ) = ( XPART.EQ.NUMBER ) .OR. ( XPART.EQ.IDENT )
C
C  START
C
      IF(NTOKS.EQ.0)THEN
        CALL ERROR (132,' ')
        CORR=.FALSE.
C
C       XPR WAS CALLED BUT NO EXPRESSION PRESENT.
C
        RETURN
      ENDIF
C
C     ZERO-INSERTION FOR NEGATIONS:
C     ZEROS ARE INSERTED INFRONT OF EVERY + OR - WHEN USED AS SIGNS.
C     THE RESULTING CODE IS NOT SO EFFICIENT, SO DO NOT PLACE AN
C     UNNECESSARY + INFRONT OF AN EXPRESSION.
C
      INDEX=1
      IF ( EX(1).EQ.PLUS.OR.EX(1).EQ.MINUS)THEN
         NEWEX(INDEX)=ZERO
         INDEX=INDEX+1
      ENDIF
      NEWEX(INDEX)=EX ( 1 )
      IF(NTOKS.EQ.1) GO TO 224
      DO 212, I=2, NTOKS-1
         IF ( EX(I).EQ.PLUS.OR.EX(I).EQ.MINUS ) THEN
            IF(.NOT.(
     >               (OPERAN(EX(I-1)).AND.( OPERAN(EX(I+1)).OR.
     >                                      EX(I+1).EQ.LEFTP.OR.
     >                                      EX(I+1).EQ.NOT))
     >                .OR.
     >                EX(I-1).EQ.RIGHTP
     >                .OR.
     >                EX(I-1).EQ.RBRA
     >      ))THEN
               INDEX=INDEX+1
               NEWEX ( INDEX ) =ZERO
            ENDIF
         ENDIF
         INDEX=INDEX+1
         NEWEX ( INDEX ) =EX ( I )
 212  CONTINUE
      INDEX=INDEX+1
      NEWEX ( INDEX ) =EX ( NTOKS )
C
C     ZEROS INSERTED. CONTINUE TO INSERT ONES FOR "NOT"
C     PROCESSING.
C     DURING THE CODE-GENERATING STAGE OF XGEN, "ONE"S ARE IGNORED.
C     THEY SERVE THE PURPOSE OF TRANSFORMING "NOT" TO A PSEUDO
C     BINARY OPERATOR.
C
 224  NTOKS=INDEX
      DO 190,J=1,NTOKS
        EXP2(J)=NEWEX(J)
 190  CONTINUE
      INDEX=1
      IF(EXP2(1).EQ.NOT)THEN
         NEWEX(INDEX)=ONE
         INDEX=INDEX+1
      ENDIF
      NEWEX(INDEX)=EXP2(1)
      IF (NTOKS.EQ.1)GOTO 225
      DO 213, K=2,NTOKS-1
         IF(EXP2(K).EQ.NOT)THEN
            INDEX=INDEX+1
            NEWEX(INDEX)=ONE
         ENDIF
         INDEX=INDEX+1
         NEWEX(INDEX)=EXP2(K)
 213  CONTINUE
      INDEX=INDEX+1
      NEWEX(INDEX)=EXP2(NTOKS)
C
C INSERTIONS COMPLETED. CONTINUE FOR VALIDITY-CHECKING OF
C THE NEW LAYOUT OF EXPRESSION. NOTE THAT:
C IF THE EXPRESSION WITHOUT THE INSERTED ITEMS IS CORRECT
C IT REMAINS SO AND AFTER THE INSERTIONS.
C
 225  CONTINUE
C
      CALL XCHECK ( NEWEX, INDEX, CORR , ALPHAC , NA , FDECLR ,
     >              STVAR , DIMARG , ADDR , INFPOS , VRKIND , VAR )
C
      IF  (  .NOT.CORR  )  RETURN
C
C     IF CORR IS TRUE,THEN THE SOLE SITUATION LEFT UNCHECKED
C     IS THE CALLING TO A FUNCTION WITH WRONG NUMBER OF ARGUMENTS OR
C     THE REFERENCE TO AN ARRAY ELEMENT WITH A WRONG NUMBER
C     OF SUBSCRIPTS. THOSE CHECK-UPS ARE PERFORMED VIA XGEN.
C     NOW CALL PFIX TO TRANSFORM NEWEX TO THE HYBRID POSTFIX FORM.
C     (SEE DOCUMENTATION OF PFIX FOR DETAILS).
C
      CALL PFIX  ( NEWEX ,  INDEX ,  FINTOK  )
C
C     CALL XGEN TO GENERATE MOC.
C
      CALL XGEN  (  NEWEX, FINTOK, ALPHAC, NUMBC, NA, NN, VAR, CORR,
     >FDECLR, BLOC, ADDR, DIMARG, INFO, INFPOS, VRKIND, SCRUNT, STVAR)
C
      RETURN
      END
C  ---------------------------------------------------------------------
      SUBROUTINE ZEROM ( IND1,IND2,TYPE,TDEF,TREL )
C  ---------------------------------------------------------------------
C
C  THIS ROUTINE WILL ASSIGN A VALUE OF 7 TO ALL "MOVE TO" ENTRIES
C  IN THE TYPE TABLE, RESIDING IN A BLOCK SPECIFIED BY TWO LIMITS,
C  IF THEIR TARGET LABEL LIES ALSO IN THE BLOCK.
C
C  INPUT ARGUMENTS:
C
C    IND1       INTEGER VARIABLE. THIS IS A POINTER TO THE TYPE TABLE,
C               AND IS THE LOWER LIMIT FROM WHERE THE SEARCH FOR
C               ANY "MOVE TO" STATEMENTS WILL START.
C    IND2       INTEGER VARIABLE. THIS IS A POINTER TO THE TYPE TABLE,
C               AND IS THE UPPER LIMIT TO WHERE THE SEARCH FOR
C               ANY "MOVE TO" STATEMENTS WILL TERMINATE.
C
C  INPUT / OUTPUT ARGUMENTS:
C
C    TYPE       INTEGER ARRAY. THIS IS THE TYPE TABLE.
C    TDEF       INTEGER ARRAY. THIS IS THE TYPE-DEFINITION TABLE.
C    TREL       INTEGER ARRAY. THIS IS THE TYPE-LINK TABLE.
C
C  ---------------------------------------------------------------------
C
      INTEGER TYPE(*),TDEF(*),TREL(*)
C
C  GET LOCATIONS FOR THE BLOCK LIMITS.
C  LOC1 IS THE LOCATION FOR THE LOWER LIMIT.
C  LOC2 IS THE LOCATION FOR THE UPPER LIMIT.
C
      LOC1 = TDEF(IND1)
      LOC2 = TDEF(IND2)
C
C  SEARCH THE TYPE TABLE.
C
      DO 10,I=IND1,IND2
        IF (TYPE(I).EQ.-7) THEN
C  A "MOVE TO" STATEMENT WAS FOUND.
C  SINCE THE TYPE-LINK TABLE HOLDS ONLY RELATIVE NUMBERS, CALCULATE
C  THE ABSOLUTE LOCATION OF THE TARGET LABEL.
           IR=TREL(I)
           IF (IR.LT.0) IR=IR-1
           JT=TDEF(I)+IR
C  CHECK IF THE LOCATION LIES INSIDE THE ALLOWABLE BOUNDS.
           IF (JT.GE.LOC1 .AND. JT.LE.LOC2) TYPE(I)=7
        END IF
 10   CONTINUE
C
      RETURN
      END
