*-----------------------------------------------------------------------
*
*       THE  OPTIMIZATION SOFTWARE SYSTEM,
*
*               ** M E R L I N **
*
*      IS MADE UP FROM THE COLLECTION OF THE ROUTINES
*      (ALL IN ANSI FORTRAN 77)  CONTAINED IN THIS FILE.
*
*       MERLIN, IS DESCRIBED IN:
* 
*      A)
*       MERLIN - A PORTABLE SYSTEM FOR MULTIDIMENSIONAL MINIMIZATION
*       G.A. EVANGELAKIS, J.P.RIZOS, I.E. LAGARIS & I.N.DEMETROPOULOS
*       COMPUTER PHYSICS COMMUNICATIONS 46 (1987) 401-415
*      B)
*       MERLIN - 2.0, ENHANCED AND PROGRAMMABLE VERSION
*       D.G.PAPAGEORGIOU, C.S.CHASSAPIS & I.E. LAGARIS
*       COMPUTER PHYSICS COMMUNICATIONS 52 (1989) 241-247
*      C)
*       THE PROGRAMMING OPTION IS BASED ON THE MCL LANGUAGE DESCRIBED IN
*       MCL, OPTIMIZATION ORIENTED PROGRAMMING LANGUAGE
*       C.S.CHASSAPIS, D.G.PAPAGEORGIOU & I.E. LAGARIS
*       COMPUTER PHYSICS COMMUNICATIONS 52 (1989) 223-239
*      D)
*       MERLIN - 2.1 - DOUBLE PRECISION
*       D.G.PAPAGEORGIOU & I.E. LAGARIS
*       COMPUTER PHYSICS COMMUNICATIONS 58 (1990) 119-125
* 
*      MERLIN 1.0, IS SUPPLEMENTED BY A 54-PAGE USERS MANUAL
*      MCL, IS SUPPLEMENTED BY A 129-PAGE DEVELOPERS MANUAL.
*      BOTH MANUALS ARE USEFUL FOR INSTALLING, MODIFYING  AND OPERATING
*      THE  PROGRAMS. 
* 
* 
*     PLEASE SEND YOUR COMMENTS AND/OR  SUGGESTIONS (FOR EXAMPLE,
*     OPTIONS THAT YOU WOULD LIKE TO SEE INCLUDED IN FUTURE VERSIONS) 
*     AT THE E-MAIL ADDRESS:
*
*            MERLIN @ CC.UOI.GR
*            MERLIN @ GRIOANUN.BITNET
*
*-----------------------------------------------------------------------
      SUBROUTINE ACCUDE(X,VAL,GRAD,NOC,ERB,RE,IERR)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   INPUT: N,POINT,VAL,ERB
C   OUTPUT: GRAD,NOC,IERR
C   DERMER  CALCULATES THE GRADIENT OF THE OBJECTIVE FUNCTION AT
C   THE VARIABLE VALUES CONTAINED IN ARRAY  POINT(N).
C   VAL  IS THE FUNCTION'S VALUE AT THAT POINT AND
C   ERB  IS THE ERROR-BOUND FOR THE DERIVATIVE CALCULATION.
C   THE GRADIENT COMPONENTS ARE STORED IN ARRAY   GRAD(N) AND
C   THEIR CORRESPONDING ERROR-CODES IN ARRAY  IERR
C   NOC   IS THE NUMBER OF EVALUATIONS OF THE OBJECTIVE FUNCTION
C   THAT WERE PERFORMED BY DERMER
C
      COMMON/ACCURA/COMACC,COMTOL
C
      NOC=0
      ONE = 1.
      XDIV=511*63*45
      A19=2**19/XDIV
      A15=(2**15+2**13+2**10)/XDIV
      A9=(2**9+2**6+2**4)/XDIV
        ICHEK=0
        IERR=0
        H=0.001
        IF(ABS(ABS(X)-ABS(H)).LT.ABS(X/10.))H=H*0.01
        IF((X+H).EQ.X) H=(1.+ABS(X))*COMTOL
        ICOUNT=0
    2   CONTINUE
        ICOUNT=ICOUNT+1
        XX = X+H
        FP1=ACSQ1(XX)
        CHEDER=ABS(FP1-VAL)/(ABS(VAL)+COMACC)
         IF(CHEDER.LT.COMACC) THEN
          ICHEK=ICHEK+1
          ICOUNT=ICOUNT-1
          NOC=NOC+1
          H=10.*H
          IF(ICHEK.LE.5) GO TO  2
         ENDIF
        XX=X-H
        FM1=ACSQ1(XX)
        XX=X+2.*H
        FP2=ACSQ1(XX)
        XX=X-2.*H
        FM2=ACSQ1(XX)
        XX=X+4.*H
        FP4=ACSQ1(XX)
        XX=X-4.*H
        FM4=ACSQ1(XX)
        NOC=NOC+6
        DH1=2.*(FP1-FM1)/H-(FP2-FM2)/(4.*H)
        DH2=(FP2-FM2)/H-(FP4-FM4)/(8.*H)
        DF1=DH1/3.
        E1=(DH1-DH2)/45.
        RE=ABS(E1)/MAX(ONE,ABS(DF1+E1))
         IF(RE.LT.ERB) THEN
C      DERIVATIVE CALCULATED WITHIN DESIRED ACCURACY
          GRAD=DF1+E1
          GO TO  3
         ENDIF
        FMX=ABS(VAL)+COMACC
        XSTEP=(ERB*FMX*4.*H**4/(ABS(E1)+COMACC))**0.2
         IF(H.GT.XSTEP) THEN
           IF(ICOUNT.LT.2) THEN
            H=XSTEP/5.
            GO TO  2
           ELSE
C   FAILURE TO CALCULATE DERIVATIVE WITHIN THE DESIRED ACCURACY
C   INSPITE STEP REDUCTION AND RECALCULATION OF FP1,FP2,... ETC.
            IERR=-1
            GRAD=DF1+E1
         RE = 1.
            GO TO  3
           ENDIF
         ELSE
          XX=X+H/2.
          FPH=ACSQ1(XX)
          XX=X-H/2.
          FMH=ACSQ1(XX)
          DHH=4.*(FPH-FMH)/H-(FP1-FM1)/(2.*H)
          NOC=NOC+2
          GRAD=(1024.*DHH-80.*DH1+DH2)/2835.
          E2=GRAD-DF1-E1
          RE=ABS(E2)/MAX(ONE,ABS(GRAD))
          IF(RE.LT.ERB) GO TO  3
          XX=X+H/4.
          FPQ=ACSQ1(XX)
          XX=X-H/4.
          FMQ=ACSQ1(XX)
          NOC=NOC+2
          DHQ=8.*(FPQ-FMQ)/H-(FPH-FMH)/H
          DERN=A19*DHQ-A15*DHH+A9*DH1-DH2/XDIV
          E3=DERN-GRAD
          RE=ABS(E3)/MAX(ONE,ABS(DERN))
          GRAD=DERN
          IF(RE.GT.ERB) IERR=-1
C    ACCURACY REQUIREMENT NOT SATISFIED ,INSPITE HIGHER ORDER
C    TERMS ARE CALCULATED IN THE TAYLOR EXPANSION.
          GO TO  3
         ENDIF
    3  CONTINUE
CC    WRITE(*,*)'NUMER ERROR = ',RE
      RETURN
      END
      SUBROUTINE ACCUMP(NOPOI,NOCAL,VALTA,IERFL,IPOI)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C   ACCUMULATES AT RANDOM POINTS THAT SATISFY THE CRITERION:
C   ACSQ(X,N) < VALTA
C  NOPOI: INPUT . THE DESIRED NUMBER OF POINTS TO BE ACCUMULATED
C  NOCAL: INPUT . THE MAXIMUM NUMBER OF CALLS TO ACSQ PERMITTED.
C                 NOCAL = MIN(100000,NOCAL)
C  VALTA: INPUT . THE TARGET VALUE,THAT SETS THE CRITERION.
C  IERFL: OUTPUT. = 0,1 FOR NORMAL / ABNORMAL TERMINATION
C  IPOI : OUTPUT. THE NUMBER OF POINTS THAT ARE FINALY ACCUMULATED.
C  THE POINTS ALONG WITH THE CORRESPONDING ACSQ VALUE ARE DISPOSED IN
C  MERLIN'S FILE   INIPO  .
C  THIS ROUTINE TERMINATES NORMALLY (IERFL=0) IF EITHER
C    NOPOI POINTS ARE ACUMULATED OR
C  IF  ACSQ IS CALLED NOCAL TIMES ,WHATEVER COMES FIRST.
C  TO USE THESE ROUTINE ALL NON-FIXED VARIABLES MUST BE BOUNDED
C  FROM BOTH ABOVE AND BELOW. BOTH LEFT AND RIGHT MARGINS SHOULD EXIST.
C  ABNORMAL TERMINATION (IERFL=1)RESULTS DUE TO LACK OF MARGINS
C  FOR A NON-FIXED VARIABLE OR DUE TO LACK OF INITIAL POINT VALUES.
      PARAMETER ( MXV = 200 )
      COMMON/CWORK1/XT(MXV),XT1(MXV),XT2(MXV),XT3(MXV),XT4(MXV)
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      ZERO = 0.
      ONE = 1.
      IERFL=0
      IMAX=ITNOCS+NOCAL
C  SET THE COUNTER TO ZERO
      IPOI=0
C  CHECK IF INITIAL VALUES EXIST
       IF(ITNOCS.EQ.0) THEN
        IF(STRO)WRITE(IUOUT,*)'INITIAL VALUES MISSING'
        IERFL=1
        IF(STRO)WRITE(IUOUT,*)' POINT ACCUMULATION ABANDONED'
        RETURN
       ENDIF
C  CHECK FOR PROPER FRAMING OF THE NON-FIXED VARIABLES.
       DO 1 I=1,NOD
         IF(MARG(I).NE.2.AND.IXAT(I).NE.0)THEN
          IF(STRO)WRITE(IUOUT,6) I
          IERFL=1
         ENDIF
    1  CONTINUE
       IF(IERFL.EQ.1)THEN
        IF(STRO)WRITE(IUOUT,*)'RESET MARGIN AND/OR FIX-STATUS PROPERLY'
        RETURN
       ENDIF
C   THE FOLLOWING LOOP SAMPLES POINTS AT RANDOM
       DO 4 ITER=1,100000
C   SAMPLE A POINT IN BETWEEN ITS MARGINS.
         DO 2 I=1,NOD
          XT(I)=POINT(I)
          XLDIF=(XRL(I)-XLL(I))*RANDI(ZERO,ONE)+XLL(I)-XT(I)
          XT(I)=XT(I)+IXAT(I)*XLDIF
    2    CONTINUE
        VALT=ACSQ(XT,NOD)
C  CHECK IF THE TRIAL VALUE IS LESS THAN THE TARGET VALUE
C  IF THE CRITERION IS SATISFIED DISPOSE POINT AND VALUE TO INIPO
         IF(VALT.LT.VALTA) THEN
           DO 3 I=1,NOD
            POINT(I)=XT(I)
    3      CONTINUE
          VAL=VALT
          IPOI=IPOI+1
          CALL POSIT(IUINI)
          CALL WRITER(IUINI)
         ENDIF
        IF(ITNOCS.GE.IMAX)GO TO  5
        IF(IPOI.GE.NOPOI)GO TO  5
    4  CONTINUE
      IF(STRO)WRITE(IUOUT,*)'NUMBER OF CALLS SUPERSEDED MAXIMUM ALLOWED'
    5 CONTINUE
       IF(IPOI.EQ.0) THEN
        IF(STRO)WRITE(IUOUT,7)
       ELSE
        IF(STRO)WRITE(IUOUT,8)IPOI
       ENDIF
      RETURN
    6 FORMAT(3X,'VARIABLE NO.',I3,' IS NEITHER FRAMED NOR FIXED')
    7 FORMAT(T5,'* * *  FAILED TO ACCUMULATE ANY NEW POINTS * * *')
    8 FORMAT(T10,'NUMBER OF POINTS  APPENDED TO FILE INIPO:',I4)
      END
      FUNCTION ACSQ ( X, N )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   THIS IS THE FUNCTION THAT MERLIN MINIMIZES. IT CONTAINS A COUNTER
C   AND UPDATES SOME COMMON BLOCK PARAMETERS. ITS VALUE IS SET EQUAL
C   TO THE USER'S FUNCTION   FUNMIN(X,N)
C   INPUT: X ARRAY DIMENSIONED N CONTAINING THE FUNCTION'S PARAMETERS
C
      DIMENSION X(N)
      COMMON/UPDATE/IGRADU,IHESSU
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
      COMMON /MODE3/ MODEV, IFFORM
      COMMON/PARAMX/NOF
      PARAMETER ( MXT = 200 )
      DIMENSION F(MXT)
C
C  DEVELOPMENT ONLY: SOMETHING'S WRONG IF THE OBJECTIVE FUNCTION IS
C  CALLED WHILE NOEVAL IS IN EFFECT.
      IF (MODEV.EQ.0) THEN
         WRITE (*,*) 'ACSQ CALLED WITH MODEV 0'
         STOP 'ERROR'
      END IF
      ITNOCS=ITNOCS+1
      IPARTC=IPARTC+1
      IHESSU=0
      IGRADU=0
      IF (IFFORM.EQ.0) THEN
         ACSQ = FUNMIN(X,N)
      ELSE
         CALL SUBSUM(NOF,N,X,F)
         SUM = 0.
         DO 20,I=1,NOF
20          SUM = SUM+F(I)**2
         ACSQ = SUM
      END IF
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE LSQFCN ( M, N, X, F )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(N), F(M)
C
      COMMON/UPDATE/IGRADU,IHESSU
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
      COMMON /MODE3/ MODEV, IFFORM 
C
C  DEVELOPMENT ONLY: SOMETHING'S WRONG IF THE OBJECTIVE FUNCTION IS
C  CALLED WHILE NOEVAL IS IN EFFECT.
      IF (MODEV.EQ.0) THEN
         WRITE (*,*) 'LSQFCN CALLED WITH MODEV 0'
         STOP 'ERROR'
      END IF
C
      ITNOCS=ITNOCS+1
      IPARTC=IPARTC+1
      IHESSU=0
      IGRADU=0
C
      CALL SUBSUM(M,N,X,F)
      END
      FUNCTION ACSQ1 ( X )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( MXV = 200 )
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACS/II
C
      XSAVE = POINT(II)
      POINT(II) = X
      ACSQ1 = ACSQ(POINT,NOD)
      POINT(II) = XSAVE
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE NEWALI
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      PARAMETER ( MAXALI = 20 )
      CHARACTER*5 ALIAS(MAXALI)
      CHARACTER*10 ALICOM(MAXALI)
      COMMON / ALICO1 / ALIAS, ALICOM
      COMMON / ALICO2 / NALIAS
C
      PARAMETER ( NCOM = 100 )
      CHARACTER*10 COMMAN(NCOM)
      INTEGER CLASS(NCOM), IND(NCOM)
      COMMON /COM1/ COMMAN
      COMMON /COM2/ CLASS, IND
C
      CHARACTER*5 ALI
      CHARACTER*10 COM
      INTEGER IS(1), IE(1)
C
      IF (NALIAS.EQ.MAXALI) THEN
         IF (STRO) WRITE (IUOUT,30)
         CALL BATCH
         RETURN
      END IF
C
      IF (STRO) WRITE (IUOUT,10)
      CALL INLINE(ALI,.FALSE.,.TRUE.,IS,IE,NIT,IERR)
      IF (IERR.NE.0) RETURN
      DO 40,I=1,NALIAS
         IF (ALI.EQ.ALIAS(I)) THEN
            IF (STRO) WRITE(IUOUT,50)
         CALL BATCH
         RETURN
         END IF
40    CONTINUE
C
      LE = LENGTH(ALI)
      CALL MATCH(COMMAN,NCOM,ALI,LE,ICOM)
      IF (ICOM.GT.0) THEN
         IF (STRO) WRITE (IUOUT,60) COMMAN(ICOM)(1:LENGTH(COMMAN(ICOM)))
      END IF
C
      IF (STRO) WRITE (IUOUT,20)
      CALL INLINE(COM,.FALSE.,.TRUE.,IS,IE,NIT,IERR)
      IF (IERR.NE.0) RETURN
C
C  DETERMINE WHICH POSITION (IPOS) THE ALIAS SHOULD OCCUPY
C  IN THE SORTED ARRAY.
      DO 70,I=1,NALIAS
         IF (ALI.LT.ALIAS(I)) THEN
            IPOS = I
         GOTO 80
         END IF
70    CONTINUE
      IPOS = NALIAS+1
C
C  INSERT THE ALIAS IN PLACE.
80    CONTINUE
      DO 90,I=NALIAS,IPOS,-1
         ALIAS(I+1) = ALIAS(I)
         ALICOM(I+1) = ALICOM(I)
90    CONTINUE
      ALIAS(IPOS) = ALI
      ALICOM(IPOS) = COM
      NALIAS = NALIAS+1
C
10    FORMAT(T2,'ENTER ALIAS')
20    FORMAT(T2,'ENTER ACTUAL COMMAND')
30    FORMAT(T2,'YOU HAVE TOO MANY ALIASES. CANNOT DECLARE ANY MORE.')
50    FORMAT(T2,'THIS ALIAS EXISTS ALREADY.')
60    FORMAT(T2,' WARNING: THIS ALIAS WILL OVERRIDE THE ',A,' COMMAND.')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE ALIDIS
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      PARAMETER ( MAXALI = 20 )
      CHARACTER*5 ALIAS(MAXALI)
      CHARACTER*10 ALICOM(MAXALI)
      COMMON / ALICO1 / ALIAS, ALICOM
      COMMON / ALICO2 / NALIAS
C
      IF (STRO) THEN
         IF (NALIAS.EQ.0) THEN
            WRITE (IUOUT,40)
         ELSE
            WRITE (IUOUT,10)
            DO 20,I=1,NALIAS
               WRITE (IUOUT,30) ALIAS(I), ALICOM(I)
20          CONTINUE
         END IF
      END IF
C
10    FORMAT(T2,'ALIAS',T15,'COMMAND'/T2,'-----',T15,'-------')
30    FORMAT(T2,A,T15,A)
40    FORMAT(/T2,'YOU HAVE NOT DEFINED ANY ALIASES.')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE UNALI
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      PARAMETER ( MAXALI = 20 )
      CHARACTER*5 ALIAS(MAXALI)
      CHARACTER*10 ALICOM(MAXALI)
      COMMON / ALICO1 / ALIAS, ALICOM
      COMMON / ALICO2 / NALIAS
C
      CHARACTER*5 ALI
      INTEGER IS(1), IE(1)
C
      IF (STRO) WRITE (IUOUT,10)
      CALL INLINE(ALI,.FALSE.,.TRUE.,IS,IE,NIT,IERR)
      IF (IERR.NE.0) RETURN
      DO 40,I=1,NALIAS
         IF (ALI.EQ.ALIAS(I)) THEN
            DO 50,J=I+1,NALIAS
            ALIAS(J-1) = ALIAS(J)
            ALICOM(J-1) = ALICOM(J)
50          CONTINUE
            NALIAS = NALIAS-1
         RETURN
         END IF
40    CONTINUE
      IF (STRO) WRITE (IUOUT,20)
      CALL BATCH
C
10    FORMAT(T2,'ENTER ALIAS TO DELETE')
20    FORMAT(T2,'THERE IS NO SUCH ALIAS.')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE AUTIDA(IUOUT)
C
C  THIS ROUTINE ISSUES INFORMATION ABOUT THE AUTHORS OF THE PROGRAM
C
 2    FORMAT(2X,27('.'),T35,'         M E R L I N - 2.0  PLUS')
 22   FORMAT(2X,27('.'))
 8    FORMAT(2X,27('.'),T40,'E-MAIL:  MERLIN @ GRIOANUN.BITNET')
 88   FORMAT(2X,27('.'),T40,'         MERLIN @ CC.UOI.GR')
      CALL DASH
      WRITE(IUOUT,*)'  '
      WRITE(IUOUT,2)
      WRITE(IUOUT,22)
      WRITE(IUOUT,22)
      WRITE(IUOUT,8)
      WRITE(IUOUT,88)
      WRITE(IUOUT,*)'  '
      CALL DASH
 9    FORMAT(2X,'TO OBTAIN ON-LINE INFORMATION, MAKE SURE THAT ')
 10   FORMAT(22X,' FILE:  HELP  , IS PRESENT')
      WRITE(IUOUT,9)
      WRITE(IUOUT,10)
      CALL DASH
      END
C
C
      SUBROUTINE MAGIC(NTOT,VALOW,VERSIM)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  THIS ROUTINE IS AN IMPLEMENTATION OF AN AUTOMATIC ROUTE
C  TO MULTIDIMENSIONAL FUNCTION MINIMIZATION.
C  SINCE IT IS QUITE GENERAL, IT MAY NOT BE VERY EFFICIENT FOR
C  ALL FUNCTION CLASSES. THE USER IS ADVICED THEN TO LEARN TO
C  OPERATE MERLIN MANUALLY.
C
      PARAMETER ( MXV = 200 )
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      COMMON/UPDATE/IGRADU,IHESSU
      COMMON/CWORK1/XWORK(MXV),XT1(MXV),XT2(MXV),XT3(MXV),XT4(MXV)
      COMMON/CWORK2/XSTEP(MXV),FMIN(0:MXV)
      COMMON/DIRCOM/DIRCOS(MXV)
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
      COMMON/PRILOG/STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      DIMENSION VERSIM(*),IERR(MXV),RATE(4),RATES(4),ERR(4),IMET(4)
      DIMENSION EXAU(4),METH(4)
      INTEGER SCALLS , FCALLS
      LOGICAL STRO,WEAK,EXAU
      CHARACTER*10 METH,METHOD
      DATA METH / 'ROLL' , 'SIMPLEX' , 'BFGS' , 'RANDOM' /
      DATA  ERR1S , ACCH , IFMH , ICYSR , IFAILR , ACCB , ERBD /
     >.001, .001 ,   3  ,  40   ,   4    ,  .01, .001 /
C  INITIALIZE PARAMETERS
      ZERO=0.
      NREM=NTOT
      ERR(1)=ACCH
      ERR(2)=ERR1S
      ERR(3)=ACCB
      ERR(4)=0.
      ERBO=ERBD
      IFM=IFMH
      ICYS=ICYSR
      IFAILN=IFAILR
      NOV=NOD
       DO 1 I=1,4
        EXAU(I)=.TRUE.
        RATE(I)=1.
    1  CONTINUE
C
C  END OF INITIALIZATION
C
C   POSITION FILE BACKUP TO END OF INFORMATION.
C
      CALL POSIT(IUBAC)
C
C   ADJUST INITIAL STEPS
C
      ISTECL=ITNOCS
      IF(DERIVA.EQ.'NUMER') CALL PRADJ
       IF(DERIVA.EQ.'ANAL') THEN
       DERB = 0.0001
        CALL DERMER(NOD,POINT,VAL,DIRCOS,NODC,DERB,IERR)
         DO 2  I=1,NOD
          DIRCOS(I)=DIRCOS(I)*WALL(I,4,DIRCOS(I))
    2    CONTINUE
        MCAL1=30
        ER1=0.
C  SET "UPDATE STEPS"-FLAG
        IUS = 1
        CALL MINDIR(MCAL1,ER1,IUS,NOC1,ALPHA)
       ENDIF
      NREM=NREM - ITNOCS + ISTECL
C
C   DECISION SESSION STARTS HERE
    3 CONTINUE
C   FORM LIST OF CANDIDATE METHODS
       DO 4 I=1,4
        IMET(I)=I
        RATES(I)=-RATE(I)
    4  CONTINUE
C
C  ARRANGE THEM IN DESCENTING RATE ORDER
C
      CALL REORDE(RATES,IMET,4)
C
C   DECIDE WHICH METHOD TO CHOOSE
C
C   CHECK IF HIGHEST RATE IS ZERO
      IF(RATE(IMET(1)).EQ.0.)GO TO  6
C   IF SO PROCEED WITH SPECIAL HANDLING
       DO 5 I=1,4
        J=IMET(I)
C  PICK A CANDIDATE
        METHOD=METH(J)
C  CHOOSE IT , IF EXAU IS .TRUE.
        IF(EXAU(J)) GO TO  8
C  CHOOSE IT IF ERR(J) .GT. COMTOL
        IF ( ERR(J) .GT. COMTOL ) GO TO  8
    5  CONTINUE
C
C  IF ALL CANDIDATES ARE REJECTED PROCEED WITH THE FOLLOWING ROUTE
C
    6 CONTINUE
      NOC = MAX( NREM/4 , 600 )
      NOCH=NOC
      NOCR=NOC
      CALL REPORT(0,'ROLL')
      CALL DWRITE
      ACCU = 0.
      ENH = 2.
      CALL HYPER(NOC,ACCU,ENH,NOCH,0,IC,3,ZERO)
      CALL REPORT(1,'ROLL')
C  CHECK IF THE TARGET VALUE IS REACHED, AND IF SO QUIT.
      IF(VAL.LE.VALOW) GO TO  7
      CALL REPORT(0,'RANDOM')
      CALL DWRITE
      REDF = 0.85
      CALL RANDS(NOC,0,IC,0,0,REDF,ICYS,NOCR,0,ZERO,ZERO)
      CALL REPORT(1,'RANDOM')
      IF(VAL.LE.VALOW) GO TO  7
      CALL REPORT(0,'SIMPLEX')
      CALL DWRITE
      ERRS = 0.
      ERRI = 0.
      CALL SIMPL(VERSIM,NOV,1,ERRS,ERRI,NOC,100,0,IC)
      CALL REPORT(1,'SIMPLEX')
      IF(VAL.LE.VALOW) GO TO  7
      CALL REPORT(0,'BFGS')
      CALL DWRITE
      SML = 0.
      ERB = 0.
      CALL BFGSMV(VERSIM,NOC,IC,SML,IERR,3,ERB,0,ZERO)
      CALL REPORT(1,'BFGS')
      IF(VAL.LE.VALOW) GO TO  7
      RETURN
    7 CONTINUE
      IF (STRO) THEN
          CALL DASH
          CALL MID1('TARGET VALUE HAS BEEN REACHED')
          CALL DASH
      END IF
      RETURN
C
C   SET PARAMETERS FOR THE CHOSEN METHOD
C
    8 CONTINUE
      NOC = MAX (NREM/10 , 500 )
      NOC = MIN (NOC , 2000 )
       DO 9 I=1,4
        IF( (METHOD.EQ.METH(I)).AND.EXAU(I) ) GO TO  11
    9  CONTINUE
C
C   REVISE CALLING PARAMETERS IF ALL EXAU'S ARE .FALSE.
C
       DO 10 I=1,4
         IF (METHOD.EQ.METH(I)) THEN
          ERR(I) = ERR(I)/100.
           IF ( I.EQ.3 ) THEN
            ERBO = ERBO/100.
            IF(DERIVA.EQ.'NUMER') NOC = 1.3*NOC
           ENDIF
          IF ( I.EQ.1 ) IFM = IFM + 2
           IF ( I.EQ.4 ) THEN
            ICYS = 1.5*ICYS
            IFAILN = 1.5*IFAILN
            NOC = 2*NOC
           ENDIF
          GO TO  11
         ENDIF
   10  CONTINUE
C
C  EXECUTION SESSION STARTS HERE.
C
   11 CONTINUE
C
C   CHECK IF THERE ARE ANY CALLS LEFT. IF NOT QUIT.
C
       IF( NREM .LE. 0 ) THEN
        IF (STRO) THEN
           CALL DASH
           CALL MID1('ALL CALLS EXAUSTED')
           CALL DASH
        END IF
        RETURN
       ENDIF
C
C   REPORT STARTING STATUS, PRIOR TO EXECUTION.
C
      CALL REPORT( 0 , METHOD )
      JMET=I
C
C  UPDATE STARTING VALUES
C
      SVAL=VAL
      SCALLS=ITNOCS
C  BEGIN EXECUTION
      CALL DWRITE
      ENH = 3.
      ERRS = 0.
      REDF = 0.85
      IF(JMET.EQ.1) CALL HYPER(NOC,ERR(1),ENH,IFM,0,IC,3,ZERO)
      IF(JMET.EQ.2) CALL SIMPL(VERSIM,NOV,1,ERRS,ERR(2),NOC,60,0,IC)
      IF(JMET.EQ.3) CALL BFGSMV(VERSIM,NOC,IC,ERR(3),IERR,3,ERBO,0,ZERO)
      IF(JMET.EQ.4) 
     >  CALL RANDS(NOC,0,IC,0,1,REDF,ICYS,IFAILN,NOV,ZERO,ZERO)
C
C   UPDATE POST EXECUTION VALUES
C
      FCALLS = ITNOCS
      FVAL = VAL
C
C  CALCULATE RATE , REMAINING CALLS , EXAU
C
      NOCM = FCALLS - SCALLS
      DROP = SVAL - FVAL
       IF ( DROP .EQ. 0. ) THEN
        RATE(JMET)=0.
       ELSE
        RATE(JMET) = ABS(DROP)/(1. + NOCM )/MAX( ABS(SVAL) , ABS(FVAL) )
       ENDIF
      NREM = NREM - NOCM
      EXAU(JMET) = NOCM.GE.NOC
C
C  REPORT POST EXECUTION STATUS
C
      CALL REPORT(1,METHOD)
C
C  CHECK IF TARGET VALUE IS REACHED AND IF SO QUIT.
C
       IF ( VAL .LE. VALOW ) THEN
        IF (STRO) THEN
           CALL DASH
           CALL MID1('TARGET VALUE HAS BEEN REACHED')
           CALL DASH
        END IF
        RETURN
       ENDIF
C
C  START A NEW CYCLE . TRANSFER CONTROL TO DECISION SESSION.
C
      GO TO  3
      END
C
C
      SUBROUTINE REPORT(IND,METHOD)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  REPORTS THE MINIMIZATION STATUS, DURING THE AUTOMATIC
C   MINIMIZATION SESSION.
C   IND=0,1 ARRANGES TO ISSUE THE FUNCTION'S VALUE OR THE PARAMETER
C   AND THE FUNCTION'S VALUE.
C   METHOD IS A CHARACTER VARIABLE SPECIFYING THE METHOD IN USE.
C
      PARAMETER ( MXV = 200 )
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
      CHARACTER*10 NICKNA
      COMMON/NAMES/NICKNA(MXV)
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      LOGICAL STRO,WEAK
      CHARACTER*(*) METHOD
       IF(STRO) THEN
        WRITE(IUOUT,1)
        WRITE(IUOUT,2) ITNOCS
        IF(IND.EQ.0) WRITE(IUOUT,3) METHOD,VAL
        IF(IND.EQ.1) WRITE(IUOUT,4) METHOD
        IF(IND.EQ.1)CALL WRITER(IUOUT)
       ENDIF
       CALL BWRITE
      RETURN
    1 FORMAT(//2X,25('-'),2X,'REPORT',2X,25('-'))
    2 FORMAT(T10,'TOTAL NUMBER OF CALLS : ',I8)
    3 FORMAT(T10,A10,' METHOD STARTS AT : ',1PG21.14)
    4 FORMAT(T10,A10,' METHOD ENDED AT : ')
      END
C
      SUBROUTINE REORDE(VEC,IAL,N)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  ARRANGES THE ELEMENTS OF ARRAY VEC IN ASCENDING ORDER
C  AND SIMULTANEUSLY PERFORMS THE SAME TRANSFORMATION TO
C  THE ELEMENTS OF ARRAY IAL. N IS THE DIMENSIONALITY OF THE ARRAYS.
C
      DIMENSION VEC(N),IAL(N)
      NM=N-1
       DO 2 J=1,NM
        K=J
        SML=VEC(J)
         DO 1 I=J,N
           IF(SML.GT.VEC(I))THEN
            K=I
            SML=VEC(I)
           ENDIF
    1    CONTINUE
        TEMP=VEC(K)
        ITEMA=IAL(K)
        VEC(K)=VEC(J)
        IAL(K)=IAL(J)
        VEC(J)=TEMP
        IAL(J)=ITEMA
    2  CONTINUE
      RETURN
      END
      SUBROUTINE BATCH
C
C   BATCH... HANLES I/O ERRORS DURING EXECUTION IN BATCH OPTION.
C   UPON AN ERROR DETECTION THE PROGRAM ABORTS ISSUING AN
C   INFORMATIVE MESSAGE.
C   WHEN IN IAF OPTION THE ROUTINE HAS NO EFFECT.
C
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      IF(PROCES.EQ.'IAF')RETURN
      STOP 'MERLIN ABORT. BATCH ERROR - PROCESSING'
      END
      SUBROUTINE BFGSMV(H,NOMAXC,NOCEX,SML,IERR,IDIG,ERB,IPRI,EPSB)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   BFGSMV ... MINIMIZES THE FUNCTION ACSQ , USING THE BFGS ALGORITM
C   DUE TO BROYDEN,FLETCHER,GOLDFARB AND SHANNO.
C          NOMAXC: MAXIMUM NUMBER OF CALLS
C          NOCEX : NUMBER OF CALLS MADE TO ACSQ
C          SML   : TERMINATION CRITERION FOR THE LINE SEARCH
C          IERR  : ARRAY CONTAINING THE ERROR CODES THAT INDICATE
C                  THE ACCURACY OF THE NUMERICAL CALCULATION
C                  OF THE GRADIENT
C          IDIG  : INPUT FOR THE SECOND ARGUMENT OF THE FUNCTION WALL
C          ERB   : ERROR BOUND FOR THE NUMERICAL CALCULATION OF THE GRAD
C          IPRI  : (0,1,2) ARRANGES THE PRINTOUT LEVEL
C          EPSB  : MINIMUM RELATIVE DROP ACCEPTED.
C
      PARAMETER ( MXV = 200 )
      COMMON/UPDATE/IGRADU,IHESSU
      COMMON/CWORK1/XWORK(MXV),XT1(MXV),XT2(MXV),XT3(MXV),XT4(MXV)
      COMMON/DIRCOM/DIRCOS(MXV)
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/CWORK2/XSTEP(MXV),FMIN(0:MXV)
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      DIMENSION H(NOD,NOD),IERR(NOD)
C
      NOCEX=0
C  INITIALIZE TERMINATION CRITERIA SETTINGS.
      CALL INITER(IPRI,NOMAXC,EPSB)
C  CHECK IF ANY OF THE TERMINATION CRITERIA ARE MET.
      IF (ITERM(1,1,0).NE.0) GOTO 50
      ALPHA=1.
       DO 2 I=1,NOD
        STEP(I)=STEP(I)*IXAT(I)
    2  CONTINUE
       IF(IGRADU.EQ.0) THEN
        CALL DERMER(NOD,POINT,VAL,DIRCOS,NOCS1,ERB,IERR)
         DO 3 I=1,NOD
          GR=DIRCOS(I)
          DIRCOS(I)=DIRCOS(I)*WALL(I,IDIG,GR)
    3    CONTINUE
        NOCEX=NOCEX+NOCS1
       ENDIF
       IF(IHESSU.EQ.0) THEN
         DO 4 I=1,NOD
           DO 4 J=I,NOD
            H(I,J)=0.
            H(J,I)=0.
    4    CONTINUE
         DO 5 I=1,NOD
          H(I,I)=1.
    5    CONTINUE
        IHESSU=1
       ENDIF
C    THE FOLLOWING LOOP PERFORMS BFGS ITERATIONS
C
       DO 18 ITERAT=1,100000
C   CALCULATE STEP
         DO 7 I=1,NOD
          XSTEP(I)=0.
           DO 6 J=1,NOD
    6      XSTEP(I)=XSTEP(I)-H(I,J)*DIRCOS(J)
           XSTP=-XSTEP(I)
           XSTEP(I)=XSTEP(I)*WALL(I,IDIG,XSTP)
    7    CONTINUE
C  STORE DERIVATIVES IN ARRAY XT1 , AND STEPS IN ARRAY DIRCOS
         DO 8 I=1,NOD
          XT1(I)=DIRCOS(I)
    8    DIRCOS(I)=XSTEP(I)
        OLDVAL=VAL
C  PERFORM A LINE SEARCH ALONG THE STEP DIRECTION
C  SET MAXIMUM NUMBER OF CALLS
        IMN = 100
C  SET "UPDATE STEPS"-FLAG
        IUS = 1
        CALL MINDIR(IMN,SML,IUS,NOCS2,ALPHA)
        NOCEX=NOCEX+NOCS2
        IF (VAL.LT.OLDVAL) CALL DISPLA
C  CHECK IF ANY OF THE TERMINATION CRITERIA ARE MET.
        IF (ITERM(1,1,1).NE.0) GOTO 19
C  CALCULATE DERIVATIVES AT THE POINT THE LINE SEARCH STOPPED
        CALL DERMER(NOD,POINT,VAL,XT2,NOCS3,ERB,IERR)
         DO 12 I=1,NOD
          GR=XT2(I)
          XT2(I)=XT2(I)*WALL(I,IDIG,GR)
   12    CONTINUE
        NOCEX=NOCEX+NOCS3
C  CALCULATE THE DIFFERENCE IN THE GRADIENT AS XT3
C  UPDATE DERIVATIVES IN DIRCOS
         DO 13 I=1,NOD
          XT3(I)=XT2(I)-XT1(I)
   13    DIRCOS(I)=XT2(I)
C   CALCULATE AUXILIARY ARRAY XT4(K)= XT3(L)*H(L,K)
         DO 15 K=1,NOD
          XT4(K)=0.
           DO 14 L=1,NOD
   14      XT4(K)=XT4(K)+XT3(L)*H(L,K)
   15    CONTINUE
C   CALCULATE  UPDATING FACTORS.
        A2=0.
        DIVIS=0.
         DO 16 K=1,NOD
          A2=A2+XT3(K)*XT4(K)
   16    DIVIS=DIVIS+XSTEP(K)*XT3(K)
        IF(DIVIS.EQ.0.) THEN
           CALL SETCOD(2)
           GO TO  19
        ENDIF
        AFAC=(ALPHA+A2/DIVIS)/DIVIS
        BFAC=-1./DIVIS
C   UPDATE INVERSE HESSIAN ACCORDING TO BFGS PRESCRIPTION
         DO 17 I=1,NOD
           DO 17 J=I,NOD
            H(I,J)=H(I,J)+AFAC*XSTEP(I)*XSTEP(J)+BFAC*(XSTEP(I)*XT4(J)+
     >XT4(I)*XSTEP(J))
   17    H(J,I)=H(I,J)
        IHESSU=1
   18  CONTINUE
C
   19 CONTINUE
      IF (ALPHA.NE.0.) THEN
         DO 20 I=1,NOD
   20      STEP(I)=ALPHA*XSTEP(I)
      END IF
   50 CALL TRINFO
      END
      SUBROUTINE BWRITE
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      IF (BACKUP.EQ.'LASTBACK') THEN
         REWIND IUBAC
         CALL WRITER(IUBAC)
      ELSE IF (BACKUP.EQ.'FULLBACK') THEN
         CALL POSIT(IUBAC)
         CALL WRITER(IUBAC)
      END IF
C
      END
      SUBROUTINE CCHFLA
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      PARAMETER ( MAXCFL = 10, MAXCLE = 30 )
      CHARACTER*(MAXCLE) CFLAG(MAXCFL)
      COMMON / CFLAGS / CFLAG
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      IF (STRO) WRITE (IUOUT,10)
      READ (IUINP,*) IC
      IF (IC.LT.0 .OR. IC.GT.MAXCFL) THEN
         IF (STRO) WRITE (IUOUT,30) MAXCFL
         CALL BATCH
         RETURN
      END IF
      IF (STRO) WRITE (IUOUT,20)
      READ (IUINP,'(A)') CFLAG(IC)
C
10    FORMAT(T2,'WHICH CHARACTER FLAG ?')
20    FORMAT(T2,'ENTER THE CHARACTER FLAG')
30    FORMAT(T2,'THERE ARE ONLY ',I2,' CHARACTER FLAGS.')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE CFLADI
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      PARAMETER ( MAXCFL = 10, MAXCLE = 30 )
      CHARACTER*(MAXCLE) CFLAG(MAXCFL)
      COMMON / CFLAGS / CFLAG
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      IF (STRO) THEN
         WRITE (IUOUT,30)
         DO 20,I=1,MAXCFL
20          WRITE (IUOUT,10) I, CFLAG(I)
      END IF
C
10    FORMAT (T2,'FLAG(',I2,')= ''',A,'''')
30    FORMAT (T2,'CURRENT CHARACTER FLAG VALUES'/T2,29('-'))
      END
      SUBROUTINE CHANGE(XARRAY,NOV,IEOR,ICIN)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     FACILITATES CHANGING OF THE ELEMENTS OF AN ONE-DIMENSIONAL ARRAY.
C     ICIN  TAKES ON THE VALUES 0,1,2,3.
C     ICIN=1,2,3 INVOKES SPECIAL ACTION TO FACILITATE CHANGING OF
C     XLL,XRL AND POINT  CORRESPONDINGLY.  ICIN=0 IS FOR GENERAL USE.
C     IEOR = 1,0 .OUTPUT PARAMETER DENOTING IF AN END OF RECORD
C     WAS OR WAS NOT ENCOUNTERED DURING THE ROUTINE'S EXECUTION.
C     IF IEOR =1,THE UNIT  IUINP IS REWOUND.
C     IEOR  =  2   INDICATES THAT THE FIRST NON-NULL/NON-COMMA
C      CHARACTER IS A  0 (ZERO)
C
      PARAMETER ( MXV = 200 )
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/INDEFI/INDEF(MXV)
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      DIMENSION XARRAY(NOV),TEMPO(MXV),LINDEF(MXV)
      CHARACTER SCRA*79,FORG*21,FORC*10,FORI*3
      IEOR=0
C
C  STORE ENTERING VALUES TO THE SCRATCH ARRAYS: TEMPO, LINDEF
C  TO RESTORE THEM IN CASE AN ERROR IN INPUT OCCURS.
C
      DO 1 I=1,NOV
        TEMPO(I)=XARRAY(I)
        LINDEF(I)=INDEF(I)
    1 CONTINUE
    2 CONTINUE
C
C  READ THE ONE-LINE INPUT, IN THE CHARACTER VARIABLE : SCRA
C
      READ(IUINP,8,END=3)SCRA
C
C  DETERMINE IF THE INPUT LINE IS THE QUIT CHARACTER ' 0 '
C
      J1=1
C
C  FIND THE LOCATION OF THE END OF THE WORD THAT STARTS AT J1
C
      CALL PICKW(SCRA,J1,J2,IEN)
      IF (IEN.EQ.0) RETURN
      IF(SCRA(J1:J2).EQ.'0')THEN
C
C          QUIT IN CASE THE WORD IS '0'
C
           IEOR=2
           RETURN
      ENDIF
      GO TO  4
    3 REWIND IUINP
      IEOR=1
C   IF IN BATCH OPTION OF THE PROCES MODE ABORT ON ERRORS
      CALL BATCH
      RETURN
    4 CONTINUE
      I1=1
C
C   K , TAKES TWO VALUES.  K=0 ,AND K=1.  K=0 INSTRUCTS THE ROUTINE
C  TO SEARCH FOR THE INDEX OF A VARIABLE , K=1 TO SEARCH FOR ITS VALUE.
C
      K=0
    5 CONTINUE
C
C  LOCATE THE END OF THE WORD STARTING AT I1.
C
      CALL PICKW(SCRA,I1,I2,IEN)
C
C  IF THE LINE IS A BLANK LINE OR CONTAINS ONLY BLANKS AND COMMAS QUIT
C
      IF(IEN.EQ.0)GO TO  7
       IF(K.EQ.0)THEN
         IF(CALLBY.EQ.'NAME')THEN
C
C  TREAT THE INPUT IN THE NAME OPTION OF THE CALLBY MODE.
C
          FORC=SCRA(I1:I2)
C
C  DETERMINE THE INDEX ( II ) OF THE VARIABLE'S NAME CONTAINED IN FORC
C
          II=IPARIN(FORC)
           IF(II.EQ.0)THEN
C
C  IF AN ILLEGAL NAME WAS GIVEN , QUIT / ABORT.
C
            IF(STRO)WRITE(IUOUT,9)FORC
            CALL BATCH
            RETURN
           ENDIF
C
C  IF CHANGING THE MERLIN'S  MARGIN ARRAYS (XLL OR XRL ), UPDATE  THE
C  MARGIN STATUS ARRAY : MARG.
C
           IF(ICIN.EQ.1)THEN
            IF(MARG(II).EQ.0)MARG(II)=-1
            IF(MARG(II).EQ.1)MARG(II)= 2
           ELSEIF(ICIN.EQ.2)THEN
            IF(MARG(II).EQ.0)MARG(II)= 1
            IF(MARG(II).EQ.-1)MARG(II)=2
           ENDIF
         ELSE
C
C   TREAT THE INDEX OPTION OF THE ' CALLBY ' MODE.
C
          FORI=SCRA(I1:I2)
          READ(FORI,12,IOSTAT=JJJ)II
          KK=IPARIN(FORI)
          IF(ICIN.EQ.0)KK=1
           IF(JJJ.NE.0.OR.KK.EQ.0)THEN
C
C  IF AN ERROR IN INPUT HAS OCCURED, RESTORE THE OLD VALUES OF THE
C  VARIABLES AND SEND CONTROL TO THE PROMPTING POINT.
C
            IF(STRO.AND.KK.NE.0)WRITE(IUOUT,11)
            IF(STRO.AND.KK.EQ.0)WRITE(IUOUT,10)
             DO 6 I=1,NOV
                XARRAY(I)=TEMPO(I)
                INDEF(I)=LINDEF(I)
    6        CONTINUE
C   IF IN BATCH OPTION OF THE PROCES MODE ABORT ON ERRORS
            CALL BATCH
            I1=1
            K=0
            GO TO  2
           ENDIF
           IF(ICIN.EQ.1)THEN
C
C  TREAT THE MARGIN STATUS AS BEFORE.
C
            IF(MARG(II).EQ.0)MARG(II)=-1
            IF(MARG(II).EQ.1)MARG(II)= 2
           ELSEIF(ICIN.EQ.2)THEN
            IF(MARG(II).EQ.0)MARG(II)= 1
            IF(MARG(II).EQ.-1)MARG(II)=2
           ENDIF
         ENDIF
       ELSE
        FORG=SCRA(I1:I2)
C
C  ASSIGN THE VALUE CONTAINED IN FORG, TO THE ARRAY : XARRAY
C
        READ(FORG,13)XARRAY(II)
C
C   IF A FIXED VARIABLE IS ALTERED , LOOSE IT.
C
        IF(ICIN.EQ.3)THEN
          IXAT(II)=1
          INDEF(II)=1
        ENDIF
       ENDIF
C
C  IF AN INDEX WAS DETERMINED , CHANGE K TO 1-K  TO TREAT THE
C  NEXT WORD WHICH SHOULD CONTAIN THE VALUE FOR THE VARIABLE
C  THE INDEX OF WHICH IS JUST DETERMINE, AND VICE VERSA.
C
      K=1-K
      I1=I2+1
      GO TO  5
    7 CONTINUE
      RETURN
    8 FORMAT(A79)
    9 FORMAT(2X,'NON - EXISTANT NAME : ',A10,' REST OF CHANGES IGNORED')
C   IF IN BATCH OPTION OF THE PROCES MODE ABORT ON ERRORS
   10 FORMAT(T5,'WRONG INPUT INDEX'/T15,'CHANGES CANCELED')
   11 FORMAT(/2X,'CALL BY NAME DISALLOWED IN  INDEX MODE'/
     >T22,' CHANGES CANCELED')
   12 FORMAT(BN,I3)
   13 FORMAT(BN,G21.0)
      END
      SUBROUTINE CHMAR
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( MXV = 200 )
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      PARAMETER ( MEM = 2500 , LSTA = 500 )
      COMMON/MCLVAR/STACK(LSTA),CMEM(MEM),ISTACK,INFILE
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
C
      INTEGER INDE(40)
      DIMENSION VALUE(40)
C
16    CONTINUE
C
C  SET LEFT MARGINS TO VARIABLES
      IF (STRO) WRITE (IUOUT,107)
      CALL GETIV(INDE,VALUE,NPAIRS,IERR)
      IF (IERR.EQ.0) THEN
         DO 10,I=NPAIRS,1,-1
            IND = INDE(I)
            XLL(IND) = VALUE(I)
            IF (MARG(IND).EQ.0) MARG(IND) = -1
            IF (MARG(IND).EQ.1) MARG(IND) = 2
10       CONTINUE
      END IF
C
C  SET RIGHT MARGINS TO VARIABLES
      IF (STRO) WRITE(IUOUT,108)
      CALL GETIV(INDE,VALUE,NPAIRS,IERR)
      IF (IERR.EQ.0) THEN
         DO 20,I=NPAIRS,1,-1
            IND = INDE(I)
            XRL(IND) = VALUE(I)
            IF (MARG(IND).EQ.0) MARG(IND) = 1
            IF (MARG(IND).EQ.-1) MARG(IND) = 2
20       CONTINUE
      END IF
C
      ICHECK = 0
      DO 30,I=1,NOD
         IF (XLL(I).GT.XRL(I)) THEN
            IF (STRO) WRITE(IUOUT,109) I
            ICHECK = 1
         ELSE IF (XLL(I).EQ.XRL(I)) THEN
C  IF LEFT AND RIGHT MARGINS COINCIDE , FIX THE VARIABLE.
            IXAT(I) = 0
            IF (STRO) WRITE (IUOUT,110) I
         END IF
30    CONTINUE
      IF (ICHECK.EQ.1) THEN
         IF (STRO) WRITE (IUOUT,111)
         IF (IUINP.NE.INFILE) GOTO 16
         CALL BATCH
      END IF
      IF (ITNOCS.EQ.0) RETURN
C  CHECK IF POINT IS INSIDE THE MARGINS. IF NOT READJUST IT.
      CALL POIMAR(IFEV)
C UPDATE FILE : DATA
      CALL DWRITE
C
 107  FORMAT(20X,'ENTER LEFT  MARGINS SEQUENCE')
 108  FORMAT(20X,'ENTER RIGHT MARGINS SEQUENCE')
 109  FORMAT(20X,'ILLEGAL MARGINS IN VARIABLE NO. ',I3)
 110  FORMAT(20X,'VARIABLE NO. ',I3,' EFFECTIVELY FIXED')
 111  FORMAT(20X,'RESET MARGINS APROPRIATELY')
      END
      SUBROUTINE CHPOI ( NPAIRS )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( MXV = 200 )
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/INDEFI/INDEF(MXV)
C
      INTEGER INDE(40)
      DIMENSION VALUE(40)
C
      CALL GETIV(INDE,VALUE,NPAIRS,IERR)
      IF (IERR.EQ.0) THEN
         DO 10,I=1,NPAIRS
            IND = INDE(I)
            POINT(IND) = VALUE(I)
            IXAT(IND) = 1
            INDEF(IND) = 1
10       CONTINUE
      END IF
C
      END
      SUBROUTINE CHSTEP
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( MXV = 200 )
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
C
      INTEGER INDE(40)
      DIMENSION VALUE(40)
C
      CALL GETIV(INDE,VALUE,NPAIRS,IERR)
      IF (IERR.EQ.0) THEN
         DO 10,I=1,NPAIRS
            STEP(INDE(I)) = VALUE(I)
10       CONTINUE
      END IF
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE CMSORT ( COMMAN, ICLASS, IND, NCOM )
C
      CHARACTER*(*) COMMAN(NCOM)
      INTEGER ICLASS(NCOM), IND(NCOM)
C
      LOGICAL SORTED
      CHARACTER*15 CTEMP
C
      DO 10,K=1,NCOM-1
        SORTED = .TRUE.
        DO 20,I=NCOM,K+1,-1
          IF (COMMAN(I).LT.COMMAN(I-1)) THEN
             SORTED = .FALSE.
          IM1 = I-1
             CTEMP = COMMAN(I)
             COMMAN(I) = COMMAN(IM1)
             COMMAN(IM1) = CTEMP
             ITEMP = ICLASS(I)
             ICLASS(I) = ICLASS(IM1)
             ICLASS(IM1) = ITEMP
             ITEMP = IND(I)
             IND(I) = IND(IM1)
             IND(IM1) = ITEMP
          END IF
 20     CONTINUE
        IF (SORTED) RETURN
 10   CONTINUE
C
      END
      SUBROUTINE COMPDR(IERR,ICH,DMODE)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C   ICH = 0 ,1 . FOR ICH=0 DISPLAYS PARTIAL DERIVATIVES ALONG WITH
C   THEIR ERROR CODES. FOR ICH=1 COMPARISON OF USER SUPPLIED PARTIAL
C   DERIVATIVES VERSUS MERLIN NUMERICALLY CALCULATED DERIVATIVES
C   TAKES PLACE.
C
C   IERR : ARRAY CONTAINING THE ERROR CODES FOR EACH PARTIAL
C   DERIVATIVE
C
C
      PARAMETER ( MXV = 200 )
      CHARACTER*(*) DMODE
      COMMON/CWORK1/XT(MXV),XT1(MXV),XT2(MXV),XT3(MXV),XT4(MXV)
      COMMON/DIRCOM/DIRCOS(MXV)
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER NICKNA*10
      COMMON/NAMES/NICKNA(MXV)
      COMMON/ERRBOU/GLERB
      CHARACTER*5 UMODE
      COMMON/UPMODE/UMODE
      COMMON/LOCAL/GRALOC(MXV),LERR(MXV)
      DIMENSION IERR(NOD)
C
      CHARACTER*5 GRADI
      COMMON/DERIMO/GRADI(MXV)
      CHARACTER*5 OLDMO, OLDGR(MXV), HELPMO
      CHARACTER*2 CERR, CERR2
      DIMENSION IERR2(MXV)
C
      ONE = 1.
       IF(ICH.EQ.1)THEN
         CALL DERMER(NOD,POINT,VAL,DIRCOS,NOC,GLERB,IERR)
         OLDMO = DERIVA
         DO 10,I=1,NOD
 10        OLDGR(I) = GRADI(I)
         DERIVA = DMODE
         DO 20,I=1,NOD
 20        GRADI(I) = DMODE
         CALL DERMER(NOD,POINT,VAL,XT,NOC,GLERB,IERR2)
         DERIVA = OLDMO
         DO 30,I=1,NOD
 30        GRADI(I) = OLDGR(I)
         UMODE = DERIVA
         DO 40,I=1,NOD
           GRALOC(I) = DIRCOS(I)
           LERR(I) = IERR(I)
 40      CONTINUE
         DO 1 I=1,NOD
           XT1(I)=ABS((XT(I)-DIRCOS(I)))*100./
     >            MAX(ONE,MIN(ABS(XT(I)),ABS(DIRCOS(I))))
 1       CONTINUE
         IF (.NOT.STRO) RETURN
         CALL DASH
         WRITE(IUOUT,5) DMODE, OLDMO
         CALL DASH
         DO 2 I=1,NOD
           HELPMO = ' '
           CERR = ' '
           IF (GRADI(I).NE.'ANAL') WRITE(CERR,7)IERR(I)

           CERR2 = ' '
           IF (DMODE.NE.'ANAL') WRITE(CERR2,7)IERR2(I)
           IF (DERIVA.EQ.'MIXED') HELPMO = GRADI(I)
           WRITE (IUOUT,4) I, NICKNA(I), XT(I), CERR2, DIRCOS(I),
     >                     CERR,XT1(I), HELPMO
 2       CONTINUE
      ELSE
         CALL DERMER(NOD,POINT,VAL,DIRCOS,NOC,GLERB,IERR)
         IF(.NOT.STRO)RETURN
         DO 3 I=1,NOD
           CERR = ' '
           IF (GRADI(I).NE.'ANAL') WRITE (CERR,7) IERR(I)
           WRITE (IUOUT,6) I,NICKNA(I),DIRCOS(I),CERR,GRADI(I)
 3       CONTINUE
      ENDIF
 4    FORMAT(T2,I3,')',T8,A10,T20,1PE12.5,T35,A,T40,
     >       1PE12.5,T55,A,T60,1PE10.3,T73,A)
 5    FORMAT(T24,A,T44,A,T62,'ERROR %')
 6    FORMAT(T2,I3,')',T8,A10,T20,1PE15.8,T40,A,T50,A)
 7    FORMAT(I2)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE COGRMV(NOMAXC,NOCEX,SML,IERR,IDIG,ERB,IPRI,EPSC,METHOD)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   COGRMV ... MINIMIZES THE FUNCTION ACSQ USING THE ALGORITHM
C   OF THE CONJUGATE GRADIENT DUE TO POLAK AND RIBIERE.
C          NOMAXC: MAXIMUM NUMBER OF CALLS
C          NOCEX : NUMBER OF CALLS MADE TO ACSQ
C          SML   : TERMINATION CRITERION FOR THE LINE SEARCH
C          IERR  : ARRAY CONTAINING THE ERROR CODES THAT INDICATE
C                  THE ACCURACY OF THE NUMERICAL CALCULATION
C                  OF THE GRADIENT
C          IDIG  : INPUT FOR THE SECOND ARGUMENT OF THE FUNCTION WALL
C          ERB   : ERROR BOUND FOR THE NUMERICAL CALCULATION OF THE GRAD
C          IPRI  : (0,1,2) ARRANGES THE PRINTOUT LEVEL
C          EPSC  : MINIMUM RELATIVE DROP ACCEPTED.
C          METHOD: 1=POLAK-RIBIERE, 2=FLETCHER-REEVES
C
      PARAMETER ( MXV = 200 )
      COMMON/UPDATE/IGRADU,IHESSU
      COMMON/CWORK1/XWORK(MXV),XT1(MXV),XT2(MXV),XT3(MXV),XT4(MXV)
      COMMON/CWORK2/XSTEP(MXV),FMIN(0:MXV)
      COMMON/DIRCOM/DIRCOS(MXV)
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      DIMENSION IERR(NOD)
C
      NOCEX=0
C  INITIALIZE TERMINATION CRITERIA SETTINGS.
      CALL INITER(IPRI,NOMAXC,EPSC)
C  CHECK IF ANY OF THE TERMINATION CRITERIA ARE MET.
      IF (ITERM(1,1,0).NE.0) GOTO 50
      DO 1 I=1,NOD
        STEP(I)=STEP(I)*IXAT(I)
    1  CONTINUE
      ALPHA=1.
       IF(IGRADU.EQ.0) THEN
        CALL DERMER(NOD,POINT,VAL,DIRCOS,NOCS1,ERB,IERR)
         DO 2 I=1,NOD
          GR=DIRCOS(I)
          DIRCOS(I)=DIRCOS(I)*WALL(I,IDIG,GR)
    2    CONTINUE
        NOCEX=NOCEX+NOCS1
       ENDIF
C   CALCULATE STEP
       DO 3 I=1,NOD
        GR=DIRCOS(I)
        XSTEP(I)=-DIRCOS(I)*WALL(I,IDIG,GR)
    3  CONTINUE
C    THE FOLLOWING LOOP PERFORMS CONJUGATE GRADIENT ITERATIONS
C
       DO 9 ITERAT=1,100000
C  STORE DERIVATIVES IN ARRAY XT1 , AND STEPS IN ARRAY DIRCOS
         DO 4 I=1,NOD
          XT1(I)=DIRCOS(I)
    4    DIRCOS(I)=XSTEP(I)
        OLDVAL=VAL
C  PERFORM A LINE SEARCH ALONG THE STEP DIRECTION
        CALL MINDIR(100,SML,1,NOCS2,ALPHA)
        NOCEX=NOCEX+NOCS2
        IF (VAL.LT.OLDVAL) CALL DISPLA
C  CHECK IF ANY OF THE TERMINATION CRITERIA ARE MET.
        IF (ITERM(1,1,1).NE.0) GOTO 10
C  CALCULATE DERIVATIVES AT THE POINT THE LINE SEARCH STOPPED
        CALL DERMER(NOD,POINT,VAL,XT2,NOCS3,ERB,IERR)
         DO 5 I=1,NOD
          GR=XT2(I)
          XT2(I)=XT2(I)*WALL(I,IDIG,GR)
          DIRCOS(I)=XT2(I)
    5    CONTINUE
        NOCEX=NOCEX+NOCS3
      IF (METHOD.EQ.1) THEN
C  CALCULATE THE DIFFERENCE IN THE GRADIENT AS XT3
C  UPDATE DERIVATIVES IN DIRCOS
         DO 6 I=1,NOD
6          XT3(I)=XT2(I)-XT1(I)
C   CALCULATE THE NORM  ANORM OF THE OLD GRADIENT
C   AS WELL AS THE SCALAR PRODUCT (GNEW-GOLD).GNEW
C   THIS IS THE   POLAK - RIBIERE  PRESCRIPTION
        BETA=0.
        ANORM=0.
         DO 7 I=1,NOD
          BETA=BETA+XT3(I)*DIRCOS(I)
          ANORM=ANORM+XT1(I)*XT1(I)
    7    CONTINUE
         IF(ANORM.EQ.0.) THEN
            CALL SETCOD(2)
            GO TO  10
         ENDIF
         BETA = BETA/ANORM
      ELSE
C  PROCEDE WITH FLETCHER-REEVES
         GK = 0.
         GK1 = 0.
         DO 30,I=1,NOD
            GK = GK+XT1(I)*XT1(I)
            GK1 = GK1+DIRCOS(I)*DIRCOS(I)
30       CONTINUE
         IF (GK.EQ.0.) THEN
            CALL SETCOD(2)
            GO TO  10
         ENDIF
         BETA = GK1/GK
      END IF
        ZERO = 0.
        BETA = MAX(ZERO,BETA)
        DO 8 I=1,NOD
          XSTEP(I)=-DIRCOS(I)+BETA*XSTEP(I)
          XSTP=-XSTEP(I)
          XSTEP(I)=XSTEP(I)*WALL(I,IDIG,XSTP)
    8   CONTINUE
    9  CONTINUE
C
   10 CONTINUE
      IF (ALPHA.NE.0.) THEN
         DO 20 I=1,NOD
   20      STEP(I)=ALPHA*XSTEP(I)
      END IF
   50 CALL TRINFO
      END
      SUBROUTINE CONMAC
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     CONSTRUCTS A " MACRO " (COMMAND PACKAGE)
C    INITIATES MACRO CONSTRUCTION , NO MACRO MODIFICATION.
C    IT CONTAINS SEVERAL SAFEGUARDS AND RELEVANT MESSAGES TO HELP
C    THE USER TO CONSTRUCT A PROPER MACRO, SO THAT NO MODIFICATIONS
C    WILL BE NECESSARY AND USE OF THE MEDIT WILL BE AVOIDED.
C
C
      CHARACTER SOME,STAR,COM*79,MACNM*10
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      REWIND IUMAC
      STAR='*'
      INDEX=0
    1 CONTINUE
      READ(IUMAC,12,END=2)SOME
      INDEX=INDEX+1
      GO TO  1
    2 REWIND IUMAC
       DO 3 I=1,INDEX
    3  READ(IUMAC,12)SOME
    4 IF(STRO)WRITE(IUOUT,*)'** ENTER MACRO - NAME **'
      READ(IUINP,10,END=5)MACNM
      GO TO  6
    5 REWIND IUINP
C   IF IN BATCH OPTION OF THE PROCES MODE ABORT ON ERRORS
      CALL BATCH
      IF(STRO)WRITE(IUOUT,*)'** CR - IGNORED **'
      GO TO  4
    6 CONTINUE
      SOME=MACNM(1:1)
       IF(SOME.NE.'.')THEN
        IF(STRO)WRITE(IUOUT,*)'** ERROR .  INITIAL DOT MISSING **'
C   IF IN BATCH OPTION OF THE PROCES MODE ABORT ON ERRORS
        CALL BATCH
        GO TO  4
       ENDIF
      WRITE(IUMAC,12)STAR
      WRITE(IUMAC,10)MACNM
      IF(STRO)WRITE(IUOUT,*)'** MACRO EDITING STARTS **'
    7 IF(STRO)WRITE(IUOUT,*)'** ENTER DESIRED COMMAND ** '
      READ(IUINP,11,END=8)COM
       IF(COM.EQ.'GODFATHER')THEN
        IF(STRO)WRITE(IUOUT,13)
C   IF IN BATCH OPTION OF THE PROCES MODE ABORT ON ERRORS
        CALL BATCH
        GO TO  7
       ENDIF
      GO TO  9
    8 REWIND IUINP
C   IF IN BATCH OPTION OF THE PROCES MODE ABORT ON ERRORS
      CALL BATCH
      IF(STRO)WRITE(IUOUT,*)'** CR - IGNORED **'
      GO TO  7
    9 CONTINUE
      WRITE(IUMAC,11)COM
      CALL UPCASE(COM)
       IF(COM.EQ.'CLEAR')THEN
        IF(STRO)WRITE(IUOUT,*)' ** MACRO EDITING COMPLETE **'
        RETURN
       ELSE
        GO TO  7
       ENDIF
   10 FORMAT(A10)
   11 FORMAT(A79)
   12 FORMAT(A1)
   13 FORMAT(5X,'GODFATHER CANNOT BE INSIDE A MACRO')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE CONPRO ( ICONU, IFILEU, IERRU, IERROR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( MXV = 200 )
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/PARAMX/NOF
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      COMMON/MAGEX/VALOW,NTOT,MADA
C
      LOGICAL HEADER, PRECON
      CHARACTER*15 IN, OUT
      CHARACTER*80 PROLO, EPILO
      COMMON /CP1/ BI, BL, SM, NOVA, NOTE, MODPRO, MUIN, MUOUT,
     >             HEADER,  PRECON 
      COMMON /CP2/ PROLO, EPILO, IN, OUT
C
      CHARACTER*80 LINE, ORIG
      INTEGER IS(40), IE(40)
      LOGICAL EOF, EXI, OPE
C
      IERROR = 0
      INQUIRE(FILE='CONFIG',EXIST=EXI,OPENED=OPE,IOSTAT=IERR)
      IF (IERR.NE.0) RETURN
      IF (.NOT.EXI) RETURN
      IF (OPE) RETURN
C
      OPEN(ICONU,FILE='CONFIG')
      OPEN(IERRU)
      REWIND ICONU
      REWIND IERRU
C
 10   READ (ICONU,'(A)',END=30) ORIG
      IF (ORIG(1:1).EQ.'*') GOTO 10
      LINE = ORIG
      CALL UPCASE(LINE)
      CALL LCR(LINE,IS,IE,NIT)
      IF (NIT.EQ.0) GOTO 10
      CALL CMDPRO(LINE,ORIG,IS,IE,NIT,EOF,ICONU,IFILEU,IERRU,IERROR)
      IF (.NOT.EOF) GOTO 10
C
 30   CLOSE(ICONU)
      IF (IERROR.EQ.2) RETURN
C
      IF (NOVA.NE.-1) NOD = NOVA
      IF (NOTE.NE.-1) NOF = NOTE
      IF (MODPRO.EQ.1) PROCES = 'BATCH'
      IF (MODPRO.EQ.2) PROCES = 'IAF'
      IF (MUIN.NE.-1) IU31 = MUIN
      IF (MUOUT.NE.-1) IU32 = MUOUT
      IF (BI.GE.0.) THEN
         BIGNUM = BI
         VALOW = -BI
      END IF
      IF (BL.GE.0.) BLIM = BL
      IF (SM.GE.0.) DWARF = SM
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE CMDPRO ( LINE, ORIG, IS, IE, NIT, EOF,
     >                    ICONU, IFILEU, IERRU, IERROR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*(*) LINE, ORIG
      INTEGER IS(*), IE(*)
      LOGICAL EOF
C
      LOGICAL HEADER, PRECON
      CHARACTER*15 IN, OUT
      CHARACTER*80 PROLO, EPILO
      COMMON /CP1/ BI, BL, SM, NOVA, NOTE, MODPRO, MUIN, MUOUT,
     >             HEADER,  PRECON 
      COMMON /CP2/ PROLO, EPILO, IN, OUT
      COMMON /UALLOC/ MINU, MAXU, LASTU
      CHARACTER*10 CSET, DATAMO, DISMOD
      COMMON /MODE2/ CSET, DATAMO, DISMOD
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
C
      CHARACTER ITEM*15, PAR*15, FILE*15, FL*80
C
      PARAMETER ( NDIR = 20 )
      CHARACTER*15 DIR(NDIR)
      INTEGER NITAL(NDIR)
C
      DATA (DIR(I),NITAL(I),I=1,10) /
     >     'VAR            ', 2,
     >     'HEADER         ', 1,
     >     'NOHEADER       ', 1,
     >     'MODE           ', 2,
     >     'FILE           ', 2,
     >     'PROLOG         ', 2,
     >     'EPILOG         ', 2,
     >     'INPUT_FILE     ', 2,
     >     'INPUT_UNIT     ', 2,
     >     'OUTPUT_FILE    ', 2 /
      DATA (DIR(I),NITAL(I),I=11,NDIR) /
     >     'OUTPUT_UNIT    ', 2,
     >     'PRECONNECT     ', 1,
     >     'BIGGER         ', 2,
     >     'LIMIT          ', 2,
     >     'UNIT_RANGE     ', 3,
     >     'CSET           ', 2,
     >     'TERMS          ', 2,
     >     'NOPRECONNECT   ', 1,
     >     'SMALLER        ', 2,
     >     'PRINTOUT       ', 2 /
C
      EOF = .FALSE.
      ITEM = LINE(IS(1):IE(1))
C
      DO 500,I=1,NDIR
        IF (ITEM.EQ.DIR(I)) THEN
           IF (NIT.NE.NITAL(I)) THEN
              WRITE(IERRU,*)'ERROR: NUMBER OF ARGUMENTS TO ',ITEM,
     >               ' IS INCORRECT.'
              GOTO 1000
           ELSE
              GOTO (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,
     >              20),I
           END IF
        END IF
 500  CONTINUE
C
      WRITE(IERRU,*)'ERROR: INCORRECT COMMAND: ',ITEM
      GOTO 1000
C
C  ......................................................... VAR
C
 1    CONTINUE
      PAR = LINE(IS(2):IE(2))
      LE = IE(2)-IS(2)+1
      CALL TOINT(PAR,LE,NOD,IERR)
      IF (IERR.NE.0) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      IF (NOD.LE.0) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      NOVA = NOD
      RETURN
C
C  ......................................................... HEADER
C
 2    CONTINUE
      HEADER = .TRUE.
      RETURN
C
C  ......................................................... NOHEADER
C
 3    CONTINUE
      HEADER = .FALSE.
      RETURN
C
C  ......................................................... MODE
C
 4    CONTINUE
      PAR = LINE(IS(2):IE(2))
      IF (PAR.EQ.'IAF') THEN
         MODPRO = 2
      ELSE IF (PAR.EQ.'BATCH') THEN
         MODPRO = 1
      ELSE
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      RETURN
C
C  ......................................................... FILE
C
 5    CONTINUE
      IF (IE(2)-IS(2)+1.GT.15) THEN
      WRITE(IERRU,*)'ERROR: TOO LONG A NAME IN FILE. MAX IS 15 CHARS.'
         GOTO 1000
      END IF
      FILE = ORIG(IS(2):IE(2))
      OPEN(IFILEU,FILE=FILE,IOSTAT=IERR)
      IF (IERR.NE.0) THEN
      WRITE(IERRU,*)'ERROR: COULD NOT OPEN FILE: ',FILE
         GOTO 1000
      END IF
      REWIND IFILEU
 200  READ (ICONU,'(A)',END=100) FL
      IF (FL(1:6).EQ.'...END') GOTO 300
      LE = LENGTH(FL)
      IF (LE.EQ.0) THEN
         WRITE (ICONU,'('' '')')
      ELSE
         WRITE (IFILEU,'(A)') FL(1:LE)
      END IF
      GOTO 200
C
 100  EOF = .TRUE.
 300  REWIND IFILEU
      CLOSE(IFILEU)
      RETURN
C
C  ......................................................... PROLOG
C
 6    CONTINUE
      IF (IE(2)-IS(2)+1.GT.15) THEN
      WRITE(IERRU,*)'ERROR: TOO LONG A NAME IN PROLOG. MAX IS 15 CHARS.'
         GOTO 1000
      END IF
      PROLO = ORIG(IS(2):IE(2))
      RETURN
C
C  ......................................................... EPILOG
C
 7    CONTINUE
      IF (IE(2)-IS(2)+1.GT.15) THEN
      WRITE(IERRU,*)'ERROR: TOO LONG A NAME IN EPILOG. MAX IS 15 CHARS.'
         GOTO 1000
      END IF
      EPILO = ORIG(IS(2):IE(2))
      RETURN
C
C  ......................................................... INPUT_FILE
C
 8    CONTINUE
      IF (IE(2)-IS(2)+1.GT.15) THEN
      WRITE(IERRU,*)'ERROR: TOO LONG A NAME IN INPUT. MAX IS 15 CHARS.'
         GOTO 1000
      END IF
      IN = ORIG(IS(2):IE(2))
      RETURN
C
C  ......................................................... INPUT_UNIT
C
 9    CONTINUE
      PAR = LINE(IS(2):IE(2))
      LE = IE(2)-IS(2)+1
      CALL TOINT(PAR,LE,NU,IERR)
      IF (IERR.NE.0) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      IF (NU.LT.0) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      MUIN = NU
      RETURN
C
C  ......................................................... OUTPUT_FILE
C
 10   CONTINUE
      IF (IE(2)-IS(2)+1.GT.15) THEN
      WRITE(IERRU,*)'ERROR: TOO LONG A NAME IN OUTPUT. MAX IS 15 CHARS.'
         GOTO 2000
      END IF
      OUT = ORIG(IS(2):IE(2))
      RETURN
C
C  ......................................................... OUTPUT_UNIT
C
 11   CONTINUE
      PAR = LINE(IS(2):IE(2))
      LE = IE(2)-IS(2)+1
      CALL TOINT(PAR,LE,NU,IERR)
      IF (IERR.NE.0) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 2000
      END IF
      IF (NU.LT.0) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 2000
      END IF
      MUOUT = NU
      RETURN
C
C  ......................................................... PRECONNECT
C
 12   CONTINUE
      IF (IN.NE.' ' .OR. OUT.NE.' ') THEN
      WRITE(IERRU,*)'ERROR: NAMED I/O FILES MAY NOT BE PRECONNECTED'
         IF (OUT.NE.' ') GOTO 2000
         GOTO 2000
      END IF
      PRECON = .TRUE.
      RETURN
C
C  ......................................................... BIGGER
C
 13   CONTINUE
      PAR = LINE(IS(2):IE(2))
      LE = IE(2)-IS(2)+1
      CALL TOREAL(PAR,LE,X,IERR)
      IF (IERR.NE.0) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      IF (X.LE.0.) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      BI = X
      RETURN
C
C  ......................................................... LIMIT
C
 14   CONTINUE
      PAR = LINE(IS(2):IE(2))
      LE = IE(2)-IS(2)+1
      CALL TOREAL(PAR,LE,X,IERR)
      IF (IERR.NE.0) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      IF (X.LE.0.) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      BL = X
      RETURN
C
C  ......................................................... UNIT_RANGE
C
 15   CONTINUE
      PAR = LINE(IS(2):IE(2))
      LE = IE(2)-IS(2)+1
      CALL TOINT(PAR,LE,NU1,IERR)
      IF (IERR.NE.0) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      IF (NU1.LT.1 .OR. NU1.GT.999) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
C
      PAR = LINE(IS(3):IE(3))
      LE = IE(3)-IS(3)+1
      CALL TOINT(PAR,LE,NU2,IERR)
      IF (IERR.NE.0) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      IF (NU2.LT.1 .OR. NU2.GT.999 .OR. NU2.LT.NU1+9) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
            WRITE (*,*) 'SECOND TEST FOR NU2 FAILED'
         GOTO 1000
      END IF
      MINU = NU1
      MAXU = NU2
      LASTU = MINU-1
      RETURN
C
C  ......................................................... CSET
C
 16   CONTINUE
      PAR = LINE(IS(2):IE(2))
      LE = IE(2)-IS(2)+1
      IF (PAR.EQ.'NORMAL' .OR. PAR.EQ.'ASCII' .OR. PAR.EQ.'EBCDIC') THEN
         CSET = PAR
      ELSE
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 2000
      END IF
      RETURN
C
C  ......................................................... TERMS
C
17    CONTINUE
      PAR = LINE(IS(2):IE(2))
      LE = IE(2)-IS(2)+1
      CALL TOINT(PAR,LE,NOF,IERR)
      IF (IERR.NE.0) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      IF (NOF.LT.0) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      NOTE = NOF
      RETURN
C
C  ......................................................... NOPRECONNECT
C
 18   CONTINUE
      PRECON = .FALSE.
      RETURN
C
C  ......................................................... SMALLER
C
 19   CONTINUE
      PAR = LINE(IS(2):IE(2))
      LE = IE(2)-IS(2)+1
      CALL TOREAL(PAR,LE,X,IERR)
      IF (IERR.NE.0) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      IF (X.LE.0. .OR. X.GT.1.) THEN
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      SM = X
      RETURN
C
C  ......................................................... PRINTOUT
C
20    CONTINUE
      PAR = LINE(IS(2):IE(2))
      IF (PAR.EQ.'FULL') THEN
         WEAK=.TRUE.
         STRO=.TRUE.
      ELSE IF (PAR.EQ.'HALF') THEN
         STRO=.TRUE.
         WEAK=.FALSE.
      ELSE IF (PAR.EQ.'NO') THEN
         WEAK=.FALSE.
         STRO=.FALSE.
      ELSE
      WRITE(IERRU,*)'ERROR: PARAMETER ',PAR,' TO ',ITEM,' IS INCORRECT.'
         GOTO 1000
      END IF
      RETURN
C
C  ---------------------------------------------------------------------
C
C  JUMP HERE IF ANY ERRORS OCCUR. JUST SET THE ERROR
C  FLAG  AND RETURN.
 1000 CONTINUE
      IF (IERROR.EQ.0) IERROR = 1
      RETURN
 2000 CONTINUE
      IF (IERROR.EQ.0) IERROR = 2
      RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE UPCASE ( CHA )
      CHARACTER*(*) CHA
C
      CHARACTER*10 CSET, DATAMO, DISMOD
      COMMON /MODE2/ CSET, DATAMO, DISMOD
C
      IF (CSET.EQ.'ASCII') THEN
         CALL ACASE(CHA)
      ELSE IF (CSET.EQ.'EBCDIC') THEN
         CALL ECASE(CHA)
      END IF
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE ACASE ( CHA )
      CHARACTER*(*) CHA
C
      CHARACTER C
      LOGICAL ISCAP
C
      DO 10,I=1,LEN(CHA)
         C = CHA(I:I)
         IF (.NOT.ISCAP(C)) THEN
            IC = ICHAR(C)
            IF (IC.GE.97 .AND. IC.LE.122) CHA(I:I) = CHAR(IC-32)
         END IF
10    CONTINUE
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE ECASE ( CHA )
      CHARACTER*(*) CHA
C
      CHARACTER C
      LOGICAL ISCAP
C
      DO 10,I=1,LEN(CHA)
         C = CHA(I:I)
         IF (.NOT.ISCAP(C)) THEN
            IC = ICHAR(CHA(I:I))
            IF (IC.GE.129 .AND. IC.LE.137 .OR. IC.GE.145 .AND. IC.LE.153
     >          .OR. IC.GE.162 .AND. IC.LE.169) CHA(I:I) = CHAR(IC+64)
         END IF
10    CONTINUE
C
      END
C
C  ---------------------------------------------------------------------
C
      LOGICAL FUNCTION ISCAP ( C )
      CHARACTER C
C
      CHARACTER*26 LETT
      DATA LETT / 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' /
C
      ISCAP = INDEX(LETT,C).NE.0
      END
      SUBROUTINE DEMAR
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( MXV = 200 )
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
C
      INTEGER INDE(40)
C
C  INITIATE DEMARGIN OPERATION FOR THE LEFT MARGINS
      IF (STRO) WRITE (IUOUT,107)
      CALL GETI(INDE,NIT,IERR)
      IF (IERR.EQ.0) THEN
         DO 10,I=1,NIT
            IND = INDE(I)
            IF (MARG(IND).EQ.-1) MARG(IND) = 0
            IF (MARG(IND).EQ.2) MARG(IND) = 1
10       CONTINUE
      END IF
C
C  INITIATE DEMARGIN OPERATION FOR THE RIGHT MARGINS
      IF (STRO) WRITE(IUOUT,108)
      CALL GETI(INDE,NIT,IERR)
      IF (IERR.EQ.0) THEN
         DO 20,I=1,NIT
            IND = INDE(I)
            IF (MARG(IND).EQ.2) MARG(IND) = -1
            IF (MARG(IND).EQ.1) MARG(IND) = 0
20       CONTINUE
      END IF
C
C  THE FLAGS MARG(I) ARE NOW SET.
C  RESET THE VALUES FOR THE CLEARED MARGINS TO DEFAULT.
C  THE LEFT MARGINS DEFAULT IS   -BLIM
C  THE RIGHT MARGINS DEFAULT IS   BLIM
C
      DO 30 I=1,NOD
         IF (MARG(I).EQ.0) THEN
            XLL(I) = -BLIM
            XRL(I) = BLIM
         ELSE IF (MARG(I).EQ.-1) THEN
            XRL(I) = BLIM
         ELSE IF (MARG(I).EQ.1) THEN
            XLL(I) = -BLIM
         END IF
30    CONTINUE
C
 107  FORMAT(20X,'ENTER LEFT  MARGINS SEQUENCE')
 108  FORMAT(20X,'ENTER RIGHT MARGINS SEQUENCE')
      END
      SUBROUTINE DERMER ( N, POINT, VAL, GRAD, NOC, DERB, IERR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      SAVE
      DIMENSION POINT(N), GRAD(N), IERR(N)
C
      PARAMETER ( MXV = 200 )
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/UPDATE/IGRADU,IHESSU
      CHARACTER*5 UMODE
      COMMON/UPMODE/UMODE
      COMMON/LOCAL/GRALOC(MXV),LERR(MXV)
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      CHARACTER*5 GRADI
      COMMON/DERIMO/GRADI(MXV)
      COMMON/ACS/II
      COMMON/MADEX/TELOW,TEHIGH,FQTHRE,QNTHRE,IPRIM
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
      LOGICAL AUTODE
      COMMON/AUTOMA/AUTODE
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER NICKNA*10
      COMMON/NAMES/NICKNA(MXV)
C
      CHARACTER*5 GRMODE
      LOGICAL LPRI, LNAM
C
      IF (IGRADU.EQ.1 .AND. UMODE.EQ.DERIVA) THEN
         DO 40,I=1,N
           GRAD(I) = GRALOC(I)
           IERR(I) = LERR(I)
 40      CONTINUE
         RETURN
      END IF
C
      ERB = ABS(DERB)
      IF (ERB.LT.COMTOL) ERB = COMTOL
C
      NODIG = -LOG10(COMACC)-4
C  TOTAL NUMBER OF CALLS FOR THE GRADIENT EVALUATION.
      NOC = 0
C  NUMBER OF ANALYTIC GRADIENT COMPONENTS.
      NANAL = 0
      LPRI = AUTODE .AND. IPRIM.EQ.1 .AND. STRO
      LNAM = CALLBY.EQ.'NAME'
C      
      DO 10,I=1,N
 10      IERR(I) = 0
C      
      DO 20,II=1,N
        X = POINT(II)
        GRMODE = GRADI(II)
        IF (IXAT(II).EQ.0) THEN
           GRAD(II) = 0.
        ELSE IF (GRMODE.EQ.'FAST') THEN
           CALL FDERI(X,VAL,NODIG,DER,ERB,TE,IE)
           GRAD(II) = DER
           IERR(II) = IE
           IF (AUTODE .AND. TE.GT.TEHIGH) THEN
           GRADI(II) = 'QUAD'
           IF (LPRI) THEN
              IF (LNAM) THEN
                 WRITE(IUOUT,200) NICKNA(II),
     >                   ' SET TO QUAD, DUE TO TAYLOR ERROR'
              ELSE
                 WRITE(IUOUT,100) II,
     >                   ' SET TO QUAD, DUE TO TAYLOR ERROR'
              END IF
           END IF
           END IF
           IF (AUTODE .AND. ABS(DER).LT.FQTHRE) THEN
           GRADI(II) = 'QUAD'
           IF (LPRI) THEN
              IF (LNAM) THEN
                 WRITE(IUOUT,200) NICKNA(II),
     >                   ' SET TO QUAD, DUE TO SMALL DERIVATIVE'
              ELSE
                 WRITE(IUOUT,100) II,
     >                   ' SET TO QUAD, DUE TO SMALL DERIVATIVE'
              END IF
              END IF
           END IF       
           NOC = NOC+2
        ELSE IF (GRMODE.EQ.'QUAD') THEN
           CALL CDERI(X,VAL,NODIG,DER,ERB,TE,IE)
           GRAD(II) = DER
           IERR(II) = IE
           IF (AUTODE .AND. TE.GT.TEHIGH) THEN
           GRADI(II) = 'NUMER'
           IF (LPRI) THEN
              IF (LNAM) THEN
                 WRITE(IUOUT,200) NICKNA(II),
     >                   ' SET TO NUMER, DUE TO TAYLOR ERROR'
              ELSE
                 WRITE(IUOUT,100) II,
     >                   ' SET TO NUMER, DUE TO TAYLOR ERROR'
              END IF
           END IF
           END IF
           IF (AUTODE .AND. ABS(DER).LT.QNTHRE) THEN
           GRADI(II) = 'NUMER'
           IF (LPRI) THEN
              IF (LNAM) THEN
                 WRITE(IUOUT,200) NICKNA(II),
     >                   ' SET TO NUMER, DUE TO SMALL DERIVATIVE'
              ELSE
                 WRITE(IUOUT,100) II,
     >                   ' SET TO NUMER, DUE TO SMALL DERIVATIVE'
              END IF
              END IF
           END IF       
           IF (AUTODE .AND. TE.LT.TELOW .AND. ABS(DER).GT.FQTHRE) THEN
           GRADI(II) = 'FAST'
           IF (LPRI) THEN
              IF (LNAM) THEN
                 WRITE(IUOUT,200) NICKNA(II),' RESET TO FAST'
              ELSE
                 WRITE(IUOUT,100) II,' RESET TO FAST'
              END IF
              END IF
           END IF
           NOC = NOC+4
        ELSE IF (GRMODE.EQ.'NUMER') THEN
           CALL ACCUDE(X,VAL,DER,NOC1,ERB,TE,IE)
           GRAD(II) = DER
           IERR(II) = IE
           IF (AUTODE .AND. TE.LT.TELOW .AND. ABS(DER).GT.QNTHRE) THEN
            GRADI(II) = 'QUAD'
           IF (LPRI) THEN
              IF (LNAM) THEN
                 WRITE(IUOUT,200) NICKNA(II),' RESET TO QUAD'
              ELSE
                 WRITE(IUOUT,100) II,' RESET TO QUAD'
              END IF
              END IF
           END IF
           NOC = NOC+NOC1
        ELSE
           NANAL = NANAL+1
        END IF
C
 20   CONTINUE
C
      IF (NANAL.NE.0) THEN
         ITDER = ITDER+1
         IPDER = IPDER+1
         CALL GRANAL(N,POINT,VAL,GRALOC)
      END IF
C
      DO 30,I=1,N
        IF (GRADI(I).EQ.'ANAL') GRAD(I) = GRALOC(I)
        GRALOC(I) = GRAD(I)
        LERR(I) = IERR(I)
 30   CONTINUE
      IF (AUTODE) DERIVA = 'MIXED'
      IGRADU = 1
      UMODE = DERIVA
C
 100  FORMAT(T2,'VARIABLE: ',I3,1X,A)
 200  FORMAT(T2,'VARIABLE: ',A10,2X,A)
      END
      SUBROUTINE DETUNT ( IUDEF, IU )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  INPUT : IUOUT , INFILE
C  OUTPUT: IU .  A UNIT NUMBER
C  THIS ROUTINE DETERMINES THE UNIT - NUMBER ( IU ) ASSOCIATED WITH
C  A FILE FOR I/O OPERATIONS.
C
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      CHARACTER*30 NAME
      LOGICAL OPE, NEW
C
      PARAMETER ( MAXMFI = 20 )
      CHARACTER*30 MCLFIL(MAXMFI)
      INTEGER MCLUNI(MAXMFI)
      LOGICAL MCLNEW(MAXMFI)
      COMMON / MCLIO1 / MCLFIL
      COMMON / MCLIO2 / MCLUNI, MCLNEW, NUMFI
C
C  GET THE FILENAME.
      CALL INNOCR(NAME,.FALSE.,IERR)
      IF (IERR.NE.0) CALL STOPER('ERROR IN DETUNT')
      IF (NAME.EQ.'.STANDARD') THEN
         IU = IUDEF
         RETURN
      END IF
C
C  HAS IT BEEN USED ALREADY ?
      DO 10,I=1,NUMFI
         IF (NAME.EQ.MCLFIL(I)) THEN
            IU = MCLUNI(I)
         RETURN
         END IF
10    CONTINUE
C
C  IS IT OPEN ?
      INQUIRE (FILE=NAME,OPENED=OPE,NUMBER=IU,IOSTAT=IERR)
      IF (IERR.NE.0) CALL STOPER('INQUIRE ERROR 1')
      IF (OPE) THEN
         NEW = .FALSE.
      ELSE
C  NOT OPEN. ASSIGN NEW UNIT AND OPEN THE FILE.
         IU = NUNIT()
         IF (IU.EQ.-1) CALL STOPER('ALL UNITS OCCUPIED ')
         OPEN (UNIT=IU,FILE=NAME,IOSTAT=IERR)
         IF (IERR.NE.0) CALL STOPER('OPEN ERROR IN DETUNT')
         NEW = .TRUE.
      END IF
C
      NUMFI = NUMFI+1
      IF (NUMFI.GT.MAXMFI) CALL STOPER('TOO MANY MCL FILES')
      MCLFIL(NUMFI) = NAME
      MCLUNI(NUMFI) = IU
      MCLNEW(NUMFI) = NEW
C
      END
      SUBROUTINE DFPMV(H,NOMAXC,NOCEX,SML,IERR,IDIG,ERB,IPRI,EPSD)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   DFPMV ... MINIMIZES THE FUNCTION ACSQ  USING THE DFP ALGORITHM
C   DUE TO DAVIDON FLETCHER AND POWELL.
C          NOMAXC: MAXIMUM NUMBER OF CALLS
C          NOCEX : NUMBER OF CALLS MADE TO ACSQ
C          SML   : TERMINATION CRITERION FOR THE LINE SEARCH
C          IERR  : ARRAY CONTAINING THE ERROR CODES THAT INDICATE
C                  THE ACCURACY OF THE NUMERICAL CALCULATION
C                  OF THE GRADIENT
C          IDIG  : INPUT FOR THE SECOND ARGUMENT OF THE FUNCTION WALL
C          ERB   : ERROR BOUND FOR THE NUMERICAL CALCULATION OF THE GRAD
C          IPRI  : (0,1,2) ARRANGES THE PRINTOUT LEVEL
C          EPSD  : MINIMUM RELATIVE DROP ACCEPTED.
C
      PARAMETER ( MXV = 200 )
      COMMON/UPDATE/IGRADU,IHESSU
      COMMON/CWORK1/XWORK(MXV),XT1(MXV),XT2(MXV),XT3(MXV),XT4(MXV)
      COMMON/DIRCOM/DIRCOS(MXV)
      COMMON/CWORK2/XSTEP(MXV),FMIN(0:MXV)
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      DIMENSION H(NOD,NOD),IERR(NOD)
C
      NOCEX=0
C  INITIALIZE TERMINATION CRITERIA SETTINGS.
      CALL INITER(IPRI,NOMAXC,EPSD)
C  CHECK IF ANY OF THE TERMINATION CRITERIA ARE MET.
      IF (ITERM(1,1,0).NE.0) GOTO 50
      DO 1 I=1,NOD
        STEP(I)=STEP(I)*IXAT(I)
    1 CONTINUE
      ALPHA=1.
       IF(IGRADU.EQ.0) THEN
        CALL DERMER(NOD,POINT,VAL,DIRCOS,NOCS1,ERB,IERR)
         DO 2 I=1,NOD
          GR=DIRCOS(I)
          DIRCOS(I)=DIRCOS(I)*WALL(I,IDIG,GR)
    2    CONTINUE
        NOCEX=NOCEX+NOCS1
       ENDIF
       IF(IHESSU.EQ.0) THEN
         DO 3 I=1,NOD
           DO 3 J=I,NOD
            H(I,J)=0.
            H(J,I)=0.
    3    CONTINUE
         DO 4 I=1,NOD
          H(I,I)=1.
    4    CONTINUE
        IHESSU=1
       ENDIF
C    THE FOLLOWING LOOP PERFORMS DFP ITERATIONS
C
       DO 14 ITERAT=1,100000
C   CALCULATE STEP
         DO 6 I=1,NOD
          XSTEP(I)=0.
           DO 5 J=1,NOD
    5      XSTEP(I)=XSTEP(I)-H(I,J)*DIRCOS(J)
          XSTP=-XSTEP(I)
          XSTEP(I)=XSTEP(I)*WALL(I,IDIG,XSTP)
    6    CONTINUE
C  STORE DERIVATIVES IN ARRAY XT1 , AND STEPS IN ARRAY DIRCOS
         DO 7 I=1,NOD
          XT1(I)=DIRCOS(I)
    7    DIRCOS(I)=XSTEP(I)
        OLDVAL=VAL
C  PERFORM A LINE SEARCH ALONG THE STEP DIRECTION
        CALL MINDIR(100,SML,1,NOCS2,ALPHA)
        NOCEX=NOCEX+NOCS2
        IF (VAL.LT.OLDVAL) CALL DISPLA
C  CHECK IF ANY OF THE TERMINATION CRITERIA ARE MET.
        IF (ITERM(1,1,1).NE.0) GOTO 15
C  CALCULATE DERIVATIVES AT THE POINT THE LINE SEARCH STOPPED
        CALL DERMER(NOD,POINT,VAL,XT2,NOCS3,ERB,IERR)
         DO 8 I=1,NOD
          GR=XT2(I)
          XT2(I)=XT2(I)*WALL(I,IDIG,GR)
    8    CONTINUE
        NOCEX=NOCEX+NOCS3
C  CALCULATE THE DIFFERENCE IN THE GRADIENT AS XT3
C  UPDATE DERIVATIVES IN DIRCOS
         DO 9 I=1,NOD
          XT3(I)=XT2(I)-XT1(I)
    9    DIRCOS(I)=XT2(I)
C   CALCULATE AUXILIARY ARRAY XT4(K)= XT3(L)*H(L,K)
         DO 11 K=1,NOD
          XT4(K)=0.
           DO 10 L=1,NOD
   10      XT4(K)=XT4(K)+XT3(L)*H(L,K)
   11    CONTINUE
C   CALCULATE  UPDATING FACTORS.
        A2=0.
        DIVIS=0.
         DO 12 K=1,NOD
          A2=A2+XT3(K)*XT4(K)
   12    DIVIS=DIVIS+STEP(K)*XT3(K)
        IF(DIVIS.EQ.0. .OR. A2.EQ.0.) THEN
           CALL SETCOD(2)
           GO TO  15
        ENDIF
        AFAC=ALPHA/DIVIS
        BFAC=-1./A2
C   UPDATE INVERSE HESSIAN ACCORDING TO DFP PRESCRIPTION
         DO 13 I=1,NOD
           DO 13 J=I,NOD
            H(I,J)=H(I,J)+AFAC*XSTEP(I)*XSTEP(J)+BFAC*XT4(I)*XT4(J)
   13    H(J,I)=H(I,J)
        IHESSU=1
   14  CONTINUE
   15 CONTINUE
      IF (ALPHA.NE.0.) THEN
         DO 20 I=1,NOD
   20      STEP(I)=ALPHA*XSTEP(I)
      END IF
   50 CALL TRINFO
      END
      SUBROUTINE DISPLA
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   DISPLA ... IS CALLED BY THE MINIMIZATION ROUTINES WHENEVER
C   A LOWER VALUE IS FOUND. IT PRINTS THE RESULTS ACCORDING TO
C   THE ARRANGED PRINTOUT LEVEL DEFINED BY THE VARIABLE
C   IPSTAT OF THE COMMON BLOCK /TERMIN/.
C   IPSTAT =  0,1,2 FOR NO PRINTOUT,ONLY THE FUNCTION'S VALUE,
C                 BOTH FUNCTION'S VALUE AND THE PARAMETER VALUES.
C
      PARAMETER ( MXV = 200 )
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      COMMON/TERMIN/STAVAL,EPS,INIC,ISTCAL,IPSTAT,MAXOC,ITCODE,ILOW
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
C
      ILOW = 1
      IF (IPSTAT.EQ.0) RETURN
      IF (WEAK) THEN
         IF (IPSTAT.EQ.1) THEN
           NOC = ITNOCS-ISTCAL
           WRITE(IUOUT,2)VAL,NOC,MAXOC
         ELSE IF (IPSTAT.EQ.2)THEN
           CALL WRITER(IUOUT)
         END IF
         CALL DASH
      END IF
    2 FORMAT(7X,'LOWER VALUE : ',2X,1PE21.14,5X,I5,' OUT OF',I5)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE DISPL1 ( VAL, X, N )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   DISPL1 IS A SIMPLE VERSION OF DISPLA, USED WHEN PORTING MINIMIZATION
C   CODES BY OTHER AUTHORS.
C   IT PRINTS THE RESULTS ACCORDING TO
C   THE ARRANGED PRINTOUT LEVEL DEFINED BY THE VARIABLE
C   IPSTAT OF THE COMMON BLOCK /TERMIN/.
C   IPSTAT =  0,1,2 FOR NO PRINTOUT,ONLY THE FUNCTION'S VALUE,
C                 BOTH FUNCTION'S VALUE AND THE PARAMETER VALUES.
C
      DIMENSION X(N)
      PARAMETER ( MXV = 200 )
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      COMMON/TERMIN/STAVAL,EPS,INIC,ISTCAL,IPSTAT,MAXOC,ITCODE,ILOW
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
C
      ILOW = 1
      IF (IPSTAT.EQ.0) RETURN
      IF (WEAK) THEN
         IF (IPSTAT.EQ.1) THEN
           NOC = ITNOCS-ISTCAL
           WRITE (IUOUT,2) VAL, NOC, MAXOC
         ELSE IF (IPSTAT.EQ.2) THEN
           CALL WRITE1(VAL,X,N,IUOUT)
         END IF
         CALL DASH
      END IF
    2 FORMAT(7X,'LOWER VALUE : ',2X,1PE21.14,5X,I5,' OUT OF',I5)
      END
      SUBROUTINE DISWRI
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 CSET, DATAMO, DISMOD
      COMMON /MODE2/ CSET, DATAMO, DISMOD
C
      IF (DISMOD.EQ.'DISPOWRITE') THEN
         CALL POSIT(IUDIS)
         CALL WRITER(IUDIS)
      END IF
C
      END
      SUBROUTINE DUPNAM(I,NOV)
      PARAMETER ( MXV = 200 )
      CHARACTER*10 NICKNA
      COMMON/NAMES/NICKNA(MXV)
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
        DO 2,J=1,NOV
          IF (I.EQ.J .OR. NICKNA(J).EQ.' ') GOTO 2
          IF (NICKNA(I).EQ.NICKNA(J)) THEN
          IF (STRO) WRITE(IUOUT,3) NICKNA(J)(1:LENGTH(NICKNA(J))),J
             NICKNA(J) = ' '
          CALL BATCH
          END IF
 2      CONTINUE
 3    FORMAT(T2,'WARNING: DUPLICATE NAME: ',A,'. VARIABLE: ',
     >       I3,') IS RESET TO BLANK.')
      END
      SUBROUTINE DWRITE
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 CSET, DATAMO, DISMOD
      COMMON /MODE2/ CSET, DATAMO, DISMOD
C
      IF (DATAMO.EQ.'DATAWRITE') THEN
         REWIND IUDAT
         CALL WRITER(IUDAT)
      END IF
C
      END
      BLOCK DATA EQUIP
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    EQUIP ... SETS THE DEFAULTS AND ASSIGNS INITIAL VALUES.
C  VARIABLES THAT ARE NOT SUPPOSED TO CHANGE THROUGHOUT EXECUTION ARE
C  INITIALIZED HERE. SUBROUTINE INIT TAKES CARE OF THE REST.
C
      PARAMETER ( BIG1 = 10.**38 )
      PARAMETER ( BLIM1 = 10.**30 )
      PARAMETER ( SMALL1 = 10.**(-36) )
      PARAMETER ( PBIG1 = -BIG1 )
C
      PARAMETER ( MXV = 200 )
      PARAMETER ( MEM = 2500 , LSTA = 500 )
      COMMON/MCLVAR/STACK(LSTA),CMEM(MEM),ISTACK,INFILE
      COMMON/MAGEX/VALOW,NTOT,MADA
      COMMON/SIMPEX/ERR1,ERR,INIS,MNOC1,MNOC,IPRIS
      COMMON/ROLLEX/ACCH,ENHH,EPSH,NOCH,IFMH,IPRIH,IDIGH
      COMMON/BFGSEX/ACCB,ERBB,EPSB,NOCB,IPRIB,IDIGB,MADB
      COMMON/DFPEX/ACCD,ERBD,EPSD,NOCD,IPRID,IDIGD,MADD
      COMMON/CGREX/ACCC,ERBC,EPSC,NOCC,IPRIC,IDIGC,MADC,METHOC
      COMMON/RANDEX/REDFR,ACCR,EPSR,NOCR,IVOLR,ICYSR,IPRIR,IFAILR,LSEAR
      COMMON/TOLEX/ACCT,ERBT,NOCT,IPRIT,MADT
      COMMON/LEVEX/GTOLL,FTOLL,XTOLL,FACTL,EPSFL,NOCL,IPRIL
      COMMON/GRCOM/GXMIN,GXMAX,XAX,YAX,NOCGR,ILIN,ICOL,IWVAR
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      INTEGER DEBVAR
      COMMON/DEBUG/DEBVAR
      COMMON/MADEX/TELOW,TEHIGH,FQTHRE,QNTHRE,IPRIM
      COMMON/ACCEX/TARAC,NOCAC,NOPAC
      DATA NOCAC,NOPAC,TARAC / 300 , 1 , 0.0 /
      DATA DEBVAR / 0 /
      DATA NOCGR,ILIN,ICOL,IWVAR,GXMIN,GXMAX,XAX,YAX/
     >30 ,   17  , 64  , 1 ,  -1.E5 ,  1.E5, 0.  ,0.  /
      DATA  NTOT , VALOW , MADA / 10000 , PBIG1 , 0 /
      DATA  INIS,ERR1,MNOC1,ERR,MNOC,IPRIS /
     >1  , 0.10  , 100  , 0. ,   500  ,  1  /
      DATA NOCH, ACCH, ENHH, IFMH, IPRIH, IDIGH, EPSH /
     >      300, .001,   3.,    4,     1,     3,   0. /
      DATA NOCB , ACCB , ERBB , IPRIB , IDIGB , MADB , EPSB /
     >      300 , .001 , .01  ,   1   ,     3 ,    0 ,   0. /
      DATA NOCD , ACCD , ERBD , IPRID , IDIGD , MADD , EPSD /
     >      300 , .001 , .01  ,   1   ,     3 ,    0 ,   0. /
      DATA NOCC , ACCC , ERBC , IPRIC , IDIGC , MADC , EPSC, METHOC /
     >      300 , .001 , .01  ,   1   ,     3 ,    0 ,   0.,     1 /
      DATA NOCR, IVOLR, REDFR, ICYSR, IPRIR, IFAILR, LSEAR, ACCR, EPSR /
     >      300 ,    0,    .7,    30,     1,      5,     3, .001,   0. /
      DATA NOCT, ACCT,  ERBT, IPRIT, MADT /
     >      300,   0.,   .01,     1,    0 /
C  EPSFL WILL BE SET AT RUN TIME IN SUBROUTINE INIT; A MODERATE VALUE
C  IS GIVEN HERE.
      DATA GTOLL, FTOLL, XTOLL, FACTL, EPSFL, NOCL, IPRIL /
     >        0.,    0.,    0.,  100., 1.E-7,  300,     1 /
      DATA TELOW , TEHIGH , FQTHRE , QNTHRE , IPRIM /
     >    1.E-10 ,  1.E-3 ,  1.E-4 ,  1.E-7 ,     0 /
      DATA BIGNUM, BLIM, DWARF / BIG1, BLIM1, SMALL1  /
      DATA STACK /LSTA*-11111./
      DATA CMEM  / MEM*-22222./
      DATA INFILE,ISTACK / 0 , 0 /
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE EXECOM ( ICOM, VERSIM, MAXW, OLDCOM, COM )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      SAVE
C
      DIMENSION VERSIM(*)
      CHARACTER*10 OLDCOM, COM
C  --------------------------------------------------------------------
      PARAMETER ( NCOM = 100 )
      CHARACTER*10 COMMAN(NCOM)
      INTEGER CLASS(NCOM), IND(NCOM)
      COMMON /COM1/ COMMAN
      COMMON /COM2/ CLASS, IND
C
      PARAMETER ( MXV = 200 )
      COMMON/INDEFI/INDEF(MXV)
      PARAMETER ( MEM = 2500 , LSTA = 500 )
      COMMON/MCLVAR/STACK(LSTA),CMEM(MEM),ISTACK,INFILE
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
      COMMON/GRCOM/GXMIN,GXMAX,XAX,YAX,NOCGR,ILIN,ICOL,IWVAR
      COMMON/MAGEX/VALOW,NTOT,MADA
      COMMON/SIMPEX/ERR1,ERR,INIS,MNOC1,MNOC,IPRIS
      COMMON/ROLLEX/ACCH,ENHH,EPSH,NOCH,IFMH,IPRIH,IDIGH
      COMMON/BFGSEX/ACCB,ERBB,EPSB,NOCB,IPRIB,IDIGB,MADB
      COMMON/DFPEX/ACCD,ERBD,EPSD,NOCD,IPRID,IDIGD,MADD
      COMMON/CGREX/ACCC,ERBC,EPSC,NOCC,IPRIC,IDIGC,MADC,METHOC
      COMMON/RANDEX/REDFR,ACCR,EPSR,NOCR,IVOLR,ICYSR,IPRIR,IFAILR,LSEAR
      COMMON/TOLEX/ACCT,ERBT,NOCT,IPRIT,MADT
      COMMON/LEVEX/GTOLL,FTOLL,XTOLL,FACTL,EPSFL,NOCL,IPRIL
      COMMON/ACCEX/TARAC,NOCAC,NOPAC
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      COMMON/UPDATE/IGRADU,IHESSU
      COMMON/CWORK1/XWORK(MXV),XT1(MXV),XT2(MXV),XT3(MXV),XT4(MXV)
      COMMON/CWORK2/XSTEP(MXV),FMIN(0:MXV)
      COMMON/DIRCOM/DIRCOS(MXV)
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/PARAMX/NOF
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      COMMON/FIRST/ICATA,IDAT
      CHARACTER*10 NICKNA
      COMMON/NAMES/NICKNA(MXV)
      COMMON/ERRBOU/GLERB
      LOGICAL AUTODE
      COMMON/AUTOMA/AUTODE
      COMMON/INDE/INDIC(6)
      LOGICAL AIMON
      COMMON/AIM/TARGET,AIMON
      COMMON /MODE3/ MODEV, IFFORM 
C
      LOGICAL HEADER, PRECON
      CHARACTER*15 IN, OUT
      CHARACTER*80 PROLO, EPILO
      COMMON /CP1/ BI, BL, SM, NOVA, NOTE, MODPRO, MUIN, MUOUT,
     >             HEADER,  PRECON 
      COMMON /CP2/ PROLO, EPILO, IN, OUT
C
      COMMON /GETOUT/ IRET, IRET2, IQU
C
      PARAMETER ( MAXNL = 20 )
      INTEGER UBACK(MAXNL)
      CHARACTER*10 CBACK(MAXNL)
      COMMON / NEST1 / UBACK, LEVEL, IEXIT, IDISK
      COMMON / NEST2 / CBACK
C
      CHARACTER*5 GRADI
      COMMON/DERIMO/GRADI(MXV)
      CHARACTER*10 FILSTA
      COMMON/FSTAT/FILSTA(6)
C
      INTEGER IS(40), IE(40)
      CHARACTER*80 INPLIN
      COMMON /INDAT1/ IS, IE, NIT, LEC, ISTAT
      COMMON /INDAT2/ INPLIN
C
      CHARACTER*10 CSET, DATAMO, DISMOD
      COMMON /MODE2/ CSET, DATAMO, DISMOD
C
      PARAMETER ( MAXMFI = 20 )
      CHARACTER*30 MCLFIL(MAXMFI)
      INTEGER MCLUNI(MAXMFI)
      LOGICAL MCLNEW(MAXMFI)
      COMMON / MCLIO1 / MCLFIL
      COMMON / MCLIO2 / MCLUNI, MCLNEW, NUMFI
C  --------------------------------------------------------------------
      DIMENSION IERR(MXV)
      CHARACTER*10 UNITN,SCR
      CHARACTER*10 FNAME,MCLNAM*30,DMODE
      CHARACTER*10 FILENA(6)
      CHARACTER*80 LINE, CHS
C  --------------------------------------------------------------------
      LOGICAL LO,LOPEN
      LO(I1,I2)=I.GE.I1.AND.I.LE.I2
C  ---------------------------------------------------------------------
C               MERLIN - FILES
C  ---------------------------------------------------------------------
      DATA FILENA/'DATA','STORE','INIPO','DISPO','BACKUP','MACROF'/
C
C  ARE WE ABOUT TO EXECUTE A MINIMIZATION COMMAND ? (CLASS=1)
C  IF YES, SAVE THE OLD POINT IN FILE "DATA".
      IF (CLASS(ICOM).EQ.1) THEN
         CALL DWRITE
         IDAT=1
      END IF
C
      I = IND(ICOM)
      IF (LO(1,10))  GOTO (61,62,35,77,44,74,9,58,76,24),I
      IF (LO(11,20)) GOTO (75,26,14,64,31,36,13,49,46,50),I-10
      IF (LO(21,30)) GOTO (37,54,89,43,41,60,6,56,32,51),I-20
      IF (LO(31,40)) GOTO (27,15,57,16,59,22,45,42,33,38),I-30
      IF (LO(41,50)) GOTO (34,47,23,12,20,55,73,91,5,90),I-40
      IF (LO(51,60)) GOTO (88,72,63,48,71,18,29,4,65,66),I-50
      IF (LO(61,70)) GOTO (67,68,82,83,84,85,86,81,69,70),I-60
      IF (LO(71,80)) GOTO (78,79,80,87,39,40,190,191,192,193),I-70
      IF (LO(81,90)) GOTO (194,195,196,197,198,199,200,301,302,303),I-80
      IF (LO(91,100)) GOTO(304,305,306,307,308,309,310,311,312,313),I-90
      WRITE(*,*) 'INDEX NOT VALID',ICOM,I
      STOP'PROGRAMMING ERROR'
C  .........................................................  STOP
 4    CONTINUE
      IRET = 2
      IRET2 = 2
C  SUPRESS EPILOG IF ANYTHING ELSE IS ENTERED IN THE COMMAND LINE.
      IF (NIT.GT.1 .AND. EPILO.NE.' ') THEN
         CALL MID1('# EPILOG EXECUTION HAS BEEN SUPRESSED #')
         EPILO = ' '
      END IF
      GOTO 1
C  .........................................................  RETURN
 5    CONTINUE
      IRET = 1
      IRET2 = 1
      IQU = 0
C  SUPRESS EPILOG IF ANYTHING ELSE IS ENTERED IN THE COMMAND LINE.
      IF (NIT.GT.1 .AND. EPILO.NE.' ') THEN
         CALL MID1('# EPILOG EXECUTION HAS BEEN SUPRESSED #')
         EPILO = ' '
      END IF
      GOTO 1
C  .........................................................  INSPECT
 6    CONTINUE
      IF (STRO) WRITE(IUOUT,*)'ENTER FILE NAME TO INSPECT'
      READ(IUINP,99,END=92)UNITN
       DO 7 I=1,6
C
C   CHECK IF THE FILENAME IS ONE OF MERLIN VALID FILENAMES
C
         IF (UNITN.EQ.FILENA(I)) THEN
          IFILE=INDIC(I)
          JJ=I
          GOTO 8
         END IF
    7  CONTINUE
      IF (STRO)
     >WRITE(IUOUT,*)'NO SUCH FILE - FOR FILE NAMES ENTER: CATALOG'
C   IF IN BATCH OPTION OF THE PROCES MODE ABORT.
      CALL BATCH
      GOTO 1
    8 CONTINUE
       IF (IFILE.EQ.IUMAC) THEN
C   CHECK IF FILENAME IS MACROF (UNIT: IUMAC) THAT CANNOT BE INSPECTED
        IF (STRO) WRITE(IUOUT,101)
        GOTO 1
       END IF
       IF (FILSTA(JJ).EQ.'EMPTY') THEN
C
C  IF FILE WAS EMPTY, SET COM = ' ' TO DISALLOW A CONSECUTIVE PICK.
C
        COM=' '
        IF (STRO) WRITE(IUOUT,102)FILENA(JJ)
        GOTO 1
       END IF
      CALL INSPEC(IFILE,NOD)
      GOTO 1
C  .........................................................  CATALOG
 9    CONTINUE
C
C  CHECK  WHICH MERLIN FILES EXIST.
C  UPDATE FILE STATUS. EMPTY OR NOT. NON-EMPTY CORRESPONDS TO : '  * '
C
       DO 11 I=1,6
         IF (FILSTA(I).EQ.'EMPTY') THEN
          INDE=INDIC(I)
          REWIND INDE
          READ(INDE,99,END=10)SCR
          FILSTA(I)='  *    '
          GOTO 11
   10     REWIND INDE
          GOTO 11
         END IF
   11  CONTINUE
      IF (ICATA.EQ.0) THEN
         ICATA = 1
         GOTO 1
      END IF
C
C   DISPLAY FILENAMES AND THEIR STATUS.
C
      IF (STRO) WRITE(IUOUT,104)(FILENA(I),I=1,6)
      IF (STRO) WRITE(IUOUT,103)(FILSTA(I),I=1,6)
      GOTO 1
C  .........................................................  PICK
 12   CONTINUE
       IF (OLDCOM.NE.'INSPECT') THEN
        IF (STRO) WRITE(IUOUT,105)
        CALL BATCH
        GOTO 1
       END IF
       CALL DISWRI
C
       IF (IFILE.EQ.IUDAT.AND.IDAT.EQ.0) THEN
        JCON=0
       ELSE
        JCON=1
       END IF
C
C   PICK NEW POINT
C
      CALL READER(IU45,JCON)
      IDAT=1
C
C  CHECK POINT IN RESPECT TO MARGINS (IF ANY) AND READJUST IT
C  IF IT IS OUT OF MARGINS RANGE.
C
      CALL POIMAR(IFEV)
      IF (IFEV.EQ.0 .AND. MODEV.EQ.1) VAL = ACSQ(POINT,NOD)
      GOTO 1
C  .........................................................  GODFATHER
 13   CONTINUE
C
C  SET OR RESET NAMES FOR THE VARIALES
C
      CALL NAMEM(NOD)
C
C   UPDATE FILE : DATA
      CALL DWRITE
      GOTO 1
C  .........................................................  FIX
 14   CONTINUE
      IF (ITNOCS.EQ.0) GOTO 1
C  INITIATE VARIABLE FIX - OPERATION.
C  SET THE FIX-FLAG
      ICFIX = 0
      CALL FIXLOO(ICFIX)
C   UPDATE FILE : DATA
      CALL DWRITE
      GOTO 1
C  .........................................................  LOOSE
 15   CONTINUE
      IF (ITNOCS.EQ.0) GOTO 1
C  INITIATE VARIABLE LOOSE - OPERATION
C  SET THE LOOSE-FLAG
      ICFIX = 1
      CALL FIXLOO(ICFIX)
C  UPATE FILE : DATA
      CALL DWRITE
      GOTO 1
C  .........................................................  MARGIN
 16   CONTINUE
      CALL CHMAR
      GOTO 1
C  .........................................................  STEP
 18   CONTINUE
C
C  SET THE SEARCH STEPS FOR THE VARIABLES.(USED BY ROLL,RANDOM,
C  AND BY THE LINE-SEARCH
      CALL CHSTEP
      GOTO 1
C  .........................................................  POINT
 20   CONTINUE
C  APPEND OLD POINT IN FILE : DISPO
      CALL DISWRI
C  SET VALUES TO THE OBJECTIVE FUNCTION VARIABLES
C  SET THE POINT-FLAG
21    CALL CHPOI(NPOI)
      IF (NPOI.NE.0 .AND. IUINP.NE.INFILE) THEN
         IF (STRO) WRITE (IUOUT,'(T2,''POINT'')')
         GOTO 21
      END IF
C
C  CHECK IF THERE ARE ANY UNDEFINED VARIABLES
C
      IF ( ITNOCS .EQ. 0 ) THEN
        CALL UNDEF(INDEF,NOD,IUOUT,STRO,IGO)
        IF (IGO .EQ. 0 ) GOTO 1
      END IF
      CALL POIMAR(IFEV)
      IF (IFEV.EQ.0 .AND. MODEV.EQ.1) VAL = ACSQ(POINT,NOD)
      GOTO 1
C  .........................................................  MEMO
 22   CONTINUE
C  STORE CURRENT POINT IN FILE : STORE
C  UPDATE STATUS OF FILE STORE
      CALL POSIT(IUSTO)
      CALL WRITER(IUSTO)
      WRITE(IUOUT,*) ' POINT IS STORED IN FILE: STORE '
      GOTO 1
C  .........................................................  PANELON
 23   CONTINUE
C   ACTIVATE PANEL PROMPTING
      PANEL = 'PANELON'
      GOTO 1
C  .........................................................  DEMARGIN
 24   CONTINUE
      CALL DEMAR
      GOTO 1
C  .........................................................  DISCARD
 26   CONTINUE
      IF (STRO) WRITE(IUOUT,113)
      READ(IUINP,99,END=92)UNITN
C
C  IF THE FILE IS PERMITED TO BE DISCARDED, THEN DISCARD IT.
C  DISCARD = REWIND FIRST AND THEN WRITE ON IT THE CURRENT POINT.
C
       IF (UNITN.EQ.'STORE') THEN
        JJ=2
       ELSE IF (UNITN.EQ.'DISPO') THEN
        JJ=4
       ELSE IF (UNITN.EQ.'BACKUP') THEN
        JJ=5
       ELSE
        IF (STRO) WRITE(IUOUT,114)
        GOTO 1
       END IF
      IFILE=INDIC(JJ)
      CLOSE(IFILE,STATUS='DELETE')
      OPEN(IFILE,FILE=UNITN(1:LENGTH(UNITN)))
      FILSTA(JJ)='EMPTY'   
      REWIND IFILE
      CALL WRITER(IFILE)
      GOTO 1
C  .........................................................  LOOSALL
 27   CONTINUE
      DO 28 I=1,NOD
C  SET AN ARBITRARY VALUE FOR THE SEARCH STEPS.
        IF (IXAT(I).EQ.0 .AND. ITNOCS.NE.0)
     >                         STEP(I)=(1.+ABS(POINT(I)))*COMTOL
C  LOOSE ALL VARIABLES BY SETTING THE IXAT FLAGS TO 1.
        IXAT(I)=1
   28 CONTINUE
      GOTO 1
C  .........................................................  STEPALL
 29   CONTINUE
      IF (ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
      CALL DWRITE
C  CALCULATE THE GRADIENT . (DIRCOS(I) CONTAINS IT UPON RETURN)
      CALL DERMER(NOD,POINT,VAL,DIRCOS,NODC,GLERB,IERR)
      DO 30  I=1,NOD
        DIRCOS(I)=DIRCOS(I)*WALL(I,4,DIRCOS(I))
 30   CONTINUE
      MCAL1=30
      ER1=0.
C  MAKE A LINE SEARCH ALONG THE DIRECTION OF THE GRADIENT
C  SET UPDATE STEPS-FLAG
      IUS = 1
      CALL MINDIR(MCAL1,ER1,IUS,NOC1,ALPHA)
C  MINDIR UPDATED THE STEPS THROUGH THE COMMON - BLOCKS
      GOTO 1
C  .........................................................  FULLBACK
 31   CONTINUE
      BACKUP = 'FULLBACK'
      GOTO 1
C  .........................................................  LASTBACK
 32   CONTINUE
      BACKUP = 'LASTBACK'
      GOTO 1
C  .........................................................  NOBACK
 33   CONTINUE
      BACKUP = 'NOBACK'
      GOTO 1
C  .........................................................  NUMER
 34   CONTINUE
      DERIVA = 'NUMER'
      DO 202,I=1,NOD
 202    GRADI(I) = 'NUMER'
      GOTO 1
C  .........................................................  ANAL
 35   CONTINUE
      DERIVA = 'ANAL'
      DO 205,I=1,NOD
 205    GRADI(I) = 'ANAL'
      GOTO 1
C  .........................................................  FULLPRINT
 36   CONTINUE
      PRINTO = 'FULLPRINT'
      WEAK=.TRUE.
      STRO=.TRUE.
      GOTO 1
C  .........................................................  HALFPRINT
 37   CONTINUE
      PRINTO = 'HALFPRINT'
      STRO=.TRUE.
      WEAK=.FALSE.
      GOTO 1
C  .........................................................  NOPRINT
 38   CONTINUE
      WEAK=.FALSE.
      STRO=.FALSE.
      PRINTO = 'NOPRINT'
      GOTO 1
C  .........................................................  ALIASDIS
 39   CONTINUE
      CALL ALIDIS
      GOTO 1
C  .........................................................  UNALIAS
 40   CONTINUE
      CALL UNALI
      GOTO 1
C  .........................................................  INDEX
 41   CONTINUE
      CALLBY = 'INDEX'
      GOTO 1
C  .........................................................  NAME
 42   CONTINUE
      CALLBY = 'NAME'
      GOTO 1
C  .........................................................  IAF
 43   CONTINUE
      PROCES = 'IAF'
      GOTO 1
C  .........................................................  BATCH
 44   CONTINUE
      PROCES = 'BATCH'
      GOTO 1
C  .........................................................  MODEDIS
 45   CONTINUE
      CALL MODIS
      GOTO 1
C  .........................................................  GRADDIS
 46   CONTINUE
      IF (ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
C      
C   SET THE RIGHT INPUT FLAG FOR THE CALL TO SUBROUTINE COMPDR
      ICHGRA = 0
      CALL COMPDR(IERR,ICHGRA,DERIVA)
      GOTO 1
C  .........................................................  PANELOFF
 47   CONTINUE
C  DISABLE PANEL PROMPTING.
      PANEL = 'PANELOFF'
      GOTO 1
C  .........................................................  SHORTDIS
 48   CONTINUE
      IF (ITNOCS .EQ. 0 ) GOTO 1
      IF (STRO) THEN
         CALL DASH
         WRITE (IUOUT,133)
         CALL DASH
         WRITE (IUOUT,134) ITNOCS,ITDER,IPARTC,IPDER
         CALL DASH
         CALL WRITER(IUOUT)
      END IF
      GOTO 1
C  .........................................................  GRADCHECK
 49   CONTINUE
      IF (ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
C  CHECK USER SUPPLIED GRADIENT WITH THE ONE CALCULATED ALA-MERLIN
      IF(STRO)WRITE(IUOUT,*)'ENTER DERIVATIVE MODE'
      CALL INNOCR(DMODE,.TRUE.,IOCH)
      IF (IOCH.NE.0) THEN
         CALL BATCH
         GOTO 1
      END IF
      IF (DMODE.NE.'NUMER' .AND. DMODE.NE.'QUAD' .AND.
     >    DMODE.NE.'FAST'  .AND. DMODE.NE.'ANAL') THEN
         IF(STRO)WRITE(IUOUT,*)'ENTER ONE OF: NUMER, QUAD, FAST, ANAL'
         CALL BATCH
         GOTO 1
      END IF
      IF (DMODE.EQ.DERIVA) THEN
         IF(STRO)WRITE(IUOUT,127)DMODE
         CALL BATCH
         GOTO 1
      END IF
C      
C   SET THE RIGHT INPUT FLAG FOR THE CALL TO SUBROUTINE COMPDR
      ICHGRA = 1
      CALL COMPDR(IERR,ICHGRA,DMODE)
      GOTO 1
C  .........................................................  GRAPH
 50   CONTINUE
      IF (PANEL.EQ.'PANELON') CALL GRPAN(ICANC)
      IF (ICANC.NE.1 .OR. ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
C   MAKE A ROUGH TERMINAL GRAPH
      CALL GRA1(NOCGR,ILIN,ICOL,IWVAR,GXMIN,GXMAX,XAX,YAX)
      GOTO 1
C  .........................................................  LIST
 51   CONTINUE
      WRITE (IUOUT,'(5(5X,A))') (COMMAN(I),I=1,NCOM)
      GOTO 1
C  .........................................................  HELP
 54   CONTINUE
C   INITIATE A HELP SESSION
C   SET THE HELP-FLAG
      IHF =1
      CALL HELP(IHF)
      GOTO 1
C  .........................................................  QUIT
 55   CONTINUE
      IF (STRO) WRITE(IUOUT,*)'      ...  ENTER RETURN - FLAG  ...'
      READ(IUINP,*,END=92)  IQU
      IF (STRO) WRITE(IUOUT,*)' ... RETURN FLAG IS SET EQUAL TO: ',IQU
      IRET = 1
      IRET2 = 1
C  SUPRESS EPILOG IF ANYTHING ELSE IS ENTERED IN THE COMMAND LINE.
      IF (NIT.GT.1 .AND. EPILO.NE.' ') THEN
         CALL MID1('# EPILOG EXECUTION HAS BEEN SUPRESSED #')
         EPILO = ' '
      END IF
      GOTO 1
C  .........................................................  INTRODUCE
 56   CONTINUE
C  SET THE INTRODUCE FLAG
      IIF = 2
      CALL HELP(IIF)
      GOTO 1
C  .........................................................  MACRO
 57   CONTINUE
      FILSTA(6)='  * '
C   INITIATE MACRO CONSTRUCTION.
      CALL CONMAC
      GOTO 1
C  .........................................................  CLEAR
 58   CONTINUE
      IF (LEVEL.EQ.0) THEN
         CALL MID2('CLEAR HAS NO EFFECT OUTSIDE A MACRO')
      ELSE
         IF (STRO) THEN
            LE = LENGTH(CBACK(LEVEL))
            LINE = 'PROCESSING OF MACRO '//CBACK(LEVEL)(1:LE)//
     >             ' HAS BEEN COMPLETED'
            CALL MID1(LINE)
         END IF
         CALL BACKLE
      END IF
      GOTO 1
C  .........................................................  ZZUSCOM01
 59   CONTINUE
      GOTO 1
C  .........................................................  INIT
 60   CONTINUE
C DISPOSE CURRENT POINT IN FILE :  DISPO
      CALL DISWRI
C  READ IN THE NEW POINT FROM SPECIFIED FILE.
      CALL SETPOI(FILENA)
      GOTO 1
C  .........................................................  ACCUM
 61   CONTINUE
      IF (PANEL.EQ.'PANELON') CALL ACCPAN(ICANC)
      IF (ICANC.NE.1 .OR. ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
      CALL DISWRI
C  ACCUMULATE IN FILE: INIPO ,NOPOI NU. OF POINTS,WITH VALUE
C  LOWER OR EQUAL TO VALTA. MAXIMUM ATTEMPTS ALLOWED: NOCAL
      CALL ACCUMP(NOPAC,NOCAC,TARAC,IERFL,IPOI)
      IF (IERFL.EQ.1) CALL BATCH
      GOTO 1
C  .........................................................  ADJUST
 62   CONTINUE
      IF (ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
      CALL DWRITE
C  ADJUST THE SIZES OF THE SEARCH STEPS.
      CALL PRADJ
      GOTO 1
C  .........................................................  RUNMCL
 63   CONTINUE
      ISTACK = 0
      WRITE(IUOUT,*) ' ENTER FILE-NAME'
      READ(IUINP,'(A)',END=92) FNAME
      DO 131 MM = 1,6
      IF (FNAME.EQ.FILENA(MM)) THEN
      WRITE(IUOUT,*) ' RESERVED FILE-NAME : ',FNAME
      GOTO 92
      END IF
 131  CONTINUE
      INQUIRE(FILE=FNAME,OPENED=LOPEN,IOSTAT= INQERR)
      IF (INQERR .NE. 0 ) THEN
      IF (STRO) WRITE(IUOUT,*) ' INCORRECT FILE NAME: ',FNAME
      GOTO 1
      END IF
      IOPS=0
      IF (.NOT.LOPEN) OPEN(IU47,FILE=FNAME,IOSTAT=IOPS)
      IF (IOPS .NE. 0 ) THEN
      IF (STRO) WRITE(IUOUT,*)' INCORRECT FILE NAME  ',FNAME
      GOTO 1
      END IF
      REWIND IU47
      IUINP=IU47
      INFILE=IU47
      NUMFI = 0
      GOTO 1
C  .........................................................  FIXALL
 64   CONTINUE
      IF (ITNOCS.EQ.0) GOTO 1
      DO 201,I=1,NOD
        IXAT(I) = 0
 201  CONTINUE
C  UPATE FILE : DATA
      CALL DWRITE
      GOTO 1
C  .........................................................  VALDIS
 65   CONTINUE
      IF (ITNOCS.EQ.0) GOTO 1
      IF (STRO) THEN
         CALL DASH
         WRITE (IUOUT,133)
         CALL DASH
         WRITE (IUOUT,134) ITNOCS,ITDER,IPARTC,IPDER
         CALL DASH
         WRITE(IUOUT,'(/T20,''VALUE = '',1PG21.14/)') VAL
      END IF
      GOTO 1
C  .........................................................  NOEVAL
 66   CONTINUE
      MODEV = 0
      VAL = 0.
      GOTO 1
C  .........................................................  FAST
 67   CONTINUE
      DERIVA = 'FAST'
      DO 204,I=1,NOD
 204    GRADI(I) = 'FAST'
      GOTO 1
C  .........................................................  QUAD
 68   CONTINUE
      DERIVA = 'QUAD'
      DO 203,I=1,NOD
 203    GRADI(I) = 'QUAD'
      GOTO 1
C  .........................................................  ZZUSCOM14
 69   CONTINUE
      GOTO 1
C  .........................................................  ZZUSCOM15
 70   CONTINUE
      GOTO 1
C  .........................................................  SIMPLEX
 71   CONTINUE
      IF (PANEL.EQ.'PANELON')CALL SIMPAN(ICANC)
      IF (ICANC.NE.1 .OR. ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
      NEED = NOD**2+NOD
      IF (MAXW.LT.NEED) THEN
         WRITE (IUOUT,321) 'SIMPLEX', 'N**2+N', MAXW, NEED
         GOTO 1
      END IF
C   INITIATE THE SIMPLEX METHOD
      CALL SIMPL(VERSIM,NOD,INIS,ERR,ERR1,MNOC,MNOC1,IPRIS,NOCAL)
      IF (STRO) WRITE(IUOUT,126)'SIMPLEX',NOCAL
      GOTO 1
C  .........................................................  ROLL
 72   CONTINUE
      GOTO 1
C  .........................................................  RANDOM
  73  CONTINUE
      IF (PANEL.EQ.'PANELON') CALL RANPAN(ICANC)
      IF (ICANC.NE.1 .OR. ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
      IOPT=0
      IF (LSEAR.NE.0) IOPT=1
C  INITIATE THE RANDOM METHOD
      CALL RANDS(NOCR,IPRIR,NOCEXR,IVOLR,IOPT,REDFR,ICYSR,
     >           IFAILR,LSEAR,ACCR,EPSR)
      IF (STRO) WRITE(IUOUT,126)'RANDOM',NOCEXR
      GOTO 1
C  .........................................................  BFGS
 74   CONTINUE
      IF (PANEL.EQ.'PANELON') CALL BFGPAN(ICANC)
      IF (ICANC.NE.1 .OR. ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
      NEED = NOD**2
      IF (MAXW.LT.NEED) THEN
         WRITE (IUOUT,321) 'BFGS', 'N**2', MAXW, NEED
         GOTO 1
      END IF
C  INITIATE THE BFGS METHOD
      AUTODE=MADB.EQ.1
      CALL BFGSMV(VERSIM,NOCB,NOCEXB,ACCB,IERR,IDIGB,ERBB,IPRIB,EPSB)
      AUTODE=.FALSE.
      IF (STRO) WRITE(IUOUT,126)'BFGS',NOCEXB
      GOTO 1
C  .........................................................  DFP
 75   CONTINUE
      IF (PANEL.EQ.'PANELON') CALL DFPPAN(ICANC)
      IF (ICANC.NE.1 .OR. ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
      NEED = NOD**2
      IF (MAXW.LT.NEED) THEN
         WRITE (IUOUT,321) 'DFP', 'N**2', MAXW, NEED
         GOTO 1
      END IF
C  INITIATE THE DFP METHOD
      AUTODE=MADD.EQ.1
      CALL DFPMV(VERSIM,NOCD,NOCEXD,ACCD,IERR,IDIGD,ERBD,IPRID,EPSD)
      AUTODE=.FALSE.
      IF (STRO) WRITE(IUOUT,126)' DFP',NOCEXD
      GOTO 1
C  .........................................................  CONGRA
 76   CONTINUE
      IF (PANEL.EQ.'PANELON') CALL CGRPAN(ICANC)
      IF (ICANC.NE.1 .OR. ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
C  INITIATE THE CONGRA METHOD
      AUTODE=MADC.EQ.1
      CALL COGRMV(NOCC,NOCEXC,ACCC,IERR,IDIGC,ERBC,IPRIC,EPSC,METHOC)
      AUTODE=.FALSE.
      IF (STRO) WRITE(IUOUT,126)'CONGRA',NOCEXC
      GOTO 1
C  .........................................................  AUTO
 77   CONTINUE
      IF (PANEL.EQ.'PANELON') CALL SPELL(ICANC)
      IF (ICANC.NE.1 .OR. ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
      NEED = NOD**2+NOD
      IF (MAXW.LT.NEED) THEN
         WRITE (IUOUT,321) 'AUTO', 'N**2+N', MAXW, NEED
         GOTO 1
      END IF
      AUTODE=MADA.EQ.1
      CALL MAGIC(NTOT,VALOW,VERSIM)
      AUTODE=.FALSE.
      GOTO 1
C  .........................................................  TOLMIN
 78   CONTINUE
      IF (PANEL.EQ.'PANELON') CALL TOLPAN(ICANC)
      IF (ICANC.NE.1 .OR. ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
      NEED = NOD**2+11*NOD
      IF (MAXW.LT.NEED) THEN
         WRITE (IUOUT,321) 'TOLMIN', 'N**2+11*N', MAXW, NEED
         GOTO 1
      END IF
C  INITIATE THE TOLMIN METHOD
      AUTODE=MADT.EQ.1
      CALL TOLI(VERSIM,NOCT,ACCT,IPRIT,NOCEXC)
      AUTODE=.FALSE.
      IF (STRO) WRITE(IUOUT,126)'TOLMIN',NOCEXC
      GOTO 1
C  .........................................................  CSET
 79   CONTINUE
      IF (NIT.EQ.1) THEN
         IF (STRO) WRITE (IUOUT,136) CSET
         IF (STRO) WRITE (IUOUT,*) 'ENTER NEW CHARACTER SET'
         READ(IUINP,'(A)',END=92) CHS
         CALL UPCASE(CHS)
         IF (CHS.EQ.'NORMAL' .OR. CHS.EQ.'ASCII' .OR.
     >       CHS.EQ.'EBCDIC') THEN
            CSET = CHS
         IF (STRO) WRITE (IUOUT,*) 'CHARACTER SET IS SET TO ',CSET
         ELSE IF (CHS.NE.' ') THEN
            IF (STRO) WRITE (IUOUT,*) 'INCORRECT CHARACTER SET'
         CALL BATCH
         END IF
      ELSE IF (NIT.EQ.2) THEN
         CHS = INPLIN(IS(2):IE(2))
         IF (CHS.EQ.'NORMAL' .OR. CHS.EQ.'ASCII' .OR.
     >       CHS.EQ.'EBCDIC') THEN
            CSET = CHS
         IF (STRO) WRITE (IUOUT,*) 'CHARACTER SET IS SET TO ',CSET
         ELSE
            IF (STRO) WRITE (IUOUT,*) 'INCORRECT CHARACTER SET'
         CALL BATCH
         END IF
      ELSE
         IF (STRO) WRITE (IUOUT,*) 'TOO MANY PARAMETERS'
         CALL BATCH
      END IF
      GOTO 1
C  .........................................................  STEPDIS
 80   CONTINUE
      IF (STRO) THEN
         DO 19,I=1,NOD
            WRITE(IUOUT,'(T5,I6,'')'',3X,''....'',1PG11.4)')I,STEP(I)
19       CONTINUE
      END IF
      GOTO 1
C  .........................................................  NOTARGET
 81   CONTINUE
      AIMON = .FALSE.
      GOTO 1
C  .........................................................  MIXED
 82   CONTINUE
      DERIVA = 'MIXED'
      CALL MIXED
      GOTO 1
C  .........................................................  EVALUATE
 83   CONTINUE
      MOP = MODEV
      MODEV = 1
C
C  CHECK IF THERE ARE ANY UNDEFINED VARIABLES
      IF (ITNOCS.EQ.0) THEN
        CALL UNDEF(INDEF,NOD,IUOUT,STRO,IGO)
        IF (IGO.EQ.0) GOTO 1
      END IF
      VALNEW = ACSQ(POINT,NOD)
      IF (VALNEW.NE.VAL .AND. MOP.EQ.1) THEN
         WRITE(IUOUT,128)
         CALL BATCH
      END IF
      VAL = VALNEW
      GOTO 1
C  .........................................................  GERROR
 84   CONTINUE
      IF (STRO) WRITE (IUOUT,132) GLERB
      IF (STRO) WRITE (IUOUT,*) 'ENTER GRADIENT ERROR-BOUND'
      READ (IUINP,'(A)',END=92) LINE
      LE = LENGTH(LINE)
      IF (LE.EQ.0) GOTO 1
      CALL TOREAL(LINE,LE,GE,IER)
      IF (IER.NE.0) THEN
         WRITE (IUOUT,'(T10,A,'' IS NOT A NUMBER'')') LINE(1:LE)
         WRITE (IUOUT,129)
         CALL BATCH
      ELSE
         GE = ABS(GE)
         IF (GE.GT.1.) THEN
            WRITE (IUOUT,129)
         CALL BATCH
         ELSE
            IF (GE.LT.COMTOL) GE = COMTOL
            GLERB = GE
         END IF
      END IF
      GOTO 1
C  .........................................................  MAD
 85   CONTINUE
      IF (PANEL.EQ.'PANELON') CALL MADPAN
      GOTO 1
C  .........................................................  TARGET
 86   CONTINUE
      IF (STRO) WRITE (IUOUT,135) TARGET
      IF (STRO) WRITE (IUOUT,*) 'ENTER NEW TARGET VALUE'
      READ (IUINP,'(A)',END=92) LINE
      LE = LENGTH(LINE)
      IF (LE.EQ.0) THEN
         AIMON = .TRUE.
      ELSE
         CALL TOREAL(LINE,LE,TA,IER)
         IF (IER.NE.0) THEN
            WRITE (IUOUT,'(T10,A,'' IS NOT A NUMBER'')') LINE(1:LE)
            CALL BATCH
         ELSE
            TARGET = TA
            AIMON = .TRUE.
         END IF
      END IF
      GOTO 1
C  .........................................................  ALIAS
 87   CONTINUE
      CALL NEWALI
      GOTO 1
C  .........................................................  REWIND
 88   CONTINUE
      IF (STRO) WRITE(IUOUT,*)'     ...  ENTER  FILE  NAME  ...'
      READ(IUINP,99,END=92)   FNAME
      INQUIRE(FILE=FNAME,OPENED=LOPEN,NUMBER=IUNUM)
      IF (LOPEN) REWIND IUNUM
      IF (.NOT.LOPEN) THEN
       OPEN(IU36,FILE=FNAME,IOSTAT = ISCR )
C   CHECK IF AN ERROR HAS OCCURED .
         IF ( ISCR .NE. 0 ) THEN
         IF (STRO) WRITE(IUOUT,'(T5,''INCORRECT FILE NAME'',3X,A)')FNAME
          CALL BATCH
          GOTO 1
         END IF
        REWIND IU36
      CLOSE(IU36)
      END IF
      GOTO 1
C  .........................................................  HIDEOUT
 89   CONTINUE
C
C CHECK IF MERLIN IS UNDER MANUAL CONTROL, OR UNDER MCL CONTROL
C
      IF ( IUINP .EQ. IU31 ) THEN
      IF (STRO) WRITE(IUOUT,*)'.. ENTER UNIT NU. FOR OUTPUT DISPOSAL ..'
      READ(IUINP,*,END=92)IHIDOU
      INQUIRE(UNIT=IHIDOU , OPENED = LOPEN)
      IF ( LOPEN ) THEN
      WRITE(IUOUT,*) ' UNIT NUMBER: ',IHIDOU,' IS OCCUPIED '
      WRITE(IUOUT,*) ' HIDEOUT  IGNORED '
      GOTO 1
      END IF
      I1=IHIDOU
      I2=(I1-IU31)*(I1-IU32)*(I1-IUDAT)*(I1-IUINI)*(I1-IUBAC)*(I1-IU36)
      I2=I2*(I1-IUMAC)*(I1-IU42)*(I1-IUSTO)*(I1-IUDIS)*(I1-IU45)
      I2=I2*(I1-IU47)
       IF (I2.EQ.0) THEN
        IF (STRO) WRITE(IUOUT,*)'UNIT ',IHIDOU,' IS RESERVED BY MERLIN'
        CALL BATCH
        GOTO 1
       END IF
        WRITE(FNAME,'(''HIDE'',I2.2)')IHIDOU
        OPEN(IHIDOU,FILE=FNAME)
      REWIND IHIDOU
C  REASSIGN OUTPUT UNIT TO SPECIFIED ALTERNATE UNIT.
      IUOUT=IHIDOU
      ELSE
C
C  MERLIN UNDER  MCL CONTROL. READ IN THE HIDEOUT FILENAME
C
      MCLNAM= '   '
      READ(IUINP,'(A)' )MCLNAM
      INQUIRE (FILE = MCLNAM , EXIST = LOPEN)
      IF (LOPEN) THEN
      WRITE(IUOUT,*) 'FILE: ', MCLNAM, ' ALREADY EXISTS'
      STOP
      END IF
      DO 130 I=200,800
      INQUIRE(UNIT = I , OPENED = LOPEN)
      IF ( .NOT. LOPEN ) THEN
      IHIDOU = I
      OPEN(IHIDOU, FILE = MCLNAM )
      GOTO 140
      END IF
 130  CONTINUE
      WRITE(IUOUT,*) ' ALL UNITS IN USE '
      STOP
 140  CONTINUE
      IUOUT=IHIDOU
      END IF
      GOTO 1
C  .........................................................  REVEAL
 90   CONTINUE
C   REASSIGN OUTPUT UNIT TO DEFAULT.
      IUOUT=IU32
      GOTO 1
C  .........................................................  RESET
 91   CONTINUE
C  RESET PARTIAL CALL-COUNTERS TO ZERO.
      IPARTC=0
      IPDER = 0
      GOTO 1
C  .........................................................  DISKON
190   CONTINUE
      IDISK = 1
      GOTO 1
C  .........................................................  DISKOFF
191   CONTINUE
      IDISK = 0
      GOTO 1
C  .........................................................  LIMITS
192   CONTINUE
      CALL LIMITS
      GOTO 1
C  .........................................................  DATAWRITE
193   CONTINUE
      DATAMO = 'DATAWRITE'
      GOTO 1
C  .........................................................  DATASKIP
194   CONTINUE
      DATAMO = 'DATASKIP'
      GOTO 1
C  .........................................................  FLAG
195   CONTINUE
      CALL CHFLAG
      GOTO 1
C  .........................................................  FLAGDIS
196   CONTINUE
      CALL FLADIS
      GOTO 1
C  .........................................................  CFLAG
197   CONTINUE
      CALL CCHFLA
      GOTO 1
C  .........................................................  CFLAGDIS
198   CONTINUE
      CALL CFLADI
      GOTO 1
C  .........................................................  DISPOSKIP
199   CONTINUE
      DISMOD = 'DISPOSKIP'
      GOTO 1
C  .........................................................  DISPOWRITE
200   CONTINUE
      DISMOD = 'DISPOWRITE'
      GOTO 1
C  .........................................................  ZZUSCOM15
301   CONTINUE
      GOTO 1
C  .........................................................  LEVE
302   CONTINUE
      IF (PANEL.EQ.'PANELON') CALL LEVPAN(ICANC)
      IF (ICANC.NE.1 .OR. ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
      NEED = NOD*NOF
      IF (MAXW.LT.NEED) THEN
         WRITE (IUOUT,321) 'LEVE', 'N*M', MAXW, NEED
         GOTO 1
      END IF
C  INITIATE THE LEVENBERG-MARQUARD METHOD
      IF (IFFORM.EQ.0) THEN
         WRITE (IUOUT,137) 
      ELSE
         CALL LEVE(VERSIM,NOCL,IPRIL,GTOLL,FTOLL,XTOLL,FACTL,EPSFL,NOCX)
         IF (STRO) WRITE (IUOUT,126) 'LEVE', NOCX
      END IF
      GOTO 1
C  .........................................................  GENERAL
303   CONTINUE
      IFFORM = 0
      GOTO 1
C  .........................................................  SOS
304   CONTINUE
C
C  YOU CANNOT ENTER THE SUM-OF-SQUARES MODE IF "0" TERMS
C  WERE SPECIFIED.
      IF (NOF.EQ.0) THEN
         WRITE (IUOUT,138)
         CALL BATCH
      ELSE
         IFFORM = 1
      END IF
      GOTO 1
C  .........................................................  ROLL
305   CONTINUE
      IF (PANEL.EQ.'PANELON') CALL ROLPAN(ICANC)
      IF (ICANC.NE.1 .OR. ITNOCS.EQ.0 .OR. MODEV.EQ.0) GOTO 1
C  INITIATE THE  ROLL(3) METHOD
      CALL HYPER(NOCH,ACCH,ENHH,IFMH,IPRIH,ICOUNT,IDIGH,EPSH)
      IF (STRO) WRITE(IUOUT,126)'ROLL',ICOUNT
      GOTO 1
C  .........................................................  ZZUSCOM17
306   CONTINUE
      GOTO 1
C  .........................................................  ZZUSCOM18
307   CONTINUE
      GOTO 1
C  .........................................................  ZZUSCOM19
308   CONTINUE
      GOTO 1
C  .........................................................  ZZUSCOM20
309   CONTINUE
      GOTO 1
C  .........................................................  ZZUSCOM21
310   CONTINUE
      GOTO 1
C  .........................................................  ZZUSCOM22
311   CONTINUE
      GOTO 1
C  .........................................................  ZZUSCOM23
312   CONTINUE
      GOTO 1
C  .........................................................  ZZUSCOM24
313   CONTINUE
      GOTO 1
C  ---------------------------------------------------------------------
C  CONTROL TRANSFER UPON END-OF-FILE IN INPUT UNIT
C  ---------------------------------------------------------------------
   92 CONTINUE
C  IF IN BATCH OPTION OF THE PROCES MODE ABORT ON ERRORS
      CALL BATCH
      REWIND IUINP
      IF (IUINP.EQ.INFILE)IUINP=IU31
      GOTO 1
C  ---------------------------------------------------------------------
C  CONTROL IS TRANSFERED HERE UPON TERMINATION OF EACH COMMAND.
C  ---------------------------------------------------------------------
 1    CONTINUE
C
C  CHECK IF THE PREVIOUS COMMAND IS A MINIMIZATION COMMAND AND
C  UPDATE FILE "BACKUP" ACCORDING TO THE BACKUP MODE.
      IF (CLASS(ICOM).EQ.1 .AND. ICANC.EQ.1) CALL BWRITE
      RETURN
C  ---------------------------------------------------------------------
 99   FORMAT(A10)
 101  FORMAT(/'MACROF CONTAINS MACROS. IT CANNOT BE INSPECTED.'/)
 102  FORMAT(3X,'FILE : ',A10,' IS EMPTY. PICK NOT ALLOWED')
 103  FORMAT(6(2X,A10)/70('-')/)
 104  FORMAT(/20X,'*** MERLIN - FILES ***'/70('-')/6(2X,A10)/)
 105  FORMAT(2X,'PICK IS ALLOWED ONLY WHEN FOLLOWS AN INSPECT COMMAND')
 113  FORMAT(10X,'*** ENTER FILE TO DISCARD ***')
 114  FORMAT(9X,'WARNING: ....  CANNOT DISCARD FILES OTHER THAN :'/
     >       15X, 'STORE  DISPO  BACKUP')
 115  FORMAT(' NO NAMES SPECIFIED AS OF YET. USE THE GODFATHER COMMAND')
 118  FORMAT(/10X,'***  VALUE = ',1PG21.14,'  ***')
 119  FORMAT(8(A10))
 120  FORMAT(T10,'MERLIN ADMINISTRATION COMMANDS '/T10,30('-'))
 121  FORMAT(/T10,'MERLIN   INFORMATION  COMMANDS '/T10,30('-'))
 122  FORMAT(//T5,'ENTER  CARRIAGE RETURN OR   C   TO CONTINUE')
 123  FORMAT(/T10,'MERLIN    EDITING     COMMANDS '/T10,30('-'))
 124  FORMAT(/T10,'MERLIN  MINIMIZATION  COMMANDS '/T10,30('-'))
 125  FORMAT(/T10,'MERLIN/USER RESERVED  COMMANDS '/T10,30('-'))
 126  FORMAT(T10,'NUMBER OF',A10,'  CALLS: ',I10)
 127  FORMAT(T10,A,' IS YOUR CURRENT MODE. SWITCH MODE.')
 128  FORMAT(/T10,'WARNING: FUNCTION EVALUATION IS INCONSISTENT')
 129  FORMAT(/T10,'PLEASE ENTER A REAL NUMBER IN THE INTERVAL (0,1)')
 132  FORMAT(/T2,'CURRENT ERROR BOUND IS: ',1PG14.3)
 133  FORMAT(T2,'NUMBER OF EVALUATIONS:    FUNCTION        GRADIENT')
 134  FORMAT(T2,'TOTAL:',T28,I8,T44,I8/
     >       T2,'SINCE LAST RESET:',T28,I8,T44,I8)
 135  FORMAT(/T2,'CURRENT TARGET VALUE IS: ',1PG14.5)
 136  FORMAT(/T2,'CURRENT CHARACTER SET IS: ',A)
 137  FORMAT(/T2,'YOU CANNOT USE THIS METHOD ON A GENERAL FUNCTION')
 138  FORMAT(/T2,'YOU CANNOT ENTER THE SUM-OF-SQUARES MODE BECAUSE'/
     >        T2,'THE NUMBER OF TERMS SPECIFIED WAS 0.')
 321  FORMAT(/T2,'YOU CANNOT USE THE ',A,' METHOD BECAUSE YOU DID NOT'/
     >       T2,'ALLOCATE SUFFICIENT STORAGE IN THE "VERSIM" ARRAY.'/
     >       T2,'THE REQUIRED MEMORY IS: ',A/
     >       T2,'ALLOCATED ',I8,' MEMORY WORDS.'/
     >       T2,'NEEDED    ',I8,' MEMORY WORDS.')
      END
      FUNCTION FACT ( N )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      SAVE NFACT, FIRST, FCT, ATLIM
C
      PARAMETER ( MAXFCT = 200 )
      DIMENSION FCT(0:MAXFCT)
C
      LOGICAL FIRST, ATLIM
      DATA FIRST / .TRUE. /
C
      IF (FIRST) THEN
         FIRST = .FALSE.
         FCT(0) = 1
         SUMLOG = 0
         BIGLOG = LOG(BIGNUM)
         DO 10,I=1,MAXFCT
            X = I
            SUMLOG = SUMLOG+LOG(X)
         IF (SUMLOG.GE.BIGLOG) THEN
            NFACT = I-1
            ATLIM = .TRUE.
            GOTO 20
         ELSE
            FCT(I) = I*FCT(I-1)
         END IF
10       CONTINUE
         NFACT = MAXFCT
         ATLIM = .FALSE.
20       CONTINUE
      END IF
C
      IF (N.LT.0) THEN
         WRITE (*,*) '-- ERROR: NEGATIVE ARGUMENT IN FACT'
         FACT = -1
      ELSE IF (N.LE.NFACT) THEN
         FACT = FCT(N)
      ELSE IF (ATLIM) THEN
         WRITE (*,*) '-- ERROR: RESULTING FACTORIAL TOO BIG.'
         FACT = -1
      ELSE
         F = FCT(NFACT)
         DO 30,I=NFACT+1,N
30          F = F*I
         FACT = F
      END IF
C
      END
      SUBROUTINE FIXLOO ( LL )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     IMPLEMENTS THE  FIX/LOOSE OPERATION FOR ONE OR MORE VARIABLES
C     DEPENDING ON IF LL= 0 (FIX) OR LL=1 (LOOSE)
C
      PARAMETER ( MXV = 200 )
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
C
      INTEGER INDE(40)
C
      CALL GETI(INDE,NIT,IERR)
C
C  DO NOT CHANGE FIX STATUS IF ANY ERROR OCCURED.
      IF (IERR.EQ.0) THEN
         DO 30,I=1,NIT
30          IXAT(INDE(I)) = LL
      END IF
C
      END
      SUBROUTINE CHFLAG
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      PARAMETER ( MAXFLA = 20 )
      COMMON / FLAGS / FLAG(MAXFLA)
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      INTEGER INDE(40)
      DIMENSION VALUE(40)
C
      CALL GGIV(MAXFLA,INDE,VALUE,NPAIRS,IERR)
      IF (IERR.EQ.0) THEN
         DO 20,I=1,NPAIRS
            FLAG(INDE(I)) = VALUE(I)
20       CONTINUE
      END IF
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE FLADIS
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      PARAMETER ( MAXFLA = 20 )
      COMMON / FLAGS / FLAG(MAXFLA)
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      IF (STRO) THEN
         WRITE (IUOUT,30)
         DO 20,I=1,MAXFLA,4
20          WRITE (IUOUT,10) (I+K,FLAG(I+K),K=0,3)
      END IF
C
10    FORMAT (T2,4('FLAG(',I2,')= ',F6.2,3X))
30    FORMAT (T30,'CURRENT FLAG VALUES')
      END
      SUBROUTINE GETI ( INDE, NIT, IERR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  POSSIBLE OUTPUT VALUES FOR IERR: 0,2 : AS IN SUBROUTINE INLINE.
C                                   3   : INCORRECT INDEX.
C
      INTEGER INDE(*)
C
      PARAMETER ( MEM = 2500, LSTA = 500 )
      COMMON/MCLVAR/STACK(LSTA),CMEM(MEM),ISTACK,INFILE
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      IF (IUINP.NE.INFILE) THEN
         CALL GETI1(INDE,NIT,IERR)
      ELSE
         CALL GETI2(INDE,NIT,IERR)
      END IF
C
C  ANY INDEX WAS OUT OF THE ALLOWED RANGE (1..NOD) ?
      IF (IERR.EQ.3) CALL BATCH
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE GETI1 ( INDE, NIT, IERR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      INTEGER INDE(*)
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      CHARACTER*80 LINE
      INTEGER IS(40), IE(40)
C
      CALL INLINE(LINE,.TRUE.,.TRUE.,IS,IE,NIT,IERR)
      IF (IERR.NE.0) RETURN
C
      DO 10,I=1,NIT
         IND = IPARIN(LINE(IS(I):IE(I)))
         IF (IND.EQ.0) THEN
            IF (STRO) WRITE (IUOUT,11) LINE(IS(I):IE(I))
            IERR = 3
         END IF
         INDE(I) = IND
10    CONTINUE
C
11    FORMAT(T2,'INCORRECT VARIABLE INDEX: ',A)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE GETI2 ( INDE, NIT, IERR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      INTEGER INDE(*)
C
      PARAMETER ( MXV = 200 )
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
C
      PARAMETER ( MEM = 2500, LSTA = 500 )
      COMMON/MCLVAR/STACK(LSTA),CMEM(MEM),ISTACK,INFILE
C
      IERR = 0
      NIT = NINT(STACK(ISTACK))
      ISTACK = ISTACK-1
      DO 20,I=1,NIT
        IND = NINT(STACK(ISTACK))
        IF (IND.LT.1 .OR. IND.GT.NOD) THEN
            IF (STRO) WRITE (IUOUT,12) IND
            INDE(I) = 0
            IERR = 3
         ELSE
            INDE(I) = IND
         END IF
         ISTACK = ISTACK-1
20    CONTINUE
C
12    FORMAT(T2,'INCORRECT VARIABLE INDEX: ',I4)
      END
      SUBROUTINE GETIV ( INDE, VALUE, NPAIRS, IERR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  POSSIBLE OUTPUT VALUES FOR IERR: 0,2 : AS IN SUBROUTINE INLINE.
C                                   3   : INCORRECT INDEX.
C                                   4   : INCORRECT VALUE.
C
      INTEGER INDE(*)
      DIMENSION VALUE(*)
C
      PARAMETER ( MEM = 2500, LSTA = 500 )
      COMMON/MCLVAR/STACK(LSTA),CMEM(MEM),ISTACK,INFILE
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      IF (IUINP.NE.INFILE) THEN
         CALL GETIV1(INDE,VALUE,NPAIRS,IERR)
      ELSE
         CALL GETIV2(INDE,VALUE,NPAIRS,IERR)
      END IF
C
C  ANY INDEX WAS OUT OF THE ALLOWED RANGE (1..NOD) ?
      IF (IERR.EQ.3) CALL BATCH
C
C  AN INCORRECT VALUE WAS GIVEN ?
      IF (IERR.EQ.4) CALL BATCH
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE GETIV1 ( INDE, VALUE, NPAIRS, IERR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      INTEGER INDE(*)
      DIMENSION VALUE(*)
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      CHARACTER*80 LINE
      INTEGER IS(40), IE(40)
C
      CALL INLINE(LINE,.TRUE.,.TRUE.,IS,IE,NIT,IERR)
      IF (IERR.NE.0) RETURN
C
      I = 0
      NPAIRS = 0
 200  CONTINUE
      I = I+1
      IF (I.GT.NIT) GOTO 400
      IND = IPARIN(LINE(IS(I):IE(I)))
      IF (IND.EQ.0) THEN
         IF (STRO) WRITE(IUOUT,11) LINE(IS(I):IE(I))
         CALL BATCH
         GOTO 400
      END IF
      I = I+1
      IF (I.GT.NIT) THEN
         IF (STRO) WRITE(IUOUT,12)
         CALL BATCH
         GOTO 400
      END IF
      LE = IE(I)-IS(I)+1
      CALL TOREAL(LINE(IS(I):IE(I)),LE,VL,IVERR)
      NPAIRS = NPAIRS+1
      IF (IVERR.EQ.0) THEN
         INDE(NPAIRS) = IND
         VALUE(NPAIRS) = VL
      ELSE
         IERR = 4
         INDE(NPAIRS) = 0
         VALUE(NPAIRS) = 0
      END IF
      GOTO 200
C
 400  CONTINUE
C
11    FORMAT(T2,'INCORRECT VARIABLE INDEX: ',A)
12    FORMAT(T2,'LAST ITEM IN INPUT LINE IS OBSOLETE. IGNORED.')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE GETIV2 ( INDE, VALUE, NPAIRS, IERR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      INTEGER INDE(*)
      DIMENSION VALUE(*)
C
      PARAMETER ( MXV = 200 )
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
C
      PARAMETER ( MEM = 2500, LSTA = 500 )
      COMMON/MCLVAR/STACK(LSTA),CMEM(MEM),ISTACK,INFILE
C
      IERR = 0
      NPAIRS = NINT(STACK(ISTACK))
      ISTACK = ISTACK-1
      DO 20,I=NPAIRS,1,-1
         VALUE(I) = STACK(ISTACK)
         ISTACK=ISTACK-1
         IND = NINT(STACK(ISTACK))
         IF (IND.LT.1 .OR. IND.GT.NOD) THEN
            IF (STRO) WRITE (IUOUT,12) IND
            INDE(I) = 0
            IERR = 3
         ELSE
            INDE(I) = IND
         END IF
         ISTACK = ISTACK-1
20    CONTINUE
C
12    FORMAT(T2,'INCORRECT VARIABLE INDEX: ',I4)
      END
      SUBROUTINE GGIV ( NIND, INDE, VALUE, NPAIRS, IERR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  POSSIBLE OUTPUT VALUES FOR IERR: 0,2 : AS IN SUBROUTINE INLINE.
C                                   3   : INCORRECT INDEX.
C                                   4   : INCORRECT VALUE.
C
      INTEGER INDE(*)
      DIMENSION VALUE(*)
C
      PARAMETER ( MEM = 2500, LSTA = 500 )
      COMMON/MCLVAR/STACK(LSTA),CMEM(MEM),ISTACK,INFILE
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      IF (IUINP.NE.INFILE) THEN
         CALL GGIV1(NIND,INDE,VALUE,NPAIRS,IERR)
      ELSE
         CALL GGIV2(NIND,INDE,VALUE,NPAIRS,IERR)
      END IF
C
C  ANY INDEX WAS OUT OF THE ALLOWED RANGE (1..NIND) ?
      IF (IERR.EQ.3) CALL BATCH
C
C  AN INCORRECT VALUE WAS GIVEN ?
      IF (IERR.EQ.4) CALL BATCH
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE GGIV1 ( NIND, INDE, VALUE, NPAIRS, IERR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      INTEGER INDE(*)
      DIMENSION VALUE(*)
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      CHARACTER*80 LINE
      INTEGER IS(40), IE(40)
C
      CALL INLINE(LINE,.TRUE.,.TRUE.,IS,IE,NIT,IERR)
      IF (IERR.NE.0) RETURN
C
      I = 0
      NPAIRS = 0
 200  CONTINUE
      I = I+1
      IF (I.GT.NIT) GOTO 400
C
C  GET THE INDEX.
      LE = IE(I)-IS(I)+1
      CALL TOINT(LINE(IS(I):IE(I)),LE,II,JJ)
      IF (JJ.NE.0) THEN
         IND = 0
      ELSE IF (II.GT.NIND .OR. II.LE.0) THEN
         IND = 0
      ELSE
         IND = II
      ENDIF
C
      IF (IND.EQ.0) THEN
         IF (STRO) WRITE(IUOUT,11) LINE(IS(I):IE(I))
         CALL BATCH
         GOTO 400
      END IF
      I = I+1
      IF (I.GT.NIT) THEN
         IF (STRO) WRITE(IUOUT,12)
         CALL BATCH
         GOTO 400
      END IF
      LE = IE(I)-IS(I)+1
      CALL TOREAL(LINE(IS(I):IE(I)),LE,VL,IVERR)
      NPAIRS = NPAIRS+1
      IF (IVERR.EQ.0) THEN
         INDE(NPAIRS) = IND
         VALUE(NPAIRS) = VL
      ELSE
         IERR = 4
         INDE(NPAIRS) = 0
         VALUE(NPAIRS) = 0
      END IF
      GOTO 200
C
 400  CONTINUE
C
11    FORMAT(T2,'INCORRECT INDEX: ',A)
12    FORMAT(T2,'LAST ITEM IN INPUT LINE IS OBSOLETE. IGNORED.')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE GGIV2 ( NIND, INDE, VALUE, NPAIRS, IERR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      INTEGER INDE(*)
      DIMENSION VALUE(*)
C
      PARAMETER ( MXV = 200 )
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
C
      PARAMETER ( MEM = 2500, LSTA = 500 )
      COMMON/MCLVAR/STACK(LSTA),CMEM(MEM),ISTACK,INFILE
C
      IERR = 0
      NPAIRS = NINT(STACK(ISTACK))
      ISTACK = ISTACK-1
      DO 20,I=NPAIRS,1,-1
         VALUE(I) = STACK(ISTACK)
         ISTACK=ISTACK-1
         IND = NINT(STACK(ISTACK))
         IF (IND.LT.1 .OR. IND.GT.NIND) THEN
            IF (STRO) WRITE (IUOUT,12) IND
            INDE(I) = 0
            IERR = 3
         ELSE
            INDE(I) = IND
         END IF
         ISTACK = ISTACK-1
20    CONTINUE
C
12    FORMAT(T2,'INCORRECT INDEX: ',I4)
      END
C
C
      SUBROUTINE GRA1(NOC,ILIN,ICOL,IWVAR,XMIN,XMAX,XAX,YAX)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   GRA1...PERFORMS THE NECESSARY PREPARATIONS FOR THE GRAF
C    NOC: NUMBER OF POINTS TO BE PLOTTED
C    ILIN: NUMBER OF LINES TO APPEAR IN THE PLOT
C    ICOL: NUMBER OF COLUMNS TO APPEAR IN THE PLOT
C    IWVAR: VARIABLES TO BE PLOTTED
C    XMIN: LOW BOUND ON THE X AXIS
C    XMAX: UPPER BOUND ON THE X AXIS
C    XAX : ZERO POSITION ON X AXIS (DEFAULT 0)
C    YAX : ZERO POSITION ON Y AXIS ( DEFAULT 0)
C
      PARAMETER ( MXV = 200 )
      PARAMETER (ICMAX = 124)
      LOGICAL STRO,WEAK
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 NICKNA
      COMMON/NAMES/NICKNA(MXV)
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/PRILOG/STRO,WEAK
      COMMON/FV1COM/XVAR(MXV),N,IVAR
      CHARACTER*50 MSX,MSY
      DIMENSION X(124),Y(124)
      IF(IWVAR.LT.1.OR.IWVAR.GT.NOD)RETURN
C  CHECK IF VARIABLE IS FIXED
C
      IF(IXAT(IWVAR).EQ.0)RETURN
C
C CHECK XMIN XMAX (IF OUT OF MARGINS)
C
       IF(XMAX.LT.XMIN)THEN
        XX=XMAX
        XMAX=XMIN
        XMIN=XX
       ENDIF
      MI=MARG(IWVAR)
       IF(MI.NE.0)THEN
        IF((MI.EQ.-1.OR.MI.EQ.2).AND.XMIN.LT.XLL(IWVAR))XMIN=XLL(IWVAR)
        IF((MI.EQ.1.OR.MI.EQ.2).AND.XMAX.GT.XRL(IWVAR))XMAX=XRL(IWVAR)
       ENDIF
C   CHECK ILIN-ICOL
      IF(ICOL.LT.2)ICOL=2
      IF(ICOL.GT.ICMAX)ICOL=ICMAX
      IF(ILIN.LT.2)ILIN=2
      IF(NOC.LT.2)NOC=2
      IF(NOC.GT.ICOL)NOC=ICOL
       DO 1 I=1,NOD
    1  XVAR(I)=POINT(I)
      IVAR=IWVAR
      N=NOD
      XSTEP=(XMAX-XMIN)/REAL(NOC-1)
C  PREPARE X - Y
       DO 2 I=1,NOC
        XX=XMIN+REAL(I-1)*XSTEP
        X(I)=XX
        Y(I)=FVAR1(XX)
    2  CONTINUE
C  ADD CURRENT POINT (CPP) IF   XMIN =< CPP =< XMAX
      CPP=POINT(IWVAR)
      NCC=NOC
       IF(CPP.GE.XMIN.AND.CPP.LE.XMAX)THEN
        NCC=NOC+1
        Y(NCC)=VAL
        X(NCC)=CPP
       ENDIF
C
C            FIND YMIN
C
      IMIN=1
       DO 3 I=2,NCC
        IF(Y(I).LT.Y(IMIN))IMIN=I
    3  CONTINUE
C
C    IF A BETTER POINT HAS BEEN FOUND UPDATE POINT-VALUE
C
       IF(Y(IMIN).LT.VAL)THEN
        POINT(IWVAR)=X(IMIN)
        VAL=Y(IMIN)
       ENDIF
C   MARK CURRENT POINT
      X0=POINT(IWVAR)
      Y0=VAL
C   PREPARE MSX, MSY
      WRITE(MSX,4)IWVAR,NICKNA(IWVAR)
      MSY='  FUNCTION  VALUE'
      IT=IUOUT
       IF(STRO)THEN
        CALL GRAF(X,Y,NCC,ILIN,ICOL,XAX,YAX,X0,Y0,IT,MSX,MSY,MIS)
       ENDIF
      RETURN
    4 FORMAT('    VARIABLE : ',I3,')  ',A10)
      END
C
C
      SUBROUTINE GRAF(X,Y,NOV,ILIN,ICOL,XAX,YAX,X0,Y0,ITAPE,MSX,MSY,MIS)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NMAX=155)
C
C  ----- INPUT
C        PLOTS Y VERSUS X IN UNIT ITAPE
C       THE DIMENSIONS OF THE GRAF ARE (ILIN X ICOL)
C       NOV = NUMBER OF DATA < NMAX
C   XAX,YAX ARE THE ORIGIN'S COORDINATES
C   X0,Y0   ARE THE COORDINATES OF A POINT TO BE MARKED
C  MSX - MSY  ARE STRINGS *50 CONTAINING MESSAGES TO BE DISPLAYED
C
C  -----  OUTPUT
C    MIS : IF MIS=1 INPUT ARGUMENTS ARE WRONG AND NO GRAPH TAKES PLACE
C---SUBROUTINES CALLED:
C                       GRCHE
C
C
      DIMENSION X(NOV),Y(NOV),IYY(NMAX),XAA(15)
      CHARACTER*124,GRA,BLANK,MSX*50,MSY*50
      CHARACTER*1,PL,ST,DOT,EX,MPR,B
      DATA PL,ST,DOT,EX,B/'+','*','.','X',' '/
C
C     CHECK INPUT ARGUMENTS
      CALL GRCHE(NOV,ILIN,ICOL,ITAPE,NMAX,MIS)
      IF(MIS.EQ.1)RETURN
C    PREPARE BLANK
       DO 1 I=1,124
    1  BLANK(I:I)=B
      GRA=BLANK
C FIND XMIN-XMAX, YMIN-YMAX
      XMIN=X(1)
      XMAX=X(1)
      YMIN=Y(1)
      YMAX=Y(1)
       DO 2 I=2,NOV
        IF(X(I).LT.XMIN)XMIN=X(I)
        IF(X(I).GT.XMAX)XMAX=X(I)
        IF(Y(I).LT.YMIN)YMIN=Y(I)
        IF(Y(I).GT.YMAX)YMAX=Y(I)
    2  CONTINUE
C
C   FIND X-STEP, Y-STEP
C
      STEPX=(XMAX-XMIN)/REAL(ICOL-1)
      STEPY=(YMAX-YMIN)/REAL(ILIN-1)
      IF(STEPX.EQ.0.)STEPX=1.
      IF(STEPY.EQ.0.)STEPY=1.
C
C  PREPARE Y DIRECTIVES
C
       DO 3 I=1,NOV
        IYY(I)=-INT((Y(I)-YMIN)/STEPY+.5)+ILIN
    3  CONTINUE
C
C PREPARE XAX-YAX DIRECTIVES
C
      IXAX=0
      IYAX=0
      IF(XAX.GE.XMIN.AND.XAX.LE.XMAX)IXAX=INT((XAX-XMIN)/STEPX+.5)+2
      IF(YAX.GE.YMIN.AND.YAX.LE.YMAX)IYAX=ILIN-INT((YAX-YMIN)/STEPY+.5)
C
C   PREPARE    X0 - Y0  DIRECTIVES
      IX0=0
      IY0=0
      IF(X0.GE.XMIN.AND.X0.LE.XMAX)IX0=INT((X0-XMIN)/STEPX+.5)+2
      IF(Y0.GE.YMIN.AND.Y0.LE.YMAX)IY0=ILIN-INT((Y0-YMIN)/STEPY+.5)
      IC2=ICOL+2
       DO 4 K=1,IC2
    4  GRA(K:K)=DOT
       DO 5 K=2,IC2,5
    5  GRA(K:K)=PL
      WRITE(ITAPE,17)BLANK
      WRITE(ITAPE,17)GRA(1:IC2)
       DO 8 I=1,ILIN
         IF(I.LE.50)THEN
          MPR=MSY(I:I)
         ELSE
          MSY=B
         ENDIF
         IF(I.NE.IYAX)THEN
          GRA=BLANK
         ELSE
           DO 6 K=2,ICOL+1
    6      GRA(K:K)=DOT
         ENDIF
        IF(IXAX.NE.0)GRA(IXAX:IXAX)=DOT
        LY=((I-1)/2)*2-I+1
         IF(LY.NE.0)THEN
          GRA(1:1)=DOT
          GRA(IC2:IC2)=DOT
         ELSE
          GRA(1:1)=PL
          GRA(IC2:IC2)=PL
         ENDIF
C   CHECK IF THERE IS A POINT AT THIS LINE
C
         DO 7 J=1,NOV
           IF(I.EQ.IYY(J))THEN
            IX=INT((X(J)-XMIN)/STEPX+.5)+2
            GRA(IX:IX)=ST
           ENDIF
    7    CONTINUE
        IF(I.EQ.IY0)GRA(IX0:IX0)=EX
         IF(LY.NE.0)THEN
          WRITE(ITAPE,13)MPR,GRA(1:IC2)
         ELSE
          YY=YMIN-REAL(I-ILIN)*STEPY
          WRITE(ITAPE,14)MPR,YY,GRA(1:IC2)
         ENDIF
    8  CONTINUE
       DO 9 K=1,IC2
    9  GRA(K:K)=DOT
       DO 10 K=2,IC2,5
   10  GRA(K:K)=PL
      WRITE(ITAPE,17)GRA(1:IC2)
      NX=0
       DO 11 I=2,IC2,10
        NX=NX+1
        XAA(NX)=XMIN+REAL(I-2)*STEPX
   11  CONTINUE
      WRITE(BLANK,15)(XAA(I),I=1,NX,2)
      ICN=IC2+13
       IF(ICN.GT.74.AND.ICN.LT.80)THEN
         DO 12 I=60,72
   12    BLANK(I:I)=B
        WRITE(ITAPE,18)BLANK(1:71)
       ELSE
        WRITE(ITAPE,18)BLANK(1:IC2)
       ENDIF
       IF(NX.GE.2)THEN
        WRITE(ITAPE,16)(XAA(I),I=2,NX,2)
       ENDIF
      WRITE(ITAPE,17)MSX(1:50)
      RETURN
   13 FORMAT(1X,A1,11X,A)
   14 FORMAT(1X,A1,1X,1PG10.3,A)
   15 FORMAT(6(1PE13.6,'   I   '))
   16 FORMAT(18X,5(1PE13.6,7X))
   17 FORMAT(13X,A)
   18 FORMAT(8X,A)
      END
C
C
      SUBROUTINE GRCHE(NOV,ILIN,ICOL,ITAPE,NMAX,MIS)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   GRCHE...CHECKS THE INPUT ARGUMENTS FOR THE GRAF
C   IF AN ERROR OCCURS,MIS=1, ELSE MIS=0
C   ALSO RESETS THE VALUES FOR ILIN,ICOL, IF NOT PROPRIATE
C
      MIS=0
      IF(NOV.LE.1.OR.NOV.GT.NMAX.OR.ITAPE.LT.0)MIS=1
      IF(ILIN.LT.2)ILIN=2
      IF(ICOL.LT.2)ICOL=2
      IF(ICOL.GT.122)ICOL=122
      RETURN
      END
C
C
      SUBROUTINE HELP(IHELP)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   HELP ... IMPLEMENTS BOTH THE * HELP * AND THE * INTRODUCE *
C   COMMANDS.
C   COMMAN  INPUT CHARACTER*10 ARRAY CONTAINING THE COMMAND NAMES.
C   IHELP = 1,2  FOR COMMANDS  HELP,INTRODUCE
C
      PARAMETER ( NCOM = 100 )
      CHARACTER*10 COMMAN(NCOM)
      COMMON /COM1/ COMMAN
C
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      CHARACTER COM*10,LINE*79,SCR*1, UL*15, UL2*25
C
      IF(PROCES.EQ.'BATCH')RETURN
      REWIND IU42
C   CHECK IF UNIT IU42 EXISTS
      READ(IU42,'(A1)',END=16)SCR
C   THE FOLLOWING IF IMPLEMENTS THE INTRODUCE COMMAND
       IF(IHELP.EQ.2) THEN
        CALL DASH
        IEXT=0
         DO 1 I=1,100000
          READ(IU42,23,END=2)LINE
          UL = LINE(1:15)
          CALL UPCASE(UL)
           IF(UL.EQ.'***EXTENSION***')THEN
            IEXT=IEXT+1
            WRITE(IUOUT,17)IEXT,LINE(16:25)
           ENDIF
    1    CONTINUE
    2   REWIND IU42
        IF(IEXT.EQ.0)WRITE(IUOUT,*)
     > '... NO EXTENSIONS / STANDARD PROGRAMABLE VERSION'
        CALL DASH
        RETURN
       ENDIF
C
C  IMPLEMENTATION OF THE "HELP" COMMAND STARTS HERE.
       DO 3 I=1,21
    3  WRITE(IUOUT,19)
      CALL DASH
      WRITE(IUOUT,19)
      WRITE(IUOUT,20)
      WRITE(IUOUT,19)
      CALL DASH
      REWIND IU42
       DO 4 I=1,1000
        READ(IU42,23)LINE
        IF(LINE(1:3).EQ.'***') GO TO  5
    4   WRITE(IUOUT,'(T2,A)')LINE
    5 CALL DASH
      KTEXT=I-1
    6 WRITE(IUOUT,19)
      WRITE(IUOUT,21)
      READ(IUINP,22,END=15)COM
      CALL UPCASE(COM)
       IF(COM.EQ.'EXIT')THEN
        CALL DASH
        WRITE(IUOUT,19)
        WRITE(IUOUT,24)
        WRITE(IUOUT,19)
        CALL DASH
        RETURN
       ELSEIF(COM.EQ.'ALL')THEN
        REWIND IU42
         DO 7 I=1,KTEXT
    7    READ(IU42,'(A1)')SCR
         DO 8 I=1,100000
          READ(IU42,23,END=6)LINE
           UL = LINE(1:12)
           CALL UPCASE(UL)
           IF(UL(:11).EQ.'***STANDARD')THEN
            WRITE(IUOUT,19)
            WRITE(IUOUT,25)LINE(15:24),LINE(4:11)
           ELSEIF(UL(:12).EQ.'***EXTENSION')THEN
            WRITE(IUOUT,19)
            WRITE(IUOUT,25)LINE(16:25),LINE(4:12)
           ELSE
            WRITE(IUOUT,'(T2,A)')LINE
           ENDIF
    8    CONTINUE
       ELSE
         DO 9 I=1,NCOM
          IF(COM.EQ.COMMAN(I).OR.COM.EQ.'AUTO')GO TO  10
    9    CONTINUE
        CALL DASH
        WRITE(IUOUT,19)
        WRITE(IUOUT,*)'THERE IS NO SUCH COMMAND'
        WRITE(IUOUT,19)
        CALL DASH
        GO TO  6
       ENDIF
   10 CONTINUE
      REWIND IU42
       DO 12 I=1,100000
        READ(IU42,23,END=14)LINE
        IF(LINE(:3).NE.'***')GO TO  12
        UL2 = LINE(1:25)
        CALL UPCASE(UL2)
        IF(UL2(4:11).EQ.'STANDARD')IND=15
        IF(UL2(4:12).EQ.'EXTENSION')IND=16
        INDF=IND+9
        INDM=IND-4
         IF(UL2(IND:INDF).EQ.COM) THEN
          WRITE(IUOUT,25)COM,LINE(4:INDM)
           DO 11 J=1,1000
            READ(IU42,23,END=13)LINE
            IF(LINE(1:3).EQ.'***') GO TO  6
            WRITE(IUOUT,'(T2,A)')LINE
   11      CONTINUE
         ENDIF
   12  CONTINUE
   13 REWIND IU42
      GO TO  6
   14 CONTINUE
      CALL DASH
      WRITE(IUOUT,19)
      WRITE(IUOUT,*)'THERE IS NO DESCRIPTION FOR THIS COMMAND'
      WRITE(IUOUT,19)
      CALL DASH
      GO TO  6
   15 CONTINUE
      REWIND IUINP
      WRITE(IUOUT,*)'TO TERMINATE THE HELP SESSION ENTER:   EXIT'
      GO TO  6
   16 WRITE(IUOUT,*)' ***** SORRY THERE IS NO HELP FILE *****'
      CALL DASH
      IF(IHELP.NE.2)WRITE(IUOUT,24)
      IF(IHELP.NE.2) CALL DASH
      RETURN
   17 FORMAT(T4,'EXTENSION NO. ',I2,')  ...  ',A10)
   19 FORMAT(/)
   20 FORMAT(2X,T34,'START HELP')
   21 FORMAT(2X,T40,'ENTER COMMAND')
   22 FORMAT(A10)
   23 FORMAT(A79)
   24 FORMAT(2X,T34,'END HELP')
   25 FORMAT(T4,'-- ',A10,' --',T30,'(',A,')')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE INIT
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( MXV = 200 )
      PARAMETER ( MAXNL = 20 )
      INTEGER IBACK(MAXNL)
      CHARACTER*10 CBACK(MAXNL)
      COMMON / NEST1 / IBACK, LEVEL, IEXIT, IDISK
      COMMON / NEST2 / CBACK
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
      CHARACTER*5 GRADI
      COMMON/DERIMO/GRADI(MXV)
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      CHARACTER*10 NICKNA
      COMMON/NAMES/NICKNA(MXV)
      COMMON/INDEFI/INDEF(MXV)
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      COMMON/ERRBOU/GLERB
      CHARACTER*5 UMODE
      COMMON/UPMODE/UMODE
C
      PARAMETER ( MAXALI = 20 )
      CHARACTER*5 ALIAS(MAXALI)
      CHARACTER*10 ALICOM(MAXALI)
      COMMON / ALICO1 / ALIAS, ALICOM
      COMMON / ALICO2 / NALIAS
      PARAMETER ( MAXFLA = 20 )
      COMMON / FLAGS / FLAG(MAXFLA)
      COMMON /MODE3/ MODEV, IFFORM 
      PARAMETER ( MAXCFL = 10, MAXCLE = 30 )
      CHARACTER*(MAXCLE) CFLAG(MAXCFL)
      COMMON / CFLAGS / CFLAG
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/LEVEX/GTOLL,FTOLL,XTOLL,FACTL,EPSFL,NOCL,IPRIL
      COMMON/UPDATE/IGRADU,IHESSU
      COMMON/FIRST/ICATA,IDAT
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      CHARACTER*10 CSET, DATAMO, DISMOD
      COMMON /MODE2/ CSET, DATAMO, DISMOD
      LOGICAL AUTODE
      COMMON/AUTOMA/AUTODE
      CHARACTER*10 FILSTA
      COMMON/FSTAT/FILSTA(6)
C
      LEVEL = 0
      IEXIT = 1
      IDISK = 1
C
      ITNOCS = 0
      IPARTC = 0
      ITDER = 0
      IPDER = 0
C
      DO 10,I=1,MXV
        XLL(I) = -BLIM
        XRL(I) = BLIM
        GRADI(I) = 'NUMER'
        NICKNA(I) = ' '
        INDEF(I) = 0
        IXAT(I) = 1
        MARG(I) = 0
        STEP(I) = 1.
 10   CONTINUE
C
      GLERB = 0.0001
      UMODE = ' '
C
      NALIAS = 0
C
      DO 20,I=1,MAXFLA
20       FLAG(I) = 0.
C
      MODEV = 1
      IFFORM = 0
C
      DO 30,I=1,MAXCFL
30       CFLAG(I) = ' '
C
      EPSFL = COMACC
C
      IGRADU = 0
      IHESSU = 0
C
      ICATA = 0
      IDAT = 0
C
      STRO = .TRUE.
      WEAK = .TRUE.
C
      IU31 = 5
      IU32 = 6
      IU36 = 36
      IU37 = 37
      IU42 = 42
      IU45 = 45
      IU47 = 47
C
      BACKUP = 'NOBACK    '
      DERIVA = 'NUMER     '
      PRINTO = 'FULLPRINT '
      CALLBY = 'INDEX     '
      PROCES = 'IAF       '
      PANEL  = 'PANELON   '
      DATAMO = 'DATASKIP'
      DISMOD = 'DISPOSKIP'
C
      AUTODE = .FALSE.
C
      DO 40,I=1,6
40       FILSTA(I) = 'EMPTY'
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE CPINIT
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      LOGICAL HEADER, PRECON
      CHARACTER*15 IN, OUT
      CHARACTER*80 PROLO, EPILO
      COMMON /CP1/ BI, BL, SM, NOVA, NOTE, MODPRO, MUIN, MUOUT,
     >             HEADER,  PRECON 
      COMMON /CP2/ PROLO, EPILO, IN, OUT
      COMMON /UALLOC/ MINU, MAXU, LASTU
      CHARACTER*10 CSET, DATAMO, DISMOD
      COMMON /MODE2/ CSET, DATAMO, DISMOD
C
      NOVA = -1
      NOTE = -1
      HEADER = .TRUE.
      MODPRO = 0
      MUIN = -1
      MUOUT = -1
      PRECON = .TRUE.
      BI = -1.0
      BL = -1.0
      PROLO = ' '
      EPILO = ' '
      IN = ' '
      OUT = ' '
      SM = -1.0
C
      MINU = 50
      MAXU = 80
      LASTU = MINU-1
C
      CSET = 'ASCII'
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE ZINIT
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      LOGICAL AIMON
      COMMON/AIM/TARGET,AIMON
C
      DWALOG = LOG(DWARF)
      TARGET  = -BIGNUM
      AIMON = .FALSE.
C
      END
      SUBROUTINE INLINE ( LINE, LCRACK, TOUPP, IS, IE, NIT, IERR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  READS A LINE FROM THE INPUT FILE.
C  LCRACK = .TRUE. IF LINE CRACKING (LCR) IS NEEDED.
C  TOUPP = .TRUE. IF CONVERSION TO UPPER CASE IS NEEDED.
C  IERR IS OUTPUT: IERR = 0     NO PROBLEM.
C                  IERR = 1     BLANK LINE OR '0' WAS GIVEN. (IN CASE
C                                OF LCRACK=.FALSE.; OTHERWISE THIS SITUATION
C                                IS SIGNIFIED BY NIT=0)
C                  IERR = 2     EOF ENCOUNTERED.
C
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      CHARACTER*(*) LINE
      LOGICAL LCRACK, TOUPP
      INTEGER IS(*), IE(*)
C
      IERR = 0
      READ (IUINP,'(A)',END=100) LINE
      IF (TOUPP) CALL UPCASE(LINE)
      IF (LCRACK) THEN
         CALL LCR(LINE,IS,IE,NIT)
         IF (NIT.EQ.1) THEN
            IF (LINE(IS(1):IE(1)).EQ.'0') NIT = 0
         END IF
      ELSE
         IF (LINE.EQ.' ' .OR. LINE.EQ.'0') IERR = 1
      END IF
      RETURN
C
100   CONTINUE
      CALL BATCH
      REWIND IUINP
      IERR = 2
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE INNOCR ( LINE, TOUPP, IERR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      CHARACTER*(*) LINE
      LOGICAL TOUPP
C
      INTEGER IS(1), IE(1)
C
      CALL INLINE(LINE,.FALSE.,TOUPP,IS,IE,NIT,IERR)
C
      END
      SUBROUTINE INSPEC(IFILE,NOV)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     FACILITATES FILE INSPECTION. THE FILE IS SPECIFIED BY
C     THE  IFILE  INPUT INTEGER VARIABLE
C
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      COMMON/FIRST/ICATA,IDAT
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*79 SCRA
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      REWIND IFILE
    1 CONTINUE
      REWIND IU45
       DO 2 I=1,NOV
        READ(IFILE,6,END=3)SCRA
        WRITE(IU45,6)SCRA
        IF(STRO)WRITE(IUOUT,6)SCRA
    2  CONTINUE
       IF(IFILE.NE.IUDAT.OR.IDAT.EQ.1)THEN
        READ(IFILE,6,END=3)SCRA
        WRITE(IU45,6)SCRA
        IF(STRO)WRITE(IUOUT,6)SCRA
       ENDIF
      IF(IFILE.EQ.IUDAT)RETURN
      IF(STRO)WRITE(IUOUT,*)'ENTER SPACE OR $ TO CONTINUE'
      READ(IUINP,6,END=4)SCRA
      IF(SCRA(1:2).EQ.'CR')GO TO  5
      IF(SCRA(1:1).EQ.' '.OR.SCRA(1:1).EQ.'$')GO TO  1
      RETURN
    3 REWIND IFILE
      RETURN
    4 REWIND IUINP
C   IF IN BATCH OPTION OF THE PROCES MODE ABORT ON ERRORS
      CALL BATCH
    5 CONTINUE
      RETURN
    6 FORMAT(A79)
      END
C
C  ---------------------------------------------------------------------
C
      FUNCTION IPARIN(SCRA)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  SCRA IS SUPPOSED TO CONTAIN EITHER A VARIABLE'S NAME OR INDEX
C  IF SO IT IS IDENTIFIED AND RETURNS ITS INDEX.
C  IF NOT RETURNS A ZERO VALUE.
C
      CHARACTER*(*) SCRA
C
      PARAMETER ( MXV = 200 )
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      CHARACTER*10 NICKNA
      COMMON/NAMES/NICKNA(MXV)
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      IPARIN=0
      IF (CALLBY.EQ.'NAME') THEN
         DO 1 I=1,NOD
           IF(SCRA.EQ.NICKNA(I))THEN
              IPARIN=I
              RETURN
           ENDIF
 1       CONTINUE
      ELSE
         LE = LENGTH(SCRA)
         CALL TOINT(SCRA,LE,II,IJI)
         IF (IJI.NE.0) THEN
            IF(STRO)WRITE(IUOUT,3)
         ELSE IF (II.GT.NOD .OR. II.LE.0) THEN
            IPARIN = 0
         ELSE
            IPARIN=II
         ENDIF
      ENDIF
C
 3    FORMAT(T2,'** CAUTION: OPERATION CALLBY MODE IS INDEX **')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE LCR(LINE,IS,IE,NIT)
      CHARACTER*(*) LINE
      INTEGER IS(*), IE(*)
      LOGICAL INWORD
      CHARACTER C
C
      L = LEN(LINE)
      NIT = 0
      INWORD = LINE(1:1).NE.' '
      IF (INWORD) THEN
         NIT = 1
         IS(1) = 1
      END IF
C
      DO 10,I=2,L
        C = LINE(I:I)
        IF (C.EQ.' ') THEN
           IF (INWORD) THEN
            INWORD = .FALSE.
              IE(NIT) = I-1
           END IF
        ELSE
           IF (.NOT.INWORD) THEN
              INWORD = .TRUE.
              NIT = NIT+1
              IS(NIT) = I
           END IF
        END IF
 10   CONTINUE
C
      IF (INWORD) IE(NIT) = L
      RETURN
      END
C
C  ---------------------------------------------------------------------
C
      FUNCTION LENGTH(STRING)
      CHARACTER STRING*(*)
      DO 71,LENGTH=LEN(STRING),1,-1
      IF(STRING(LENGTH:LENGTH).NE.' ') RETURN
 71   CONTINUE
      RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE LEVE ( W, MAXNOC, IPRINT, GTOL, FTOL, XTOL, FACTOR,
     >                  EPSFCN, NFEV )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION W(*)
      PARAMETER ( MXV = 200 )
      PARAMETER ( MXT = 200 )
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/PARAMX/NOF
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
C
      DIMENSION FVEC(MXT), DIAG(MXV), QTF(MXV), ZZ(MXV)
      DIMENSION WA1(MXV), WA2(MXV), WA3(MXV), WA4(MXT)
      INTEGER IPVT(MXV)
      EXTERNAL FCN
C
C  LMDIF OPERATES ON A SET OF TRANSFORMED VARIABLES THAT TAKE CARE 
C  OF THE MARGINS AND FIX STATUS.
C
      DO 1,I=1,NOD
         ZZ(I) = POINT(I)
1     CONTINUE
      CALL XTOY(ZZ,NOD)
C
      EPSIL = 0.
      CALL INITER(IPRINT,MAXNOC,EPSIL)
      MODE = 1
      LDFJAC = NOF
      CALL LMDIF(FCN,NOF,NOD,ZZ,FVEC,FTOL,XTOL,GTOL,MAXNOC,EPSFCN,
     >           DIAG,MODE,FACTOR,IPRINT,INFO,NFEV,W,LDFJAC,
     >           IPVT,QTF,WA1,WA2,WA3,WA4)
C
      CALL YTOX(ZZ,NOD)
      DO 2,I=1,NOD
         POINT(I) = ZZ(I) 
2     CONTINUE
C
      IF (STRO) WRITE (IUOUT,10) INFO
      VAL = ACSQ(POINT,NOD)
10    FORMAT(3X,'EXIT PARAMETER',I10)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE FCN ( M, N, X, FVEC, IFLAG )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(N), FVEC(M)
      PARAMETER ( MXV = 200 )
      DIMENSION Y(MXV)
C
C  A ZERO FLAG INDICATES A REQUEST FOR PRINTOUTS; IGNORE.
      IF (IFLAG.EQ.0) RETURN
C
      DO 10,I=1,N
10       Y(I) = X(I)
      CALL YTOX(Y,N)
      CALL LSQFCN(M,N,Y,FVEC)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE LMDIF(FCN,M,N,X,FVEC,FTOL,XTOL,GTOL,MAXFEV,EPSFCN,
     *                 DIAG,MODE,FACTOR,NPRINT,INFO,NFEV,FJAC,LDFJAC,
     *                 IPVT,QTF,WA1,WA2,WA3,WA4)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER M,N,MAXFEV,MODE,NPRINT,INFO,NFEV,LDFJAC
      INTEGER IPVT(N)
      DIMENSION X(N),FVEC(M),DIAG(N),FJAC(LDFJAC,N),QTF(N),WA1(N),
     *     WA2(N),WA3(N),WA4(M)
      EXTERNAL FCN
C     **********
C
C     SUBROUTINE LMDIF
C
C     THE PURPOSE OF LMDIF IS TO MINIMIZE THE SUM OF THE SQUARES OF
C     M NONLINEAR FUNCTIONS IN N VARIABLES BY A MODIFICATION OF
C     THE LEVENBERG-MARQUARDT ALGORITHM. THE USER MUST PROVIDE A
C     SUBROUTINE WHICH CALCULATES THE FUNCTIONS. THE JACOBIAN IS
C     THEN CALCULATED BY A FORWARD-DIFFERENCE APPROXIMATION.
C
C     THE SUBROUTINE STATEMENT IS
C
C       SUBROUTINE LMDIF(FCN,M,N,X,FVEC,FTOL,XTOL,GTOL,MAXFEV,EPSFCN,
C                        DIAG,MODE,FACTOR,NPRINT,INFO,NFEV,FJAC,
C                        LDFJAC,IPVT,QTF,WA1,WA2,WA3,WA4)
C
C     WHERE
C
C       FCN IS THE NAME OF THE USER-SUPPLIED SUBROUTINE WHICH
C         CALCULATES THE FUNCTIONS. FCN MUST BE DECLARED
C         IN AN EXTERNAL STATEMENT IN THE USER CALLING
C         PROGRAM, AND SHOULD BE WRITTEN AS FOLLOWS.
C
C         SUBROUTINE FCN(M,N,X,FVEC,IFLAG)
C         INTEGER M,N,IFLAG
C         REAL X(N),FVEC(M)
C         ----------
C         CALCULATE THE FUNCTIONS AT X AND
C         RETURN THIS VECTOR IN FVEC.
C         ----------
C         RETURN
C         END
C
C         THE VALUE OF IFLAG SHOULD NOT BE CHANGED BY FCN UNLESS
C         THE USER WANTS TO TERMINATE EXECUTION OF LMDIF.
C         IN THIS CASE SET IFLAG TO A NEGATIVE INTEGER.
C
C       M IS A POSITIVE INTEGER INPUT VARIABLE SET TO THE NUMBER
C         OF FUNCTIONS.
C
C       N IS A POSITIVE INTEGER INPUT VARIABLE SET TO THE NUMBER
C         OF VARIABLES. N MUST NOT EXCEED M.
C
C       X IS AN ARRAY OF LENGTH N. ON INPUT X MUST CONTAIN
C         AN INITIAL ESTIMATE OF THE SOLUTION VECTOR. ON OUTPUT X
C         CONTAINS THE FINAL ESTIMATE OF THE SOLUTION VECTOR.
C
C       FVEC IS AN OUTPUT ARRAY OF LENGTH M WHICH CONTAINS
C         THE FUNCTIONS EVALUATED AT THE OUTPUT X.
C
C       FTOL IS A NONNEGATIVE INPUT VARIABLE. TERMINATION
C         OCCURS WHEN BOTH THE ACTUAL AND PREDICTED RELATIVE
C         REDUCTIONS IN THE SUM OF SQUARES ARE AT MOST FTOL.
C         THEREFORE, FTOL MEASURES THE RELATIVE ERROR DESIRED
C         IN THE SUM OF SQUARES.
C
C       XTOL IS A NONNEGATIVE INPUT VARIABLE. TERMINATION
C         OCCURS WHEN THE RELATIVE ERROR BETWEEN TWO CONSECUTIVE
C         ITERATES IS AT MOST XTOL. THEREFORE, XTOL MEASURES THE
C         RELATIVE ERROR DESIRED IN THE APPROXIMATE SOLUTION.
C
C       GTOL IS A NONNEGATIVE INPUT VARIABLE. TERMINATION
C         OCCURS WHEN THE COSINE OF THE ANGLE BETWEEN FVEC AND
C         ANY COLUMN OF THE JACOBIAN IS AT MOST GTOL IN ABSOLUTE
C         VALUE. THEREFORE, GTOL MEASURES THE ORTHOGONALITY
C         DESIRED BETWEEN THE FUNCTION VECTOR AND THE COLUMNS
C         OF THE JACOBIAN.
C
C       MAXFEV IS A POSITIVE INTEGER INPUT VARIABLE. TERMINATION
C         OCCURS WHEN THE NUMBER OF CALLS TO FCN IS AT LEAST
C         MAXFEV BY THE END OF AN ITERATION.
C
C       EPSFCN IS AN INPUT VARIABLE USED IN DETERMINING A SUITABLE
C         STEP LENGTH FOR THE FORWARD-DIFFERENCE APPROXIMATION. THIS
C         APPROXIMATION ASSUMES THAT THE RELATIVE ERRORS IN THE
C         FUNCTIONS ARE OF THE ORDER OF EPSFCN. IF EPSFCN IS LESS
C         THAN THE MACHINE PRECISION, IT IS ASSUMED THAT THE RELATIVE
C         ERRORS IN THE FUNCTIONS ARE OF THE ORDER OF THE MACHINE
C         PRECISION.
C
C       DIAG IS AN ARRAY OF LENGTH N. IF MODE = 1 (SEE
C         BELOW), DIAG IS INTERNALLY SET. IF MODE = 2, DIAG
C         MUST CONTAIN POSITIVE ENTRIES THAT SERVE AS
C         MULTIPLICATIVE SCALE FACTORS FOR THE VARIABLES.
C
C       MODE IS AN INTEGER INPUT VARIABLE. IF MODE = 1, THE
C         VARIABLES WILL BE SCALED INTERNALLY. IF MODE = 2,
C         THE SCALING IS SPECIFIED BY THE INPUT DIAG. OTHER
C         VALUES OF MODE ARE EQUIVALENT TO MODE = 1.
C
C       FACTOR IS A POSITIVE INPUT VARIABLE USED IN DETERMINING THE
C         INITIAL STEP BOUND. THIS BOUND IS SET TO THE PRODUCT OF
C         FACTOR AND THE EUCLIDEAN NORM OF DIAG*X IF NONZERO, OR ELSE
C         TO FACTOR ITSELF. IN MOST CASES FACTOR SHOULD LIE IN THE
C         INTERVAL (.1,100.). 100. IS A GENERALLY RECOMMENDED VALUE.
C
C       NPRINT IS AN INTEGER INPUT VARIABLE THAT ENABLES CONTROLLED
C         PRINTING OF ITERATES IF IT IS POSITIVE. IN THIS CASE,
C         FCN IS CALLED WITH IFLAG = 0 AT THE BEGINNING OF THE FIRST
C         ITERATION AND EVERY NPRINT ITERATIONS THEREAFTER AND
C         IMMEDIATELY PRIOR TO RETURN, WITH X AND FVEC AVAILABLE
C         FOR PRINTING. IF NPRINT IS NOT POSITIVE, NO SPECIAL CALLS
C         OF FCN WITH IFLAG = 0 ARE MADE.
C
C       INFO IS AN INTEGER OUTPUT VARIABLE. IF THE USER HAS
C         TERMINATED EXECUTION, INFO IS SET TO THE (NEGATIVE)
C         VALUE OF IFLAG. SEE DESCRIPTION OF FCN. OTHERWISE,
C         INFO IS SET AS FOLLOWS.
C
C         INFO = 0  IMPROPER INPUT PARAMETERS.
C
C         INFO = 1  BOTH ACTUAL AND PREDICTED RELATIVE REDUCTIONS
C                   IN THE SUM OF SQUARES ARE AT MOST FTOL.
C
C         INFO = 2  RELATIVE ERROR BETWEEN TWO CONSECUTIVE ITERATES
C                   IS AT MOST XTOL.
C
C         INFO = 3  CONDITIONS FOR INFO = 1 AND INFO = 2 BOTH HOLD.
C
C         INFO = 4  THE COSINE OF THE ANGLE BETWEEN FVEC AND ANY
C                   COLUMN OF THE JACOBIAN IS AT MOST GTOL IN
C                   ABSOLUTE VALUE.
C
C         INFO = 5  NUMBER OF CALLS TO FCN HAS REACHED OR
C                   EXCEEDED MAXFEV.
C
C         INFO = 6  FTOL IS TOO SMALL. NO FURTHER REDUCTION IN
C                   THE SUM OF SQUARES IS POSSIBLE.
C
C         INFO = 7  XTOL IS TOO SMALL. NO FURTHER IMPROVEMENT IN
C                   THE APPROXIMATE SOLUTION X IS POSSIBLE.
C
C         INFO = 8  GTOL IS TOO SMALL. FVEC IS ORTHOGONAL TO THE
C                   COLUMNS OF THE JACOBIAN TO MACHINE PRECISION.
C
C       NFEV IS AN INTEGER OUTPUT VARIABLE SET TO THE NUMBER OF
C         CALLS TO FCN.
C
C       FJAC IS AN OUTPUT M BY N ARRAY. THE UPPER N BY N SUBMATRIX
C         OF FJAC CONTAINS AN UPPER TRIANGULAR MATRIX R WITH
C         DIAGONAL ELEMENTS OF NONINCREASING MAGNITUDE SUCH THAT
C
C                T     T           T
C               P *(JAC *JAC)*P = R *R,
C
C         WHERE P IS A PERMUTATION MATRIX AND JAC IS THE FINAL
C         CALCULATED JACOBIAN. COLUMN J OF P IS COLUMN IPVT(J)
C         (SEE BELOW) OF THE IDENTITY MATRIX. THE LOWER TRAPEZOIDAL
C         PART OF FJAC CONTAINS INFORMATION GENERATED DURING
C         THE COMPUTATION OF R.
C
C       LDFJAC IS A POSITIVE INTEGER INPUT VARIABLE NOT LESS THAN M
C         WHICH SPECIFIES THE LEADING DIMENSION OF THE ARRAY FJAC.
C
C       IPVT IS AN INTEGER OUTPUT ARRAY OF LENGTH N. IPVT
C         DEFINES A PERMUTATION MATRIX P SUCH THAT JAC*P = Q*R,
C         WHERE JAC IS THE FINAL CALCULATED JACOBIAN, Q IS
C         ORTHOGONAL (NOT STORED), AND R IS UPPER TRIANGULAR
C         WITH DIAGONAL ELEMENTS OF NONINCREASING MAGNITUDE.
C         COLUMN J OF P IS COLUMN IPVT(J) OF THE IDENTITY MATRIX.
C
C       QTF IS AN OUTPUT ARRAY OF LENGTH N WHICH CONTAINS
C         THE FIRST N ELEMENTS OF THE VECTOR (Q TRANSPOSE)*FVEC.
C
C       WA1, WA2, AND WA3 ARE WORK ARRAYS OF LENGTH N.
C
C       WA4 IS A WORK ARRAY OF LENGTH M.
C
C     SUBPROGRAMS CALLED
C
C       USER-SUPPLIED ...... FCN
C
C       MINPACK-SUPPLIED ... SPMPAR,ENORM,FDJAC2,LMPAR,QRFAC
C
C       FORTRAN-SUPPLIED ... ABS,AMAX1,AMIN1,SQRT,MOD
C
C     ARGONNE NATIONAL LABORATORY. MINPACK PROJECT. MARCH 1980.
C     BURTON S. GARBOW, KENNETH E. HILLSTROM, JORGE J. MORE
C
C     **********
      INTEGER I,IFLAG,ITER,J,L
      DATA ONE,P1,P5,P25,P75,P0001,ZERO
     *     /1.0E0,1.0E-1,5.0E-1,2.5E-1,7.5E-1,1.0E-4,0.0E0/
C
C     EPSMCH IS THE MACHINE PRECISION.
C
      EPSMCH = SPMPAR(1)
C
      INFO = 0
      IFLAG = 0
      NFEV = 0
C
C     CHECK THE INPUT PARAMETERS FOR ERRORS.
C
      IF (N .LE. 0 .OR. M .LT. N .OR. LDFJAC .LT. M
     *    .OR. FTOL .LT. ZERO .OR. XTOL .LT. ZERO .OR. GTOL .LT. ZERO
     *    .OR. MAXFEV .LE. 0 .OR. FACTOR .LE. ZERO) GO TO 300
      IF (MODE .NE. 2) GO TO 20
      DO 10 J = 1, N
         IF (DIAG(J) .LE. ZERO) GO TO 300
   10    CONTINUE
   20 CONTINUE
C
C     EVALUATE THE FUNCTION AT THE STARTING POINT
C     AND CALCULATE ITS NORM.
C
      IFLAG = 1
      CALL FCN(M,N,X,FVEC,IFLAG)
      NFEV = 1
      IF (IFLAG .LT. 0) GO TO 300
      FNORM = ENORM(M,FVEC)
C
C     INITIALIZE LEVENBERG-MARQUARDT PARAMETER AND ITERATION COUNTER.
C
      PAR = ZERO
      ITER = 1
C
C     BEGINNING OF THE OUTER LOOP.
C
   30 CONTINUE
C
C        CALCULATE THE JACOBIAN MATRIX.
C
         IFLAG = 2
         CALL FDJAC2(FCN,M,N,X,FVEC,FJAC,LDFJAC,IFLAG,EPSFCN,WA4)
         NFEV = NFEV + N
         IF (IFLAG .LT. 0) GO TO 300
C
C        IF REQUESTED, CALL FCN TO ENABLE PRINTING OF ITERATES.
C
         IF (NPRINT .LE. 0) GO TO 40
         IFLAG = 0
CC         IF (MOD(ITER-1,NPRINT) .EQ. 0) CALL FCN(M,N,X,FVEC,IFLAG)
         CALL DISPL1(FNORM**2,X,N)
         
         IF (IFLAG .LT. 0) GO TO 300
   40    CONTINUE
C
C        COMPUTE THE QR FACTORIZATION OF THE JACOBIAN.
C
         CALL QRFAC(M,N,FJAC,LDFJAC,.TRUE.,IPVT,N,WA1,WA2,WA3)
C
C        ON THE FIRST ITERATION AND IF MODE IS 1, SCALE ACCORDING
C        TO THE NORMS OF THE COLUMNS OF THE INITIAL JACOBIAN.
C
         IF (ITER .NE. 1) GO TO 80
         IF (MODE .EQ. 2) GO TO 60
         DO 50 J = 1, N
            DIAG(J) = WA2(J)
            IF (WA2(J) .EQ. ZERO) DIAG(J) = ONE
   50       CONTINUE
   60    CONTINUE
C
C        ON THE FIRST ITERATION, CALCULATE THE NORM OF THE SCALED X
C        AND INITIALIZE THE STEP BOUND DELTA.
C
         DO 70 J = 1, N
            WA3(J) = DIAG(J)*X(J)
   70       CONTINUE
         XNORM = ENORM(N,WA3)
         DELTA = FACTOR*XNORM
         IF (DELTA .EQ. ZERO) DELTA = FACTOR
   80    CONTINUE
C
C        FORM (Q TRANSPOSE)*FVEC AND STORE THE FIRST N COMPONENTS IN
C        QTF.
C
         DO 90 I = 1, M
            WA4(I) = FVEC(I)
   90       CONTINUE
         DO 130 J = 1, N
            IF (FJAC(J,J) .EQ. ZERO) GO TO 120
            SUM = ZERO
            DO 100 I = J, M
               SUM = SUM + FJAC(I,J)*WA4(I)
  100          CONTINUE
            TEMP = -SUM/FJAC(J,J)
            DO 110 I = J, M
               WA4(I) = WA4(I) + FJAC(I,J)*TEMP
  110          CONTINUE
  120       CONTINUE
            FJAC(J,J) = WA1(J)
            QTF(J) = WA4(J)
  130       CONTINUE
C
C        COMPUTE THE NORM OF THE SCALED GRADIENT.
C
         GNORM = ZERO
         IF (FNORM .EQ. ZERO) GO TO 170
         DO 160 J = 1, N
            L = IPVT(J)
            IF (WA2(L) .EQ. ZERO) GO TO 150
            SUM = ZERO
            DO 140 I = 1, J
               SUM = SUM + FJAC(I,J)*(QTF(I)/FNORM)
  140          CONTINUE
            GNORM = MAX(GNORM,ABS(SUM/WA2(L)))
  150       CONTINUE
  160       CONTINUE
  170    CONTINUE
C
C        TEST FOR CONVERGENCE OF THE GRADIENT NORM.
C
         IF (GNORM .LE. GTOL) INFO = 4
         IF (INFO .NE. 0) GO TO 300
C
C        RESCALE IF NECESSARY.
C
         IF (MODE .EQ. 2) GO TO 190
         DO 180 J = 1, N
            DIAG(J) = MAX(DIAG(J),WA2(J))
  180       CONTINUE
  190    CONTINUE
C
C        BEGINNING OF THE INNER LOOP.
C
  200    CONTINUE
C
C           DETERMINE THE LEVENBERG-MARQUARDT PARAMETER.
C
            CALL LMPAR(N,FJAC,LDFJAC,IPVT,DIAG,QTF,DELTA,PAR,WA1,WA2,
     *                 WA3,WA4)
C
C           STORE THE DIRECTION P AND X + P. CALCULATE THE NORM OF P.
C
            DO 210 J = 1, N
               WA1(J) = -WA1(J)
               WA2(J) = X(J) + WA1(J)
               WA3(J) = DIAG(J)*WA1(J)
  210          CONTINUE
            PNORM = ENORM(N,WA3)
C
C           ON THE FIRST ITERATION, ADJUST THE INITIAL STEP BOUND.
C
            IF (ITER .EQ. 1) DELTA = MIN(DELTA,PNORM)
C
C           EVALUATE THE FUNCTION AT X + P AND CALCULATE ITS NORM.
C
            IFLAG = 1
            CALL FCN(M,N,WA2,WA4,IFLAG)
            NFEV = NFEV + 1
            IF (IFLAG .LT. 0) GO TO 300
            FNORM1 = ENORM(M,WA4)
C
C           COMPUTE THE SCALED ACTUAL REDUCTION.
C
            ACTRED = -ONE
            IF (P1*FNORM1 .LT. FNORM) ACTRED = ONE - (FNORM1/FNORM)**2
C
C           COMPUTE THE SCALED PREDICTED REDUCTION AND
C           THE SCALED DIRECTIONAL DERIVATIVE.
C
            DO 230 J = 1, N
               WA3(J) = ZERO
               L = IPVT(J)
               TEMP = WA1(L)
               DO 220 I = 1, J
                  WA3(I) = WA3(I) + FJAC(I,J)*TEMP
  220             CONTINUE
  230          CONTINUE
            TEMP1 = ENORM(N,WA3)/FNORM
            TEMP2 = (SQRT(PAR)*PNORM)/FNORM
            PRERED = TEMP1**2 + TEMP2**2/P5
            DIRDER = -(TEMP1**2 + TEMP2**2)
C
C           COMPUTE THE RATIO OF THE ACTUAL TO THE PREDICTED
C           REDUCTION.
C
            RATIO = ZERO
            IF (PRERED .NE. ZERO) RATIO = ACTRED/PRERED
C
C           UPDATE THE STEP BOUND.
C
            IF (RATIO .GT. P25) GO TO 240
               IF (ACTRED .GE. ZERO) TEMP = P5
               IF (ACTRED .LT. ZERO)
     *            TEMP = P5*DIRDER/(DIRDER + P5*ACTRED)
               IF (P1*FNORM1 .GE. FNORM .OR. TEMP .LT. P1) TEMP = P1
               DELTA = TEMP*MIN(DELTA,PNORM/P1)
               PAR = PAR/TEMP
               GO TO 260
  240       CONTINUE
               IF (PAR .NE. ZERO .AND. RATIO .LT. P75) GO TO 250
               DELTA = PNORM/P5
               PAR = P5*PAR
  250          CONTINUE
  260       CONTINUE
C
C           TEST FOR SUCCESSFUL ITERATION.
C
            IF (RATIO .LT. P0001) GO TO 290
C
C           SUCCESSFUL ITERATION. UPDATE X, FVEC, AND THEIR NORMS.
C
            DO 270 J = 1, N
               X(J) = WA2(J)
               WA2(J) = DIAG(J)*X(J)
  270          CONTINUE
            DO 280 I = 1, M
               FVEC(I) = WA4(I)
  280          CONTINUE
            XNORM = ENORM(N,WA2)
            FNORM = FNORM1
            ITER = ITER + 1
  290       CONTINUE
C
C           TESTS FOR CONVERGENCE.
C
            IF (ABS(ACTRED) .LE. FTOL .AND. PRERED .LE. FTOL
     *          .AND. P5*RATIO .LE. ONE) INFO = 1
            IF (DELTA .LE. XTOL*XNORM) INFO = 2
            IF (ABS(ACTRED) .LE. FTOL .AND. PRERED .LE. FTOL
     *          .AND. P5*RATIO .LE. ONE .AND. INFO .EQ. 2) INFO = 3
            IF (INFO .NE. 0) GO TO 300
C
C           TESTS FOR TERMINATION AND STRINGENT TOLERANCES.
C
            IF (NFEV .GE. MAXFEV) INFO = 5
            IF (ABS(ACTRED) .LE. EPSMCH .AND. PRERED .LE. EPSMCH
     *          .AND. P5*RATIO .LE. ONE) INFO = 6
            IF (DELTA .LE. EPSMCH*XNORM) INFO = 7
            IF (GNORM .LE. EPSMCH) INFO = 8
            IF (INFO .NE. 0) GO TO 300
C
C           END OF THE INNER LOOP. REPEAT IF ITERATION UNSUCCESSFUL.
C
            IF (RATIO .LT. P0001) GO TO 200
C
C        END OF THE OUTER LOOP.
C
         GO TO 30
  300 CONTINUE
C
C     TERMINATION, EITHER NORMAL OR USER IMPOSED.
C
      IF (IFLAG .LT. 0) INFO = IFLAG
      IFLAG = 0
      IF (NPRINT .GT. 0) CALL FCN(M,N,X,FVEC,IFLAG)
      RETURN
C
C     LAST CARD OF SUBROUTINE LMDIF.
C
      END
      FUNCTION ENORM(N,X)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER N
      DIMENSION X(N)
C     **********
C
C     FUNCTION ENORM
C
C     GIVEN AN N-VECTOR X, THIS FUNCTION CALCULATES THE
C     EUCLIDEAN NORM OF X.
C
C     THE EUCLIDEAN NORM IS COMPUTED BY ACCUMULATING THE SUM OF
C     SQUARES IN THREE DIFFERENT SUMS. THE SUMS OF SQUARES FOR THE
C     SMALL AND LARGE COMPONENTS ARE SCALED SO THAT NO OVERFLOWS
C     OCCUR. NON-DESTRUCTIVE UNDERFLOWS ARE PERMITTED. UNDERFLOWS
C     AND OVERFLOWS DO NOT OCCUR IN THE COMPUTATION OF THE UNSCALED
C     SUM OF SQUARES FOR THE INTERMEDIATE COMPONENTS.
C     THE DEFINITIONS OF SMALL, INTERMEDIATE AND LARGE COMPONENTS
C     DEPEND ON TWO CONSTANTS, RDWARF AND RGIANT. THE MAIN
C     RESTRICTIONS ON THESE CONSTANTS ARE THAT RDWARF**2 NOT
C     UNDERFLOW AND RGIANT**2 NOT OVERFLOW. THE CONSTANTS
C     GIVEN HERE ARE SUITABLE FOR EVERY KNOWN COMPUTER.
C
C     THE FUNCTION STATEMENT IS
C
C       REAL FUNCTION ENORM(N,X)
C
C     WHERE
C
C       N IS A POSITIVE INTEGER INPUT VARIABLE.
C
C       X IS AN INPUT ARRAY OF LENGTH N.
C
C     SUBPROGRAMS CALLED
C
C       FORTRAN-SUPPLIED ... ABS,SQRT
C
C     ARGONNE NATIONAL LABORATORY. MINPACK PROJECT. MARCH 1980.
C     BURTON S. GARBOW, KENNETH E. HILLSTROM, JORGE J. MORE
C
C     **********
      INTEGER I
      DATA ONE,ZERO,RDWARF,RGIANT /1.0E0,0.0E0,3.834E-20,1.304E19/
      S1 = ZERO
      S2 = ZERO
      S3 = ZERO
      X1MAX = ZERO
      X3MAX = ZERO
      FLOATN = N
      AGIANT = RGIANT/FLOATN
      DO 90 I = 1, N
         XABS = ABS(X(I))
         IF (XABS .GT. RDWARF .AND. XABS .LT. AGIANT) GO TO 70
            IF (XABS .LE. RDWARF) GO TO 30
C
C              SUM FOR LARGE COMPONENTS.
C
               IF (XABS .LE. X1MAX) GO TO 10
                  S1 = ONE + S1*(X1MAX/XABS)**2
                  X1MAX = XABS
                  GO TO 20
   10          CONTINUE
                  S1 = S1 + (XABS/X1MAX)**2
   20          CONTINUE
               GO TO 60
   30       CONTINUE
C
C              SUM FOR SMALL COMPONENTS.
C
               IF (XABS .LE. X3MAX) GO TO 40
                  S3 = ONE + S3*(X3MAX/XABS)**2
                  X3MAX = XABS
                  GO TO 50
   40          CONTINUE
                  IF (XABS .NE. ZERO) S3 = S3 + (XABS/X3MAX)**2
   50          CONTINUE
   60       CONTINUE
            GO TO 80
   70    CONTINUE
C
C           SUM FOR INTERMEDIATE COMPONENTS.
C
            S2 = S2 + XABS**2
   80    CONTINUE
   90    CONTINUE
C
C     CALCULATION OF NORM.
C
      IF (S1 .EQ. ZERO) GO TO 100
         ENORM = X1MAX*SQRT(S1+(S2/X1MAX)/X1MAX)
         GO TO 130
  100 CONTINUE
         IF (S2 .EQ. ZERO) GO TO 110
            IF (S2 .GE. X3MAX)
     *         ENORM = SQRT(S2*(ONE+(X3MAX/S2)*(X3MAX*S3)))
            IF (S2 .LT. X3MAX)
     *         ENORM = SQRT(X3MAX*((S2/X3MAX)+(X3MAX*S3)))
            GO TO 120
  110    CONTINUE
            ENORM = X3MAX*SQRT(S3)
  120    CONTINUE
  130 CONTINUE
      RETURN
C
C     LAST CARD OF FUNCTION ENORM.
C
      END
      SUBROUTINE FDJAC2(FCN,M,N,X,FVEC,FJAC,LDFJAC,IFLAG,EPSFCN,WA)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER M,N,LDFJAC,IFLAG
      DIMENSION X(N),FVEC(M),FJAC(LDFJAC,N),WA(M)
C     **********
C
C     SUBROUTINE FDJAC2
C
C     THIS SUBROUTINE COMPUTES A FORWARD-DIFFERENCE APPROXIMATION
C     TO THE M BY N JACOBIAN MATRIX ASSOCIATED WITH A SPECIFIED
C     PROBLEM OF M FUNCTIONS IN N VARIABLES.
C
C     THE SUBROUTINE STATEMENT IS
C
C       SUBROUTINE FDJAC2(FCN,M,N,X,FVEC,FJAC,LDFJAC,IFLAG,EPSFCN,WA)
C
C     WHERE
C
C       FCN IS THE NAME OF THE USER-SUPPLIED SUBROUTINE WHICH
C         CALCULATES THE FUNCTIONS. FCN MUST BE DECLARED
C         IN AN EXTERNAL STATEMENT IN THE USER CALLING
C         PROGRAM, AND SHOULD BE WRITTEN AS FOLLOWS.
C
C         SUBROUTINE FCN(M,N,X,FVEC,IFLAG)
C         INTEGER M,N,IFLAG
C         REAL X(N),FVEC(M)
C         ----------
C         CALCULATE THE FUNCTIONS AT X AND
C         RETURN THIS VECTOR IN FVEC.
C         ----------
C         RETURN
C         END
C
C         THE VALUE OF IFLAG SHOULD NOT BE CHANGED BY FCN UNLESS
C         THE USER WANTS TO TERMINATE EXECUTION OF FDJAC2.
C         IN THIS CASE SET IFLAG TO A NEGATIVE INTEGER.
C
C       M IS A POSITIVE INTEGER INPUT VARIABLE SET TO THE NUMBER
C         OF FUNCTIONS.
C
C       N IS A POSITIVE INTEGER INPUT VARIABLE SET TO THE NUMBER
C         OF VARIABLES. N MUST NOT EXCEED M.
C
C       X IS AN INPUT ARRAY OF LENGTH N.
C
C       FVEC IS AN INPUT ARRAY OF LENGTH M WHICH MUST CONTAIN THE
C         FUNCTIONS EVALUATED AT X.
C
C       FJAC IS AN OUTPUT M BY N ARRAY WHICH CONTAINS THE
C         APPROXIMATION TO THE JACOBIAN MATRIX EVALUATED AT X.
C
C       LDFJAC IS A POSITIVE INTEGER INPUT VARIABLE NOT LESS THAN M
C         WHICH SPECIFIES THE LEADING DIMENSION OF THE ARRAY FJAC.
C
C       IFLAG IS AN INTEGER VARIABLE WHICH CAN BE USED TO TERMINATE
C         THE EXECUTION OF FDJAC2. SEE DESCRIPTION OF FCN.
C
C       EPSFCN IS AN INPUT VARIABLE USED IN DETERMINING A SUITABLE
C         STEP LENGTH FOR THE FORWARD-DIFFERENCE APPROXIMATION. THIS
C         APPROXIMATION ASSUMES THAT THE RELATIVE ERRORS IN THE
C         FUNCTIONS ARE OF THE ORDER OF EPSFCN. IF EPSFCN IS LESS
C         THAN THE MACHINE PRECISION, IT IS ASSUMED THAT THE RELATIVE
C         ERRORS IN THE FUNCTIONS ARE OF THE ORDER OF THE MACHINE
C         PRECISION.
C
C       WA IS A WORK ARRAY OF LENGTH M.
C
C     SUBPROGRAMS CALLED
C
C       USER-SUPPLIED ...... FCN
C
C       MINPACK-SUPPLIED ... SPMPAR
C
C       FORTRAN-SUPPLIED ... ABS,AMAX1,SQRT
C
C     ARGONNE NATIONAL LABORATORY. MINPACK PROJECT. MARCH 1980.
C     BURTON S. GARBOW, KENNETH E. HILLSTROM, JORGE J. MORE
C
C     **********
      INTEGER I,J
      DATA ZERO /0.0E0/
C
C     EPSMCH IS THE MACHINE PRECISION.
C
      EPSMCH = SPMPAR(1)
C
      EPS = SQRT(MAX(EPSFCN,EPSMCH))
      DO 20 J = 1, N
         TEMP = X(J)
         H = EPS*ABS(TEMP)
         IF (H .EQ. ZERO) H = EPS
         X(J) = TEMP + H
         CALL FCN(M,N,X,WA,IFLAG)
         IF (IFLAG .LT. 0) GO TO 30
         X(J) = TEMP
         DO 10 I = 1, M
            FJAC(I,J) = (WA(I) - FVEC(I))/H
   10       CONTINUE
   20    CONTINUE
   30 CONTINUE
      RETURN
C
C     LAST CARD OF SUBROUTINE FDJAC2.
C
      END
      SUBROUTINE LMPAR(N,R,LDR,IPVT,DIAG,QTB,DELTA,PAR,X,SDIAG,WA1,
     *                 WA2)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER N,LDR
      INTEGER IPVT(N)
      DIMENSION R(LDR,N),DIAG(N),QTB(N),X(N),SDIAG(N),WA1(N),WA2(N)
C     **********
C
C     SUBROUTINE LMPAR
C
C     GIVEN AN M BY N MATRIX A, AN N BY N NONSINGULAR DIAGONAL
C     MATRIX D, AN M-VECTOR B, AND A POSITIVE NUMBER DELTA,
C     THE PROBLEM IS TO DETERMINE A VALUE FOR THE PARAMETER
C     PAR SUCH THAT IF X SOLVES THE SYSTEM
C
C           A*X = B ,     SQRT(PAR)*D*X = 0 ,
C
C     IN THE LEAST SQUARES SENSE, AND DXNORM IS THE EUCLIDEAN
C     NORM OF D*X, THEN EITHER PAR IS ZERO AND
C
C           (DXNORM-DELTA) .LE. 0.1*DELTA ,
C
C     OR PAR IS POSITIVE AND
C
C           ABS(DXNORM-DELTA) .LE. 0.1*DELTA .
C
C     THIS SUBROUTINE COMPLETES THE SOLUTION OF THE PROBLEM
C     IF IT IS PROVIDED WITH THE NECESSARY INFORMATION FROM THE
C     QR FACTORIZATION, WITH COLUMN PIVOTING, OF A. THAT IS, IF
C     A*P = Q*R, WHERE P IS A PERMUTATION MATRIX, Q HAS ORTHOGONAL
C     COLUMNS, AND R IS AN UPPER TRIANGULAR MATRIX WITH DIAGONAL
C     ELEMENTS OF NONINCREASING MAGNITUDE, THEN LMPAR EXPECTS
C     THE FULL UPPER TRIANGLE OF R, THE PERMUTATION MATRIX P,
C     AND THE FIRST N COMPONENTS OF (Q TRANSPOSE)*B. ON OUTPUT
C     LMPAR ALSO PROVIDES AN UPPER TRIANGULAR MATRIX S SUCH THAT
C
C            T   T                   T
C           P *(A *A + PAR*D*D)*P = S *S .
C
C     S IS EMPLOYED WITHIN LMPAR AND MAY BE OF SEPARATE INTEREST.
C
C     ONLY A FEW ITERATIONS ARE GENERALLY NEEDED FOR CONVERGENCE
C     OF THE ALGORITHM. IF, HOWEVER, THE LIMIT OF 10 ITERATIONS
C     IS REACHED, THEN THE OUTPUT PAR WILL CONTAIN THE BEST
C     VALUE OBTAINED SO FAR.
C
C     THE SUBROUTINE STATEMENT IS
C
C       SUBROUTINE LMPAR(N,R,LDR,IPVT,DIAG,QTB,DELTA,PAR,X,SDIAG,
C                        WA1,WA2)
C
C     WHERE
C
C       N IS A POSITIVE INTEGER INPUT VARIABLE SET TO THE ORDER OF R.
C
C       R IS AN N BY N ARRAY. ON INPUT THE FULL UPPER TRIANGLE
C         MUST CONTAIN THE FULL UPPER TRIANGLE OF THE MATRIX R.
C         ON OUTPUT THE FULL UPPER TRIANGLE IS UNALTERED, AND THE
C         STRICT LOWER TRIANGLE CONTAINS THE STRICT UPPER TRIANGLE
C         (TRANSPOSED) OF THE UPPER TRIANGULAR MATRIX S.
C
C       LDR IS A POSITIVE INTEGER INPUT VARIABLE NOT LESS THAN N
C         WHICH SPECIFIES THE LEADING DIMENSION OF THE ARRAY R.
C
C       IPVT IS AN INTEGER INPUT ARRAY OF LENGTH N WHICH DEFINES THE
C         PERMUTATION MATRIX P SUCH THAT A*P = Q*R. COLUMN J OF P
C         IS COLUMN IPVT(J) OF THE IDENTITY MATRIX.
C
C       DIAG IS AN INPUT ARRAY OF LENGTH N WHICH MUST CONTAIN THE
C         DIAGONAL ELEMENTS OF THE MATRIX D.
C
C       QTB IS AN INPUT ARRAY OF LENGTH N WHICH MUST CONTAIN THE FIRST
C         N ELEMENTS OF THE VECTOR (Q TRANSPOSE)*B.
C
C       DELTA IS A POSITIVE INPUT VARIABLE WHICH SPECIFIES AN UPPER
C         BOUND ON THE EUCLIDEAN NORM OF D*X.
C
C       PAR IS A NONNEGATIVE VARIABLE. ON INPUT PAR CONTAINS AN
C         INITIAL ESTIMATE OF THE LEVENBERG-MARQUARDT PARAMETER.
C         ON OUTPUT PAR CONTAINS THE FINAL ESTIMATE.
C
C       X IS AN OUTPUT ARRAY OF LENGTH N WHICH CONTAINS THE LEAST
C         SQUARES SOLUTION OF THE SYSTEM A*X = B, SQRT(PAR)*D*X = 0,
C         FOR THE OUTPUT PAR.
C
C       SDIAG IS AN OUTPUT ARRAY OF LENGTH N WHICH CONTAINS THE
C         DIAGONAL ELEMENTS OF THE UPPER TRIANGULAR MATRIX S.
C
C       WA1 AND WA2 ARE WORK ARRAYS OF LENGTH N.
C
C     SUBPROGRAMS CALLED
C
C       MINPACK-SUPPLIED ... SPMPAR,ENORM,QRSOLV
C
C       FORTRAN-SUPPLIED ... ABS,AMAX1,AMIN1,SQRT
C
C     ARGONNE NATIONAL LABORATORY. MINPACK PROJECT. MARCH 1980.
C     BURTON S. GARBOW, KENNETH E. HILLSTROM, JORGE J. MORE
C
C     **********
      INTEGER I,ITER,J,JM1,JP1,K,L,NSING
      DATA P1,P001,ZERO /1.0E-1,1.0E-3,0.0E0/
C
C     DWARF IS THE SMALLEST POSITIVE MAGNITUDE.
C
      DWARF = SPMPAR(2)
C
C     COMPUTE AND STORE IN X THE GAUSS-NEWTON DIRECTION. IF THE
C     JACOBIAN IS RANK-DEFICIENT, OBTAIN A LEAST SQUARES SOLUTION.
C
      NSING = N
      DO 10 J = 1, N
         WA1(J) = QTB(J)
         IF (R(J,J) .EQ. ZERO .AND. NSING .EQ. N) NSING = J - 1
         IF (NSING .LT. N) WA1(J) = ZERO
   10    CONTINUE
      IF (NSING .LT. 1) GO TO 50
      DO 40 K = 1, NSING
         J = NSING - K + 1
         WA1(J) = WA1(J)/R(J,J)
         TEMP = WA1(J)
         JM1 = J - 1
         IF (JM1 .LT. 1) GO TO 30
         DO 20 I = 1, JM1
            WA1(I) = WA1(I) - R(I,J)*TEMP
   20       CONTINUE
   30    CONTINUE
   40    CONTINUE
   50 CONTINUE
      DO 60 J = 1, N
         L = IPVT(J)
         X(L) = WA1(J)
   60    CONTINUE
C
C     INITIALIZE THE ITERATION COUNTER.
C     EVALUATE THE FUNCTION AT THE ORIGIN, AND TEST
C     FOR ACCEPTANCE OF THE GAUSS-NEWTON DIRECTION.
C
      ITER = 0
      DO 70 J = 1, N
         WA2(J) = DIAG(J)*X(J)
   70    CONTINUE
      DXNORM = ENORM(N,WA2)
      FP = DXNORM - DELTA
      IF (FP .LE. P1*DELTA) GO TO 220
C
C     IF THE JACOBIAN IS NOT RANK DEFICIENT, THE NEWTON
C     STEP PROVIDES A LOWER BOUND, PARL, FOR THE ZERO OF
C     THE FUNCTION. OTHERWISE SET THIS BOUND TO ZERO.
C
      PARL = ZERO
      IF (NSING .LT. N) GO TO 120
      DO 80 J = 1, N
         L = IPVT(J)
         WA1(J) = DIAG(L)*(WA2(L)/DXNORM)
   80    CONTINUE
      DO 110 J = 1, N
         SUM = ZERO
         JM1 = J - 1
         IF (JM1 .LT. 1) GO TO 100
         DO 90 I = 1, JM1
            SUM = SUM + R(I,J)*WA1(I)
   90       CONTINUE
  100    CONTINUE
         WA1(J) = (WA1(J) - SUM)/R(J,J)
  110    CONTINUE
      TEMP = ENORM(N,WA1)
      PARL = ((FP/DELTA)/TEMP)/TEMP
  120 CONTINUE
C
C     CALCULATE AN UPPER BOUND, PARU, FOR THE ZERO OF THE FUNCTION.
C
      DO 140 J = 1, N
         SUM = ZERO
         DO 130 I = 1, J
            SUM = SUM + R(I,J)*QTB(I)
  130       CONTINUE
         L = IPVT(J)
         WA1(J) = SUM/DIAG(L)
  140    CONTINUE
      GNORM = ENORM(N,WA1)
      PARU = GNORM/DELTA
      IF (PARU .EQ. ZERO) PARU = DWARF/MIN(DELTA,P1)
C
C     IF THE INPUT PAR LIES OUTSIDE OF THE INTERVAL (PARL,PARU),
C     SET PAR TO THE CLOSER ENDPOINT.
C
      PAR = MAX(PAR,PARL)
      PAR = MIN(PAR,PARU)
      IF (PAR .EQ. ZERO) PAR = GNORM/DXNORM
C
C     BEGINNING OF AN ITERATION.
C
  150 CONTINUE
         ITER = ITER + 1
C
C        EVALUATE THE FUNCTION AT THE CURRENT VALUE OF PAR.
C
         IF (PAR .EQ. ZERO) PAR = MAX(DWARF,P001*PARU)
         TEMP = SQRT(PAR)
         DO 160 J = 1, N
            WA1(J) = TEMP*DIAG(J)
  160       CONTINUE
         CALL QRSOLV(N,R,LDR,IPVT,WA1,QTB,X,SDIAG,WA2)
         DO 170 J = 1, N
            WA2(J) = DIAG(J)*X(J)
  170       CONTINUE
         DXNORM = ENORM(N,WA2)
         TEMP = FP
         FP = DXNORM - DELTA
C
C        IF THE FUNCTION IS SMALL ENOUGH, ACCEPT THE CURRENT VALUE
C        OF PAR. ALSO TEST FOR THE EXCEPTIONAL CASES WHERE PARL
C        IS ZERO OR THE NUMBER OF ITERATIONS HAS REACHED 10.
C
         IF (ABS(FP) .LE. P1*DELTA
     *       .OR. PARL .EQ. ZERO .AND. FP .LE. TEMP
     *            .AND. TEMP .LT. ZERO .OR. ITER .EQ. 10) GO TO 220
C
C        COMPUTE THE NEWTON CORRECTION.
C
         DO 180 J = 1, N
            L = IPVT(J)
            WA1(J) = DIAG(L)*(WA2(L)/DXNORM)
  180       CONTINUE
         DO 210 J = 1, N
            WA1(J) = WA1(J)/SDIAG(J)
            TEMP = WA1(J)
            JP1 = J + 1
            IF (N .LT. JP1) GO TO 200
            DO 190 I = JP1, N
               WA1(I) = WA1(I) - R(I,J)*TEMP
  190          CONTINUE
  200       CONTINUE
  210       CONTINUE
         TEMP = ENORM(N,WA1)
         PARC = ((FP/DELTA)/TEMP)/TEMP
C
C        DEPENDING ON THE SIGN OF THE FUNCTION, UPDATE PARL OR PARU.
C
         IF (FP .GT. ZERO) PARL = MAX(PARL,PAR)
         IF (FP .LT. ZERO) PARU = MIN(PARU,PAR)
C
C        COMPUTE AN IMPROVED ESTIMATE FOR PAR.
C
         PAR = MAX(PARL,PAR+PARC)
C
C        END OF AN ITERATION.
C
         GO TO 150
  220 CONTINUE
C
C     TERMINATION.
C
      IF (ITER .EQ. 0) PAR = ZERO
      RETURN
C
C     LAST CARD OF SUBROUTINE LMPAR.
C
      END
      SUBROUTINE QRFAC(M,N,A,LDA,PIVOT,IPVT,LIPVT,RDIAG,ACNORM,WA)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER M,N,LDA,LIPVT
      INTEGER IPVT(LIPVT)
      LOGICAL PIVOT
      DIMENSION A(LDA,N),RDIAG(N),ACNORM(N),WA(N)
C     **********
C
C     SUBROUTINE QRFAC
C
C     THIS SUBROUTINE USES HOUSEHOLDER TRANSFORMATIONS WITH COLUMN
C     PIVOTING (OPTIONAL) TO COMPUTE A QR FACTORIZATION OF THE
C     M BY N MATRIX A. THAT IS, QRFAC DETERMINES AN ORTHOGONAL
C     MATRIX Q, A PERMUTATION MATRIX P, AND AN UPPER TRAPEZOIDAL
C     MATRIX R WITH DIAGONAL ELEMENTS OF NONINCREASING MAGNITUDE,
C     SUCH THAT A*P = Q*R. THE HOUSEHOLDER TRANSFORMATION FOR
C     COLUMN K, K = 1,2,...,MIN(M,N), IS OF THE FORM
C
C                           T
C           I - (1/U(K))*U*U
C
C     WHERE U HAS ZEROS IN THE FIRST K-1 POSITIONS. THE FORM OF
C     THIS TRANSFORMATION AND THE METHOD OF PIVOTING FIRST
C     APPEARED IN THE CORRESPONDING LINPACK SUBROUTINE.
C
C     THE SUBROUTINE STATEMENT IS
C
C       SUBROUTINE QRFAC(M,N,A,LDA,PIVOT,IPVT,LIPVT,RDIAG,ACNORM,WA)
C
C     WHERE
C
C       M IS A POSITIVE INTEGER INPUT VARIABLE SET TO THE NUMBER
C         OF ROWS OF A.
C
C       N IS A POSITIVE INTEGER INPUT VARIABLE SET TO THE NUMBER
C         OF COLUMNS OF A.
C
C       A IS AN M BY N ARRAY. ON INPUT A CONTAINS THE MATRIX FOR
C         WHICH THE QR FACTORIZATION IS TO BE COMPUTED. ON OUTPUT
C         THE STRICT UPPER TRAPEZOIDAL PART OF A CONTAINS THE STRICT
C         UPPER TRAPEZOIDAL PART OF R, AND THE LOWER TRAPEZOIDAL
C         PART OF A CONTAINS A FACTORED FORM OF Q (THE NON-TRIVIAL
C         ELEMENTS OF THE U VECTORS DESCRIBED ABOVE).
C
C       LDA IS A POSITIVE INTEGER INPUT VARIABLE NOT LESS THAN M
C         WHICH SPECIFIES THE LEADING DIMENSION OF THE ARRAY A.
C
C       PIVOT IS A LOGICAL INPUT VARIABLE. IF PIVOT IS SET TRUE,
C         THEN COLUMN PIVOTING IS ENFORCED. IF PIVOT IS SET FALSE,
C         THEN NO COLUMN PIVOTING IS DONE.
C
C       IPVT IS AN INTEGER OUTPUT ARRAY OF LENGTH LIPVT. IPVT
C         DEFINES THE PERMUTATION MATRIX P SUCH THAT A*P = Q*R.
C         COLUMN J OF P IS COLUMN IPVT(J) OF THE IDENTITY MATRIX.
C         IF PIVOT IS FALSE, IPVT IS NOT REFERENCED.
C
C       LIPVT IS A POSITIVE INTEGER INPUT VARIABLE. IF PIVOT IS FALSE,
C         THEN LIPVT MAY BE AS SMALL AS 1. IF PIVOT IS TRUE, THEN
C         LIPVT MUST BE AT LEAST N.
C
C       RDIAG IS AN OUTPUT ARRAY OF LENGTH N WHICH CONTAINS THE
C         DIAGONAL ELEMENTS OF R.
C
C       ACNORM IS AN OUTPUT ARRAY OF LENGTH N WHICH CONTAINS THE
C         NORMS OF THE CORRESPONDING COLUMNS OF THE INPUT MATRIX A.
C         IF THIS INFORMATION IS NOT NEEDED, THEN ACNORM CAN COINCIDE
C         WITH RDIAG.
C
C       WA IS A WORK ARRAY OF LENGTH N. IF PIVOT IS FALSE, THEN WA
C         CAN COINCIDE WITH RDIAG.
C
C     SUBPROGRAMS CALLED
C
C       MINPACK-SUPPLIED ... SPMPAR,ENORM
C
C       FORTRAN-SUPPLIED ... AMAX1,SQRT,MIN0
C
C     ARGONNE NATIONAL LABORATORY. MINPACK PROJECT. MARCH 1980.
C     BURTON S. GARBOW, KENNETH E. HILLSTROM, JORGE J. MORE
C
C     **********
      INTEGER I,J,JP1,K,KMAX,MINMN
      DATA ONE,P05,ZERO /1.0E0,5.0E-2,0.0E0/
C
C     EPSMCH IS THE MACHINE PRECISION.
C
      EPSMCH = SPMPAR(1)
C
C     COMPUTE THE INITIAL COLUMN NORMS AND INITIALIZE SEVERAL ARRAYS.
C
      DO 10 J = 1, N
         ACNORM(J) = ENORM(M,A(1,J))
         RDIAG(J) = ACNORM(J)
         WA(J) = RDIAG(J)
         IF (PIVOT) IPVT(J) = J
   10    CONTINUE
C
C     REDUCE A TO R WITH HOUSEHOLDER TRANSFORMATIONS.
C
      MINMN = MIN0(M,N)
      DO 110 J = 1, MINMN
         IF (.NOT.PIVOT) GO TO 40
C
C        BRING THE COLUMN OF LARGEST NORM INTO THE PIVOT POSITION.
C
         KMAX = J
         DO 20 K = J, N
            IF (RDIAG(K) .GT. RDIAG(KMAX)) KMAX = K
   20       CONTINUE
         IF (KMAX .EQ. J) GO TO 40
         DO 30 I = 1, M
            TEMP = A(I,J)
            A(I,J) = A(I,KMAX)
            A(I,KMAX) = TEMP
   30       CONTINUE
         RDIAG(KMAX) = RDIAG(J)
         WA(KMAX) = WA(J)
         K = IPVT(J)
         IPVT(J) = IPVT(KMAX)
         IPVT(KMAX) = K
   40    CONTINUE
C
C        COMPUTE THE HOUSEHOLDER TRANSFORMATION TO REDUCE THE
C        J-TH COLUMN OF A TO A MULTIPLE OF THE J-TH UNIT VECTOR.
C
         AJNORM = ENORM(M-J+1,A(J,J))
         IF (AJNORM .EQ. ZERO) GO TO 100
         IF (A(J,J) .LT. ZERO) AJNORM = -AJNORM
         DO 50 I = J, M
            A(I,J) = A(I,J)/AJNORM
   50       CONTINUE
         A(J,J) = A(J,J) + ONE
C
C        APPLY THE TRANSFORMATION TO THE REMAINING COLUMNS
C        AND UPDATE THE NORMS.
C
         JP1 = J + 1
         IF (N .LT. JP1) GO TO 100
         DO 90 K = JP1, N
            SUM = ZERO
            DO 60 I = J, M
               SUM = SUM + A(I,J)*A(I,K)
   60          CONTINUE
            TEMP = SUM/A(J,J)
            DO 70 I = J, M
               A(I,K) = A(I,K) - TEMP*A(I,J)
   70          CONTINUE
            IF (.NOT.PIVOT .OR. RDIAG(K) .EQ. ZERO) GO TO 80
            TEMP = A(J,K)/RDIAG(K)
            RDIAG(K) = RDIAG(K)*SQRT(MAX(ZERO,ONE-TEMP**2))
            IF (P05*(RDIAG(K)/WA(K))**2 .GT. EPSMCH) GO TO 80
            RDIAG(K) = ENORM(M-J,A(JP1,K))
            WA(K) = RDIAG(K)
   80       CONTINUE
   90       CONTINUE
  100    CONTINUE
         RDIAG(J) = -AJNORM
  110    CONTINUE
      RETURN
C
C     LAST CARD OF SUBROUTINE QRFAC.
C
      END
      SUBROUTINE QRSOLV(N,R,LDR,IPVT,DIAG,QTB,X,SDIAG,WA)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER N,LDR
      INTEGER IPVT(N)
      DIMENSION R(LDR,N),DIAG(N),QTB(N),X(N),SDIAG(N),WA(N)
C     **********
C
C     SUBROUTINE QRSOLV
C
C     GIVEN AN M BY N MATRIX A, AN N BY N DIAGONAL MATRIX D,
C     AND AN M-VECTOR B, THE PROBLEM IS TO DETERMINE AN X WHICH
C     SOLVES THE SYSTEM
C
C           A*X = B ,     D*X = 0 ,
C
C     IN THE LEAST SQUARES SENSE.
C
C     THIS SUBROUTINE COMPLETES THE SOLUTION OF THE PROBLEM
C     IF IT IS PROVIDED WITH THE NECESSARY INFORMATION FROM THE
C     QR FACTORIZATION, WITH COLUMN PIVOTING, OF A. THAT IS, IF
C     A*P = Q*R, WHERE P IS A PERMUTATION MATRIX, Q HAS ORTHOGONAL
C     COLUMNS, AND R IS AN UPPER TRIANGULAR MATRIX WITH DIAGONAL
C     ELEMENTS OF NONINCREASING MAGNITUDE, THEN QRSOLV EXPECTS
C     THE FULL UPPER TRIANGLE OF R, THE PERMUTATION MATRIX P,
C     AND THE FIRST N COMPONENTS OF (Q TRANSPOSE)*B. THE SYSTEM
C     A*X = B, D*X = 0, IS THEN EQUIVALENT TO
C
C                  T       T
C           R*Z = Q *B ,  P *D*P*Z = 0 ,
C
C     WHERE X = P*Z. IF THIS SYSTEM DOES NOT HAVE FULL RANK,
C     THEN A LEAST SQUARES SOLUTION IS OBTAINED. ON OUTPUT QRSOLV
C     ALSO PROVIDES AN UPPER TRIANGULAR MATRIX S SUCH THAT
C
C            T   T               T
C           P *(A *A + D*D)*P = S *S .
C
C     S IS COMPUTED WITHIN QRSOLV AND MAY BE OF SEPARATE INTEREST.
C
C     THE SUBROUTINE STATEMENT IS
C
C       SUBROUTINE QRSOLV(N,R,LDR,IPVT,DIAG,QTB,X,SDIAG,WA)
C
C     WHERE
C
C       N IS A POSITIVE INTEGER INPUT VARIABLE SET TO THE ORDER OF R.
C
C       R IS AN N BY N ARRAY. ON INPUT THE FULL UPPER TRIANGLE
C         MUST CONTAIN THE FULL UPPER TRIANGLE OF THE MATRIX R.
C         ON OUTPUT THE FULL UPPER TRIANGLE IS UNALTERED, AND THE
C         STRICT LOWER TRIANGLE CONTAINS THE STRICT UPPER TRIANGLE
C         (TRANSPOSED) OF THE UPPER TRIANGULAR MATRIX S.
C
C       LDR IS A POSITIVE INTEGER INPUT VARIABLE NOT LESS THAN N
C         WHICH SPECIFIES THE LEADING DIMENSION OF THE ARRAY R.
C
C       IPVT IS AN INTEGER INPUT ARRAY OF LENGTH N WHICH DEFINES THE
C         PERMUTATION MATRIX P SUCH THAT A*P = Q*R. COLUMN J OF P
C         IS COLUMN IPVT(J) OF THE IDENTITY MATRIX.
C
C       DIAG IS AN INPUT ARRAY OF LENGTH N WHICH MUST CONTAIN THE
C         DIAGONAL ELEMENTS OF THE MATRIX D.
C
C       QTB IS AN INPUT ARRAY OF LENGTH N WHICH MUST CONTAIN THE FIRST
C         N ELEMENTS OF THE VECTOR (Q TRANSPOSE)*B.
C
C       X IS AN OUTPUT ARRAY OF LENGTH N WHICH CONTAINS THE LEAST
C         SQUARES SOLUTION OF THE SYSTEM A*X = B, D*X = 0.
C
C       SDIAG IS AN OUTPUT ARRAY OF LENGTH N WHICH CONTAINS THE
C         DIAGONAL ELEMENTS OF THE UPPER TRIANGULAR MATRIX S.
C
C       WA IS A WORK ARRAY OF LENGTH N.
C
C     SUBPROGRAMS CALLED
C
C       FORTRAN-SUPPLIED ... ABS,SQRT
C
C     ARGONNE NATIONAL LABORATORY. MINPACK PROJECT. MARCH 1980.
C     BURTON S. GARBOW, KENNETH E. HILLSTROM, JORGE J. MORE
C
C     **********
      INTEGER I,J,JP1,K,KP1,L,NSING
      DATA P5,P25,ZERO /5.0E-1,2.5E-1,0.0E0/
C
C     COPY R AND (Q TRANSPOSE)*B TO PRESERVE INPUT AND INITIALIZE S.
C     IN PARTICULAR, SAVE THE DIAGONAL ELEMENTS OF R IN X.
C
      DO 20 J = 1, N
         DO 10 I = J, N
            R(I,J) = R(J,I)
   10       CONTINUE
         X(J) = R(J,J)
         WA(J) = QTB(J)
   20    CONTINUE
C
C     ELIMINATE THE DIAGONAL MATRIX D USING A GIVENS ROTATION.
C
      DO 100 J = 1, N
C
C        PREPARE THE ROW OF D TO BE ELIMINATED, LOCATING THE
C        DIAGONAL ELEMENT USING P FROM THE QR FACTORIZATION.
C
         L = IPVT(J)
         IF (DIAG(L) .EQ. ZERO) GO TO 90
         DO 30 K = J, N
            SDIAG(K) = ZERO
   30       CONTINUE
         SDIAG(J) = DIAG(L)
C
C        THE TRANSFORMATIONS TO ELIMINATE THE ROW OF D
C        MODIFY ONLY A SINGLE ELEMENT OF (Q TRANSPOSE)*B
C        BEYOND THE FIRST N, WHICH IS INITIALLY ZERO.
C
         QTBPJ = ZERO
         DO 80 K = J, N
C
C           DETERMINE A GIVENS ROTATION WHICH ELIMINATES THE
C           APPROPRIATE ELEMENT IN THE CURRENT ROW OF D.
C
            IF (SDIAG(K) .EQ. ZERO) GO TO 70
            IF (ABS(R(K,K)) .GE. ABS(SDIAG(K))) GO TO 40
               COTAN = R(K,K)/SDIAG(K)
               SIN = P5/SQRT(P25+P25*COTAN**2)
               COS = SIN*COTAN
               GO TO 50
   40       CONTINUE
               TAN = SDIAG(K)/R(K,K)
               COS = P5/SQRT(P25+P25*TAN**2)
               SIN = COS*TAN
   50       CONTINUE
C
C           COMPUTE THE MODIFIED DIAGONAL ELEMENT OF R AND
C           THE MODIFIED ELEMENT OF ((Q TRANSPOSE)*B,0).
C
            R(K,K) = COS*R(K,K) + SIN*SDIAG(K)
            TEMP = COS*WA(K) + SIN*QTBPJ
            QTBPJ = -SIN*WA(K) + COS*QTBPJ
            WA(K) = TEMP
C
C           ACCUMULATE THE TRANFORMATION IN THE ROW OF S.
C
            KP1 = K + 1
            IF (N .LT. KP1) GO TO 70
            DO 60 I = KP1, N
               TEMP = COS*R(I,K) + SIN*SDIAG(I)
               SDIAG(I) = -SIN*R(I,K) + COS*SDIAG(I)
               R(I,K) = TEMP
   60          CONTINUE
   70       CONTINUE
   80       CONTINUE
   90    CONTINUE
C
C        STORE THE DIAGONAL ELEMENT OF S AND RESTORE
C        THE CORRESPONDING DIAGONAL ELEMENT OF R.
C
         SDIAG(J) = R(J,J)
         R(J,J) = X(J)
  100    CONTINUE
C
C     SOLVE THE TRIANGULAR SYSTEM FOR Z. IF THE SYSTEM IS
C     SINGULAR, THEN OBTAIN A LEAST SQUARES SOLUTION.
C
      NSING = N
      DO 110 J = 1, N
         IF (SDIAG(J) .EQ. ZERO .AND. NSING .EQ. N) NSING = J - 1
         IF (NSING .LT. N) WA(J) = ZERO
  110    CONTINUE
      IF (NSING .LT. 1) GO TO 150
      DO 140 K = 1, NSING
         J = NSING - K + 1
         SUM = ZERO
         JP1 = J + 1
         IF (NSING .LT. JP1) GO TO 130
         DO 120 I = JP1, NSING
            SUM = SUM + R(I,J)*WA(I)
  120       CONTINUE
  130    CONTINUE
         WA(J) = (WA(J) - SUM)/SDIAG(J)
  140    CONTINUE
  150 CONTINUE
C
C     PERMUTE THE COMPONENTS OF Z BACK TO COMPONENTS OF X.
C
      DO 160 J = 1, N
         L = IPVT(J)
         X(L) = WA(J)
  160    CONTINUE
      RETURN
C
C     LAST CARD OF SUBROUTINE QRSOLV.
C
      END
      FUNCTION SPMPAR(I)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION RMACH(3)
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
C
      RMACH(1) = COMACC
      RMACH(2) = DWARF
      RMACH(3) = BIGNUM
      SPMPAR = RMACH(I)
      END
      SUBROUTINE LIMITS
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
C
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      PARAMETER ( MXV = 200 )
      PARAMETER ( MXT = 200 )
      PARAMETER ( MAXALI = 20 )
      PARAMETER ( MEM = 2500 , LSTA = 500 )
      PARAMETER ( MAXMFI = 20 )
      PARAMETER ( MAXNL = 20 )
      PARAMETER ( MAXFLA = 20 )
      PARAMETER ( MAXCFL = 10, MAXCLE = 30 )
C
      IF (STRO) THEN
         WRITE (IUOUT,10)
         WRITE (IUOUT,20) MXV
         WRITE (IUOUT,21) MXT
         WRITE (IUOUT,30) MAXALI
         WRITE (IUOUT,40) MEM
         WRITE (IUOUT,50) LSTA
         WRITE (IUOUT,60) MAXMFI
         WRITE (IUOUT,70) MAXNL
         WRITE (IUOUT,80) MAXFLA
         WRITE (IUOUT,90) MAXCFL
         WRITE (IUOUT,91) MAXCLE
      END IF
C
10    FORMAT (T2,'CURRENT LIMITS ARE:'/T2,50('-'))
20    FORMAT (T2,'NUMBER OF VARIABLES: ',T45,I7)
21    FORMAT (T2,'TERMS IN THE SUM OF SQUARES: ',T45,I7)
30    FORMAT (T2,'NUMBER OF ALIASES: ',T45,I7)
40    FORMAT (T2,'MCL MEMORY: ',T45,I7)
50    FORMAT (T2,'MCL STACK: ',T45,I7)
60    FORMAT (T2,'NUMBER OF MCL FILES: ',T45,I7)
70    FORMAT (T2,'MACRO NESTING LEVEL: ',T45,I7)
80    FORMAT (T2,'NUMBER OF NUMERICAL FLAGS: ',T45,I7)
90    FORMAT (T2,'NUMBER OF CHARACTER FLAGS: ',T45,I7)
91    FORMAT (T2,'LENGTH OF A CHARACTER FLAG: ',T45,I7)
      END
      SUBROUTINE LOCMAC(COMM,ISUCC)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C .......   MACRO - FINDER   .........
C  .... UPON RETURN :  ISUCC=-1  MEANS :  ILLEGAL MACRO SYNTAX
C  ....                ISUCC =0  MEANS :  MACRO NOT FOUND
C  ....                ISUCC = 1 MEANS :  MACRO FOUND AND POSITIONED
C
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER COMM*(*),MAC*10,STAR
      ISUCC=1
      REWIND IUMAC
    1 CONTINUE
      READ( IUMAC,5,END=2)STAR
       IF(STAR.EQ.'*')THEN
        READ( IUMAC,4,END=3)MAC
        CALL UPCASE(MAC)
         IF(MAC.EQ.COMM)THEN
          RETURN
         ELSE
          GO TO  1
         ENDIF
       ELSE
        GO TO  1
       ENDIF
    2 REWIND IUMAC
      ISUCC=0
      RETURN
    3 REWIND IUMAC
      ISUCC=-1
      RETURN
    4 FORMAT(A10)
    5 FORMAT(A1)
      END
      SUBROUTINE MACAC(COMAC)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  THIS SUBROUTINE DETERMINES THE MACHINE'S ACCURACY WHICH IS
C  STORED IN VARIABLE COMAC UPON RETURN
      ONE=1.
       DO 1 I=1,100
        COMAC=10.**(-I)
        X1=ONE+COMAC
        X2=ONE-COMAC
         IF(X1.EQ.ONE.OR.X2.EQ.ONE)THEN
          COMAC=10.**(-I+1)
          RETURN
         ENDIF
    1  CONTINUE
      END
C
C  ---------------------------------------------------------------------
C
 
      SUBROUTINE MATCH ( ARRAY, N, NAME, LE, IND )
      CHARACTER*(*) ARRAY(N), NAME
C
      IF (LE.GT.LEN(ARRAY(1))) THEN
         IND = -2
         RETURN
      END IF
      DO 10,I=1,N
        IF (ARRAY(I)(1:LE).EQ.NAME) THEN
           IF (I.EQ.N) THEN
              IND = I
           ELSE IF (LENGTH(ARRAY(I)).EQ.LE) THEN
              IND = I
           ELSE IF (ARRAY(I+1)(1:LE).EQ.NAME) THEN
              IND = -1
           ELSE
              IND = I
           END IF
           RETURN
        END IF
 10   CONTINUE
C
      IND = -2
      END
      SUBROUTINE MATCHA ( CMD, LE, ACTCOM, IAL )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      CHARACTER*(*) CMD, ACTCOM
C
      PARAMETER ( MAXALI = 20 )
      CHARACTER*5 ALIAS(MAXALI)
      CHARACTER*10 ALICOM(MAXALI)
      COMMON / ALICO1 / ALIAS, ALICOM
      COMMON / ALICO2 / NALIAS
C
      IAL = 0
      IF (LE.GT.LEN(ALIAS(1))) RETURN
C
      DO 10,I=1,NALIAS
         IF (CMD.EQ.ALIAS(I)) THEN
            IAL = I
         ACTCOM = ALICOM(I)
         RETURN
         END IF
10    CONTINUE
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE CDERI ( X, V0, N, DER, ERB, ERROR, IE )
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      COMMON/ACCURA/COMACC,COMTOL
C
      ONE = 1.
      C = 0.
      IF (ABS(V0).LE.COMACC) C = 1.
      F0 = V0+C
C
      POT = 10.**(-N)
      E = MAX(POT,COMACC)
C
      H = ABS(X*E)
      IF(ABS(X).LT.1.) H = E
C
      FP = ACSQ1(X+H)+C
      FM = ACSQ1(X-H)+C
      AK = LOG(ABS(FP/F0))/LOG(1+E)
      IF (AK*(AK-1.)*(AK-2.).EQ.0.) AK = AK+COMTOL
C
      E1 = SQRT(ABS(6./(AK-1)/(AK-2)*((FP-FM)/AK/F0/2-E)))
      E1 = MAX(E1,COMTOL)
      H1 = ABS(E1*X)
      IF (ABS(X).LT.1.) H1 = E1
      FP1 = ACSQ1(X+H1)+C
      FM1 = ACSQ1(X-H1)+C
      DER = (FP1-FM1)/(2*H1)
C
      D = (FP-FM)/(2*H)
      AK = X*D/F0
      IF (X .NE.0) THEN
         AMODEL = AK*F0/X
         ERROR = (AMODEL-DER)/MAX(ONE,MIN(ABS(AMODEL),ABS(DER)))
      ELSE
         ERROR = (D-DER)/MAX(ONE,MIN(ABS(DER),ABS(D)))
         ERROR = ERROR*H1**2/(H1**2-H**2)
      END IF
      ERROR = ABS(ERROR)
C
      IE = 0
      IF (ERROR.GT.ERB) IE = -1
C
      END
      SUBROUTINE MCLEND
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  DO WHATEVER IS APPROPRIATE UPON TERMINATION OF AN MCL PROGRAM.
C
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      PARAMETER ( MAXMFI = 20 )
      CHARACTER*30 MCLFIL(MAXMFI)
      INTEGER MCLUNI(MAXMFI)
      LOGICAL MCLNEW(MAXMFI)
      COMMON / MCLIO1 / MCLFIL
      COMMON / MCLIO2 / MCLUNI, MCLNEW, NUMFI
C
C  RESET THE INPUT UNIT.
      IUINP=IU31
C
C  CLOSE ALL FILES OPENED FOR MCL I/O OPERATIONS.
      DO 10,I=1,NUMFI
         IF (MCLNEW(I)) CLOSE(UNIT=MCLUNI(I))
10    CONTINUE
      NUMFI = 0
C
C  CLOSE THE MOC FILE.
      CLOSE (IU47)
C
      END
      SUBROUTINE USEMCL(COM)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  ROUTINE TO INTERFACE MERLIN WITH THE MCL OBJECT CODE.
C
      PARAMETER ( MXV = 200 )
      PARAMETER ( MEM = 2500 , LSTA = 500 , MR = 80 , MB = 20 ,MC= 90)
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/DIRCOM/DIRCOS(MXV)
      COMMON/UPDATE/IGRADU,IHESSU
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/MCLVAR/STACK(LSTA),CMEM(MEM),ISTACK,INFILE
      CHARACTER COM*10,MCLCOM(MC)*10,CHARA*61,COMW*10
      CHARACTER*(MR) BUFFER(MB),TEMPO,CARRY*22
      DIMENSION IERR(MXV)
      INTEGER DEBVAR
      COMMON/DEBUG/DEBVAR
      SAVE
      DATA ( MCLCOM(I),I=1,35 )  /
     >'BACKUP    ','DERIVA    ','PRINTO    ','USERSO    ',
     >'POP       ','PUSH      ','PUSHC     ',
     >'>         ','<         ','=         ','#         ',
     >'>=        ','<=        ',
     >'+         ','-         ','/         ','*         ','**       ',
     >'PAUSE     ','CONTINUE  ','FINISH    ','FACT      ','SWAP      ',
     >'>>        ','LENGTH    ','FLUSH     ','NOTE      ','OUT       ',
     >'IN        ','WMULTI    ','WPICK     ',
     >'WGRAD     ','WGODF     ','WDIS      ','WSUM      '    /
      DATA (  MCLCOM(I),I=36,MC  )    /
     >'LOCCHECK  ','GETCONT   ','WQUIT     ','NOT       ',
     >'OR        ','AND       ','XOR       '  ,
     >'ABS       ','LOG10     ','LOG       ','SIN       ','COS       ',
     >'TAN       ','ASIN      ','ACOS      ','ATAN      ','SINH      ',
     >'COSH      ','TANH      ','ASINH     ','ACOSH     ','ATANH     ',
     >'MIN       ','MAX       ','TRUNC     ','ROUND     ','MOD       ',
     >'MEAN      ','POPLV     ','POPCONT   ','X         ','R         ',
     >'L         ','STEP      ','FIX       ','GRAD      ','MARG      ',
     >'TCOUNT    ','PCOUNT    ','DIM       ','VALUE     ','PRECISION ',
     >'SQRT      ','CALLBY    ','PROCES    ','PANEL     ','WACCUM    ',
     >'TESTMOVE  ','MOVE      ','LINE      ','++        ','--        ',
     >'EXP       ','RAN       ','GNORM     '    /
C
      DATA TEMPO / ' '/
C
      DATA LENBUF , IBUF / 0 , 1 /
      DATA IECHO / 0 /
C
C   RECOGNIZE COMMAND
C
      IF ( IECHO .EQ. 1 ) THEN
         WRITE(*,*) '------   ',COM,'   ',COM
         WRITE(*,*) ' STACK LEVEL : ISTACK = ',ISTACK
      ENDIF
      JCOM=0
      DO 101 I=1,MC
      IF(COM(2:).EQ.MCLCOM(I)) THEN
         JCOM=I
         GO TO 102
      ENDIF
 101  CONTINUE
      WRITE(*,*)'>> ',COM,' << IS NOT A VALID INTERFACE COMMAND '
      CALL BATCH
      RETURN
 102  CONTINUE
C
C   SEND THE CONTROL TO APPROPRIATE POINT
C
      GOTO(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,
     >  23,24,25,26,27,28,29,30,31,34,38,39,
     >40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,
     >  59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,
     >76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,
     > 104) , JCOM
C  ---------------------------------------------------------------------
C
C          ...................          ....................   BACKUP
  1   CONTINUE
      IF (BACKUP.EQ.'NOBACK') IBACK = 1
      IF (BACKUP.EQ.'LASTBACK') IBACK = 2
      IF (BACKUP.EQ.'FULLBACK') IBACK = 3
      ISTACK = ISTACK + 1
      STACK(ISTACK) = IBACK
      RETURN
C          ...................          ....................   DERIVA
 2    CONTINUE
      IF (DERIVA.EQ.'ANAL') IDERV = 1
      IF (DERIVA.EQ.'NUMER') IDERV = 2
      ISTACK = ISTACK + 1
      STACK(ISTACK) = IDERV
      RETURN
C          ...................          ....................   PRINTO
 3    CONTINUE
      IF (PRINTO.EQ.'NOPRINT') IPRIN = 1
      IF (PRINTO.EQ.'HALFPRINT') IPRIN = 2
      IF (PRINTO.EQ.'FULLPRINT') IPRIN = 3
      ISTACK = ISTACK + 1
      STACK(ISTACK) = IPRIN
      RETURN
C          ...................          ....................   USERSO
C  SINCE THE USERSO MODE DOES NOT EXIST IN MERLIN-1.0 PLUS, THIS
C  INSTRUCTION IS MAINTAINED HERE UNTIL MCL-1.0 PLUS REACHES
C  ITS FINAL DEVELOPMENT STAGE.
 4    CONTINUE
      ISTACK = ISTACK + 1
      STACK(ISTACK) = 2
      RETURN
C          ...................          ....................   POP
5     CONTINUE
      READ(INFILE,*,IOSTAT=III)NUMBER
      CMEM(NUMBER)=STACK(ISTACK)
      ISTACK=ISTACK-1
      RETURN
C          ...................          ....................   PUSH
6     CONTINUE
      READ(INFILE,*,IOSTAT=III)NUMBER
      ISTACK=ISTACK+1
      IF(NUMBER.LE.0) THEN
      CALL STOPER (  'MCLCOM BUG 1 AT PUSH')
      ENDIF
      STACK(ISTACK)=CMEM(NUMBER)
      RETURN
C          ...................          ....................   PUSHC
7     CONTINUE
      READ(INFILE,*,IOSTAT=III)XNUM
      ISTACK=ISTACK+1
      STACK(ISTACK)=XNUM
      RETURN
C          ...................          ....................   >
8     CONTINUE
      CALL TOPTWO(UPS,DOS)
      IF(DOS.GT.UPS) STACK(ISTACK)=1.
      RETURN
C          ...................          ....................   <
9     CONTINUE
      CALL TOPTWO(UPS,DOS)
      IF(DOS.LT.UPS) STACK(ISTACK)=1.
      RETURN
C          ...................          ....................   =
10    CONTINUE
      CALL TOPTWO(UPS,DOS)
      IF(DOS.EQ.UPS) STACK(ISTACK)=1.
      RETURN
C          ...................          ....................   #
11    CONTINUE
      CALL TOPTWO(UPS,DOS)
      IF(DOS.NE.UPS) STACK(ISTACK)=1.
      RETURN
C          ...................          ....................   >=
12    CONTINUE
      CALL TOPTWO(UPS,DOS)
      IF(DOS.GE.UPS) STACK(ISTACK)=1.
      RETURN
C          ...................          ....................   <=
13    CONTINUE
      CALL TOPTWO(UPS,DOS)
      IF(DOS.LE.UPS) STACK(ISTACK)=1.
      RETURN
C          ...................          ....................   +
14    CONTINUE
      CALL TOPTWO(UPS,DOS)
      STACK(ISTACK)=DOS + UPS
      RETURN
C          ...................          ....................   -
15    CONTINUE
      CALL TOPTWO(UPS,DOS)
      STACK(ISTACK)=DOS - UPS
      RETURN
C          ...................          ....................   /
16    CONTINUE
      CALL TOPTWO(UPS,DOS)
      STACK(ISTACK)=DOS / UPS
      RETURN
C          ...................          ....................   *
17    CONTINUE
      CALL TOPTWO(UPS,DOS)
      STACK(ISTACK)=DOS * UPS
      RETURN
C          ...................          ....................   **
18    CONTINUE
      CALL TOPTWO(UPS,DOS)
      IF (DOS .GE. 0.)  THEN
          STACK(ISTACK) = DOS**UPS
      ELSE
          CHECK = ABS(UPS - NINT(UPS))
          IF (CHECK .LT. ABS(UPS)*COMTOL )   THEN
              STACK(ISTACK) = DOS**NINT(UPS)
          ELSE
              CALL STOPER('USEMCL ERROR IN: EXP')
          ENDIF
      ENDIF
      RETURN
C          ...................          ....................   PAUSE
 19   WRITE(IUOUT,*)' ...  PAUSE  ... '
      PAUSE
      RETURN
C          ...................          ....................   CONTINUE
 20   RETURN
C          ...................          ....................   FINISH
 21   CONTINUE
      CALL MCLEND
      RETURN
C          ...................          ....................   FACT
 22   CONTINUE
      STACK(ISTACK) = FACT(NINT(STACK(ISTACK)))
      RETURN
C          ...................          ....................   SWAP
 23   IUINP =IU47 + IU37 - IUINP
      INFILE= IUINP
      RETURN
C          ...................          ....................   >>
 24   RETURN
C          ...................          ....................   LENGTH
 25   READ(INFILE,*)MEMCHE
      IF(MEMCHE .GT. MEM ) THEN
      WRITE(IUOUT,*) 'ALLOCATED MEMORY : ',MEM
      WRITE(IUOUT,*) 'REQUIRED  MEMORY : ',MEMCHE
      WRITE(IUOUT,*) '==>> INCREASE MEMORY SIZE TO RUN THIS MCL OBJECT'
      STOP
      ENDIF
      RETURN
C          ...................          ....................   FLUSH
 26   TEMPO(LENBUF+1 : ) = ' '
      BUFFER(IBUF) = TEMPO
      TEMPO = ' '
      CALL DETUNT ( IUOUT, IOUNIT )
      CALL FLUSHB(BUFFER,IBUF,IOUNIT)
      LENBUF = 0
      RETURN
C          ...................          ....................   NOTE
 27   CHARA = ' '
      READ(INFILE,*) INTE
      INTEP = INTE + 1
      READ(INFILE,'(A)')CHARA(1:INTEP)
      CALL DETUNT (IUOUT, IOUN )
      CALL TREBUF(BUFFER,IBUF,LENBUF,CHARA,INTEP,TEMPO,IOUN )
      RETURN
C          ...................          ....................   OUT
 28   XX= STACK(ISTACK)
      ISTACK=ISTACK - 1
      CALL STORIC(XX,CARRY,LLL)
      CALL DETUNT (IUOUT, IOUT )
      CALL TREBUF(BUFFER,IBUF,LENBUF,CARRY,LLL,TEMPO,IOUT)
      RETURN
C          ...................          ....................   IN
 29   READ(INFILE,*)INTE
      CALL DETUNT ( IU31, IUT )
      ILO = ISTACK + 1
      ISTACK = ISTACK + INTE
      READ ( IUT , * , IOSTAT = III ) (STACK(KK),KK=ISTACK,ILO,-1)
      IF (III.NE.0) THEN
      REWIND IUT
      CALL STOPER (  'ERROR IN "IN" ')
      ENDIF
      RETURN
C          ...................          ....................   WMULTI
 30   CONTINUE
      READ(INFILE,*)ICAIND
      READ(INFILE,'(A)') COMW
      READ(INFILE,*) NOCHAN
      IF(NOCHAN.EQ.0) THEN
      REWIND IU37
      IF (PANEL.EQ.'PANELOFF') IPANEL = 1
      IF (PANEL.EQ.'PANELON') IPANEL = 2
      IPA=IPANEL
      PANEL='PANELOFF'
      WRITE(IU37,'(A10)') COMW
      IF(IPA.EQ.2) WRITE(IU37,'(A7)') 'PANELON'
      WRITE(IU37,'(A5)') '$SWAP'
      REWIND IU37
      RETURN
      ENDIF
      REWIND IU37
      IF(PANEL .EQ. 'PANELOFF') WRITE(IU37,'(A7)')'PANELON'
      WRITE(IU37,'(A10)') COMW
C
C  SEE IF A CANCEL BUTTON IS DEFINED.
C
      IBOT= ISTACK + 1 -2*NOCHAN
      IBOTP = IBOT +1
      DO 491 I = ISTACK-1,IBOT+2,-2
      IF( STACK(I) .EQ. REAL(ICAIND) ) THEN
      VC=STACK(I+1)
      STACK(I+1)=STACK(IBOTP)
      STACK(I)=STACK(IBOT)
      STACK(IBOTP)=VC
      STACK(IBOT)=ICAIND
      GO TO 492
      ENDIF
 491  CONTINUE
 492  CONTINUE
      DO 130 I= 1 , NOCHAN
      VAL1 = STACK(ISTACK)
      ISTACK = ISTACK - 1
      IND1 = NINT(STACK(ISTACK))
      ISTACK = ISTACK - 1
      IF(I.LE.NOCHAN - 1) THEN
       WRITE(IU37,*) IND1 , VAL1 , ICAIND,' 2'
      ENDIF
      IF(I.EQ.NOCHAN ) WRITE(IU37,*) IND1 , VAL1
 130  CONTINUE
      IF(PANEL .EQ. 'PANELOFF') WRITE(IU37,'(A8)') 'PANELOFF'
      WRITE(IU37,'(A5)') '$SWAP'
      REWIND IU37
      RETURN
C          ...................          ....................   WPICK
 31   CONTINUE
      READ(INFILE,'(A)') COMW
      REWIND IU37
      WRITE(IU37,'(A7)') 'INSPECT'
      WRITE(IU37,'(A10)') COMW
      NUM=NINT(STACK(ISTACK)) - 1
      ISTACK=ISTACK - 1
      DO 131 I= 1, NUM
 131  WRITE(IU37,'(A1)') '$'
      IF(COMW.NE.'DATA')WRITE(IU37,'(A1)') '0'
      WRITE(IU37,'(A4)') 'PICK'
      WRITE(IU37,'(A5)') '$SWAP'
      REWIND IU37
      RETURN
C          ...................          ....................   WGRAD
 34   CONTINUE
      REWIND IU37
      WRITE(IU37,'(A9)') 'GRADCHECK'
      WRITE(IU37,*) STACK(ISTACK)
      ISTACK= ISTACK - 1
       WRITE(IU37,'(A5)') '$SWAP'
      REWIND IU37
      RETURN
C          ...................          ....................   WGODF
 38   CONTINUE
      READ(INFILE,*)NUM
      REWIND IU37
      DO 138 I=1,NUM
      WRITE(IU37,'(A9)') 'GODFATHER'
      IND1 = NINT(STACK(ISTACK))
      ISTACK = ISTACK - 1
      READ(INFILE,'(A)') COMW
      WRITE(IU37,'(I5,2X,A10)') IND1,COMW
 138  CONTINUE
      WRITE(IU37,'(A5)') '$SWAP'
      REWIND IU37
      RETURN
C          ...................          ....................   WDIS
 39   CONTINUE
      REWIND IU37
      WRITE(IU37,'(A7)') 'GRADDIS'
      IF(DERIVA.EQ.'NUMER') THEN
      WRITE(IU37,*) STACK(ISTACK)
      ISTACK = ISTACK - 1
      ENDIF
      WRITE(IU37,'(A5)') '$SWAP'
      REWIND IU37
      RETURN
C........................................................WSUM
 40   CONTINUE
C
C   CHECK IF THERE ARE ENOUGH NUMBERS IN THE STACK
C
      READ(INFILE,*,IOSTAT=III)NUMBER
      IF (ISTACK .LT. 2*NUMBER) THEN
      CALL STOPER (  'ERROR IN MCLCOM - WSUM')
      ENDIF
C
      XSUM=0.
      DO 201 II=0,NUMBER - 1
      XSUM = XSUM + STACK(ISTACK - II) * STACK(ISTACK - II - NUMBER)
 201  CONTINUE
      ISTACK = ISTACK + 1 - 2*NUMBER
      STACK(ISTACK) = XSUM
      RETURN
C........................................................LOCCHECK
 41   CONTINUE
      IF( (STACK(ISTACK).GE.STACK(ISTACK-2)).AND.
     >  (STACK(ISTACK) .LE. STACK(ISTACK-1))) THEN
      ISTACK=ISTACK-2
      STACK(ISTACK)=INT(STACK(ISTACK+2))
      RETURN
       ELSE
      CALL STOPER (  ' ARRAY-BOUNDARY VIOLATION.')
      ENDIF
C........................................................GETCONT
 42   CONTINUE
      II=STACK(ISTACK)
      STACK(ISTACK)=CMEM(II)
      RETURN
C........................................................WQUIT
 43   CONTINUE
      REWIND IU37
      WRITE(IU37,'(A4)') 'QUIT'
      WRITE(IU37,*) STACK( ISTACK )
      ISTACK = ISTACK - 1
      WRITE(IU37,'(A5)') '$SWAP'
      REWIND IU37
      RETURN
C........................................................NOT
 44   CONTINUE
      IF (STACK(ISTACK).EQ.0.) THEN
      STACK(ISTACK) = 1.
      ELSE
      STACK(ISTACK) = 0.
      ENDIF
      RETURN
C........................................................OR
 45   CONTINUE
      IF (ISTACK .GE. 2) THEN
      IB = NINT(STACK(ISTACK))
      ISTACK = ISTACK -1
      IA = NINT(STACK(ISTACK))
      IF ( IB .NE. 0 ) THEN
        STACK(ISTACK) = 1
      ELSE
        STACK(ISTACK) = IA
      ENDIF
      ELSE
      CALL STOPER( ' ERROR IN OR ')
      ENDIF
      RETURN
C........................................................AND
 46   CONTINUE
      IF (ISTACK .GE. 2 ) THEN
      IB = NINT(STACK(ISTACK))
      ISTACK = ISTACK - 1
      IA = NINT(STACK(ISTACK))
      IF (IB .NE. 0 ) THEN
        STACK(ISTACK) = IA
      ELSE
        STACK(ISTACK) = 0
      ENDIF
      ELSE
      CALL STOPER(' ERROR IN AND ' )
      ENDIF
      RETURN
C........................................................XOR
 47   CONTINUE
      IF ( ISTACK .GE. 2 ) THEN
        IB=NINT(STACK(ISTACK))
        ISTACK = ISTACK - 1
        IA = NINT(STACK(ISTACK))
        IF ( IA .NE. 0 .AND. IB .EQ. 0 ) THEN
          STACK(ISTACK)= IA
        ELSEIF (IA .EQ. 0  .AND. IB .NE. 0 ) THEN
          STACK (ISTACK) = IB
        ELSE
          STACK(ISTACK) = 0
        ENDIF
      ELSE
      CALL STOPER ('ERROR IN XOR ')
      ENDIF
      RETURN
C          ...................          ....................   ABS
 48   STACK(ISTACK)=ABS(STACK(ISTACK))
      RETURN
C          ...................          ....................   LOG10
 49   STACK(ISTACK) = LOG10(STACK(ISTACK))
      RETURN
C          ...................          ....................   LOG
 50   STACK(ISTACK) = LOG(STACK(ISTACK))
      RETURN
C          ...................          ....................   SIN
 51   STACK(ISTACK) = SIN(STACK(ISTACK))
      RETURN
C          ...................          ....................   COS
 52   STACK(ISTACK) = COS(STACK(ISTACK))
      RETURN
C          ...................          ....................   TAN
 53   STACK(ISTACK) = TAN(STACK(ISTACK))
      RETURN
C          ...................          ....................   ASIN
 54   STACK(ISTACK) = ASIN(STACK(ISTACK))
      RETURN
C          ...................          ....................   ACOS
 55   STACK(ISTACK) = ACOS(STACK(ISTACK))
      RETURN
C          ...................          ....................   ATAN
 56   STACK(ISTACK) = ATAN(STACK(ISTACK))
      RETURN
C          ...................          ....................   SINH
 57   STACK(ISTACK) = SINH(STACK(ISTACK))
      RETURN
C          ...................          ....................   COSH
 58   STACK(ISTACK) = COSH(STACK(ISTACK))
      RETURN
C          ...................          ....................   TANH
 59   STACK(ISTACK) = TANH(STACK(ISTACK))
      RETURN
C          ...................          ....................   ASINH
 60   STACK(ISTACK) = ASINH0(STACK(ISTACK))
      RETURN
C          ...................          ....................   ACOSH
 61   STACK(ISTACK) = ACOSH0(STACK(ISTACK))
      RETURN
C          ...................          ....................   ATANH
 62   STACK(ISTACK) = ATANH0(STACK(ISTACK))
      RETURN
C          ...................          ....................   MIN
 63   READ(INFILE,*) NUMBER
      X1=STACK(ISTACK)
      DO 203 II = 1 , NUMBER - 1
      ISTACK = ISTACK - 1
 203  IF(X1.GT.STACK(ISTACK) ) X1 = STACK(ISTACK)
      STACK(ISTACK) = X1
      RETURN
C          ...................          ....................   MAX
 64   READ(INFILE,*) NUMBER
      X1=STACK(ISTACK)
      DO 204 II=1,NUMBER-1
      ISTACK=ISTACK - 1
 204  IF(X1.LT.STACK(ISTACK)) X1 = STACK(ISTACK)
      STACK(ISTACK) = X1
      RETURN
C          ...................          ....................   TRUNC
 65   STACK(ISTACK) = AINT(STACK(ISTACK))
      RETURN
C          ...................          ....................   ROUND
 66   STACK(ISTACK) = ANINT(STACK(ISTACK))
      RETURN
C          ...................          ....................   MOD
 67   X2=STACK(ISTACK)
      ISTACK = ISTACK - 1
      X1=STACK(ISTACK)
      STACK(ISTACK) = X1-INT(X1/X2)*X2
      RETURN
C          ...................          ....................   MEAN
 68   READ(INFILE,*) NUMBER
      XX=0.
      DO 205 I=1,NUMBER
      XX=XX+STACK(ISTACK)
 205  ISTACK= ISTACK - 1
      ISTACK= ISTACK + 1
      STACK(ISTACK) = XX/NUMBER
      RETURN
C          ...................          ....................   POPLV
 69   CONTINUE
      CMEM( NINT(STACK( ISTACK ))  ) = STACK( ISTACK - 1 )
      ISTACK = ISTACK - 2
      RETURN
C          ...................          ....................   POPCONT
 70   CONTINUE
      CMEM( NINT(STACK ( ISTACK -1 )) ) = STACK( ISTACK )
      ISTACK = ISTACK - 2
      RETURN
C          ...................          ....................   X
 71   CONTINUE
      STACK( ISTACK ) = POINT( NINT(STACK(ISTACK)) )
      RETURN
C          ...................          ....................   R
 72   CONTINUE
      STACK ( ISTACK ) = XRL ( NINT(STACK( ISTACK )) )
      RETURN
C          ...................          ....................   L
 73   CONTINUE
      STACK (ISTACK) = XLL ( NINT(STACK (ISTACK)) )
      RETURN
C          ...................          ....................   STEP
 74   CONTINUE
      STACK (ISTACK) = STEP ( NINT(STACK (ISTACK)) )
      RETURN
C          ...................          ....................   FIX
 75   CONTINUE
      STACK (ISTACK) = IXAT ( NINT(STACK (ISTACK)) )
      RETURN
C          ...................          ....................   GRAD
 76   CONTINUE
      ZERO = 0.
      CALL DERMER (NOD,POINT,VAL,DIRCOS,NOGCAL,ZERO,IERR)
      STACK (ISTACK) = DIRCOS( NINT(STACK (ISTACK)) )
      RETURN
C          ...................          ....................   MARG
 77   CONTINUE
      STACK (ISTACK) = MARG( NINT(STACK (ISTACK)) )
      RETURN
C          ...................          ....................   TCOUNT
 78   CONTINUE
      ISTACK = ISTACK + 1
      STACK (ISTACK) = ITNOCS
      RETURN
C          ...................          ....................   PCOUNT
 79   CONTINUE
      ISTACK = ISTACK + 1
      STACK (ISTACK) = IPARTC
      RETURN
C          ...................          ....................   DIM
 80   CONTINUE
      ISTACK = ISTACK + 1
      STACK(ISTACK) = NOD
      RETURN
C          ...................          ....................   VALUE
 81   CONTINUE
      ISTACK = ISTACK + 1
      STACK (ISTACK) = VAL
      RETURN
C          ...................          ....................   PRECISION
 82   CONTINUE
      ISTACK=ISTACK + 1
      STACK (ISTACK) = COMACC
      RETURN
C          ...................          ....................   SQRT
 83   CONTINUE
      IF( STACK (ISTACK) .LT. 0. ) THEN
      CALL STOPER('ERROR IN SQRT - NEG. ARGUM')
      ENDIF
      STACK (ISTACK) = SQRT( STACK (ISTACK) )
      RETURN
C          ...................          ....................   CALLBY
 84   CONTINUE
      IF (CALLBY.EQ.'NAME') ICALL = 1
      IF (CALLBY.EQ.'INDEX') ICALL = 2
      ISTACK = ISTACK + 1
      STACK(ISTACK) = ICALL
      RETURN
C          ...................          ....................   PROCES
 85   CONTINUE
      IF (PROCES.EQ.'BATCH') IPROC = 1
      IF (PROCES.EQ.'IAF') IPROC = 2
      ISTACK = ISTACK + 1
      STACK(ISTACK) = IPROC
      RETURN
C          ...................          ....................   PANEL
 86   CONTINUE
      IF (PANEL.EQ.'PANELOFF') IPANEL = 1
      IF (PANEL.EQ.'PANELON') IPANEL = 2
      ISTACK = ISTACK + 1
      STACK(ISTACK) = IPANEL
      RETURN
C          ...................          ....................   WACCUM
 87   CONTINUE
      IF ( ISTACK -6 .LT. 0 ) THEN
      CALL STOPER (  ' ERROR 1 IN WACCUM ')
      ENDIF
      REWIND IU37
      WRITE(IU37,'(A5)' ) 'ACCUM'
      I1=0
      I2=0
      I3=0
      DO 301 I = ISTACK - 1 , ISTACK - 5 , -2
      IF ( STACK(I) .EQ. 1. ) I1=I
      IF ( STACK(I) .EQ. 2. ) I2=I
      IF ( STACK(I) .EQ. 3. ) I3=I
 301  CONTINUE
      IF( I1*I2*I3 .EQ. 0 ) THEN
       CALL STOPER (  ' ERROR 2 IN WACCUM ')
      ENDIF
      WRITE(IU37,*)STACK(I1+1),STACK(I2+1),STACK(I3+1)
      WRITE(IU37,'(A5)' ) '$SWAP'
      REWIND IU37
      ISTACK = ISTACK - 6
      RETURN
C          ...................          ....................   TESTMOVE
 88   CONTINUE
      READ(INFILE,*,IOSTAT = III ) NUMBER
      IF ( III .NE. 0 ) THEN
       CALL STOPER (  ' ERROR 1 IN TESTMOVE ')
      ENDIF
      IF ( NINT(STACK (ISTACK)) .EQ. 1 ) THEN
       ISTACK=ISTACK-1
       GO TO 389
      ELSEIF(NINT(STACK (ISTACK)) .EQ. 0 ) THEN
      ISTACK=ISTACK-1
      RETURN
      ELSE
      CALL STOPER (  ' ERROR 2 IN TESTMOVE ')
      ENDIF
C          ...................          ....................   MOVE
 89   CONTINUE
      READ(INFILE,*,IOSTAT = III ) NUMBER
      IF ( III .NE. 0 ) CALL STOPER (  ' ERROR 1 IN MOVE ')
 389  CONTINUE
      IF(NUMBER.EQ.0)CALL STOPER(' ERROR 2 IN MOVE ')
      IF (NUMBER .GT. 0 )  THEN
      CALL FORWAR(NUMBER + 1 ,INFILE )
      ELSE
      CALL BACWAR(NUMBER, INFILE )
      ENDIF
      RETURN
C          ...................          ....................   LINE
 90   CONTINUE
      READ(INFILE,*,IOSTAT = III ) NUMBER
      DEBVAR=NUMBER
      IF(III .NE. 0 ) CALL STOPER (  ' ERROR IN LINE ')
      RETURN
C          ...................          ....................   ++
 91   CONTINUE
      STACK ( ISTACK ) = STACK ( ISTACK ) + 1.
      RETURN
C          ...................          ....................   --
 92   CONTINUE
      STACK ( ISTACK ) = STACK ( ISTACK ) - 1.
      RETURN
C          ...................          ....................   EXP
 93   CONTINUE
      STACK ( ISTACK ) = EXP(STACK(ISTACK))
      RETURN
C          ...................          ....................   RAN
 94   CONTINUE
      STACK ( ISTACK ) = RANM()
      RETURN
C          ...................          ....................   GNORM
104   CONTINUE
      INO = NINT(STACK(ISTACK))
      ZERO = 0.
      CALL DERMER (NOD,POINT,VAL,DIRCOS,NOGCAL,ZERO,IERR)
      IF (INO.EQ.1) THEN
         GSUM = 0.
         DO 105,I=1,NOD
105         GSUM = GSUM+ABS(DIRCOS(I))
      ELSE IF (INO.EQ.2) THEN
         GSUM = 0.
         DO 106,I=1,NOD
106         GSUM = GSUM+DIRCOS(I)**2
         GSUM = SQRT(GSUM)
      ELSE
         CALL STOPER ('GNORM NOT 1 OR 2')
      END IF
      STACK(ISTACK) = GSUM
      RETURN
      END
C
C
      SUBROUTINE FORWAR(NUMBER,INFILE)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  INPUT: NUMBER , INFILE
C  THIS ROUTINE SKIPS ( NUMBER - 2 ) RECORDS OF THE FILE CONNECTED
C  TO THE UNIT SPECIFIED BY THE  INFILE   INTEGER VARIABLE.
C
      CHARACTER CHA*1,FILEN*15
      DO 1 I=1,NUMBER-2
 1    READ(INFILE,'(A)',END=100) CHA
      RETURN
 100  REWIND INFILE
      INQUIRE(UNIT=INFILE,NAME=FILEN)
      WRITE(*,*) 'ERROR IN SUBROUTINE FORWAR.'
      WRITE(*,*) 'END OF FILE ENCOUNTERED IN FILE : '
      CALL STOPER (  FILEN  )
      END
C
C
      SUBROUTINE BACWAR(NUMBER,INFILE)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  INPUT: NUMBER , INFILE
C  THIS ROUTINE BACKSPACES  IABS(NUMBER) + 2  TIMES, THE FILE
C  CONNECTED TO THE UNIT SPECIFIED BY THE  INFILE PARAMETER
C
      DO 1 I=1,IABS(NUMBER)+2
 1    BACKSPACE INFILE
      END
      SUBROUTINE STORIC ( X , CH , L )
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  THIS ROUTINE STORES A REAL NUMBER X , IN A CHARACTER VARIABLE CH,
C  IN ITS FIRST L   POSITIONS , THE LAST ONE ALWAYS BEING A BLANK.
C  CH AND L ARE OUTPUT, X IS INPUT
C
      COMMON/ACCURA/COMACC,COMTOL
      INTEGER WORDIG
      CHARACTER FORMAE*11 , FORMAF*11 , C*64 , CH*64
      DATA FORMAF / '(T1,F67.90)'     /
      DATA FORMAE/  '(T1,E67.90)'    /
      CH=' '
C
C  IF  X = 0 , TREAT IT SEPARATELY, TO OBTAIN A CLEAR FORMAT
C
      IF ( X .EQ. 0. )  THEN
        CH(1:3) = '.0 '
        L=3
        RETURN
      ENDIF
C
C  FIND OUT DECIMAL DIGITS SUPPORTED BY THE MACHINE, IN ONE WORD
C
      WORDIG = -NINT(LOG10(COMACC))
      C = ' '
C
C   SET THE MAXIMUM ALLOWED FORMAT WIDTH, FOR  F  AND  E  DESCRIPTORS
C
      WRITE(FORMAF(6:7),'(I2)') WORDIG + 2
      WRITE(FORMAE(6:7),'(I2)') WORDIG + 7
      WRITE(FORMAE(9:10),'(I2)') WORDIG
      AX = ABS(X)
C
C   CHECK IN WHICH RANGE THE  ABS(X) FALLS IN.
C
      IF ( AX .LE. COMACC .OR. AX .GE. 1./COMACC ) THEN
C
C   IN THIS CASE USE THE E-FORMAT
C
        WRITE( C(:WORDIG + 7) , FORMAE ) X
C
C  DELETE INSIGNIFICANT ZEROES
C
      JDOT = INDEX( C , '.')
      JCHE = JDOT +WORDIG
      JF = JCHE
      DO 11 I = JCHE , JDOT + 1 , -1
      IF ( C(I:I) .NE. '0' ) THEN
      JF = I
      GO TO 12
      ENDIF
 11   CONTINUE
 12   CONTINUE
      CH(1:JF) = C(1:JF)
      CH(JF+1:) = C(JCHE+1:WORDIG+7)
      L = WORDIG + JF + 8 -JCHE
      CH(L:L)=' '
      RETURN
      ELSE
C
C   IN THIS CASE USE AN F-FORMAT TO IMPROVE READABILITY
C
        INTDIG = LOG10(AX) + 1
C
C   INTIG , IS THE NUMBER OF DIGITS TO APPEAR TO THE LEFT OF THE
C   DECIMAL PERIOD.
C
        IF ( INTDIG .GT. 0 ) THEN
          WRITE(FORMAF(9:10) , '(I2)') WORDIG - INTDIG
        ELSE
          WRITE(FORMAF(9:10) , '(I2)' ) WORDIG
        ENDIF
        WRITE( C(:WORDIG + 2 ) , FORMAF ) X
      JDOT = INDEX(C,'.')
      JF = WORDIG + 2
      DO 13 I = WORDIG + 2 ,JDOT + 1, -1
      IF ( C(I:I) .NE. '0' .AND. C(I:I) .NE. ' ') THEN
      JF = I
      GO TO 14
      ENDIF
 13   CONTINUE
      JF=JDOT
 14   CONTINUE
      CH(1:JF) = C(1:JF)
      L = JF + 1
      CH(L:L)=' '
      RETURN
      ENDIF
      END
C
C
      SUBROUTINE TREBUF(BUFFER,IBUF,LENBUF,CARRY,L,TEMPO,IUOUT)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  THIS ROUTINE TREATS THE BUFFER. ADDS TO THE BUFFER THE CONTENTS OF
C  CARRY(1:L) IN THE CURRENT RECORD NUMBER IBUF,ONLY IF IT FITS.
C  IF IT DOES NOT FIT, FILLS THE REST OF THE RECORD WITH BLANKS AND
C  STORES CARRY(1:L) IN THE NEXT BUFFER RECORD. IF IT HAPPENS THAT THE
C  BUFFER IS FILLED COMPLETELY THE BUFFER IS AUTOMATICALLY FLUSHED TO
C  UNIT IUOUT, AND CARRY(1:L) IS PLACED AT THE TOP OF THE BUFFER.
C  TEMPO IS THE CURRENT BUFFER RECORD.
C
      PARAMETER (MR = 80 , MB = 20 )
      CHARACTER*(*) BUFFER(MB),TEMPO,CARRY
      LOGICAL CHECKR, CHECKB
C
C  CHECK IF TEMPO CAN HOLD CARRY
C
      CHECKR = ( LENBUF + L ) .LE. MR
C
C  IF SO, ADD CARRY TO TEMPO AND UPDATE THE FILLED TEMPO POSITIONS
C
      IF (CHECKR) THEN
         TEMPO(LENBUF + 1 : LENBUF + L ) = CARRY(1:L)
         LENBUF = LENBUF + L
C
C  SINCE WE USE LENBUF + 1 ABOVE , PROTECT THE CASE WHEN  THE VALUE OF
C  LENBUF = MR (THE MAXIMUM RECORD LENGTH), BY DECREMENTING LENBUF BY 1
C  THIS DOES NOT CREATE OTHER PROBLEMS , BECAUSE L MUST BE .GE. 2
C
         IF (LENBUF .EQ. MR ) LENBUF = LENBUF-1
         RETURN
      ENDIF
C
C  SINCE TEMPO CAN NOT HOLD CARRY(1:L) FILL THE REST TEMPO-POSITIONS
C  WITH BLANKS. FILL THE CURRENT BUFFER RECORD WITH TEMPO.
C
      TEMPO(LENBUF + 1 : MR) = ' '
      BUFFER(IBUF) = TEMPO
      TEMPO(1:)=' '
C
C  CHECK IF THIS IS THE LAST RECORD OF THE BUFFER
C
      CHECKB = IBUF .EQ. MB
      IF(CHECKB) THEN
C
C  FLUSH THE WHOLE BUFFER TO UNIT IUOUT, UPDATE THE BUFFER RECORD TO 1
C  ADD CARRY TO TEMPO AND UPDATE ITS LENGTH BY SETTING IT EQUAL TO L.
C
         CALL FLUSHB(BUFFER,IBUF,IUOUT)
         TEMPO(1:L)=CARRY(1:L)
         TEMPO(L+1:) = ' '
         IBUF=1
         LENBUF=L
         RETURN
      ENDIF
C
C  IF NOT THE LAST RECORD PROCESS NORMALY:
C
      IBUF = IBUF + 1
      TEMPO(1:L) = CARRY(1:L)
      TEMPO(L+1:) = ' '
      LENBUF = L
C
      END
C
C
      SUBROUTINE FLUSHB(BUFFER,IBUF,IUOUT)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  INPUT: BUFFER , IBUF , IUOUT
C  OUTPUT: IBUF
C  THIS ROUTINE FLUSHES THE CONTENTS OF THE CHARACTER ARRAY BUFFER
C  TO THE FILE CONNECTED TO THE UNIT IUOUT , AND RESETS THE BUFFER
C  LENGTH TO 1 ( ONE ).
C
      PARAMETER (MR = 80 , MB = 20 )
      CHARACTER*(MR) BUFFER(MB)
      DO 1 I=1,IBUF
         LE = LENGTH(BUFFER(I))
         IF (LE.EQ.0) LE = 1
         WRITE(IUOUT,'(T2,A)')BUFFER(I)(1:LE)
  1   CONTINUE
      IBUF=1
      END
C
C
      FUNCTION ACOSH0(X)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  CALCULATES THE HYPERBOLIC  ARC-COSINE
C  (THE POSITIVE VALUE IS ALWAYS RETURNED)
C
      IF(X .GE. 1 ) THEN
      ACOSH0 = LOG( X + SQRT(X*X - 1.) )
      ELSE
      WRITE(*,*)'INVALID ARGUMENT VALUE = ',X,' IN ACOSH '
       STOP  ' RUN - TIME ERROR. '
      ENDIF
      END
C
C
      FUNCTION ASINH0(X)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   CALCULATES THE  HYPERBOLIC ARC-SINE OF  X
C
      ASINH0 = LOG( X + SQRT( X*X + 1.) )
      END
C
C
      FUNCTION ATANH0(X)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  CALCULATES THE HYPERBOLIC ARC-TANGENT OF X
C
      IF ( ABS(X) .LT. 1. ) THEN
         ATANH0 = .5*LOG( (1.+X)/(1.-X) )
      ELSE
      WRITE(*,*)'INVALID ARGUMENT VALUE = ',X,' IN ATANH'
      STOP' RUN - TIME ERROR.'
      ENDIF
      END
C
C
      SUBROUTINE STOPER(CHARI)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  INPUT: CHARI
C  ROUTINE ISSUING INFORMATIVE MESSAGES UPON ERRORS.
C  IT IS A DIAGNOSTIC / DEBBUGING AID.
C
      COMMON/DEBUG/DEBVAR
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*(*) CHARI
      INTEGER DEBVAR
      WRITE(IUOUT,'(/)')
      WRITE(IUOUT,'(T2,A)')CHARI
      WRITE(IUOUT,*)'ERROR IN LINE : ',DEBVAR,' OF THE SOURCE PROGRAM'
      WRITE(IUOUT,'(/)' )
      IF(PROCES .EQ. 'BATCH') THEN
      CALL BATCH
      ELSE
      IUINP = IU31
      RETURN
      ENDIF
      END
C
C
      SUBROUTINE TOPTWO ( UPS, DOS )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( MEM = 2500, LSTA = 500 )
      COMMON/MCLVAR/STACK(LSTA),CMEM(MEM),ISTACK,INFILE
C
      UPS = STACK(ISTACK)
      ISTACK = ISTACK-1
      DOS = STACK(ISTACK)
      STACK(ISTACK) = 0.
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE MCOPY ( IU1, IU2, LINE, IS, IE, NIT )
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*(*) LINE
      INTEGER IS(*), IE(*)
C
      CHARACTER LI*80, NEW*80, ARG*80, NUM*80
      INTEGER BS(40), BE(40)
C
C  REWIND THE OUTPUT UNIT (IU2) ONLY. THE INPUT UNIT (IU1)
C  IS ASSUMED TO BE CORRECTLY POSITIONED.
      REWIND IU2
C
 1000 READ (IU1,10,END=100) LI
C
C  A STAR INDICATES WE ARE IN A MULTI-MACRO FILE (SUCH AS THE
C  MERLIN FILE: MACROF) AND THE END OF THE MACRO HAS BEEN REACHED.
      IF (LI.EQ.'*') GOTO 100
C
      CALL BCR(LI,NBR,BS,BE)
      NN = 0
      NL = 0
C
      DO 600,I=1,NBR
        ISTART = BS(I)
        IEND = BE(I)
        L = ISTART-NL-1
        IF (L.NE.0) THEN
           NEW(NN+1:NN+L) = LI(NL+1:ISTART-1)
           NN = NN+L
        END IF
C
C  ARG CONTAINS THE CHARACTERS IN BETWEEN THE BRACKETS.
        ARG = LI(ISTART+1:IEND-1)
C
C  IS THERE ANYTHING IN BETWEEN THE BRACKETS ?
C  IF NOT, DO NOT SUBSTITUTE.
        IF (ARG.EQ.' ') GOTO 700
C
C  DO WE NEED THE NUMBER OF ARGUMENTS IN THE COMMAND LINE ?
        IF (ARG.EQ.'#') THEN
           WRITE (NUM,'(I80)') NIT
C  DETERMINE THE FIRST NON BLANK CHARACTER POSITION AT NUM.
           DO 200,J=1,80
 200         IF (NUM(J:J).NE.' ') GOTO 300
 300       CONTINUE
           L = 80-J+1
           NEW(NN+1:NN+L) = NUM(J:80)
           NN = NN+L
        END IF
C
C  FIND OUT WHICH NUMBER IS IN BETWEEN THE BRACKETS.
        LE = LENGTH(ARG)
        CALL TOINT(ARG,LE,N,IERR)
C
C  AN ERROR INDICATES SOMETHING WRONG IS IN BETWEEN THE
C  BRACKETS. DO NOT SUBSTITUTE.
        IF (IERR.NE.0) GOTO 700
C
C  A NEGATIVE NUMBER CAUSES NO SUBSTITUTION.
        IF (N.LT.0) GOTO 700
C
C  DO WE HAVE SO MANY ARGUMENTS IN THE COMMAND LINE ?
C  IF NOT, JUST OMIT THE PATTERN "[...]" FROM THE LINE.
        N1 = N+1
        IF (N1.GT.NIT) GOTO 700
C
C  N=0 IMPLIES THE MACRO NAME ITSELF.
        IF (N.EQ.0) THEN
           L = IE(1)-IS(1)+1
           NEW(NN+1:NN+L) = LINE(IS(1):IE(1))
           NN = NN+L
           GOTO 700
        END IF
C
C  NOW SUBSTITUTE THE COMMAND LINE ARGUMENT THAT CORRESPONDS TO N.
        N1 = N+1
        L = IE(N1)-IS(N1)+1
        NEW(NN+1:NN+L) = LINE(IS(N1):IE(N1))
        NN = NN+L
C
 700    CONTINUE
        NL = IEND
CC    IF(NN.NE.0)WRITE(*,'(A,A)')'NEWLINESOFAR=',NEW(1:NN)
CC    IF(NL.NE.0)WRITE(*,'(A,A)')'OLD LI PROCESSED SO FAR=',LI(1:NL)
 600  CONTINUE
C
      L = 80-NL
      IF (L.NE.0) THEN
         NEW(NN+1:NN+L) = LI(NL+1:80)
         NN = NN+L
      END IF
      WRITE(IU2,10) NEW(1:NN)
CC    WRITE(*,'(''NEW LINE = '',A)')NEW(1:NN)
      GOTO 1000
C
C  END OF FILE ENCOUNTERED IN THE INPUT UNIT (IU1).
 100  CONTINUE
C
C  WRITE A 'CLEAR' COMMAND TO ENSURE PROPER MACRO TERMINATION.
      WRITE (IU2,10) 'CLEAR'
C
C  REWIND IU2, SO AS TO BE READY FOR EXECUTION.
      REWIND IU2
      RETURN
C
 10   FORMAT (A)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE BCR ( LINE, NBR, BS, BE )
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      CHARACTER*(*) LINE
      INTEGER BS(*), BE(*)
C
      LOGICAL INBRA
      CHARACTER C
C
      NBR = 0
      INBRA = .FALSE.
      DO 10,I=1,LEN(LINE)
        C = LINE(I:I)
        IF (C.EQ.'[') THEN
           ISTART = I
           INBRA = .TRUE.
        ELSE IF (C.EQ.']') THEN
           IF (INBRA) THEN
              NBR = NBR+1
              BS(NBR) = ISTART
              BE(NBR) = I
              INBRA = .FALSE.
           END IF
        END IF
 10   CONTINUE
C
      RETURN
      END
C
      SUBROUTINE MENU(ISET,IUOUT,IUINP)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  THIS ROUTINE IMPLEMENTS THE INIT SELECTION MENU TO INITIALIZE
C  AT WILL THE MERLIN ARRAYS CORRESPONDING TO LEFT-MARGINS,
C  POINT, OR RIGHT-MARGINS.
C  IUINP AND IUOUT ARE THE INPUT/OUTPUT  UNIT NUMBERS FOR THE
C  "CONVERSATION"   .
C  ISET  IS OUTPUT.
C  ISET = -1,0,1 ARE VALID VALUES , INDICATING THE ARRAY TO BE
C  INITIALIZED. ISET = 100 IS THE ONLY OTHER POSSIBLE VALUE ,
C  INDICATING THAT THE REQUEST IS TO BE IGNORED.(NO INITIALIZATION).
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
C
      ISET=100
      IF (STRO) THEN
         CALL DASH
         WRITE(IUOUT,2)
         CALL DASH
         WRITE(IUOUT,3)
         WRITE(IUOUT,4)
         WRITE(IUOUT,5)
         WRITE(IUOUT,55)
         WRITE(IUOUT,56)
         CALL DASH
         WRITE(IUOUT,6)
      END IF
      READ(IUINP,*,IOSTAT=III)ISET
       IF(III.NE.0) THEN
        REWIND IUINP
        IF (STRO) WRITE(IUOUT,9)
        ISET=100
        CALL BATCH
        RETURN
       ENDIF
      JJ=ISET*(ISET**2-1)*(ISET-2)*(ISET-3)
       IF(JJ.NE.0) THEN
        IF (STRO) WRITE(IUOUT,8)
        ISET=100
        CALL BATCH
        RETURN
       ENDIF
    2 FORMAT(T2,':',T33,'SELECTION MENU',T79,':')
    3 FORMAT(T2,':',T40,'-1 ... FOR LEFT - MARGINS',T79,':')
    4 FORMAT(T2,':',T28,'ENTER : ',T40,' 0 ... FOR    POINT',T79,':')
    5 FORMAT(T2,':',T40,'+1 ... FOR RIGHT - MARGINS',T79,':')
   55 FORMAT(T2,':',T40,' 2 ... FOR FIX - STATUS',T79,':')
   56 FORMAT(T2,':',T40,' 3 ... FOR STEPS',T79,':')
    6 FORMAT(T20,'ENTER SELECTION : >>')
   8  FORMAT (T2,' ... NON - EXISTANT OPTION ...  REQUEST CANCELED')
   9  FORMAT (T2,' WARNING FROM MENU :  ERROR UPON INPUT')
      END
      SUBROUTINE MEREND
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER ( MEM = 2500, LSTA = 500 )
      COMMON/MCLVAR/STACK(LSTA),CMEM(MEM),ISTACK,INFILE
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      PARAMETER ( MAXNL = 20 )
      INTEGER UBACK(MAXNL)
      COMMON / NEST1 / UBACK, LEVEL, IEXIT, IDISK
C
C  ARE WE RUNNING MCL ?
      IF (IUINP.EQ.INFILE) CALL MCLEND
C
C  EXIT FROM ALL MACROS.
      IF (LEVEL.GT.0) CALL TOPLEV
C
C  CLOSE THE MERLIN FILES.
      CALL FCLOSE
C
C  CLOSE THE OUTPUT UNIT.
      CALL CLOUT
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE MERLIN ( NOV, NOSQ, VERSIM, MAXW, IQUIT )
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  THIS IS THE  * MERLIN * OPERATING * SYSTEM * (MOS).
C  UPON A COMMAND IDENTIFICATION CALLS APPROPRIATE ROUTINES
C  THAT IMPLEMENT THE DESIRED OPERATION.
C
      DIMENSION VERSIM(*)
C
      PARAMETER ( MXV = 200 )
      PARAMETER ( MEM = 2500 , LSTA = 500 )
      COMMON/MCLVAR/STACK(LSTA),CMEM(MEM),ISTACK,INFILE
C
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
C
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/PARAMX/NOF
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      LOGICAL HEADER, PRECON
      CHARACTER*15 IN, OUT
      CHARACTER*80 PROLO, EPILO
      COMMON /CP1/ BI, BL, SM, NOVA, NOTE, MODPRO, MUIN, MUOUT,
     >             HEADER,  PRECON 
      COMMON /CP2/ PROLO, EPILO, IN, OUT
C
      PARAMETER ( MAXNL = 20 )
      INTEGER UBACK(MAXNL)
      CHARACTER*10 CBACK(MAXNL)
      COMMON / NEST1 / UBACK, LEVEL, IEXIT, IDISK
      COMMON / NEST2 / CBACK
C
      COMMON /GETOUT/ IRET, IRET2, IQU
C
      PARAMETER ( NCOM = 100 )
      CHARACTER*10 COMMAN(NCOM)
      INTEGER CLASS(NCOM), IND(NCOM)
      COMMON /COM1/ COMMAN
      COMMON /COM2/ CLASS, IND
C
      INTEGER IS(40), IE(40)
      CHARACTER*80 INPLIN
      COMMON /INDAT1/ IS, IE, NIT, LEC, ISTAT
      COMMON /INDAT2/ INPLIN
      COMMON /MODE3/ MODEV, IFFORM 
C
      COMMON/INDE/INDIC(6)
      CHARACTER*10 OLDCOM, COM, ACTCOM
C
      CHARACTER*80 LINE
      LOGICAL ERRORO, INPROL, INEPIL, FERROR, FIRST
      SAVE FIRST
      DATA FIRST / .TRUE. /
C
      NOD = NOV
      NOF = NOSQ
C  ---------------------------------------------------------------------
C  DETERMINE MACHINE'S ACCURACY
C  ---------------------------------------------------------------------
      CALL MACAC(COMACC)
      COMTOL=SQRT(COMACC)
C  ---------------------------------------------------------------------
C  INITIALIZE VARIABLES NEEDED SPECIFICALY FOR PROCESSING THE
C  CONFIGURATION FILE.
C  ---------------------------------------------------------------------
      CALL CPINIT
C  ---------------------------------------------------------------------
C  INITIALIZE VARIABLES THAT CAN NOT BE IN A BLOCK DATA DUE TO THE FACT
C  THAT MERLIN MAY BE CALLED AS A SUBROUTINE.
C  ---------------------------------------------------------------------
      CALL INIT
C  ---------------------------------------------------------------------
C  READ AND PROCESS THE CONFIGURATION FILE.
C  ---------------------------------------------------------------------
      ICONU = NUNIT()
      IFILEU = NUNIT()
      IERRU = NUNIT()
      IF (ICONU.EQ.-1 .OR. IFILEU.EQ.-1 .OR. IERRU.EQ.-1) THEN
         IQUIT = -1
         RETURN
      END IF
      CALL CONPRO(ICONU,IFILEU,IERRU,IERROR)
      IF (IERROR.EQ.2) THEN
         IQUIT = -4
         RETURN
      END IF
C  ---------------------------------------------------------------------
C  OPEN THE OUTPUT FILE.
C  ---------------------------------------------------------------------
      CALL OPOUT(ERRORO)
      IF (ERRORO) THEN
         CLOSE(IERRU,STATUS='DELETE')
         IQUIT = -2
         IF (IERROR.NE.0) IQUIT = -3
         RETURN
      END IF
C  ---------------------------------------------------------------------
C  COPY ANY ERRORS ENCOUNTERED DURING CONFIGURATION FILE PROCESSING
C  TO THE OUTPUT UNIT.
C  ---------------------------------------------------------------------
      REWIND IERRU
 300  READ (IERRU,'(A)',END=301) LINE
      LE = LENGTH(LINE)
      IF (LE.EQ.0) THEN
         WRITE (IUOUT,'('' '')')
      ELSE
         WRITE (IUOUT,'(A)') LINE(1:LE)
      END IF
      GOTO 300
 301  CLOSE(IERRU,STATUS='DELETE')
      IF (IERROR.NE.0) THEN
         IQUIT = -4
         CALL CLOUT
         RETURN
      END IF
C  ---------------------------------------------------------------------
C  INITIALIZE VARIABLES THAT DEPEND ON CONFIGURATION VALUES.
C  ---------------------------------------------------------------------
      CALL ZINIT
C  ---------------------------------------------------------------------
C  OPEN THE MERLIN FILES.
C  ---------------------------------------------------------------------
      CALL FOPEN(FERROR)
C  ---------------------------------------------------------------------
C  SORT THE COMMAND ARRAY, ADJUSTING COMMAND CLASSES AND INDEXES.
C  ---------------------------------------------------------------------
      IF (FIRST) THEN
         FIRST = .FALSE.
         DO 222,I=1,NCOM
 222       IND(I) = I
         CALL CMSORT(COMMAN,CLASS,IND,NCOM)
      END IF
C  ---------------------------------------------------------------------
C  MERLIN UNIT NUMBERS AND ASSOCIATED INDICES
C  ---------------------------------------------------------------------
      INDIC(1)=IUDAT
      INDIC(2)=IUSTO
      INDIC(3)=IUINI
      INDIC(4)=IUDIS
      INDIC(5)=IUBAC
      INDIC(6)=IUMAC
C  ---------------------------------------------------------------------
      IQU = 0
      IRET = 0
C  ---------------------------------------------------------------------
C  INFORMATIVE PRINTOUT
      IF (STRO .AND. HEADER) CALL AUTIDA(IUOUT)
      IF (STRO) THEN
         WRITE (IUOUT,128) NOD
         WRITE (IUOUT,131) NOF
         WRITE (IUOUT,127) COMACC
      END IF
C  ---------------------------------------------------------------------
C THE FOLLOWING CALL TO ROUTINE SETMOD, SETS THE PROCESSING MODE OPTION.
      IF (MODPRO.EQ.0) CALL SETMOD(IUINP,IUOUT)
C  ---------------------------------------------------------------------
C  EXECUTE A CATALOG COMMAND TO DETERMINE FILE STATUSES.
      CALL MATCH(COMMAN,NCOM,'CATALOG',7,ICOM)
      CALL EXECOM(ICOM,VERSIM,MAXW,OLDCOM,COM)
C  ---------------------------------------------------------------------
C  TAKE APPROPRIATE ACTION IF A PROLOG COMMAND IS DEFINED.
      INPROL = .FALSE.
      INEPIL =.FALSE.
      IPROLE = 0
      IEPILE = 0
      IF (PROLO.NE.' ') THEN
         INPROL = .TRUE.
         IPROLE = LEVEL
         CALL MID1('# PROLOG EXECUTION HAS BEEN INITIATED #')
         CALL READCO(COM,MACERR,PROLO)
         GOTO 700
      END IF
C  ---------------------------------------------------------------------
C  THE MAIN LOOP: COMMAND IDENTIFICATION AND EXECUTION STARTS HERE.
C  ---------------------------------------------------------------------
 1    CONTINUE
C
C  ISSUE MESSAGE FOR VARIABLE INITIALIZATION AT THE BEGINING.
      IF (LEVEL.EQ.0 .AND. IUINP.EQ.IU31) THEN
         IF (MODEV.EQ.0) THEN
            CALL EVWARN
         ELSE IF (ITNOCS.EQ.0) THEN
            WRITE (IUOUT,130)
         END IF
      END IF
      IF (LEVEL.EQ.0 .AND. STRO .AND. IUINP.EQ.IU31) WRITE (IUOUT,93)
C
C  READ THE NEXT COMMAND
C
      CALL READCO(COM,MACERR,' ')
C
C  CHECK IF ERRORS OCCURED, AND IF SO, ISSUE ERROR MESSAGE
C
700   CONTINUE
      IF (MACERR.EQ.1) THEN
         CALL BATCH
         CALL EXITM
         GOTO 2
      ELSE IF (MACERR.EQ.2) THEN
         GOTO 2
      END IF
      IF (COM(1:1).EQ.'$') THEN
         CALL USEMCL(COM)
         GOTO 2
      END IF
C
C   RECOGNIZE COMMAND AND TRANSFER THE CONTROL APPROPRIATELY
C
      CALL MATCHA(COM,LEC,ACTCOM,IAL)
      IF (IAL.NE.0) THEN
         COM = ACTCOM
         LEC = LENGTH(ACTCOM)
      END IF
      CALL MATCH(COMMAN,NCOM,COM,LEC,ICOM)
      IF (ICOM.EQ.-2) THEN
         IF (LEC.GT.10) THEN
            IF (STRO) WRITE(IUOUT,100) COM//'...'
         ELSE
            IF (STRO) WRITE(IUOUT,100) COM(1:LEC)
         END IF
         CALL BATCH
      ELSE IF (ICOM.EQ.-1) THEN
         IF (STRO) WRITE(IUOUT,*)'AMBIGOUS COMMAND: ',COM
         CALL BATCH
      ELSE
         IF (STRO) WRITE(IUOUT,'(T2,A)') COMMAN(ICOM)
         ISTAT = 0
         CALL EXECOM(ICOM,VERSIM,MAXW,OLDCOM,COM)
         IF (ISTAT.NE.0) CALL BATCH
         IF (IRET.NE.0) THEN
            IF (INEPIL) THEN
C  WE ARE ALREADY RUNNING OUR EPILOG. A RETURN COMMAND HAS BEEN
C  ISSUED FROM WITHIN THE EPILOG. RETURN IMMEDIATELY.
               IQUIT = IQU
            CALL MEREND
            IF (IRET2.EQ.2) STOP
               RETURN
         ELSE
C  A RETURN/QUIT ISSUED FOR THE FIRST TIME.
C  DO WE HAVE AN EPILOG TO EXECUTE ?
            IF (EPILO.NE.' ') THEN
               IRET = 0
                  INEPIL = .TRUE.
               IEPILE = LEVEL
                  CALL MID1('# EPILOG EXECUTION HAS BEEN INITIATED #')
                  CALL READCO(COM,MACERR,EPILO)
                  GOTO 700
               ELSE
C  NO EPILOG. RETURN AT ONCE.
                  IQUIT = IQU
               CALL MEREND
               IF (IRET.EQ.2) STOP
                  RETURN
            END IF
         END IF
         END IF
         OLDCOM = COMMAN(ICOM)
      END IF
C
2     CONTINUE
      IF (LEVEL.EQ.IPROLE .AND. INPROL) THEN
         INPROL = .FALSE.
         CALL MID1('# PROLOG EXECUTION HAS BEEN COMPLETED #')
      END IF
      IF (LEVEL.EQ.IEPILE .AND. INEPIL) THEN
         INEPIL = .FALSE.
         CALL MID1('# EPILOG EXECUTION HAS BEEN COMPLETED #')
         IQUIT = IQU
         CALL MEREND
         IF (IRET2.EQ.2) STOP
         RETURN
      END IF
      GOTO 1
C
 93   FORMAT(//5X,'/\/\/\/\/\/\     MERLIN   IS AT YOUR COMMAND !!! ')
100   FORMAT(3X,A,3X,'IS NOT, A VALID COMMAND')
127   FORMAT(T2,'ESTIMATED MACHINE''S ACCURACY: ',1PE8.2)
128   FORMAT(T2,'NUMBER OF VARIABLES: ',I6)
130   FORMAT(//T2,' ....   W A R N I N G   ....'/
     >            '...  INITIALIZE VARIABLES  ...')
131   FORMAT(T2,'NUMBER OF TERMS:     ',I6)
      END
C
C  ---------------------------------------------------------------------
C
      BLOCK DATA COMINI
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( NCOM = 100 )
      CHARACTER*10 COMMAN(NCOM)
      INTEGER CLASS(NCOM), IND(NCOM)
      COMMON /COM1/ COMMAN
      COMMON /COM2/ CLASS, IND
C
      DATA (COMMAN(I),CLASS(I),I= 1,30) /
     >    'ACCUM     ', 0,   'ADJUST    ', 0,   'ANAL      ', 0,
     >    'AUTO      ', 1,   'BATCH     ', 0,   'BFGS      ', 1,
     >    'CATALOG   ', 0,   'CLEAR     ', 0,   'CONGRA    ', 1,
     >    'DEMARGIN  ', 0,   'DFP       ', 1,   'DISCARD   ', 0,
     >    'FIX       ', 0,   'FIXALL    ', 0,   'FULLBACK  ', 0,
     >    'FULLPRINT ', 0,   'GODFATHER ', 0,   'GRADCHECK ', 0,
     >    'GRADDIS   ', 0,   'GRAPH     ', 0,   'HALFPRINT ', 0,
     >    'HELP      ', 0,   'HIDEOUT   ', 0,   'IAF       ', 0,
     >    'INDEX     ', 0,   'INIT      ', 0,   'INSPECT   ', 0,
     >    'INTRODUCE ', 0,   'LASTBACK  ', 0,   'LIST      ', 0  /
      DATA (COMMAN(I),CLASS(I),I=31,60) /
     >    'LOOSALL   ', 0,   'LOOSE     ', 0,   'MACRO     ', 0,
     >    'MARGIN    ', 0,   'ZZUSCOM01 ', 0,   'MEMO      ', 0,
     >    'MODEDIS   ', 0,   'NAME      ', 0,   'NOBACK    ', 0,
     >    'NOPRINT   ', 0,   'NUMER     ', 0,   'PANELOFF  ', 0,
     >    'PANELON   ', 0,   'PICK      ', 0,   'POINT     ', 0,
     >    'QUIT      ', 0,   'RANDOM    ', 1,   'RESET     ', 0,
     >    'RETURN    ', 0,   'REVEAL    ', 0,   'REWIND    ', 0,
     >    'ZZUSCOM13 ', 0,   'RUNMCL    ', 0,   'SHORTDIS  ', 0,
     >    'SIMPLEX   ', 1,   'STEP      ', 0,   'STEPALL   ', 0,
     >    'STOP      ', 0,   'VALDIS    ', 0,   'NOEVAL    ', 0  /
      DATA (COMMAN(I),CLASS(I),I=61,NCOM) /
     >    'FAST      ', 0,   'QUAD      ', 0,   'MIXED     ', 0,
     >    'EVALUATE  ', 0,   'GERROR    ', 0,   'MAD       ', 0,
     >    'TARGET    ', 0,   'NOTARGET  ', 0,   'ZZUSCOM14 ', 0,
     >    'ZZUSCOM15 ', 1,   'TOLMIN    ', 1,   'CSET      ', 0,
     >    'STEPDIS   ', 0,   'ALIAS     ', 0,   'ALIASDIS  ', 0,
     >    'UNALIAS   ', 0,   'DISKON    ', 0,   'DISKOFF   ', 0,
     >    'LIMITS    ', 0,   'DATAWRITE ', 0,   'DATASKIP  ', 0,
     >    'FLAG      ', 0,   'FLAGDIS   ', 0,   'CFLAG     ', 0,
     >    'CFLAGDIS  ', 0,   'DISPOSKIP ', 0,   'DISPOWRITE', 0,
     >    'ZZUSCOM15 ', 0,   'LEVE      ', 1,   'GENERAL   ', 0,
     >    'SOS       ', 0,   'ROLL      ', 1,   'ZZUSCOM17 ', 0,
     >    'ZZUSCOM18 ', 0,   'ZZUSCOM19 ', 0,   'ZZUSCOM20 ', 0,
     >    'ZZUSCOM21 ', 0,   'ZZUSCOM22 ', 0,   'ZZUSCOM23 ', 0,
     >    'ZZUSCOM24 ', 0  /
C
      END
      SUBROUTINE DASH
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      WRITE (IUOUT,10)
 10   FORMAT(T2,78('-'))
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE MID1 ( MSG )
      CHARACTER*(*) MSG
      CALL MIDPRI('.',MSG)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE MID2 ( MSG )
      CHARACTER*(*) MSG
      CALL MIDPRI('-',MSG)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE MID3 ( MSG )
      CHARACTER*(*) MSG
      CALL MIDPRI('.',MSG)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE MIDPRI ( C, MSG )
C
      CHARACTER C, MSG*(*)
C
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      CHARACTER LINE*80, THREE*3
C
      IF (STRO) THEN
         THREE = C//C//C
         LE = LENGTH(MSG)
         IB = (79-LE)/2
         IBLE = IB+LE
         LINE = ' '
         LINE(IB-4:IB-2) = THREE
         LINE(IB+1:IBLE) = MSG(1:LE)
         LINE(IBLE+3:IBLE+5) = THREE
         WRITE(IUOUT,'(A)') LINE(1:IBLE+5)
         WRITE(IUOUT,*)
      END IF
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE EVWARN
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      IF (STRO) WRITE (IUOUT,10)
10    FORMAT (/T12,'*** WARNING: FUNCTION EVALUATION HAS BEEN ',
     >             'TURNED OFF ***')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE FDERI ( X, V0, N, DER, ERB, ERROR, IE )
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      COMMON/ACCURA/COMACC,COMTOL
C
      ONE = 1.
      C = 0.
      IF (ABS(V0).LE.COMACC) C = 1.
      F0 = V0+C
C
      POT = 10.**(-N)
      E = MAX(POT,COMACC)
C
      H = ABS(X*E)
      IF (ABS(X).LT.1) H = E
C
      FP = ACSQ1(X+H)+C
      AK = LOG(ABS(FP/F0))/LOG(1+E)
      IF (AK*(AK-1.).EQ.0.) AK = AK+COMTOL
C
      E1 = SQRT(ABS(2./(AK-1)*((FP-F0)/AK/F0-E)))
      E1 = MAX(E1,COMTOL)
      H1 = ABS(E1*X)
      IF (ABS(X).LT.1) H1 = E1
      FP1 = ACSQ1(X+H1)+C
      DER = (FP1-F0)/H1
C
      D = (FP-F0)/H
      AK = X*(FP-F0)/(H*F0)
      IF (X .NE.0) THEN
         AMODEL = AK*F0/X
         ERROR = (AMODEL-DER)/MAX(ONE,MIN(ABS(AMODEL),ABS(DER)))
      ELSE
         ERROR = (D-DER)/MAX(ONE,MIN(ABS(DER),ABS(D)))
         ERROR = ERROR*H1/(H-H1)
      END IF
      ERROR = ABS(ERROR)
C
      IE = 0
      IF (ERROR.GT.ERB) IE = -1
C
      END
      SUBROUTINE MINDIR(MNOC,ERR,ISTEPC,NOCALL,ALPHA)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   MINDIR ... PERFORMS A LINE-SEARCH IN THE DIRECTION STORED IN
C   ARRAY DIRCOS (COMMON-BLOCK : DIRCOM )
C   INPUT
C   MNOC  APPROXIMATE MAXIMUM NUMBER OF ACSQ EVALUATIONS ALLOWED.
C   ERR   A TOLERANCE, USED TO TERMINATE THE SEARCH PREMATURELY.
C   ISTEPC = 1,0 FOR SEARCH-STEP UPDATING OR NOT
C   OUTPUT
C   NOCALL  NUMBER OF ACSQ EVALUATIONS.
C   ALPHA   IS PROPORTIONAL TO THE DISTANCE OF PROGRESS.
C           THE PROPORTIONALITY DIVISOR IS THE SQUARE-ROOT OF THE
C           SUM OF THE SQUARES OF THE ELEMENTS OF DIRCOS.
C
      PARAMETER ( MXV = 200 )
      LOGICAL STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      COMMON/DIRCOM/DIRCOS(MXV)
      COMMON/CWORK1/XWORK(MXV),XT1(MXV),XT2(MXV),XT3(MXV),XT4(MXV)
      EXTERNAL FDIR1
      ALPHA=0.
      NOV=NOD
      ZERO=0.
      NOCALL=0
C
C  IF FIXED PARAMETERS, SET DIRECTION COSINES TO ZERO
C
      NOFIX=0
       DO 1 I=1,NOD
         IF(IXAT(I).EQ.0.OR.DIRCOS(I).EQ.0.)THEN
          DIRCOS(I)=0.
          NOFIX=NOFIX+1
         ENDIF
    1  CONTINUE
C
C IF ALL PARAMETERS ARE  FIXED THEN RETURN
C
       IF(NOFIX.EQ.NOD)THEN
        IF(STRO)WRITE(IUOUT,*)'ALL VARIABLES FIXED '
        RETURN
       ENDIF
C
C
C   FIND LEFT LIMIT IF IT EXISTS
C
      NL=0
      XALIM=0.
       DO 4 I=1,NOD
        IF(DIRCOS(I).EQ.0..OR.MARG(I).EQ.0)GOTO  4
        NL=NL+1
         IF(DIRCOS(I).GT.0)THEN
          XTLIM=(XLL(I)-POINT(I))/DIRCOS(I)
         ELSE
          XTLIM=(XRL(I)-POINT(I))/DIRCOS(I)
         ENDIF
         IF(NL.EQ.1)THEN
          XALIM=XTLIM
         ELSE
          IF(XALIM.LT.XTLIM)XALIM=XTLIM
         ENDIF
    4  CONTINUE
C
C
C      FIND RIGHT LIMIT
C
      NR=0
      XBLIM=0.
       DO 5 I=1,NOD
        IF(DIRCOS(I).EQ.0..OR.MARG(I).EQ.0)GOTO  5
        NR=NR+1
         IF(DIRCOS(I).GT.0.)THEN
          XTLIM=(XRL(I)-POINT(I))/DIRCOS(I)
         ELSE
          XTLIM=(XLL(I)-POINT(I))/DIRCOS(I)
         ENDIF
         IF(NR.EQ.1)THEN
          XBLIM=XTLIM
         ELSE
          IF(XBLIM.GT.XTLIM)XBLIM=XTLIM
         ENDIF
    5  CONTINUE
C
C   PREPARE LIM
C
       IF(NL.EQ.0.AND.NR.EQ.0)THEN
        LIM=0
       ELSE IF(NL.EQ.0)THEN
        LIM=1
       ELSE IF(NR.EQ.0)THEN
        LIM=-1
       ELSE
        LIM=2
       ENDIF
C
C    PREPARE STEP
C
      SST = 1
C
C  DEFINE FA-FB
C
      FA=0.
      FB=0.
       IF(LIM.EQ.2.OR.LIM.EQ.-1)THEN
        FA=FDIR1(XALIM)
        NOCALL=NOCALL+1
       ENDIF
       IF(LIM.EQ.2.OR.LIM.EQ.1)THEN
        FB=FDIR1(XBLIM)
        NOCALL=NOCALL+1
       ENDIF
      CALL MINI(FDIR1,ZERO,SST,MNOC,ERR,LIM,XALIM,XBLIM,FA,FB,XMIN,
     >FMIN,NOCL,MINS)
      NOCALL=NOCALL+NOCL
C
C UPDATE POINT-VALUE
C
       DO 7 I=1,NOD
        POINT(I)=POINT(I)+DIRCOS(I)*XMIN
    7  CONTINUE
C
C
C
      VAL=ACSQ(POINT,NOD)
      NOCALL=NOCALL+1
       IF(XMIN.NE.0..AND.ISTEPC.EQ.1) THEN
         DO 8 I=1,NOV
          IF(IXAT(I).NE.0)STEP(I)=XMIN*DIRCOS(I)
    8    CONTINUE
       ENDIF
      ALPHA=XMIN
      RETURN
      END
C
C
      FUNCTION FDIR1(X0)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   ASSUMES THE VALUES OF THE OBJECTIVE FUNCTION F(X0+X*D)
C   WHERE D IS THE VECTOR WHOSE COMPONENTS ARE THE DIRECTION
C   COSINES AND X0 IS THE VECTOR OF THE CURRENT POSITION.
C
      PARAMETER ( MXV = 200 )
      COMMON/CWORK1/XWORK(MXV),XT1(MXV),XT2(MXV),XT3(MXV),XT4(MXV)
      COMMON/DIRCOM/DIRCOS(MXV)
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
       DO 1 I=1,NOD
        XWORK(I)=POINT(I)+DIRCOS(I)*X0
    1  CONTINUE
      FDIR1=ACSQ(XWORK,NOD)
      RETURN
      END
      SUBROUTINE MINI(F,X0,STEP,MNOC,TOL,LIM,ALIM,BLIM,FA,FB,XMIN,FMIN,
     >NOCALL,MINS)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    THIS SUBROUTINE LOCATES THE MINIMUM OF A FUNCTION
C    OF ONE VARIABLE F(X)
C
C
C   LIM=0   :   NO BOUNDS
C   LIM=1   :  UPPER BOUND ONLY
C   LIM=-1  :  LOWER BOUND ONLY
C   LIM=2   :  BOTH, UPPER AND LOWER BOUNDS
C
C
C    IN ANY CASE  FA AND FB ,WHICH REPRESENT THE FUNCTION'S VALUE
C   AT ALIM AND BLIM RESPECTIVELY , MUST BE INITIALIZED.
C   BUT WHEN LIM=0 NONE OF THEM IS USED,WHEN LIM=1 ONLY FA IS USED,
C   ETC.
C
C      STEP = INITIAL STEP VALUE .UPON EXITING STEP IS UPDATED
C
C   MINS=2   A BETTER POINT HAS BEEN FOUND INSIDE A QUADRATIC REGION
C   MINS=1  A BETTER POINT HAS BEEN FOUND OUT OF A QUADRATIC REGION
C            (IT CAN BE IMPROVED DRASTICALLY WITH MORE CALLS)
C  MINS=0  ALREADY AT A MINIMUM OR A SADDLE POINT
C  MINS=-1 FUNCTION SEEMS FLAT BUT MORE CALLS MAY GIVE A BETTER POINT
C  MINS=-2 FUNCTION SEEMS TO BE CONSTANT FROM X0-STEP TO X0+STEP
C
C   SUBROUTINES CALLED:         MFLAT
C                               MPARAB
C                               FTRA
C
      EXTERNAL F
      MINS=1
      ITOL=0
      A=10.
      B=-.3
      NTOL=2
      S=STEP
      NOCALL=0
C
C   CHECK IF X0 IS OUT OF LIMITS
      IF((LIM.EQ.-1.OR.LIM.EQ.2).AND.(X0.LT.ALIM))X0=ALIM
      IF((LIM.EQ.1.OR.LIM.EQ.2).AND.(X0.GT.BLIM))X0=BLIM
      XOLD=X0
      CALL FTRA(F,X0,LIM,ALIM,BLIM,FA,FB,FOLD,NOCALL)
      F0=FOLD
       IF(LIM.EQ.2.AND.ALIM.EQ.BLIM)THEN
        MINS=0
        XMIN=X0
        FMIN=F0
        RETURN
       ENDIF
C   CHECK STEP
      S=ABS(S)
       IF(S.EQ.0.)THEN
        S=(ABS(XOLD)+2.)/10.
       ENDIF
       DO 1 I=1,500
        XTEST=XOLD+S/100.
        IF(XTEST.NE.XOLD)GOTO  2
        S=S*100.
    1  CONTINUE
    2 CONTINUE
C
C   CHECK IF FUNCTION IS FLAT
C
      XM=XOLD-S
      XP=XOLD+S
       IF((LIM.EQ.-1.OR.LIM.EQ.2).AND.(XM.LE.ALIM))THEN
        XM=ALIM
        FM=FA
       ELSE
        CALL FTRA(F,XM,LIM,ALIM,BLIM,FA,FB,FM,NOCALL)
       ENDIF
       IF((LIM.EQ.1.OR.LIM.EQ.2).AND.(XP.GE.BLIM))THEN
        XP=BLIM
        FP=FB
       ELSE
        CALL FTRA(F,XP,LIM,ALIM,BLIM,FA,FB,FP,NOCALL)
       ENDIF
       IF((XM.EQ.XOLD.AND.FP.EQ.FOLD).OR.(XP.EQ.XOLD.AND.FM.EQ.FOLD)
     >.OR.(FM.EQ.FOLD.AND.FP.GE.FOLD)
     >.OR.(FP.EQ.FOLD.AND.FM.GE.FOLD))   THEN
        NMAX=MNOC-NOCALL
        CALL MFLAT(F,X0,FOLD,S,LIM,ALIM,BLIM,FA,FB,NMAX,XN,
     >FN,SN,NOCN,MSF)
        NOCALL=NOCALL+NOCN
        S=SN
         IF(MSF.NE.2)THEN
          IF(MSF.LT.0)MINS=-2
          IF(MSF.EQ.1)MINS=0
          IF(MSF.EQ.0)MINS=-1
          XMIN=X0
          FMIN=F0
          STEP=S
          RETURN
         ELSE
          FNEW=FN
          XNEW=XN
         ENDIF
       ELSE
         IF(XP.EQ.XOLD)THEN
          XNEW=XM
          FNEW=FM
         ELSEIF(XM.EQ.XOLD)THEN
          XNEW=XP
          FNEW=FP
         ELSEIF(FM.LE.FP)THEN
          XNEW=XM
          FNEW=FM
         ELSE
          XNEW=XP
          FNEW=FP
         ENDIF
        S=XNEW-XOLD
       ENDIF
C
C
       IF(FNEW.GT.FOLD)THEN
        XT=XOLD
        FT=FOLD
        XOLD=XNEW
        FOLD=FNEW
       ELSE
        XT=XNEW
        FT=FNEW
       ENDIF
C
    3 CONTINUE
      XNEW=XT+S
      CALL FTRA(F,XNEW,LIM,ALIM,BLIM,FA,FB,FNEW,NOCALL)
       IF(FNEW.LT.FOLD)THEN
        S=S*A
        FOLD=FT
        XOLD=XT
        XT=XNEW
        FT=FNEW
        IF(NOCALL.GE.MNOC)GO TO  6
        GO TO  3
       ENDIF
      MINS=2
      A=2.
      B=-.3
    4 CONTINUE
      CALL MPARAB(FOLD,FT,FNEW,XOLD,XT,XNEW,XP,MIS)
       IF(MIS.EQ.1)THEN
        S=S*B
        XP=XT+S
       ENDIF
      S=XP-XT
       IF(XP.NE.XT)THEN
        CALL FTRA(F,XP,LIM,ALIM,BLIM,FA,FB,FP,NOCALL)
       ELSE
        FP=FT
       ENDIF
       IF(FP.GE.FT)THEN
        XOLD=XP
        FOLD=FP
        S=S*B
       ELSE
        XOLD=XT
        FOLD=FT
        XT=XP
        FT=FP
       ENDIF
    5 CONTINUE
      XNEW=XT+S
       IF(XNEW.NE.XT)THEN
        CALL FTRA(F,XNEW,LIM,ALIM,BLIM,FA,FB,FNEW,NOCALL)
       ELSE
        FNEW=FT
       ENDIF
      ERR=ABS(FNEW-FT)
       IF(ERR.LE.TOL*ABS(FT))THEN
        ITOL=ITOL+1
        IF(ITOL.GE.NTOL)GO TO  6
       ELSE
        ITOL=0
       ENDIF
      IF(NOCALL.GT.MNOC)GO TO  6
      IF(FNEW.GE.FT)GO TO  4
      S=S*A
      XOLD=XT
      FOLD=FT
      XT=XNEW
      FT=FNEW
      GO TO  5
    6 CONTINUE
      XMIN=XT
      FMIN=FT
      IF(XMIN.EQ.X0)MINS=0
      STEP=S
C
      IF(LIM.EQ.0)RETURN
       IF(((LIM.EQ.1).OR.(LIM.EQ.2)).AND.(XMIN.GT.BLIM))THEN
        XMIN=BLIM
        FMIN=FB
       ELSEIF(((LIM.EQ.-1).OR.(LIM.EQ.2)).AND.(XMIN.LT.ALIM))THEN
        XMIN=ALIM
        FMIN=FA
       ENDIF
      RETURN
      END
C
C
      SUBROUTINE MFLAT(F,X0,F0,S0,LIM,ALIM,BLIM,FA,FB,MNOC,XN,
     >FN,SN,NOC,MSF)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C  THIS ROUTINE IS CALLED IN ORDER TO FIND A LOWER (THAN F0)
C  VALUE OF A ONE VARIABLE FUNCTION F WHICH SEEMS TO BE FLAT.
C  MFLAT AT FIRST DEFINES A SEARCH AREA AND THEN TRIES A GRID
C  SEARCH IN IT.
C  IF A BETTER POINT IS FOUND MFLAT RETURNS THIS POINT XN AND THE
C  CORRESPONDING FUNCTION VALUE FN AND A STEP SN
C
C  MSF IS AN ERROR FLAG
C  WHICH IS SET BY MFLAT ACCORDING TO
C   MSF=2  MFLAT HAS FOUND A POINT WITH LOWER FUNCTION VALUE
C   MSF=1  FUNCTION IS ALREADY AT MINIMUM
C   MSF=0  NO LOWER VALUE POINT FOUND.
C   MSF=-1 FUNCTION SEEMS CONSTANT. NO GRID-SEARCH APPLIED
C   MSF=-2 FUNCTION CONSTANT. GRID-SEARCH APPLIED
C
      LOGICAL RSW,LSW,SRT,RSW1,LSW1
      STEPF=3.
      NOGS=11
      NOC=0
      SRT=.FALSE.
      RSW=.TRUE.
      LSW=.TRUE.
      LSW1=LSW
      RSW1=RSW
      S=ABS(S0)
      XN=X0
      FN=F0
      A=ALIM
      B=BLIM
    1 CONTINUE
      IF(.NOT.RSW.AND..NOT.LSW)GO TO  6
      S=STEPF*S
      IF(LSW)XM=X0-S
      IF(RSW)XP=X0+S
       IF(LIM.NE.0)THEN
         IF((LIM.EQ.-1.OR.LIM.EQ.2).AND.(XM.LE.ALIM).AND.LSW)THEN
          XM=ALIM
          FM=FA
          LSW=.FALSE.
         ENDIF
         IF((LIM.EQ.1.OR.LIM.EQ.2).AND.(XP.GT.BLIM).AND.RSW)THEN
          XP=BLIM
          FP=FB
          RSW=.FALSE.
         ENDIF
       ENDIF
       IF(LSW)THEN
        FM=F(XM)
        NOC=NOC+1
       ENDIF
       IF(RSW)THEN
        FP=F(XP)
        NOC=NOC+1
       ENDIF
       IF(FM.EQ.F0.AND.F0.EQ.FP)THEN
        MSF=-1
        GO TO  3
       ELSEIF(FM.GT.F0.AND.FP.GT.F0)THEN
        MSF=1
        A=XM
        B=XP
        LSW1=.FALSE.
        LSW=LSW1
        RSW1=LSW
        RSW=RSW1
        GO TO  6
       ELSEIF(FM.EQ.F0.AND.FP.GT.F0)THEN
        MSF=0
        RSW=.FALSE.
        RSW1=.FALSE.
        XN=XP
        FN=FP
        B=XP
        GO TO  3
       ELSEIF(FM.GT.F0.AND.FP.EQ.F0)THEN
        MSF=0
        LSW=.FALSE.
        LSW1=.FALSE.
        XN=XM
        FN=FM
        A=XM
        GOTO  3
       ELSE
         IF(FM.LE.FP)THEN
          XN=XM
          FN=FM
         ELSE
          XN=XP
          FN=FP
         ENDIF
        MSF=2
       ENDIF
    2 CONTINUE
      SN=XN-X0
      RETURN
    3 CONTINUE
      IF(NOC.LT.MNOC)GO TO  1
      IF(LIM.EQ.0.AND.SRT)GOTO  4
      SRT=.TRUE.
       IF(LIM.EQ.-1)THEN
        XM=ALIM
       ELSEIF(LIM.EQ.1)THEN
        XP=BLIM
       ELSEIF(LIM.EQ.2)THEN
        XM=ALIM
        XP=BLIM
       ENDIF
      GOTO  1
    4 CONTINUE
      IF((.NOT.RSW1.AND..NOT.LSW1).OR.(.NOT.RSW1.AND.LSW1.AND..NOT.LSW).
     >OR.(.NOT.LSW1.AND.RSW1.AND..NOT.RSW))MSF=1
    5 CONTINUE
       IF((X0-XM).LE.(XP-X0))THEN
        S=X0-XM
       ELSE
        S=XP-X0
       ENDIF
      XN=X0
      SN=S
      FN=F0
      RETURN
    6 CONTINUE
C   ........    TRY GRID  .......
       IF(RSW1.AND.LSW1)THEN
        MSF=-2
       ELSE
        MSF=1
       ENDIF
      IMAX=NOGS-1
    7 CONTINUE
      IS=1
      XL=A
      XR=B
      S=(B-A)/REAL(NOGS)
    8 CONTINUE
C   ............     SEARCH AREA : (A,B)      STEP=S
      RSW1=.TRUE.
      LSW1=.TRUE.
      IL=0
      IR=IMAX+1
       DO 9 I=1,IMAX,IS
        XT=REAL(I)*S+A
        IF(XT.GT.B)XT=B
        FT=F(XT)
        NOC=NOC+1
         IF(FT.LT.F0)THEN
          XN=XT
          FN=FT
          MSF=2
          GO TO  2
         ELSEIF(FT.GT.F0.AND.XT.LT.X0)THEN
          XL=XT
          LSW1=.FALSE.
          IL=I
         ELSEIF(FT.GT.F0.AND.XT.GT.X0)THEN
          XR=XT
          RSW1=.FALSE.
          IR=I
          GO TO  10
         ENDIF
    9  CONTINUE
   10 CONTINUE
      A=XL
      B=XR
      IMAX=(IR-IL)*2-1
      NC=NOC+(IR-IL)
      S1=(B-A)/REAL(NOGS)
      S=S/2.
      IS=2
      IF(.NOT.(RSW1.AND.LSW1))MSF=1
       IF(NOC.GE.MNOC)THEN
        XM=A
        XP=B
        GOTO  5
       ENDIF
       IF((RSW1.AND.LSW1).OR.(S1.GT.S))THEN
        IF(NC.GT.MNOC)IMAX=(MNOC-NOC)*2
        GO TO  8
       ELSE
        IMAX=NOGS-1
        NC=NOC+IMAX
        IF(NC.GT.MNOC)IMAX=MNOC-NOC
        GO TO  7
       ENDIF
      END
C
C
      SUBROUTINE FTRA(F,X,LIM,ALIM,BLIM,FA,FB,FVAL,NOCALL)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   THIS IS THE FUNCTION WHICH MINI CALLS AND MINIMIZES
C   IT'S VALUES ARE THE SAME WITH THE EXTERNAL FUNCTION F
C   IF THE VARIABLE -  X - ,LIES BETWEEN THE MARGINS
C
       IF((LIM.EQ.-1.OR.LIM.EQ.2).AND.(X.LT.ALIM))THEN
        FVAL=-X+FA+ALIM
       ELSEIF((LIM.EQ.1.OR.LIM.EQ.2).AND.(X.GT.BLIM))THEN
        FVAL=X-BLIM+FB
       ELSE
        FVAL=F(X)
        NOCALL=NOCALL+1
       ENDIF
      RETURN
      END
C
C
      SUBROUTINE MPARAB(F1,F2,F3,X1,X2,X3,XMIN,MIS)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  RETURNS THE POSITION OF THE MINIMUM (XMIN) OF THE
C  PARABOLA PASSING THROUGH (X1,F1),(X2,F2),(X3,F3)
C  IF THERE IS NO MINIMUM  MIS=1 ELSE MIS=0
C
      MIS=1
      D=(X1-X2)*(X3-X2)*(X3-X1)/2.
      IF(D.EQ.0.)RETURN
      B=(X1-X2)*(F3-F2)-(F1-F2)*(X3-X2)
       IF((D.GT.0..AND.B.GT.0.).OR.(B.LT.0..AND.D.LT.0.))THEN
        A=((F1-F2)*(X3-X2)**2-(F3-F2)*(X1-X2)**2)/2.
        MIS=0
        XMIN=X2-A/B
       ENDIF
      RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE MIXED
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  IEND IS OUTPUT: 0/1 NO EOF, EOF ENCOUNTERED.
C
      PARAMETER ( MXV = 200 )
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
C
      CHARACTER*5 GRADI
      COMMON/DERIMO/GRADI(MXV)
      COMMON/UPDATE/IGRADU,IHESSU
C
      CHARACTER*80 LINE, GRMODE
      INTEGER IS(40), IE(40)
C
      CALL INLINE(LINE,.TRUE.,.TRUE.,IS,IE,NIT,IERR)
      IF (IERR.NE.0) GOTO 500
      IF (NIT.EQ.0) RETURN
C
      N = 1
 200  CONTINUE
      IND = IPARIN(LINE(IS(N):IE(N)))
      IF (IND.EQ.0) THEN
         IF(STRO)WRITE(IUOUT,11)LINE(IS(N):IE(N))
         CALL BATCH
         GOTO 400
      END IF
      N = N+1
      IF (N.GT.NIT) THEN
         IF(STRO)WRITE(IUOUT,12)
         CALL BATCH
         GOTO 400
      END IF
      GRMODE = LINE(IS(N):IE(N))
      IF (GRMODE.EQ.'F') GRMODE = 'FAST'
      IF (GRMODE.EQ.'Q') GRMODE = 'QUAD'
      IF (GRMODE.EQ.'N') GRMODE = 'NUMER'
      IF (GRMODE.EQ.'A') GRMODE = 'ANAL'
      IF (GRMODE.EQ.'FAST'  .OR. GRMODE.EQ.'QUAD' .OR.
     >    GRMODE.EQ.'NUMER' .OR. GRMODE.EQ.'ANAL') THEN
         IF (GRADI(IND).NE.GRMODE) IGRADU = 0
         GRADI(IND) = GRMODE
      ELSE
         IF(STRO)WRITE(IUOUT,10)LINE(IS(N):IE(N))
         CALL BATCH
         GOTO 400
      END IF
      N = N+1
      IF (N.LE.NIT) GOTO 200
C
 400  CONTINUE
      RETURN
C
C  EOF ENCOUNTERED IN THE INPUT UNIT.
C
 500  CONTINUE
      CALL BATCH
C
 10   FORMAT(T2,'DERIVATIVE MODE NOT RECOGNIZED: ',A,/)
 11   FORMAT(T2,'INCORRECT VARIABLE INDEX: ',A,/)
 12   FORMAT(T2,'LAST ITEM IN INPUT LINE IS OBSOLETE AND WAS IGNORED',/)
      END
      SUBROUTINE MODIS
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
      PARAMETER ( MXV = 200 )
      LOGICAL STRO, WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/PARAMX/NOF
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      COMMON/ACCURA/COMACC,COMTOL
C
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
C
      CHARACTER*10 CSET, DATAMO, DISMOD
      COMMON /MODE2/ CSET, DATAMO, DISMOD
C
      PARAMETER ( MAXNL = 20 )
      INTEGER UBACK(MAXNL)
      CHARACTER*10 CBACK(MAXNL)
      COMMON / NEST1 / UBACK, LEVEL, IEXIT, IDISK
      COMMON / NEST2 / CBACK
C
      LOGICAL HEADER, PRECON
      CHARACTER*15 IN, OUT
      CHARACTER*80 PROLO, EPILO
      COMMON /CP1/ BI, BL, SM, NOVA, NOTE, MODPRO, MUIN, MUOUT,
     >             HEADER,  PRECON 
      COMMON /CP2/ PROLO, EPILO, IN, OUT
      COMMON /MODE3/ MODEV, IFFORM 
      LOGICAL AIMON
      COMMON/AIM/TARGET,AIMON
C
      IF (STRO) THEN
         WRITE (IUOUT,117)
         CALL DASH
         WRITE (IUOUT,128) NOD, BIGNUM
         WRITE (IUOUT,131) NOF, DWARF
         WRITE (IUOUT,200) BACKUP, COMACC
         IF (AIMON) THEN
            WRITE (IUOUT,201) DERIVA, TARGET
         ELSE
            WRITE (IUOUT,301) DERIVA
         END IF
         WRITE (IUOUT,202) PRINTO
         WRITE (IUOUT,203) CALLBY
         WRITE (IUOUT,204) PROCES
         WRITE (IUOUT,205) PANEL
         WRITE (IUOUT,206) CSET
         IF (IDISK.EQ.0) THEN
            WRITE (IUOUT,207) 'NO'
         ELSE
            WRITE (IUOUT,207) 'YES'
         END IF
         IF (PROLO.EQ.' ') THEN
            WRITE (IUOUT,208) ' - NONE -'
         ELSE
            WRITE (IUOUT,208) PROLO
         END IF
         IF (EPILO.EQ.' ') THEN
            WRITE (IUOUT,209) ' - NONE -'
         ELSE
            WRITE (IUOUT,209) EPILO
         END IF
         WRITE (IUOUT,210) DATAMO
         WRITE (IUOUT,212) DISMOD
         IF (MODEV.EQ.0) THEN
            WRITE (IUOUT,211) 'OFF'
         ELSE
            WRITE (IUOUT,211) 'ON'
         END IF
         IF (IFFORM.EQ.0) THEN
            WRITE (IUOUT,213) 'GENERAL'
         ELSE
            WRITE (IUOUT,213) 'SUM OF SQUARES'
         END IF
         CALL DASH
      END IF
C
117   FORMAT (T2,'CURRENT OPERATION MODES')
128   FORMAT (T2,'NUMBER OF VARIABLES: ',I4,
     >        T45,'LARGEST REAL:          ',1PE9.2E3)
131   FORMAT (T2,'NUMBER OF TERMS:     ',I4,
     >        T45,'SMALLEST NONZERO REAL: ',1PE9.2E3)
200   FORMAT (T2,'BACKUP MODE: ',T27,A,
     >        T45,'MACHINE''S ACCURACY:    ',1PE8.2)
201   FORMAT (T2,'DERIVATIVE MODE: ',T27,A,
     >        T45,'TARGET VALUE:          ',1PE9.2E3)
301   FORMAT (T2,'DERIVATIVE MODE: ',T27,A,
     >        T45,'TARGET VALUE:          ',' - NONE -')
202   FORMAT (T2,'PRINTOUT MODE: ',T27,A)
203   FORMAT (T2,'VARIABLES ARE CALLED BY: ',T27,A)
204   FORMAT (T2,'PROCESSING MODE: ',T27,A)
205   FORMAT (T2,'PANEL MODE: ',T27,A)
206   FORMAT (T2,'CHARACTER SET: ',T27,A)
207   FORMAT (T2,'LOOK ON DISK FOR MACROS: ',T27,A)
208   FORMAT (T2,'PROLOG COMMAND: ',T27,A)
209   FORMAT (T2,'EPILOG COMMAND: ',T27,A)
210   FORMAT (T2,'DATA FILE MODE: ',T27,A)
212   FORMAT (T2,'DISPO FILE MODE: ',T27,A)
211   FORMAT (T2,'FUNCTION EVALUATION: ',T27,A)
213   FORMAT (T2,'FUNCTIONAL FORM: ',T27,A)
      END
      SUBROUTINE NAMEM(NOV)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     ...NAMEM   ASSIGNS NAMES TO VARIABLES .
C     ...NAMEM IS CALLED BY THE   GODFATHER  COMMAND
C
      PARAMETER ( MXV = 200 )
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 NICKNA
      COMMON/NAMES/NICKNA(MXV)
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      CHARACTER*10 READTR*79
        READ(IUINP,7,END=5)READTR
        I1=1
        K=0
    3   CONTINUE
        CALL PICKW(READTR,I1,I2,IEN)
        IF(IEN.EQ.0)GO TO  4
         IF(K.EQ.0)THEN
            II=IPARIN(READTR(I1:I2))
         IF (II.EQ.0) THEN
            IF (STRO) WRITE(IUOUT,9) READTR(I1:I2)
            CALL BATCH
         END IF
         ELSE
          NICKNA(II)=READTR(I1:I2)
          CALL DUPNAM(II,NOV)
         ENDIF
        K=1-K
        I1=I2+1
        GO TO  3
    4   CONTINUE
        RETURN
    5 REWIND IU31
C   IF IN BATCH OPTION OF THE PROCES MODE ABORT ON ERRORS
      CALL BATCH
      RETURN
    6 FORMAT(A10)
    7 FORMAT(A79)
    8 FORMAT(BN,I3)
    9 FORMAT(T2,'INCORRECT REFERENCE TO A VARIABLE: ',A)
      END
C
C  ---------------------------------------------------------------------
C
      FUNCTION NUNIT()
C
      COMMON /UALLOC/ MINU, MAXU, LASTU
C
      LOGICAL OPEN
C
      DO 10,I=LASTU+1,MAXU
        INQUIRE(UNIT=I,OPENED=OPEN,IOSTAT=IERR)
        IF (IERR.EQ.0) THEN
           IF (.NOT.OPEN) THEN
              LASTU = I
              NUNIT = I
CC      WRITE(*,*) 'UNIT ALLOCATED = ',LASTU
              RETURN
           END IF
        END IF
 10   CONTINUE
C
      DO 20,I=MINU,LASTU
        INQUIRE(UNIT=I,OPENED=OPEN,IOSTAT=IERR)
        IF (IERR.EQ.0) THEN
           IF (.NOT.OPEN) THEN
              LASTU = I
              NUNIT = I
CC      WRITE(*,*) 'UNIT ALLOCATED = ',LASTU
              RETURN
           END IF
        END IF
 20   CONTINUE
C
      NUNIT = -1
CC      WRITE(*,*) 'UNIT ALLOCATED = ',-1
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE OPOUT ( ERROR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  OPENS THE MERLIN OUTPUT UNIT.
C
      LOGICAL ERROR
C
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      LOGICAL HEADER, PRECON
      CHARACTER*15 IN, OUT
      CHARACTER*80 PROLO, EPILO
      COMMON /CP1/ BI, BL, SM, NOVA, NOTE, MODPRO, MUIN, MUOUT,
     >             HEADER,  PRECON 
      COMMON /CP2/ PROLO, EPILO, IN, OUT
      LOGICAL ADOPI, ADOPO
      COMMON /ADDIOP/ ADOPI, ADOPO
C
      LOGICAL OPE
C
      ERROR = .FALSE.
      IERR = 0
C
      ADOPO = .FALSE.
      IF (OUT.EQ.' ') THEN
         IF (.NOT.PRECON) OPEN(IU32,IOSTAT=IERR)
      ELSE
         IF (PRECON) THEN
            INQUIRE(FILE=OUT,OPENED=OPE,NUMBER=IU,IOSTAT=IERR)
         IF (IERR.EQ.0) THEN
            IF (OPE) THEN
               IU32 = IU
            ELSE
                  IU32 = NUNIT()
                  IF (IU32.EQ.-1) THEN
                     IERR = -1
                  ELSE
                     OPEN(IU32,FILE=OUT,IOSTAT=IERR)
               ADOPO = .TRUE.
                  END IF
            END IF
         END IF
         ELSE
            IU32 = NUNIT()
            IF (IU32.EQ.-1) THEN
               IERR = -1
            ELSE
               OPEN(IU32,FILE=OUT,IOSTAT=IERR)
            END IF
         END IF
      END IF
C
      ERROR = IERR.NE.0
      IUOUT = IU32
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE CLOUT
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  CLOSES THE MERLIN OUTPUT UNIT.
C
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
C
      LOGICAL HEADER, PRECON
      CHARACTER*15 IN, OUT
      CHARACTER*80 PROLO, EPILO
      COMMON /CP1/ BI, BL, SM, NOVA, NOTE, MODPRO, MUIN, MUOUT,
     >             HEADER,  PRECON 
      COMMON /CP2/ PROLO, EPILO, IN, OUT
      LOGICAL ADOPI, ADOPO
      COMMON /ADDIOP/ ADOPI, ADOPO
C
      IERR = 0
      IF (OUT.EQ.' ') THEN
         IF (.NOT.PRECON) CLOSE(IU32,IOSTAT=IERR)
      ELSE
         IF (PRECON) THEN
            IF (ADOPO) CLOSE(IU32,IOSTAT=IERR)
         ELSE
            CLOSE(IU32,IOSTAT=IERR)
         END IF
      END IF
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE FOPEN ( ERROR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  OPENS ALL MERLIN UNITS, EXCEPT THE OUTPUT UNIT.
C
      LOGICAL ERROR
C
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
C
      LOGICAL HEADER, PRECON
      CHARACTER*15 IN, OUT
      CHARACTER*80 PROLO, EPILO
      COMMON /CP1/ BI, BL, SM, NOVA, NOTE, MODPRO, MUIN, MUOUT,
     >             HEADER,  PRECON 
      COMMON /CP2/ PROLO, EPILO, IN, OUT
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      LOGICAL ADOPI, ADOPO
      COMMON /ADDIOP/ ADOPI, ADOPO
C
      LOGICAL OPE
C
      ADOPI = .FALSE.
      ERROR = .TRUE.
      IF (IN.EQ.' ') THEN
         IF (.NOT.PRECON) THEN
            OPEN(IU31,IOSTAT=IERR)
         IF (IERR.NE.0) THEN
            WRITE (IUOUT,10) IERR
            RETURN
         END IF
         END IF
      ELSE
         IF (PRECON) THEN
            INQUIRE(FILE=IN,OPENED=OPE,NUMBER=IU,IOSTAT=IERR)
         IF (IERR.EQ.0) THEN
            IF (OPE) THEN
               IU31 = IU
            ELSE
                  IU31 = NUNIT()
                  IF (IU32.EQ.-1) THEN
                  WRITE (IUOUT,20)
                  RETURN
                  ELSE
                     OPEN(IU31,FILE=IN,IOSTAT=IERR)
                  IF (IERR.NE.0) THEN
                     WRITE (IUOUT,30) IN(1:LENGTH(IN)), IERR
                     RETURN
                  END IF
               ADOPI = .TRUE.
                  END IF
            END IF
         END IF
         ELSE
            IU31 = NUNIT()
            IF (IU32.EQ.-1) THEN
            WRITE (IUOUT,20)
            RETURN
            ELSE
               OPEN(IU31,FILE=IN,IOSTAT=IERR)
            IF (IERR.NE.0) THEN
               WRITE (IUOUT,30) IN(1:LENGTH(IN)), IERR
               RETURN
            END IF
            END IF
         END IF
      END IF
      IUINP = IU31
C
C  OPEN THE MERLIN FILE: DATA
      CALL OPEN1(IUDAT,'DATA',ERROR)
      IF (ERROR) RETURN
C
C  OPEN THE MERLIN FILE: INIPO
      CALL OPEN1(IUINI,'INIPO',ERROR)
      IF (ERROR) RETURN
C
C  OPEN THE MERLIN FILE: BACKUP
      CALL OPEN1(IUBAC,'BACKUP',ERROR)
      IF (ERROR) RETURN
C
C  OPEN THE MERLIN FILE: MACROF
      CALL OPEN1(IUMAC,'MACROF',ERROR)
      IF (ERROR) RETURN
C
C  OPEN THE MERLIN FILE: HELP
      CALL OPEN1(IU42,'HELP',ERROR)
      IF (ERROR) RETURN
C
C  OPEN THE MERLIN FILE: STORE
      CALL OPEN1(IUSTO,'STORE',ERROR)
      IF (ERROR) RETURN
C
C  OPEN THE MERLIN FILE: DISPO
      CALL OPEN1(IUDIS,'DISPO',ERROR)
      IF (ERROR) RETURN
C
C  IU45 IS RESERVED BY MERLIN FOR INTERNAL USE.
      CALL OPEN1(IU45,'SCRATCH',ERROR)
      IF (ERROR) RETURN
C
10    FORMAT (T2,'### THE MERLIN INPUT FILE COULD NOT BE OPENED,'/
     >        T2,'### DUE TO AN ERROR. IOSTAT = ',I6)
20    FORMAT (T2,'### THE MERLIN INPUT FILE COULD NOT BE OPENED,'/
     >        T2,'### BECAUSE NO MORE UNIT NUMBERS ARE AVAILABLE.')
30    FORMAT (T2,'### THE MERLIN INPUT FILE: ',A,' COULD NOT BE ',
     >        'OPENED,'/T2,'### DUE TO AN ERROR. IOSTAT = ',I6)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE FCLOSE
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  CLOSES ALL MERLIN UNITS, EXCEPT THE OUTPUT ONE.
C
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      LOGICAL HEADER, PRECON
      CHARACTER*15 IN, OUT
      CHARACTER*80 PROLO, EPILO
      COMMON /CP1/ BI, BL, SM, NOVA, NOTE, MODPRO, MUIN, MUOUT,
     >             HEADER,  PRECON 
      COMMON /CP2/ PROLO, EPILO, IN, OUT
      LOGICAL ADOPI, ADOPO
      COMMON /ADDIOP/ ADOPI, ADOPO
      LOGICAL ERROR
C
      IF (IN.EQ.' ') THEN
         IF (.NOT.PRECON) THEN
            CLOSE(IU31,IOSTAT=IERR)
         IF (IERR.NE.0) WRITE (IUOUT,10) IERR
         END IF
      ELSE
         IF (PRECON) THEN
            IF (ADOPI) THEN
               CLOSE(IU31,IOSTAT=IERR)
            IF (IERR.NE.0) WRITE (IUOUT,20) IN(1:LENGTH(IN)), IERR
         END IF
         ELSE
            CLOSE(IU31,IOSTAT=IERR)
         IF (IERR.NE.0) WRITE (IUOUT,20) IN(1:LENGTH(IN)), IERR
         END IF
      END IF
C
C  CLOSE THE MERLIN FILE: DATA
      CALL CLOSE1(IUDAT,ERROR)
C
C  CLOSE THE MERLIN FILE: INIPO
      CALL CLOSE1(IUINI,ERROR)
C
C  CLOSE THE MERLIN FILE: BACKUP
      CALL CLOSE1(IUBAC,ERROR)
C
C  CLOSE THE MERLIN FILE: MACROF
      CALL CLOSE1(IUMAC,ERROR)
C
C  CLOSE THE MERLIN FILE: HELP
      CALL CLOSE1(IU42,ERROR)
C
C  CLOSE THE MERLIN FILE: STORE
      CALL CLOSE1(IUSTO,ERROR)
C
C  CLOSE THE MERLIN FILE: DISPO
      CALL CLOSE1(IUDIS,ERROR)
C
C  IU45 IS RESERVED BY MERLIN FOR INTERNAL USE.
      CALL CLOSE1(IU45,ERROR)
C
10    FORMAT (T2,'### THE MERLIN INPUT FILE COULD NOT BE CLOSED,'/
     >        T2,'### DUE TO AN ERROR. IOSTAT = ',I6)
20    FORMAT (T2,'### THE MERLIN INPUT FILE: ',A,' COULD NOT BE ',
     >        'CLOSED,'/T2,'### DUE TO AN ERROR. IOSTAT = ',I6)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE OPEN1 ( IU, FNAME, ERROR )
      CHARACTER*(*) FNAME
      LOGICAL ERROR
C
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      IU = NUNIT()
      IF (IU.EQ.-1) THEN
         IF (FNAME.NE.'SCRATCH') THEN
            WRITE (IUOUT,40) FNAME
         ELSE
            WRITE (IUOUT,10)
         END IF
         ERROR = .TRUE.
      ELSE
         IF (FNAME.NE.'SCRATCH') THEN
            OPEN(IU,FILE=FNAME,STATUS='UNKNOWN',IOSTAT=IERR)
         ELSE
            OPEN(IU,STATUS='SCRATCH',IOSTAT=IERR)
         END IF
         IF (IERR.NE.0) THEN
            IF (FNAME.NE.'SCRATCH') THEN
               WRITE (IUOUT,20) FNAME, IERR
         ELSE
               WRITE (IUOUT,30) IERR
         END IF
            ERROR = .TRUE.
         ELSE
            ERROR = .FALSE.
         END IF
      END IF
C
10    FORMAT (T2,'### A MERLIN SCRATCH FILE COULD NOT BE OPENED,'/
     >        T2,'### BECAUSE NO MORE UNIT NUMBERS ARE AVAILABLE.')
40    FORMAT (T2,'### THE MERLIN FILE: ',A,' COULD NOT BE OPENED,'/
     >        T2,'### BECAUSE NO MORE UNIT NUMBERS ARE AVAILABLE.')
20    FORMAT (T2,'### THE MERLIN FILE: ',A,' COULD NOT BE OPENED,'/
     >        T2,'### DUE TO AN ERROR. IOSTAT = ',I6)
30    FORMAT (T2,'### A MERLIN SCRATCH FILE COULD NOT BE OPENED,'/
     >        T2,'### DUE TO AN ERROR. IOSTAT = ',I6)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE CLOSE1 ( IU, ERROR )
      LOGICAL ERROR
C
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      CHARACTER*20 FNAME
C
      INQUIRE (UNIT=IU,NAME=FNAME,IOSTAT=IERR)
      IF (IERR.NE.0) FNAME = ' '
      CLOSE (IU,IOSTAT=IERR)
      IF (IERR.NE.0) THEN
         IF (FNAME.EQ.' ') THEN
            WRITE (IUOUT,10) IERR
         ELSE
            LE = LENGTH(FNAME)
         IF (LE.EQ.0) LE = 1
            WRITE (IUOUT,20) FNAME(1:LE), IERR
         END IF
         ERROR = .TRUE.
      ELSE
         ERROR = .FALSE.
      END IF
C
10    FORMAT (T2,'### A MERLIN SCRATCH FILE COULD NOT BE CLOSED,'/
     >        T2,'### DUE TO AN ERROR. IOSTAT = ',I6)
20    FORMAT (T2,'### THE MERLIN FILE: ',A,' COULD NOT BE CLOSED,'/
     >        T2,'### DUE TO AN ERROR. IOSTAT = ',I6)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE SPELL (ICANC)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    AUTO      CONTROL PANEL
C    ICANC IS OUTPUT.
C    ICANC=1,0  TO PERMIT OR NOT SUBSEQUENT CALL TO
C    SUBROUTINE ** MAGIC ** FOR 'AUTO' MINIMIZATION
C
      COMMON/PRILOG/STRO,WEAK
      COMMON/MAGEX/VALOW,NTOT,MADA
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      LOGICAL AIMON
      COMMON/AIM/TARGET,AIMON
      DIMENSION X(4)
      LOGICAL MODEC,STRO,WEAK
C
C  THE USER MAY HAVE ISSUED A "TARGET" COMMAND.
C  SET THE TARGET VALUE IN THE AUTO-PANEL ACCORDINGLY.
      IF (AIMON .AND. VALOW.LT.TARGET) VALOW = TARGET
C
      ICANC=1
      X(1)=NTOT
      X(2)=VALOW
      X(3)=ICANC
      X(4)=MADA
      MODEC=CALLBY.EQ.'NAME'
      CALLBY = 'INDEX'
       IF (STRO)  THEN
        CALL DASH
        WRITE(IUOUT,1)
        CALL DASH
        WRITE(IUOUT,2)
        WRITE(IUOUT,3) NTOT
        WRITE(IUOUT,4) VALOW
        WRITE(IUOUT,6) ICANC
        WRITE(IUOUT,7) MADA
        CALL DASH
       ENDIF
 10   IF (STRO) WRITE(IUOUT,5)
      CALL CHANGE(X,4,IEOR,0)
      IF(MODEC) CALLBY = 'NAME'
      NTOT=NINT(X(1))
      VALOW=X(2)
      ICANC=NINT(X(3))
      MADA=NINT(X(4))
      IF (ICANC.EQ.2) THEN
         X(3) = 1
         GOTO 10
      END IF
C
    1 FORMAT(25X,'  AUTO  - PANEL')
    2 FORMAT(2X,'INDEX',T12,'DESCRIPTION',T40,'VALUE',T52,'MENU')
    3 FORMAT(T5,'1)',T10,'NUMBER OF CALLS ',T38,I6,T52,
     >'ANY INTEGER')
    4 FORMAT(T5,'2)',T10,'TARGET VALUE ',T38,1PE11.4,T52,'ANY REAL')
    5 FORMAT(T5,'ENTER CHANGES')
    6 FORMAT(T5,'3)',T10,'CANCEL-BUTTON',T40,I2,T52,'( 0/1/2 )')
    7 FORMAT(T5,'4)',T10,'GRADIENT MODE (FIXED/MAD)',T40,I2,T52,'(0/1)')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE RANPAN(ICANC)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    RANDOM    CONTROL PANEL
C    ICANC IS OUTPUT.
C    ICANC=1,0  TO PERMIT OR NOT SUBSEQUENT CALL TO
C    SUBROUTINE ** RANDS ** FOR 'RANDOM' MINIMIZATION
C
      COMMON/PRILOG/STRO,WEAK
      COMMON/RANDEX/REDFR,ACCR,EPSR,NOCR,IVOLR,ICYSR,IPRIR,IFAILR,LSEAR
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      DIMENSION X(10)
      LOGICAL MODEC,STRO,WEAK
      ICANC=1
      X(1)=NOCR
      X(2)=IVOLR
      X(3)=REDFR
      X(5)=IFAILR
      X(7)=IPRIR
      X(4)=ICYSR
      X(6)=LSEAR
      X(8)=ICANC
      X(9)=ACCR
      X(10)=EPSR
      MODEC=CALLBY.EQ.'NAME'
      CALLBY = 'INDEX'
       IF (STRO)  THEN
        CALL DASH
        WRITE(IUOUT,1)
        CALL DASH
        WRITE(IUOUT,2)
        WRITE(IUOUT,3) NOCR
        WRITE(IUOUT,4) IVOLR
        WRITE(IUOUT,5) REDFR
        WRITE(IUOUT,8) ICYSR
        WRITE(IUOUT,6) IFAILR
        WRITE(IUOUT,11)LSEAR
        WRITE(IUOUT,7) IPRIR
        WRITE(IUOUT,10) ICANC
        WRITE(IUOUT,12) ACCR
        WRITE(IUOUT,13) EPSR
        CALL DASH
       ENDIF
 20   IF (STRO) WRITE(IUOUT,9)
      CALL CHANGE(X,10,IEOR,0)
      IF(MODEC) CALLBY ='NAME'
      NOCR=NINT(X(1))
      IVOLR=NINT(X(2))
      REDFR=X(3)
      IFAILR=NINT(X(5))
      IPRIR=NINT(X(7))
      ICYSR=NINT(X(4))
      LSEAR=NINT(X(6))
      ICANC=NINT(X(8))
      ACCR=X(9)
      EPSR=X(10)
      IF (ICANC.EQ.2) THEN
         X(8) = 1
         GOTO 20
      END IF
C
    1 FORMAT(25X,' RANDOM - PANEL')
    2 FORMAT(2X,'INDEX',T12,'DESCRIPTION',T40,'VALUE',T52,'MENU')
    3 FORMAT(T5,'1)',T10,'NUMBER OF CALLS ',T38,I6,T52,
     >'ANY INTEGER')
    4 FORMAT(T5,'2)',T10,'ACTIVATE/DEACTIVATE',T40,I4,T52,'(0/1)'/
     >T10,'VOLUME EXCLUSION')
    5 FORMAT(T5,'3)',T10,'STEP FACTOR',T37,1PG9.2,T52,
     >'ANY REAL IN (0,1)')
    6 FORMAT(T5,'5)',T10,'NUMBER OF CONSECUTIVE',T38,I4,T52,'ANY INTEGER
     >'/T10,'CYCLE FAILURES ALLOWED')
    7 FORMAT(T5,'7)',T10,'PRINTOUT SELECTION',T40,I2,T52,'0/1/2')
    8 FORMAT(T5,'4)',T10,'CYCLE-SIZE',T39,I3,T52,'ANY INTEGER')
    9 FORMAT(T5,'ENTER CHANGES')
   10 FORMAT(T5,'8)',T10,'CANCEL-BUTTON',T40,I2,T52,'( 0/1/2 )')
   11 FORMAT(T5,'6)',T10,'LINE-SEARCH PERIOD',T39,I3,T52,'ANY INTEGER')
   12 FORMAT(T5,'9)',T10,'TOLERANCE',T38,1PG10.3,T52,
     >'ANY REAL IN (0,1)')
   13 FORMAT(T4,'10)',T10,'MIN. RELATIVE DROP',T38,1PG10.4,
     >       T52,'ANY REAL IN (0,1)')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE CGRPAN(ICANC)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    CONGRA   CONTROL PANEL
C    ICANC IS OUTPUT.
C    ICANC=1,0  TO PERMIT OR NOT SUBSEQUENT CALL TO
C    SUBROUTINE ** COGRMV ** FOR 'CONGRA' MINIMIZATION
C
      COMMON/PRILOG/STRO,WEAK
      COMMON/UPDATE/IGRADU,IHESSU
      CHARACTER*5 UMODE
      COMMON/UPMODE/UMODE
      COMMON/CGREX/ACCC,ERBC,EPSC,NOCC,IPRIC,IDIGC,MADC,METHOC
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/ERRBOU/GLERB
      DIMENSION X(10)
      LOGICAL MODEC,STRO,WEAK
      ICANC=1
      X(1)=NOCC
      X(2)=ACCC
      X(3)=ERBC
      X(4)=0
      IF (IGRADU.EQ.1 .AND. UMODE.EQ.DERIVA) X(4)=1
      X(5)=IPRIC
      X(6)=IDIGC
      X(7)=ICANC
      X(8)=MADC
      X(9)=EPSC
      X(10) = METHOC
      MODEC=CALLBY.EQ.'NAME'
      CALLBY ='INDEX'
      IF (STRO)  THEN
        CALL DASH
        WRITE(IUOUT,1)
        CALL DASH
        WRITE(IUOUT,2)
        WRITE(IUOUT,3) NOCC
        WRITE(IUOUT,4) ACCC
        WRITE(IUOUT,5) ERBC
        WRITE(IUOUT,6) NINT(X(4))
        WRITE(IUOUT,7) IPRIC
        WRITE(IUOUT,8) IDIGC
        WRITE(IUOUT,10) ICANC
        WRITE(IUOUT,12) MADC
        WRITE(IUOUT,13) EPSC
        WRITE(IUOUT,14) METHOC
        CALL DASH
      ENDIF
 20   IF (STRO) WRITE(IUOUT,9)
      CALL CHANGE(X,10,IEOR,0)
      IF(MODEC) CALLBY ='NAME'
      NOCC=NINT(X(1))
      ACCC=X(2)
      ERBC=X(3)
      IPRIC=NINT(X(5))
      IDIGC=NINT(X(6))
      ICANC=NINT(X(7))
      MADC=NINT(X(8))
      EPSC=X(9)
      METHOC = NINT(X(10))
      GLERB = ERBC
      IF (ICANC.EQ.2) THEN
         X(7) = 1
         GOTO 20
      END IF
      IF (ICANC.EQ.1) THEN
         IGRADU=NINT(X(4))
         IF (IGRADU.EQ.1) UMODE = DERIVA
      END IF
C
    1 FORMAT(25X,'  CONGRA - PANEL')
    2 FORMAT(2X,'INDEX',T12,'DESCRIPTION',T40,'VALUE',T52,'MENU')
    3 FORMAT(T5,'1)',T10,'NUMBER OF CALLS ',T38,I6,T52,
     >'ANY INTEGER')
    4 FORMAT(T5,'2)',T10,'TOLERANCE',T38,1PG10.3,T52,
     >'ANY REAL IN (0,1)')
    5 FORMAT(T5,'3)',T10,'ERROR-BOUND',T38,1PG10.3,T52,
     >'ANY REAL IN (0,1)')
    6 FORMAT(T5,'4)',T10,'USE/RECALCULATE GRADIENT',T38,I4,T52,'(1/0)')
    7 FORMAT(T5,'5)',T10,'PRINTOUT SELECTION',T40,I2,T52,'0/1/2')
    8 FORMAT(T5,'6)',T10,'WALL-PARAMETER',T39,I3,T52,'ANY INTEGER')
    9 FORMAT(T5,'ENTER CHANGES')
   10 FORMAT(T5,'7)',T10,'CANCEL-BUTTON',T40,I2,T52,'( 0/1/2 )')
   12 FORMAT(T5,'8)',T10,'GRADIENT MODE (FIXED/MAD)',T40,I2,T52,'(0/1)')
   13 FORMAT(T5,'9)',T10,'MIN. RELATIVE DROP',T38,1PG10.4,
     >       T52,'ANY REAL IN (0,1)')
   14 FORMAT(T4'10)',T10,'METHOD  ( PR / FR )',T40,I2,T52,'( 1/2 )')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE DFPPAN(ICANC)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    DFP    CONTROL PANEL
C    ICANC IS OUTPUT.
C    ICANC=1,0  TO PERMIT OR NOT SUBSEQUENT CALL TO
C    SUBROUTINE ** DFPMV ** FOR 'DFP' MINIMIZATION
C
      COMMON/PRILOG/STRO,WEAK
      COMMON/UPDATE/IGRADU,IHESSU
      CHARACTER*5 UMODE
      COMMON/UPMODE/UMODE
      COMMON/DFPEX/ACCD,ERBD,EPSD,NOCD,IPRID,IDIGD,MADD
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/ERRBOU/GLERB
      DIMENSION X(10)
      LOGICAL MODEC,STRO,WEAK
      ICANC=1
      X(1)=NOCD
      X(2)=ACCD
      X(3)=ERBD
      X(4)=0
      IF (IGRADU.EQ.1 .AND. UMODE.EQ.DERIVA) X(4)=1
      X(5)=IHESSU
      X(6)=IPRID
      X(7)=IDIGD
      X(8)=ICANC
      X(9)=MADD
      X(10)=EPSD
      MODEC=CALLBY.EQ.'NAME'
      CALLBY = 'INDEX'
       IF (STRO)  THEN
        CALL DASH
        WRITE(IUOUT,1)
        CALL DASH
        WRITE(IUOUT,2)
        WRITE(IUOUT,3) NOCD
        WRITE(IUOUT,4) ACCD
        WRITE(IUOUT,5) ERBD
        WRITE(IUOUT,6) NINT(X(4))
        WRITE(IUOUT,11) IHESSU
        WRITE(IUOUT,7) IPRID
        WRITE(IUOUT,8) IDIGD
        WRITE(IUOUT,10) ICANC
        WRITE(IUOUT,12) MADD
        WRITE(IUOUT,13) EPSD
        CALL DASH
       ENDIF
 20   IF (STRO) WRITE(IUOUT,9)
      CALL CHANGE(X,10,IEOR,0)
      IF(MODEC) CALLBY = 'NAME'
      NOCD=NINT(X(1))
      ACCD=X(2)
      ERBD=X(3)
      IPRID=NINT(X(6))
      IDIGD=NINT(X(7))
      ICANC=NINT(X(8))
      IHESSU=NINT(X(5))
      MADD=NINT(X(9))
      EPSD=X(10)
      GLERB = ERBD
      IF (ICANC.EQ.2) THEN
         X(8) = 1
         GOTO 20
      END IF
      IF (ICANC.EQ.1) THEN
         IGRADU=NINT(X(4))
         IF (IGRADU.EQ.1) UMODE = DERIVA
      END IF
C
    1 FORMAT(25X,'  DFP - PANEL')
    2 FORMAT(2X,'INDEX',T12,'DESCRIPTION',T40,'VALUE',T52,'MENU')
    3 FORMAT(T5,'1)',T10,'NUMBER OF CALLS ',T38,I6,T52,
     >'ANY INTEGER')
    4 FORMAT(T5,'2)',T10,'TOLERANCE',T38,1PG10.3,T52,
     >'ANY REAL IN (0,1)')
    5 FORMAT(T5,'3)',T10,'ERROR-BOUND',T37,1PG10.3,T52,
     >'ANY REAL IN (0,1)')
    6 FORMAT(T5,'4)',T10,'USE/RECALCULATE GRADIENT',T38,I4,T52,'(1/0)')
    7 FORMAT(T5,'6)',T10,'PRINTOUT SELECTION',T40,I2,T52,'0/1/2')
    8 FORMAT(T5,'7)',T10,'WALL-PARAMETER',T39,I3,T52,'ANY INTEGER')
    9 FORMAT(T5,'ENTER CHANGES')
   10 FORMAT(T5,'8)',T10,'CANCEL-BUTTON',T40,I2,T52,'( 0/1/2 )')
   11 FORMAT(T5,'5)',T10,'USE/RESET  HESSIAN',T38,I4,T52,'(1/0)')
   12 FORMAT(T5,'9)',T10,'GRADIENT MODE (FIXED/MAD)',T40,I2,T52,'(0/1)')
   13 FORMAT(T4,'10)',T10,'MIN. RELATIVE DROP',T38,1PG10.4,
     >       T52,'ANY REAL IN (0,1)')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE BFGPAN(ICANC)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    BFGS    CONTROL PANEL
C    ICANC IS OUTPUT.
C    ICANC=1,0  TO PERMIT OR NOT SUBSEQUENT CALL TO
C    SUBROUTINE ** BFGSMV ** FOR 'BFGS' MINIMIZATION
C
      COMMON/PRILOG/STRO,WEAK
      COMMON/UPDATE/IGRADU,IHESSU
      CHARACTER*5 UMODE
      COMMON/UPMODE/UMODE
      COMMON/BFGSEX/ACCB,ERBB,EPSB,NOCB,IPRIB,IDIGB,MADB
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/ERRBOU/GLERB
      DIMENSION X(10)
      LOGICAL MODEC,STRO,WEAK
      ICANC=1
      X(1)=NOCB
      X(2)=ACCB
      X(3)=ERBB
      X(4)=0
      IF (IGRADU.EQ.1 .AND. UMODE.EQ.DERIVA) X(4)=1
      X(5)=IHESSU
      X(6)=IPRIB
      X(7)=IDIGB
      X(8)=ICANC
      X(9)=MADB
      X(10)=EPSB
      MODEC=CALLBY.EQ.'NAME'
      CALLBY = 'INDEX'
      IF (STRO)  THEN
        CALL DASH
        WRITE(IUOUT,1)
        CALL DASH
        WRITE(IUOUT,2)
        WRITE(IUOUT,3) NOCB
        WRITE(IUOUT,4) ACCB
        WRITE(IUOUT,5) ERBB
        WRITE(IUOUT,6) NINT(X(4))
        WRITE(IUOUT,11) IHESSU
        WRITE(IUOUT,7) IPRIB
        WRITE(IUOUT,8) IDIGB
        WRITE(IUOUT,10) ICANC
        WRITE(IUOUT,12) MADB
        WRITE(IUOUT,13) EPSB
        CALL DASH
      ENDIF
 20   IF (STRO) WRITE(IUOUT,9)
      CALL CHANGE(X,10,IEOR,0)
      IF(MODEC) CALLBY = 'NAME'
      NOCB=NINT(X(1))
      ACCB=X(2)
      ERBB=X(3)
      IPRIB=NINT(X(6))
      IDIGB=NINT(X(7))
      ICANC=NINT(X(8))
      IHESSU=NINT(X(5))
      MADB=NINT(X(9))
      EPSB=X(10)
      GLERB = ERBB
      IF (ICANC.EQ.2) THEN
         X(8) = 1
         GOTO 20
      END IF
      IF (ICANC.EQ.1) THEN
         IGRADU=NINT(X(4))
         IF (IGRADU.EQ.1) UMODE = DERIVA
      END IF
C
    1 FORMAT(25X,'  BFGS - PANEL')
    2 FORMAT(2X,'INDEX',T12,'DESCRIPTION',T40,'VALUE',T52,'MENU')
    3 FORMAT(T5,'1)',T10,'NUMBER OF CALLS ',T38,I6,T52,
     >'ANY INTEGER')
    4 FORMAT(T5,'2)',T10,'TOLERANCE',T38,1PG10.3,T52,
     >'ANY REAL IN (0,1)')
    5 FORMAT(T5,'3)',T10,'ERROR-BOUND',T37,1PG9.2,T52,'ANY REAL < 1')
    6 FORMAT(T5,'4)',T10,'USE/RECALCULATE GRADIENT',T38,I4,T52,'(1/0)')
    7 FORMAT(T5,'6)',T10,'PRINTOUT SELECTION',T40,I2,T52,'0/1/2')
    8 FORMAT(T5,'7)',T10,'WALL-PARAMETER',T39,I3,T52,'ANY INTEGER')
    9 FORMAT(T5,'ENTER CHANGES')
   10 FORMAT(T5,'8)',T10,'CANCEL-BUTTON',T40,I2,T52,'( 0/1/2 )')
   11 FORMAT(T5,'5)',T10,'USE/RESET  HESSIAN',T38,I4,T52,'(1/0)')
   12 FORMAT(T5,'9)',T10,'GRADIENT MODE (FIXED/MAD)',T40,I2,T52,'(0/1)')
   13 FORMAT(T4,'10)',T10,'MIN. RELATIVE DROP',T38,1PG10.4,
     >       T52,'ANY REAL IN (0,1)')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE ROLPAN(ICANC)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    ROLL    CONTROL PANEL
C    ICANC IS OUTPUT.
C    ICANC=1,0  TO PERMIT OR NOT SUBSEQUENT CALL TO
C    SUBROUTINE ** HYPER ** FOR 'ROLL' MINIMIZATION
C
      COMMON/PRILOG/STRO,WEAK
      COMMON/ROLLEX/ACCH,ENHH,EPSH,NOCH,IFMH,IPRIH,IDIGH
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      DIMENSION X(8)
      LOGICAL MODEC,STRO,WEAK
      ICANC=1
      X(1)=NOCH
      X(2)=ACCH
      X(3)=ENHH
      X(4)=IFMH
      X(5)=IPRIH
      X(6)=IDIGH
      X(7)=ICANC
      X(8)=EPSH
      MODEC=CALLBY.EQ.'NAME'
      CALLBY='INDEX'
       IF (STRO)  THEN
        CALL DASH
        WRITE(IUOUT,1)
        CALL DASH
        WRITE(IUOUT,2)
        WRITE(IUOUT,3) NOCH
        WRITE(IUOUT,4) ACCH
        WRITE(IUOUT,5) ENHH
        WRITE(IUOUT,6) IFMH
        WRITE(IUOUT,7) IPRIH
        WRITE(IUOUT,8) IDIGH
        WRITE(IUOUT,10) ICANC
        WRITE(IUOUT,11) EPSH
        CALL DASH
        ENDIF
 20   IF (STRO) WRITE(IUOUT,9)
      CALL CHANGE(X,8,IEOR,0)
      IF(MODEC) CALLBY = 'NAME'
      NOCH=NINT(X(1))
      ACCH=X(2)
      ENHH=X(3)
      IFMH=NINT(X(4))
      IPRIH=NINT(X(5))
      IDIGH=NINT(X(6))
      ICANC=NINT(X(7))
      EPSH=X(8)
      IF (ICANC.EQ.2) THEN
         X(7) = 1
         GOTO 20
      END IF
C
    1 FORMAT(25X,'  ROLL - PANEL')
    2 FORMAT(2X,'INDEX',T12,'DESCRIPTION',T40,'VALUE',T52,'MENU')
    3 FORMAT(T5,'1)',T10,'NUMBER OF CALLS ',T38,I6,T52,
     >'ANY INTEGER')
    4 FORMAT(T5,'2)',T10,'TOLERANCE',T38,1PG10.3,T52,
     >'ANY REAL IN (0,1)')
    5 FORMAT(T5,'3)',T10,'STEP FACTOR',T37,1PG9.2,T52,'ANY REAL > 1')
    6 FORMAT(T5,'4)',T10,'FAILURES ALLOWED',T38,I4,T52,'ANY INTEGER')
    7 FORMAT(T5,'5)',T10,'PRINTOUT SELECTION',T40,I2,T52,'0/1/2')
    8 FORMAT(T5,'6)',T10,'WALL-PARAMETER',T39,I3,T52,'ANY INTEGER')
    9 FORMAT(T5,'ENTER CHANGES')
   10 FORMAT(T5,'7)',T10,'CANCEL-BUTTON',T40,I2,T52,'( 0/1/2 )')
   11 FORMAT(T5,'8)',T10,'MIN. RELATIVE DROP',T38,1PG10.4,
     >       T52,'ANY REAL IN (0,1)')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE GRPAN(ICANC)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    GRAPH CONTROL PANEL
C    ICANC IS OUTPUT.
C    ICANC=1,0  TO PERMIT OR NOT SUBSEQUENT CALL TO
C    SUBROUTINE ** GRAPH ** FOR LINE PRINTER GRAPHS
C
      PARAMETER ( MXV = 200 )
      LOGICAL MODEC,STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/GRCOM/GXMIN,GXMAX,XAX,YAX,NOCGR,ILIN,ICOL,IWVAR
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      DIMENSION X(7)
      ICANC=1
      X(1)=IWVAR
      X(2)=NOCGR
      X(3)=GXMIN
      X(4)=GXMAX
      X(5)=ILIN
      X(6)=ICOL
      X(7)=ICANC
      XAX=0.
      YAX=0.
      MODEC=CALLBY.EQ.'NAME'
      CALLBY='INDEX'
      IF (STRO) THEN
         CALL DASH
         WRITE(IUOUT,1)
         CALL DASH
         WRITE(IUOUT,2)
         WRITE(IUOUT,3) IWVAR,NOD
         WRITE(IUOUT,4) NOCGR
         WRITE(IUOUT,5) GXMIN
         WRITE(IUOUT,6) GXMAX
         WRITE(IUOUT,7) ILIN
         WRITE(IUOUT,8) ICOL
         WRITE(IUOUT,10) ICANC
         CALL DASH
      END IF
 20   IF(STRO)WRITE(IUOUT,9)
      CALL CHANGE(X,7,IEOR,0)
      IF(MODEC) CALLBY = 'NAME'
      IWVAR=NINT(X(1))
      NOCGR=NINT(X(2))
      GXMIN=X(3)
      GXMAX=X(4)
      ILIN=NINT(X(5))
      ICOL=NINT(X(6))
      ICANC=NINT(X(7))
      IF (ICANC.EQ.2) THEN
         X(7) = 1
         GOTO 20
      END IF
C
    1 FORMAT(25X,' GRAPH  - PANEL')
    2 FORMAT(2X,'INDEX',T12,'DESCRIPTION',T40,'VALUE',T52,'MENU')
    3 FORMAT(T5,'1)',T10,' VARIABLE   INDEX   ',T40,I3,T52,'(1 -',I3
     >,')')
    4 FORMAT(T5,'2)',T10,'N0 OF POINTS  ',T40,I3,T52,'ANY INTEGER')
    5 FORMAT(T5,'3)',T10,'FROM',T39,1PG10.3,T52)
    6 FORMAT(T5,'4)',T10,'TO  ',T39,1PG10.3,T52)
    7 FORMAT(T5,'5)',T10,' LINES ',T40,I3,T52,'ANY INTEGER')
    8 FORMAT(T5,'6)',T10,'COLOUMNS',T40,I3,T52,'(2-124)')
    9 FORMAT(T5,'ENTER CHANGES')
   10 FORMAT(T5,'7)',T10,'CANCEL-BUTTON',T40,I2,T52,'( 0/1/2 )')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE SIMPAN(ICANC)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    SIMPLEX CONTROL PANEL
C    ICANC IS OUTPUT.
C    ICANC=1,0  TO PERMIT OR NOT SUBSEQUENT CALL TO
C    SUBROUTINE ** SIMPL ** FOR SIMPLEX MINIMIZATION
C
      COMMON/PRILOG/STRO,WEAK
      COMMON/SIMPEX/ERR1,ERR,INIS,MNOC1,MNOC,IPRIS
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      DIMENSION X(7)
      LOGICAL MODEC,STRO,WEAK
      ICANC=1
      X(1)=INIS
      X(2)=ERR1
      X(3)=MNOC1
      X(4)=ERR
      X(5)=MNOC
      X(6)=IPRIS
      X(7)=ICANC
      MODEC=CALLBY.EQ.'NAME'
      CALLBY = 'INDEX'
       IF(STRO) THEN
        CALL DASH
        WRITE(IUOUT,1)
        CALL DASH
        WRITE(IUOUT,2)
        WRITE(IUOUT,3) INIS
        WRITE(IUOUT,4) ERR1
        WRITE(IUOUT,5) MNOC1
        WRITE(IUOUT,6) ERR
        WRITE(IUOUT,7) MNOC
        WRITE(IUOUT,8) IPRIS
        WRITE(IUOUT,10) ICANC
        CALL DASH
       ENDIF
 20   IF (STRO) WRITE(IUOUT,9)
      CALL CHANGE(X,7,IEOR,0)
      IF(MODEC) CALLBY = 'NAME'
      INIS=NINT(X(1))
      ERR1=X(2)
      MNOC1=NINT(X(3))
      ERR=X(4)
      MNOC=NINT(X(5))
      IPRIS=NINT(X(6))
      ICANC=NINT(X(7))
      IF (ICANC.EQ.2) THEN
         X(7) = 1
         GOTO 20
      END IF
C
    1 FORMAT(25X,'SIMPLEX - PANEL')
    2 FORMAT(2X,'INDEX',T12,'DESCRIPTION',T40,'VALUE',T52,'MENU')
    3 FORMAT(T5,'1)',T10,'INITIALIZATION SCHEME',T45,I1,T52,'(1/2)'/
     >T10,'(SYSTEMATIC/RANDOM)')
    4 FORMAT(T5,'2)',T10,'INITIALIZATION TOLERANCE',T39,1PG10.3
     >,T52,'ANY REAL')
    5 FORMAT(T5,'3)',T10,'INIT. CALLS/VARIABLE',T40,I5,T52
     >,'ANY INTEGER')
    6 FORMAT(T5,'4)',T10,'SIMPLEX TOLERANCE',T39,1PG10.3,T52,'ANY REAL')
    7 FORMAT(T5,'5)',T10,'SIMPLEX CALLS',T40,I5,T52,'ANY INTEGER')
    8 FORMAT(T5,'6)',T10,'PRINTOUT SELECTION',T43,I1,T51,'(0/1/2)')
    9 FORMAT(T5,'ENTER CHANGES')
   10 FORMAT(T5,'7)',T10,'CANCEL-BUTTON',T40,I2,T52,'( 0/1/2 )')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE MADPAN
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    MAD    CONTROL PANEL
C
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/MADEX/TELOW,TEHIGH,FQTHRE,QNTHRE,IPRIM
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      DIMENSION X(5)
      LOGICAL MODEC
C
      X(1)=TELOW
      X(2)=TEHIGH
      X(3)=FQTHRE
      X(4)=QNTHRE
      X(5)=IPRIM
      MODEC=CALLBY.EQ.'NAME'
      CALLBY = 'INDEX'
      IF (STRO)  THEN
        CALL DASH
        WRITE(IUOUT,1)
        CALL DASH
        WRITE(IUOUT,2)
        WRITE(IUOUT,3) TELOW
        WRITE(IUOUT,4) TEHIGH
        WRITE(IUOUT,5) FQTHRE
        WRITE(IUOUT,6) QNTHRE
        WRITE(IUOUT,7) IPRIM
        CALL DASH
      END IF
 20   IF (STRO) WRITE(IUOUT,8)
      CALL CHANGE(X,5,IEOR,0)
      IF (MODEC) CALLBY = 'NAME'
      TELOW=X(1)
      TEHIGH=X(2)
      FQTHRE=X(3)
      QNTHRE=X(4)
      IPRIM=NINT(X(5))
C
    1 FORMAT(25X,'   MAD - PANEL')
    2 FORMAT(2X,'INDEX',T12,'DESCRIPTION',T40,'VALUE',T52,'MENU')
    3 FORMAT(T5,'1)',T10,'MINIMUM TAYLOR ERROR',T38,1PG10.3,T52,
     >       'ANY REAL IN (0,1)')
    4 FORMAT(T5,'2)',T10,'MAXIMUM TAYLOR ERROR',T38,1PG10.3,T52,
     >       'ANY REAL IN (0,1)')
    5 FORMAT(T5,'3)',T10,'FAST <==> QUAD THRESHOLD',T38,1PG10.3,T52,
     >       'ANY REAL IN (0,1)')
    6 FORMAT(T5,'4)',T10,'QUAD <==> NUMER THRESHOLD',T38,1PG10.3,T52,
     >       'ANY REAL IN (0,1)')
    7 FORMAT(T5,'5)',T10,'MAD PRINTOUT (OFF/ON)',T43,I1,T52,'(0/1)')
    8 FORMAT(T5,'ENTER CHANGES')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE ACCPAN (ICANC)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    ACCUM      CONTROL PANEL
C    ICANC IS OUTPUT.
C    ICANC=1,0  TO PERMIT OR NOT SUBSEQUENT CALL TO
C    SUBROUTINE ** ACCUMP ** FOR INITIAL POINT ACCUMULATION.
C
      COMMON/PRILOG/STRO,WEAK
      COMMON/ACCEX/TARAC,NOCAC,NOPAC
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      DIMENSION X(4)
      LOGICAL MODEC,STRO,WEAK
      ICANC=1
      X(1)=NOCAC
      X(2)=NOPAC
      X(3)=TARAC
      X(4)=ICANC
      MODEC=CALLBY.EQ.'NAME'
      CALLBY = 'INDEX'
       IF (STRO)  THEN
        CALL DASH
        WRITE(IUOUT,1)
        CALL DASH
        WRITE(IUOUT,2)
        WRITE(IUOUT,3) NOCAC
        WRITE(IUOUT,4) NOPAC
        WRITE(IUOUT,6) TARAC
        WRITE(IUOUT,7) ICANC
        CALL DASH
       ENDIF
 10   IF (STRO) WRITE(IUOUT,5)
      CALL CHANGE(X,4,IEOR,0)
      IF(MODEC) CALLBY = 'NAME'
      NOCAC=NINT(X(1))
      NOPAC=NINT(X(2))
      TARAC=X(3)
      ICANC=NINT(X(4))
      IF (ICANC.EQ.2) THEN
         X(4) = 1
         GOTO 10
      END IF
C
    1 FORMAT(25X,'  ACCUM - PANEL')
    2 FORMAT(2X,'INDEX',T12,'DESCRIPTION',T40,'VALUE',T52,'MENU')
    3 FORMAT(T5,'1)',T10,'NUMBER OF CALLS ',T38,I6,T52,
     >'ANY INTEGER')
    4 FORMAT(T5,'2)',T10,'NUMBER OF POINTS',T38,I6,T52,
     >'ANY INTEGER')
    6 FORMAT(T5,'3)',T10,'TARGET VALUE ',T38,1PG11.4,T52,'ANY REAL')
    5 FORMAT(T5,'ENTER CHANGES')
    7 FORMAT(T5,'4)',T10,'CANCEL-BUTTON',T40,I2,T52,'( 0/1/2 )')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE TOLPAN ( ICANC )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    TOLMIN CONTROL PANEL
C    ICANC IS OUTPUT.
C    ICANC=1,0  TO PERMIT OR NOT SUBSEQUENT CALL TO
C    SUBROUTINE ** TOLI ** FOR 'TOLMIN' MINIMIZATION
C
      COMMON/PRILOG/STRO,WEAK
      COMMON/TOLEX/ACCT,ERBT,NOCT,IPRIT,MADT
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/ERRBOU/GLERB
      DIMENSION X(6)
      LOGICAL MODEC,STRO,WEAK
      ICANC=1
      X(1)=NOCT
      X(2)=ACCT
      X(3)=ERBT
      X(4)=IPRIT
      X(5)=MADT
      X(6)=ICANC
      MODEC=CALLBY.EQ.'NAME'
      CALLBY = 'INDEX'
      IF (STRO)  THEN
        CALL DASH
        WRITE(IUOUT,1)
        CALL DASH
        WRITE(IUOUT,2)
        WRITE(IUOUT,3) NOCT
        WRITE(IUOUT,4) ACCT
        WRITE(IUOUT,5) ERBT
        WRITE(IUOUT,7) IPRIT
        WRITE(IUOUT,12) MADT
        WRITE(IUOUT,10) ICANC
        CALL DASH
      ENDIF
 20   IF (STRO) WRITE(IUOUT,9)
      CALL CHANGE(X,6,IEOR,0)
      IF(MODEC) CALLBY = 'NAME'
      NOCT=NINT(X(1))
      ACCT=X(2)
      ERBT=X(3)
      IPRIT=NINT(X(4))
      MADT=NINT(X(5))
      ICANC=NINT(X(6))
      GLERB = ERBT
      IF (ICANC.EQ.2) THEN
         X(6) = 1
         GOTO 20
      END IF
C
    1 FORMAT(25X,'TOLMIN - PANEL')
    2 FORMAT(2X,'INDEX',T12,'DESCRIPTION',T40,'VALUE',T52,'MENU')
    3 FORMAT(T5,'1)',T10,'NUMBER OF CALLS ',T38,I6,T52,
     >'ANY INTEGER')
    4 FORMAT(T5,'2)',T10,'TERMINATION ACCURACY',T38,1PG10.3,T52,
     >'ANY REAL IN (0,1)')
    5 FORMAT(T5,'3)',T10,'ERROR-BOUND',T37,1PG9.2,T52,'ANY REAL < 1')
    7 FORMAT(T5,'4)',T10,'PRINTOUT SELECTION',T40,I2,T52,'0/1')
   12 FORMAT(T5,'5)',T10,'GRADIENT MODE (FIXED/MAD)',T40,I2,T52,'(0/1)')
   10 FORMAT(T5,'6)',T10,'CANCEL-BUTTON',T40,I2,T52,'( 0/1/2 )')
    9 FORMAT(T5,'ENTER CHANGES')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE LEVPAN ( ICANC )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C    LMDIF CONTROL PANEL
C    ICANC IS OUTPUT.
C    ICANC=1,0  TO PERMIT OR NOT SUBSEQUENT CALL TO
C    SUBROUTINE ** LEVE ** FOR 'LMDIF' MINIMIZATION
C
      COMMON/PRILOG/STRO,WEAK
      COMMON/LEVEX/GTOLL,FTOLL,XTOLL,FACTL,EPSFL,NOCL,IPRIL
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      DIMENSION X(8)
      LOGICAL MODEC,STRO,WEAK
      ICANC=1
      X(1) = NOCL
      X(2) = FTOLL
      X(3) = XTOLL
      X(4) = GTOLL
      X(5) = FACTL
      X(6) = EPSFL
      X(7) = IPRIL
      X(8) = ICANC
      MODEC=CALLBY.EQ.'NAME'
      CALLBY = 'INDEX'
      IF (STRO)  THEN
        CALL DASH
        WRITE(IUOUT,1)
        CALL DASH
        WRITE(IUOUT,2)
        WRITE(IUOUT,3) NOCL
        WRITE(IUOUT,4) FTOLL
        WRITE(IUOUT,5) XTOLL
        WRITE(IUOUT,7) GTOLL
        WRITE(IUOUT,12) FACTL
        WRITE(IUOUT,10) EPSFL
        WRITE(IUOUT,13) IPRIL
        WRITE(IUOUT,14) ICANC
        CALL DASH
      ENDIF
 20   IF (STRO) WRITE(IUOUT,9)
      CALL CHANGE(X,8,IEOR,0)
      IF(MODEC) CALLBY = 'NAME'
      NOCL = NINT(X(1))
      FTOLL = X(2)
      XTOLL = X(3)
      GTOLL = X(4)
      FACTL = X(5)
      EPSFL = X(6)
      IPRIL = NINT(X(7))
      ICANC = NINT(X(8))
      IF (ICANC.EQ.2) THEN
         X(7) = 1
         GOTO 20
      END IF
C
    1 FORMAT(20X,'LEVENBERG-MARQUARD - PANEL')
    2 FORMAT(2X,'INDEX',T12,'DESCRIPTION',T40,'VALUE',T52,'MENU')
    3 FORMAT(T5,'1)',T10,'NUMBER OF CALLS ',T38,I6,T52,
     >'ANY INTEGER')
    4 FORMAT(T5,'2)',T10,'F-TOLERANCE',T38,1PG10.3,T52,
     >'ANY REAL IN (0,1)')
    5 FORMAT(T5,'3)',T10,'X-TOLERANCE',T38,1PG10.3,T52,
     >'ANY REAL IN (0,1)')
    7 FORMAT(T5,'4)',T10,'G-TOLERANCE',T38,1PG10.3,T52,
     >'ANY REAL IN (0,1)')
   12 FORMAT(T5,'5)',T10,'STEP FACTOR',T37,1PG9.2,T52,' (1,100)')
   10 FORMAT(T5,'6)',T10,'FUNCTION ACCURACY',T37,1PG9.2,T52,' (0,1)')
   13 FORMAT(T5,'7)',T10,'PRINTOUT SELECTION',T40,I2,T52,'0/1')
   14 FORMAT(T5,'8)',T10,'CANCEL-BUTTON',T40,I2,T52,'( 0/1/2 )')
    9 FORMAT(T5,'ENTER CHANGES')
      END
C
C
C
C
      SUBROUTINE PICKW(SCRA,I1,I2,IEN)
C
C     SCRA IS AN INPUT VARIABLE OF TYPE CHARACTER*79
C     I1 IS INPUT. IT REFERS TO THE SUBSTRING ELEMENT SCRA(I1:I1)
C      ...PICKW CALCULATES I2 WHERE I2 ALSO REFERS TO SCRA(I2:I2)
C     AND DESIGNATES THE END OF A "WORD" STARTING AT (OR BEFORE) I1.
C     ONE "WORD" IS SEPARATED FROM ANOTHER BY EITHER A COMMA OR ANY
C     NUMBER OF BLANKS
C     IEN IS A FLAG INDICATING WHETHER A WORD IS FOUND (IEN=1) 
C     OR NOT (IEN=0).
C
      CHARACTER*79 SCRA
      IEN=1
       DO 1 I=I1,79
        IF(SCRA(I:I).NE.' '.AND.SCRA(I:I).NE.',')GO TO  2
    1  CONTINUE
      IEN=0
      RETURN
    2 CONTINUE
      I1=I
       DO 3 I=I1,79
         IF(SCRA(I:I).EQ.' '.OR.SCRA(I:I).EQ.',')THEN
          I2=I-1
          GO TO  4
         ENDIF
    3  CONTINUE
      I2=79
    4 CONTINUE
      RETURN
      END
      SUBROUTINE POIMAR ( LABEL )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C       ...POIMAR  CHECKS IF ANY VARIABLE IS OUT OF THE MARGIN RANGE.
C       IN THAT CASE RESETS VARIABLE VALUE TO THE CLOSEST MARGIN
C      AND UPDATES CURRENT FUNCTION VALUE.
C     IF A FIXED VARIABLE IS OUT-OF-MARGIN RANGE ,...POIMAR LOOSES IT
C     AND POSITIONS IT AS DESCRIBED ABOVE.
C  LABEL IS OUTPUT: 0   POINT IS WITHIN THE MARGINS.
C                   1   AT LEAST ONE VARIABLE WAS OUT OF MARGINS, SO THE
C                       OBJECTIVE FUNCTION HAD TO BE EVALUATED.
C
      PARAMETER ( MXV = 200 )
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 NICKNA
      COMMON/NAMES/NICKNA(MXV)
      COMMON /MODE3/ MODEV, IFFORM 
      LABEL=0
       DO 1 I=1,NOD
         IF(MARG(I).EQ.0)THEN
          GO TO  1
         ELSEIF(MARG(I).EQ.-1)THEN
           IF(POINT(I).LT.XLL(I))THEN
            LABEL=1
            IF(STRO.AND.CALLBY.EQ.'INDEX')WRITE(IUOUT,2)I
            IF(STRO.AND.CALLBY.EQ.'NAME')WRITE(IUOUT,3)NICKNA(I)
            POINT(I)=XLL(I)
            IXAT(I)=1
           ENDIF
         ELSEIF(MARG(I).EQ.1)THEN
           IF(POINT(I).GT.XRL(I))THEN
            LABEL=1
            IF(STRO.AND.CALLBY.EQ.'INDEX')WRITE(IUOUT,2)I
            IF(STRO.AND.CALLBY.EQ.'NAME')WRITE(IUOUT,3)NICKNA(I)
            POINT(I)=XRL(I)
            IXAT(I)=1
           ENDIF
         ELSEIF(MARG(I).EQ.2)THEN
           IF(POINT(I).LT.XLL(I))THEN
            LABEL=1
            IF(STRO.AND.CALLBY.EQ.'INDEX')WRITE(IUOUT,2)I
            IF(STRO.AND.CALLBY.EQ.'NAME')WRITE(IUOUT,3)NICKNA(I)
            POINT(I)=XLL(I)
            IXAT(I)=1
           ELSEIF(POINT(I).GT.XRL(I))THEN
            LABEL=1
            IF(STRO.AND.CALLBY.EQ.'INDEX')WRITE(IUOUT,2)I
            IF(STRO.AND.CALLBY.EQ.'NAME')WRITE(IUOUT,3)NICKNA(I)
            POINT(I)=XRL(I)
            IXAT(I)=1
           ENDIF
         ENDIF
    1  CONTINUE
      IF (LABEL.EQ.1 .AND. MODEV.EQ.1) VAL=ACSQ(POINT,NOD)
      RETURN
    2 FORMAT(10X,'*** VARIABLE NO. ',I3,'  WAS OUT-OF-MARGIN RANGE')
    3 FORMAT(10X,'*** VARIABLE : ',A10,' WAS OUT-OF-MARGIN RANGE')
      END
      SUBROUTINE POSIT(IUNIT)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     POSITIONS THE UNIT SPECIFIED BY IUNIT AT THE END OF INFORMATION
C
      CHARACTER TRIC
      REWIND IUNIT
      I=0
    1 CONTINUE
      READ(IUNIT,4,END=2)TRIC
      I=I+1
      GO TO  1
    2 REWIND IUNIT
       DO 3 J=1,I
        READ(IUNIT,4)TRIC
    3  CONTINUE
      RETURN
    4 FORMAT(A1)
      END
      SUBROUTINE PRADJ
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   PRADJ ... IS A STEP-SIZE PROCEDURE. IT DOES DOT REQUIRE THE
C   VALUES OF THE GRADIENT COMPONENTS. IT ALSO PERFORMS A MINOR
C   MINIMIZATION. IT IS INVOKED BY THE * ADJUST * COMMAND.
C
      PARAMETER ( MXV = 200 )
      COMMON/PARAMS/X(MXV),H(MXV),XLL(MXV),XRL(MXV),SML,N
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      DIMENSION Y(MXV)
      LOGICAL LM, RM
C
       DO 3 I=1,N
        IF(IXAT(I).EQ.0)GO TO  3
        LM = MARG(I).EQ.-1 .OR. MARG(I).EQ.2
        RM = MARG(I).EQ.1  .OR. MARG(I).EQ.2
        ISTORY=0
        CALL POIMAR(IFEV)
         DO 1 II=1,N
          IF(X(II).EQ.(X(II)+H(II))) H(II)=(1.+ABS(X(II)))*COMTOL
    1    Y(II)=X(II)
        JJJ=0
    2   CONTINUE
        JJJ=JJJ+1
        IF(JJJ.GT.20)GO TO  3
        Y(I)=X(I)+H(I)
        IF(LM .AND. Y(I).LT.XLL(I)) Y(I)=XLL(I)
        IF(RM .AND. Y(I).GT.XRL(I)) Y(I)=XRL(I)
        TRY=ACSQ(Y,N)
        H1=Y(I)-X(I)
        FP1=0.
        IF(H1.NE.0.)FP1=(TRY-SML)/H1
         IF(TRY.LT.SML)THEN
          SML=TRY
          X(I)=Y(I)
          H(I)=3.0*H(I)
          ISTORY=1
          GO TO  2
         ELSE
           IF(ISTORY.EQ.1)THEN
            H(I)=H(I)/2.
            GO TO  3
           ELSE
            Y(I)=X(I)-H(I)
            IF(LM .AND. Y(I).LT.XLL(I)) Y(I)=XLL(I)
            IF(RM .AND. Y(I).GT.XRL(I)) Y(I)=XRL(I)
            TRYS=ACSQ(Y,N)
            H2=Y(I)-X(I)
            FP2=0.
            IF(H2.NE.0.)FP2=(TRYS-SML)/H2
             IF(TRYS.LT.SML)THEN
              SML=TRYS
              H(I)=-H(I)*3.0
              X(I)=Y(I)
              ISTORY=1
              GO TO  2
             ELSE
               IF(ABS(FP1-FP2).GT.1.E-30) THEN
                H(I)=-.5*(H1*FP2-H2*FP1)/(FP1-FP2)
                GO TO  3
               ELSE
                H(I)=H(I)/2.
                IF(TRY.GT.TRYS)H(I)=-H(I)
                GO TO  2
               ENDIF
             ENDIF
           ENDIF
         ENDIF
    3  CONTINUE
      RETURN
      END
      SUBROUTINE RANDS ( NOCA, IPRI, NOCEX, IREL, IOPT, REDF,
     >                   ICYC, MNOCY, LSEA, ACCR, EPSR )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   RANDS ... MINIMIZES THE FUNCTION ACSQ USING A RANDOM SEARCH.
C   INPUT
C   NOCA   APPROXIMATELY MAXIMUM NUMBER OF ACSQ EVALUATIONS ALLOWED.
C   IPRI   = 0,1,2 ARRANGES THE PRINTOUT LEVEL.
C   IREL   = 0,1 ACTIVATES OR NOT VOLUME EXCLUSION.
C   IOPT   = 1,0 ACTIVATES OR NOT LINE-SEARCHES.
C   RED    STEP REDUCTION FACTOR.
C   ICYC   NUMBER OF ACSQ EVALUATIONS DEFINING A CYCLE.
C   MNOCY  NUMBER OF MAXIMUM CONSECUTIVE CYCLE FAILURES ALLOWED.
C   LSEA   IF IOPT = 1 DEFINES THE PERIOD OF THE LINE-SEARCH.
C   ACCR   LINE SEARCH TOLERANCE.
C   EPSR   MINIMUM RELATIVE DROP ACCEPTED.
C   OUTPUT
C   NOCEX  NUMBER OF ACSQ EVALUATIONS.
C
      PARAMETER ( MXV = 200 )
      COMMON/CWORK1/XT(MXV),XT1(MXV),XT2(MXV),XT3(MXV),XT4(MXV)
      COMMON/TOBOX/SL(MXV),SR(MXV),IFAIL,IRELAX
      COMMON/DIRCOM/DIRCOS(MXV)
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
C  INITIALIZE TERMINATION CRITERIA SETTINGS.
      CALL INITER ( IPRI, NOCA, EPSR )
C  SET INITIAL VALUES TO RELEVANT PARAMETERS
       DO 1 I=1,NOD
       XT(I)=POINT(I)
    1  DIRCOS(I)=0.
      NOCY=0
      ICOS=0
      IRELAX=IREL
      IFAIL=0
      NOCEX=0
      NFAIL=0
      ICUT=ICYC
      IF (ITERM ( 1, 0, 0 ).NE.0) GO TO 50
C  START ITERATION LOOP
       DO 5 ITER=1,NOCA
C  SAMPLE  A TRIAL POINT
        CALL BOXSAM
C  EVALUATE OBJECTIVE FUNCTION
        FT=ACSQ(XT,NOD)
C  COUNT EVALUATION OF THE OBJECTIVE FUNCTION
        NOCEX=NOCEX+1
C  CHECK IF SUCCESFUL
         IF(FT.LT.VAL) THEN
          ICOS=ICOS+1
          IFAIL=0
          ICUT=ICYC
          NOCY=0
          NFAIL=0
          VAL=FT
           DO 2 I=1,NOD
            DIRCOS(I)=XT(I)-POINT(I)+DIRCOS(I)
            POINT(I)=XT(I)
            STEP(I)=MAX(ABS(STEP(I)),ABS(SR(I)-SL(I)))
    2      CONTINUE
          CALL DISPLA
          IF (ITERM ( 1, 1, 1 ) .NE. 0) GO TO 50
C  FAILURE
         ELSE
          IFAIL=1
          NFAIL=NFAIL+1
           IF(NFAIL.GE.ICUT) THEN
            ICUT=NFAIL+ICYC
             IF(IRELAX.EQ.1) THEN
               DO 3 I=1,NOD
                SL(I)=SL(I)*REDF
                SR(I)=SR(I)*REDF
                STEP(I)=STEP(I)*REDF
    3          CONTINUE
             ENDIF
            NOCY=NOCY+1
            IF(NOCY.GT.MNOCY) THEN
C  CYCLE FAILURES EXCEEDED MAXIMUM NUMBER ALLOWED  
           CALL SETCOD ( 3 )
           GO TO 50
         ENDIF
           ENDIF
         ENDIF
        IF(IFAIL.EQ.1)GO TO  5
        IF(IOPT.EQ.0)GO TO  5
         IF(ICOS.EQ.LSEA) THEN
          ICOS=0
          CALL MINDIR(30,ACCR,1,NOC,ALP)
           DO 4 I=1,NOD
    4      DIRCOS(I)=0.
          IF(IPRI.EQ.1.AND.WEAK)WRITE(IUOUT,6)
          IF(IPRI.EQ.2)WRITE(IUOUT,6)
          CALL DISPLA
          NOCEX=NOCEX+NOC
          IF (ITERM ( 0, 0, 1 ) .EQ. 1) NOCY = NOCY+1
          IF (ITERM ( 1, 1, 0) .NE.0) GO TO 50
         ENDIF
    5  CONTINUE
C
  50  CONTINUE
      CALL TRINFO
    6 FORMAT(30X,'\/ -- \/ -- \/  LINE-SEARCH')
      END
C
C
      SUBROUTINE BOXSAM
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   BOXSAM ... SAMPLES AT RANDOM A TRIAL POINT FROM INSIDE THE BOX
C   CONSTRUCTED BY RANDS. IT ALSO IMPLEMENTS A VOLUME
C   EXCLUSION IF SO INSTRUCTUTED BY RANDS.
C   RESIZES BOX APPROPRIATELY IF THERE IS A CONFLICT WITH THE MARGINS.
C
      PARAMETER ( MXV = 200 )
      COMMON/CWORK1/XT(MXV),XT1(MXV),XT2(MXV),XT3(MXV),XT4(MXV)
      COMMON/TOBOX/SL(MXV),SR(MXV),IFAIL,IRELAX
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
C
      ZERO = 0.
      ONE = 1.
C  CHECK  AND REVISE STEPS IF NECESSARY
       DO 1 I=1,NOD
        XCH=STEP(I)+POINT(I)
        IF(XCH.EQ.POINT(I))STEP(I)=(1.+ABS(XCH))*COMTOL
    1  CONTINUE
C   IF LAST TRY WAS SUCCESFUL REDEFINE BOX
       IF(IFAIL.EQ.0) THEN
         DO 2 I=1,NOD
          SL(I)=-ABS(STEP(I))
          SR(I)=ABS(STEP(I))
    2    CONTINUE
       ENDIF
C  CHECK IF BOX INTERFERES WITH MARGINS AND RESET BOX IF NEEDED
       DO 3 I=1,NOD
        IF(MARG(I).EQ.0) GO TO  3
        SL(I)=MAX((XLL(I)-POINT(I)),SL(I))
        SR(I)=MIN((XRL(I)-POINT(I)),SR(I))
    3  CONTINUE
C  IF LAST TRY WAS A FAILURE EXCLUDE FAILURE AREA RESIZING BOX
       IF(IFAIL.EQ.1.AND.IRELAX.EQ.0) THEN
         DO 4 I=1,NOD
          DD=XT(I)-POINT(I)
          IF(DD.LT.0.) SL(I)=MAX(SL(I),DD)
          IF(DD.GT.0.) SR(I)=MIN(SR(I),DD)
    4    CONTINUE
       ENDIF
C  BEGIN SAMPLING INSIDE THE BOX
       DO 5 I=1,NOD
        IF(IXAT(I).EQ.0) GO TO  5
        XT(I)=(SR(I)-SL(I))*RANDI(ZERO,ONE)+SL(I)+POINT(I)
    5  CONTINUE
      END
      FUNCTION RANM()
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION R(97)
      PARAMETER (M1=259200,IA1=7141,IC1=54773,RM1=3.8580247E-6)
      PARAMETER (M2=134456,IA2=8121,IC2=28411,RM2=7.4373773E-6)
      PARAMETER (M3=243000,IA3=4561,IC3=51349)
      SAVE
      DATA IFF /0/
      DATA IDUM/-1/
C
      IF (IDUM.LT.0.OR.IFF.EQ.0) THEN
        IFF=1
        IX1=MOD(IC1-IDUM,M1)
        IX1=MOD(IA1*IX1+IC1,M1)
        IX2=MOD(IX1,M2)
        IX1=MOD(IA1*IX1+IC1,M1)
        IX3=MOD(IX1,M3)
        DO 11 J=1,97
          IX1=MOD(IA1*IX1+IC1,M1)
          IX2=MOD(IA2*IX2+IC2,M2)
          R(J)=(IX1+IX2*RM2)*RM1
11      CONTINUE
        IDUM=1
      ENDIF
      IX1=MOD(IA1*IX1+IC1,M1)
      IX2=MOD(IA2*IX2+IC2,M2)
      IX3=MOD(IA3*IX3+IC3,M3)
      J=1+(97*IX3)/M3
      IF (J.GT.97) J = 97
      IF (J.LT.1) J = 1
      RANM=R(J)
      R(J)=(IX1+IX2*RM2)*RM1
      END
C
C
      FUNCTION RANDI(A,B)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   RANDI...RETURNS A RANDOM NUMBER BETWEEN A AND B
      CALL RANDOM(R)
      RANDI=A+(B-A)*R
      RETURN
      END
C
C
      SUBROUTINE RANDOM(R)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C  THIS SUBROUTINE RETURNS A RANDOM NUMBER  R IN (0.,1.)
      R = RANM()
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE READCO ( CMD, MACERR, FORCE )
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  COMMAND READER. UPON RETURN COM IS THE COMMAND TO BE EXECUTED AND
C  MACERR MAY TAKE ON THE VALUES:
C        0   IF EVERYTHING OK.
C        1   IF SOME ERROR OCCURED.
C        2   IF CONTROL NEEDS TO BE TRANSFERRED TO THE PROMPTING POINT.
C
      CHARACTER*(*) CMD, FORCE
C
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      INTEGER IS(40), IE(40)
      CHARACTER*80 INPLIN
      COMMON /INDAT1/ IS, IE, NIT, LEC, ISTAT
      COMMON /INDAT2/ INPLIN
C
      PARAMETER ( MAXNL = 20 )
      INTEGER UBACK(MAXNL)
      CHARACTER*10 CBACK(MAXNL)
      COMMON / NEST1 / UBACK, LEVEL, IEXIT, IDISK
      COMMON / NEST2 / CBACK
C
      CHARACTER*10 MAC, COM
      LOGICAL EXI, OPE
C
 1    CONTINUE
C
C  READ AND CRACK THE NEXT COMMAND LINE. CHECK FOR EOF.
      IF (FORCE.EQ.' ') THEN
         CALL INLINE(INPLIN,.TRUE.,.TRUE.,IS,IE,NIT,IERR)
         IF (IERR.EQ.2) GOTO 2
      ELSE
         INPLIN = FORCE
         CALL UPCASE(INPLIN)
         CALL LCR(INPLIN,IS,IE,NIT)
      END IF
C
C  DO WE HAVE A BLANK LINE ?
      IF (NIT.EQ.0) THEN
         IF (STRO) WRITE(IUOUT,99)
         MACERR = 2
         RETURN
      END IF
C
      CMD = INPLIN(IS(1):IE(1))
      LEC = IE(1)-IS(1)+1
C
C  DID THE USER SPECIFY A MACRO NAME ?
      IF (CMD(1:1).NE.'.') THEN
         MACERR = 0
         RETURN
      END IF
      COM = CMD
C
C  IS IT JUST THE MACRO DOT ?
      IF (LEC.EQ.1) THEN
         CALL MID2('NO NAME FOLLOWS THE INITIAL MACRO DOT')
         MACERR = 2
         RETURN
      END IF
C
C  ALLOCATE A SCRATCH UNIT. THE MACRO WITH PARAMETERS SUBSTITUTED
C  IS COPIED IN THIS UNIT.
      ISC = NUNIT()
      IF (ISC.EQ.-1) THEN
         CALL SYSERR('SCRATCH UNIT CANNOT BE ALLOCATED')
         MACERR = 1
         RETURN
      END IF
C
C  CHECK FOR RECURSIVE MACRO CALLS.
      DO 10,I=1,LEVEL
        IF (COM.EQ.CBACK(I)) THEN
           CALL MID2('MACRO RECURSION NOT ALLOWED')
           MACERR = 1
           RETURN
        END IF
 10   CONTINUE
C
C  TRY TO LOCATE THE MACRO IN THE MACRO FILE.
      CALL LOCMAC(COM,ISUC)
C
C  THE MACRO WAS NOT FOUND IN THE MACRO FILE.
      IF (ISUC.EQ.0) THEN
C
C  ARE WE ALLOWED TO LOOK ON DISK FILES FOR THE MACRO ?
         IF (IDISK.EQ.0) THEN
            CALL MID2('MACRO '//COM(1:LEC)//' DOES NOT EXIST')
            MACERR = 1
            RETURN
         END IF
         MAC = COM(2:)
         INQUIRE (FILE=MAC,EXIST=EXI,OPENED=OPE)
C
C  DOES THE FILE EXIST ?
         IF (.NOT.EXI) THEN
            CALL MID2('MACRO '//COM(1:LEC)//' DOES NOT EXIST')
            MACERR = 1
            RETURN
         END IF
C  IS THE FILE ALREADY OPEN ?
         IF (OPE) THEN
            CALL MID2('MACRO '//COM(1:LEC)//' IS ON A BUSY FILE')
            MACERR = 1
            RETURN
         END IF
C
C  ALLOCATE UNIT FOR THE MACRO FILE ITSELF.
         IM = NUNIT()
         IF (IM.EQ.-1) THEN
            CALL SYSERR('SCRATCH UNIT CANNOT BE ALLOCATED')
            MACERR = 1
            RETURN
         END IF
C
C  COPY THE MACRO TO THE SCRATCH UNIT SUBSTITUTING ANY PARAMETERS.
         OPEN (UNIT=IM,FILE=MAC)
         OPEN(ISC)
         REWIND IM
         CALL MCOPY(IM,ISC,INPLIN,IS,IE,NIT)
         CLOSE(IM)
C
C  UPDATE THE NESTING LEVEL. IS IT TOO BIG FOR US ?
         LEVEL = LEVEL+1
         IF (LEVEL.GT.MAXNL) THEN
            CALL SYSERR('MAXIMUM NESTING LEVEL EXCEEDED')
            CLOSE(ISC,STATUS='DELETE')
            MACERR = 1
            RETURN
         END IF
C
C  COMMANDS NOW ARE TO BE INPUT FROM THE SCRATCH UNIT JUST ALLOCATED.
         CALL MID1('EXECUTION OF MACRO '//COM(1:LEC)//
     >                       ' HAS BEEN INITIATED')
         UBACK(LEVEL) = IUINP
         CBACK(LEVEL) = COM
         IUINP = ISC
         MACERR = 2
C
C  THE SPECIFIED MACRO HAS BEEN FOUND IN THE MACRO FILE.
      ELSE IF (ISUC.EQ.1) THEN
C
         OPEN(ISC)
         CALL MCOPY(IUMAC,ISC,INPLIN,IS,IE,NIT)
C
C  UPDATE THE NESTING LEVEL. IS IT TOO BIG FOR US ?
         LEVEL = LEVEL+1
         IF (LEVEL.GT.MAXNL) THEN
            CALL SYSERR('MAXIMUM NESTING LEVEL EXCEEDED')
            CLOSE(ISC,STATUS='DELETE')
            MACERR = 1
            RETURN
         END IF
C
C  COMMANDS NOW ARE TO BE INPUT FROM THE SCRATCH UNIT JUST ALLOCATED.
         CALL MID1('EXECUTION OF MACRO '//COM(1:LEC)//
     >                       ' HAS BEEN INITIATED')
         UBACK(LEVEL) = IUINP
         CBACK(LEVEL) = COM
         IUINP = ISC
         MACERR = 2
C
C  THE MACRO FILE IS CORRUPT. THE USER PROBABLY DIDN'T READ THE
C  APPROPRIATE DOCUMENTATION.
      ELSE
         CALL MID2('THE MERLIN MACRO FILE IS CORRUPT')
         MACERR = 1
      END IF
      RETURN
C
C  THIS SITUATION SHOULD NEVER HAPPEN. (EOF IN A MACRO FILE)
 2    CONTINUE
      IF (LEVEL.GT.0) THEN
         CALL BACKLE
      ELSE
         IF (STRO) WRITE(IUOUT,97)
         REWIND IUINP
         CALL BATCH
      END IF
      MACERR = 2
      RETURN
C
 3    FORMAT(A)
 97   FORMAT(3X,' CARRIAGE RETURN IGNORED')
 99   FORMAT(3X,' BLANK LINE IGNORED.')
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE BACKLE
C
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      PARAMETER ( MAXNL = 20 )
      INTEGER UBACK(MAXNL)
      CHARACTER*10 CBACK(MAXNL)
      COMMON / NEST1 / UBACK, LEVEL, IEXIT, IDISK
      COMMON / NEST2 / CBACK
C
      CLOSE (IUINP,STATUS='DELETE')
      IUINP = UBACK(LEVEL)
      LEVEL = LEVEL-1
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE TOPLEV
C
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      PARAMETER ( MAXNL = 20 )
      INTEGER UBACK(MAXNL)
      CHARACTER*10 CBACK(MAXNL)
      COMMON / NEST1 / UBACK, LEVEL, IEXIT, IDISK
      COMMON / NEST2 / CBACK
C
      CLOSE (IUINP,STATUS='DELETE')
      DO 10,I=2,LEVEL
 10     CLOSE (UBACK(I),STATUS='DELETE')
C
      IUINP = UBACK(1)
      LEVEL = 0
C
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE EXITM
C
      PARAMETER ( MAXNL = 20 )
      INTEGER UBACK(MAXNL)
      CHARACTER*10 CBACK(MAXNL)
      COMMON / NEST1 / UBACK, LEVEL, IEXIT, IDISK
      COMMON / NEST2 / CBACK
C
      IF (IEXIT.EQ.1 .AND. LEVEL.GT.0) CALL TOPLEV
C
      END
      SUBROUTINE READER(IUNIT,JALL)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     THE ... READER IS USED TO READ A DESIRED POINT FROM A UNIT
C     SPECIFIED BY THE IUNIT PARAMETER. THE JALL PARAMETER CAN TAKE ON
C     THE VALUES 0 OR 1.  IF JALL=1 THE READER READS BOTH POINT
C     PARAMETERS,AND THE FUNCTION'S VALUE. FOR JALL=0 THE VALUE OF THE
C     FUNCTION IS NOT READ BUT IS CALCULATED.PARAMETERS READ ARE
C     BECOMING THE CURRENT PARAMETERS.( IXAT,NICKNA,POINT,XLL,XRL )
C
      PARAMETER ( MXV = 200 )
      COMMON/FIRST/ICATA,IDAT
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER*10 NICKNA
      COMMON/NAMES/NICKNA(MXV)
      COMMON /MODE3/ MODEV, IFFORM 
      COMMON/INDEFI/INDEF(MXV)
      CHARACTER MARL*11,MARR*11,CFIX*3
      REWIND IUNIT
       DO 1 I=1,NOD
        READ(IUNIT,3) NICKNA(I),CFIX,POINT(I),MARL,MARR
         CALL DUPNAM(I,NOD)
         IF(CFIX.EQ.' * ')THEN
          IXAT(I)=0
         ELSE IF(CFIX.EQ.'...')THEN
          IXAT(I)=1
         ELSE
          WRITE(IUOUT,5)IUNIT,I,CFIX
          RETURN
         ENDIF
         IF(MARL.EQ.' ')THEN
           IF(MARR.EQ.' ')THEN
            MARG(I)=0
           ELSE
            MARG(I)=1
            READ(MARR,2) XRL(I)
           ENDIF
         ELSE
           IF(MARR.EQ.' ') THEN
            MARG(I)=-1
            READ(MARL,2) XLL(I)
           ELSE
            MARG(I)=2
            READ(MARL,2) XLL(I)
            READ(MARR,2) XRL(I)
           ENDIF
         ENDIF
         INDEF(I) = 1
    1  CONTINUE
      IF (MODEV.EQ.1) THEN
         IF (JALL.EQ.0) THEN
            VAL=ACSQ(POINT,NOD)
         ELSE
            READ(IUNIT,4)VAL
         ENDIF
      END IF
    2 FORMAT(G11.0)
    3 FORMAT(6X,A10,2X,A3,2X,T25,G21.0,T48,2(2X,A11))
    4 FORMAT(T25,G21.0)
    5 FORMAT(10X,'*** CAUTION: FORMAT ERROR IN UNIT:',I3,/T40,
     >'RECORD : ',I3,2X,A3)
      END
      SUBROUTINE HYPER(NOCAL,ACCU,ENH,IFM,IPRI,ICOUNT,IDIG,EPSH)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   HYPER ... MINIMIZES THE FUNCTION  ACSQ.
C   INPUT
C   NOCAL   APPROXIMATE MAXIMUM NUMBER OF ACSQ EVALUATIONS ALLOWED.
C   ACCU    LINE SEARCH TOLERANCE.
C   ENH     ENHANCEMENT STEP-FACTOR.
C   IFM     MAXIMUM NUMBER OF CONSECUTIVE FAILURE-CYCLES ALLOWED.
C   IPRI  = 0,1,2  ARRANGES PRINTOUT LEVEL.
C   IDIG    INPUT ARGUMENT TO FUNCTION WALL. (SEE WALL-DOCUMENTATION)
C   EPSH    MINIMUM RELATIVE DROP ACCEPTED.
C   OUTPUT
C   ICOUNT  NUMBER OF ACSQ EVALUATIONS.
C
      PARAMETER ( MXV = 200 )
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/PRILOG/STRO,WEAK
      COMMON/DIRCOM/DIRCOS(MXV)
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      LOGICAL STRO,WEAK
      COMMON/CWORK1/XPO(MXV),XT1(MXV),XT2(MXV),XT3(MXV),XT4(MXV)
      DIMENSION IPROG(MXV)
C
C  INITIALIZE CALL AND FAILURE COUNTERS
      ICOUNT=0
      IFAIL=0
C  INITIALIZE PROGRESS STATUS
      DO 10 I = 1,NOD
 10   IPROG(I) = 0
C  INITIALIZE TERMINATION CRITERIA SETTINGS.
      CALL INITER(IPRI,NOCAL,EPSH)
C  CHECK IF ANY OF THE TERMINATION CRITERIA ARE MET.
      IF (ITERM(1,1,0).NE.0) GO TO 50
C
C  BEGINING OF THE CYCLE-LOOP
C
      DO 8 III=1,100000
C
C  LOOP TO CHECK IF STEPS ARE MEANINGFULL.  IF NOT ADJUST THEM.
        DO 1 K=1,NOD
          XXX=POINT(K)+STEP(K)
          IF ( XXX.EQ.POINT(K) ) STEP(K)=(1.+ABS(XXX))*COMTOL
  1     CONTINUE
C
        IMIN=0
C
C  BEGINING OF LOOP OVER ALL VARIABLES
C
         DO 5 I=1,NOD
C
C  SET THE TRIAL POINT (XPO) EQUAL TO CURRENT POINT
C
           DO 2 J=1,NOD
             XPO(J)=POINT(J)
    2      CONTINUE
C
C  SKIP TO THE NEXT VARIABLE IF THIS ONE IS FIXED
          IF(IXAT(I).EQ.0) GO TO  5
C  MODIFY THE I-TH COMPONENT OF THE TRIAL POINT (XPO)
          XPO(I)=POINT(I)+STEP(I)
C
C  CHECK IF THIS COMPONENT COMPLIES WITH THE MARGINS.
C  IF IT IS INSIDE AN INFEASIBLE REGION, RESET IT APPROPRIATELY.
C
          IF(MARG(I).EQ.-1)      THEN
            IF(XPO(I).LT.XLL(I)) XPO(I)=(XLL(I)+POINT(I))/2.
          ELSEIF (MARG(I).EQ.1)  THEN
            IF(XPO(I).GT.XRL(I)) XPO(I)=(XRL(I)+POINT(I))/2.
          ELSEIF (MARG(I).EQ.2)  THEN
            IF(XPO(I).LT.XLL(I)) XPO(I)=(XLL(I)+POINT(I))/2.
            IF(XPO(I).GT.XRL(I)) XPO(I)=(XRL(I)+POINT(I))/2.
          ENDIF
C
          STEP(I)=XPO(I) - POINT(I)
          H1 = STEP(I)
C  CHECK  IF POINT IS ANY DIFFERENT THAN XPO.
C  IF SO, LOOP TO THE NEXT VARIABLE.
          IF ( POINT(I).EQ.XPO(I) ) GO TO  5
C
C  EVALUATE THE FUNCTION AT THE FIRST TRIAL POINT.
          TRIAL1=ACSQ(XPO,NOD)
          ICOUNT=ICOUNT+1
C  CALCULATE AN ESTIMATE FOR THE DERIVATIVE.
          FP1 = (TRIAL1-VAL)/H1
          IF(TRIAL1.LT.VAL)  THEN
C  SUCCESS AT THE FIRST TRY.
         IPROG(I) = 1
            IMIN=I
C  UPDATE VALUE AND POINT
C
             VAL=TRIAL1
C         
             DO 3 K=1,NOD
               POINT(K)=XPO(K)
    3        CONTINUE
             CALL DISPLA
C 
C  MODIFY STEP AND CHECK TARGET  AND CALLS  CRITERIA.
C
          STEP(I) = ENH*STEP(I)
          IF(ITERM(1,1,0).NE.0) GO TO 50
           ELSE
C          
C  FAILURE  AT THE FIRST TRY.
C
C  MODIFY THE I-TH COMPONENT OF THE CURRENT POINT,
C  IN THE OPPOSITE DIRECTION.
C
             XPO(I)=POINT(I)-STEP(I)
C         
C  CHECK IF THIS COMPONENT COMPLIES WITH THE MARGINS.
C  IF IT IS INSIDE AN INFEASIBLE REGION, RESET IT APPROPRIATELY.
C
             IF(MARG(I).EQ.-1)      THEN
               IF(XPO(I).LT.XLL(I)) XPO(I)=(XLL(I)+POINT(I))/2.
             ELSEIF (MARG(I).EQ.1)  THEN
               IF(XPO(I).GT.XRL(I)) XPO(I)=(XRL(I)+POINT(I))/2.
             ELSEIF (MARG(I).EQ.2)  THEN
               IF(XPO(I).LT.XLL(I)) XPO(I)=(XLL(I)+POINT(I))/2.
               IF(XPO(I).GT.XRL(I)) XPO(I)=(XRL(I)+POINT(I))/2.
             ENDIF
C
             STEP(I) = POINT(I) - XPO(I)
             H2 = STEP(I)
C
C  CHECK  IF POINT IS ANY DIFFERENT THAN XPO.
C  IF SO, LOOP TO THE NEXT VARIABLE.
C
             IF ( POINT(I).EQ.XPO(I) ) GO TO  5
C         
C  EVALUATE THE FUNCTION AT THE SECOND TRIAL POINT.
             TRIAL2=ACSQ(XPO,NOD)
             ICOUNT=ICOUNT+1
C  CALCULATE AN ESTIMATE FOR THE DERIVATIVE.
             FP2 = -(TRIAL2-VAL)/H2
             IF(TRIAL2.LT.VAL)THEN
C  SUCCESS AT THE SECOND TRY. 
            IPROG(I) = 1
               IMIN=I
C  UPDATE VALUE AND POINT
               VAL=TRIAL2
               DO 4 K=1,NOD
                 POINT(K)=XPO(K)
    4          CONTINUE
               CALL DISPLA
C          
               STEP(I)=-STEP(I)*ENH
C  CHECK IF THE SECOND DERIVATIVE IS POSITIVE.
C  IN THAT CASE FORM THE NEWTON STEP XXX = -F'/F"
            IF( (FP1-FP2)*(H1+H2) .GT. 0.) THEN
              XXX = -.5*(H2*FP1+H1*FP2)/(FP1-FP2)
C  PICK AS STEP THE LONGEST ONE.         
              IF(ABS(XXX).GT.ABS(STEP(I))) STEP(I) = XXX
            ENDIF
            IF (ITERM(1,1,0).NE.0) GO TO 50
C           
             ELSE
C         
C  FAILURE AT THE SECOND TRY.
            IPROG(I) = 0
C
C  THE FOLLOWING EXPRESION APPROXIMATES  THE RATIO OF THE
C  FIRST TO SECOND PARTIAL DERIVATIVES
               IF((FP1-FP2).NE.0.) THEN
                 STEP(I)=-.5*(H2*FP1+H1*FP2)/(FP1-FP2)
               ELSE
              SIGN = 1
              IF( (H2*FP1+H1*FP2)*(H1+H2).LT.0)SIGN =-1
                 STEP(I) = -SIGN*ABS(STEP(I))/2.
               ENDIF
             ENDIF
           ENDIF
    5    CONTINUE
C
C  END OF THE VARIABLE LOOP
C
         IF (ITERM(1,1,0).NE.0) GO TO 50
C        
           DO 7 I = 1,NOD
             XPO(I) = POINT(I)
             DSIGN= - STEP(I)
             IF(IMIN.EQ.0) THEN
C  NO PROGRESS  HAS BEEN ACHIEVED.     
               DIRCOS(I) = STEP(I)*WALL(I,IDIG,DSIGN)
          ELSE
C  PROGRESS HAS BEEN ACHIEVED IN THE VARIABLE LOOP.   
            DIRCOS(I) = STEP(I)*WALL(I,IDIG,DSIGN)*IPROG(I)
          ENDIF
 7         CONTINUE
C  PERFORM A LINE SEARCH ALONG THE DIRECTION STORED IN DIRCOS
           OLDV=VAL
           CALL MINDIR(30,ACCU,0,NOCMIN,ALPHA)
           ICOUNT=ICOUNT+NOCMIN
C
           IF(VAL.LT.OLDV) THEN
             STEP(I) = POINT(I)-XPO(I)
             IF(IPRI.EQ.1.AND.WEAK) WRITE(IUOUT,9)
             IF(IPRI.EQ.2.AND.STRO) WRITE(IUOUT,9)
             CALL DISPLA
           ENDIF
C          
C
        IF (ITERM(1,1,0).NE.0) GO TO 50
C  CHECK CONVERGENCE CRITERIA FOR CYCLE FAILURES.    
           IF (ITERM(0,0,1).EQ.1) THEN
             IFAIL=IFAIL+1
           ELSE
             IFAIL=0
           ENDIF
C
           IF(IFAIL.GT.IFM) THEN
             CALL SETCOD(3)
          GOTO 50
           END IF
C          
C  END OF THE CYCLE-LOOP 
C
    8  CONTINUE
C
C
 50   CONTINUE
      CALL TRINFO
C
    9 FORMAT(30X,'\/ -- \/ -- \/  LINE-SEARCH')
      END
      SUBROUTINE SETMOD ( IUINP, IUOUT )
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER*10 BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      COMMON/MODEIN/BACKUP,DERIVA,PRINTO,CALLBY,PROCES,PANEL
      CHARACTER*10 PROMOD
C
 1    CONTINUE
      CALL DASH
      WRITE(IUOUT,*)'... ENTER PROCESSING MODE OPTION. ( BATCH / IAF )'
      CALL DASH
      CALL INNOCR(PROMOD,.TRUE.,IERR)
      IF (IERR.NE.0) THEN
         CALL BATCH
         REWIND IUINP
         GOTO 1
      ENDIF
      IF (PROMOD.NE.'IAF  ' .AND. PROMOD.NE.'BATCH') THEN
         CALL BATCH
         GOTO 1
      ELSE
         PROCES = PROMOD
      ENDIF
      END
      SUBROUTINE SETPOI(FILENA)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  SETPOI ... ASSIGNES INITIAL VALUES TO THE ARRAYS:
C  XLL (LEFT-MARGINS), POINT , AND XRL (RIGHT MARGINS).
C  THESE VALUES MUST RESIDE IN SOME UNIT,
C  THE NUMBER OF WHICH IS PROMPT BY THE ROUTINE.
C
      PARAMETER ( MXV = 200 )
      COMMON/UNTSDF/IU31,IU32,IU36,IU37,IU42,IU45,IU47
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
      COMMON/PRILOG/STRO,WEAK
      LOGICAL STRO,WEAK,LOGOP
      CHARACTER*10 FNAME,FILENA(6)
      COMMON /MODE3/ MODEV, IFFORM 
      COMMON/INDEFI/INDEF(MXV)
C
C   DISPLAY MENU OPTIONS
C
      CALL MENU(JDIR,IUOUT,IUINP)
C
C   CANCEL REQUEST UPON ERRORS
      IF(JDIR .EQ. 100) RETURN
      IF(JDIR.NE.0 .AND. ITNOCS .EQ. 0 ) THEN
      WRITE(IUOUT,*)' INITIALIZE FUNMIN VARIABLES FIRST'
      CALL BATCH
      RETURN
      ENDIF
C
        IF(STRO)WRITE(IUOUT,*)' ...   ENTER FILE NAME   ...'
C
C  SPECIFICATION OF THE FILE NAME
C
        READ(IUINP,'(A10)',END=3) FNAME
C
         DO 2 I=1,6
           IF (FNAME .EQ. FILENA(I) .OR. FNAME .EQ. 'HELP') THEN
            IF(STRO)WRITE(IUOUT,10)FNAME
            CALL BATCH
            RETURN
           ENDIF
    2    CONTINUE
C
C  CHECK IF FILE  FNAME  IS ALREADY OPENED.
C
        INQUIRE (FILE= FNAME,OPENED=LOGOP,NUMBER=NU,IOSTAT=ISTA)
         IF (ISTA .NE. 0 ) THEN
C
C  CHECK IF FILE NAME IS INCORRECT AND IF SO , ISSUE APPROPRIATE MESSAGE
C
          IF(STRO)WRITE(IUOUT,'(T5,''INCORRECT FILE-NAME'',3X,A10)')FNAM
     >E
          CALL BATCH
          RETURN
         ENDIF
C
C IF FILE IS NOT OPEN, OPEN IT.
C
        IF (.NOT.LOGOP) THEN
           NU = NUNIT()
           IF (NU.EQ.-1) THEN
              IF (STRO) WRITE (IUOUT,*) 'ALL UNITS OCCUPIED.'
           CALL BATCH
           RETURN
           END IF
           OPEN (NU,FILE=FNAME)
           REWIND NU
        END IF
C
C   READ  YOUR DATA FROM THE SPECIFIED FILE.
C
        IF (JDIR.EQ.0) THEN
           READ(NU,*,END=4)(POINT(I),I=1,NOD)
           DO 88,I=1,NOD
88            INDEF(I) = 1
        END IF
        IF(JDIR.EQ.-1)READ(NU,*,END=4)(XLL(I),I=1,NOD)
        IF(JDIR.EQ.1)READ(NU,*,END=4)(XRL(I),I=1,NOD)
        IF(JDIR.EQ.2)READ(NU,*,END=4)(IXAT(I),I=1,NOD)
        IF(JDIR.EQ.3)READ(NU,*,END=4)(STEP(I),I=1,NOD)
      IF (JDIR**2.EQ.1) THEN
      DO 37 K=1,NOD
      IF(XLL(K) .EQ. -BLIM .AND. XRL(K) .EQ. BLIM) MARG(K)= 0
      IF(XLL(K) .EQ. -BLIM .AND. XRL(K) .NE. BLIM) MARG(K)= 1
      IF(XLL(K) .NE. -BLIM .AND. XRL(K) .EQ. BLIM) MARG(K)=-1
      IF(XLL(K) .NE. -BLIM .AND. XRL(K) .NE. BLIM) MARG(K)= 2
 37   CONTINUE
      ENDIF
C  READJUST POINT IF CONFLICTS WITH MARGINS
      CALL POIMAR(IFEV)
      IF (IFEV.EQ.0 .AND. MODEV.EQ.1) VAL = ACSQ(POINT,NOD)
      IF (.NOT.LOGOP) CLOSE(NU)
      RETURN
    3 REWIND IUINP
      CALL BATCH
      RETURN
    4 REWIND NU
      WRITE(IUOUT,8) FNAME
      WRITE(IUOUT,9)
      IF (.NOT.LOGOP) CLOSE(NU)
      CALL BATCH
      RETURN
    5 FORMAT(5X,'UNIT ',I3,' IS RESERVED FOR INTERNAL USE ONLY')
    6 FORMAT(5X,'USE A DIFFERENT UNIT FOR PROPER INITIALIZATION')
    7 FORMAT(5X,'INSUFFICIENT NUMBER OF DATA IN UNIT ',I3)
    8 FORMAT(5X,'INSUFFICIENT NUMBER OF DATA IN FILE ',A10)
    9 FORMAT(5X,'REFORMAT INITIALIZATION UNIT CORRECTLY')
   10 FORMAT(5X,'FILE ',A10,' IS RESERVED FOR INTERNAL USE ONLY')
      END
      SUBROUTINE SIMPL(P,NOP,INIS,ERR,ERR1,MNOCC,MNOC1,IPRI,NOCALL)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   SIMPL... IMPLEMENTATION OF THE SIMPLEX METHOD.
C   P: ARRAY OF THE SIZE NOP*(NOP+1) CONTAINING THE
C      SIMPLEX VERTICES
C   NOP: THE NUMBER OF VARIABLES OF THE OBJECTIVE FUNCTION
C   INIS: (1,2) CORRESPONDING TO PICKING THE INITIAL
C        SYSTEMATICALLY OR AT RANDOM
C   ERR: TERMINATION CRITERION OF THE SIMPLEX SEARCH
C   ERR1: TERMINATION CRITERION FOR THE LINE SEARCH
C   MNOCC: MAXIMUM ALLOWED CALLS TO THE OBJECTIVE FUNCTION
C   MNOC1: MAXIMUM CALLS ALLOWED FOR EACH LINE SEARCH
C   IPRI : (0,1,2) ARRANGES THE PRINTOUT LEVEL
C   NOCALL: ACTUAL CALLS OF THE OBJECTIVE FUNCTION
C
      PARAMETER ( MXV = 200 )
      LOGICAL STRO,WEAK
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      COMMON/FV1COM/XVAR(MXV),N,IVAR
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      COMMON/CWORK1/XPO(MXV),XT1(MXV),PCEN(MXV),PST1(MXV),PST2(MXV)
      COMMON/CWORK2/XSTEP(MXV),FMIN(0:MXV)
      EXTERNAL FVAR1
      DIMENSION P(0:NOP,NOP)
      ZERO = 0.
C  INITIALIZE TERMINATION CRITERIA SETTINGS.
      CALL INITER ( IPRI, MNOCC, ZERO )
      NOCALL=0
      IF(MNOC1.LE.0.OR.MNOCC.LE.0)RETURN
      N=NOD
      A=1.
      B=.5
      C=2.
      MNOC=MNOCC
C INITIAL SIMPLEX VERTICES CALCULATION
C ACCORDING TO INIS VALUE(1,2)
C
       DO 1 I=0,NOP
         DO 1 J=1,NOP
          P(I,J)=POINT(J)
    1  CONTINUE
      FMIN(0)=ACSQ(POINT,NOP)
      NOCALL=1
       DO 2 K=1,NOP
        XVAR(K)=POINT(K)
    2  CONTINUE
      GOTO (3,5)INIS
    3 CONTINUE
C SIMPLEX VERTICES CALCULATED BY FINDING THE MINIMA IN ALL DIRECTIONS
C
      FA=-1.
      FB=+1.
      ITOMIN=0
       DO 4 J=1,NOP
         IF(IXAT(J).EQ.0)THEN
C      COUNT FIXED VARIABLES
          ITOMIN=ITOMIN+1
          FMIN(J)=FMIN(0)
         ELSE
          XX0=XVAR(J)
          XXX0=XX0
          IVAR=J
          SST=STEP(J)
           IF(MARG(J).EQ.-1.OR.MARG(J).EQ.2)THEN
            FA=FVAR1(XLL(J))
            NOCALL=NOCALL+1
           ENDIF
           IF(MARG(J).EQ.1.OR.MARG(J).EQ.2)THEN
            FB=FVAR1(XRL(J))
            NOCALL=NOCALL+1
           ENDIF
          CALL MINI(FVAR1,XX0,SST,MNOC1,ERR1,MARG(J),XLL(J),XRL(J),FA,FB
     >,
     >XMIN,FMN,NOC,MINS)
          NOCALL=NOCALL+NOC
           IF(MINS.GE.0)THEN
            P(J,J)=XMIN
            FMIN(J)=FMN
            XVAR(J)=XXX0
            IF(MINS.EQ.0)ITOMIN=ITOMIN+1
           ELSEIF(MINS.LT.0)THEN
            XXM=XX0-SST/2.
            XXP=XX0+SST/2.
            XVAR(J)=XXP
            FFP=FSIMP(XVAR,NOD,NOCALL)
            XVAR(J)=XXM
            FFM=FSIMP(XVAR,NOD,NOCALL)
            XVAR(J)=XXX0
             IF(FFP.LE.FMIN(0).AND.FFM.LE.FMIN(0))THEN
               IF(FFP.LE.FFM)THEN
                P(J,J)=XXP
                FMIN(J)=FFP
               ELSE
                P(J,J)=XXM
                FMIN(J)=FFM
               ENDIF
             ELSEIF(FFP.LE.FMIN(0))THEN
              P(J,J)=XXP
              FMIN(J)=FFP
             ELSEIF(FFM.LE.FMIN(0))THEN
              P(J,J)=XXM
              FMIN(J)=FFM
             ELSE
              P(J,J)=XX0
              FMIN(J)=FMIN(0)
              ICOUNT=ICOUNT+1
             ENDIF
           ENDIF
         ENDIF
    4  CONTINUE
      IF(ITOMIN.LT.NOP)GO TO  7
      IF(STRO)WRITE(IUOUT,*)
     >'ALREADY AT MINIMUM. IMPOSSIBLE TO FORM INITIAL SIMPLEX'
      RETURN
    5 CONTINUE
      CALL VARSTA(ISTA)
       IF(ISTA.EQ.0)THEN
        IF(STRO)WRITE(IUOUT,*)
     >'EVERY VARIABLE MUST BE EITHER FRAMED OR FIXED'
        RETURN
       ENDIF
       DO 6 I=1,NOP
         IF(IXAT(I).EQ.1)THEN
          XX0=XVAR(I)
          XVAR(I)=RANDI(XLL(I),XRL(I))
          P(I,I)=XVAR(I)
          FMIN(I)=ACSQ(XVAR,NOP)
          NOCALL=NOCALL+1
          XVAR(I)=XX0
         ELSE
          FMIN(I)=FMIN(0)
         ENDIF
    6  CONTINUE
C
C
C
    7 CONTINUE
C
C   INITIAL SIMPLEX VERTICES CALCULATION FINISHED
C
      MNOC=MNOC+NOCALL
      IF(WEAK.AND.(IPRI.EQ.1.OR.IPRI.EQ.2))WRITE(IUOUT,26)NOCALL
      N=NOP
      ITER=0
    8 CONTINUE
      ITER=ITER+1
      FLOW=FMIN(0)
      FHIGH=FLOW
      IHIGH=0
      ILOW=0
       DO 9 KIM=1,N
        FM=FMIN(KIM)
         IF(FM.GT.FHIGH)THEN
          IHIGH=KIM
          FHIGH=FM
         ELSE IF(FM.LT.FLOW)THEN
          FLOW=FM
          ILOW=KIM
         ENDIF
    9  CONTINUE
      ATE=0.
       DO 10 MM=0,N
        ATE=ATE+FMIN(MM)
   10  CONTINUE
      SAM=0.
      FMESO=ATE/REAL(N+1)
       DO 11 KI=0,N
        SAM=SAM+ABS(FMIN(KI) -FMESO)
   11  CONTINUE
      ERROR=SAM/REAL(N+1)
       IF(FMIN(ILOW).LT.VAL)THEN
         DO 12 IU=1,NOP
          POINT(IU)=P(ILOW,IU)
   12    CONTINUE
        IF(WEAK.AND.IPRI.EQ.2)WRITE(IUOUT,27)ERROR,NOCALL
        VAL=FMIN(ILOW)
        CALL DISPLA
       ENDIF
       IF((ERROR.LE.ERR).OR.(NOCALL.GE.MNOC))THEN
        IF(WEAK.AND.IPRI.NE.0)WRITE(IUOUT,28)
        RETURN
       ENDIF
       DO 14 KK=1,N
         IF(IXAT(KK).EQ.1)THEN
          SUM=0.
           DO 13 L1=0,N
            IF(L1.EQ.IHIGH)GO TO  13
            SUM=SUM+P(L1,KK)
   13      CONTINUE
          PCEN(KK)=SUM/(REAL(N))
         ELSE
          PCEN(KK)=P(0,KK)
         ENDIF
   14  CONTINUE
       DO 15 K=1,N
         IF(IXAT(K).EQ.1)THEN
          PST1(K)=(1.+A)*PCEN(K)-A*P(IHIGH,K)
         ELSE
          PST1(K)=PCEN(K)
         ENDIF
   15  CONTINUE
      FST1=FSIMP(PST1,N,NOCALL)
       IF(FST1.LE.FLOW)THEN
         DO 16 LL=1,N
           IF(IXAT(LL).EQ.1)THEN
            PST2(LL)=(1.+C)*PST1(LL)-C*PCEN(LL)
           ELSE
            PST2(LL)=PST1(LL)
           ENDIF
   16    CONTINUE
        FST2=FSIMP(PST2,N,NOCALL)
C      F(P*) <FLOW .....P** CALCULATED
        IF(FST2-FST1)22,24,24
       ELSE
C      F(P*)>F(ILOW)
         DO 17 II=0,N
          FMMM=FMIN(II)
          IF((II.EQ.IHIGH).OR.(FST1.GE.FMMM))GO TO  17
          GO TO  24
   17    CONTINUE
         IF(FST1.LT.FHIGH)THEN
C     F(P*)<FHIGH
           DO 18 LM=1,N
   18      P(IHIGH,LM)=PST1(LM)
          FMIN(IHIGH)=FST1
C      PHIGH REPLACED BY P*
          FHIGH=FST1
         ENDIF
         DO 19 LM=1,N
           IF(IXAT(LM).EQ.1)THEN
            PST2(LM)=B*P(IHIGH,LM)+(1.-B)*PCEN(LM)
           ELSE
            PST2(LM)=PCEN(LM)
           ENDIF
   19    CONTINUE
        FST2=FSIMP(PST2,N,NOCALL)
C     P** CALCULATED
         IF(FST2.GE.FHIGH)THEN
C     F(P**)>=FHIGH REPLACE ALL P(I) BY (P(I)+PLOW)/2.
           DO 21 IP=0,N
            IF(IP.EQ.ILOW)GOTO  21
             DO 20 IL=1,N
               IF(IXAT(IL).EQ.1)THEN
                P(IP,IL)=(P(IP,IL)+P(ILOW,IL))/2.
               ENDIF
              XVAR(IL)=P(IP,IL)
   20        CONTINUE
            FMIN(IP)=FSIMP(XVAR,N,NOCALL)
   21      CONTINUE
          GO TO  8
         ENDIF
       ENDIF
   22 CONTINUE
C     F(P**)<FLOW
       DO 23 IX=1,N
   23  P(IHIGH,IX)=PST2(IX)
      FMIN(IHIGH)=FST2
C      PHIGH REPLACED BY P**
      GO TO  8
   24 CONTINUE
C     FST2>=FLOW
       DO 25 LL=1,N
   25  P(IHIGH,LL)=PST1(LL)
      FMIN(IHIGH)=FST1
C      PHIGH REPLACED BY P*
      GO TO  8
   26 FORMAT(//'INITIAL SIMPLEX VERTICES CALCULATED .   CALLS = ',I8//)
   27 FORMAT(//10X,'ERROR = ',1PG21.14,10X,' CALLS =',I10/)
   28 FORMAT(/15X,' END OF SIMPLEX MINIMIZATION '//)
      END
C
C
      FUNCTION FVAR1(X)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C   FVAR1... ASSINGES THE VALUE X TO THE IVAR COMPONENT
C   OF THE LOCATION VECTOR XVAR AND ASSUMES THE VALUE:
C   ACSQ(XVAR,N)
C   IVAR AND XVAR ARE CONTAINED IN THE COMMON BLOCK FV1COM
C
      PARAMETER ( MXV = 200 )
      COMMON/FV1COM/XVAR(MXV),N,IVAR
      XVAR(IVAR)=X
      FVAR1=ACSQ(XVAR,N)
      RETURN
      END
C
C
      FUNCTION FSIMP(X,N,NOCALL)
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C   FSIMP... THIS IS THE FUNCTION THAT SIMPL IS MINIMIZING
C   IT'S VALUE IS THE SAME AS THE OBJECTIVE FUNCTION INSIDE
C   THE MARGINS AND IT IS SET TO A HIGH VALUE OUTSIDE THE MARGINS
C
      PARAMETER ( MXV = 200 )
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      DIMENSION X(N)
       DO 1 I=1,N
        IF(MARG(I).EQ.0.OR.IXAT(I).EQ.0)GOTO  1
        IF((MARG(I).EQ.1.OR.MARG(I).EQ.2).AND.(X(I).GT.XRL(I)))GO TO  2
        IF((MARG(I).EQ.-1.OR.MARG(I).EQ.2).AND.(X(I).LT.XLL(I)))GO TO  2
    1  CONTINUE
      FSIMP=ACSQ(X,N)
      NOCALL=NOCALL+1
      RETURN
    2 CONTINUE
      FSIMP=BIGNUM
      RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE SYSERR ( MESS )
C
      CHARACTER*(*) MESS
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      CALL DASH
      WRITE (IUOUT,10) MESS
      CALL DASH
C
 10   FORMAT (T2,'### ERROR: ',A)
      END
      FUNCTION IVCHE ( )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER ( MXV = 200 )
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      LOGICAL AIMON
      COMMON/AIM/TARGET,AIMON
C
      IVCHE = 0
      IF (AIMON .AND. VAL.LE.TARGET) IVCHE = 1
      END
C
C
      FUNCTION ICCHE ( NOC, MAXNOC )
      ICCHE = 0
      IF (NOC.GE.MAXNOC) ICCHE = 1
      END
C
C
      FUNCTION ICONV ( V, EPS, NCALLS )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER ( MXV = 200 )
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      ICONV = 0
      IF (ABS(VAL-V).LE.EPS*ABS(MIN(VAL,V))*NCALLS) ICONV = 1
      END
C
C
      FUNCTION ITERM ( J1, J2, J3 )
C
C  EACH OF THE J1,J2,J3 TAKE ON THE VALUES: 0 OR 1.
C  J1 = 1 PERFORMS THE TARGET VALUE CHECK,
C  J2 = 1 PERFORMS THE NUMBER OF CALLS CHECK  AND
C  J3 = 1 PERFORMS THE CONVERGENCE RATE CHECK.
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER ( MXV = 200 )
      COMMON/TERMIN/STAVAL,EPS,INIC,ISTCAL,IPSTAT,MAXOC,ITCODE,ILOW
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
C
      I1 = 0
      I2 = 0
      I3 = 0
      IF (J1.EQ.1) I1 = IVCHE()
      IF (J2.EQ.1) I2 = ICCHE(ITNOCS-ISTCAL,MAXOC)
      IF (J3.EQ.1) THEN
         I3 = ICONV(STAVAL,EPS,ITNOCS-INIC)
         INIC = ITNOCS
         STAVAL = VAL
      END IF
      ITERM = 100*I1+10*I2+I3
      ITCODE = ITERM
      END
C
C
      SUBROUTINE INITER ( IPRI, MAXNOC, EPSIL )
C
C  INITIALIZES THE PARAMETERS USED IN THE ROUTINE THAT CHECKS
C  THE TERMINATION CRITERIA.
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER ( MXV = 200 )
      COMMON/TERMIN/STAVAL,EPS,INIC,ISTCAL,IPSTAT,MAXOC,ITCODE,ILOW
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
C  CALLS SINCE LAST CONVERGENCE TEST.
      INIC = ITNOCS
C  TOTAL CALLS SINCE THE BEGINING OF THE MINIMIZATION ROUTINE.
      ISTCAL = ITNOCS
C  VALUE OF THE OBJECTIVE FUNCTION BEFORE THIS CONVERGENCE TEST.
      STAVAL = VAL
C  PRINTOUT OPTION OF THE MINIMIZATION ROUTINE.
      IPSTAT = IPRI
C  MAXIMUM NUMBER OF CALLS ALLOWED.
      MAXOC = MAXNOC
C  MINIMUM RELATIVE DROP ACCEPTED.
      EPS = EPSIL
C  ILOW = 0,1 IF THE MINIMIZATION ROUTINE DID NOT - DID FIND 
C  A BETTER POINT. (THIS CORRESPONDS TO: DISPLA WASNT - WAS CALLED.
      ILOW = 0
      END
C
C
      SUBROUTINE TRINFO
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/TERMIN/STAVAL,EPS,INIC,ISTCAL,IPSTAT,MAXOC,ITCODE,ILOW
      LOGICAL STRO,WEAK
      COMMON/PRILOG/STRO,WEAK
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
      I1 = ITCODE/100
      I2 = (ITCODE-100*I1)/10
      I3 = ITCODE-100*I1-10*I2
C
C     IF (WEAK .AND. IPSTAT.NE.0) THEN
      IF (STRO) THEN
         IF (.NOT.(WEAK .AND. ILOW.EQ.1 .AND. IPSTAT.NE.0)) CALL DASH
         WRITE (IUOUT,*)
C
         IF (I1.EQ.1) CALL MID3('TARGET VALUE HAS BEEN REACHED')
         IF (I2.EQ.1) CALL MID3('ALL CALLS EXAUSTED')
         IF (I3.EQ.1) CALL MID3('CONVERGENCE SLOWER THAN SPECIFIED')
C  ITCODE=2 ISSUED BY BFGS, DFP AND CONGRA.
C  ITCODE=3 ISSUED BY ROLL AND RANDOM.
         IF (ITCODE.EQ.2) THEN 
            CALL MID3('FURTHER PROGRESS IMPOSSIBLE')
         ELSE IF (ITCODE.EQ.3) THEN
            CALL MID3('TOO MANY CYCLE-FAILURES')
         END IF
C
         IF (ILOW.EQ.0) CALL MID3('A BETTER POINT COULD NOT BE FOUND')
         CALL DASH
      END IF
      END
C
C
      SUBROUTINE SETCOD ( ICODE )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/TERMIN/STAVAL,EPS,INIC,ISTCAL,IPSTAT,MAXOC,ITCODE,ILOW
      ITCODE = ICODE
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE TOINT ( STR, LE, N, IERR )
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  CONVERTS A STRING OF DIGITS TO ITS INTEGER REPRESENTATION.
C  ASSUMES NO BLANKS ARE PRESENT.
C
      CHARACTER*(*) STR
C
C
      CHARACTER C
      LOGICAL NUMBER
      NUMBER(C) = C.GE.'0' .AND. C.LE.'9'
C
      N = 0
      IOFF = ICHAR('0')
C
C  CHECK FOR A SIGN.
      C = STR(1:1)
      IF (C.EQ.'+') THEN
         ISI = 1
         IPOS = 2
      ELSE IF (C.EQ.'-') THEN
         ISI = -1
         IPOS = 2
      ELSE
         ISI = 1
         IPOS = 1
      END IF
C
C  CONVERT THE NUMBER.
      DO 10,I=IPOS,LE
        C = STR(I:I)
        IF (NUMBER(C)) THEN
           N = 10*N+ICHAR(C)-IOFF
        ELSE
           IERR = -1
           RETURN
        END IF
 10   CONTINUE
C
      N = ISI*N
      IERR = 0
      RETURN
      END
      SUBROUTINE TOLI ( W, NOCMAX, ACC, IPRINT, NOCEXC)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION W(*)
      PARAMETER ( MXV = 200 )
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
C
      DIMENSION A(MXV,1), B(1), XL(MXV), XU(MXV), IACT(2*MXV), PAR(MXV)
C
      DO 10,I=1,NOD
         IF (IXAT(I).EQ.0) THEN
            XL(I) = POINT(I)
         XU(I) = POINT(I)
         ELSE
            MA = MARG(I)
            IF (MA.EQ.-1) THEN
               XL(I) = XLL(I)
            XU(I) = BLIM
            ELSE IF (MA.EQ.1) THEN
               XU(I) = XRL(I)
            XL(I) = -BLIM
            ELSE IF (MA.EQ.2) THEN
               XU(I) = XRL(I)
            XL(I) = XLL(I)
         ELSE IF (MA.EQ.0) THEN
               XU(I) = BLIM
            XL(I) = -BLIM
            END IF
         END IF
10    CONTINUE
      INFO = NOCMAX
      M = 0
      MEQ = 0
      IA = MXV
      INICAL = ITNOCS
      EPSIL = 0.
      CALL INITER(IPRINT,NOCMAX,EPSIL)
      ITP = 0
      IF (IPRINT.NE.0) ITP = 1
      CALL GETMIN(NOD,M,MEQ,A,IA,B,XL,XU,POINT,ACC,IACT,NACT,PAR,ITP,
     >            INFO,W)
      NOCEXC = ITNOCS-INICAL
      VAL = ACSQ(POINT,NOD)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE ADDCON (N,M,A,IA,IACT,NACT,Z,U,RELACC,INDXBD,ZTC,
     1  CGRAD)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),IACT(*),Z(*),U(*),ZTC(*),CGRAD(*)
      NP=NACT+1
      ICON=IACT(INDXBD)
      IACT(INDXBD)=IACT(NP)
      IACT(NP)=ICON
C
C     Form ZTC when the new constraint is a bound.
C
      IF (ICON .GT. M) THEN
          INEWBD=ICON-M
          IF (INEWBD .LE. N) THEN
              TEMP=-1.0
          ELSE
              INEWBD=INEWBD-N
              TEMP=1.0
          END IF
          IZNBD=INEWBD*N-N
          DO 10 J=1,N
   10     ZTC(J)=TEMP*Z(IZNBD+J)
C
C     Else form ZTC for an ordinary constraint.
C
      ELSE
          DO 20 I=1,N
   20     CGRAD(I)=A(I,ICON)
          DO 30 J=1,N
          ZTC(J)=0.0
          IZ=J
          DO 30 I=1,N
          ZTC(J)=ZTC(J)+Z(IZ)*CGRAD(I)
   30     IZ=IZ+N
      END IF
C
C     Find any Givens rotations to apply to the last columns of Z.
C
      J=N
   40 JP=J
      J=J-1
      IF (J .GT. NACT) THEN
          IF (ZTC(JP) .EQ. 0.0) GOTO 40
          IF (ABS(ZTC(JP)) .LE. RELACC*ABS(ZTC(J))) THEN
              TEMP=ABS(ZTC(J))
          ELSE IF (ABS(ZTC(J)) .LE. RELACC*ABS(ZTC(JP))) THEN
              TEMP=ABS(ZTC(JP))
          ELSE
              TEMP=ABS(ZTC(JP))*SQRT(1.0+(ZTC(J)/ZTC(JP))**2)
          END IF
          WCOS=ZTC(J)/TEMP
          WSIN=ZTC(JP)/TEMP
          ZTC(J)=TEMP
C
C     Apply the rotation when the new constraint is a bound.
C
          IZ=J
          IF (ICON .GT. M) THEN
              DO 50 I=1,N
              TEMP=WCOS*Z(IZ+1)-WSIN*Z(IZ)
              Z(IZ)=WCOS*Z(IZ)+WSIN*Z(IZ+1)
              Z(IZ+1)=TEMP
   50         IZ=IZ+N
              Z(IZNBD+JP)=0.0
C
C     Else apply the rotation for an ordinary constraint.
C
          ELSE
              WPIV=0.0
              DO 60 I=1,N
              TEMPA=WCOS*Z(IZ+1)
              TEMPB=WSIN*Z(IZ)
              TEMP=ABS(CGRAD(I))*(ABS(TEMPA)+ABS(TEMPB))
              IF (TEMP .GT. WPIV) THEN
                  WPIV=TEMP
                  IPIV=I
              END IF
              Z(IZ)=WCOS*Z(IZ)+WSIN*Z(IZ+1)
              Z(IZ+1)=TEMPA-TEMPB
   60         IZ=IZ+N
C
C     Ensure orthogonality of Z(.,JP) to CGRAD.
C
              SUM=0.0
              IZ=JP
              DO 70 I=1,N
              SUM=SUM+Z(IZ)*CGRAD(I)
   70         IZ=IZ+N
              IF (SUM .NE. 0.0) THEN
                  IZ=IPIV*N-N+JP
                  Z(IZ)=Z(IZ)-SUM/CGRAD(IPIV)
              END IF
          END IF
          GO TO 40
      END IF
C
C     Test for linear independence in the proposed new active set.
C
      IF (ZTC(NP) .EQ. 0.0) GOTO 90
      IF (ICON .LE. M) THEN
          SUM=0.0
          SUMABS=0.0
          IZ=NP
          DO 80 I=1,N
          TEMP=Z(IZ)*CGRAD(I)
          SUM=SUM+TEMP
          SUMABS=SUMABS+ABS(TEMP)
   80     IZ=IZ+N
          IF (ABS(SUM) .LE. RELACC*SUMABS) GOTO 90
      END IF
C
C     Set the new diagonal element of U and return.
C
      U(NP)=1.0/ZTC(NP)
      NACT=NP
   90 RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE ADJTOL (N,M,A,IA,B,XL,XU,X,IACT,NACT,XBIG,RELACC,TOL,
     1  MEQL)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),B(*),XL(*),XU(*),X(*),IACT(*),XBIG(*)
C
C     Set VIOL to the greatest relative constraint residual of the first
C       NACT constraints.
C
      VIOL=0.0
      IF (NACT .GT. MEQL) THEN
          KL=MEQL+1
          DO 20 K=KL,NACT
          J=IACT(K)
          IF (J .LE. M) THEN
              RES=B(J)
              RESABS=ABS(B(J))
              DO 10 I=1,N
              RES=RES-A(I,J)*X(I)
   10         RESABS=RESABS+ABS(A(I,J)*XBIG(I))
          ELSE
              JM=J-M
              IF (JM .LE. N) THEN
                  RES=X(JM)-XL(JM)
                  RESABS=XBIG(JM)+ABS(XL(JM))
              ELSE
                  JM=JM-N
                  RES=XU(JM)-X(JM)
                  RESABS=XBIG(JM)+ABS(XU(JM))
              END IF
          END IF
          IF (RES .GT. 0.0) VIOL=MAX(VIOL,RES/RESABS)
   20     CONTINUE
      END IF
C
C     Adjust TOL.
C
      TOL=0.1*MIN(TOL,VIOL)
      IF (TOL .LE. RELACC+RELACC) THEN
          TOL=RELACC
          DO 30 I=1,N
   30     XBIG(I)=ABS(X(I))
      END IF
      RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE CONRES (N,M,A,IA,B,XL,XU,X,IACT,NACT,PAR,G,Z,U,XBIG,
     1  BRES,D,ZTG,RELACC,TOL,STEPCB,SUMRES,MEQL,MSAT,MTOT,INDXBD,
     2  GM,GMNEW,PARNEW,CGRAD)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),B(*),XL(*),XU(*),X(*),IACT(*),PAR(*),G(*),
     1  Z(*),U(*),XBIG(*),BRES(*),D(*),ZTG(*),GM(*),GMNEW(*),PARNEW(*),
     2  CGRAD(*)
      IDIFF=MTOT-MSAT
C
C     Calculate and partition the residuals of the inactive constraints,
C       and set the gradient vector when seeking feasibility.
C
      IF (IDIFF .GT. 0.0) THEN
          DO 10 I=1,N
   10     G(I)=0.0
          SUMRES=0.0
      END IF
      MSATK=MSAT
      MDEG=NACT
      MSAT=NACT
      KL=MEQL+1
      DO 50 K=KL,MTOT
      J=IACT(K)
C
C     Calculate the residual of the current constraint.
C
      IF (J .LE. M) THEN
          RES=B(J)
          RESABS=ABS(B(J))
          DO 20 I=1,N
          RES=RES-X(I)*A(I,J)
   20     RESABS=RESABS+ABS(XBIG(I)*A(I,J))
      ELSE
          JM=J-M
          IF (JM .LE. N) THEN
              RES=X(JM)-XL(JM)
              RESABS=ABS(XBIG(JM))+ABS(XL(JM))
          ELSE
              JM=JM-N
              RES=XU(JM)-X(JM)
              RESABS=ABS(XBIG(JM))+ABS(XU(JM))
          END IF
      END IF
      BRES(J)=RES
C
C     Set TEMP to the relative residual.
C
      TEMP=0.0
      IF (RESABS .NE. 0.0) TEMP=RES/RESABS
      IF (K .GT. MSATK .AND. TEMP .LT. 0.0) THEN
          IF (TEMP+RELACC .GE. 0.0) THEN
              IF (J .LE. M) THEN
                  SUM=ABS(B(J))
                  DO 30 I=1,N
   30             SUM=SUM+ABS(X(I)*A(I,J))
              ELSE
                  JM=J-M
                  IF (JM .LE. N) THEN
                      SUM=ABS(X(JM))+ABS(XL(JM))
                  ELSE
                      SUM=ABS(X(JM-N))+ABS(XU(JM-N))
                  END IF
              END IF
              IF (ABS(RES) .LE. SUM*RELACC) TEMP=0.0
          END IF
      END IF
C
C     Place the residual in the appropriate position.
C
      IF (K .LE. NACT) GOTO 50
      IF (K .LE. MSATK .OR. TEMP .GE. 0.0) THEN
          MSAT=MSAT+1
          IF (MSAT .LT. K) THEN
              IACT(K)=IACT(MSAT)
          END IF
          IF (TEMP .GT. TOL) THEN
              IACT(MSAT)=J
          ELSE
              MDEG=MDEG+1
              IACT(MSAT)=IACT(MDEG)
              IACT(MDEG)=J
          END IF
C
C     Update the gradient and SUMRES if the constraint is violated when
C       seeking feasibility.
C
      ELSE
          IF (J .LE. M) THEN
              DO 40 I=1,N
   40         G(I)=G(I)+A(I,J)
          ELSE
              J=J-M
              IF (J .LE. N) THEN
                  G(J)=G(J)-1.0
              ELSE
                  G(J-N)=G(J-N)+1.0
              END IF
          END IF
          SUMRES=SUMRES+ABS(RES)
      END IF
   50 CONTINUE
C
C     Seek the next search direction unless CONRES was called from GETFES
C       and feasibility has been achieved.
C
      STEPCB=0.0
      IF (IDIFF .GT. 0 .AND. MSAT .EQ. MTOT) GOTO 60
      CALL GETD (N,M,A,IA,IACT,NACT,PAR,G,Z,U,D,ZTG,RELACC,DDOTG,MEQL,
     1  MDEG,GM,GMNEW,PARNEW,CGRAD)
C
C     Calculate the (bound on the) step-length due to the constraints.
C
      IF (DDOTG .LT. 0.0) THEN
          CALL STEPBD (N,M,A,IA,IACT,BRES,D,STEPCB,DDOTG,MDEG,MSAT,
     1      MTOT,INDXBD)
      END IF
      IF (IDIFF .EQ. 0) SUMRES=DDOTG
   60 RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE DELCON (N,M,A,IA,IACT,NACT,Z,U,RELACC,IDROP)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),IACT(*),Z(*),U(*)
      NM=NACT-1
      IF (IDROP .EQ. NACT) GOTO 60
      ISAVE=IACT(IDROP)
C
C     Cycle through the constraint exchanges that are needed.
C
      DO 50 J=IDROP,NM
      JP=J+1
      ICON=IACT(JP)
      IACT(J)=ICON
C
C     Calculate the (J,JP) element of R.
C
      IF (ICON .LE. M) THEN
          RJJP=0.0
          IZ=J
          DO 10 I=1,N
          RJJP=RJJP+Z(IZ)*A(I,ICON)
   10     IZ=IZ+N
      ELSE
          IBD=ICON-M
          IF (IBD .LE. N) THEN
              IZBD=IBD*N-N
              RJJP=-Z(IZBD+J)
          ELSE
              IBD=IBD-N
              IZBD=IBD*N-N
              RJJP=Z(IZBD+J)
          END IF
      END IF
C
C     Calculate the parameters of the next rotation.
C
      UJP=U(JP)
      TEMP=RJJP*UJP
      DENOM=ABS(TEMP)
      IF (DENOM*RELACC .LT. 1.0) DENOM=SQRT(1.0+DENOM*DENOM)
      WCOS=TEMP/DENOM
      WSIN=1.0/DENOM
C
C     Rotate Z when a bound constraint is promoted.
C
      IZ=J
      IF (ICON .GT. M) THEN
          DO 20 I=1,N
          TEMP=WCOS*Z(IZ+1)-WSIN*Z(IZ)
          Z(IZ)=WCOS*Z(IZ)+WSIN*Z(IZ+1)
          Z(IZ+1)=TEMP
   20     IZ=IZ+N
          Z(IZBD+JP)=0.0
C
C     Rotate Z when an ordinary constraint is promoted.
C
      ELSE
          WPIV=0.0
          DO 30 I=1,N
          TEMPA=WCOS*Z(IZ+1)
          TEMPB=WSIN*Z(IZ)
          TEMP=ABS(A(I,ICON))*(ABS(TEMPA)+ABS(TEMPB))
          IF (TEMP .GT. WPIV) THEN
              WPIV=TEMP
              IPIV=I
          END IF
          Z(IZ)=WCOS*Z(IZ)+WSIN*Z(IZ+1)
          Z(IZ+1)=TEMPA-TEMPB
   30     IZ=IZ+N
C
C     Ensure orthogonality to promoted constraint.
C
          SUM=0.0
          IZ=JP
          DO 40 I=1,N
          SUM=SUM+Z(IZ)*A(I,ICON)
   40     IZ=IZ+N
          IF (SUM .NE. 0.0) THEN
              IZ=IPIV*N-N+JP
              Z(IZ)=Z(IZ)-SUM/A(IPIV,ICON)
          END IF
      END IF
C
C     Set the new diagonal elements of U.
C
      U(JP)=-DENOM*U(J)
      U(J)=UJP/DENOM
   50 CONTINUE
C
C     Return.
C
      IACT(NACT)=ISAVE
   60 NACT=NM
      RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE EQCONS (N,M,MEQ,A,IA,B,XU,IACT,MEQL,INFO,Z,U,RELACC,
     1  AM,CGRAD)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),B(*),XU(*),IACT(*),Z(*),U(*),AM(*),CGRAD(*)
C
C     Try to add the next equality constraint to the active set.
C
      DO 50 KEQ=1,MEQ
      IF (MEQL .LT. N) THEN
          NP=MEQL+1
          IACT(NP)=KEQ
          CALL ADDCON (N,M,A,IA,IACT,MEQL,Z,U,RELACC,NP,AM,CGRAD)
          IF (MEQL .EQ. NP) GOTO 50
      END IF
C
C     If linear dependence occurs then find the multipliers of the
C       dependence relation and apply them to the right hand sides.
C
      SUM=B(KEQ)
      SUMABS=ABS(B(KEQ))
      IF (MEQL .GT. 0) THEN
          DO 10 I=1,N
   10     AM(I)=A(I,KEQ)
          K=MEQL
   20     VMULT=0.0
          IZ=K
          DO 30 I=1,N
          VMULT=VMULT+Z(IZ)*AM(I)
   30     IZ=IZ+N
          VMULT=VMULT*U(K)
          J=IACT(K)
          IF (J .LE. M) THEN
              DO 40 I=1,N
   40         AM(I)=AM(I)-VMULT*A(I,J)
              RHS=B(J)
          ELSE
              JM=J-M-N
              AM(JM)=AM(JM)-VMULT
              RHS=XU(JM)
          END IF
          SUM=SUM-RHS*VMULT
          SUMABS=SUMABS+ABS(RHS*VMULT)
          K=K-1
          IF (K .GE. 1) GOTO 20
      END IF
C
C     Error return if the constraints are inconsistent.
C
      IF (ABS(SUM) .GT. RELACC*SUMABS) THEN
          INFO=5
          GOTO 60
      END IF
   50 CONTINUE
   60 RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE FGCALC (N,X,F,G)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(*),G(*)
      PARAMETER ( MXV = 200 )
      INTEGER IERR(MXV)
C
      F = ACSQ(X,N)
      DERB = 0.0
      CALL DERMER(N,X,F,G,NOC,DERB,IERR)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE GETD (N,M,A,IA,IACT,NACT,PAR,G,Z,U,D,ZTG,RELACC,
     1  DDOTG,MEQL,MDEG,GM,GMNEW,PARNEW,CGRAD)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),IACT(*),PAR(*),G(*),Z(*),U(*),D(*),ZTG(*),
     1  GM(*),GMNEW(*),PARNEW(*),CGRAD(*)
C
C     Initialize GM and cycle backwards through the active set.
C
cc dgp
      abcd = 1.
10    CONTINUE
      DO 20 II=1,N
 20   GM(II)=G(II)
      K=NACT
 30   CONTINUE
      IF (K .GT. 0) THEN
C
C     Set TEMP to the next multiplier, but reduce the active set if
C       TEMP has an unacceptable sign.
C
          TEMP=0.0
          IZ=K
          DO 40 I=1,N
          TEMP=TEMP+Z(IZ)*GM(I)
   40     IZ=IZ+N
          TEMP=TEMP*U(K)
          IF (K .GT. MEQL .AND. TEMP .GT. 0.0) THEN
              CALL DELCON (N,M,A,IA,IACT,NACT,Z,U,RELACC,K)
              GOTO 10
          END IF
C
C     Update GM using the multiplier that has just been calculated.
C
          J=IACT(K)
          IF (J .LE. M) THEN
              DO 50 I=1,N
   50         GM(I)=GM(I)-TEMP*A(I,J)
          ELSE
              JM=J-M
              IF (JM .LE. N) THEN
                  GM(JM)=GM(JM)+TEMP
              ELSE
                  GM(JM-N)=GM(JM-N)-TEMP
              END IF
          END IF
          PAR(K)=TEMP
          K=K-1
          GOTO 30
      END IF
C
C     Calculate the search direction and DDOTG.
C
      DDOTG=0.0
      IF (NACT .LT. N) THEN
          CALL SDEGEN (N,M,A,IA,IACT,NACT,PAR,Z,U,D,ZTG,GM,RELACC,
     1      DDOTGM,MEQL,MDEG,GMNEW,PARNEW,CGRAD)
          IF (DDOTGM .LT. 0.0) THEN
              DO 60 I=1,N
   60         DDOTG=DDOTG+D(I)*G(I)
          END IF
      END IF
      RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE GETFES (N,M,A,IA,B,XL,XU,X,IACT,NACT,PAR,INFO,G,Z,
     1  U,XBIG,RELACC,TOL,MEQL,MSAT,MTOT,BRES,D,ZTG,GM,GMNEW,PARNEW,
     2  CGRAD)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),B(*),XL(*),XU(*),X(*),IACT(*),PAR(*),G(*),Z(*),
     1  U(*),XBIG(*),BRES(*),D(*),ZTG(*),GM(*),GMNEW(*),PARNEW(*),
     2  CGRAD(*)
C
C     Make the correction to X for the active constraints.
C
      INFO=0
   10 CALL SATACT (N,M,A,IA,B,XL,XU,X,IACT,NACT,INFO,Z,U,XBIG,RELACC,
     1  TOL,MEQL)
      IF (INFO .GT. 0) MSAT=NACT
      IF (MSAT .EQ. MTOT) GOTO 60
C
C     Try to correct the infeasibility.
C
   20 MSATK=MSAT
      SUMRSK=0.0
   30 CALL CONRES (N,M,A,IA,B,XL,XU,X,IACT,NACT,PAR,G,Z,U,XBIG,BRES,
     1  D,ZTG,RELACC,TOL,STEPCB,SUMRES,MEQL,MSAT,MTOT,INDXBD,GM,GMNEW,
     2  PARNEW,CGRAD)
C
C     Include the new constraint in the active set.
C
      IF (STEPCB .GT. 0.0) THEN
          DO 40 I=1,N
          X(I)=X(I)+STEPCB*D(I)
   40     XBIG(I)=MAX(XBIG(I),ABS(X(I)))
          CALL ADDCON (N,M,A,IA,IACT,NACT,Z,U,RELACC,INDXBD,GMNEW,CGRAD)
      END IF
C
C     Test whether to continue the search for feasibility.
C
      IF (MSAT .LT. MTOT) THEN
          IF (STEPCB .EQ. 0.0) GOTO 50
          IF (MSATK .LT. MSAT) GOTO 20
          IF (SUMRSK .EQ. 0.0 .OR. SUMRES .LT. SUMRSK) THEN
              SUMRSK=SUMRES
              ITEST=0
          END IF
          ITEST=ITEST+1
          IF (ITEST .LE. 2) GOTO 30
C
C     Reduce TOL if it may be too large to allow feasibility.
C
   50     IF (TOL .GT. RELACC) THEN
              CALL ADJTOL (N,M,A,IA,B,XL,XU,X,IACT,NACT,XBIG,RELACC,
     1          TOL,MEQL)
              GOTO 10
          END IF
      END IF
   60 RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE GETMIN (N,M,MEQ,A,IA,B,XL,XU,X,ACC,IACT,NACT,PAR,
     1  IPRINT,INFO,W)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),B(*),XL(*),XU(*),X(*),IACT(*),PAR(*),W(*)
C
C  This is the entry point to a package of subroutines that calculate the
C     the least value of a differentiable function of several variables
C     subject to linear constraints on the values of the variables.
C  N is the number of variables and must be set by the user.
C  M is the number of linear constraints (excluding simple bounds) and
C     must be set by the user.
C  MEQ is the number of constraints that are equalities and must be set
C     by the user.
C  A(.,.) is a 2-dimensional array whose columns are the gradients of the
C     M constraint functions.  Its entries must be set by the user and
C     its dimensions must be at least N by M.
C  IA is the actual first dimension of the array A that is supplied by the
C     user, so its value may not be less than N.
C  B(.) is a vector of constraint right hand sides that must also be set
C     by the user.  Specifically the constraints on the variables X(I)
C     I=1(1)N are
C          A(1,K)*X(1)+...+A(N,K)*X(N) .EQ. B(K)  K=1,...,MEQ
C          A(1,K)*X(1)+...+A(N,K)*X(N) .LE. B(K)  K=MEQ+1,...,M  .
C     Note that the data that define the equality constraints come before
C     the data of the inequalities.
C  XL(.) and XU(.) are vectors whose components must be set to lower and
C     upper bounds on the variables.  Choose very large negative and
C     positive entries if a component should be unconstrained, or set
C     XL(I)=XU(I) to freeze the I-th variable.  Specifically these simple
C     bounds are
C          XL(I) .LE. X(I) and X(I) .LE. XU(I)  I=1,...,N  .
C  X(.) is the vector of variables of the optimization calculation.  Its
C     initial elements must be set by the user to an estimate of the
C     required solution.  The subroutines can usually cope with poor
C     estimates, and there is no need for X(.) to be feasible initially.
C     These variables are adjusted automatically and the values that give
C     the least feasible calculated value of the objective function are
C     available in X(.) on the return from GETMIN.
C  ACC is a tolerance on the first order conditions at the calculated
C     solution of the optimization problem.  These first order conditions
C     state that, if X(.) is a solution, then there is a set of active
C     constraints with indices IACT(K) K=1(1)NACT, say, such that X(.) is
C     on the boundaries of these constraints, and the gradient of the
C     objective function can be expressed in the form
C          GRAD(F)=PAR(1)*GRAD(C(IACT(1)))+...
C                        ...+PAR(NACT)*GRAD(C(IACT(NACT)))  .
C     Here PAR(K) K=1(1)NACT are Lagrange multipliers that are nonpositive
C     for inequality constraints, and GRAD(C(IACT(K))) is the gradient of
C     the IACT(K)-th constraint function, so it is A(.,IACT(K)) if IACT(K)
C     .LE. M, and it is minus or plus the J-th coordinate vector if the
C     constraint is the lower or upper bound on X(J) respectively.  The
C     normal return from the calculation occurs when X(.) is feasible and
C     the sum of squares of components of the vector RESKT(.) is at most
C     ACC**2, where RESKT(.) is the N-component vector of residuals of
C     the first order condition that is displayed above.  Sometimes the
C     package cannot satisfy this condition, because noise in the function
C     values can prevent a change to the variables, no line search being
C     allowed to increase the objective function.
C  IACT(.) is a working space array of integer variables that must be
C     provided by the user.  Its length must be at least (M+2*N).  Its
C     leading entries on the return from the subroutine are the indices
C     IACT(K) K=1(1)NACT that are mentioned in the previous paragraph:
C     in other words they are the indices of the final active constraints.
C     Here the indices M+1,...,M+N and M+N+1,...,M+2*N denote the lower
C     and upper bounds respectively.
C  NACT is set automatically to the integer variable of this ilk that has
C     been introduced already.
C  PAR is a one-dimensional array that will hold the Lagrange multipliers
C     PAR(K) K=1(1)NACT on the return from GETMIN, these parameters being
C     defined in the above paragraph on ACC.  The length of PAR should be
C     at least N.
C  IPRINT must be set by the user to specify the frequency of printing
C     during the execution of the optimization package.  There is no
C     printed output if IPRINT=0.  Otherwise, after ensuring feasibility,
C     information is given every IABS(IPRINT) iterations and whenever a
C     parameter called TOL is reduced.  The printing provides the values
C     of X(.), F(.) and G(.)=GRAD(F) if IPRINT is positive, while if
C     IPRINT is negative this information is augmented by the current
C     values of IACT(K) K=1(1)NACT, PAR(K) K=1(1)NACT and RESKT(I)
C     I=1(1)N.  The reason for returning to the calling program is also
C     displayed when IPRINT is nonzero.
C  INFO is an integer variable that should be set to zero initially,
C     unless the user wishes to impose an upper bound on the number of
C     evaluations of the objective function and its gradient, in which
C     case INFO should be set to the value of this bound.  On the exit
C     from GETMIN it will have one of the following integer values to
C     indicate the reason for leaving the optimization package:
C          INFO=1   X(.) is feasible and the condition that depends on
C     ACC is satisfied.
C          INFO=2   X(.) is feasible and rounding errors are preventing
C     further progress.
C          INFO=3   X(.) is feasible but the objective function fails to
C     decrease although a decrease is predicted by the current gradient
C     vector.  If this return occurs and KTRES(.) has large components
C     then the user's calculation of the gradient of the objective
C     function may be incorrect.  One should also question the coding of
C     the gradient when the final rate of convergence is slow.
C          INFO=4   In this case the calculation cannot begin because IA
C     is less than N or because the lower bound on a variable is greater
C     than the upper bound.
C          INFO=5   This value indicates that the equality constraints
C     are inconsistent.   These constraints include any components of
C     X(.) that are frozen by setting XL(I)=XU(I).
C          INFO=6   In this case there is an error return because the
C     equality constraints and the bounds on the variables are found to
C     be inconsistent.
C          INFO=7   This value indicates that there is no vector of
C     variables that satisfies all of the constraints.  Specifically,
C     when this return or an INFO=6 return occurs, the current active
C     constraints (whose indices are IACT(K) K=1(1)NACT) prevent any
C     change in X(.) that reduces the sum of constraint violations,
C     where only bounds are included in this sum if INFO=6.
C          INFO=8   In this case the limit on the number of calls of
C     subroutine FGCALC (see below) has been reached, and there would
C     have been further calculation otherwise.
C  W(.) is a working space array of real variables that must be provided
C     by the user.  Its length must be at least (M+11*N+N**2).  On exit
C     from the package one can find the final components of GRAD(F) and
C     RESKT(.) in W(1),...,W(N) and W(N+1),...,W(2*N) respectively.
C  Note 1.   The variables N, M, MEQ, IA, ACC and IPRINT and the elements
C     of the arrays A(,.,), B(.), XL(.) and XU(.) are not altered by the
C     optimization procedure.  Their values, the value of INFO and the
C     initial components of X(.) must be set on entry to GETMIN.
C  Note 2.   Of course the package needs the objective function and its
C     gradient.  Therefore the user must provide a subroutine called
C     FGCALC, its first two lines being
C          SUBROUTINE FGCALC (N,X,F,G)
C          DIMENSION X(*),G(*)   .
C     It is called automatically with N set as above and with X(.) set
C     to a feasible vector of variables.  It should calculate the value
C     of the objective function and its gradient for this X(.) and should
C     set them in F and G(I) I=1(1)N respectively, without disturbing N
C     or any of the components of X(.).
C  Note 3.   A paper on the method of calculation and a report on the
C     main features of the computer code are available from the author
C     M.J.D.Powell (D.A.M.T.P., University of Cambridge, Silver Street,
C     Cambridge CB3 9EW, England).
C
C     Partition the workspace array.
C
      IG=1
      IRESKT=IG+N
      IZ=IRESKT+N
      IU=IZ+N*N
      IXBIG=IU+N
      IBRES=IXBIG+N
      ID=IBRES+M+N+N
      IZTG=ID+N
      IGM=IZTG+N
      IXS=IGM+N
      IGS=IXS+N
C
C     Call the optimization package.
C
      CALL MINFLC (N,M,MEQ,A,IA,B,XL,XU,X,ACC,IACT,NACT,PAR,IPRINT,
     1  INFO,W(IG),W(IZ),W(IU),W(IXBIG),W(IRESKT),W(IBRES),W(ID),
     2  W(IZTG),W(IGM),W(IXS),W(IGS))
      RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE INITZU (N,M,XL,XU,X,IACT,MEQL,INFO,Z,U,XBIG,RELACC)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION XL(*),XU(*),X(*),IACT(*),Z(*),U(*),XBIG(*)
C
C     Set RELACC.
C
      ZTPAR=100.0
      RELACC=1.0
   10 RELACC=0.5*RELACC
      TEMPA=ZTPAR+0.5*RELACC
      TEMPB=ZTPAR+RELACC
      IF (ZTPAR .LT. TEMPA .AND. TEMPA .LT. TEMPB) GOTO 10
C
C     Seek bound inconsistencies and bound equality constraints.
C
      MEQL=0
      DO 20 I=1,N
      IF (XL(I) .GT. XU(I)) GOTO 50
      IF (XL(I) .EQ. XU(I)) MEQL=MEQL+1
   20 CONTINUE
C
C     Initialize U, Z and XBIG.
C
      JACT=0
      NN=N*N
      DO 30 I=1,NN
   30 Z(I)=0.0
      IZ=0
      DO 40 I=1,N
      IF (XL(I) .EQ. XU(I)) THEN
          X(I)=XU(I)
          JACT=JACT+1
          U(JACT)=1.0
          IACT(JACT)=I+M+N
          J=JACT
      ELSE
          J=I+MEQL-JACT
      END IF
      Z(IZ+J)=1.0
      IZ=IZ+N
   40 XBIG(I)=ABS(X(I))
      INFO=1
   50 RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE KTVEC (N,M,A,IA,IACT,NACT,PAR,G,RESKT,Z,U,BRES,RELAXF,
     1  MEQL,SSQKT,PARW,RESKTW)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),IACT(*),PAR(*),G(*),RESKT(*),Z(*),U(*),
     1  BRES(*),PARW(*),RESKTW(*)
C
C     Calculate the Lagrange parameters and the residual vector.
C
      DO 10 I=1,N
   10 RESKT(I)=G(I)
      IF (NACT .GT. 0) THEN
          ICASE=0
   20     DO 50 KK=1,NACT
          K=NACT+1-KK
          J=IACT(K)
          TEMP=0.0
          IZ=K
          DO 30 I=1,N
          TEMP=TEMP+Z(IZ)*RESKT(I)
   30     IZ=IZ+N
          TEMP=TEMP*U(K)
          IF (ICASE .EQ. 0) PAR(K)=0.0
          IF (K .LE. MEQL .OR. PAR(K)+TEMP .LT. 0.0) THEN
              PAR(K)=PAR(K)+TEMP
          ELSE
              TEMP=-PAR(K)
              PAR(K)=0.0
          END IF
          IF (TEMP .NE. 0.0) THEN
              IF (J .LE. M) THEN
                  DO 40 I=1,N
   40             RESKT(I)=RESKT(I)-TEMP*A(I,J)
              ELSE
                  JM=J-M
                  IF (JM .LE. N) THEN
                      RESKT(JM)=RESKT(JM)+TEMP
                  ELSE
                      RESKT(JM-N)=RESKT(JM-N)-TEMP
                  END IF
              END IF
          END IF
   50     CONTINUE
C
C     Calculate the sum of squares of the KT residual vector.
C
          SSQKT=0.0
          IF (NACT .EQ. N) GOTO 130
          DO 60 I=1,N
   60     SSQKT=SSQKT+RESKT(I)**2
C
C     Apply iterative refinement to the residual vector.
C
          IF (ICASE .EQ. 0) THEN
              ICASE=1
              DO 70 K=1,NACT
   70         PARW(K)=PAR(K)
              DO 80 I=1,N
   80         RESKTW(I)=RESKT(I)
              SSQKTW=SSQKT
              GOTO 20
          END IF
C
C     Undo the iterative refinement if it does not reduce SSQKT.
C
          IF (SSQKTW .LT. SSQKT) THEN
              DO 90 K=1,NACT
   90         PAR(K)=PARW(K)
              DO 100 I=1,N
  100         RESKT(I)=RESKTW(I)
              SSQKT=SSQKTW
          END IF
C
C     Calculate SSQKT when there are no active constraints.
C
      ELSE
          SSQKT=0.0
          DO 110 I=1,N
  110     SSQKT=SSQKT+G(I)**2
      END IF
C
C     Predict the reduction in F if one corrects any positive residuals
C       of active inequality constraints.
C
      RELAXF=0.0
      IF (MEQL .LT. NACT) THEN
          KL=MEQL+1
          DO 120 K=KL,NACT
          J=IACT(K)
          IF (BRES(J) .GT. 0.0) THEN
              RELAXF=RELAXF-PAR(K)*BRES(J)
          END IF
  120     CONTINUE
      END IF
  130 RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE LSRCH (N,X,G,D,XS,GS,RELACC,STEPCB,DDOTG,F,STEP,
     1  NFVALS,NFMAX,GOPT)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(*),G(*),D(*),XS(*),GS(*),GOPT(*)
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
C
C     Initialization.
C
      RELINT=0.9
      ICOUNT=0
      RATIO=-1.0
      DO 10 I=1,N
      XS(I)=X(I)
      GS(I)=G(I)
      GOPT(I)=G(I)
      IF (D(I) .NE. 0.0) THEN
          TEMP=ABS(X(I)/D(I))
          IF (RATIO .LT. 0.0 .OR. TEMP .LT. RATIO) RATIO=TEMP
      END IF
   10 CONTINUE
      AMX = 1.0
      STEP=MIN(AMX,STEPCB)
      STPMIN=MAX(RELACC*RATIO,1.0E-12*STEP)
      STEP=MAX(STPMIN,STEP)
      SBASE=0.0
      FBASE=F
      DDOTGB=DDOTG
      STPLOW=0.0
      FLOW=F
      DGLOW=DDOTG
      STPHGH=0.0
      STPOPT=0.0
      FOPT=F
      DGOPT=ABS(DDOTG)
C
C     Calculate another function and gradient value.
C
   20 DO 30 I=1,N
   30 X(I)=XS(I)+STEP*D(I)
      ISOFAR = ITNOCS
      CALL FGCALC (N,X,F,G)
      ICOUNT=ICOUNT+(ITNOCS-ISOFAR)
      DGMID=0.0
      DO 40 I=1,N
   40 DGMID=DGMID+D(I)*G(I)
      IF (F .LE. FOPT) THEN
          IF (F .LT. FOPT .OR. ABS(DGMID) .LT. DGOPT) THEN
              STPOPT=STEP
              FOPT=F
              DO 50 I=1,N
   50         GOPT(I)=G(I)
              DGOPT=ABS(DGMID)
          END IF
      END IF
      IF (NFVALS+ICOUNT .EQ. NFMAX) GOTO 70
C
C      Modify the bounds on the steplength or convergence.
C
      IF (F .GE. FBASE+0.1*(STEP-SBASE)*DDOTGB) THEN
          IF (STPHGH .GT. 0.0 .OR. F .GT. FBASE .OR. DGMID .GT.
     1      0.5*DDOTG) THEN
              STPHGH=STEP
              FHGH=F
              DGHGH=DGMID
              GOTO 60
          END IF
          SBASE=STEP
          FBASE=F
          DDOTGB=DGMID
      END IF
      IF (DGMID .GE. 0.7*DDOTGB) GOTO 70
      STPLOW=STEP
      FLOW=F
      DGLOW=DGMID
   60 IF (STPHGH .GT. 0.0 .AND. STPLOW .GE. RELINT*STPHGH) GOTO 70
C
C     Calculate the next step length or end the iterations.
C
      IF (STPHGH .EQ. 0.0) THEN
          IF (STEP .EQ. STEPCB) GOTO 70
          TEMP=10.0
          IF (DGMID .GT. 0.9*DDOTG) TEMP=DDOTG/(DDOTG-DGMID)
          STEP=MIN(TEMP*STEP,STEPCB)
          GOTO 20
      ELSE IF (ICOUNT .EQ. 1 .OR. STPLOW .GT. 0.0) THEN
          DGKNOT=2.0*(FHGH-FLOW)/(STPHGH-STPLOW)-0.5*(DGLOW+DGHGH)
          IF (DGKNOT .GE. 0.0) THEN
              AMX = 0.1
              RATIO=MAX(AMX,0.5*DGLOW/(DGLOW-DGKNOT))
          ELSE
              RATIO=(0.5*DGHGH-DGKNOT)/(DGHGH-DGKNOT)
          END IF
          STEP=STPLOW+RATIO*(STPHGH-STPLOW)
          GOTO 20
      ELSE
          STEP=0.1*STEP
          IF (STEP .GE. STPMIN) GOTO 20
      END IF
C
C     Return from subroutine.
C
   70 IF (STEP .NE. STPOPT) THEN
          STEP=STPOPT
          F=FOPT
          DO 80 I=1,N
          X(I)=XS(I)+STEP*D(I)
   80     G(I)=GOPT(I)
      END IF
      NFVALS=NFVALS+ICOUNT
      RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE MINFLC (N,M,MEQ,A,IA,B,XL,XU,X,ACC,IACT,NACT,PAR,
     1  IPRINT,INFO,G,Z,U,XBIG,RESKT,BRES,D,ZTG,GM,XS,GS)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),B(*),XL(*),XU(*),X(*),IACT(*),PAR(*),G(*),
     1  Z(*),U(*),XBIG(*),RESKT(*),BRES(*),D(*),ZTG(*),GM(*),XS(*),
     2  GS(*)
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
C
C     Initialize ZZNORM, ITERC, NFVALS and NFMAX.
C
      ZZNORM=-1.0
      ITERC=0
      NFVALS=0
      NFMAX=0
      IF (INFO .GT. 0) NFMAX=INFO
C
C     Check the bounds on N, M and MEQ.
C
      INFO=4
      IF (MAX0(1-N,MEQ*(MEQ-M)) .GT. 0) THEN
          IF (IPRINT .NE. 0) WRITE (IUOUT,1010)
 1010     FORMAT (/5X,'ERROR RETURN FROM GETMIN BECAUSE A CONDITION',
     1      ' ON N, M OR MEQ IS VIOLATED')
          GOTO 40
      END IF
C
C     Initialize RELACC, Z, U and TOL.
C
      CALL INITZU (N,M,XL,XU,X,IACT,MEQL,INFO,Z,U,XBIG,RELACC)
      AMX = 0.01
      TOL=MAX(AMX,10.0*RELACC)
      IF (INFO .EQ. 4) THEN
          IF (IPRINT .NE. 0) WRITE (IUOUT,1020)
 1020     FORMAT (/5X,'ERROR RETURN FROM GETMIN BECAUSE A LOWER',
     1      ' BOUND EXCEEDS AN UPPER BOUND')
          GOTO 40
      END IF
C
C     Add any equality constraints to the active set.
C
      IF (MEQ .GT. 0) THEN
          CALL EQCONS (N,M,MEQ,A,IA,B,XU,IACT,MEQL,INFO,Z,U,RELACC,XS,
     1      GS)
          IF (INFO .EQ. 5) THEN
              IF (IPRINT .NE. 0) WRITE (IUOUT,1030)
 1030         FORMAT (/5X,'ERROR RETURN FROM GETMIN BECAUSE THE',
     1          ' EQUALITY CONSTRAINTS ARE INCONSISTENT')
              GOTO 40
          END IF
      END IF
      NACT=MEQL
      MSAT=MEQL
C
C     Add the bounds to the list of constraints.
C
      MTOT=NACT
      DO 10 I=1,N
      IF (XL(I) .LT. XU(I)) THEN
          MTOT=MTOT+2
          IACT(MTOT-1)=M+I
          IACT(MTOT)=M+N+I
      END IF
   10 CONTINUE
C
C     Try to satisfy the bound constraints.
C
      CALL GETFES (N,M,A,IA,B,XL,XU,X,IACT,NACT,PAR,INFO,G,Z,U,XBIG,
     1  RELACC,TOL,MEQL,MSAT,MTOT,BRES,D,ZTG,GM,RESKT,XS,GS)
      IF (MSAT .LT. MTOT) THEN
          IF (IPRINT .NE. 0) WRITE (IUOUT,1040)
 1040     FORMAT (/5X,'ERROR RETURN FROM GETMIN BECAUSE THE',
     1      ' EQUALITIES AND BOUNDS ARE INCONSISTENT')
          INFO=6
          GOTO 40
      END IF
C
C     Add the ordinary inequalities to the list of constraints.
C
      IF (M .GT. MEQ) THEN
          MP=MEQ+1
          DO 20 K=MP,M
          MTOT=MTOT+1
   20     IACT(MTOT)=K
      END IF
C
C     Correct any constraint violations.
C
   30 CALL GETFES (N,M,A,IA,B,XL,XU,X,IACT,NACT,PAR,INFO,G,Z,U,XBIG,
     1  RELACC,TOL,MEQL,MSAT,MTOT,BRES,D,ZTG,GM,RESKT,XS,GS)
      IF (MSAT .LT. MTOT) THEN
          IF (IPRINT .NE. 0) WRITE (IUOUT,1050)
 1050     FORMAT (/5X,'ERROR RETURN FROM GETMIN BECAUSE THE',
     1      ' CONSTRAINTS ARE INCONSISTENT')
          INFO=7
          GOTO 40
      ELSE IF (MEQL .EQ. N) THEN
          IF (IPRINT .NE. 0) WRITE (IUOUT,1060)
 1060     FORMAT (/5X,'GETMIN FINDS THAT THE VARIABLES ARE',
     1      ' DETERMINED BY THE EQUALITY CONSTRAINTS')
          GOTO 40
      END IF
C
C     Minimize the objective function in the case when constraints are
C       treated as degenerate if their residuals are less than TOL.
C
      CALL MINFUN (N,M,A,IA,B,XL,XU,X,ACC,IACT,NACT,PAR,IPRINT,INFO,G,Z,
     1  U,XBIG,RELACC,ZZNORM,TOL,MEQL,MTOT,ITERC,NFVALS,NFMAX,RESKT,
     2  BRES,D,ZTG,GM,XS,GS,F)
C
C     Reduce TOL if necessary.
C
      IF (TOL .GT. RELACC .AND. NACT .GT. 0) THEN
          IF (NFVALS .LT. NFMAX) THEN
              CALL ADJTOL (N,M,A,IA,B,XL,XU,X,IACT,NACT,XBIG,RELACC,TOL,
     1          MEQL)
              GOTO 30
          ELSE
              INFO=8
          END IF
      END IF
      IF (IPRINT .NE. 0) THEN
          IF (INFO .EQ. 1) WRITE (IUOUT,1070)
 1070     FORMAT (/5X,'GETMIN HAS ACHIEVED THE REQUIRED ACCURACY')
          IF (INFO .EQ. 2) WRITE (IUOUT,1080)
 1080     FORMAT (/5X,'GETMIN CAN MAKE NO FURTHER PROGRESS BECAUSE',
     1      ' OF ROUNDING ERRORS')
          IF (INFO .EQ. 3) WRITE (IUOUT,1090)
 1090     FORMAT (/5X,'GETMIN CAN MAKE NO FURTHER PROGRESS BECAUSE',
     1      ' F WILL NOT DECREASE ANY MORE')
          IF (INFO .EQ. 8) WRITE (IUOUT,1100)
 1100     FORMAT (/5X,'GETMIN HAS REACHED THE GIVEN LIMIT ON THE',
     1      ' NUMBER OF CALLS OF FGCALC')
      END IF
   40 RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE MINFUN (N,M,A,IA,B,XL,XU,X,ACC,IACT,NACT,PAR,IPRINT,
     1  INFO,G,Z,U,XBIG,RELACC,ZZNORM,TOL,MEQL,MTOT,ITERC,NFVALS,
     2  NFMAX,RESKT,BRES,D,ZTG,GM,XS,GS,F)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),B(*),XL(*),XU(*),X(*),IACT(*),PAR(*),G(*),Z(*),
     1  U(*),XBIG(*),RESKT(*),BRES(*),D(*),ZTG(*),GM(*),XS(*),GS(*)
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
C
C     Initialize the minimization calculation.
C
      MSAT=MTOT
      ITERK=ITERC
      NFVALK=NFVALS
      IF (NFVALS .EQ. 0 .OR. INFO .EQ. 1) THEN
          ISOFAR = ITNOCS
          CALL FGCALC (N,X,F,G)
          NFVALS=NFVALS+(ITNOCS-ISOFAR)
      END IF
      FPREV=ABS(F+F+1.0)
      ITERP=-1
      IF (IPRINT .NE. 0) THEN
          WRITE (IUOUT,1000) TOL
 1000     FORMAT (/5X,'NEW VALUE OF TOL =',1PE13.5)
          ITERP=ITERC
      END IF
C
C     Calculate the next search direction.
C
   10 CALL CONRES (N,M,A,IA,B,XL,XU,X,IACT,NACT,PAR,G,Z,U,XBIG,BRES,D,
     1  ZTG,RELACC,TOL,STEPCB,DDOTG,MEQL,MSAT,MTOT,INDXBD,GM,RESKT,XS,
     2  GS)
C
C     Calculate the Kuhn Tucker residual vector.
C
      CALL KTVEC (N,M,A,IA,IACT,NACT,PAR,G,RESKT,Z,U,BRES,RELAXF,MEQL,
     1  SSQKT,XS,GS)
C
C     Test for convergence.
C
      IF (SSQKT .LE. ACC*ACC) THEN
          INFO=1
          GOTO 70
      END IF
      IF (DDOTG .GE. 0.0) THEN
          INFO=2
          GOTO 70
      END IF
C
C     Test for termination due to no decrease in F.
C
      IF (F .GE. FPREV) THEN
          IF (TOL .EQ. RELACC .OR. NACT .EQ. 0) THEN
              IF (DIFF .GT. 0.0) GOTO 20
          END IF
          INFO=3
          GOTO 70
      END IF
   20 DIFF=FPREV-F
      FPREV=F
C
C     Test that more calls of FGCALC are allowed.
C
      IF (NFVALS .GE. NFMAX) THEN
          INFO=8
          GOTO 70
      END IF
C
C     Test whether to reduce TOL and to provide printing.
C
      IF (TOL .GT. RELACC .AND. ITERC .GT. ITERK .AND.
     1  0.1*RELAXF .GE. MAX(DIFF,-0.5*DDOTG)) GOTO 70
      IF (ITERP .EQ. ITERC) THEN
          ITERP=ITERC+IABS(IPRINT)
          GOTO 80
      END IF
C
C     Calculate the step along the search direction.
C
   40 ITERC=ITERC+1
      CALL LSRCH (N,X,G,D,XS,GS,RELACC,STEPCB,DDOTG,F,STEP,NFVALS,
     1  NFMAX,BRES)
      IF (STEP .EQ. 0.0) THEN
          INFO=3
          SUM=0.0
          DO 50 I=1,N
   50     SUM=SUM+ABS(D(I)*GS(I))
          IF (DDOTG+RELACC*SUM .GE. 0.0) INFO=2
          GOTO 70
      END IF
C
C     Revise XBIG.
C
      DO 60 I=1,N
   60 XBIG(I)=MAX(XBIG(I),ABS(X(I)))
C
C     Revise the second derivative approximation.
C
      CALL ZBFGS (N,X,NACT,G,Z,ZTG,XS,GS,ZZNORM)
C
C     Add a constraint to the active set if it restricts the step.
C
      IF (STEP .EQ. STEPCB) THEN
          K=IACT(INDXBD)
          IF (K .GT. M) THEN
              K=K-M
              IF (K .LE. N) THEN
                  X(K)=XL(K)
              ELSE
                  X(K-N)=XU(K-N)
              END IF
          END IF
          CALL ADDCON (N,M,A,IA,IACT,NACT,Z,U,RELACC,INDXBD,XS,GS)
      END IF
      GOTO 10
C
C     Printing from the subroutine.
C
   70 IF (IPRINT .EQ. 0) GOTO 90
      ITERP=-1
cc   80 WRITE (*,1010) ITERC,NFVALS,F
80    call displ1(f,x,n)
 1010 FORMAT (/5X,'ITERS =',I4,5X,'F.VALS =',I4,5X,'F =',1PE15.7)
cc      IF (NFVALS .GT. NFVALK) THEN
cc          WRITE (*,1020) (X(I),I=1,N)
cc 1020     FORMAT ('  X =',(1P5E14.5))
cc          WRITE (*,1030) (G(I),I=1,N)
cc 1030     FORMAT ('  G =',(1P5E14.5))
cc      ELSE
cc          WRITE (*,1040)
cc 1040     FORMAT (5X,'NO CHANGE TO X AND G SINCE PREVIOUS OUTPUT')
cc      END IF
cc      IF (IPRINT .LT. 0) THEN
cc          IF (NACT .EQ. 0) THEN
cc              WRITE (*,1050)
cc 1050         FORMAT (5X,'NO ACTIVE CONSTRAINTS')
cc          ELSE
cc              WRITE (*,1060) (IACT(I),I=1,NACT)
cc 1060         FORMAT (' IA =',(14I5))
cc              WRITE (*,1070) (PAR(I),I=1,NACT)
cc 1070         FORMAT (' LP =',(1P5E14.5))
cc          END IF
cc          IF (NACT .EQ. N) THEN
cc              WRITE (*,1080)
cc 1080         FORMAT (5X,'KT RESIDUAL VECTOR IS ZERO')
cc          ELSE
cc              WRITE (*,1090) (RESKT(I),I=1,N)
cc 1090         FORMAT (' KT =',(1P5E14.5))
cc          END IF
cc      END IF
      IF (ITERP .GE. 0) GOTO 40
   90 RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE NEWCON (N,M,A,IA,IACT,NACT,Z,U,D,RELACC,MDEG,ZZDIAG,
     1  GMNEW,CGRAD)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),IACT(*),Z(*),U(*),D(*),ZZDIAG(*),GMNEW(*),
     1  CGRAD(*)
C
C     Initialization.
C
      NP=NACT+1
      KHIGH=MDEG
      IZ=0
      DO 20 I=1,N
      ZZDIAG(I)=0.0
      DO 10 J=NP,N
   10 ZZDIAG(I)=ZZDIAG(I)+Z(IZ+J)**2
   20 IZ=IZ+N
C
C     Calculate the scalar products of D with its constraints.
C
   30 CVMAX=0.0
      DO 50 K=NP,KHIGH
      J=IACT(K)
      IF (J .LE. M) THEN
          SUM=0.0
          SUMABS=0.0
          SUMD=0.0
          DO 40 I=1,N
          TEMP=D(I)*A(I,J)
          SUM=SUM+TEMP
          SUMABS=SUMABS+ABS(TEMP)
   40     SUMD=SUMD+ZZDIAG(I)*A(I,J)**2
      ELSE
          JM=J-M
          IF (JM .LE. N) THEN
              SUM=-D(JM)
          ELSE
              JM=JM-N
              SUM=D(JM)
          END IF
          SUMABS=ABS(SUM)
          SUMD=ZZDIAG(JM)
      END IF
C
C     Pick out the most violated constraint, or return if the
C       violation is negligible.
C
      IF (SUM .GT. RELACC*SUMABS) THEN
          CVIOL=SUM*SUM/SUMD
          IF (CVIOL .GT. CVMAX) THEN
              CVMAX=CVIOL
              IADD=K
              SAVSUM=SUM
              SAVABS=SUMABS
          END IF
      END IF
   50 CONTINUE
      IF (CVMAX .LE. 0.0) GOTO 140
      IF (NACT .EQ. 0) GOTO 120
C
C     Set GMNEW to the gradient of the most violated constraint.
C
      J=IACT(IADD)
      IF (J .LE. M) THEN
          JMV=0
          DO 60 I=1,N
   60     GMNEW(I)=A(I,J)
      ELSE
          JMV=J-M
          DO 70 I=1,N
   70     GMNEW(I)=0.0
          IF (JMV .LE. N) THEN
              GMNEW(JMV)=-1.0
          ELSE
              JMV=JMV-N
              GMNEW(JMV)=1.0
          END IF
      END IF
C
C     Modify GMNEW for the next active constraint.
C
      K=NACT
   80 TEMP=0.0
      IZ=K
      DO 90 I=1,N
      TEMP=TEMP+Z(IZ)*GMNEW(I)
   90 IZ=IZ+N
      TEMP=TEMP*U(K)
      J=IACT(K)
      IF (J .LE. M) THEN
          DO 100 I=1,N
  100     GMNEW(I)=GMNEW(I)-TEMP*A(I,J)
      ELSE
          JM=J-M
          IF (JM .LE. N) THEN
              GMNEW(JM)=GMNEW(JM)+TEMP
          ELSE
              GMNEW(JM-N)=GMNEW(JM-N)-TEMP
          END IF
      END IF
C
C     Revise the values of SAVSUM and SAVABS.
C
      SUM=0.0
      SUMABS=0.0
      DO 110 I=1,N
      TEMP=D(I)*GMNEW(I)
      SUM=SUM+TEMP
  110 SUMABS=SUMABS+ABS(TEMP)
      SAVSUM=MIN(SAVSUM,SUM)
      SAVABS=MAX(SAVABS,SUMABS)
      K=K-1
      IF (K .GE. 1) GOTO 80
C
C     Add the new constraint to the active set if the constraint
C       violation is still significant.
C
      IF (JMV .GT. 0) D(JMV)=0.0
      IF (SAVSUM .LE. RELACC*SAVABS) GOTO 130
  120 K=NACT
      CALL ADDCON (N,M,A,IA,IACT,NACT,Z,U,RELACC,IADD,GMNEW,CGRAD)
      IF (NACT .GT. K) GOTO 140
C
C     Seek another constraint violation.
C
      IADD=NP
  130 IF (NP .LT. KHIGH) THEN
          K=IACT(KHIGH)
          IACT(KHIGH)=IACT(IADD)
          IACT(IADD)=K
          KHIGH=KHIGH-1
          GOTO 30
      END IF
  140 RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE SATACT (N,M,A,IA,B,XL,XU,X,IACT,NACT,INFO,Z,U,XBIG,
     1  RELACC,TOL,MEQL)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),B(*),XL(*),XU(*),X(*),IACT(*),Z(*),U(*),
     1  XBIG(*)
      IF (NACT .EQ. 0) GOTO 50
      DO 30 K=1,NACT
C
C     Calculate the next constraint residual.
C
      J=IACT(K)
      IF (J .LE. M) THEN
          RES=B(J)
          RESABS=ABS(B(J))
          RESBIG=RESABS
          DO 10 I=1,N
          TEMPA=A(I,J)
          TEMP=TEMPA*X(I)
          RES=RES-TEMP
          RESABS=RESABS+ABS(TEMP)
   10     RESBIG=RESBIG+ABS(TEMPA)*XBIG(I)
      ELSE
          JX=J-M
          IF (JX .LE. N) THEN
              RES=X(JX)-XL(JX)
              RESABS=ABS(X(JX))+ABS(XL(JX))
              RESBIG=XBIG(JX)+ABS(XL(JX))
              SAVEX=XL(JX)
          ELSE
              JX=JX-N
              RES=XU(JX)-X(JX)
              RESABS=ABS(X(JX))+ABS(XU(JX))
              RESBIG=XBIG(JX)+ABS(XU(JX))
              SAVEX=XU(JX)
          END IF
      END IF
C
C     Shift X if necessary.
C
      IF (RES .NE. 0.0) THEN
          TEMP=RES/RESABS
          IF (K .LE. MEQL) TEMP=-ABS(TEMP)
          IF (TOL .EQ. RELACC .OR. TEMP+RELACC .LT. 0.0) THEN
              INFO=1
              SCALE=RES*U(K)
              IZ=K
              DO 20 I=1,N
              X(I)=X(I)+SCALE*Z(IZ)
              IZ=IZ+N
   20         XBIG(I)=MAX(XBIG(I),ABS(X(I)))
              IF (J .GT. M) X(JX)=SAVEX
C
C     Else flag a constraint deletion if necessary.
C
          ELSE IF (RES/RESBIG .GT. TOL) THEN
              IACT(K)=-IACT(K)
          END IF
      END IF
   30 CONTINUE
C
C     Delete any flagged constraints and then return.
C
      IDROP=NACT
  40  IF (IACT(IDROP) .LT. 0) THEN
          IACT(IDROP)=-IACT(IDROP)
          CALL DELCON (N,M,A,IA,IACT,NACT,Z,U,RELACC,IDROP)
      END IF
      IDROP=IDROP-1
      IF (IDROP .GT. MEQL) GOTO 40
   50 RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE SDEGEN (N,M,A,IA,IACT,NACT,PAR,Z,U,D,ZTG,GM,RELACC,
     1  DDOTGM,MEQL,MDEG,GMNEW,PARNEW,CGRAD)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),IACT(*),PAR(*),Z(*),U(*),D(*),ZTG(*),GM(*),
     1  GMNEW(*),PARNEW(*),CGRAD(*)
      MP=MEQL+1
      DTEST=0.0
C
C     Calculate the search direction and branch if it is not downhill.
C
   10 CALL SDIRN (N,NACT,Z,D,ZTG,GM,RELACC,DDOTGM)
      IF (DDOTGM .EQ. 0.0) GOTO 120
C
C     Branch if there is no need to consider any degenerate constraints.
C     The test gives termination if two consecutive additions to the
C       active set fail to increase the predicted new value of F.
C
      IF (NACT .EQ. MDEG) GOTO 120
      NP=NACT+1
      SUM=0.0
      DO 20 J=NP,N
   20 SUM=SUM+ZTG(J)**2
      IF (DTEST .GT. 0.0 .AND. SUM .GE. DTEST) THEN
          IF (ITEST .EQ. 1) GOTO 120
          ITEST=1
      ELSE
          DTEST=SUM
          ITEST=0
      END IF
C
C     Add a constraint to the active set if there are any significant
C       violations of degenerate constraints.
C
      K=NACT
      CALL NEWCON (N,M,A,IA,IACT,NACT,Z,U,D,RELACC,MDEG,GMNEW,PARNEW,
     1  CGRAD)
      IF (NACT .EQ. K) GOTO 120
      PAR(NACT)=0.0
C
C     Calculate the new reduced gradient and Lagrange parameters.
C
   30 DO 40 I=1,N
   40 GMNEW(I)=GM(I)
      K=NACT
   50 TEMP=0.0
      IZ=K
      DO 60 I=1,N
      TEMP=TEMP+Z(IZ)*GMNEW(I)
   60 IZ=IZ+N
      TEMP=TEMP*U(K)
      PARNEW(K)=PAR(K)+TEMP
      AMX = 0.
      IF (K .EQ. NACT) PARNEW(K)=MIN(PARNEW(K),AMX)
      J=IACT(K)
      IF (J .LE. M) THEN
          DO 70 I=1,N
   70     GMNEW(I)=GMNEW(I)-TEMP*A(I,J)
      ELSE
          JM=J-M
          IF (JM .LE. N) THEN
              GMNEW(JM)=GMNEW(JM)+TEMP
          ELSE
              GMNEW(JM-N)=GMNEW(JM-N)-TEMP
          END IF
      END IF
      K=K-1
      IF (K .GT. MEQL) GOTO 50
C
C     Set RATIO for linear interpolation between PAR and PARNEW.
C
      RATIO=0.0
      IF (MP .LT. NACT) THEN
          KU=NACT-1
          DO 80 K=MP,KU
          IF (PARNEW(K) .GT. 0.0) THEN
              RATIO=PARNEW(K)/(PARNEW(K)-PAR(K))
              IDROP=K
          END IF
   80     CONTINUE
      END IF
C
C     Apply the linear interpolation.
C
      THETA=1.0-RATIO
      AMX = 0.
      DO 90 K=MP,NACT
   90 PAR(K)=MIN(THETA*PARNEW(K)+RATIO*PAR(K),AMX)
      DO 100 I=1,N
  100 GM(I)=THETA*GMNEW(I)+RATIO*GM(I)
C
C     Drop a constraint if RATIO is positive.
C
      IF (RATIO .GT. 0.0) THEN
          CALL DELCON (N,M,A,IA,IACT,NACT,Z,U,RELACC,IDROP)
          DO 110 K=IDROP,NACT
  110     PAR(K)=PAR(K+1)
          GOTO 30
      END IF
C
C     Return if there is no freedom for a new search direction.
C
      IF (NACT .LT. N) GOTO 10
      DDOTGM=0.0
  120 RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE SDIRN (N,NACT,Z,D,ZTG,GM,RELACC,DDOTGM)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION Z(*),D(*),ZTG(*),GM(*)
      DDOTGM=0.0
      IF (NACT .GE. N) GOTO 60
C
C     Premultiply GM by the transpose of Z.
C
      NP=NACT+1
      DO 20 J=NP,N
      SUM=0.0
      SUMABS=0.0
      IZ=J
      DO 10 I=1,N
      TEMP=Z(IZ)*GM(I)
      SUM=SUM+TEMP
      SUMABS=SUMABS+ABS(TEMP)
   10 IZ=IZ+N
      IF (ABS(SUM) .LE. RELACC*SUMABS) SUM=0.0
   20 ZTG(J)=SUM
C
C     Form D by premultiplying ZTG by -Z.
C
      IZ=0
      DO 40 I=1,N
      SUM=0.0
      SUMABS=0.0
      DO 30 J=NP,N
      TEMP=Z(IZ+J)*ZTG(J)
      SUM=SUM-TEMP
   30 SUMABS=SUMABS+ABS(TEMP)
      IF (ABS(SUM) .LE. RELACC*SUMABS) SUM=0.0
      D(I)=SUM
   40 IZ=IZ+N
C
C     Test that the search direction is downhill.
C
      SUMABS=0.0
      DO 50 I=1,N
      TEMP=D(I)*GM(I)
      DDOTGM=DDOTGM+TEMP
   50 SUMABS=SUMABS+ABS(TEMP)
      IF (DDOTGM+RELACC*SUMABS .GE. 0.0) DDOTGM=0.0
   60 RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE STEPBD (N,M,A,IA,IACT,BRES,D,STEPCB,DDOTG,MDEG,MSAT,
     1  MTOT,INDXBD)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(IA,*),IACT(*),BRES(*),D(*)
C
C     Set steps to constraint boundaries and find the least positive one.
C
      IFLAG=0
      STEPCB=0.0
      INDXBD=0
      K=MDEG
   10 K=K+1
      IF (K .gt. MTOT) goto 700
C
C     Form the scalar product of D with the current constraint normal.
C
   20     J=IACT(K)
          IF (J .LE. M) THEN
              SP=0.0
              DO 30 I=1,N
   30         SP=SP+D(I)*A(I,J)
          ELSE
              JM=J-M
              IF (JM .LE. N) THEN
                  SP=-D(JM)
              ELSE
                  SP=D(JM-N)
              END IF
          END IF
C
C     The next branch is taken if label 20 was reached via label 40.
C
          IF (IFLAG .EQ. 1) GOTO 50
C
C     Set BRES(J) to indicate the status of the j-th constraint.
C
          IF (SP*BRES(J) .LE. 0.0) THEN
              BRES(J)=0.0
          ELSE
              BRES(J)=BRES(J)/SP
              IF (STEPCB .EQ. 0.0 .OR. BRES(J) .LT. STEPCB) THEN
                  STEPCB=BRES(J)
                  INDXBD=K
              END IF
          END IF
          GO TO 10
700   continue
C
C     Try to pass through the boundary of a violated constraint.
C
   40 IF (INDXBD .le. MSAT) goto 800
          IFLAG=1
          K=INDXBD
          GOTO 20
   50     MSAT=MSAT+1
          IACT(INDXBD)=IACT(MSAT)
          IACT(MSAT)=J
          BRES(J)=0.0
          INDXBD=MSAT
          DDOTG=DDOTG-SP
          IF (DDOTG .LT. 0.0 .AND. MSAT .LT. MTOT) THEN
C
C     Seek the next constraint boundary along the search direction.
C
              TEMP=0.0
              KL=MDEG+1
              DO 60 K=KL,MTOT
              J=IACT(K)
              IF (BRES(J) .GT. 0.0) THEN
                  IF (TEMP .EQ. 0.0 .OR. BRES(J) .LT. TEMP) THEN
                      TEMP=BRES(J)
                      INDXBD=K
                  END IF
              END IF
   60         CONTINUE
              IF (TEMP .GT. 0.0) THEN
                  STEPCB=TEMP
                  GOTO 40
              END IF
          END IF
800   continue
      RETURN
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE ZBFGS (N,X,NACT,G,Z,ZTG,XS,GS,ZZNORM)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(*),G(*),Z(*),ZTG(*),XS(*),GS(*)
C
C     Test if there is sufficient convexity for the update.
C
      DD=0.0
      DG=0.0
      TEMP=0.0
      DO 10 I=1,N
      XS(I)=X(I)-XS(I)
      DD=DD+XS(I)**2
      TEMP=TEMP+GS(I)*XS(I)
      GS(I)=G(I)-GS(I)
   10 DG=DG+GS(I)*XS(I)
      IF (DG .LT. 0.1*ABS(TEMP)) GOTO 90
C
C     Transform the Z matrix.
C
      K=N
   20 KP=K
      K=K-1
      IF (K .GT. NACT) THEN
          IF (ZTG(KP) .EQ. 0.0) GOTO 20
          TEMP=ABS(ZTG(KP))*SQRT(1.0+(ZTG(K)/ZTG(KP))**2)
          WCOS=ZTG(K)/TEMP
          WSIN=ZTG(KP)/TEMP
          ZTG(K)=TEMP
          IZ=K
          DO 30 I=1,N
          TEMP=WCOS*Z(IZ+1)-WSIN*Z(IZ)
          Z(IZ)=WCOS*Z(IZ)+WSIN*Z(IZ+1)
          Z(IZ+1)=TEMP
   30     IZ=IZ+N
          GOTO 20
      END IF
C
C     Update the value of ZZNORM.
C
      IF (ZZNORM .LT. 0.0) THEN
          ZZNORM=DD/DG
      ELSE
          TEMP=SQRT(ZZNORM*DD/DG)
          ZZNORM=MIN(ZZNORM,TEMP)
          ZZNORM=MAX(ZZNORM,0.1*TEMP)
      END IF
C
C     Complete the updating of Z.
C
      NP=NACT+1
      TEMP=SQRT(DG)
      IZ=NP
      DO 40 I=1,N
      Z(IZ)=XS(I)/TEMP
   40 IZ=IZ+N
      IF (NP .LT. N) THEN
          KM=NP+1
          DO 80 K=KM,N
          TEMP=0.0
          IZ=K
          DO 50 I=1,N
          TEMP=TEMP+GS(I)*Z(IZ)
   50     IZ=IZ+N
          TEMP=TEMP/DG
          SUM=0.0
          IZ=K
          DO 60 I=1,N
          Z(IZ)=Z(IZ)-TEMP*XS(I)
          SUM=SUM+Z(IZ)**2
   60     IZ=IZ+N
          IF (SUM .LT. ZZNORM) THEN
              TEMP=SQRT(ZZNORM/SUM)
              IZ=K
              DO 70 I=1,N
              Z(IZ)=TEMP*Z(IZ)
   70         IZ=IZ+N
          END IF
   80     CONTINUE
      END IF
   90 RETURN
      END
C  
C  --------------------------------------------------------------------- 
C  
      SUBROUTINE TOREAL ( STR, LE, X, IERR )
C
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C  
C  CONVERT A STRING TO ITS REAL REPRESENTATION.  
C  
      CHARACTER*(*) STR  
      CHARACTER C 
      LOGICAL IPART, FPART 
      LOGICAL NUMBER 
      NUMBER(C) = C.GE.'0' .AND. C.LE.'9'  
C      
      IZERO = ICHAR('0') 
      V = 0.0 
      W = 0.0 
      IERR = 0 
      POT= 1.0 
      IEX = 0 
      IPART = .FALSE. 
      FPART = .FALSE. 
C
C  SKIP INITIAL BLANKS.
      DO 60,I=1,LE
         IF (STR(I:I).NE.' ') THEN
         NC = I
         GOTO 70
         END IF
60    CONTINUE
C
C  THE STRING IS FULL OF BLANKS.
      IERR = -1
      RETURN
C
70    CONTINUE
      C = STR(NC:NC)
      IF (C.EQ.'+') THEN 
         IS = 1 
      ELSE IF (C.EQ.'-') THEN 
         IS = -1 
      ELSE  
         IS = 1 
         GOTO 10 
      END IF 
C 
      NC = NC+1 
      IF (NC.GT.LE) THEN 
         IERR = -1 
         RETURN 
      END IF 
      C = STR(NC:NC) 
C 
 10   CONTINUE 
      IF (NUMBER(C)) THEN 
         V = 10*V + ICHAR(C)-IZERO 
         IPART = .TRUE. 
         NC = NC+1 
         IF (NC.GT.LE) THEN 
            X = IS*V 
         RETURN 
         END IF 
         C = STR(NC:NC) 
         GOTO 10 
      END IF 
C 
 20   CONTINUE 
      IF (C.EQ.'.') GOTO 30 
      IF (C.EQ.'E' .OR. C.EQ.'D') GOTO 40 
      IERR = -1 
      RETURN 
C 
 30   CONTINUE 
      NC = NC+1 
      IF (NC.GT.LE) THEN 
         IF (.NOT.(IPART.OR.FPART)) IERR = -1 
         X = IS*(V+W) 
         RETURN 
      END IF 
      C = STR(NC:NC) 
      IF (NUMBER(C)) THEN 
         POT = 10*POT 
         W = W + (ICHAR(C)-IZERO)/POT 
         FPART = .TRUE. 
         GOTO 30 
      END IF 
      IF (C.EQ.'E' .OR. C.EQ.'D') GOTO 40 
      IERR = -1 
      RETURN 
C 
 40   NC = NC+1 
      IF (NC.GT.LE) THEN 
         IERR = -1 
         RETURN 
      END IF 
      C = STR(NC:NC) 
C 
      IF (.NOT.(IPART.OR.FPART)) THEN 
         IERR = -1 
         RETURN 
      END IF 
C 
      IF (C.EQ.'+') THEN 
         IES = 1 
      ELSE IF (C.EQ.'-') THEN 
         IES = -1 
      ELSE 
         IES = 1 
         GOTO 50 
      END IF 
      NC = NC+1 
      IF (NC.GT.LE) THEN 
         IERR = -1 
         RETURN 
      END IF 
      C = STR(NC:NC) 
C 
 50   CONTINUE 
      IF (NUMBER(C)) THEN 
         IEX = 10*IEX + ICHAR(C)-IZERO 
         NC = NC+1 
         IF (NC.GT.LE) THEN 
            X = IS*(V+W) * 10.0**(IES*IEX) 
         RETURN 
         END IF 
         C = STR(NC:NC) 
         GOTO 50 
      END IF 
C 
      IERR = -1 
      RETURN 
C 
 
      END 
      SUBROUTINE UNDEF(INDEF,NOV,IUOUT,STRO,IGO)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  THIS ROUTINE CHECKS IF SOME OF THE POINT-VARIABLES IS INDEFINITE.
C  IF INDEF(I) = 0 , THEN THE VARIABLE  POINT(I) IS INDEFINITE.
C   AN INFORMATIVE MESSAGE IS ISSUED FOR EVERY INDEFINITE VARIABLE.
C
      PARAMETER ( MXV = 200 )
      DIMENSION INDEF(MXV)
      LOGICAL STRO
      IGO = 1
      DO 1 I= 1, NOV
      J=INDEF(I)
      IF ( J .EQ. 0  .AND. STRO ) THEN
      WRITE(IUOUT,2) I
      ENDIF
      IGO = IGO*J
 1    CONTINUE
 2    FORMAT(5X,'UNDEFINED VARIABLE : ',I5 )
      END
      SUBROUTINE VARSTA(MSTA)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  CHECKS THE STATUS OF ALL VARIABLES
C  IF ANY VARIABLE IS NOT FIXED AND NOT FRAMED   MSTA=0
C  ELSE                                          MSTA=1
      PARAMETER ( MXV = 200 )
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      MSTA=0
       DO 1 I=1,NOD
        IF(IXAT(I).EQ.0.OR.MARG(I).EQ.2)GO TO  1
        RETURN
    1  CONTINUE
      MSTA=1
      RETURN
      END
      FUNCTION WALL(I,IDIG,G)
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  WALL ASSUMES ONLY TWO VALUES: ZERO AND ONE. FOR A FIXED VARIABLE
C  WALL IS ZERO. (THE VARIABLE IS SPECIFIED BY THE FIRST ARGUMENT  I  )
C  IF MARGINS EXIST FOR THAT VARIABLE , AND THE VARIABLE'S VALUE
C  IS WITHIN A DISTANCE FROM A MARGIN WHICH IS LESS THAN ITS ABSOLUTE
C  VALUE PLUS ONE,DIVIDED BY 10**IDIG,AND THE PARTIAL DERIVATIVE G
C  INDICATES THAT THE VARIABLE TENDS TO GET EVEN CLOSER TO THAT MARGIN,
C  WALL ASSUMES AGAIN THE VALUE OF ZERO. IN ALL OTHER CASES WALL IS SET
C  TO EQUAL ONE.
C
      PARAMETER ( MXV = 200 )
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      WALL=IXAT(I)
      IF(MARG(I).EQ.0.)RETURN
      VAR=POINT(I)
      XL=ABS(VAR-XLL(I))
      XR=ABS(VAR-XRL(I))
      SML=(1.+ABS(VAR))*.1**IDIG
       IF(XL.LT.SML.AND.G.GT.0.)THEN
        WALL=0.
        RETURN
       ENDIF
       IF(XR.LT.SML.AND.G.LT.0.)THEN
        WALL=0.
        RETURN
       ENDIF
      RETURN
      END
      SUBROUTINE WRITER ( IFILE )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     WRITES TO THE FILE SPECIFIED BY THE  IFILE INPUT PARAMETER,
C     THE CURRENT POINT,FIX-STATUS,NAMES AND MARGINS IF EXIST.
C
      PARAMETER ( MXV = 200 )
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/FIRST/ICATA,IDAT
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER MARL*11,MARR*11,CFIX*3
      CHARACTER*10 NICKNA      
      COMMON/NAMES/NICKNA(MXV)
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
      COMMON/INDE/INDIC(6)
      CHARACTER*10 FILSTA
      COMMON/FSTAT/FILSTA(6)
C
      IF (ITNOCS.EQ.0) RETURN
      DO 10,I=1,6
        IF (IFILE.EQ.INDIC(I)) THEN
           FILSTA(I)= '  *  '
           GOTO 11
        END IF
 10   CONTINUE
 11   CONTINUE
C
      IF(IFILE.EQ.IUDAT)IDAT=1
      DO 1 I=1,NOD
         IF (IXAT(I).EQ.0) THEN
            CFIX=' * '
         ELSE
            CFIX='...'
         END IF
         MARL='   '
         MARR='   '
         MA = MARG(I)
         IF (MA.EQ.-1) THEN
            WRITE (MARL,2) XLL(I)
            WRITE (IFILE,3) I,NICKNA(I),CFIX,POINT(I),MARL
         ELSE IF (MA.EQ.1) THEN
            WRITE (MARR,2) XRL(I)
            WRITE (IFILE,3) I,NICKNA(I),CFIX,POINT(I),MARL,MARR
         ELSE IF (MA.EQ.2) THEN
            WRITE (MARL,2) XLL(I)
            WRITE (MARR,2) XRL(I)
            WRITE (IFILE,3) I,NICKNA(I),CFIX,POINT(I),MARL,MARR
         ELSE
            WRITE (IFILE,3) I,NICKNA(I),CFIX,POINT(I)
         END IF
    1 CONTINUE
      WRITE(IFILE,4)'VALUE',VAL
      RETURN
    2 FORMAT (1PG11.4)
    3 FORMAT (1X,I3,') ',A10,'..',A3,'..',T25,1PG21.14:,
     >        T48,2X,A11:,2X,A11)
    4 FORMAT (4X,A10,7('.'),T25,1PG21.14)
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE WRITE1 ( VALUE, X, N, IFILE )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     WRITES TO THE FILE SPECIFIED BY THE  IFILE INPUT PARAMETER,
C     THE CURRENT POINT,FIX-STATUS,NAMES AND MARGINS IF EXIST.
C
      DIMENSION X(N)
      PARAMETER ( MXV = 200 )
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/FIRST/ICATA,IDAT
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
      COMMON/UNITS/IUINP,IUOUT,IUDAT,IUSTO,IUINI,IUDIS,IUBAC,IUMAC
      CHARACTER MARL*11,MARR*11,CFIX*3
      CHARACTER*10 NICKNA
      COMMON/NAMES/NICKNA(MXV)
      COMMON/TOTCAL/ITNOCS,IPARTC,ITDER,IPDER
C
      IF (ITNOCS.EQ.0) RETURN
      DO 1 I=1,NOD
         IF (IXAT(I).EQ.0) THEN
            CFIX=' * '
         ELSE
            CFIX='...'
         END IF
         MARL='   '
         MARR='   '
         MA = MARG(I)
         IF (MA.EQ.-1) THEN
            WRITE (MARL,2) XLL(I)
            WRITE (IFILE,3) I,NICKNA(I),CFIX,X(I),MARL
         ELSE IF (MA.EQ.1) THEN
            WRITE (MARR,2) XRL(I)
            WRITE (IFILE,3) I,NICKNA(I),CFIX,X(I),MARL,MARR
         ELSE IF (MA.EQ.2) THEN
            WRITE (MARL,2) XLL(I)
            WRITE (MARR,2) XRL(I)
            WRITE (IFILE,3) I,NICKNA(I),CFIX,X(I),MARL,MARR
         ELSE
            WRITE (IFILE,3) I,NICKNA(I),CFIX,X(I)
         END IF
    1 CONTINUE
      WRITE(IFILE,4)'VALUE',VALUE
      RETURN
    2 FORMAT (1PG11.4)
    3 FORMAT (1X,I3,') ',A10,'..',A3,'..',T25,1PG21.14:,
     >        T48,2X,A11:,2X,A11)
    4 FORMAT (4X,A10,7('.'),T25,1PG21.14)
      END
      SUBROUTINE XTOY ( X, N )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(N)
      PARAMETER ( MXV = 200 )
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
*
      DO 1,I=1,N
         MA = MARG(I)
         XL = XLL(I)
         XR = XRL(I)
         IF (IXAT(I).EQ.0) THEN
            MA = 2
            XL = X(I)
            XR = X(I)
         END IF
         IF (MA.EQ.-1) THEN
            X(I) = SQRT(X(I)-XL)
         ELSE IF (MA.EQ.1) THEN
            X(I) = SQRT(XR-X(I))
         ELSE IF (MA.EQ.2) THEN
            BMA = XR-XL
         IF (BMA.NE.0) THEN
               D = 0
            DXL = X(I)-XL
            IF (DXL.EQ.0) D = COMACC
               X(I) = SQRT(LOG(BMA/(DXL+D)))
         ELSE
            X(I) = 1
         ENDIF
         ENDIF
1     CONTINUE
      END
C
C  ---------------------------------------------------------------------
C
      SUBROUTINE YTOX ( X, N )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(N)
      PARAMETER ( MXV = 200 )
      COMMON/DETAIL/IXAT(MXV),MARG(MXV)
      COMMON/PARAMS/POINT(MXV),STEP(MXV),XLL(MXV),XRL(MXV),VAL,NOD
      COMMON/ACCURA/COMACC,COMTOL
*
      DO 1,I=1,N
         MA = MARG(I)
         XL = XLL(I)
         XR = XRL(I)
         IF (IXAT(I).EQ.0) THEN
            MA = 2
C  "LEVE" OPERATES ON A SET OF TRANSFORMED VARIABLES, AND THUS 
C  POINT(I) DOES NOT CONTAIN THE CURRENT POINT THROUGHOUT "LEVE" EXECUTION.
C  POINT(I) WILL BE UPDATED UPON "LEVE" TERMINATION.
            XL = POINT(I)
            XR = POINT(I)
         END IF
         IF (MA.EQ.-1) THEN
            X(I) = XL + X(I)**2 
         ELSE IF (MA.EQ.1) THEN
            X(I) = XR - X(I)**2
         ELSE IF (MA.EQ.2) THEN
            X(I) = (XR-XL)*AXP(-X(I)**2) + XL
         END IF
1     CONTINUE
*      
      END
C
C  ---------------------------------------------------------------------
C
      FUNCTION AXP ( Z )
******IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/BIGCOM/BIGNUM,BLIM,DWARF,DWALOG
      IF (Z.LE.DWALOG) THEN
         AXP = 0
      ELSE
         AXP = EXP(Z)
      END IF
      END
